/*
 *  Copyright (c) 2018 The WebRTC project authors. All Rights Reserved.
 *
 *  Use of this source code is governed by a BSD-style license
 *  that can be found in the LICENSE file in the root of the source
 *  tree. An additional intellectual property rights grant can be found
 *  in the file PATENTS.  All contributing project authors may
 *  be found in the AUTHORS file in the root of the source tree.
 */

#include "modules/rtp_rtcp/source/fec_private_tables_random.h"

namespace {
// clang-format off
#define kMaskRandom1_1 \
  0x80, 0x00

#define kMaskRandom2_1 \
  0xc0, 0x00

#define kMaskRandom2_2 \
  0xc0, 0x00, \
  0x80, 0x00

#define kMaskRandom3_1 \
  0xe0, 0x00

#define kMaskRandom3_2 \
  0xc0, 0x00, \
  0xa0, 0x00

#define kMaskRandom3_3 \
  0xc0, 0x00, \
  0xa0, 0x00, \
  0x60, 0x00

#define kMaskRandom4_1 \
  0xf0, 0x00

#define kMaskRandom4_2 \
  0xc0, 0x00, \
  0xb0, 0x00

#define kMaskRandom4_3 \
  0xc0, 0x00, \
  0xb0, 0x00, \
  0x60, 0x00

#define kMaskRandom4_4 \
  0xc0, 0x00, \
  0xa0, 0x00, \
  0x30, 0x00, \
  0x50, 0x00

#define kMaskRandom5_1 \
  0xf8, 0x00

#define kMaskRandom5_2 \
  0xa8, 0x00, \
  0xd0, 0x00

#define kMaskRandom5_3 \
  0xb0, 0x00, \
  0xc8, 0x00, \
  0x50, 0x00

#define kMaskRandom5_4 \
  0xc8, 0x00, \
  0xb0, 0x00, \
  0x50, 0x00, \
  0x28, 0x00

#define kMaskRandom5_5 \
  0xc0, 0x00, \
  0x30, 0x00, \
  0x18, 0x00, \
  0xa0, 0x00, \
  0x48, 0x00

#define kMaskRandom6_1 \
  0xfc, 0x00

#define kMaskRandom6_2 \
  0xa8, 0x00, \
  0xd4, 0x00

#define kMaskRandom6_3 \
  0xd0, 0x00, \
  0x68, 0x00, \
  0xa4, 0x00

#define kMaskRandom6_4 \
  0xa8, 0x00, \
  0x58, 0x00, \
  0x64, 0x00, \
  0x94, 0x00

#define kMaskRandom6_5 \
  0xa8, 0x00, \
  0x84, 0x00, \
  0x64, 0x00, \
  0x90, 0x00, \
  0x58, 0x00

#define kMaskRandom6_6 \
  0x98, 0x00, \
  0x64, 0x00, \
  0x50, 0x00, \
  0x14, 0x00, \
  0xa8, 0x00, \
  0xe0, 0x00

#define kMaskRandom7_1 \
  0xfe, 0x00

#define kMaskRandom7_2 \
  0xd4, 0x00, \
  0xaa, 0x00

#define kMaskRandom7_3 \
  0xd0, 0x00, \
  0xaa, 0x00, \
  0x64, 0x00

#define kMaskRandom7_4 \
  0xd0, 0x00, \
  0xaa, 0x00, \
  0x64, 0x00, \
  0x1c, 0x00

#define kMaskRandom7_5 \
  0x0c, 0x00, \
  0xb0, 0x00, \
  0x1a, 0x00, \
  0xc4, 0x00, \
  0x62, 0x00

#define kMaskRandom7_6 \
  0x8c, 0x00, \
  0x4a, 0x00, \
  0x64, 0x00, \
  0xd0, 0x00, \
  0xa0, 0x00, \
  0x32, 0x00

#define kMaskRandom7_7 \
  0x4a, 0x00, \
  0x94, 0x00, \
  0x1a, 0x00, \
  0xc4, 0x00, \
  0x28, 0x00, \
  0xc2, 0x00, \
  0x34, 0x00

#define kMaskRandom8_1 \
  0xff, 0x00

#define kMaskRandom8_2 \
  0xaa, 0x00, \
  0xd5, 0x00

#define kMaskRandom8_3 \
  0xc5, 0x00, \
  0x92, 0x00, \
  0x6a, 0x00

#define kMaskRandom8_4 \
  0x45, 0x00, \
  0xb4, 0x00, \
  0x6a, 0x00, \
  0x89, 0x00

#define kMaskRandom8_5 \
  0x8c, 0x00, \
  0x92, 0x00, \
  0x2b, 0x00, \
  0x51, 0x00, \
  0x64, 0x00

#define kMaskRandom8_6 \
  0xa1, 0x00, \
  0x52, 0x00, \
  0x91, 0x00, \
  0x2a, 0x00, \
  0xc4, 0x00, \
  0x4c, 0x00

#define kMaskRandom8_7 \
  0x15, 0x00, \
  0xc2, 0x00, \
  0x25, 0x00, \
  0x62, 0x00, \
  0x58, 0x00, \
  0x8c, 0x00, \
  0xa3, 0x00

#define kMaskRandom8_8 \
  0x25, 0x00, \
  0x8a, 0x00, \
  0x91, 0x00, \
  0x68, 0x00, \
  0x32, 0x00, \
  0x43, 0x00, \
  0xc4, 0x00, \
  0x1c, 0x00

#define kMaskRandom9_1 \
  0xff, 0x80

#define kMaskRandom9_2 \
  0xaa, 0x80, \
  0xd5, 0x00

#define kMaskRandom9_3 \
  0xa5, 0x00, \
  0xc8, 0x00, \
  0x52, 0x80

#define kMaskRandom9_4 \
  0xa2, 0x00, \
  0xc9, 0x00, \
  0x52, 0x80, \
  0x24, 0x80

#define kMaskRandom9_5 \
  0x8c, 0x00, \
  0x25, 0x00, \
  0x92, 0x80, \
  0x41, 0x80, \
  0x58, 0x00

#define kMaskRandom9_6 \
  0x84, 0x80, \
  0x27, 0x00, \
  0x51, 0x80, \
  0x1a, 0x00, \
  0x68, 0x00, \
  0x89, 0x00

#define kMaskRandom9_7 \
  0x8c, 0x00, \
  0x47, 0x00, \
  0x81, 0x80, \
  0x12, 0x80, \
  0x58, 0x00, \
  0x28, 0x80, \
  0xb4, 0x00

#define kMaskRandom9_8 \
  0x2c, 0x00, \
  0x91, 0x00, \
  0x40, 0x80, \
  0x06, 0x80, \
  0xc8, 0x00, \
  0x45, 0x00, \
  0x30, 0x80, \
  0xa2, 0x00

#define kMaskRandom9_9 \
  0x4c, 0x00, \
  0x62, 0x00, \
  0x91, 0x00, \
  0x42, 0x80, \
  0xa4, 0x00, \
  0x13, 0x00, \
  0x30, 0x80, \
  0x88, 0x80, \
  0x09, 0x00

#define kMaskRandom10_1 \
  0xff, 0xc0

#define kMaskRandom10_10 \
  0x4c, 0x00, \
  0x51, 0x00, \
  0xa0, 0x40, \
  0x04, 0xc0, \
  0x03, 0x80, \
  0x86, 0x00, \
  0x29, 0x00, \
  0x42, 0x40, \
  0x98, 0x00, \
  0x30, 0x80

#define kMaskRandom10_2 \
  0xaa, 0x80, \
  0xd5, 0x40

#define kMaskRandom10_3 \
  0xa4, 0x40, \
  0xc9, 0x00, \
  0x52, 0x80

#define kMaskRandom10_4 \
  0xca, 0x00, \
  0x32, 0x80, \
  0xa1, 0x40, \
  0x55, 0x00

#define kMaskRandom10_5 \
  0xca, 0x00, \
  0x32, 0x80, \
  0xa1, 0x40, \
  0x55, 0x00, \
  0x08, 0xc0

#define kMaskRandom10_6 \
  0x0e, 0x00, \
  0x33, 0x00, \
  0x10, 0xc0, \
  0x45, 0x40, \
  0x88, 0x80, \
  0xe0, 0x00

#define kMaskRandom10_7 \
  0x46, 0x00, \
  0x33, 0x00, \
  0x80, 0xc0, \
  0x0c, 0x40, \
  0x28, 0x80, \
  0x94, 0x00, \
  0xc1, 0x00

#define kMaskRandom10_8 \
  0x2c, 0x00, \
  0x81, 0x80, \
  0xa0, 0x40, \
  0x05, 0x40, \
  0x18, 0x80, \
  0xc2, 0x00, \
  0x22, 0x80, \
  0x50, 0x40

#define kMaskRandom10_9 \
  0x4c, 0x00, \
  0x23, 0x00, \
  0x88, 0xc0, \
  0x21, 0x40, \
  0x52, 0x80, \
  0x94, 0x00, \
  0x26, 0x00, \
  0x48, 0x40, \
  0x91, 0x80

#define kMaskRandom11_1 \
  0xff, 0xe0

#define kMaskRandom11_10 \
  0x64, 0x40, \
  0x51, 0x40, \
  0xa9, 0x00, \
  0x04, 0xc0, \
  0xd0, 0x00, \
  0x82, 0x40, \
  0x21, 0x20, \
  0x0c, 0x20, \
  0x4a, 0x00, \
  0x12, 0xa0

#define kMaskRandom11_11 \
  0x46, 0x40, \
  0x33, 0x20, \
  0x99, 0x00, \
  0x05, 0x80, \
  0x80, 0xa0, \
  0x84, 0x40, \
  0x40, 0x60, \
  0x0a, 0x80, \
  0x68, 0x00, \
  0x10, 0x20, \
  0x30, 0x40

#define kMaskRandom11_2 \
  0xec, 0xc0, \
  0x9b, 0xa0

#define kMaskRandom11_3 \
  0xca, 0xc0, \
  0xf1, 0x40, \
  0xb6, 0x20

#define kMaskRandom11_4 \
  0xc4, 0xc0, \
  0x31, 0x60, \
  0x4b, 0x20, \
  0x2c, 0xa0

#define kMaskRandom11_5 \
  0x86, 0x80, \
  0x23, 0x20, \
  0x16, 0x20, \
  0x4c, 0x20, \
  0x41, 0xc0

#define kMaskRandom11_6 \
  0x64, 0x40, \
  0x51, 0x40, \
  0x0c, 0xa0, \
  0xa1, 0x20, \
  0x12, 0xa0, \
  0x8a, 0x40

#define kMaskRandom11_7 \
  0x46, 0x40, \
  0x33, 0x20, \
  0x91, 0x80, \
  0xa4, 0x20, \
  0x50, 0xa0, \
  0x84, 0xc0, \
  0x09, 0x60

#define kMaskRandom11_8 \
  0x0c, 0x80, \
  0x80, 0x60, \
  0xa0, 0x80, \
  0x05, 0x40, \
  0x43, 0x00, \
  0x1a, 0x00, \
  0x60, 0x20, \
  0x14, 0x20

#define kMaskRandom11_9 \
  0x46, 0x40, \
  0x62, 0x60, \
  0x8c, 0x00, \
  0x01, 0x60, \
  0x07, 0x80, \
  0xa0, 0x80, \
  0x18, 0xa0, \
  0x91, 0x00, \
  0x78, 0x00

#define kMaskRandom12_1 \
  0xff, 0xf0

#define kMaskRandom12_10 \
  0x51, 0x40, \
  0x45, 0x10, \
  0x80, 0xd0, \
  0x24, 0x20, \
  0x0a, 0x20, \
  0x00, 0xe0, \
  0xb8, 0x00, \
  0x09, 0x10, \
  0x56, 0x00, \
  0xa2, 0x80

#define kMaskRandom12_11 \
  0x53, 0x60, \
  0x21, 0x30, \
  0x10, 0x90, \
  0x00, 0x70, \
  0x0c, 0x10, \
  0x40, 0xc0, \
  0x6a, 0x00, \
  0x86, 0x00, \
  0x24, 0x80, \
  0x89, 0x00, \
  0xc0, 0x20

#define kMaskRandom12_12 \
  0x10, 0x60, \
  0x02, 0x30, \
  0x40, 0x50, \
  0x21, 0x80, \
  0x81, 0x10, \
  0x14, 0x80, \
  0x98, 0x00, \
  0x08, 0x90, \
  0x62, 0x00, \
  0x24, 0x20, \
  0x8a, 0x00, \
  0x84, 0x40

#define kMaskRandom12_2 \
  0xec, 0xc0, \
  0x93, 0xb0

#define kMaskRandom12_3 \
  0x9b, 0x80, \
  0x4f, 0x10, \
  0x3c, 0x60

#define kMaskRandom12_4 \
  0x8b, 0x20, \
  0x14, 0xb0, \
  0x22, 0xd0, \
  0x45, 0x50

#define kMaskRandom12_5 \
  0x53, 0x60, \
  0x64, 0x20, \
  0x0c, 0xc0, \
  0x82, 0xa0, \
  0x09, 0x30

#define kMaskRandom12_6 \
  0x51, 0x40, \
  0xc5, 0x10, \
  0x21, 0x80, \
  0x12, 0x30, \
  0x08, 0xe0, \
  0x2e, 0x00

#define kMaskRandom12_7 \
  0x53, 0x60, \
  0x21, 0x30, \
  0x90, 0x90, \
  0x02, 0x50, \
  0x06, 0xa0, \
  0x2c, 0x00, \
  0x88, 0x60

#define kMaskRandom12_8 \
  0x20, 0x60, \
  0x80, 0x30, \
  0x42, 0x40, \
  0x01, 0x90, \
  0x14, 0x10, \
  0x0a, 0x80, \
  0x38, 0x00, \
  0xc5, 0x00

#define kMaskRandom12_9 \
  0x53, 0x60, \
  0xe4, 0x20, \
  0x24, 0x40, \
  0xa1, 0x10, \
  0x18, 0x30, \
  0x03, 0x90, \
  0x8a, 0x10, \
  0x04, 0x90, \
  0x00, 0xe0

#define kMaskRandom13_1 \
  0xff, 0xf8

#define kMaskRandom13_10 \
  0xd1, 0x00, \
  0x44, 0x50, \
  0x10, 0x98, \
  0xa0, 0x50, \
  0x4a, 0x08, \
  0x40, 0x30, \
  0x80, 0x28, \
  0x0c, 0x90, \
  0x05, 0x88, \
  0x62, 0x20

#define kMaskRandom13_11 \
  0x51, 0x20, \
  0x22, 0x10, \
  0x13, 0x40, \
  0x25, 0x00, \
  0x18, 0x18, \
  0x0a, 0x20, \
  0x88, 0x88, \
  0x06, 0x80, \
  0xe0, 0x20, \
  0x84, 0x40, \
  0x44, 0x18

#define kMaskRandom13_12 \
  0x28, 0x28, \
  0x84, 0x50, \
  0x60, 0x40, \
  0x05, 0x48, \
  0x02, 0x98, \
  0x01, 0x30, \
  0x48, 0x10, \
  0x24, 0x80, \
  0x94, 0x00, \
  0x8a, 0x00, \
  0x11, 0x80, \
  0x52, 0x20

#define kMaskRandom13_13 \
  0x51, 0x20, \
  0x66, 0x40, \
  0x05, 0x48, \
  0x81, 0x20, \
  0x94, 0x00, \
  0x30, 0x80, \
  0x21, 0x10, \
  0x03, 0xc0, \
  0xe8, 0x00, \
  0x0a, 0x10, \
  0x80, 0x18, \
  0x04, 0x90, \
  0x08, 0xa8

#define kMaskRandom13_2 \
  0xec, 0xc0, \
  0x1b, 0x38

#define kMaskRandom13_3 \
  0x99, 0xb0, \
  0x46, 0xd8, \
  0x37, 0x28

#define kMaskRandom13_4 \
  0x49, 0xb0, \
  0x26, 0xd0, \
  0x85, 0x68, \
  0x52, 0x58

#define kMaskRandom13_5 \
  0x51, 0x30, \
  0x66, 0x40, \
  0x0c, 0x68, \
  0xa1, 0xc0, \
  0x22, 0x98

#define kMaskRandom13_6 \
  0xd1, 0x20, \
  0x46, 0xd0, \
  0x15, 0x48, \
  0x21, 0x70, \
  0x28, 0xc8, \
  0xaa, 0x20

#define kMaskRandom13_7 \
  0x59, 0x20, \
  0x26, 0x50, \
  0xb1, 0x40, \
  0x2b, 0x08, \
  0x14, 0xc8, \
  0xc8, 0x88, \
  0x84, 0xb0

#define kMaskRandom13_8 \
  0x80, 0xa8, \
  0x30, 0x90, \
  0x16, 0x08, \
  0x03, 0x30, \
  0x44, 0x60, \
  0x08, 0x18, \
  0xd8, 0x00, \
  0xa1, 0x40

#define kMaskRandom13_9 \
  0x59, 0x20, \
  0x66, 0x40, \
  0x14, 0x40, \
  0x21, 0x48, \
  0x02, 0xc8, \
  0x94, 0x10, \
  0x80, 0xa8, \
  0x0a, 0x90, \
  0x40, 0x18

#define kMaskRandom14_1 \
  0xff, 0xfc

#define kMaskRandom14_10 \
  0xc0, 0xd4, \
  0x1d, 0x40, \
  0xd4, 0x08, \
  0x02, 0x60, \
  0x04, 0x28, \
  0x20, 0x98, \
  0x40, 0x44, \
  0x08, 0x84, \
  0x68, 0x00, \
  0x23, 0x10

#define kMaskRandom14_11 \
  0x62, 0xd0, \
  0x35, 0x20, \
  0x14, 0x14, \
  0xc5, 0x08, \
  0x22, 0x0c, \
  0x88, 0xb8, \
  0x42, 0x54, \
  0x28, 0xa4, \
  0x94, 0x20, \
  0x1b, 0x04, \
  0x22, 0xc0

#define kMaskRandom14_12 \
  0x81, 0x04, \
  0x40, 0x68, \
  0x90, 0x24, \
  0x28, 0x28, \
  0x52, 0x10, \
  0x41, 0x88, \
  0x09, 0x30, \
  0x48, 0x44, \
  0x04, 0x44, \
  0x0e, 0x80, \
  0xa5, 0x90, \
  0x12, 0x0c

#define kMaskRandom14_13 \
  0x62, 0x54, \
  0x34, 0x60, \
  0x48, 0x04, \
  0x00, 0xac, \
  0x28, 0x08, \
  0x81, 0x08, \
  0x23, 0x04, \
  0x06, 0x80, \
  0x80, 0x14, \
  0x30, 0x10, \
  0x8c, 0x20, \
  0x54, 0x00, \
  0x80, 0xc0

#define kMaskRandom14_14 \
  0x40, 0x54, \
  0x15, 0x40, \
  0xc0, 0x04, \
  0x28, 0x10, \
  0x05, 0x0c, \
  0x64, 0x80, \
  0x81, 0x80, \
  0x10, 0x98, \
  0x84, 0x20, \
  0x12, 0x30, \
  0x62, 0x00, \
  0x28, 0x60, \
  0x0e, 0x08, \
  0x10, 0x84

#define kMaskRandom14_2 \
  0xec, 0xe8, \
  0x3b, 0x9c

#define kMaskRandom14_3 \
  0xac, 0xd8, \
  0x55, 0x6c, \
  0x27, 0xb4

#define kMaskRandom14_4 \
  0x2c, 0xd8, \
  0x93, 0x68, \
  0x1a, 0xb4, \
  0x47, 0x2c

#define kMaskRandom14_5 \
  0x64, 0xd8, \
  0xa5, 0x68, \
  0x52, 0xb4, \
  0x1d, 0xa8, \
  0x9c, 0x54

#define kMaskRandom14_6 \
  0x4a, 0x54, \
  0x95, 0x48, \
  0x14, 0xb4, \
  0x51, 0xa8, \
  0x22, 0x6c, \
  0x88, 0x8c

#define kMaskRandom14_7 \
  0x62, 0x54, \
  0xb9, 0x20, \
  0x18, 0xb4, \
  0x54, 0x98, \
  0x06, 0x6c, \
  0x85, 0x54, \
  0xaa, 0x88

#define kMaskRandom14_8 \
  0xc0, 0x14, \
  0x41, 0x60, \
  0x88, 0x30, \
  0x20, 0xa4, \
  0x0a, 0x48, \
  0x04, 0x98, \
  0x94, 0x40, \
  0x72, 0x00

#define kMaskRandom14_9 \
  0xa2, 0x54, \
  0x34, 0x60, \
  0x4a, 0x24, \
  0x20, 0xa8, \
  0x11, 0x84, \
  0x49, 0x08, \
  0x86, 0x0c, \
  0x20, 0xd4, \
  0x88, 0x48

#define kMaskRandom15_1 \
  0xff, 0xfe

#define kMaskRandom15_10 \
  0xc0, 0xa0, \
  0x15, 0x56, \
  0x74, 0x40, \
  0x00, 0x9c, \
  0x01, 0x2c, \
  0x44, 0x92, \
  0x88, 0x50, \
  0x20, 0xa4, \
  0xaa, 0x04, \
  0x02, 0x62

#define kMaskRandom15_11 \
  0x62, 0x22, \
  0xf1, 0x10, \
  0x10, 0x0e, \
  0x10, 0xb0, \
  0x24, 0x24, \
  0x01, 0x12, \
  0x00, 0xc4, \
  0x04, 0xa2, \
  0x02, 0x58, \
  0x2b, 0x00, \
  0x98, 0x40

#define kMaskRandom15_12 \
  0x88, 0x90, \
  0x40, 0x54, \
  0x82, 0x62, \
  0x21, 0xa4, \
  0x10, 0x64, \
  0x44, 0x0a, \
  0x10, 0xc8, \
  0x4d, 0x2a, \
  0x38, 0x02, \
  0x17, 0x48, \
  0x90, 0x84, \
  0x72, 0x14

#define kMaskRandom15_13 \
  0x62, 0xa2, \
  0x34, 0x44, \
  0x40, 0x4a, \
  0xc4, 0x04, \
  0x08, 0x60, \
  0x94, 0x12, \
  0x88, 0xc0, \
  0x21, 0x32, \
  0xc1, 0x40, \
  0x10, 0x68, \
  0x06, 0x90, \
  0x59, 0x00, \
  0x0a, 0x0c

#define kMaskRandom15_14 \
  0x40, 0x82, \
  0x15, 0x54, \
  0x88, 0x12, \
  0xc0, 0x10, \
  0x80, 0xa0, \
  0x01, 0x22, \
  0x40, 0x2c, \
  0x22, 0x02, \
  0x90, 0x04, \
  0x12, 0x40, \
  0x5d, 0x00, \
  0x20, 0x54, \
  0x86, 0x08, \
  0x28, 0x88

#define kMaskRandom15_15 \
  0x62, 0x22, \
  0x31, 0x10, \
  0x58, 0x00, \
  0x01, 0x12, \
  0x88, 0x20, \
  0x44, 0x02, \
  0x29, 0x04, \
  0x82, 0xa0, \
  0x0a, 0x1a, \
  0x11, 0xe0, \
  0x84, 0x04, \
  0x86, 0x40, \
  0x00, 0x86, \
  0x44, 0x48, \
  0x10, 0x98

#define kMaskRandom15_2 \
  0xec, 0xea, \
  0xbb, 0x9c

#define kMaskRandom15_3 \
  0xac, 0x92, \
  0x55, 0x4a, \
  0x43, 0x36

#define kMaskRandom15_4 \
  0x25, 0xaa, \
  0x95, 0x54, \
  0x1a, 0x6a, \
  0x43, 0xd4

#define kMaskRandom15_5 \
  0x64, 0xa2, \
  0x25, 0x54, \
  0x49, 0x68, \
  0x53, 0x90, \
  0x8e, 0x30

#define kMaskRandom15_6 \
  0x62, 0x8a, \
  0x15, 0x54, \
  0x4c, 0x46, \
  0x52, 0x94, \
  0x23, 0x64, \
  0x8a, 0x58

#define kMaskRandom15_7 \
  0x62, 0xa2, \
  0xb1, 0x14, \
  0x18, 0x6a, \
  0x44, 0xd4, \
  0x13, 0x64, \
  0x49, 0x1a, \
  0x86, 0x8c

#define kMaskRandom15_8 \
  0x90, 0x22, \
  0x09, 0x50, \
  0x00, 0x6a, \
  0x20, 0x34, \
  0x14, 0x44, \
  0xc2, 0x10, \
  0x00, 0xc6, \
  0x65, 0x80

#define kMaskRandom15_9 \
  0x62, 0x22, \
  0x24, 0x44, \
  0xc0, 0x50, \
  0x03, 0x0c, \
  0x16, 0x28, \
  0x89, 0x00, \
  0x82, 0x90, \
  0x08, 0xa4, \
  0x90, 0x48

#define kMaskRandom16_1 \
  0xff, 0xff

#define kMaskRandom16_10 \
  0x45, 0x51, \
  0x10, 0xa2, \
  0x01, 0x25, \
  0x0b, 0x42, \
  0xd8, 0x20, \
  0x82, 0x8c, \
  0x24, 0x4a, \
  0x38, 0x18, \
  0x2a, 0x25, \
  0x84, 0x92

#define kMaskRandom16_11 \
  0x55, 0x55, \
  0x2a, 0x22, \
  0x31, 0x11, \
  0x83, 0x42, \
  0x06, 0x98, \
  0x40, 0xe1, \
  0x2c, 0x44, \
  0xd8, 0x28, \
  0x92, 0x81, \
  0x84, 0x32, \
  0x68, 0x0c

#define kMaskRandom16_12 \
  0x84, 0x31, \
  0x18, 0xa2, \
  0x4e, 0x01, \
  0x44, 0xc8, \
  0x0e, 0x90, \
  0x20, 0xcc, \
  0x93, 0x40, \
  0x2d, 0x10, \
  0x31, 0x44, \
  0xc0, 0x23, \
  0x11, 0x25, \
  0xe8, 0x80

#define kMaskRandom16_13 \
  0x45, 0x15, \
  0x22, 0x22, \
  0x96, 0x0c, \
  0x0c, 0x50, \
  0x62, 0x04, \
  0x49, 0x06, \
  0x11, 0x82, \
  0x12, 0x38, \
  0x40, 0x71, \
  0xa8, 0x8a, \
  0x08, 0xa1, \
  0xa0, 0xc0, \
  0xc5, 0x10

#define kMaskRandom16_14 \
  0x45, 0x51, \
  0x22, 0x0a, \
  0x84, 0xd0, \
  0x0c, 0x8a, \
  0x18, 0x06, \
  0x30, 0x03, \
  0x61, 0x08, \
  0x40, 0x11, \
  0x10, 0x2c, \
  0x09, 0x60, \
  0x00, 0x94, \
  0x52, 0x40, \
  0xa4, 0x24, \
  0x82, 0x88

#define kMaskRandom16_15 \
  0x55, 0x11, \
  0x22, 0x22, \
  0x11, 0x11, \
  0x80, 0x45, \
  0x20, 0x1a, \
  0x08, 0x68, \
  0x22, 0x84, \
  0x48, 0x09, \
  0x07, 0x01, \
  0x94, 0x20, \
  0x82, 0x06, \
  0x60, 0x48, \
  0x89, 0x80, \
  0x00, 0x8e, \
  0x18, 0x22

#define kMaskRandom16_16 \
  0xa4, 0x10, \
  0x01, 0x2a, \
  0x06, 0x42, \
  0x08, 0x68, \
  0x81, 0x90, \
  0x00, 0xf0, \
  0x50, 0x05, \
  0x20, 0x51, \
  0x43, 0x08, \
  0x68, 0x80, \
  0x80, 0x0b, \
  0x10, 0x4c, \
  0x12, 0x30, \
  0x40, 0x85, \
  0x0e, 0x04, \
  0x18, 0x12

#define kMaskRandom16_2 \
  0xae, 0xae, \
  0x79, 0x79

#define kMaskRandom16_3 \
  0xad, 0x2d, \
  0x76, 0x36, \
  0x26, 0xdb

#define kMaskRandom16_4 \
  0x55, 0x55, \
  0xaa, 0xaa, \
  0x35, 0x35, \
  0xca, 0xca

#define kMaskRandom16_5 \
  0x55, 0x55, \
  0x2a, 0x2a, \
  0x24, 0x25, \
  0x84, 0xc8, \
  0x10, 0xb6

#define kMaskRandom16_6 \
  0x51, 0x51, \
  0x0a, 0x2a, \
  0xa2, 0x15, \
  0x84, 0x4a, \
  0x30, 0x92, \
  0x04, 0xac

#define kMaskRandom16_7 \
  0x45, 0x51, \
  0x22, 0x2a, \
  0x91, 0x11, \
  0x2e, 0x08, \
  0x48, 0x34, \
  0x90, 0x29, \
  0x09, 0x86

#define kMaskRandom16_8 \
  0x20, 0x54, \
  0x18, 0x88, \
  0x84, 0x07, \
  0x60, 0x48, \
  0x12, 0x82, \
  0x81, 0x41, \
  0x40, 0x62, \
  0x16, 0x30

#define kMaskRandom16_9 \
  0x55, 0x51, \
  0x22, 0x2a, \
  0x05, 0x85, \
  0x09, 0x4a, \
  0x84, 0x32, \
  0xc0, 0x0d, \
  0x20, 0xa6, \
  0x1a, 0x09, \
  0x44, 0x64

#define kMaskRandom17_1 \
  0xff, 0xff, 0x80, 0x00, 0x00, 0x00

#define kMaskRandom17_2 \
  0xce, 0xce, 0x00, 0x00, 0x00, 0x00, \
  0xb9, 0x39, 0x80, 0x00, 0x00, 0x00

#define kMaskRandom17_3 \
  0xcd, 0xcc, 0x00, 0x00, 0x00, 0x00, \
  0x97, 0x27, 0x00, 0x00, 0x00, 0x00, \
  0xb8, 0xd1, 0x80, 0x00, 0x00, 0x00

#define kMaskRandom17_4 \
  0xca, 0xec, 0x00, 0x00, 0x00, 0x00, \
  0xa9, 0x67, 0x00, 0x00, 0x00, 0x00, \
  0x3a, 0xb1, 0x80, 0x00, 0x00, 0x00, \
  0x55, 0x5a, 0x80, 0x00, 0x00, 0x00

#define kMaskRandom17_5 \
  0x55, 0x44, 0x80, 0x00, 0x00, 0x00, \
  0x2a, 0x66, 0x00, 0x00, 0x00, 0x00, \
  0x25, 0xa1, 0x80, 0x00, 0x00, 0x00, \
  0xe2, 0x12, 0x80, 0x00, 0x00, 0x00, \
  0x99, 0x98, 0x00, 0x00, 0x00, 0x00

#define kMaskRandom17_6 \
  0xd1, 0x4c, 0x00, 0x00, 0x00, 0x00, \
  0xa2, 0xc5, 0x00, 0x00, 0x00, 0x00, \
  0x95, 0x30, 0x80, 0x00, 0x00, 0x00, \
  0xca, 0x0a, 0x80, 0x00, 0x00, 0x00, \
  0xa4, 0xaa, 0x00, 0x00, 0x00, 0x00, \
  0x78, 0x15, 0x00, 0x00, 0x00, 0x00

#define kMaskRandom17_7 \
  0x15, 0x44, 0x80, 0x00, 0x00, 0x00, \
  0x8a, 0x23, 0x00, 0x00, 0x00, 0x00, \
  0x85, 0x91, 0x00, 0x00, 0x00, 0x00, \
  0x32, 0x0a, 0x80, 0x00, 0x00, 0x00, \
  0x58, 0x34, 0x00, 0x00, 0x00, 0x00, \
  0x2c, 0x0d, 0x00, 0x00, 0x00, 0x00, \
  0x43, 0xc8, 0x00, 0x00, 0x00, 0x00

#define kMaskRandom17_8 \
  0x64, 0x16, 0x00, 0x00, 0x00, 0x00, \
  0xa2, 0xc2, 0x00, 0x00, 0x00, 0x00, \
  0x51, 0x60, 0x80, 0x00, 0x00, 0x00, \
  0x4a, 0x85, 0x00, 0x00, 0x00, 0x00, \
  0x38, 0x4c, 0x00, 0x00, 0x00, 0x00, \
  0x89, 0x29, 0x00, 0x00, 0x00, 0x00, \
  0x07, 0x11, 0x80, 0x00, 0x00, 0x00, \
  0x94, 0xb0, 0x00, 0x00, 0x00, 0x00

#define kMaskRandom17_9 \
  0x8e, 0xcc, 0x00, 0x00, 0x00, 0x00, \
  0x6a, 0x2b, 0x00, 0x00, 0x00, 0x00, \
  0x36, 0x32, 0x80, 0x00, 0x00, 0x00, \
  0xd1, 0x25, 0x80, 0x00, 0x00, 0x00, \
  0x55, 0x8c, 0x80, 0x00, 0x00, 0x00, \
  0xaa, 0x27, 0x00, 0x00, 0x00, 0x00, \
  0xa5, 0x32, 0x80, 0x00, 0x00, 0x00, \
  0x62, 0x61, 0x80, 0x00, 0x00, 0x00, \
  0x3c, 0x5c, 0x00, 0x00, 0x00, 0x00

#define kMaskRandom17_10 \
  0x55, 0x8c, 0x80, 0x00, 0x00, 0x00, \
  0xaa, 0x27, 0x00, 0x00, 0x00, 0x00, \
  0xa5, 0x32, 0x80, 0x00, 0x00, 0x00, \
  0x62, 0x61, 0x80, 0x00, 0x00, 0x00, \
  0x3c, 0x5c, 0x00, 0x00, 0x00, 0x00, \
  0x8e, 0xcc, 0x00, 0x00, 0x00, 0x00, \
  0x6a, 0x2b, 0x00, 0x00, 0x00, 0x00, \
  0x36, 0x32, 0x80, 0x00, 0x00, 0x00, \
  0xd1, 0x25, 0x80, 0x00, 0x00, 0x00, \
  0xc8, 0x02, 0x80, 0x00, 0x00, 0x00

#define kMaskRandom17_11 \
  0x55, 0x8c, 0x80, 0x00, 0x00, 0x00, \
  0xaa, 0x27, 0x00, 0x00, 0x00, 0x00, \
  0xa5, 0x32, 0x80, 0x00, 0x00, 0x00, \
  0x62, 0x61, 0x80, 0x00, 0x00, 0x00, \
  0x3c, 0x5c, 0x00, 0x00, 0x00, 0x00, \
  0x51, 0x84, 0x80, 0x00, 0x00, 0x00, \
  0xa2, 0x27, 0x00, 0x00, 0x00, 0x00, \
  0x95, 0x51, 0x80, 0x00, 0x00, 0x00, \
  0x4a, 0x1a, 0x00, 0x00, 0x00, 0x00, \
  0x30, 0x68, 0x00, 0x00, 0x00, 0x00, \
  0x2c, 0x89, 0x00, 0x00, 0x00, 0x00

#define kMaskRandom17_12 \
  0x51, 0x84, 0x80, 0x00, 0x00, 0x00, \
  0xa2, 0x27, 0x00, 0x00, 0x00, 0x00, \
  0x95, 0x51, 0x80, 0x00, 0x00, 0x00, \
  0x4a, 0x1a, 0x00, 0x00, 0x00, 0x00, \
  0x30, 0x68, 0x00, 0x00, 0x00, 0x00, \
  0x2c, 0x89, 0x00, 0x00, 0x00, 0x00, \
  0x55, 0x8c, 0x80, 0x00, 0x00, 0x00, \
  0xaa, 0x27, 0x00, 0x00, 0x00, 0x00, \
  0xa5, 0x32, 0x80, 0x00, 0x00, 0x00, \
  0x62, 0x61, 0x80, 0x00, 0x00, 0x00, \
  0x3c, 0x5c, 0x00, 0x00, 0x00, 0x00, \
  0x51, 0x35, 0x00, 0x00, 0x00, 0x00

#define kMaskRandom17_13 \
  0x51, 0x84, 0x80, 0x00, 0x00, 0x00, \
  0xa2, 0x27, 0x00, 0x00, 0x00, 0x00, \
  0x95, 0x51, 0x80, 0x00, 0x00, 0x00, \
  0x4a, 0x1a, 0x00, 0x00, 0x00, 0x00, \
  0x30, 0x68, 0x00, 0x00, 0x00, 0x00, \
  0x2c, 0x89, 0x00, 0x00, 0x00, 0x00, \
  0x15, 0x8c, 0x00, 0x00, 0x00, 0x00, \
  0x8a, 0x47, 0x00, 0x00, 0x00, 0x00, \
  0x25, 0x81, 0x80, 0x00, 0x00, 0x00, \
  0x62, 0x12, 0x80, 0x00, 0x00, 0x00, \
  0x58, 0x58, 0x00, 0x00, 0x00, 0x00, \
  0x0e, 0x28, 0x80, 0x00, 0x00, 0x00, \
  0x83, 0x34, 0x00, 0x00, 0x00, 0x00

#define kMaskRandom17_14 \
  0x15, 0x8c, 0x00, 0x00, 0x00, 0x00, \
  0x8a, 0x47, 0x00, 0x00, 0x00, 0x00, \
  0x25, 0x81, 0x80, 0x00, 0x00, 0x00, \
  0x62, 0x12, 0x80, 0x00, 0x00, 0x00, \
  0x58, 0x58, 0x00, 0x00, 0x00, 0x00, \
  0x0e, 0x28, 0x80, 0x00, 0x00, 0x00, \
  0x83, 0x34, 0x00, 0x00, 0x00, 0x00, \
  0x51, 0x84, 0x80, 0x00, 0x00, 0x00, \
  0xa2, 0x27, 0x00, 0x00, 0x00, 0x00, \
  0x95, 0x51, 0x80, 0x00, 0x00, 0x00, \
  0x4a, 0x1a, 0x00, 0x00, 0x00, 0x00, \
  0x30, 0x68, 0x00, 0x00, 0x00, 0x00, \
  0x2c, 0x89, 0x00, 0x00, 0x00, 0x00, \
  0xb0, 0xde, 0x80, 0x00, 0x00, 0x00

#define kMaskRandom17_15 \
  0x15, 0x8c, 0x00, 0x00, 0x00, 0x00, \
  0x8a, 0x47, 0x00, 0x00, 0x00, 0x00, \
  0x25, 0x81, 0x80, 0x00, 0x00, 0x00, \
  0x62, 0x12, 0x80, 0x00, 0x00, 0x00, \
  0x58, 0x58, 0x00, 0x00, 0x00, 0x00, \
  0x0e, 0x28, 0x80, 0x00, 0x00, 0x00, \
  0x83, 0x34, 0x00, 0x00, 0x00, 0x00, \
  0x25, 0x2c, 0x00, 0x00, 0x00, 0x00, \
  0x8a, 0x91, 0x00, 0x00, 0x00, 0x00, \
  0x91, 0xc0, 0x80, 0x00, 0x00, 0x00, \
  0x68, 0x06, 0x80, 0x00, 0x00, 0x00, \
  0x32, 0xc8, 0x00, 0x00, 0x00, 0x00, \
  0x43, 0x45, 0x00, 0x00, 0x00, 0x00, \
  0xc4, 0x30, 0x80, 0x00, 0x00, 0x00, \
  0x1c, 0xa2, 0x00, 0x00, 0x00, 0x00

#define kMaskRandom17_16 \
  0x25, 0x2c, 0x00, 0x00, 0x00, 0x00, \
  0x8a, 0x91, 0x00, 0x00, 0x00, 0x00, \
  0x91, 0xc0, 0x80, 0x00, 0x00, 0x00, \
  0x68, 0x06, 0x80, 0x00, 0x00, 0x00, \
  0x32, 0xc8, 0x00, 0x00, 0x00, 0x00, \
  0x43, 0x45, 0x00, 0x00, 0x00, 0x00, \
  0xc4, 0x30, 0x80, 0x00, 0x00, 0x00, \
  0x1c, 0xa2, 0x00, 0x00, 0x00, 0x00, \
  0x15, 0x8c, 0x00, 0x00, 0x00, 0x00, \
  0x8a, 0x47, 0x00, 0x00, 0x00, 0x00, \
  0x25, 0x81, 0x80, 0x00, 0x00, 0x00, \
  0x62, 0x12, 0x80, 0x00, 0x00, 0x00, \
  0x58, 0x58, 0x00, 0x00, 0x00, 0x00, \
  0x0e, 0x28, 0x80, 0x00, 0x00, 0x00, \
  0x83, 0x34, 0x00, 0x00, 0x00, 0x00, \
  0x0a, 0x1c, 0x00, 0x00, 0x00, 0x00

#define kMaskRandom17_17 \
  0x25, 0x2c, 0x00, 0x00, 0x00, 0x00, \
  0x8a, 0x91, 0x00, 0x00, 0x00, 0x00, \
  0x91, 0xc0, 0x80, 0x00, 0x00, 0x00, \
  0x68, 0x06, 0x80, 0x00, 0x00, 0x00, \
  0x32, 0xc8, 0x00, 0x00, 0x00, 0x00, \
  0x43, 0x45, 0x00, 0x00, 0x00, 0x00, \
  0xc4, 0x30, 0x80, 0x00, 0x00, 0x00, \
  0x1c, 0xa2, 0x00, 0x00, 0x00, 0x00, \
  0x25, 0x4c, 0x00, 0x00, 0x00, 0x00, \
  0x8a, 0x66, 0x00, 0x00, 0x00, 0x00, \
  0x91, 0x91, 0x00, 0x00, 0x00, 0x00, \
  0x68, 0x42, 0x80, 0x00, 0x00, 0x00, \
  0x32, 0xa4, 0x00, 0x00, 0x00, 0x00, \
  0x43, 0x13, 0x00, 0x00, 0x00, 0x00, \
  0xc4, 0x30, 0x80, 0x00, 0x00, 0x00, \
  0x1c, 0x88, 0x80, 0x00, 0x00, 0x00, \
  0x3c, 0x09, 0x00, 0x00, 0x00, 0x00

#define kMaskRandom18_1 \
  0xff, 0xff, 0xc0, 0x00, 0x00, 0x00

#define kMaskRandom18_2 \
  0xce, 0x67, 0x00, 0x00, 0x00, 0x00, \
  0x39, 0x9c, 0xc0, 0x00, 0x00, 0x00

#define kMaskRandom18_3 \
  0xcc, 0x66, 0x00, 0x00, 0x00, 0x00, \
  0x27, 0x15, 0x80, 0x00, 0x00, 0x00, \
  0x92, 0xc9, 0x40, 0x00, 0x00, 0x00

#define kMaskRandom18_4 \
  0xec, 0x76, 0x00, 0x00, 0x00, 0x00, \
  0x67, 0x33, 0x80, 0x00, 0x00, 0x00, \
  0xb1, 0xd8, 0xc0, 0x00, 0x00, 0x00, \
  0x5a, 0xad, 0x40, 0x00, 0x00, 0x00

#define kMaskRandom18_5 \
  0x4c, 0xa6, 0x40, 0x00, 0x00, 0x00, \
  0x66, 0x33, 0x00, 0x00, 0x00, 0x00, \
  0x19, 0xd0, 0xc0, 0x00, 0x00, 0x00, \
  0x9c, 0x89, 0x40, 0x00, 0x00, 0x00, \
  0xe3, 0x4c, 0x00, 0x00, 0x00, 0x00

#define kMaskRandom18_6 \
  0xcc, 0x26, 0x00, 0x00, 0x00, 0x00, \
  0x45, 0x62, 0x80, 0x00, 0x00, 0x00, \
  0xb0, 0x98, 0x40, 0x00, 0x00, 0x00, \
  0x8a, 0x85, 0x40, 0x00, 0x00, 0x00, \
  0x29, 0x53, 0x00, 0x00, 0x00, 0x00, \
  0xa6, 0x0a, 0x80, 0x00, 0x00, 0x00

#define kMaskRandom18_7 \
  0x44, 0xa2, 0x40, 0x00, 0x00, 0x00, \
  0x23, 0x11, 0x80, 0x00, 0x00, 0x00, \
  0x91, 0x48, 0x80, 0x00, 0x00, 0x00, \
  0x0a, 0x85, 0x40, 0x00, 0x00, 0x00, \
  0x34, 0x1a, 0x00, 0x00, 0x00, 0x00, \
  0x0b, 0x06, 0x80, 0x00, 0x00, 0x00, \
  0xe0, 0x64, 0x00, 0x00, 0x00, 0x00

#define kMaskRandom18_8 \
  0x16, 0x0b, 0x00, 0x00, 0x00, 0x00, \
  0xc2, 0x61, 0x00, 0x00, 0x00, 0x00, \
  0x60, 0xb0, 0x40, 0x00, 0x00, 0x00, \
  0x85, 0x42, 0x80, 0x00, 0x00, 0x00, \
  0x4c, 0x26, 0x00, 0x00, 0x00, 0x00, \
  0x29, 0x14, 0x80, 0x00, 0x00, 0x00, \
  0x11, 0x88, 0xc0, 0x00, 0x00, 0x00, \
  0xb0, 0x58, 0x00, 0x00, 0x00, 0x00

#define kMaskRandom18_9 \
  0x44, 0xa2, 0x40, 0x00, 0x00, 0x00, \
  0x66, 0x26, 0x00, 0x00, 0x00, 0x00, \
  0x90, 0x49, 0x40, 0x00, 0x00, 0x00, \
  0x01, 0xa5, 0x80, 0x00, 0x00, 0x00, \
  0x0e, 0x12, 0x80, 0x00, 0x00, 0x00, \
  0x13, 0x0b, 0x00, 0x00, 0x00, 0x00, \
  0x20, 0xd0, 0x40, 0x00, 0x00, 0x00, \
  0xc2, 0x51, 0x00, 0x00, 0x00, 0x00, \
  0x29, 0x0c, 0x80, 0x00, 0x00, 0x00

#define kMaskRandom18_10 \
  0x8c, 0xc6, 0x40, 0x00, 0x00, 0x00, \
  0x27, 0x13, 0x80, 0x00, 0x00, 0x00, \
  0x32, 0x99, 0x40, 0x00, 0x00, 0x00, \
  0x61, 0xb0, 0xc0, 0x00, 0x00, 0x00, \
  0x5c, 0x2e, 0x00, 0x00, 0x00, 0x00, \
  0xcc, 0x66, 0x00, 0x00, 0x00, 0x00, \
  0x2b, 0x15, 0x80, 0x00, 0x00, 0x00, \
  0x32, 0x99, 0x40, 0x00, 0x00, 0x00, \
  0x25, 0x92, 0xc0, 0x00, 0x00, 0x00, \
  0xfd, 0x9d, 0xc0, 0x00, 0x00, 0x00

#define kMaskRandom18_11 \
  0x8c, 0xc6, 0x40, 0x00, 0x00, 0x00, \
  0x27, 0x13, 0x80, 0x00, 0x00, 0x00, \
  0x32, 0x99, 0x40, 0x00, 0x00, 0x00, \
  0x61, 0xb0, 0xc0, 0x00, 0x00, 0x00, \
  0x5c, 0x2e, 0x00, 0x00, 0x00, 0x00, \
  0x84, 0xc2, 0x40, 0x00, 0x00, 0x00, \
  0x27, 0x13, 0x80, 0x00, 0x00, 0x00, \
  0x51, 0xa8, 0xc0, 0x00, 0x00, 0x00, \
  0x1a, 0x0d, 0x00, 0x00, 0x00, 0x00, \
  0x68, 0x34, 0x00, 0x00, 0x00, 0x00, \
  0x89, 0x44, 0x80, 0x00, 0x00, 0x00

#define kMaskRandom18_12 \
  0x84, 0xc2, 0x40, 0x00, 0x00, 0x00, \
  0x27, 0x13, 0x80, 0x00, 0x00, 0x00, \
  0x51, 0xa8, 0xc0, 0x00, 0x00, 0x00, \
  0x1a, 0x0d, 0x00, 0x00, 0x00, 0x00, \
  0x68, 0x34, 0x00, 0x00, 0x00, 0x00, \
  0x89, 0x44, 0x80, 0x00, 0x00, 0x00, \
  0x8c, 0xc6, 0x40, 0x00, 0x00, 0x00, \
  0x27, 0x13, 0x80, 0x00, 0x00, 0x00, \
  0x32, 0x99, 0x40, 0x00, 0x00, 0x00, \
  0x61, 0xb0, 0xc0, 0x00, 0x00, 0x00, \
  0x5c, 0x2e, 0x00, 0x00, 0x00, 0x00, \
  0x5b, 0x0c, 0x40, 0x00, 0x00, 0x00

#define kMaskRandom18_13 \
  0x84, 0xc2, 0x40, 0x00, 0x00, 0x00, \
  0x27, 0x13, 0x80, 0x00, 0x00, 0x00, \
  0x51, 0xa8, 0xc0, 0x00, 0x00, 0x00, \
  0x1a, 0x0d, 0x00, 0x00, 0x00, 0x00, \
  0x68, 0x34, 0x00, 0x00, 0x00, 0x00, \
  0x89, 0x44, 0x80, 0x00, 0x00, 0x00, \
  0x8c, 0x46, 0x00, 0x00, 0x00, 0x00, \
  0x47, 0x23, 0x80, 0x00, 0x00, 0x00, \
  0x81, 0xc0, 0xc0, 0x00, 0x00, 0x00, \
  0x12, 0x89, 0x40, 0x00, 0x00, 0x00, \
  0x58, 0x2c, 0x00, 0x00, 0x00, 0x00, \
  0x28, 0x94, 0x40, 0x00, 0x00, 0x00, \
  0x34, 0x1a, 0x00, 0x00, 0x00, 0x00

#define kMaskRandom18_14 \
  0x8c, 0x46, 0x00, 0x00, 0x00, 0x00, \
  0x47, 0x23, 0x80, 0x00, 0x00, 0x00, \
  0x81, 0xc0, 0xc0, 0x00, 0x00, 0x00, \
  0x12, 0x89, 0x40, 0x00, 0x00, 0x00, \
  0x58, 0x2c, 0x00, 0x00, 0x00, 0x00, \
  0x28, 0x94, 0x40, 0x00, 0x00, 0x00, \
  0x34, 0x1a, 0x00, 0x00, 0x00, 0x00, \
  0x84, 0xc2, 0x40, 0x00, 0x00, 0x00, \
  0x27, 0x13, 0x80, 0x00, 0x00, 0x00, \
  0x51, 0xa8, 0xc0, 0x00, 0x00, 0x00, \
  0x1a, 0x0d, 0x00, 0x00, 0x00, 0x00, \
  0x68, 0x34, 0x00, 0x00, 0x00, 0x00, \
  0x89, 0x44, 0x80, 0x00, 0x00, 0x00, \
  0x7f, 0x4f, 0xc0, 0x00, 0x00, 0x00

#define kMaskRandom18_15 \
  0x8c, 0x46, 0x00, 0x00, 0x00, 0x00, \
  0x47, 0x23, 0x80, 0x00, 0x00, 0x00, \
  0x81, 0xc0, 0xc0, 0x00, 0x00, 0x00, \
  0x12, 0x89, 0x40, 0x00, 0x00, 0x00, \
  0x58, 0x2c, 0x00, 0x00, 0x00, 0x00, \
  0x28, 0x94, 0x40, 0x00, 0x00, 0x00, \
  0x34, 0x1a, 0x00, 0x00, 0x00, 0x00, \
  0x2c, 0x16, 0x00, 0x00, 0x00, 0x00, \
  0x91, 0x48, 0x80, 0x00, 0x00, 0x00, \
  0xc0, 0xe0, 0x40, 0x00, 0x00, 0x00, \
  0x06, 0x83, 0x40, 0x00, 0x00, 0x00, \
  0xc8, 0x64, 0x00, 0x00, 0x00, 0x00, \
  0x45, 0x22, 0x80, 0x00, 0x00, 0x00, \
  0x30, 0x98, 0x40, 0x00, 0x00, 0x00, \
  0xa2, 0x51, 0x00, 0x00, 0x00, 0x00

#define kMaskRandom18_16 \
  0x2c, 0x16, 0x00, 0x00, 0x00, 0x00, \
  0x91, 0x48, 0x80, 0x00, 0x00, 0x00, \
  0xc0, 0xe0, 0x40, 0x00, 0x00, 0x00, \
  0x06, 0x83, 0x40, 0x00, 0x00, 0x00, \
  0xc8, 0x64, 0x00, 0x00, 0x00, 0x00, \
  0x45, 0x22, 0x80, 0x00, 0x00, 0x00, \
  0x30, 0x98, 0x40, 0x00, 0x00, 0x00, \
  0xa2, 0x51, 0x00, 0x00, 0x00, 0x00, \
  0x8c, 0x46, 0x00, 0x00, 0x00, 0x00, \
  0x47, 0x23, 0x80, 0x00, 0x00, 0x00, \
  0x81, 0xc0, 0xc0, 0x00, 0x00, 0x00, \
  0x12, 0x89, 0x40, 0x00, 0x00, 0x00, \
  0x58, 0x2c, 0x00, 0x00, 0x00, 0x00, \
  0x28, 0x94, 0x40, 0x00, 0x00, 0x00, \
  0x34, 0x1a, 0x00, 0x00, 0x00, 0x00, \
  0xef, 0xf2, 0x00, 0x00, 0x00, 0x00

#define kMaskRandom18_17 \
  0x2c, 0x16, 0x00, 0x00, 0x00, 0x00, \
  0x91, 0x48, 0x80, 0x00, 0x00, 0x00, \
  0xc0, 0xe0, 0x40, 0x00, 0x00, 0x00, \
  0x06, 0x83, 0x40, 0x00, 0x00, 0x00, \
  0xc8, 0x64, 0x00, 0x00, 0x00, 0x00, \
  0x45, 0x22, 0x80, 0x00, 0x00, 0x00, \
  0x30, 0x98, 0x40, 0x00, 0x00, 0x00, \
  0xa2, 0x51, 0x00, 0x00, 0x00, 0x00, \
  0x4c, 0x26, 0x00, 0x00, 0x00, 0x00, \
  0x66, 0x33, 0x00, 0x00, 0x00, 0x00, \
  0x91, 0x48, 0x80, 0x00, 0x00, 0x00, \
  0x42, 0xa1, 0x40, 0x00, 0x00, 0x00, \
  0xa4, 0x52, 0x00, 0x00, 0x00, 0x00, \
  0x13, 0x09, 0x80, 0x00, 0x00, 0x00, \
  0x30, 0x98, 0x40, 0x00, 0x00, 0x00, \
  0x88, 0xc4, 0x40, 0x00, 0x00, 0x00, \
  0x09, 0x04, 0x80, 0x00, 0x00, 0x00

#define kMaskRandom18_18 \
  0x4c, 0x26, 0x00, 0x00, 0x00, 0x00, \
  0x66, 0x33, 0x00, 0x00, 0x00, 0x00, \
  0x91, 0x48, 0x80, 0x00, 0x00, 0x00, \
  0x42, 0xa1, 0x40, 0x00, 0x00, 0x00, \
  0xa4, 0x52, 0x00, 0x00, 0x00, 0x00, \
  0x13, 0x09, 0x80, 0x00, 0x00, 0x00, \
  0x30, 0x98, 0x40, 0x00, 0x00, 0x00, \
  0x88, 0xc4, 0x40, 0x00, 0x00, 0x00, \
  0x09, 0x04, 0x80, 0x00, 0x00, 0x00, \
  0x2c, 0x16, 0x00, 0x00, 0x00, 0x00, \
  0x91, 0x48, 0x80, 0x00, 0x00, 0x00, \
  0xc0, 0xe0, 0x40, 0x00, 0x00, 0x00, \
  0x06, 0x83, 0x40, 0x00, 0x00, 0x00, \
  0xc8, 0x64, 0x00, 0x00, 0x00, 0x00, \
  0x45, 0x22, 0x80, 0x00, 0x00, 0x00, \
  0x30, 0x98, 0x40, 0x00, 0x00, 0x00, \
  0xa2, 0x51, 0x00, 0x00, 0x00, 0x00, \
  0xd0, 0x03, 0x40, 0x00, 0x00, 0x00

#define kMaskRandom19_1 \
  0xff, 0xff, 0xe0, 0x00, 0x00, 0x00

#define kMaskRandom19_10 \
  0x8c, 0xe3, 0x00, 0x00, 0x00, 0x00, \
  0x27, 0x11, 0xc0, 0x00, 0x00, 0x00, \
  0x32, 0x8d, 0x20, 0x00, 0x00, 0x00, \
  0x61, 0x92, 0x60, 0x00, 0x00, 0x00, \
  0x5c, 0x38, 0x80, 0x00, 0x00, 0x00, \
  0xcc, 0x75, 0x00, 0x00, 0x00, 0x00, \
  0x2b, 0x19, 0xc0, 0x00, 0x00, 0x00, \
  0x32, 0xd2, 0x60, 0x00, 0x00, 0x00, \
  0x25, 0x8e, 0xa0, 0x00, 0x00, 0x00, \
  0x50, 0x88, 0xc0, 0x00, 0x00, 0x00

#define kMaskRandom19_11 \
  0x8c, 0xe3, 0x00, 0x00, 0x00, 0x00, \
  0x27, 0x11, 0xc0, 0x00, 0x00, 0x00, \
  0x32, 0x8d, 0x20, 0x00, 0x00, 0x00, \
  0x61, 0x92, 0x60, 0x00, 0x00, 0x00, \
  0x5c, 0x38, 0x80, 0x00, 0x00, 0x00, \
  0x84, 0x87, 0x00, 0x00, 0x00, 0x00, \
  0x27, 0x19, 0x80, 0x00, 0x00, 0x00, \
  0x51, 0x88, 0x60, 0x00, 0x00, 0x00, \
  0x1a, 0x22, 0xa0, 0x00, 0x00, 0x00, \
  0x68, 0x44, 0x40, 0x00, 0x00, 0x00, \
  0x89, 0x70, 0x00, 0x00, 0x00, 0x00

#define kMaskRandom19_12 \
  0x84, 0x87, 0x00, 0x00, 0x00, 0x00, \
  0x27, 0x19, 0x80, 0x00, 0x00, 0x00, \
  0x51, 0x88, 0x60, 0x00, 0x00, 0x00, \
  0x1a, 0x22, 0xa0, 0x00, 0x00, 0x00, \
  0x68, 0x44, 0x40, 0x00, 0x00, 0x00, \
  0x89, 0x70, 0x00, 0x00, 0x00, 0x00, \
  0x8c, 0xe3, 0x00, 0x00, 0x00, 0x00, \
  0x27, 0x11, 0xc0, 0x00, 0x00, 0x00, \
  0x32, 0x8d, 0x20, 0x00, 0x00, 0x00, \
  0x61, 0x92, 0x60, 0x00, 0x00, 0x00, \
  0x5c, 0x38, 0x80, 0x00, 0x00, 0x00, \
  0x90, 0xc8, 0x80, 0x00, 0x00, 0x00

#define kMaskRandom19_13 \
  0x84, 0x87, 0x00, 0x00, 0x00, 0x00, \
  0x27, 0x19, 0x80, 0x00, 0x00, 0x00, \
  0x51, 0x88, 0x60, 0x00, 0x00, 0x00, \
  0x1a, 0x22, 0xa0, 0x00, 0x00, 0x00, \
  0x68, 0x44, 0x40, 0x00, 0x00, 0x00, \
  0x89, 0x70, 0x00, 0x00, 0x00, 0x00, \
  0x8c, 0x23, 0x00, 0x00, 0x00, 0x00, \
  0x47, 0x19, 0x80, 0x00, 0x00, 0x00, \
  0x81, 0x88, 0x60, 0x00, 0x00, 0x00, \
  0x12, 0x86, 0x20, 0x00, 0x00, 0x00, \
  0x58, 0x14, 0x40, 0x00, 0x00, 0x00, \
  0x28, 0xca, 0x00, 0x00, 0x00, 0x00, \
  0x34, 0x60, 0x80, 0x00, 0x00, 0x00

#define kMaskRandom19_14 \
  0x8c, 0x23, 0x00, 0x00, 0x00, 0x00, \
  0x47, 0x19, 0x80, 0x00, 0x00, 0x00, \
  0x81, 0x88, 0x60, 0x00, 0x00, 0x00, \
  0x12, 0x86, 0x20, 0x00, 0x00, 0x00, \
  0x58, 0x14, 0x40, 0x00, 0x00, 0x00, \
  0x28, 0xca, 0x00, 0x00, 0x00, 0x00, \
  0x34, 0x60, 0x80, 0x00, 0x00, 0x00, \
  0x84, 0x87, 0x00, 0x00, 0x00, 0x00, \
  0x27, 0x19, 0x80, 0x00, 0x00, 0x00, \
  0x51, 0x88, 0x60, 0x00, 0x00, 0x00, \
  0x1a, 0x22, 0xa0, 0x00, 0x00, 0x00, \
  0x68, 0x44, 0x40, 0x00, 0x00, 0x00, \
  0x89, 0x70, 0x00, 0x00, 0x00, 0x00, \
  0x6e, 0x27, 0x60, 0x00, 0x00, 0x00

#define kMaskRandom19_15 \
  0x8c, 0x23, 0x00, 0x00, 0x00, 0x00, \
  0x47, 0x19, 0x80, 0x00, 0x00, 0x00, \
  0x81, 0x88, 0x60, 0x00, 0x00, 0x00, \
  0x12, 0x86, 0x20, 0x00, 0x00, 0x00, \
  0x58, 0x14, 0x40, 0x00, 0x00, 0x00, \
  0x28, 0xca, 0x00, 0x00, 0x00, 0x00, \
  0x34, 0x60, 0x80, 0x00, 0x00, 0x00, \
  0x2c, 0x16, 0x00, 0x00, 0x00, 0x00, \
  0x91, 0x40, 0xc0, 0x00, 0x00, 0x00, \
  0xc0, 0xd0, 0x20, 0x00, 0x00, 0x00, \
  0x06, 0x82, 0xa0, 0x00, 0x00, 0x00, \
  0xc8, 0x0c, 0x40, 0x00, 0x00, 0x00, \
  0x45, 0x61, 0x00, 0x00, 0x00, 0x00, \
  0x30, 0x91, 0x40, 0x00, 0x00, 0x00, \
  0xa2, 0x28, 0x20, 0x00, 0x00, 0x00

#define kMaskRandom19_16 \
  0x2c, 0x16, 0x00, 0x00, 0x00, 0x00, \
  0x91, 0x40, 0xc0, 0x00, 0x00, 0x00, \
  0xc0, 0xd0, 0x20, 0x00, 0x00, 0x00, \
  0x06, 0x82, 0xa0, 0x00, 0x00, 0x00, \
  0xc8, 0x0c, 0x40, 0x00, 0x00, 0x00, \
  0x45, 0x61, 0x00, 0x00, 0x00, 0x00, \
  0x30, 0x91, 0x40, 0x00, 0x00, 0x00, \
  0xa2, 0x28, 0x20, 0x00, 0x00, 0x00, \
  0x8c, 0x23, 0x00, 0x00, 0x00, 0x00, \
  0x47, 0x19, 0x80, 0x00, 0x00, 0x00, \
  0x81, 0x88, 0x60, 0x00, 0x00, 0x00, \
  0x12, 0x86, 0x20, 0x00, 0x00, 0x00, \
  0x58, 0x14, 0x40, 0x00, 0x00, 0x00, \
  0x28, 0xca, 0x00, 0x00, 0x00, 0x00, \
  0x34, 0x60, 0x80, 0x00, 0x00, 0x00, \
  0x7e, 0x75, 0xe0, 0x00, 0x00, 0x00

#define kMaskRandom19_17 \
  0x2c, 0x16, 0x00, 0x00, 0x00, 0x00, \
  0x91, 0x40, 0xc0, 0x00, 0x00, 0x00, \
  0xc0, 0xd0, 0x20, 0x00, 0x00, 0x00, \
  0x06, 0x82, 0xa0, 0x00, 0x00, 0x00, \
  0xc8, 0x0c, 0x40, 0x00, 0x00, 0x00, \
  0x45, 0x61, 0x00, 0x00, 0x00, 0x00, \
  0x30, 0x91, 0x40, 0x00, 0x00, 0x00, \
  0xa2, 0x28, 0x20, 0x00, 0x00, 0x00, \
  0x4c, 0x27, 0x00, 0x00, 0x00, 0x00, \
  0x66, 0x71, 0x80, 0x00, 0x00, 0x00, \
  0x91, 0x40, 0xe0, 0x00, 0x00, 0x00, \
  0x42, 0x90, 0xa0, 0x00, 0x00, 0x00, \
  0xa4, 0x29, 0x40, 0x00, 0x00, 0x00, \
  0x13, 0x5a, 0x00, 0x00, 0x00, 0x00, \
  0x30, 0x93, 0x40, 0x00, 0x00, 0x00, \
  0x88, 0xac, 0x20, 0x00, 0x00, 0x00, \
  0x09, 0x0c, 0xc0, 0x00, 0x00, 0x00

#define kMaskRandom19_18 \
  0x4c, 0x27, 0x00, 0x00, 0x00, 0x00, \
  0x66, 0x71, 0x80, 0x00, 0x00, 0x00, \
  0x91, 0x40, 0xe0, 0x00, 0x00, 0x00, \
  0x42, 0x90, 0xa0, 0x00, 0x00, 0x00, \
  0xa4, 0x29, 0x40, 0x00, 0x00, 0x00, \
  0x13, 0x5a, 0x00, 0x00, 0x00, 0x00, \
  0x30, 0x93, 0x40, 0x00, 0x00, 0x00, \
  0x88, 0xac, 0x20, 0x00, 0x00, 0x00, \
  0x09, 0x0c, 0xc0, 0x00, 0x00, 0x00, \
  0x2c, 0x16, 0x00, 0x00, 0x00, 0x00, \
  0x91, 0x40, 0xc0, 0x00, 0x00, 0x00, \
  0xc0, 0xd0, 0x20, 0x00, 0x00, 0x00, \
  0x06, 0x82, 0xa0, 0x00, 0x00, 0x00, \
  0xc8, 0x0c, 0x40, 0x00, 0x00, 0x00, \
  0x45, 0x61, 0x00, 0x00, 0x00, 0x00, \
  0x30, 0x91, 0x40, 0x00, 0x00, 0x00, \
  0xa2, 0x28, 0x20, 0x00, 0x00, 0x00, \
  0x51, 0x97, 0x20, 0x00, 0x00, 0x00

#define kMaskRandom19_19 \
  0x4c, 0x27, 0x00, 0x00, 0x00, 0x00, \
  0x66, 0x71, 0x80, 0x00, 0x00, 0x00, \
  0x91, 0x40, 0xe0, 0x00, 0x00, 0x00, \
  0x42, 0x90, 0xa0, 0x00, 0x00, 0x00, \
  0xa4, 0x29, 0x40, 0x00, 0x00, 0x00, \
  0x13, 0x5a, 0x00, 0x00, 0x00, 0x00, \
  0x30, 0x93, 0x40, 0x00, 0x00, 0x00, \
  0x88, 0xac, 0x20, 0x00, 0x00, 0x00, \
  0x09, 0x0c, 0xc0, 0x00, 0x00, 0x00, \
  0x4c, 0x26, 0x00, 0x00, 0x00, 0x00, \
  0x66, 0x28, 0x80, 0x00, 0x00, 0x00, \
  0x91, 0x50, 0x20, 0x00, 0x00, 0x00, \
  0x42, 0x82, 0x60, 0x00, 0x00, 0x00, \
  0xa4, 0x01, 0xc0, 0x00, 0x00, 0x00, \
  0x13, 0x43, 0x00, 0x00, 0x00, 0x00, \
  0x30, 0x94, 0x80, 0x00, 0x00, 0x00, \
  0x88, 0xa1, 0x20, 0x00, 0x00, 0x00, \
  0x09, 0x4c, 0x00, 0x00, 0x00, 0x00, \
  0xcd, 0x98, 0x40, 0x00, 0x00, 0x00

#define kMaskRandom19_2 \
  0xce, 0x77, 0x00, 0x00, 0x00, 0x00, \
  0x39, 0xcc, 0xe0, 0x00, 0x00, 0x00

#define kMaskRandom19_3 \
  0xcc, 0x67, 0x00, 0x00, 0x00, 0x00, \
  0x27, 0x2c, 0xc0, 0x00, 0x00, 0x00, \
  0x92, 0xd2, 0x60, 0x00, 0x00, 0x00

#define kMaskRandom19_4 \
  0xec, 0x73, 0x00, 0x00, 0x00, 0x00, \
  0x67, 0x19, 0xc0, 0x00, 0x00, 0x00, \
  0xb1, 0xcc, 0x60, 0x00, 0x00, 0x00, \
  0x5a, 0x96, 0xa0, 0x00, 0x00, 0x00

#define kMaskRandom19_5 \
  0x4c, 0xe7, 0x00, 0x00, 0x00, 0x00, \
  0x66, 0x31, 0xc0, 0x00, 0x00, 0x00, \
  0xa1, 0xcc, 0x60, 0x00, 0x00, 0x00, \
  0x92, 0xa6, 0xa0, 0x00, 0x00, 0x00, \
  0xb8, 0x99, 0x80, 0x00, 0x00, 0x00

#define kMaskRandom19_6 \
  0x4c, 0x36, 0x00, 0x00, 0x00, 0x00, \
  0x45, 0x68, 0x80, 0x00, 0x00, 0x00, \
  0x30, 0xd0, 0x60, 0x00, 0x00, 0x00, \
  0x8a, 0x82, 0xa0, 0x00, 0x00, 0x00, \
  0x26, 0x0b, 0x40, 0x00, 0x00, 0x00, \
  0x95, 0x45, 0x00, 0x00, 0x00, 0x00

#define kMaskRandom19_7 \
  0xc4, 0xa3, 0x00, 0x00, 0x00, 0x00, \
  0x23, 0x19, 0x80, 0x00, 0x00, 0x00, \
  0x91, 0x1c, 0x20, 0x00, 0x00, 0x00, \
  0x4a, 0x82, 0xa0, 0x00, 0x00, 0x00, \
  0x34, 0x49, 0x40, 0x00, 0x00, 0x00, \
  0x8b, 0x4a, 0x00, 0x00, 0x00, 0x00, \
  0xc8, 0x24, 0xc0, 0x00, 0x00, 0x00

#define kMaskRandom19_8 \
  0x16, 0x13, 0x80, 0x00, 0x00, 0x00, \
  0xc2, 0x44, 0xc0, 0x00, 0x00, 0x00, \
  0x60, 0xe8, 0x20, 0x00, 0x00, 0x00, \
  0x85, 0x12, 0x60, 0x00, 0x00, 0x00, \
  0xcc, 0x21, 0x40, 0x00, 0x00, 0x00, \
  0x29, 0x63, 0x00, 0x00, 0x00, 0x00, \
  0x11, 0x98, 0xc0, 0x00, 0x00, 0x00, \
  0xb0, 0x0c, 0x60, 0x00, 0x00, 0x00

#define kMaskRandom19_9 \
  0x44, 0xa7, 0x00, 0x00, 0x00, 0x00, \
  0x66, 0x70, 0x80, 0x00, 0x00, 0x00, \
  0x12, 0xc0, 0xe0, 0x00, 0x00, 0x00, \
  0xc3, 0x10, 0xa0, 0x00, 0x00, 0x00, \
  0x8c, 0x29, 0x40, 0x00, 0x00, 0x00, \
  0x11, 0x5b, 0x00, 0x00, 0x00, 0x00, \
  0x21, 0x93, 0x40, 0x00, 0x00, 0x00, \
  0xa2, 0x2c, 0x00, 0x00, 0x00, 0x00, \
  0x18, 0x0c, 0xe0, 0x00, 0x00, 0x00

#define kMaskRandom20_1 \
  0xff, 0xff, 0xf0, 0x00, 0x00, 0x00

#define kMaskRandom20_10 \
  0x4c, 0x13, 0x00, 0x00, 0x00, 0x00, \
  0x51, 0x14, 0x40, 0x00, 0x00, 0x00, \
  0xa0, 0x68, 0x10, 0x00, 0x00, 0x00, \
  0x04, 0xc1, 0x30, 0x00, 0x00, 0x00, \
  0x03, 0x80, 0xe0, 0x00, 0x00, 0x00, \
  0x86, 0x21, 0x80, 0x00, 0x00, 0x00, \
  0x29, 0x0a, 0x40, 0x00, 0x00, 0x00, \
  0x42, 0x50, 0x90, 0x00, 0x00, 0x00, \
  0x98, 0x26, 0x00, 0x00, 0x00, 0x00, \
  0x30, 0x8c, 0x20, 0x00, 0x00, 0x00

#define kMaskRandom20_11 \
  0xc6, 0x31, 0x80, 0x00, 0x00, 0x00, \
  0x23, 0x88, 0xe0, 0x00, 0x00, 0x00, \
  0x1a, 0x46, 0x90, 0x00, 0x00, 0x00, \
  0x24, 0xc9, 0x30, 0x00, 0x00, 0x00, \
  0x71, 0x1c, 0x40, 0x00, 0x00, 0x00, \
  0x0e, 0x03, 0x80, 0x00, 0x00, 0x00, \
  0x33, 0x0c, 0xc0, 0x00, 0x00, 0x00, \
  0x10, 0xc4, 0x30, 0x00, 0x00, 0x00, \
  0x45, 0x51, 0x50, 0x00, 0x00, 0x00, \
  0x88, 0xa2, 0x20, 0x00, 0x00, 0x00, \
  0xe0, 0x38, 0x00, 0x00, 0x00, 0x00

#define kMaskRandom20_12 \
  0x0e, 0x03, 0x80, 0x00, 0x00, 0x00, \
  0x33, 0x0c, 0xc0, 0x00, 0x00, 0x00, \
  0x10, 0xc4, 0x30, 0x00, 0x00, 0x00, \
  0x45, 0x51, 0x50, 0x00, 0x00, 0x00, \
  0x88, 0xa2, 0x20, 0x00, 0x00, 0x00, \
  0xe0, 0x38, 0x00, 0x00, 0x00, 0x00, \
  0xc6, 0x31, 0x80, 0x00, 0x00, 0x00, \
  0x23, 0x88, 0xe0, 0x00, 0x00, 0x00, \
  0x1a, 0x46, 0x90, 0x00, 0x00, 0x00, \
  0x24, 0xc9, 0x30, 0x00, 0x00, 0x00, \
  0x71, 0x1c, 0x40, 0x00, 0x00, 0x00, \
  0xf5, 0xdc, 0x40, 0x00, 0x00, 0x00

#define kMaskRandom20_13 \
  0x0e, 0x03, 0x80, 0x00, 0x00, 0x00, \
  0x33, 0x0c, 0xc0, 0x00, 0x00, 0x00, \
  0x10, 0xc4, 0x30, 0x00, 0x00, 0x00, \
  0x45, 0x51, 0x50, 0x00, 0x00, 0x00, \
  0x88, 0xa2, 0x20, 0x00, 0x00, 0x00, \
  0xe0, 0x38, 0x00, 0x00, 0x00, 0x00, \
  0x46, 0x11, 0x80, 0x00, 0x00, 0x00, \
  0x33, 0x0c, 0xc0, 0x00, 0x00, 0x00, \
  0x10, 0xc4, 0x30, 0x00, 0x00, 0x00, \
  0x0c, 0x43, 0x10, 0x00, 0x00, 0x00, \
  0x28, 0x8a, 0x20, 0x00, 0x00, 0x00, \
  0x94, 0x25, 0x00, 0x00, 0x00, 0x00, \
  0xc1, 0x30, 0x40, 0x00, 0x00, 0x00

#define kMaskRandom20_14 \
  0x46, 0x11, 0x80, 0x00, 0x00, 0x00, \
  0x33, 0x0c, 0xc0, 0x00, 0x00, 0x00, \
  0x10, 0xc4, 0x30, 0x00, 0x00, 0x00, \
  0x0c, 0x43, 0x10, 0x00, 0x00, 0x00, \
  0x28, 0x8a, 0x20, 0x00, 0x00, 0x00, \
  0x94, 0x25, 0x00, 0x00, 0x00, 0x00, \
  0xc1, 0x30, 0x40, 0x00, 0x00, 0x00, \
  0x0e, 0x03, 0x80, 0x00, 0x00, 0x00, \
  0x33, 0x0c, 0xc0, 0x00, 0x00, 0x00, \
  0x10, 0xc4, 0x30, 0x00, 0x00, 0x00, \
  0x45, 0x51, 0x50, 0x00, 0x00, 0x00, \
  0x88, 0xa2, 0x20, 0x00, 0x00, 0x00, \
  0xe0, 0x38, 0x00, 0x00, 0x00, 0x00, \
  0x56, 0x3e, 0x20, 0x00, 0x00, 0x00

#define kMaskRandom20_15 \
  0x46, 0x11, 0x80, 0x00, 0x00, 0x00, \
  0x33, 0x0c, 0xc0, 0x00, 0x00, 0x00, \
  0x10, 0xc4, 0x30, 0x00, 0x00, 0x00, \
  0x0c, 0x43, 0x10, 0x00, 0x00, 0x00, \
  0x28, 0x8a, 0x20, 0x00, 0x00, 0x00, \
  0x94, 0x25, 0x00, 0x00, 0x00, 0x00, \
  0xc1, 0x30, 0x40, 0x00, 0x00, 0x00, \
  0x2c, 0x0b, 0x00, 0x00, 0x00, 0x00, \
  0x81, 0xa0, 0x60, 0x00, 0x00, 0x00, \
  0xa0, 0x68, 0x10, 0x00, 0x00, 0x00, \
  0x05, 0x41, 0x50, 0x00, 0x00, 0x00, \
  0x18, 0x86, 0x20, 0x00, 0x00, 0x00, \
  0xc2, 0x30, 0x80, 0x00, 0x00, 0x00, \
  0x22, 0x88, 0xa0, 0x00, 0x00, 0x00, \
  0x50, 0x54, 0x10, 0x00, 0x00, 0x00

#define kMaskRandom20_16 \
  0x2c, 0x0b, 0x00, 0x00, 0x00, 0x00, \
  0x81, 0xa0, 0x60, 0x00, 0x00, 0x00, \
  0xa0, 0x68, 0x10, 0x00, 0x00, 0x00, \
  0x05, 0x41, 0x50, 0x00, 0x00, 0x00, \
  0x18, 0x86, 0x20, 0x00, 0x00, 0x00, \
  0xc2, 0x30, 0x80, 0x00, 0x00, 0x00, \
  0x22, 0x88, 0xa0, 0x00, 0x00, 0x00, \
  0x50, 0x54, 0x10, 0x00, 0x00, 0x00, \
  0x46, 0x11, 0x80, 0x00, 0x00, 0x00, \
  0x33, 0x0c, 0xc0, 0x00, 0x00, 0x00, \
  0x10, 0xc4, 0x30, 0x00, 0x00, 0x00, \
  0x0c, 0x43, 0x10, 0x00, 0x00, 0x00, \
  0x28, 0x8a, 0x20, 0x00, 0x00, 0x00, \
  0x94, 0x25, 0x00, 0x00, 0x00, 0x00, \
  0xc1, 0x30, 0x40, 0x00, 0x00, 0x00, \
  0x28, 0x1c, 0x60, 0x00, 0x00, 0x00

#define kMaskRandom20_17 \
  0x2c, 0x0b, 0x00, 0x00, 0x00, 0x00, \
  0x81, 0xa0, 0x60, 0x00, 0x00, 0x00, \
  0xa0, 0x68, 0x10, 0x00, 0x00, 0x00, \
  0x05, 0x41, 0x50, 0x00, 0x00, 0x00, \
  0x18, 0x86, 0x20, 0x00, 0x00, 0x00, \
  0xc2, 0x30, 0x80, 0x00, 0x00, 0x00, \
  0x22, 0x88, 0xa0, 0x00, 0x00, 0x00, \
  0x50, 0x54, 0x10, 0x00, 0x00, 0x00, \
  0x4e, 0x13, 0x80, 0x00, 0x00, 0x00, \
  0xe3, 0x38, 0xc0, 0x00, 0x00, 0x00, \
  0x81, 0xe0, 0x70, 0x00, 0x00, 0x00, \
  0x21, 0x48, 0x50, 0x00, 0x00, 0x00, \
  0x52, 0x94, 0xa0, 0x00, 0x00, 0x00, \
  0xb4, 0x2d, 0x00, 0x00, 0x00, 0x00, \
  0x26, 0x89, 0xa0, 0x00, 0x00, 0x00, \
  0x58, 0x56, 0x10, 0x00, 0x00, 0x00, \
  0x19, 0x86, 0x60, 0x00, 0x00, 0x00

#define kMaskRandom20_18 \
  0x4e, 0x13, 0x80, 0x00, 0x00, 0x00, \
  0xe3, 0x38, 0xc0, 0x00, 0x00, 0x00, \
  0x81, 0xe0, 0x70, 0x00, 0x00, 0x00, \
  0x21, 0x48, 0x50, 0x00, 0x00, 0x00, \
  0x52, 0x94, 0xa0, 0x00, 0x00, 0x00, \
  0xb4, 0x2d, 0x00, 0x00, 0x00, 0x00, \
  0x26, 0x89, 0xa0, 0x00, 0x00, 0x00, \
  0x58, 0x56, 0x10, 0x00, 0x00, 0x00, \
  0x19, 0x86, 0x60, 0x00, 0x00, 0x00, \
  0x2c, 0x0b, 0x00, 0x00, 0x00, 0x00, \
  0x81, 0xa0, 0x60, 0x00, 0x00, 0x00, \
  0xa0, 0x68, 0x10, 0x00, 0x00, 0x00, \
  0x05, 0x41, 0x50, 0x00, 0x00, 0x00, \
  0x18, 0x86, 0x20, 0x00, 0x00, 0x00, \
  0xc2, 0x30, 0x80, 0x00, 0x00, 0x00, \
  0x22, 0x88, 0xa0, 0x00, 0x00, 0x00, \
  0x50, 0x54, 0x10, 0x00, 0x00, 0x00, \
  0x21, 0x7b, 0xf0, 0x00, 0x00, 0x00

#define kMaskRandom20_19 \
  0x4e, 0x13, 0x80, 0x00, 0x00, 0x00, \
  0xe3, 0x38, 0xc0, 0x00, 0x00, 0x00, \
  0x81, 0xe0, 0x70, 0x00, 0x00, 0x00, \
  0x21, 0x48, 0x50, 0x00, 0x00, 0x00, \
  0x52, 0x94, 0xa0, 0x00, 0x00, 0x00, \
  0xb4, 0x2d, 0x00, 0x00, 0x00, 0x00, \
  0x26, 0x89, 0xa0, 0x00, 0x00, 0x00, \
  0x58, 0x56, 0x10, 0x00, 0x00, 0x00, \
  0x19, 0x86, 0x60, 0x00, 0x00, 0x00, \
  0x4c, 0x13, 0x00, 0x00, 0x00, 0x00, \
  0x51, 0x14, 0x40, 0x00, 0x00, 0x00, \
  0xa0, 0x68, 0x10, 0x00, 0x00, 0x00, \
  0x04, 0xc1, 0x30, 0x00, 0x00, 0x00, \
  0x03, 0x80, 0xe0, 0x00, 0x00, 0x00, \
  0x86, 0x21, 0x80, 0x00, 0x00, 0x00, \
  0x29, 0x0a, 0x40, 0x00, 0x00, 0x00, \
  0x42, 0x50, 0x90, 0x00, 0x00, 0x00, \
  0x98, 0x26, 0x00, 0x00, 0x00, 0x00, \
  0x30, 0x8c, 0x20, 0x00, 0x00, 0x00

#define kMaskRandom20_2 \
  0xee, 0x3b, 0x80, 0x00, 0x00, 0x00, \
  0x99, 0xe6, 0x70, 0x00, 0x00, 0x00

#define kMaskRandom20_20 \
  0x4c, 0x13, 0x00, 0x00, 0x00, 0x00, \
  0x51, 0x14, 0x40, 0x00, 0x00, 0x00, \
  0xa0, 0x68, 0x10, 0x00, 0x00, 0x00, \
  0x04, 0xc1, 0x30, 0x00, 0x00, 0x00, \
  0x03, 0x80, 0xe0, 0x00, 0x00, 0x00, \
  0x86, 0x21, 0x80, 0x00, 0x00, 0x00, \
  0x29, 0x0a, 0x40, 0x00, 0x00, 0x00, \
  0x42, 0x50, 0x90, 0x00, 0x00, 0x00, \
  0x98, 0x26, 0x00, 0x00, 0x00, 0x00, \
  0x30, 0x8c, 0x20, 0x00, 0x00, 0x00, \
  0x4e, 0x13, 0x80, 0x00, 0x00, 0x00, \
  0xe3, 0x38, 0xc0, 0x00, 0x00, 0x00, \
  0x81, 0xe0, 0x70, 0x00, 0x00, 0x00, \
  0x21, 0x48, 0x50, 0x00, 0x00, 0x00, \
  0x52, 0x94, 0xa0, 0x00, 0x00, 0x00, \
  0xb4, 0x2d, 0x00, 0x00, 0x00, 0x00, \
  0x26, 0x89, 0xa0, 0x00, 0x00, 0x00, \
  0x58, 0x56, 0x10, 0x00, 0x00, 0x00, \
  0x19, 0x86, 0x60, 0x00, 0x00, 0x00, \
  0xf7, 0x8d, 0xa0, 0x00, 0x00, 0x00

#define kMaskRandom20_3 \
  0xce, 0x33, 0x80, 0x00, 0x00, 0x00, \
  0x55, 0x95, 0x60, 0x00, 0x00, 0x00, \
  0xb1, 0x6a, 0x30, 0x00, 0x00, 0x00

#define kMaskRandom20_4 \
  0xe6, 0x39, 0x80, 0x00, 0x00, 0x00, \
  0x33, 0x8c, 0xe0, 0x00, 0x00, 0x00, \
  0x98, 0xe6, 0x30, 0x00, 0x00, 0x00, \
  0x2d, 0x4b, 0x50, 0x00, 0x00, 0x00

#define kMaskRandom20_5 \
  0xce, 0x33, 0x80, 0x00, 0x00, 0x00, \
  0x63, 0x98, 0xe0, 0x00, 0x00, 0x00, \
  0x98, 0xe5, 0x30, 0x00, 0x00, 0x00, \
  0x2b, 0x53, 0x50, 0x00, 0x00, 0x00, \
  0xb4, 0x5c, 0xa0, 0x00, 0x00, 0x00

#define kMaskRandom20_6 \
  0x4c, 0x1b, 0x00, 0x00, 0x00, 0x00, \
  0x51, 0x34, 0x40, 0x00, 0x00, 0x00, \
  0x20, 0xe8, 0x30, 0x00, 0x00, 0x00, \
  0x85, 0x41, 0x50, 0x00, 0x00, 0x00, \
  0x06, 0x86, 0xa0, 0x00, 0x00, 0x00, \
  0x9a, 0x21, 0x80, 0x00, 0x00, 0x00

#define kMaskRandom20_7 \
  0x4e, 0x11, 0x80, 0x00, 0x00, 0x00, \
  0x33, 0x2c, 0x00, 0x00, 0x00, 0x00, \
  0x10, 0x0e, 0xb0, 0x00, 0x00, 0x00, \
  0x81, 0x51, 0x50, 0x00, 0x00, 0x00, \
  0x24, 0xc4, 0xa0, 0x00, 0x00, 0x00, \
  0xd4, 0x23, 0x00, 0x00, 0x00, 0x00, \
  0x0c, 0xa2, 0x60, 0x00, 0x00, 0x00

#define kMaskRandom20_8 \
  0x27, 0x09, 0xc0, 0x00, 0x00, 0x00, \
  0x89, 0xa2, 0x60, 0x00, 0x00, 0x00, \
  0xd0, 0x74, 0x10, 0x00, 0x00, 0x00, \
  0x24, 0xc9, 0x30, 0x00, 0x00, 0x00, \
  0xe2, 0x90, 0xa0, 0x00, 0x00, 0x00, \
  0xc6, 0x31, 0x80, 0x00, 0x00, 0x00, \
  0x31, 0x8c, 0x60, 0x00, 0x00, 0x00, \
  0x18, 0xc6, 0x30, 0x00, 0x00, 0x00

#define kMaskRandom20_9 \
  0x4e, 0x13, 0x80, 0x00, 0x00, 0x00, \
  0x62, 0x38, 0xc0, 0x00, 0x00, 0x00, \
  0x81, 0xe0, 0x70, 0x00, 0x00, 0x00, \
  0xe1, 0x48, 0x50, 0x00, 0x00, 0x00, \
  0x13, 0x94, 0xa0, 0x00, 0x00, 0x00, \
  0xb4, 0x2d, 0x00, 0x00, 0x00, 0x00, \
  0x26, 0x89, 0xa0, 0x00, 0x00, 0x00, \
  0x58, 0x56, 0x10, 0x00, 0x00, 0x00, \
  0x49, 0x86, 0x50, 0x00, 0x00, 0x00

#define kMaskRandom21_1 \
  0xff, 0xff, 0xf8, 0x00, 0x00, 0x00

#define kMaskRandom21_10 \
  0x4c, 0x19, 0x10, 0x00, 0x00, 0x00, \
  0x51, 0x14, 0x50, 0x00, 0x00, 0x00, \
  0xa0, 0x6a, 0x40, 0x00, 0x00, 0x00, \
  0x04, 0xc1, 0x30, 0x00, 0x00, 0x00, \
  0x03, 0xb4, 0x00, 0x00, 0x00, 0x00, \
  0x86, 0x20, 0x90, 0x00, 0x00, 0x00, \
  0x29, 0x08, 0x48, 0x00, 0x00, 0x00, \
  0x42, 0x43, 0x08, 0x00, 0x00, 0x00, \
  0x98, 0x12, 0x80, 0x00, 0x00, 0x00, \
  0x30, 0x84, 0xa8, 0x00, 0x00, 0x00

#define kMaskRandom21_11 \
  0xc6, 0x21, 0xa0, 0x00, 0x00, 0x00, \
  0x23, 0x88, 0xc8, 0x00, 0x00, 0x00, \
  0x1a, 0x45, 0x88, 0x00, 0x00, 0x00, \
  0x24, 0xd3, 0x08, 0x00, 0x00, 0x00, \
  0x71, 0x10, 0x70, 0x00, 0x00, 0x00, \
  0x0e, 0x19, 0x10, 0x00, 0x00, 0x00, \
  0x33, 0x14, 0x50, 0x00, 0x00, 0x00, \
  0x10, 0xc3, 0x28, 0x00, 0x00, 0x00, \
  0x45, 0x68, 0x48, 0x00, 0x00, 0x00, \
  0x88, 0x84, 0xa8, 0x00, 0x00, 0x00, \
  0xe0, 0x22, 0x90, 0x00, 0x00, 0x00

#define kMaskRandom21_12 \
  0x0e, 0x19, 0x10, 0x00, 0x00, 0x00, \
  0x33, 0x14, 0x50, 0x00, 0x00, 0x00, \
  0x10, 0xc3, 0x28, 0x00, 0x00, 0x00, \
  0x45, 0x68, 0x48, 0x00, 0x00, 0x00, \
  0x88, 0x84, 0xa8, 0x00, 0x00, 0x00, \
  0xe0, 0x22, 0x90, 0x00, 0x00, 0x00, \
  0xc6, 0x21, 0xa0, 0x00, 0x00, 0x00, \
  0x23, 0x88, 0xc8, 0x00, 0x00, 0x00, \
  0x1a, 0x45, 0x88, 0x00, 0x00, 0x00, \
  0x24, 0xd3, 0x08, 0x00, 0x00, 0x00, \
  0x71, 0x10, 0x70, 0x00, 0x00, 0x00, \
  0xa0, 0x65, 0x18, 0x00, 0x00, 0x00

#define kMaskRandom21_13 \
  0x0e, 0x19, 0x10, 0x00, 0x00, 0x00, \
  0x33, 0x14, 0x50, 0x00, 0x00, 0x00, \
  0x10, 0xc3, 0x28, 0x00, 0x00, 0x00, \
  0x45, 0x68, 0x48, 0x00, 0x00, 0x00, \
  0x88, 0x84, 0xa8, 0x00, 0x00, 0x00, \
  0xe0, 0x22, 0x90, 0x00, 0x00, 0x00, \
  0x46, 0x11, 0x90, 0x00, 0x00, 0x00, \
  0x33, 0x0c, 0xc8, 0x00, 0x00, 0x00, \
  0x10, 0xe4, 0x60, 0x00, 0x00, 0x00, \
  0x0c, 0x69, 0x08, 0x00, 0x00, 0x00, \
  0x28, 0x94, 0x28, 0x00, 0x00, 0x00, \
  0x94, 0x21, 0x30, 0x00, 0x00, 0x00, \
  0xc1, 0x02, 0x58, 0x00, 0x00, 0x00

#define kMaskRandom21_14 \
  0x46, 0x11, 0x90, 0x00, 0x00, 0x00, \
  0x33, 0x0c, 0xc8, 0x00, 0x00, 0x00, \
  0x10, 0xe4, 0x60, 0x00, 0x00, 0x00, \
  0x0c, 0x69, 0x08, 0x00, 0x00, 0x00, \
  0x28, 0x94, 0x28, 0x00, 0x00, 0x00, \
  0x94, 0x21, 0x30, 0x00, 0x00, 0x00, \
  0xc1, 0x02, 0x58, 0x00, 0x00, 0x00, \
  0x0e, 0x19, 0x10, 0x00, 0x00, 0x00, \
  0x33, 0x14, 0x50, 0x00, 0x00, 0x00, \
  0x10, 0xc3, 0x28, 0x00, 0x00, 0x00, \
  0x45, 0x68, 0x48, 0x00, 0x00, 0x00, \
  0x88, 0x84, 0xa8, 0x00, 0x00, 0x00, \
  0xe0, 0x22, 0x90, 0x00, 0x00, 0x00, \
  0x4d, 0xd0, 0xc0, 0x00, 0x00, 0x00

#define kMaskRandom21_15 \
  0x46, 0x11, 0x90, 0x00, 0x00, 0x00, \
  0x33, 0x0c, 0xc8, 0x00, 0x00, 0x00, \
  0x10, 0xe4, 0x60, 0x00, 0x00, 0x00, \
  0x0c, 0x69, 0x08, 0x00, 0x00, 0x00, \
  0x28, 0x94, 0x28, 0x00, 0x00, 0x00, \
  0x94, 0x21, 0x30, 0x00, 0x00, 0x00, \
  0xc1, 0x02, 0x58, 0x00, 0x00, 0x00, \
  0x2c, 0x03, 0x20, 0x00, 0x00, 0x00, \
  0x81, 0xa0, 0x18, 0x00, 0x00, 0x00, \
  0xa0, 0x68, 0x20, 0x00, 0x00, 0x00, \
  0x05, 0x41, 0x50, 0x00, 0x00, 0x00, \
  0x18, 0x90, 0xc0, 0x00, 0x00, 0x00, \
  0xc2, 0x06, 0x80, 0x00, 0x00, 0x00, \
  0x22, 0x98, 0x08, 0x00, 0x00, 0x00, \
  0x50, 0x45, 0x08, 0x00, 0x00, 0x00

#define kMaskRandom21_16 \
  0x2c, 0x03, 0x20, 0x00, 0x00, 0x00, \
  0x81, 0xa0, 0x18, 0x00, 0x00, 0x00, \
  0xa0, 0x68, 0x20, 0x00, 0x00, 0x00, \
  0x05, 0x41, 0x50, 0x00, 0x00, 0x00, \
  0x18, 0x90, 0xc0, 0x00, 0x00, 0x00, \
  0xc2, 0x06, 0x80, 0x00, 0x00, 0x00, \
  0x22, 0x98, 0x08, 0x00, 0x00, 0x00, \
  0x50, 0x45, 0x08, 0x00, 0x00, 0x00, \
  0x46, 0x11, 0x90, 0x00, 0x00, 0x00, \
  0x33, 0x0c, 0xc8, 0x00, 0x00, 0x00, \
  0x10, 0xe4, 0x60, 0x00, 0x00, 0x00, \
  0x0c, 0x69, 0x08, 0x00, 0x00, 0x00, \
  0x28, 0x94, 0x28, 0x00, 0x00, 0x00, \
  0x94, 0x21, 0x30, 0x00, 0x00, 0x00, \
  0xc1, 0x02, 0x58, 0x00, 0x00, 0x00, \
  0x3b, 0xf5, 0x38, 0x00, 0x00, 0x00

#define kMaskRandom21_17 \
  0x2c, 0x03, 0x20, 0x00, 0x00, 0x00, \
  0x81, 0xa0, 0x18, 0x00, 0x00, 0x00, \
  0xa0, 0x68, 0x20, 0x00, 0x00, 0x00, \
  0x05, 0x41, 0x50, 0x00, 0x00, 0x00, \
  0x18, 0x90, 0xc0, 0x00, 0x00, 0x00, \
  0xc2, 0x06, 0x80, 0x00, 0x00, 0x00, \
  0x22, 0x98, 0x08, 0x00, 0x00, 0x00, \
  0x50, 0x45, 0x08, 0x00, 0x00, 0x00, \
  0x4e, 0x11, 0x90, 0x00, 0x00, 0x00, \
  0xe3, 0x18, 0x98, 0x00, 0x00, 0x00, \
  0x81, 0xe3, 0x00, 0x00, 0x00, 0x00, \
  0x21, 0x40, 0x58, 0x00, 0x00, 0x00, \
  0x52, 0x81, 0xe0, 0x00, 0x00, 0x00, \
  0xb4, 0x28, 0x20, 0x00, 0x00, 0x00, \
  0x26, 0x86, 0x28, 0x00, 0x00, 0x00, \
  0x58, 0x64, 0x40, 0x00, 0x00, 0x00, \
  0x19, 0x9e, 0x00, 0x00, 0x00, 0x00

#define kMaskRandom21_18 \
  0x4e, 0x11, 0x90, 0x00, 0x00, 0x00, \
  0xe3, 0x18, 0x98, 0x00, 0x00, 0x00, \
  0x81, 0xe3, 0x00, 0x00, 0x00, 0x00, \
  0x21, 0x40, 0x58, 0x00, 0x00, 0x00, \
  0x52, 0x81, 0xe0, 0x00, 0x00, 0x00, \
  0xb4, 0x28, 0x20, 0x00, 0x00, 0x00, \
  0x26, 0x86, 0x28, 0x00, 0x00, 0x00, \
  0x58, 0x64, 0x40, 0x00, 0x00, 0x00, \
  0x19, 0x9e, 0x00, 0x00, 0x00, 0x00, \
  0x2c, 0x03, 0x20, 0x00, 0x00, 0x00, \
  0x81, 0xa0, 0x18, 0x00, 0x00, 0x00, \
  0xa0, 0x68, 0x20, 0x00, 0x00, 0x00, \
  0x05, 0x41, 0x50, 0x00, 0x00, 0x00, \
  0x18, 0x90, 0xc0, 0x00, 0x00, 0x00, \
  0xc2, 0x06, 0x80, 0x00, 0x00, 0x00, \
  0x22, 0x98, 0x08, 0x00, 0x00, 0x00, \
  0x50, 0x45, 0x08, 0x00, 0x00, 0x00, \
  0x5a, 0x56, 0x58, 0x00, 0x00, 0x00

#define kMaskRandom21_19 \
  0x4e, 0x11, 0x90, 0x00, 0x00, 0x00, \
  0xe3, 0x18, 0x98, 0x00, 0x00, 0x00, \
  0x81, 0xe3, 0x00, 0x00, 0x00, 0x00, \
  0x21, 0x40, 0x58, 0x00, 0x00, 0x00, \
  0x52, 0x81, 0xe0, 0x00, 0x00, 0x00, \
  0xb4, 0x28, 0x20, 0x00, 0x00, 0x00, \
  0x26, 0x86, 0x28, 0x00, 0x00, 0x00, \
  0x58, 0x64, 0x40, 0x00, 0x00, 0x00, \
  0x19, 0x9e, 0x00, 0x00, 0x00, 0x00, \
  0x4c, 0x19, 0x10, 0x00, 0x00, 0x00, \
  0x51, 0x14, 0x50, 0x00, 0x00, 0x00, \
  0xa0, 0x6a, 0x40, 0x00, 0x00, 0x00, \
  0x04, 0xc1, 0x30, 0x00, 0x00, 0x00, \
  0x03, 0xb4, 0x00, 0x00, 0x00, 0x00, \
  0x86, 0x20, 0x90, 0x00, 0x00, 0x00, \
  0x29, 0x08, 0x48, 0x00, 0x00, 0x00, \
  0x42, 0x43, 0x08, 0x00, 0x00, 0x00, \
  0x98, 0x12, 0x80, 0x00, 0x00, 0x00, \
  0x30, 0x84, 0xa8, 0x00, 0x00, 0x00

#define kMaskRandom21_2 \
  0xee, 0x3b, 0x30, 0x00, 0x00, 0x00, \
  0x99, 0xe6, 0xe8, 0x00, 0x00, 0x00

#define kMaskRandom21_20 \
  0x4c, 0x19, 0x10, 0x00, 0x00, 0x00, \
  0x51, 0x14, 0x50, 0x00, 0x00, 0x00, \
  0xa0, 0x6a, 0x40, 0x00, 0x00, 0x00, \
  0x04, 0xc1, 0x30, 0x00, 0x00, 0x00, \
  0x03, 0xb4, 0x00, 0x00, 0x00, 0x00, \
  0x86, 0x20, 0x90, 0x00, 0x00, 0x00, \
  0x29, 0x08, 0x48, 0x00, 0x00, 0x00, \
  0x42, 0x43, 0x08, 0x00, 0x00, 0x00, \
  0x98, 0x12, 0x80, 0x00, 0x00, 0x00, \
  0x30, 0x84, 0xa8, 0x00, 0x00, 0x00, \
  0x4e, 0x11, 0x90, 0x00, 0x00, 0x00, \
  0xe3, 0x18, 0x98, 0x00, 0x00, 0x00, \
  0x81, 0xe3, 0x00, 0x00, 0x00, 0x00, \
  0x21, 0x40, 0x58, 0x00, 0x00, 0x00, \
  0x52, 0x81, 0xe0, 0x00, 0x00, 0x00, \
  0xb4, 0x28, 0x20, 0x00, 0x00, 0x00, \
  0x26, 0x86, 0x28, 0x00, 0x00, 0x00, \
  0x58, 0x64, 0x40, 0x00, 0x00, 0x00, \
  0x19, 0x9e, 0x00, 0x00, 0x00, 0x00, \
  0x2a, 0x03, 0x30, 0x00, 0x00, 0x00

#define kMaskRandom21_21 \
  0x4c, 0x19, 0x10, 0x00, 0x00, 0x00, \
  0x51, 0x14, 0x50, 0x00, 0x00, 0x00, \
  0xa0, 0x6a, 0x40, 0x00, 0x00, 0x00, \
  0x04, 0xc1, 0x30, 0x00, 0x00, 0x00, \
  0x03, 0xb4, 0x00, 0x00, 0x00, 0x00, \
  0x86, 0x20, 0x90, 0x00, 0x00, 0x00, \
  0x29, 0x08, 0x48, 0x00, 0x00, 0x00, \
  0x42, 0x43, 0x08, 0x00, 0x00, 0x00, \
  0x98, 0x12, 0x80, 0x00, 0x00, 0x00, \
  0x30, 0x84, 0xa8, 0x00, 0x00, 0x00, \
  0x4c, 0x11, 0x90, 0x00, 0x00, 0x00, \
  0x51, 0x0c, 0xc8, 0x00, 0x00, 0x00, \
  0xa0, 0x66, 0x40, 0x00, 0x00, 0x00, \
  0x04, 0xc1, 0x60, 0x00, 0x00, 0x00, \
  0x03, 0xa0, 0x28, 0x00, 0x00, 0x00, \
  0x86, 0x21, 0x10, 0x00, 0x00, 0x00, \
  0x29, 0x10, 0x18, 0x00, 0x00, 0x00, \
  0x42, 0x42, 0xa0, 0x00, 0x00, 0x00, \
  0x98, 0x1a, 0x00, 0x00, 0x00, 0x00, \
  0x30, 0x84, 0x08, 0x00, 0x00, 0x00, \
  0xdf, 0x4c, 0x10, 0x00, 0x00, 0x00

#define kMaskRandom21_3 \
  0xce, 0x32, 0xb0, 0x00, 0x00, 0x00, \
  0x55, 0xdc, 0x50, 0x00, 0x00, 0x00, \
  0xa8, 0xed, 0x88, 0x00, 0x00, 0x00

#define kMaskRandom21_4 \
  0xe6, 0x31, 0x30, 0x00, 0x00, 0x00, \
  0x33, 0x8c, 0x58, 0x00, 0x00, 0x00, \
  0x98, 0xd2, 0xc8, 0x00, 0x00, 0x00, \
  0x2d, 0x4b, 0x28, 0x00, 0x00, 0x00

#define kMaskRandom21_5 \
  0xce, 0x31, 0xb0, 0x00, 0x00, 0x00, \
  0x63, 0x98, 0xd8, 0x00, 0x00, 0x00, \
  0x98, 0xc7, 0x68, 0x00, 0x00, 0x00, \
  0x4d, 0x6b, 0x50, 0x00, 0x00, 0x00, \
  0xb2, 0x6c, 0xa8, 0x00, 0x00, 0x00

#define kMaskRandom21_6 \
  0x4c, 0x19, 0x10, 0x00, 0x00, 0x00, \
  0x51, 0x14, 0x50, 0x00, 0x00, 0x00, \
  0x20, 0xea, 0x08, 0x00, 0x00, 0x00, \
  0x85, 0x41, 0x28, 0x00, 0x00, 0x00, \
  0x06, 0x80, 0xd8, 0x00, 0x00, 0x00, \
  0x8a, 0x24, 0x30, 0x00, 0x00, 0x00

#define kMaskRandom21_7 \
  0xc6, 0x11, 0x90, 0x00, 0x00, 0x00, \
  0x33, 0x04, 0xc8, 0x00, 0x00, 0x00, \
  0x18, 0x67, 0x40, 0x00, 0x00, 0x00, \
  0x45, 0x42, 0xd0, 0x00, 0x00, 0x00, \
  0x12, 0xd4, 0x28, 0x00, 0x00, 0x00, \
  0xb4, 0x28, 0x30, 0x00, 0x00, 0x00, \
  0x29, 0x92, 0x18, 0x00, 0x00, 0x00

#define kMaskRandom21_8 \
  0x07, 0x0a, 0x70, 0x00, 0x00, 0x00, \
  0x49, 0xa8, 0x28, 0x00, 0x00, 0x00, \
  0xb0, 0x7a, 0x00, 0x00, 0x00, 0x00, \
  0x24, 0xc5, 0xc0, 0x00, 0x00, 0x00, \
  0x52, 0x80, 0xe8, 0x00, 0x00, 0x00, \
  0xc6, 0x31, 0x80, 0x00, 0x00, 0x00, \
  0x31, 0x94, 0x18, 0x00, 0x00, 0x00, \
  0x18, 0xc7, 0x08, 0x00, 0x00, 0x00

#define kMaskRandom21_9 \
  0x4e, 0x11, 0x10, 0x00, 0x00, 0x00, \
  0x62, 0x1a, 0x08, 0x00, 0x00, 0x00, \
  0x80, 0xe9, 0x40, 0x00, 0x00, 0x00, \
  0xa1, 0x50, 0x50, 0x00, 0x00, 0x00, \
  0x53, 0x00, 0x68, 0x00, 0x00, 0x00, \
  0xa4, 0x24, 0x30, 0x00, 0x00, 0x00, \
  0x16, 0xa0, 0x88, 0x00, 0x00, 0x00, \
  0x58, 0x45, 0x20, 0x00, 0x00, 0x00, \
  0x29, 0x86, 0x80, 0x00, 0x00, 0x00

#define kMaskRandom22_1 \
  0xff, 0xff, 0xfc, 0x00, 0x00, 0x00

#define kMaskRandom22_10 \
  0xc0, 0x38, 0x88, 0x00, 0x00, 0x00, \
  0x30, 0x0e, 0x28, 0x00, 0x00, 0x00, \
  0xe8, 0x07, 0x00, 0x00, 0x00, 0x00, \
  0x85, 0x08, 0xa8, 0x00, 0x00, 0x00, \
  0xd0, 0x92, 0x10, 0x00, 0x00, 0x00, \
  0x86, 0x50, 0x48, 0x00, 0x00, 0x00, \
  0x4a, 0x68, 0x0c, 0x00, 0x00, 0x00, \
  0x01, 0xa0, 0x74, 0x00, 0x00, 0x00, \
  0x4c, 0x81, 0x90, 0x00, 0x00, 0x00, \
  0x62, 0x24, 0x04, 0x00, 0x00, 0x00

#define kMaskRandom22_11 \
  0x46, 0x48, 0xc8, 0x00, 0x00, 0x00, \
  0x33, 0x26, 0x64, 0x00, 0x00, 0x00, \
  0x99, 0x13, 0x20, 0x00, 0x00, 0x00, \
  0x05, 0x80, 0xb0, 0x00, 0x00, 0x00, \
  0x80, 0xb0, 0x14, 0x00, 0x00, 0x00, \
  0x84, 0x50, 0x88, 0x00, 0x00, 0x00, \
  0x40, 0x68, 0x0c, 0x00, 0x00, 0x00, \
  0x0a, 0x81, 0x50, 0x00, 0x00, 0x00, \
  0x68, 0x0d, 0x00, 0x00, 0x00, 0x00, \
  0x10, 0x22, 0x04, 0x00, 0x00, 0x00, \
  0x30, 0x46, 0x08, 0x00, 0x00, 0x00

#define kMaskRandom22_12 \
  0x64, 0x4c, 0x88, 0x00, 0x00, 0x00, \
  0x51, 0x4a, 0x28, 0x00, 0x00, 0x00, \
  0x0c, 0xa1, 0x94, 0x00, 0x00, 0x00, \
  0xa1, 0x34, 0x24, 0x00, 0x00, 0x00, \
  0x12, 0xa2, 0x54, 0x00, 0x00, 0x00, \
  0x8a, 0x51, 0x48, 0x00, 0x00, 0x00, \
  0x86, 0x90, 0xd0, 0x00, 0x00, 0x00, \
  0x23, 0x24, 0x64, 0x00, 0x00, 0x00, \
  0x16, 0x22, 0xc4, 0x00, 0x00, 0x00, \
  0x4c, 0x29, 0x84, 0x00, 0x00, 0x00, \
  0x41, 0xc8, 0x38, 0x00, 0x00, 0x00, \
  0xf4, 0x18, 0x9c, 0x00, 0x00, 0x00

#define kMaskRandom22_13 \
  0x64, 0x4c, 0x88, 0x00, 0x00, 0x00, \
  0x51, 0x4a, 0x28, 0x00, 0x00, 0x00, \
  0x0c, 0xa1, 0x94, 0x00, 0x00, 0x00, \
  0xa1, 0x34, 0x24, 0x00, 0x00, 0x00, \
  0x12, 0xa2, 0x54, 0x00, 0x00, 0x00, \
  0x8a, 0x51, 0x48, 0x00, 0x00, 0x00, \
  0x46, 0x48, 0xc8, 0x00, 0x00, 0x00, \
  0x33, 0x26, 0x64, 0x00, 0x00, 0x00, \
  0x91, 0x92, 0x30, 0x00, 0x00, 0x00, \
  0xa4, 0x34, 0x84, 0x00, 0x00, 0x00, \
  0x50, 0xaa, 0x14, 0x00, 0x00, 0x00, \
  0x84, 0xd0, 0x98, 0x00, 0x00, 0x00, \
  0x09, 0x61, 0x2c, 0x00, 0x00, 0x00

#define kMaskRandom22_14 \
  0x46, 0x48, 0xc8, 0x00, 0x00, 0x00, \
  0x33, 0x26, 0x64, 0x00, 0x00, 0x00, \
  0x91, 0x92, 0x30, 0x00, 0x00, 0x00, \
  0xa4, 0x34, 0x84, 0x00, 0x00, 0x00, \
  0x50, 0xaa, 0x14, 0x00, 0x00, 0x00, \
  0x84, 0xd0, 0x98, 0x00, 0x00, 0x00, \
  0x09, 0x61, 0x2c, 0x00, 0x00, 0x00, \
  0x64, 0x4c, 0x88, 0x00, 0x00, 0x00, \
  0x51, 0x4a, 0x28, 0x00, 0x00, 0x00, \
  0x0c, 0xa1, 0x94, 0x00, 0x00, 0x00, \
  0xa1, 0x34, 0x24, 0x00, 0x00, 0x00, \
  0x12, 0xa2, 0x54, 0x00, 0x00, 0x00, \
  0x8a, 0x51, 0x48, 0x00, 0x00, 0x00, \
  0xc6, 0xca, 0xe8, 0x00, 0x00, 0x00

#define kMaskRandom22_15 \
  0x46, 0x48, 0xc8, 0x00, 0x00, 0x00, \
  0x33, 0x26, 0x64, 0x00, 0x00, 0x00, \
  0x91, 0x92, 0x30, 0x00, 0x00, 0x00, \
  0xa4, 0x34, 0x84, 0x00, 0x00, 0x00, \
  0x50, 0xaa, 0x14, 0x00, 0x00, 0x00, \
  0x84, 0xd0, 0x98, 0x00, 0x00, 0x00, \
  0x09, 0x61, 0x2c, 0x00, 0x00, 0x00, \
  0x0c, 0x81, 0x90, 0x00, 0x00, 0x00, \
  0x80, 0x70, 0x0c, 0x00, 0x00, 0x00, \
  0xa0, 0x94, 0x10, 0x00, 0x00, 0x00, \
  0x05, 0x40, 0xa8, 0x00, 0x00, 0x00, \
  0x43, 0x08, 0x60, 0x00, 0x00, 0x00, \
  0x1a, 0x03, 0x40, 0x00, 0x00, 0x00, \
  0x60, 0x2c, 0x04, 0x00, 0x00, 0x00, \
  0x14, 0x22, 0x84, 0x00, 0x00, 0x00

#define kMaskRandom22_16 \
  0x0c, 0x81, 0x90, 0x00, 0x00, 0x00, \
  0x80, 0x70, 0x0c, 0x00, 0x00, 0x00, \
  0xa0, 0x94, 0x10, 0x00, 0x00, 0x00, \
  0x05, 0x40, 0xa8, 0x00, 0x00, 0x00, \
  0x43, 0x08, 0x60, 0x00, 0x00, 0x00, \
  0x1a, 0x03, 0x40, 0x00, 0x00, 0x00, \
  0x60, 0x2c, 0x04, 0x00, 0x00, 0x00, \
  0x14, 0x22, 0x84, 0x00, 0x00, 0x00, \
  0x46, 0x48, 0xc8, 0x00, 0x00, 0x00, \
  0x33, 0x26, 0x64, 0x00, 0x00, 0x00, \
  0x91, 0x92, 0x30, 0x00, 0x00, 0x00, \
  0xa4, 0x34, 0x84, 0x00, 0x00, 0x00, \
  0x50, 0xaa, 0x14, 0x00, 0x00, 0x00, \
  0x84, 0xd0, 0x98, 0x00, 0x00, 0x00, \
  0x09, 0x61, 0x2c, 0x00, 0x00, 0x00, \
  0x86, 0xc1, 0x44, 0x00, 0x00, 0x00

#define kMaskRandom22_17 \
  0x0c, 0x81, 0x90, 0x00, 0x00, 0x00, \
  0x80, 0x70, 0x0c, 0x00, 0x00, 0x00, \
  0xa0, 0x94, 0x10, 0x00, 0x00, 0x00, \
  0x05, 0x40, 0xa8, 0x00, 0x00, 0x00, \
  0x43, 0x08, 0x60, 0x00, 0x00, 0x00, \
  0x1a, 0x03, 0x40, 0x00, 0x00, 0x00, \
  0x60, 0x2c, 0x04, 0x00, 0x00, 0x00, \
  0x14, 0x22, 0x84, 0x00, 0x00, 0x00, \
  0x46, 0x48, 0xc8, 0x00, 0x00, 0x00, \
  0x62, 0x6c, 0x4c, 0x00, 0x00, 0x00, \
  0x8c, 0x11, 0x80, 0x00, 0x00, 0x00, \
  0x01, 0x60, 0x2c, 0x00, 0x00, 0x00, \
  0x07, 0x80, 0xf0, 0x00, 0x00, 0x00, \
  0xa0, 0x94, 0x10, 0x00, 0x00, 0x00, \
  0x18, 0xa3, 0x14, 0x00, 0x00, 0x00, \
  0x91, 0x12, 0x20, 0x00, 0x00, 0x00, \
  0x78, 0x0f, 0x00, 0x00, 0x00, 0x00

#define kMaskRandom22_18 \
  0x46, 0x48, 0xc8, 0x00, 0x00, 0x00, \
  0x62, 0x6c, 0x4c, 0x00, 0x00, 0x00, \
  0x8c, 0x11, 0x80, 0x00, 0x00, 0x00, \
  0x01, 0x60, 0x2c, 0x00, 0x00, 0x00, \
  0x07, 0x80, 0xf0, 0x00, 0x00, 0x00, \
  0xa0, 0x94, 0x10, 0x00, 0x00, 0x00, \
  0x18, 0xa3, 0x14, 0x00, 0x00, 0x00, \
  0x91, 0x12, 0x20, 0x00, 0x00, 0x00, \
  0x78, 0x0f, 0x00, 0x00, 0x00, 0x00, \
  0x0c, 0x81, 0x90, 0x00, 0x00, 0x00, \
  0x80, 0x70, 0x0c, 0x00, 0x00, 0x00, \
  0xa0, 0x94, 0x10, 0x00, 0x00, 0x00, \
  0x05, 0x40, 0xa8, 0x00, 0x00, 0x00, \
  0x43, 0x08, 0x60, 0x00, 0x00, 0x00, \
  0x1a, 0x03, 0x40, 0x00, 0x00, 0x00, \
  0x60, 0x2c, 0x04, 0x00, 0x00, 0x00, \
  0x14, 0x22, 0x84, 0x00, 0x00, 0x00, \
  0xe4, 0xd4, 0x6c, 0x00, 0x00, 0x00

#define kMaskRandom22_19 \
  0x46, 0x48, 0xc8, 0x00, 0x00, 0x00, \
  0x62, 0x6c, 0x4c, 0x00, 0x00, 0x00, \
  0x8c, 0x11, 0x80, 0x00, 0x00, 0x00, \
  0x01, 0x60, 0x2c, 0x00, 0x00, 0x00, \
  0x07, 0x80, 0xf0, 0x00, 0x00, 0x00, \
  0xa0, 0x94, 0x10, 0x00, 0x00, 0x00, \
  0x18, 0xa3, 0x14, 0x00, 0x00, 0x00, \
  0x91, 0x12, 0x20, 0x00, 0x00, 0x00, \
  0x78, 0x0f, 0x00, 0x00, 0x00, 0x00, \
  0x64, 0x4c, 0x88, 0x00, 0x00, 0x00, \
  0x51, 0x4a, 0x28, 0x00, 0x00, 0x00, \
  0xa9, 0x15, 0x20, 0x00, 0x00, 0x00, \
  0x04, 0xc0, 0x98, 0x00, 0x00, 0x00, \
  0xd0, 0x1a, 0x00, 0x00, 0x00, 0x00, \
  0x82, 0x50, 0x48, 0x00, 0x00, 0x00, \
  0x21, 0x24, 0x24, 0x00, 0x00, 0x00, \
  0x0c, 0x21, 0x84, 0x00, 0x00, 0x00, \
  0x4a, 0x09, 0x40, 0x00, 0x00, 0x00, \
  0x12, 0xa2, 0x54, 0x00, 0x00, 0x00

#define kMaskRandom22_2 \
  0xec, 0xdd, 0x98, 0x00, 0x00, 0x00, \
  0x9b, 0xb3, 0x74, 0x00, 0x00, 0x00

#define kMaskRandom22_20 \
  0x64, 0x4c, 0x88, 0x00, 0x00, 0x00, \
  0x51, 0x4a, 0x28, 0x00, 0x00, 0x00, \
  0xa9, 0x15, 0x20, 0x00, 0x00, 0x00, \
  0x04, 0xc0, 0x98, 0x00, 0x00, 0x00, \
  0xd0, 0x1a, 0x00, 0x00, 0x00, 0x00, \
  0x82, 0x50, 0x48, 0x00, 0x00, 0x00, \
  0x21, 0x24, 0x24, 0x00, 0x00, 0x00, \
  0x0c, 0x21, 0x84, 0x00, 0x00, 0x00, \
  0x4a, 0x09, 0x40, 0x00, 0x00, 0x00, \
  0x12, 0xa2, 0x54, 0x00, 0x00, 0x00, \
  0x46, 0x48, 0xc8, 0x00, 0x00, 0x00, \
  0x62, 0x6c, 0x4c, 0x00, 0x00, 0x00, \
  0x8c, 0x11, 0x80, 0x00, 0x00, 0x00, \
  0x01, 0x60, 0x2c, 0x00, 0x00, 0x00, \
  0x07, 0x80, 0xf0, 0x00, 0x00, 0x00, \
  0xa0, 0x94, 0x10, 0x00, 0x00, 0x00, \
  0x18, 0xa3, 0x14, 0x00, 0x00, 0x00, \
  0x91, 0x12, 0x20, 0x00, 0x00, 0x00, \
  0x78, 0x0f, 0x00, 0x00, 0x00, 0x00, \
  0x3b, 0x48, 0xc4, 0x00, 0x00, 0x00

#define kMaskRandom22_21 \
  0x64, 0x4c, 0x88, 0x00, 0x00, 0x00, \
  0x51, 0x4a, 0x28, 0x00, 0x00, 0x00, \
  0xa9, 0x15, 0x20, 0x00, 0x00, 0x00, \
  0x04, 0xc0, 0x98, 0x00, 0x00, 0x00, \
  0xd0, 0x1a, 0x00, 0x00, 0x00, 0x00, \
  0x82, 0x50, 0x48, 0x00, 0x00, 0x00, \
  0x21, 0x24, 0x24, 0x00, 0x00, 0x00, \
  0x0c, 0x21, 0x84, 0x00, 0x00, 0x00, \
  0x4a, 0x09, 0x40, 0x00, 0x00, 0x00, \
  0x12, 0xa2, 0x54, 0x00, 0x00, 0x00, \
  0x46, 0x48, 0xc8, 0x00, 0x00, 0x00, \
  0x33, 0x26, 0x64, 0x00, 0x00, 0x00, \
  0x99, 0x13, 0x20, 0x00, 0x00, 0x00, \
  0x05, 0x80, 0xb0, 0x00, 0x00, 0x00, \
  0x80, 0xb0, 0x14, 0x00, 0x00, 0x00, \
  0x84, 0x50, 0x88, 0x00, 0x00, 0x00, \
  0x40, 0x68, 0x0c, 0x00, 0x00, 0x00, \
  0x0a, 0x81, 0x50, 0x00, 0x00, 0x00, \
  0x68, 0x0d, 0x00, 0x00, 0x00, 0x00, \
  0x10, 0x22, 0x04, 0x00, 0x00, 0x00, \
  0x30, 0x46, 0x08, 0x00, 0x00, 0x00

#define kMaskRandom22_22 \
  0x46, 0x48, 0xc8, 0x00, 0x00, 0x00, \
  0x33, 0x26, 0x64, 0x00, 0x00, 0x00, \
  0x99, 0x13, 0x20, 0x00, 0x00, 0x00, \
  0x05, 0x80, 0xb0, 0x00, 0x00, 0x00, \
  0x80, 0xb0, 0x14, 0x00, 0x00, 0x00, \
  0x84, 0x50, 0x88, 0x00, 0x00, 0x00, \
  0x40, 0x68, 0x0c, 0x00, 0x00, 0x00, \
  0x0a, 0x81, 0x50, 0x00, 0x00, 0x00, \
  0x68, 0x0d, 0x00, 0x00, 0x00, 0x00, \
  0x10, 0x22, 0x04, 0x00, 0x00, 0x00, \
  0x30, 0x46, 0x08, 0x00, 0x00, 0x00, \
  0x64, 0x4c, 0x88, 0x00, 0x00, 0x00, \
  0x51, 0x4a, 0x28, 0x00, 0x00, 0x00, \
  0xa9, 0x15, 0x20, 0x00, 0x00, 0x00, \
  0x04, 0xc0, 0x98, 0x00, 0x00, 0x00, \
  0xd0, 0x1a, 0x00, 0x00, 0x00, 0x00, \
  0x82, 0x50, 0x48, 0x00, 0x00, 0x00, \
  0x21, 0x24, 0x24, 0x00, 0x00, 0x00, \
  0x0c, 0x21, 0x84, 0x00, 0x00, 0x00, \
  0x4a, 0x09, 0x40, 0x00, 0x00, 0x00, \
  0x12, 0xa2, 0x54, 0x00, 0x00, 0x00, \
  0x9e, 0xce, 0x88, 0x00, 0x00, 0x00

#define kMaskRandom22_3 \
  0xca, 0xd9, 0x58, 0x00, 0x00, 0x00, \
  0xf1, 0x5e, 0x28, 0x00, 0x00, 0x00, \
  0xb6, 0x35, 0xc4, 0x00, 0x00, 0x00

#define kMaskRandom22_4 \
  0xc4, 0xd8, 0x98, 0x00, 0x00, 0x00, \
  0x31, 0x66, 0x2c, 0x00, 0x00, 0x00, \
  0x4b, 0x29, 0x64, 0x00, 0x00, 0x00, \
  0x2c, 0xa5, 0x94, 0x00, 0x00, 0x00

#define kMaskRandom22_5 \
  0xc6, 0xd8, 0xd8, 0x00, 0x00, 0x00, \
  0x63, 0x6c, 0x6c, 0x00, 0x00, 0x00, \
  0x1d, 0xa3, 0xb4, 0x00, 0x00, 0x00, \
  0xad, 0x55, 0xa8, 0x00, 0x00, 0x00, \
  0xb2, 0xb6, 0x54, 0x00, 0x00, 0x00

#define kMaskRandom22_6 \
  0x64, 0x4c, 0x88, 0x00, 0x00, 0x00, \
  0x51, 0x4a, 0x28, 0x00, 0x00, 0x00, \
  0xa8, 0x35, 0x04, 0x00, 0x00, 0x00, \
  0xc4, 0xa0, 0x94, 0x00, 0x00, 0x00, \
  0x03, 0x60, 0x6c, 0x00, 0x00, 0x00, \
  0x90, 0xd2, 0x18, 0x00, 0x00, 0x00

#define kMaskRandom22_7 \
  0xc6, 0x48, 0xc8, 0x00, 0x00, 0x00, \
  0x13, 0x26, 0x64, 0x00, 0x00, 0x00, \
  0x8d, 0x13, 0xa0, 0x00, 0x00, 0x00, \
  0x8b, 0x41, 0x68, 0x00, 0x00, 0x00, \
  0x52, 0xaa, 0x14, 0x00, 0x00, 0x00, \
  0xa2, 0xd4, 0x18, 0x00, 0x00, 0x00, \
  0x61, 0xa8, 0x2c, 0x00, 0x00, 0x00

#define kMaskRandom22_8 \
  0x28, 0x85, 0x38, 0x00, 0x00, 0x00, \
  0x21, 0xf4, 0x04, 0x00, 0x00, 0x00, \
  0xe9, 0x1d, 0x00, 0x00, 0x00, 0x00, \
  0x17, 0x02, 0xe0, 0x00, 0x00, 0x00, \
  0x83, 0xa0, 0x54, 0x00, 0x00, 0x00, \
  0x46, 0x18, 0xe8, 0x00, 0x00, 0x00, \
  0x50, 0x6a, 0x0c, 0x00, 0x00, 0x00, \
  0x1c, 0x23, 0x84, 0x00, 0x00, 0x00

#define kMaskRandom22_9 \
  0x44, 0x48, 0xc8, 0x00, 0x00, 0x00, \
  0x28, 0x2d, 0x0c, 0x00, 0x00, 0x00, \
  0x25, 0x14, 0xa0, 0x00, 0x00, 0x00, \
  0x59, 0x0a, 0x20, 0x00, 0x00, 0x00, \
  0x03, 0xa0, 0x34, 0x00, 0x00, 0x00, \
  0xc0, 0xd0, 0x18, 0x00, 0x00, 0x00, \
  0xa2, 0x30, 0x44, 0x00, 0x00, 0x00, \
  0x14, 0x82, 0xd0, 0x00, 0x00, 0x00, \
  0x9a, 0x03, 0x80, 0x00, 0x00, 0x00

#define kMaskRandom23_1 \
  0xff, 0xff, 0xfe, 0x00, 0x00, 0x00

#define kMaskRandom23_10 \
  0x64, 0x4a, 0x28, 0x00, 0x00, 0x00, \
  0x51, 0x48, 0xa2, 0x00, 0x00, 0x00, \
  0xa9, 0x10, 0x1a, 0x00, 0x00, 0x00, \
  0x04, 0xc4, 0x84, 0x00, 0x00, 0x00, \
  0xd0, 0x01, 0x44, 0x00, 0x00, 0x00, \
  0x82, 0x40, 0x1c, 0x00, 0x00, 0x00, \
  0x21, 0x37, 0x00, 0x00, 0x00, 0x00, \
  0x0c, 0x21, 0x22, 0x00, 0x00, 0x00, \
  0x4a, 0x0a, 0xc0, 0x00, 0x00, 0x00, \
  0x12, 0xb4, 0x50, 0x00, 0x00, 0x00

#define kMaskRandom23_11 \
  0x46, 0x4a, 0x6c, 0x00, 0x00, 0x00, \
  0x33, 0x24, 0x26, 0x00, 0x00, 0x00, \
  0x99, 0x02, 0x12, 0x00, 0x00, 0x00, \
  0x05, 0x80, 0x0e, 0x00, 0x00, 0x00, \
  0x80, 0xa1, 0x82, 0x00, 0x00, 0x00, \
  0x84, 0x48, 0x18, 0x00, 0x00, 0x00, \
  0x40, 0x6d, 0x40, 0x00, 0x00, 0x00, \
  0x0a, 0x90, 0xc0, 0x00, 0x00, 0x00, \
  0x68, 0x04, 0x90, 0x00, 0x00, 0x00, \
  0x10, 0x31, 0x20, 0x00, 0x00, 0x00, \
  0x30, 0x58, 0x04, 0x00, 0x00, 0x00

#define kMaskRandom23_12 \
  0x64, 0x4a, 0x28, 0x00, 0x00, 0x00, \
  0x51, 0x58, 0xa2, 0x00, 0x00, 0x00, \
  0x0c, 0xa4, 0x30, 0x00, 0x00, 0x00, \
  0xa1, 0x22, 0x46, 0x00, 0x00, 0x00, \
  0x12, 0xa1, 0x1c, 0x00, 0x00, 0x00, \
  0x8a, 0x45, 0xc0, 0x00, 0x00, 0x00, \
  0x86, 0x8a, 0x6c, 0x00, 0x00, 0x00, \
  0x23, 0x2c, 0x84, 0x00, 0x00, 0x00, \
  0x16, 0x21, 0x98, 0x00, 0x00, 0x00, \
  0x4c, 0x30, 0x54, 0x00, 0x00, 0x00, \
  0x41, 0xc1, 0x26, 0x00, 0x00, 0x00, \
  0x19, 0x56, 0xe4, 0x00, 0x00, 0x00

#define kMaskRandom23_13 \
  0x64, 0x4a, 0x28, 0x00, 0x00, 0x00, \
  0x51, 0x58, 0xa2, 0x00, 0x00, 0x00, \
  0x0c, 0xa4, 0x30, 0x00, 0x00, 0x00, \
  0xa1, 0x22, 0x46, 0x00, 0x00, 0x00, \
  0x12, 0xa1, 0x1c, 0x00, 0x00, 0x00, \
  0x8a, 0x45, 0xc0, 0x00, 0x00, 0x00, \
  0x46, 0x4a, 0x6c, 0x00, 0x00, 0x00, \
  0x33, 0x24, 0x26, 0x00, 0x00, 0x00, \
  0x91, 0x92, 0x12, 0x00, 0x00, 0x00, \
  0xa4, 0x20, 0x4a, 0x00, 0x00, 0x00, \
  0x50, 0xa0, 0xd4, 0x00, 0x00, 0x00, \
  0x84, 0xc5, 0x80, 0x00, 0x00, 0x00, \
  0x09, 0x71, 0x0c, 0x00, 0x00, 0x00

#define kMaskRandom23_14 \
  0x46, 0x4a, 0x6c, 0x00, 0x00, 0x00, \
  0x33, 0x24, 0x26, 0x00, 0x00, 0x00, \
  0x91, 0x92, 0x12, 0x00, 0x00, 0x00, \
  0xa4, 0x20, 0x4a, 0x00, 0x00, 0x00, \
  0x50, 0xa0, 0xd4, 0x00, 0x00, 0x00, \
  0x84, 0xc5, 0x80, 0x00, 0x00, 0x00, \
  0x09, 0x71, 0x0c, 0x00, 0x00, 0x00, \
  0x64, 0x4a, 0x28, 0x00, 0x00, 0x00, \
  0x51, 0x58, 0xa2, 0x00, 0x00, 0x00, \
  0x0c, 0xa4, 0x30, 0x00, 0x00, 0x00, \
  0xa1, 0x22, 0x46, 0x00, 0x00, 0x00, \
  0x12, 0xa1, 0x1c, 0x00, 0x00, 0x00, \
  0x8a, 0x45, 0xc0, 0x00, 0x00, 0x00, \
  0x9c, 0x3f, 0xb2, 0x00, 0x00, 0x00

#define kMaskRandom23_15 \
  0x46, 0x4a, 0x6c, 0x00, 0x00, 0x00, \
  0x33, 0x24, 0x26, 0x00, 0x00, 0x00, \
  0x91, 0x92, 0x12, 0x00, 0x00, 0x00, \
  0xa4, 0x20, 0x4a, 0x00, 0x00, 0x00, \
  0x50, 0xa0, 0xd4, 0x00, 0x00, 0x00, \
  0x84, 0xc5, 0x80, 0x00, 0x00, 0x00, \
  0x09, 0x71, 0x0c, 0x00, 0x00, 0x00, \
  0x0c, 0x84, 0x0c, 0x00, 0x00, 0x00, \
  0x80, 0x70, 0x06, 0x00, 0x00, 0x00, \
  0xa0, 0x88, 0x48, 0x00, 0x00, 0x00, \
  0x05, 0x40, 0x32, 0x00, 0x00, 0x00, \
  0x43, 0x02, 0x82, 0x00, 0x00, 0x00, \
  0x1a, 0x01, 0x50, 0x00, 0x00, 0x00, \
  0x60, 0x27, 0x00, 0x00, 0x00, 0x00, \
  0x14, 0x38, 0xa0, 0x00, 0x00, 0x00

#define kMaskRandom23_16 \
  0x0c, 0x84, 0x0c, 0x00, 0x00, 0x00, \
  0x80, 0x70, 0x06, 0x00, 0x00, 0x00, \
  0xa0, 0x88, 0x48, 0x00, 0x00, 0x00, \
  0x05, 0x40, 0x32, 0x00, 0x00, 0x00, \
  0x43, 0x02, 0x82, 0x00, 0x00, 0x00, \
  0x1a, 0x01, 0x50, 0x00, 0x00, 0x00, \
  0x60, 0x27, 0x00, 0x00, 0x00, 0x00, \
  0x14, 0x38, 0xa0, 0x00, 0x00, 0x00, \
  0x46, 0x4a, 0x6c, 0x00, 0x00, 0x00, \
  0x33, 0x24, 0x26, 0x00, 0x00, 0x00, \
  0x91, 0x92, 0x12, 0x00, 0x00, 0x00, \
  0xa4, 0x20, 0x4a, 0x00, 0x00, 0x00, \
  0x50, 0xa0, 0xd4, 0x00, 0x00, 0x00, \
  0x84, 0xc5, 0x80, 0x00, 0x00, 0x00, \
  0x09, 0x71, 0x0c, 0x00, 0x00, 0x00, \
  0xfa, 0xd9, 0xf4, 0x00, 0x00, 0x00

#define kMaskRandom23_17 \
  0x0c, 0x84, 0x0c, 0x00, 0x00, 0x00, \
  0x80, 0x70, 0x06, 0x00, 0x00, 0x00, \
  0xa0, 0x88, 0x48, 0x00, 0x00, 0x00, \
  0x05, 0x40, 0x32, 0x00, 0x00, 0x00, \
  0x43, 0x02, 0x82, 0x00, 0x00, 0x00, \
  0x1a, 0x01, 0x50, 0x00, 0x00, 0x00, \
  0x60, 0x27, 0x00, 0x00, 0x00, 0x00, \
  0x14, 0x38, 0xa0, 0x00, 0x00, 0x00, \
  0x46, 0x4a, 0x6c, 0x00, 0x00, 0x00, \
  0x62, 0x7c, 0x84, 0x00, 0x00, 0x00, \
  0x8c, 0x04, 0x88, 0x00, 0x00, 0x00, \
  0x01, 0x74, 0x22, 0x00, 0x00, 0x00, \
  0x07, 0x83, 0x06, 0x00, 0x00, 0x00, \
  0xa0, 0x80, 0x72, 0x00, 0x00, 0x00, \
  0x18, 0xb1, 0x42, 0x00, 0x00, 0x00, \
  0x91, 0x00, 0x92, 0x00, 0x00, 0x00, \
  0x78, 0x00, 0x1c, 0x00, 0x00, 0x00

#define kMaskRandom23_18 \
  0x46, 0x4a, 0x6c, 0x00, 0x00, 0x00, \
  0x62, 0x7c, 0x84, 0x00, 0x00, 0x00, \
  0x8c, 0x04, 0x88, 0x00, 0x00, 0x00, \
  0x01, 0x74, 0x22, 0x00, 0x00, 0x00, \
  0x07, 0x83, 0x06, 0x00, 0x00, 0x00, \
  0xa0, 0x80, 0x72, 0x00, 0x00, 0x00, \
  0x18, 0xb1, 0x42, 0x00, 0x00, 0x00, \
  0x91, 0x00, 0x92, 0x00, 0x00, 0x00, \
  0x78, 0x00, 0x1c, 0x00, 0x00, 0x00, \
  0x0c, 0x84, 0x0c, 0x00, 0x00, 0x00, \
  0x80, 0x70, 0x06, 0x00, 0x00, 0x00, \
  0xa0, 0x88, 0x48, 0x00, 0x00, 0x00, \
  0x05, 0x40, 0x32, 0x00, 0x00, 0x00, \
  0x43, 0x02, 0x82, 0x00, 0x00, 0x00, \
  0x1a, 0x01, 0x50, 0x00, 0x00, 0x00, \
  0x60, 0x27, 0x00, 0x00, 0x00, 0x00, \
  0x14, 0x38, 0xa0, 0x00, 0x00, 0x00, \
  0x82, 0x32, 0x56, 0x00, 0x00, 0x00

#define kMaskRandom23_19 \
  0x46, 0x4a, 0x6c, 0x00, 0x00, 0x00, \
  0x62, 0x7c, 0x84, 0x00, 0x00, 0x00, \
  0x8c, 0x04, 0x88, 0x00, 0x00, 0x00, \
  0x01, 0x74, 0x22, 0x00, 0x00, 0x00, \
  0x07, 0x83, 0x06, 0x00, 0x00, 0x00, \
  0xa0, 0x80, 0x72, 0x00, 0x00, 0x00, \
  0x18, 0xb1, 0x42, 0x00, 0x00, 0x00, \
  0x91, 0x00, 0x92, 0x00, 0x00, 0x00, \
  0x78, 0x00, 0x1c, 0x00, 0x00, 0x00, \
  0x64, 0x4a, 0x28, 0x00, 0x00, 0x00, \
  0x51, 0x48, 0xa2, 0x00, 0x00, 0x00, \
  0xa9, 0x10, 0x1a, 0x00, 0x00, 0x00, \
  0x04, 0xc4, 0x84, 0x00, 0x00, 0x00, \
  0xd0, 0x01, 0x44, 0x00, 0x00, 0x00, \
  0x82, 0x40, 0x1c, 0x00, 0x00, 0x00, \
  0x21, 0x37, 0x00, 0x00, 0x00, 0x00, \
  0x0c, 0x21, 0x22, 0x00, 0x00, 0x00, \
  0x4a, 0x0a, 0xc0, 0x00, 0x00, 0x00, \
  0x12, 0xb4, 0x50, 0x00, 0x00, 0x00

#define kMaskRandom23_2 \
  0xec, 0xdd, 0x98, 0x00, 0x00, 0x00, \
  0x9b, 0xb2, 0x76, 0x00, 0x00, 0x00

#define kMaskRandom23_20 \
  0x64, 0x4a, 0x28, 0x00, 0x00, 0x00, \
  0x51, 0x48, 0xa2, 0x00, 0x00, 0x00, \
  0xa9, 0x10, 0x1a, 0x00, 0x00, 0x00, \
  0x04, 0xc4, 0x84, 0x00, 0x00, 0x00, \
  0xd0, 0x01, 0x44, 0x00, 0x00, 0x00, \
  0x82, 0x40, 0x1c, 0x00, 0x00, 0x00, \
  0x21, 0x37, 0x00, 0x00, 0x00, 0x00, \
  0x0c, 0x21, 0x22, 0x00, 0x00, 0x00, \
  0x4a, 0x0a, 0xc0, 0x00, 0x00, 0x00, \
  0x12, 0xb4, 0x50, 0x00, 0x00, 0x00, \
  0x46, 0x4a, 0x6c, 0x00, 0x00, 0x00, \
  0x62, 0x7c, 0x84, 0x00, 0x00, 0x00, \
  0x8c, 0x04, 0x88, 0x00, 0x00, 0x00, \
  0x01, 0x74, 0x22, 0x00, 0x00, 0x00, \
  0x07, 0x83, 0x06, 0x00, 0x00, 0x00, \
  0xa0, 0x80, 0x72, 0x00, 0x00, 0x00, \
  0x18, 0xb1, 0x42, 0x00, 0x00, 0x00, \
  0x91, 0x00, 0x92, 0x00, 0x00, 0x00, \
  0x78, 0x00, 0x1c, 0x00, 0x00, 0x00, \
  0xdb, 0x4a, 0x7a, 0x00, 0x00, 0x00

#define kMaskRandom23_21 \
  0x64, 0x4a, 0x28, 0x00, 0x00, 0x00, \
  0x51, 0x48, 0xa2, 0x00, 0x00, 0x00, \
  0xa9, 0x10, 0x1a, 0x00, 0x00, 0x00, \
  0x04, 0xc4, 0x84, 0x00, 0x00, 0x00, \
  0xd0, 0x01, 0x44, 0x00, 0x00, 0x00, \
  0x82, 0x40, 0x1c, 0x00, 0x00, 0x00, \
  0x21, 0x37, 0x00, 0x00, 0x00, 0x00, \
  0x0c, 0x21, 0x22, 0x00, 0x00, 0x00, \
  0x4a, 0x0a, 0xc0, 0x00, 0x00, 0x00, \
  0x12, 0xb4, 0x50, 0x00, 0x00, 0x00, \
  0x46, 0x4a, 0x6c, 0x00, 0x00, 0x00, \
  0x33, 0x24, 0x26, 0x00, 0x00, 0x00, \
  0x99, 0x02, 0x12, 0x00, 0x00, 0x00, \
  0x05, 0x80, 0x0e, 0x00, 0x00, 0x00, \
  0x80, 0xa1, 0x82, 0x00, 0x00, 0x00, \
  0x84, 0x48, 0x18, 0x00, 0x00, 0x00, \
  0x40, 0x6d, 0x40, 0x00, 0x00, 0x00, \
  0x0a, 0x90, 0xc0, 0x00, 0x00, 0x00, \
  0x68, 0x04, 0x90, 0x00, 0x00, 0x00, \
  0x10, 0x31, 0x20, 0x00, 0x00, 0x00, \
  0x30, 0x58, 0x04, 0x00, 0x00, 0x00

#define kMaskRandom23_22 \
  0x46, 0x4a, 0x6c, 0x00, 0x00, 0x00, \
  0x33, 0x24, 0x26, 0x00, 0x00, 0x00, \
  0x99, 0x02, 0x12, 0x00, 0x00, 0x00, \
  0x05, 0x80, 0x0e, 0x00, 0x00, 0x00, \
  0x80, 0xa1, 0x82, 0x00, 0x00, 0x00, \
  0x84, 0x48, 0x18, 0x00, 0x00, 0x00, \
  0x40, 0x6d, 0x40, 0x00, 0x00, 0x00, \
  0x0a, 0x90, 0xc0, 0x00, 0x00, 0x00, \
  0x68, 0x04, 0x90, 0x00, 0x00, 0x00, \
  0x10, 0x31, 0x20, 0x00, 0x00, 0x00, \
  0x30, 0x58, 0x04, 0x00, 0x00, 0x00, \
  0x64, 0x4a, 0x28, 0x00, 0x00, 0x00, \
  0x51, 0x48, 0xa2, 0x00, 0x00, 0x00, \
  0xa9, 0x10, 0x1a, 0x00, 0x00, 0x00, \
  0x04, 0xc4, 0x84, 0x00, 0x00, 0x00, \
  0xd0, 0x01, 0x44, 0x00, 0x00, 0x00, \
  0x82, 0x40, 0x1c, 0x00, 0x00, 0x00, \
  0x21, 0x37, 0x00, 0x00, 0x00, 0x00, \
  0x0c, 0x21, 0x22, 0x00, 0x00, 0x00, \
  0x4a, 0x0a, 0xc0, 0x00, 0x00, 0x00, \
  0x12, 0xb4, 0x50, 0x00, 0x00, 0x00, \
  0xea, 0x8d, 0x1a, 0x00, 0x00, 0x00

#define kMaskRandom23_23 \
  0x46, 0x4a, 0x6c, 0x00, 0x00, 0x00, \
  0x33, 0x24, 0x26, 0x00, 0x00, 0x00, \
  0x99, 0x02, 0x12, 0x00, 0x00, 0x00, \
  0x05, 0x80, 0x0e, 0x00, 0x00, 0x00, \
  0x80, 0xa1, 0x82, 0x00, 0x00, 0x00, \
  0x84, 0x48, 0x18, 0x00, 0x00, 0x00, \
  0x40, 0x6d, 0x40, 0x00, 0x00, 0x00, \
  0x0a, 0x90, 0xc0, 0x00, 0x00, 0x00, \
  0x68, 0x04, 0x90, 0x00, 0x00, 0x00, \
  0x10, 0x31, 0x20, 0x00, 0x00, 0x00, \
  0x30, 0x58, 0x04, 0x00, 0x00, 0x00, \
  0x46, 0x42, 0x0c, 0x00, 0x00, 0x00, \
  0x33, 0x20, 0x46, 0x00, 0x00, 0x00, \
  0x99, 0x08, 0x0a, 0x00, 0x00, 0x00, \
  0x05, 0x84, 0x30, 0x00, 0x00, 0x00, \
  0x80, 0xb0, 0x22, 0x00, 0x00, 0x00, \
  0x84, 0x42, 0x90, 0x00, 0x00, 0x00, \
  0x40, 0x73, 0x00, 0x00, 0x00, 0x00, \
  0x0a, 0x81, 0x12, 0x00, 0x00, 0x00, \
  0x68, 0x0c, 0x40, 0x00, 0x00, 0x00, \
  0x10, 0x24, 0x84, 0x00, 0x00, 0x00, \
  0x30, 0x51, 0x40, 0x00, 0x00, 0x00, \
  0x5f, 0x50, 0x88, 0x00, 0x00, 0x00

#define kMaskRandom23_3 \
  0xca, 0xd3, 0x64, 0x00, 0x00, 0x00, \
  0xf1, 0x49, 0x3a, 0x00, 0x00, 0x00, \
  0x76, 0x27, 0xd0, 0x00, 0x00, 0x00

#define kMaskRandom23_4 \
  0xc4, 0xd1, 0x64, 0x00, 0x00, 0x00, \
  0x31, 0x62, 0x96, 0x00, 0x00, 0x00, \
  0x4b, 0x24, 0x5a, 0x00, 0x00, 0x00, \
  0x2c, 0xa8, 0xaa, 0x00, 0x00, 0x00

#define kMaskRandom23_5 \
  0xc6, 0xca, 0x6c, 0x00, 0x00, 0x00, \
  0x63, 0x6c, 0x96, 0x00, 0x00, 0x00, \
  0x1d, 0xa1, 0xdc, 0x00, 0x00, 0x00, \
  0xad, 0x55, 0x38, 0x00, 0x00, 0x00, \
  0xb2, 0xb7, 0x06, 0x00, 0x00, 0x00

#define kMaskRandom23_6 \
  0x64, 0x4a, 0x28, 0x00, 0x00, 0x00, \
  0x51, 0x58, 0xa2, 0x00, 0x00, 0x00, \
  0x0c, 0xa4, 0x30, 0x00, 0x00, 0x00, \
  0xa1, 0x22, 0x46, 0x00, 0x00, 0x00, \
  0x12, 0xa1, 0x1c, 0x00, 0x00, 0x00, \
  0x8a, 0x45, 0xc0, 0x00, 0x00, 0x00

#define kMaskRandom23_7 \
  0x46, 0x4a, 0x6c, 0x00, 0x00, 0x00, \
  0x33, 0x24, 0x26, 0x00, 0x00, 0x00, \
  0x91, 0x92, 0x12, 0x00, 0x00, 0x00, \
  0xa4, 0x20, 0x4a, 0x00, 0x00, 0x00, \
  0x50, 0xa0, 0xd4, 0x00, 0x00, 0x00, \
  0x84, 0xc5, 0x80, 0x00, 0x00, 0x00, \
  0x09, 0x71, 0x0c, 0x00, 0x00, 0x00

#define kMaskRandom23_8 \
  0x0c, 0x84, 0x0c, 0x00, 0x00, 0x00, \
  0x80, 0x70, 0x06, 0x00, 0x00, 0x00, \
  0xa0, 0x88, 0x48, 0x00, 0x00, 0x00, \
  0x05, 0x40, 0x32, 0x00, 0x00, 0x00, \
  0x43, 0x02, 0x82, 0x00, 0x00, 0x00, \
  0x1a, 0x01, 0x50, 0x00, 0x00, 0x00, \
  0x60, 0x27, 0x00, 0x00, 0x00, 0x00, \
  0x14, 0x38, 0xa0, 0x00, 0x00, 0x00

#define kMaskRandom23_9 \
  0x46, 0x4a, 0x6c, 0x00, 0x00, 0x00, \
  0x62, 0x7c, 0x84, 0x00, 0x00, 0x00, \
  0x8c, 0x04, 0x88, 0x00, 0x00, 0x00, \
  0x01, 0x74, 0x22, 0x00, 0x00, 0x00, \
  0x07, 0x83, 0x06, 0x00, 0x00, 0x00, \
  0xa0, 0x80, 0x72, 0x00, 0x00, 0x00, \
  0x18, 0xb1, 0x42, 0x00, 0x00, 0x00, \
  0x91, 0x00, 0x92, 0x00, 0x00, 0x00, \
  0x78, 0x00, 0x1c, 0x00, 0x00, 0x00

#define kMaskRandom24_1 \
  0xff, 0xff, 0xff, 0x00, 0x00, 0x00

#define kMaskRandom24_10 \
  0x11, 0x45, 0x14, 0x00, 0x00, 0x00, \
  0x45, 0x34, 0x53, 0x00, 0x00, 0x00, \
  0x00, 0x48, 0x05, 0x00, 0x00, 0x00, \
  0x10, 0x83, 0x09, 0x00, 0x00, 0x00, \
  0x4a, 0x14, 0xa1, 0x00, 0x00, 0x00, \
  0x40, 0xa4, 0x0a, 0x00, 0x00, 0x00, \
  0xa0, 0x6a, 0x02, 0x00, 0x00, 0x00, \
  0x88, 0x80, 0x8c, 0x00, 0x00, 0x00, \
  0x86, 0x08, 0x60, 0x00, 0x00, 0x00, \
  0x54, 0x0d, 0x40, 0x00, 0x00, 0x00

#define kMaskRandom24_11 \
  0x53, 0x65, 0x34, 0x00, 0x00, 0x00, \
  0xa0, 0x32, 0x11, 0x00, 0x00, 0x00, \
  0x15, 0x11, 0x41, 0x00, 0x00, 0x00, \
  0x03, 0x50, 0x15, 0x00, 0x00, 0x00, \
  0x8c, 0x88, 0xc8, 0x00, 0x00, 0x00, \
  0x28, 0x82, 0x88, 0x00, 0x00, 0x00, \
  0x08, 0x48, 0x84, 0x00, 0x00, 0x00, \
  0x99, 0x01, 0x90, 0x00, 0x00, 0x00, \
  0x22, 0x92, 0x29, 0x00, 0x00, 0x00, \
  0x46, 0x04, 0x60, 0x00, 0x00, 0x00, \
  0x8c, 0x2c, 0x02, 0x00, 0x00, 0x00

#define kMaskRandom24_12 \
  0x10, 0x61, 0x06, 0x00, 0x00, 0x00, \
  0x02, 0x30, 0x23, 0x00, 0x00, 0x00, \
  0x40, 0x54, 0x05, 0x00, 0x00, 0x00, \
  0x21, 0x82, 0x18, 0x00, 0x00, 0x00, \
  0x81, 0x18, 0x11, 0x00, 0x00, 0x00, \
  0x14, 0x81, 0x48, 0x00, 0x00, 0x00, \
  0x98, 0x09, 0x80, 0x00, 0x00, 0x00, \
  0x08, 0x90, 0x89, 0x00, 0x00, 0x00, \
  0x62, 0x06, 0x20, 0x00, 0x00, 0x00, \
  0x24, 0x22, 0x42, 0x00, 0x00, 0x00, \
  0x8a, 0x08, 0xa0, 0x00, 0x00, 0x00, \
  0x84, 0x48, 0x44, 0x00, 0x00, 0x00

#define kMaskRandom24_13 \
  0x51, 0x45, 0x14, 0x00, 0x00, 0x00, \
  0xc5, 0x1c, 0x51, 0x00, 0x00, 0x00, \
  0x21, 0x82, 0x18, 0x00, 0x00, 0x00, \
  0x12, 0x31, 0x23, 0x00, 0x00, 0x00, \
  0x08, 0xe0, 0x8e, 0x00, 0x00, 0x00, \
  0x2e, 0x02, 0xe0, 0x00, 0x00, 0x00, \
  0x53, 0x65, 0x36, 0x00, 0x00, 0x00, \
  0x21, 0x32, 0x13, 0x00, 0x00, 0x00, \
  0x90, 0x99, 0x09, 0x00, 0x00, 0x00, \
  0x02, 0x50, 0x25, 0x00, 0x00, 0x00, \
  0x06, 0xa0, 0x6a, 0x00, 0x00, 0x00, \
  0x2c, 0x02, 0xc0, 0x00, 0x00, 0x00, \
  0x88, 0x68, 0x86, 0x00, 0x00, 0x00

#define kMaskRandom24_14 \
  0x53, 0x65, 0x36, 0x00, 0x00, 0x00, \
  0x21, 0x32, 0x13, 0x00, 0x00, 0x00, \
  0x90, 0x99, 0x09, 0x00, 0x00, 0x00, \
  0x02, 0x50, 0x25, 0x00, 0x00, 0x00, \
  0x06, 0xa0, 0x6a, 0x00, 0x00, 0x00, \
  0x2c, 0x02, 0xc0, 0x00, 0x00, 0x00, \
  0x88, 0x68, 0x86, 0x00, 0x00, 0x00, \
  0x51, 0x45, 0x14, 0x00, 0x00, 0x00, \
  0xc5, 0x1c, 0x51, 0x00, 0x00, 0x00, \
  0x21, 0x82, 0x18, 0x00, 0x00, 0x00, \
  0x12, 0x31, 0x23, 0x00, 0x00, 0x00, \
  0x08, 0xe0, 0x8e, 0x00, 0x00, 0x00, \
  0x2e, 0x02, 0xe0, 0x00, 0x00, 0x00, \
  0xf2, 0xd6, 0x8e, 0x00, 0x00, 0x00

#define kMaskRandom24_15 \
  0x53, 0x65, 0x36, 0x00, 0x00, 0x00, \
  0x21, 0x32, 0x13, 0x00, 0x00, 0x00, \
  0x90, 0x99, 0x09, 0x00, 0x00, 0x00, \
  0x02, 0x50, 0x25, 0x00, 0x00, 0x00, \
  0x06, 0xa0, 0x6a, 0x00, 0x00, 0x00, \
  0x2c, 0x02, 0xc0, 0x00, 0x00, 0x00, \
  0x88, 0x68, 0x86, 0x00, 0x00, 0x00, \
  0x20, 0x62, 0x06, 0x00, 0x00, 0x00, \
  0x80, 0x38, 0x03, 0x00, 0x00, 0x00, \
  0x42, 0x44, 0x24, 0x00, 0x00, 0x00, \
  0x01, 0x90, 0x19, 0x00, 0x00, 0x00, \
  0x14, 0x11, 0x41, 0x00, 0x00, 0x00, \
  0x0a, 0x80, 0xa8, 0x00, 0x00, 0x00, \
  0x38, 0x03, 0x80, 0x00, 0x00, 0x00, \
  0xc5, 0x0c, 0x50, 0x00, 0x00, 0x00

#define kMaskRandom24_16 \
  0x20, 0x62, 0x06, 0x00, 0x00, 0x00, \
  0x80, 0x38, 0x03, 0x00, 0x00, 0x00, \
  0x42, 0x44, 0x24, 0x00, 0x00, 0x00, \
  0x01, 0x90, 0x19, 0x00, 0x00, 0x00, \
  0x14, 0x11, 0x41, 0x00, 0x00, 0x00, \
  0x0a, 0x80, 0xa8, 0x00, 0x00, 0x00, \
  0x38, 0x03, 0x80, 0x00, 0x00, 0x00, \
  0xc5, 0x0c, 0x50, 0x00, 0x00, 0x00, \
  0x53, 0x65, 0x36, 0x00, 0x00, 0x00, \
  0x21, 0x32, 0x13, 0x00, 0x00, 0x00, \
  0x90, 0x99, 0x09, 0x00, 0x00, 0x00, \
  0x02, 0x50, 0x25, 0x00, 0x00, 0x00, \
  0x06, 0xa0, 0x6a, 0x00, 0x00, 0x00, \
  0x2c, 0x02, 0xc0, 0x00, 0x00, 0x00, \
  0x88, 0x68, 0x86, 0x00, 0x00, 0x00, \
  0xff, 0x6e, 0x0a, 0x00, 0x00, 0x00

#define kMaskRandom24_17 \
  0x20, 0x62, 0x06, 0x00, 0x00, 0x00, \
  0x80, 0x38, 0x03, 0x00, 0x00, 0x00, \
  0x42, 0x44, 0x24, 0x00, 0x00, 0x00, \
  0x01, 0x90, 0x19, 0x00, 0x00, 0x00, \
  0x14, 0x11, 0x41, 0x00, 0x00, 0x00, \
  0x0a, 0x80, 0xa8, 0x00, 0x00, 0x00, \
  0x38, 0x03, 0x80, 0x00, 0x00, 0x00, \
  0xc5, 0x0c, 0x50, 0x00, 0x00, 0x00, \
  0x53, 0x65, 0x36, 0x00, 0x00, 0x00, \
  0xe4, 0x2e, 0x42, 0x00, 0x00, 0x00, \
  0x24, 0x42, 0x44, 0x00, 0x00, 0x00, \
  0xa1, 0x1a, 0x11, 0x00, 0x00, 0x00, \
  0x18, 0x31, 0x83, 0x00, 0x00, 0x00, \
  0x03, 0x90, 0x39, 0x00, 0x00, 0x00, \
  0x8a, 0x18, 0xa1, 0x00, 0x00, 0x00, \
  0x04, 0x90, 0x49, 0x00, 0x00, 0x00, \
  0x00, 0xe0, 0x0e, 0x00, 0x00, 0x00

#define kMaskRandom24_18 \
  0x53, 0x65, 0x36, 0x00, 0x00, 0x00, \
  0xe4, 0x2e, 0x42, 0x00, 0x00, 0x00, \
  0x24, 0x42, 0x44, 0x00, 0x00, 0x00, \
  0xa1, 0x1a, 0x11, 0x00, 0x00, 0x00, \
  0x18, 0x31, 0x83, 0x00, 0x00, 0x00, \
  0x03, 0x90, 0x39, 0x00, 0x00, 0x00, \
  0x8a, 0x18, 0xa1, 0x00, 0x00, 0x00, \
  0x04, 0x90, 0x49, 0x00, 0x00, 0x00, \
  0x00, 0xe0, 0x0e, 0x00, 0x00, 0x00, \
  0x20, 0x62, 0x06, 0x00, 0x00, 0x00, \
  0x80, 0x38, 0x03, 0x00, 0x00, 0x00, \
  0x42, 0x44, 0x24, 0x00, 0x00, 0x00, \
  0x01, 0x90, 0x19, 0x00, 0x00, 0x00, \
  0x14, 0x11, 0x41, 0x00, 0x00, 0x00, \
  0x0a, 0x80, 0xa8, 0x00, 0x00, 0x00, \
  0x38, 0x03, 0x80, 0x00, 0x00, 0x00, \
  0xc5, 0x0c, 0x50, 0x00, 0x00, 0x00, \
  0x34, 0x50, 0xae, 0x00, 0x00, 0x00

#define kMaskRandom24_19 \
  0x53, 0x65, 0x36, 0x00, 0x00, 0x00, \
  0xe4, 0x2e, 0x42, 0x00, 0x00, 0x00, \
  0x24, 0x42, 0x44, 0x00, 0x00, 0x00, \
  0xa1, 0x1a, 0x11, 0x00, 0x00, 0x00, \
  0x18, 0x31, 0x83, 0x00, 0x00, 0x00, \
  0x03, 0x90, 0x39, 0x00, 0x00, 0x00, \
  0x8a, 0x18, 0xa1, 0x00, 0x00, 0x00, \
  0x04, 0x90, 0x49, 0x00, 0x00, 0x00, \
  0x00, 0xe0, 0x0e, 0x00, 0x00, 0x00, \
  0x51, 0x45, 0x14, 0x00, 0x00, 0x00, \
  0x45, 0x14, 0x51, 0x00, 0x00, 0x00, \
  0x80, 0xd8, 0x0d, 0x00, 0x00, 0x00, \
  0x24, 0x22, 0x42, 0x00, 0x00, 0x00, \
  0x0a, 0x20, 0xa2, 0x00, 0x00, 0x00, \
  0x00, 0xe0, 0x0e, 0x00, 0x00, 0x00, \
  0xb8, 0x0b, 0x80, 0x00, 0x00, 0x00, \
  0x09, 0x10, 0x91, 0x00, 0x00, 0x00, \
  0x56, 0x05, 0x60, 0x00, 0x00, 0x00, \
  0xa2, 0x8a, 0x28, 0x00, 0x00, 0x00

#define kMaskRandom24_2 \
  0xec, 0xce, 0xcc, 0x00, 0x00, 0x00, \
  0x93, 0xb9, 0x3b, 0x00, 0x00, 0x00

#define kMaskRandom24_20 \
  0x51, 0x45, 0x14, 0x00, 0x00, 0x00, \
  0x45, 0x14, 0x51, 0x00, 0x00, 0x00, \
  0x80, 0xd8, 0x0d, 0x00, 0x00, 0x00, \
  0x24, 0x22, 0x42, 0x00, 0x00, 0x00, \
  0x0a, 0x20, 0xa2, 0x00, 0x00, 0x00, \
  0x00, 0xe0, 0x0e, 0x00, 0x00, 0x00, \
  0xb8, 0x0b, 0x80, 0x00, 0x00, 0x00, \
  0x09, 0x10, 0x91, 0x00, 0x00, 0x00, \
  0x56, 0x05, 0x60, 0x00, 0x00, 0x00, \
  0xa2, 0x8a, 0x28, 0x00, 0x00, 0x00, \
  0x53, 0x65, 0x36, 0x00, 0x00, 0x00, \
  0xe4, 0x2e, 0x42, 0x00, 0x00, 0x00, \
  0x24, 0x42, 0x44, 0x00, 0x00, 0x00, \
  0xa1, 0x1a, 0x11, 0x00, 0x00, 0x00, \
  0x18, 0x31, 0x83, 0x00, 0x00, 0x00, \
  0x03, 0x90, 0x39, 0x00, 0x00, 0x00, \
  0x8a, 0x18, 0xa1, 0x00, 0x00, 0x00, \
  0x04, 0x90, 0x49, 0x00, 0x00, 0x00, \
  0x00, 0xe0, 0x0e, 0x00, 0x00, 0x00, \
  0x98, 0xa2, 0x95, 0x00, 0x00, 0x00

#define kMaskRandom24_21 \
  0x51, 0x45, 0x14, 0x00, 0x00, 0x00, \
  0x45, 0x14, 0x51, 0x00, 0x00, 0x00, \
  0x80, 0xd8, 0x0d, 0x00, 0x00, 0x00, \
  0x24, 0x22, 0x42, 0x00, 0x00, 0x00, \
  0x0a, 0x20, 0xa2, 0x00, 0x00, 0x00, \
  0x00, 0xe0, 0x0e, 0x00, 0x00, 0x00, \
  0xb8, 0x0b, 0x80, 0x00, 0x00, 0x00, \
  0x09, 0x10, 0x91, 0x00, 0x00, 0x00, \
  0x56, 0x05, 0x60, 0x00, 0x00, 0x00, \
  0xa2, 0x8a, 0x28, 0x00, 0x00, 0x00, \
  0x53, 0x65, 0x36, 0x00, 0x00, 0x00, \
  0x21, 0x32, 0x13, 0x00, 0x00, 0x00, \
  0x10, 0x91, 0x09, 0x00, 0x00, 0x00, \
  0x00, 0x70, 0x07, 0x00, 0x00, 0x00, \
  0x0c, 0x10, 0xc1, 0x00, 0x00, 0x00, \
  0x40, 0xc4, 0x0c, 0x00, 0x00, 0x00, \
  0x6a, 0x06, 0xa0, 0x00, 0x00, 0x00, \
  0x86, 0x08, 0x60, 0x00, 0x00, 0x00, \
  0x24, 0x82, 0x48, 0x00, 0x00, 0x00, \
  0x89, 0x08, 0x90, 0x00, 0x00, 0x00, \
  0xc0, 0x2c, 0x02, 0x00, 0x00, 0x00

#define kMaskRandom24_22 \
  0x53, 0x65, 0x36, 0x00, 0x00, 0x00, \
  0x21, 0x32, 0x13, 0x00, 0x00, 0x00, \
  0x10, 0x91, 0x09, 0x00, 0x00, 0x00, \
  0x00, 0x70, 0x07, 0x00, 0x00, 0x00, \
  0x0c, 0x10, 0xc1, 0x00, 0x00, 0x00, \
  0x40, 0xc4, 0x0c, 0x00, 0x00, 0x00, \
  0x6a, 0x06, 0xa0, 0x00, 0x00, 0x00, \
  0x86, 0x08, 0x60, 0x00, 0x00, 0x00, \
  0x24, 0x82, 0x48, 0x00, 0x00, 0x00, \
  0x89, 0x08, 0x90, 0x00, 0x00, 0x00, \
  0xc0, 0x2c, 0x02, 0x00, 0x00, 0x00, \
  0x51, 0x45, 0x14, 0x00, 0x00, 0x00, \
  0x45, 0x14, 0x51, 0x00, 0x00, 0x00, \
  0x80, 0xd8, 0x0d, 0x00, 0x00, 0x00, \
  0x24, 0x22, 0x42, 0x00, 0x00, 0x00, \
  0x0a, 0x20, 0xa2, 0x00, 0x00, 0x00, \
  0x00, 0xe0, 0x0e, 0x00, 0x00, 0x00, \
  0xb8, 0x0b, 0x80, 0x00, 0x00, 0x00, \
  0x09, 0x10, 0x91, 0x00, 0x00, 0x00, \
  0x56, 0x05, 0x60, 0x00, 0x00, 0x00, \
  0xa2, 0x8a, 0x28, 0x00, 0x00, 0x00, \
  0x1a, 0xaa, 0xee, 0x00, 0x00, 0x00

#define kMaskRandom24_23 \
  0x53, 0x65, 0x36, 0x00, 0x00, 0x00, \
  0x21, 0x32, 0x13, 0x00, 0x00, 0x00, \
  0x10, 0x91, 0x09, 0x00, 0x00, 0x00, \
  0x00, 0x70, 0x07, 0x00, 0x00, 0x00, \
  0x0c, 0x10, 0xc1, 0x00, 0x00, 0x00, \
  0x40, 0xc4, 0x0c, 0x00, 0x00, 0x00, \
  0x6a, 0x06, 0xa0, 0x00, 0x00, 0x00, \
  0x86, 0x08, 0x60, 0x00, 0x00, 0x00, \
  0x24, 0x82, 0x48, 0x00, 0x00, 0x00, \
  0x89, 0x08, 0x90, 0x00, 0x00, 0x00, \
  0xc0, 0x2c, 0x02, 0x00, 0x00, 0x00, \
  0x10, 0x61, 0x06, 0x00, 0x00, 0x00, \
  0x02, 0x30, 0x23, 0x00, 0x00, 0x00, \
  0x40, 0x54, 0x05, 0x00, 0x00, 0x00, \
  0x21, 0x82, 0x18, 0x00, 0x00, 0x00, \
  0x81, 0x18, 0x11, 0x00, 0x00, 0x00, \
  0x14, 0x81, 0x48, 0x00, 0x00, 0x00, \
  0x98, 0x09, 0x80, 0x00, 0x00, 0x00, \
  0x08, 0x90, 0x89, 0x00, 0x00, 0x00, \
  0x62, 0x06, 0x20, 0x00, 0x00, 0x00, \
  0x24, 0x22, 0x42, 0x00, 0x00, 0x00, \
  0x8a, 0x08, 0xa0, 0x00, 0x00, 0x00, \
  0x84, 0x48, 0x44, 0x00, 0x00, 0x00

#define kMaskRandom24_24 \
  0x10, 0x61, 0x06, 0x00, 0x00, 0x00, \
  0x02, 0x30, 0x23, 0x00, 0x00, 0x00, \
  0x40, 0x54, 0x05, 0x00, 0x00, 0x00, \
  0x21, 0x82, 0x18, 0x00, 0x00, 0x00, \
  0x81, 0x18, 0x11, 0x00, 0x00, 0x00, \
  0x14, 0x81, 0x48, 0x00, 0x00, 0x00, \
  0x98, 0x09, 0x80, 0x00, 0x00, 0x00, \
  0x08, 0x90, 0x89, 0x00, 0x00, 0x00, \
  0x62, 0x06, 0x20, 0x00, 0x00, 0x00, \
  0x24, 0x22, 0x42, 0x00, 0x00, 0x00, \
  0x8a, 0x08, 0xa0, 0x00, 0x00, 0x00, \
  0x84, 0x48, 0x44, 0x00, 0x00, 0x00, \
  0x53, 0x65, 0x36, 0x00, 0x00, 0x00, \
  0x21, 0x32, 0x13, 0x00, 0x00, 0x00, \
  0x10, 0x91, 0x09, 0x00, 0x00, 0x00, \
  0x00, 0x70, 0x07, 0x00, 0x00, 0x00, \
  0x0c, 0x10, 0xc1, 0x00, 0x00, 0x00, \
  0x40, 0xc4, 0x0c, 0x00, 0x00, 0x00, \
  0x6a, 0x06, 0xa0, 0x00, 0x00, 0x00, \
  0x86, 0x08, 0x60, 0x00, 0x00, 0x00, \
  0x24, 0x82, 0x48, 0x00, 0x00, 0x00, \
  0x89, 0x08, 0x90, 0x00, 0x00, 0x00, \
  0xc0, 0x2c, 0x02, 0x00, 0x00, 0x00, \
  0x88, 0x32, 0x59, 0x00, 0x00, 0x00

#define kMaskRandom24_3 \
  0x9b, 0x29, 0xb2, 0x00, 0x00, 0x00, \
  0x49, 0xd4, 0x9d, 0x00, 0x00, 0x00, \
  0x3e, 0x83, 0xe8, 0x00, 0x00, 0x00

#define kMaskRandom24_4 \
  0x8b, 0x28, 0xb2, 0x00, 0x00, 0x00, \
  0x14, 0xb1, 0x4b, 0x00, 0x00, 0x00, \
  0x22, 0xd2, 0x2d, 0x00, 0x00, 0x00, \
  0x45, 0x54, 0x55, 0x00, 0x00, 0x00

#define kMaskRandom24_5 \
  0x53, 0x65, 0x36, 0x00, 0x00, 0x00, \
  0x64, 0xb6, 0x4b, 0x00, 0x00, 0x00, \
  0x0e, 0xe0, 0xee, 0x00, 0x00, 0x00, \
  0xa9, 0xca, 0x9c, 0x00, 0x00, 0x00, \
  0xb8, 0x3b, 0x83, 0x00, 0x00, 0x00

#define kMaskRandom24_6 \
  0xd1, 0x4d, 0x14, 0x00, 0x00, 0x00, \
  0x45, 0x34, 0x53, 0x00, 0x00, 0x00, \
  0x22, 0xd2, 0x2d, 0x00, 0x00, 0x00, \
  0x16, 0xc1, 0x6c, 0x00, 0x00, 0x00, \
  0x0b, 0xa0, 0xba, 0x00, 0x00, 0x00, \
  0xe8, 0x8e, 0x88, 0x00, 0x00, 0x00

#define kMaskRandom24_7 \
  0xd3, 0x65, 0x36, 0x00, 0x00, 0x00, \
  0x25, 0x32, 0x53, 0x00, 0x00, 0x00, \
  0x30, 0xd3, 0x05, 0x00, 0x00, 0x00, \
  0x06, 0x48, 0x6c, 0x00, 0x00, 0x00, \
  0xc0, 0xb8, 0x1b, 0x00, 0x00, 0x00, \
  0x2a, 0xa2, 0xaa, 0x00, 0x00, 0x00, \
  0xa8, 0x4e, 0x84, 0x00, 0x00, 0x00

#define kMaskRandom24_8 \
  0x81, 0x60, 0x16, 0x00, 0x00, 0x00, \
  0x40, 0x3c, 0x03, 0x00, 0x00, 0x00, \
  0x10, 0x91, 0x09, 0x00, 0x00, 0x00, \
  0x06, 0x50, 0x65, 0x00, 0x00, 0x00, \
  0x20, 0x4a, 0x84, 0x00, 0x00, 0x00, \
  0x8a, 0xa0, 0xaa, 0x00, 0x00, 0x00, \
  0x33, 0x03, 0x30, 0x00, 0x00, 0x00, \
  0x4c, 0x84, 0xc8, 0x00, 0x00, 0x00

#define kMaskRandom24_9 \
  0xd3, 0x65, 0x36, 0x00, 0x00, 0x00, \
  0x64, 0x26, 0x42, 0x00, 0x00, 0x00, \
  0x18, 0x41, 0xc4, 0x00, 0x00, 0x00, \
  0xa0, 0x4a, 0x04, 0x00, 0x00, 0x00, \
  0x81, 0x38, 0x13, 0x00, 0x00, 0x00, \
  0x22, 0xa2, 0x2a, 0x00, 0x00, 0x00, \
  0x08, 0x70, 0x87, 0x00, 0x00, 0x00, \
  0x04, 0x90, 0x49, 0x00, 0x00, 0x00, \
  0x01, 0xc0, 0x1c, 0x00, 0x00, 0x00

#define kMaskRandom25_1 \
  0xff, 0xff, 0xff, 0x80, 0x00, 0x00

#define kMaskRandom25_10 \
  0x51, 0x4d, 0x10, 0x00, 0x00, 0x00, \
  0x45, 0x14, 0x45, 0x00, 0x00, 0x00, \
  0x80, 0xd1, 0x09, 0x80, 0x00, 0x00, \
  0x24, 0x2a, 0x05, 0x00, 0x00, 0x00, \
  0x0a, 0x24, 0xa0, 0x80, 0x00, 0x00, \
  0x00, 0xe4, 0x03, 0x00, 0x00, 0x00, \
  0xb8, 0x08, 0x02, 0x80, 0x00, 0x00, \
  0x09, 0x10, 0xc9, 0x00, 0x00, 0x00, \
  0x56, 0x00, 0x58, 0x80, 0x00, 0x00, \
  0xa2, 0x86, 0x22, 0x00, 0x00, 0x00

#define kMaskRandom25_11 \
  0x53, 0x65, 0x12, 0x00, 0x00, 0x00, \
  0x21, 0x32, 0x21, 0x00, 0x00, 0x00, \
  0x10, 0x91, 0x34, 0x00, 0x00, 0x00, \
  0x00, 0x72, 0x50, 0x00, 0x00, 0x00, \
  0x0c, 0x11, 0x81, 0x80, 0x00, 0x00, \
  0x40, 0xc0, 0xa2, 0x00, 0x00, 0x00, \
  0x6a, 0x08, 0x88, 0x80, 0x00, 0x00, \
  0x86, 0x00, 0x68, 0x00, 0x00, 0x00, \
  0x24, 0x8e, 0x02, 0x00, 0x00, 0x00, \
  0x89, 0x08, 0x44, 0x00, 0x00, 0x00, \
  0xc0, 0x24, 0x41, 0x80, 0x00, 0x00

#define kMaskRandom25_12 \
  0x10, 0x62, 0x82, 0x80, 0x00, 0x00, \
  0x02, 0x38, 0x45, 0x00, 0x00, 0x00, \
  0x40, 0x56, 0x04, 0x00, 0x00, 0x00, \
  0x21, 0x80, 0x54, 0x80, 0x00, 0x00, \
  0x81, 0x10, 0x29, 0x80, 0x00, 0x00, \
  0x14, 0x80, 0x13, 0x00, 0x00, 0x00, \
  0x98, 0x04, 0x81, 0x00, 0x00, 0x00, \
  0x08, 0x92, 0x48, 0x00, 0x00, 0x00, \
  0x62, 0x09, 0x40, 0x00, 0x00, 0x00, \
  0x24, 0x28, 0xa0, 0x00, 0x00, 0x00, \
  0x8a, 0x01, 0x18, 0x00, 0x00, 0x00, \
  0x84, 0x45, 0x22, 0x00, 0x00, 0x00

#define kMaskRandom25_13 \
  0x51, 0x4d, 0x12, 0x00, 0x00, 0x00, \
  0xc5, 0x14, 0x6d, 0x00, 0x00, 0x00, \
  0x21, 0x81, 0x54, 0x80, 0x00, 0x00, \
  0x12, 0x32, 0x17, 0x00, 0x00, 0x00, \
  0x08, 0xe2, 0x8c, 0x80, 0x00, 0x00, \
  0x2e, 0x0a, 0xa2, 0x00, 0x00, 0x00, \
  0x53, 0x65, 0x92, 0x00, 0x00, 0x00, \
  0x21, 0x32, 0x65, 0x00, 0x00, 0x00, \
  0x90, 0x9b, 0x14, 0x00, 0x00, 0x00, \
  0x02, 0x52, 0xb0, 0x80, 0x00, 0x00, \
  0x06, 0xa1, 0x4c, 0x80, 0x00, 0x00, \
  0x2c, 0x0c, 0x88, 0x80, 0x00, 0x00, \
  0x88, 0x68, 0x4b, 0x00, 0x00, 0x00

#define kMaskRandom25_14 \
  0x53, 0x65, 0x92, 0x00, 0x00, 0x00, \
  0x21, 0x32, 0x65, 0x00, 0x00, 0x00, \
  0x90, 0x9b, 0x14, 0x00, 0x00, 0x00, \
  0x02, 0x52, 0xb0, 0x80, 0x00, 0x00, \
  0x06, 0xa1, 0x4c, 0x80, 0x00, 0x00, \
  0x2c, 0x0c, 0x88, 0x80, 0x00, 0x00, \
  0x88, 0x68, 0x4b, 0x00, 0x00, 0x00, \
  0x51, 0x4d, 0x12, 0x00, 0x00, 0x00, \
  0xc5, 0x14, 0x6d, 0x00, 0x00, 0x00, \
  0x21, 0x81, 0x54, 0x80, 0x00, 0x00, \
  0x12, 0x32, 0x17, 0x00, 0x00, 0x00, \
  0x08, 0xe2, 0x8c, 0x80, 0x00, 0x00, \
  0x2e, 0x0a, 0xa2, 0x00, 0x00, 0x00, \
  0x73, 0x76, 0x61, 0x80, 0x00, 0x00

#define kMaskRandom25_15 \
  0x53, 0x65, 0x92, 0x00, 0x00, 0x00, \
  0x21, 0x32, 0x65, 0x00, 0x00, 0x00, \
  0x90, 0x9b, 0x14, 0x00, 0x00, 0x00, \
  0x02, 0x52, 0xb0, 0x80, 0x00, 0x00, \
  0x06, 0xa1, 0x4c, 0x80, 0x00, 0x00, \
  0x2c, 0x0c, 0x88, 0x80, 0x00, 0x00, \
  0x88, 0x68, 0x4b, 0x00, 0x00, 0x00, \
  0x20, 0x68, 0x0a, 0x80, 0x00, 0x00, \
  0x80, 0x33, 0x09, 0x00, 0x00, 0x00, \
  0x42, 0x41, 0x60, 0x80, 0x00, 0x00, \
  0x01, 0x90, 0x33, 0x00, 0x00, 0x00, \
  0x14, 0x14, 0x46, 0x00, 0x00, 0x00, \
  0x0a, 0x80, 0x81, 0x80, 0x00, 0x00, \
  0x38, 0x0d, 0x80, 0x00, 0x00, 0x00, \
  0xc5, 0x0a, 0x14, 0x00, 0x00, 0x00

#define kMaskRandom25_16 \
  0x20, 0x68, 0x0a, 0x80, 0x00, 0x00, \
  0x80, 0x33, 0x09, 0x00, 0x00, 0x00, \
  0x42, 0x41, 0x60, 0x80, 0x00, 0x00, \
  0x01, 0x90, 0x33, 0x00, 0x00, 0x00, \
  0x14, 0x14, 0x46, 0x00, 0x00, 0x00, \
  0x0a, 0x80, 0x81, 0x80, 0x00, 0x00, \
  0x38, 0x0d, 0x80, 0x00, 0x00, 0x00, \
  0xc5, 0x0a, 0x14, 0x00, 0x00, 0x00, \
  0x53, 0x65, 0x92, 0x00, 0x00, 0x00, \
  0x21, 0x32, 0x65, 0x00, 0x00, 0x00, \
  0x90, 0x9b, 0x14, 0x00, 0x00, 0x00, \
  0x02, 0x52, 0xb0, 0x80, 0x00, 0x00, \
  0x06, 0xa1, 0x4c, 0x80, 0x00, 0x00, \
  0x2c, 0x0c, 0x88, 0x80, 0x00, 0x00, \
  0x88, 0x68, 0x4b, 0x00, 0x00, 0x00, \
  0x16, 0xe8, 0xdc, 0x80, 0x00, 0x00

#define kMaskRandom25_17 \
  0x20, 0x68, 0x0a, 0x80, 0x00, 0x00, \
  0x80, 0x33, 0x09, 0x00, 0x00, 0x00, \
  0x42, 0x41, 0x60, 0x80, 0x00, 0x00, \
  0x01, 0x90, 0x33, 0x00, 0x00, 0x00, \
  0x14, 0x14, 0x46, 0x00, 0x00, 0x00, \
  0x0a, 0x80, 0x81, 0x80, 0x00, 0x00, \
  0x38, 0x0d, 0x80, 0x00, 0x00, 0x00, \
  0xc5, 0x0a, 0x14, 0x00, 0x00, 0x00, \
  0x53, 0x65, 0x92, 0x00, 0x00, 0x00, \
  0xe4, 0x26, 0x64, 0x00, 0x00, 0x00, \
  0x24, 0x41, 0x44, 0x00, 0x00, 0x00, \
  0xa1, 0x12, 0x14, 0x80, 0x00, 0x00, \
  0x18, 0x30, 0x2c, 0x80, 0x00, 0x00, \
  0x03, 0x99, 0x41, 0x00, 0x00, 0x00, \
  0x8a, 0x18, 0x0a, 0x80, 0x00, 0x00, \
  0x04, 0x90, 0xa9, 0x00, 0x00, 0x00, \
  0x00, 0xe4, 0x01, 0x80, 0x00, 0x00

#define kMaskRandom25_18 \
  0x53, 0x65, 0x92, 0x00, 0x00, 0x00, \
  0xe4, 0x26, 0x64, 0x00, 0x00, 0x00, \
  0x24, 0x41, 0x44, 0x00, 0x00, 0x00, \
  0xa1, 0x12, 0x14, 0x80, 0x00, 0x00, \
  0x18, 0x30, 0x2c, 0x80, 0x00, 0x00, \
  0x03, 0x99, 0x41, 0x00, 0x00, 0x00, \
  0x8a, 0x18, 0x0a, 0x80, 0x00, 0x00, \
  0x04, 0x90, 0xa9, 0x00, 0x00, 0x00, \
  0x00, 0xe4, 0x01, 0x80, 0x00, 0x00, \
  0x20, 0x68, 0x0a, 0x80, 0x00, 0x00, \
  0x80, 0x33, 0x09, 0x00, 0x00, 0x00, \
  0x42, 0x41, 0x60, 0x80, 0x00, 0x00, \
  0x01, 0x90, 0x33, 0x00, 0x00, 0x00, \
  0x14, 0x14, 0x46, 0x00, 0x00, 0x00, \
  0x0a, 0x80, 0x81, 0x80, 0x00, 0x00, \
  0x38, 0x0d, 0x80, 0x00, 0x00, 0x00, \
  0xc5, 0x0a, 0x14, 0x00, 0x00, 0x00, \
  0xce, 0x9b, 0xe1, 0x00, 0x00, 0x00

#define kMaskRandom25_19 \
  0x53, 0x65, 0x92, 0x00, 0x00, 0x00, \
  0xe4, 0x26, 0x64, 0x00, 0x00, 0x00, \
  0x24, 0x41, 0x44, 0x00, 0x00, 0x00, \
  0xa1, 0x12, 0x14, 0x80, 0x00, 0x00, \
  0x18, 0x30, 0x2c, 0x80, 0x00, 0x00, \
  0x03, 0x99, 0x41, 0x00, 0x00, 0x00, \
  0x8a, 0x18, 0x0a, 0x80, 0x00, 0x00, \
  0x04, 0x90, 0xa9, 0x00, 0x00, 0x00, \
  0x00, 0xe4, 0x01, 0x80, 0x00, 0x00, \
  0x51, 0x4d, 0x10, 0x00, 0x00, 0x00, \
  0x45, 0x14, 0x45, 0x00, 0x00, 0x00, \
  0x80, 0xd1, 0x09, 0x80, 0x00, 0x00, \
  0x24, 0x2a, 0x05, 0x00, 0x00, 0x00, \
  0x0a, 0x24, 0xa0, 0x80, 0x00, 0x00, \
  0x00, 0xe4, 0x03, 0x00, 0x00, 0x00, \
  0xb8, 0x08, 0x02, 0x80, 0x00, 0x00, \
  0x09, 0x10, 0xc9, 0x00, 0x00, 0x00, \
  0x56, 0x00, 0x58, 0x80, 0x00, 0x00, \
  0xa2, 0x86, 0x22, 0x00, 0x00, 0x00

#define kMaskRandom25_2 \
  0xec, 0xce, 0xcc, 0x00, 0x00, 0x00, \
  0x93, 0xb1, 0xb3, 0x80, 0x00, 0x00

#define kMaskRandom25_20 \
  0x51, 0x4d, 0x10, 0x00, 0x00, 0x00, \
  0x45, 0x14, 0x45, 0x00, 0x00, 0x00, \
  0x80, 0xd1, 0x09, 0x80, 0x00, 0x00, \
  0x24, 0x2a, 0x05, 0x00, 0x00, 0x00, \
  0x0a, 0x24, 0xa0, 0x80, 0x00, 0x00, \
  0x00, 0xe4, 0x03, 0x00, 0x00, 0x00, \
  0xb8, 0x08, 0x02, 0x80, 0x00, 0x00, \
  0x09, 0x10, 0xc9, 0x00, 0x00, 0x00, \
  0x56, 0x00, 0x58, 0x80, 0x00, 0x00, \
  0xa2, 0x86, 0x22, 0x00, 0x00, 0x00, \
  0x53, 0x65, 0x92, 0x00, 0x00, 0x00, \
  0xe4, 0x26, 0x64, 0x00, 0x00, 0x00, \
  0x24, 0x41, 0x44, 0x00, 0x00, 0x00, \
  0xa1, 0x12, 0x14, 0x80, 0x00, 0x00, \
  0x18, 0x30, 0x2c, 0x80, 0x00, 0x00, \
  0x03, 0x99, 0x41, 0x00, 0x00, 0x00, \
  0x8a, 0x18, 0x0a, 0x80, 0x00, 0x00, \
  0x04, 0x90, 0xa9, 0x00, 0x00, 0x00, \
  0x00, 0xe4, 0x01, 0x80, 0x00, 0x00, \
  0x1b, 0x8a, 0xa0, 0x00, 0x00, 0x00

#define kMaskRandom25_21 \
  0x51, 0x4d, 0x10, 0x00, 0x00, 0x00, \
  0x45, 0x14, 0x45, 0x00, 0x00, 0x00, \
  0x80, 0xd1, 0x09, 0x80, 0x00, 0x00, \
  0x24, 0x2a, 0x05, 0x00, 0x00, 0x00, \
  0x0a, 0x24, 0xa0, 0x80, 0x00, 0x00, \
  0x00, 0xe4, 0x03, 0x00, 0x00, 0x00, \
  0xb8, 0x08, 0x02, 0x80, 0x00, 0x00, \
  0x09, 0x10, 0xc9, 0x00, 0x00, 0x00, \
  0x56, 0x00, 0x58, 0x80, 0x00, 0x00, \
  0xa2, 0x86, 0x22, 0x00, 0x00, 0x00, \
  0x53, 0x65, 0x12, 0x00, 0x00, 0x00, \
  0x21, 0x32, 0x21, 0x00, 0x00, 0x00, \
  0x10, 0x91, 0x34, 0x00, 0x00, 0x00, \
  0x00, 0x72, 0x50, 0x00, 0x00, 0x00, \
  0x0c, 0x11, 0x81, 0x80, 0x00, 0x00, \
  0x40, 0xc0, 0xa2, 0x00, 0x00, 0x00, \
  0x6a, 0x08, 0x88, 0x80, 0x00, 0x00, \
  0x86, 0x00, 0x68, 0x00, 0x00, 0x00, \
  0x24, 0x8e, 0x02, 0x00, 0x00, 0x00, \
  0x89, 0x08, 0x44, 0x00, 0x00, 0x00, \
  0xc0, 0x24, 0x41, 0x80, 0x00, 0x00

#define kMaskRandom25_22 \
  0x53, 0x65, 0x12, 0x00, 0x00, 0x00, \
  0x21, 0x32, 0x21, 0x00, 0x00, 0x00, \
  0x10, 0x91, 0x34, 0x00, 0x00, 0x00, \
  0x00, 0x72, 0x50, 0x00, 0x00, 0x00, \
  0x0c, 0x11, 0x81, 0x80, 0x00, 0x00, \
  0x40, 0xc0, 0xa2, 0x00, 0x00, 0x00, \
  0x6a, 0x08, 0x88, 0x80, 0x00, 0x00, \
  0x86, 0x00, 0x68, 0x00, 0x00, 0x00, \
  0x24, 0x8e, 0x02, 0x00, 0x00, 0x00, \
  0x89, 0x08, 0x44, 0x00, 0x00, 0x00, \
  0xc0, 0x24, 0x41, 0x80, 0x00, 0x00, \
  0x51, 0x4d, 0x10, 0x00, 0x00, 0x00, \
  0x45, 0x14, 0x45, 0x00, 0x00, 0x00, \
  0x80, 0xd1, 0x09, 0x80, 0x00, 0x00, \
  0x24, 0x2a, 0x05, 0x00, 0x00, 0x00, \
  0x0a, 0x24, 0xa0, 0x80, 0x00, 0x00, \
  0x00, 0xe4, 0x03, 0x00, 0x00, 0x00, \
  0xb8, 0x08, 0x02, 0x80, 0x00, 0x00, \
  0x09, 0x10, 0xc9, 0x00, 0x00, 0x00, \
  0x56, 0x00, 0x58, 0x80, 0x00, 0x00, \
  0xa2, 0x86, 0x22, 0x00, 0x00, 0x00, \
  0x15, 0xa2, 0x99, 0x00, 0x00, 0x00

#define kMaskRandom25_23 \
  0x53, 0x65, 0x12, 0x00, 0x00, 0x00, \
  0x21, 0x32, 0x21, 0x00, 0x00, 0x00, \
  0x10, 0x91, 0x34, 0x00, 0x00, 0x00, \
  0x00, 0x72, 0x50, 0x00, 0x00, 0x00, \
  0x0c, 0x11, 0x81, 0x80, 0x00, 0x00, \
  0x40, 0xc0, 0xa2, 0x00, 0x00, 0x00, \
  0x6a, 0x08, 0x88, 0x80, 0x00, 0x00, \
  0x86, 0x00, 0x68, 0x00, 0x00, 0x00, \
  0x24, 0x8e, 0x02, 0x00, 0x00, 0x00, \
  0x89, 0x08, 0x44, 0x00, 0x00, 0x00, \
  0xc0, 0x24, 0x41, 0x80, 0x00, 0x00, \
  0x10, 0x62, 0x82, 0x80, 0x00, 0x00, \
  0x02, 0x38, 0x45, 0x00, 0x00, 0x00, \
  0x40, 0x56, 0x04, 0x00, 0x00, 0x00, \
  0x21, 0x80, 0x54, 0x80, 0x00, 0x00, \
  0x81, 0x10, 0x29, 0x80, 0x00, 0x00, \
  0x14, 0x80, 0x13, 0x00, 0x00, 0x00, \
  0x98, 0x04, 0x81, 0x00, 0x00, 0x00, \
  0x08, 0x92, 0x48, 0x00, 0x00, 0x00, \
  0x62, 0x09, 0x40, 0x00, 0x00, 0x00, \
  0x24, 0x28, 0xa0, 0x00, 0x00, 0x00, \
  0x8a, 0x01, 0x18, 0x00, 0x00, 0x00, \
  0x84, 0x45, 0x22, 0x00, 0x00, 0x00

#define kMaskRandom25_24 \
  0x10, 0x62, 0x82, 0x80, 0x00, 0x00, \
  0x02, 0x38, 0x45, 0x00, 0x00, 0x00, \
  0x40, 0x56, 0x04, 0x00, 0x00, 0x00, \
  0x21, 0x80, 0x54, 0x80, 0x00, 0x00, \
  0x81, 0x10, 0x29, 0x80, 0x00, 0x00, \
  0x14, 0x80, 0x13, 0x00, 0x00, 0x00, \
  0x98, 0x04, 0x81, 0x00, 0x00, 0x00, \
  0x08, 0x92, 0x48, 0x00, 0x00, 0x00, \
  0x62, 0x09, 0x40, 0x00, 0x00, 0x00, \
  0x24, 0x28, 0xa0, 0x00, 0x00, 0x00, \
  0x8a, 0x01, 0x18, 0x00, 0x00, 0x00, \
  0x84, 0x45, 0x22, 0x00, 0x00, 0x00, \
  0x53, 0x65, 0x12, 0x00, 0x00, 0x00, \
  0x21, 0x32, 0x21, 0x00, 0x00, 0x00, \
  0x10, 0x91, 0x34, 0x00, 0x00, 0x00, \
  0x00, 0x72, 0x50, 0x00, 0x00, 0x00, \
  0x0c, 0x11, 0x81, 0x80, 0x00, 0x00, \
  0x40, 0xc0, 0xa2, 0x00, 0x00, 0x00, \
  0x6a, 0x08, 0x88, 0x80, 0x00, 0x00, \
  0x86, 0x00, 0x68, 0x00, 0x00, 0x00, \
  0x24, 0x8e, 0x02, 0x00, 0x00, 0x00, \
  0x89, 0x08, 0x44, 0x00, 0x00, 0x00, \
  0xc0, 0x24, 0x41, 0x80, 0x00, 0x00, \
  0xf9, 0x0c, 0x14, 0x80, 0x00, 0x00

#define kMaskRandom25_25 \
  0x10, 0x62, 0x82, 0x80, 0x00, 0x00, \
  0x02, 0x38, 0x45, 0x00, 0x00, 0x00, \
  0x40, 0x56, 0x04, 0x00, 0x00, 0x00, \
  0x21, 0x80, 0x54, 0x80, 0x00, 0x00, \
  0x81, 0x10, 0x29, 0x80, 0x00, 0x00, \
  0x14, 0x80, 0x13, 0x00, 0x00, 0x00, \
  0x98, 0x04, 0x81, 0x00, 0x00, 0x00, \
  0x08, 0x92, 0x48, 0x00, 0x00, 0x00, \
  0x62, 0x09, 0x40, 0x00, 0x00, 0x00, \
  0x24, 0x28, 0xa0, 0x00, 0x00, 0x00, \
  0x8a, 0x01, 0x18, 0x00, 0x00, 0x00, \
  0x84, 0x45, 0x22, 0x00, 0x00, 0x00, \
  0x10, 0x65, 0x12, 0x00, 0x00, 0x00, \
  0x02, 0x36, 0x64, 0x00, 0x00, 0x00, \
  0x40, 0x50, 0x54, 0x80, 0x00, 0x00, \
  0x21, 0x88, 0x12, 0x00, 0x00, 0x00, \
  0x81, 0x19, 0x40, 0x00, 0x00, 0x00, \
  0x14, 0x83, 0x08, 0x00, 0x00, 0x00, \
  0x98, 0x02, 0x11, 0x00, 0x00, 0x00, \
  0x08, 0x90, 0x3c, 0x00, 0x00, 0x00, \
  0x62, 0x0e, 0x80, 0x00, 0x00, 0x00, \
  0x24, 0x20, 0xa1, 0x00, 0x00, 0x00, \
  0x8a, 0x08, 0x01, 0x80, 0x00, 0x00, \
  0x84, 0x40, 0x49, 0x00, 0x00, 0x00, \
  0x1c, 0x20, 0x8a, 0x80, 0x00, 0x00

#define kMaskRandom25_3 \
  0x9b, 0x89, 0x9b, 0x00, 0x00, 0x00, \
  0x4f, 0x14, 0x6d, 0x80, 0x00, 0x00, \
  0x3c, 0x63, 0x72, 0x80, 0x00, 0x00

#define kMaskRandom25_4 \
  0x8b, 0x24, 0x9b, 0x00, 0x00, 0x00, \
  0x14, 0xb2, 0x6d, 0x00, 0x00, 0x00, \
  0x22, 0xd8, 0x56, 0x80, 0x00, 0x00, \
  0x45, 0x55, 0x25, 0x80, 0x00, 0x00

#define kMaskRandom25_5 \
  0x53, 0x65, 0x13, 0x00, 0x00, 0x00, \
  0x64, 0x26, 0x64, 0x00, 0x00, 0x00, \
  0x0c, 0xc0, 0xc6, 0x80, 0x00, 0x00, \
  0x82, 0xaa, 0x1c, 0x00, 0x00, 0x00, \
  0x09, 0x32, 0x29, 0x80, 0x00, 0x00

#define kMaskRandom25_6 \
  0x51, 0x4d, 0x12, 0x00, 0x00, 0x00, \
  0xc5, 0x14, 0x6d, 0x00, 0x00, 0x00, \
  0x21, 0x81, 0x54, 0x80, 0x00, 0x00, \
  0x12, 0x32, 0x17, 0x00, 0x00, 0x00, \
  0x08, 0xe2, 0x8c, 0x80, 0x00, 0x00, \
  0x2e, 0x0a, 0xa2, 0x00, 0x00, 0x00

#define kMaskRandom25_7 \
  0x53, 0x65, 0x92, 0x00, 0x00, 0x00, \
  0x21, 0x32, 0x65, 0x00, 0x00, 0x00, \
  0x90, 0x9b, 0x14, 0x00, 0x00, 0x00, \
  0x02, 0x52, 0xb0, 0x80, 0x00, 0x00, \
  0x06, 0xa1, 0x4c, 0x80, 0x00, 0x00, \
  0x2c, 0x0c, 0x88, 0x80, 0x00, 0x00, \
  0x88, 0x68, 0x4b, 0x00, 0x00, 0x00

#define kMaskRandom25_8 \
  0x20, 0x68, 0x0a, 0x80, 0x00, 0x00, \
  0x80, 0x33, 0x09, 0x00, 0x00, 0x00, \
  0x42, 0x41, 0x60, 0x80, 0x00, 0x00, \
  0x01, 0x90, 0x33, 0x00, 0x00, 0x00, \
  0x14, 0x14, 0x46, 0x00, 0x00, 0x00, \
  0x0a, 0x80, 0x81, 0x80, 0x00, 0x00, \
  0x38, 0x0d, 0x80, 0x00, 0x00, 0x00, \
  0xc5, 0x0a, 0x14, 0x00, 0x00, 0x00

#define kMaskRandom25_9 \
  0x53, 0x65, 0x92, 0x00, 0x00, 0x00, \
  0xe4, 0x26, 0x64, 0x00, 0x00, 0x00, \
  0x24, 0x41, 0x44, 0x00, 0x00, 0x00, \
  0xa1, 0x12, 0x14, 0x80, 0x00, 0x00, \
  0x18, 0x30, 0x2c, 0x80, 0x00, 0x00, \
  0x03, 0x99, 0x41, 0x00, 0x00, 0x00, \
  0x8a, 0x18, 0x0a, 0x80, 0x00, 0x00, \
  0x04, 0x90, 0xa9, 0x00, 0x00, 0x00, \
  0x00, 0xe4, 0x01, 0x80, 0x00, 0x00

#define kMaskRandom26_1 \
  0xff, 0xff, 0xff, 0xc0, 0x00, 0x00

#define kMaskRandom26_10 \
  0xd1, 0x06, 0x88, 0x00, 0x00, 0x00, \
  0x44, 0x52, 0x22, 0x80, 0x00, 0x00, \
  0x10, 0x98, 0x84, 0xc0, 0x00, 0x00, \
  0xa0, 0x55, 0x02, 0x80, 0x00, 0x00, \
  0x4a, 0x0a, 0x50, 0x40, 0x00, 0x00, \
  0x40, 0x32, 0x01, 0x80, 0x00, 0x00, \
  0x80, 0x2c, 0x01, 0x40, 0x00, 0x00, \
  0x0c, 0x90, 0x64, 0x80, 0x00, 0x00, \
  0x05, 0x88, 0x2c, 0x40, 0x00, 0x00, \
  0x62, 0x23, 0x11, 0x00, 0x00, 0x00

#define kMaskRandom26_11 \
  0x51, 0x22, 0x89, 0x00, 0x00, 0x00, \
  0x22, 0x11, 0x10, 0x80, 0x00, 0x00, \
  0x13, 0x40, 0x9a, 0x00, 0x00, 0x00, \
  0x25, 0x01, 0x28, 0x00, 0x00, 0x00, \
  0x18, 0x18, 0xc0, 0xc0, 0x00, 0x00, \
  0x0a, 0x20, 0x51, 0x00, 0x00, 0x00, \
  0x88, 0x8c, 0x44, 0x40, 0x00, 0x00, \
  0x06, 0x80, 0x34, 0x00, 0x00, 0x00, \
  0xe0, 0x27, 0x01, 0x00, 0x00, 0x00, \
  0x84, 0x44, 0x22, 0x00, 0x00, 0x00, \
  0x44, 0x1a, 0x20, 0xc0, 0x00, 0x00

#define kMaskRandom26_12 \
  0x28, 0x29, 0x41, 0x40, 0x00, 0x00, \
  0x84, 0x54, 0x22, 0x80, 0x00, 0x00, \
  0x60, 0x43, 0x02, 0x00, 0x00, 0x00, \
  0x05, 0x48, 0x2a, 0x40, 0x00, 0x00, \
  0x02, 0x98, 0x14, 0xc0, 0x00, 0x00, \
  0x01, 0x30, 0x09, 0x80, 0x00, 0x00, \
  0x48, 0x12, 0x40, 0x80, 0x00, 0x00, \
  0x24, 0x81, 0x24, 0x00, 0x00, 0x00, \
  0x94, 0x04, 0xa0, 0x00, 0x00, 0x00, \
  0x8a, 0x04, 0x50, 0x00, 0x00, 0x00, \
  0x11, 0x80, 0x8c, 0x00, 0x00, 0x00, \
  0x52, 0x22, 0x91, 0x00, 0x00, 0x00

#define kMaskRandom26_13 \
  0x51, 0x22, 0x89, 0x00, 0x00, 0x00, \
  0x66, 0x43, 0x32, 0x00, 0x00, 0x00, \
  0x05, 0x48, 0x2a, 0x40, 0x00, 0x00, \
  0x81, 0x24, 0x09, 0x00, 0x00, 0x00, \
  0x94, 0x04, 0xa0, 0x00, 0x00, 0x00, \
  0x30, 0x81, 0x84, 0x00, 0x00, 0x00, \
  0x21, 0x11, 0x08, 0x80, 0x00, 0x00, \
  0x03, 0xc0, 0x1e, 0x00, 0x00, 0x00, \
  0xe8, 0x07, 0x40, 0x00, 0x00, 0x00, \
  0x0a, 0x10, 0x50, 0x80, 0x00, 0x00, \
  0x80, 0x1c, 0x00, 0xc0, 0x00, 0x00, \
  0x04, 0x90, 0x24, 0x80, 0x00, 0x00, \
  0x08, 0xa8, 0x45, 0x40, 0x00, 0x00

#define kMaskRandom26_14 \
  0x59, 0x22, 0xc9, 0x00, 0x00, 0x00, \
  0x26, 0x51, 0x32, 0x80, 0x00, 0x00, \
  0xb1, 0x45, 0x8a, 0x00, 0x00, 0x00, \
  0x2b, 0x09, 0x58, 0x40, 0x00, 0x00, \
  0x14, 0xc8, 0xa6, 0x40, 0x00, 0x00, \
  0xc8, 0x8e, 0x44, 0x40, 0x00, 0x00, \
  0x84, 0xb4, 0x25, 0x80, 0x00, 0x00, \
  0xd1, 0x26, 0x89, 0x00, 0x00, 0x00, \
  0x46, 0xd2, 0x36, 0x80, 0x00, 0x00, \
  0x15, 0x48, 0xaa, 0x40, 0x00, 0x00, \
  0x21, 0x71, 0x0b, 0x80, 0x00, 0x00, \
  0x28, 0xc9, 0x46, 0x40, 0x00, 0x00, \
  0xaa, 0x25, 0x51, 0x00, 0x00, 0x00, \
  0x5d, 0xa7, 0x78, 0x40, 0x00, 0x00

#define kMaskRandom26_15 \
  0x59, 0x22, 0xc9, 0x00, 0x00, 0x00, \
  0x26, 0x51, 0x32, 0x80, 0x00, 0x00, \
  0xb1, 0x45, 0x8a, 0x00, 0x00, 0x00, \
  0x2b, 0x09, 0x58, 0x40, 0x00, 0x00, \
  0x14, 0xc8, 0xa6, 0x40, 0x00, 0x00, \
  0xc8, 0x8e, 0x44, 0x40, 0x00, 0x00, \
  0x84, 0xb4, 0x25, 0x80, 0x00, 0x00, \
  0x80, 0xac, 0x05, 0x40, 0x00, 0x00, \
  0x30, 0x91, 0x84, 0x80, 0x00, 0x00, \
  0x16, 0x08, 0xb0, 0x40, 0x00, 0x00, \
  0x03, 0x30, 0x19, 0x80, 0x00, 0x00, \
  0x44, 0x62, 0x23, 0x00, 0x00, 0x00, \
  0x08, 0x18, 0x40, 0xc0, 0x00, 0x00, \
  0xd8, 0x06, 0xc0, 0x00, 0x00, 0x00, \
  0xa1, 0x45, 0x0a, 0x00, 0x00, 0x00

#define kMaskRandom26_16 \
  0x80, 0xac, 0x05, 0x40, 0x00, 0x00, \
  0x30, 0x91, 0x84, 0x80, 0x00, 0x00, \
  0x16, 0x08, 0xb0, 0x40, 0x00, 0x00, \
  0x03, 0x30, 0x19, 0x80, 0x00, 0x00, \
  0x44, 0x62, 0x23, 0x00, 0x00, 0x00, \
  0x08, 0x18, 0x40, 0xc0, 0x00, 0x00, \
  0xd8, 0x06, 0xc0, 0x00, 0x00, 0x00, \
  0xa1, 0x45, 0x0a, 0x00, 0x00, 0x00, \
  0x59, 0x22, 0xc9, 0x00, 0x00, 0x00, \
  0x26, 0x51, 0x32, 0x80, 0x00, 0x00, \
  0xb1, 0x45, 0x8a, 0x00, 0x00, 0x00, \
  0x2b, 0x09, 0x58, 0x40, 0x00, 0x00, \
  0x14, 0xc8, 0xa6, 0x40, 0x00, 0x00, \
  0xc8, 0x8e, 0x44, 0x40, 0x00, 0x00, \
  0x84, 0xb4, 0x25, 0x80, 0x00, 0x00, \
  0x3c, 0xaf, 0x88, 0x80, 0x00, 0x00

#define kMaskRandom26_17 \
  0x80, 0xac, 0x05, 0x40, 0x00, 0x00, \
  0x30, 0x91, 0x84, 0x80, 0x00, 0x00, \
  0x16, 0x08, 0xb0, 0x40, 0x00, 0x00, \
  0x03, 0x30, 0x19, 0x80, 0x00, 0x00, \
  0x44, 0x62, 0x23, 0x00, 0x00, 0x00, \
  0x08, 0x18, 0x40, 0xc0, 0x00, 0x00, \
  0xd8, 0x06, 0xc0, 0x00, 0x00, 0x00, \
  0xa1, 0x45, 0x0a, 0x00, 0x00, 0x00, \
  0x59, 0x22, 0xc9, 0x00, 0x00, 0x00, \
  0x66, 0x43, 0x32, 0x00, 0x00, 0x00, \
  0x14, 0x40, 0xa2, 0x00, 0x00, 0x00, \
  0x21, 0x49, 0x0a, 0x40, 0x00, 0x00, \
  0x02, 0xc8, 0x16, 0x40, 0x00, 0x00, \
  0x94, 0x14, 0xa0, 0x80, 0x00, 0x00, \
  0x80, 0xac, 0x05, 0x40, 0x00, 0x00, \
  0x0a, 0x90, 0x54, 0x80, 0x00, 0x00, \
  0x40, 0x1a, 0x00, 0xc0, 0x00, 0x00

#define kMaskRandom26_18 \
  0x59, 0x22, 0xc9, 0x00, 0x00, 0x00, \
  0x66, 0x43, 0x32, 0x00, 0x00, 0x00, \
  0x14, 0x40, 0xa2, 0x00, 0x00, 0x00, \
  0x21, 0x49, 0x0a, 0x40, 0x00, 0x00, \
  0x02, 0xc8, 0x16, 0x40, 0x00, 0x00, \
  0x94, 0x14, 0xa0, 0x80, 0x00, 0x00, \
  0x80, 0xac, 0x05, 0x40, 0x00, 0x00, \
  0x0a, 0x90, 0x54, 0x80, 0x00, 0x00, \
  0x40, 0x1a, 0x00, 0xc0, 0x00, 0x00, \
  0x80, 0xac, 0x05, 0x40, 0x00, 0x00, \
  0x30, 0x91, 0x84, 0x80, 0x00, 0x00, \
  0x16, 0x08, 0xb0, 0x40, 0x00, 0x00, \
  0x03, 0x30, 0x19, 0x80, 0x00, 0x00, \
  0x44, 0x62, 0x23, 0x00, 0x00, 0x00, \
  0x08, 0x18, 0x40, 0xc0, 0x00, 0x00, \
  0xd8, 0x06, 0xc0, 0x00, 0x00, 0x00, \
  0xa1, 0x45, 0x0a, 0x00, 0x00, 0x00, \
  0xaa, 0x0c, 0x83, 0x80, 0x00, 0x00

#define kMaskRandom26_19 \
  0x59, 0x22, 0xc9, 0x00, 0x00, 0x00, \
  0x66, 0x43, 0x32, 0x00, 0x00, 0x00, \
  0x14, 0x40, 0xa2, 0x00, 0x00, 0x00, \
  0x21, 0x49, 0x0a, 0x40, 0x00, 0x00, \
  0x02, 0xc8, 0x16, 0x40, 0x00, 0x00, \
  0x94, 0x14, 0xa0, 0x80, 0x00, 0x00, \
  0x80, 0xac, 0x05, 0x40, 0x00, 0x00, \
  0x0a, 0x90, 0x54, 0x80, 0x00, 0x00, \
  0x40, 0x1a, 0x00, 0xc0, 0x00, 0x00, \
  0xd1, 0x06, 0x88, 0x00, 0x00, 0x00, \
  0x44, 0x52, 0x22, 0x80, 0x00, 0x00, \
  0x10, 0x98, 0x84, 0xc0, 0x00, 0x00, \
  0xa0, 0x55, 0x02, 0x80, 0x00, 0x00, \
  0x4a, 0x0a, 0x50, 0x40, 0x00, 0x00, \
  0x40, 0x32, 0x01, 0x80, 0x00, 0x00, \
  0x80, 0x2c, 0x01, 0x40, 0x00, 0x00, \
  0x0c, 0x90, 0x64, 0x80, 0x00, 0x00, \
  0x05, 0x88, 0x2c, 0x40, 0x00, 0x00, \
  0x62, 0x23, 0x11, 0x00, 0x00, 0x00

#define kMaskRandom26_2 \
  0xec, 0xc7, 0x66, 0x00, 0x00, 0x00, \
  0x1b, 0x38, 0xd9, 0xc0, 0x00, 0x00

#define kMaskRandom26_20 \
  0xd1, 0x06, 0x88, 0x00, 0x00, 0x00, \
  0x44, 0x52, 0x22, 0x80, 0x00, 0x00, \
  0x10, 0x98, 0x84, 0xc0, 0x00, 0x00, \
  0xa0, 0x55, 0x02, 0x80, 0x00, 0x00, \
  0x4a, 0x0a, 0x50, 0x40, 0x00, 0x00, \
  0x40, 0x32, 0x01, 0x80, 0x00, 0x00, \
  0x80, 0x2c, 0x01, 0x40, 0x00, 0x00, \
  0x0c, 0x90, 0x64, 0x80, 0x00, 0x00, \
  0x05, 0x88, 0x2c, 0x40, 0x00, 0x00, \
  0x62, 0x23, 0x11, 0x00, 0x00, 0x00, \
  0x59, 0x22, 0xc9, 0x00, 0x00, 0x00, \
  0x66, 0x43, 0x32, 0x00, 0x00, 0x00, \
  0x14, 0x40, 0xa2, 0x00, 0x00, 0x00, \
  0x21, 0x49, 0x0a, 0x40, 0x00, 0x00, \
  0x02, 0xc8, 0x16, 0x40, 0x00, 0x00, \
  0x94, 0x14, 0xa0, 0x80, 0x00, 0x00, \
  0x80, 0xac, 0x05, 0x40, 0x00, 0x00, \
  0x0a, 0x90, 0x54, 0x80, 0x00, 0x00, \
  0x40, 0x1a, 0x00, 0xc0, 0x00, 0x00, \
  0xf4, 0x08, 0xec, 0x00, 0x00, 0x00

#define kMaskRandom26_21 \
  0xd1, 0x06, 0x88, 0x00, 0x00, 0x00, \
  0x44, 0x52, 0x22, 0x80, 0x00, 0x00, \
  0x10, 0x98, 0x84, 0xc0, 0x00, 0x00, \
  0xa0, 0x55, 0x02, 0x80, 0x00, 0x00, \
  0x4a, 0x0a, 0x50, 0x40, 0x00, 0x00, \
  0x40, 0x32, 0x01, 0x80, 0x00, 0x00, \
  0x80, 0x2c, 0x01, 0x40, 0x00, 0x00, \
  0x0c, 0x90, 0x64, 0x80, 0x00, 0x00, \
  0x05, 0x88, 0x2c, 0x40, 0x00, 0x00, \
  0x62, 0x23, 0x11, 0x00, 0x00, 0x00, \
  0x51, 0x22, 0x89, 0x00, 0x00, 0x00, \
  0x22, 0x11, 0x10, 0x80, 0x00, 0x00, \
  0x13, 0x40, 0x9a, 0x00, 0x00, 0x00, \
  0x25, 0x01, 0x28, 0x00, 0x00, 0x00, \
  0x18, 0x18, 0xc0, 0xc0, 0x00, 0x00, \
  0x0a, 0x20, 0x51, 0x00, 0x00, 0x00, \
  0x88, 0x8c, 0x44, 0x40, 0x00, 0x00, \
  0x06, 0x80, 0x34, 0x00, 0x00, 0x00, \
  0xe0, 0x27, 0x01, 0x00, 0x00, 0x00, \
  0x84, 0x44, 0x22, 0x00, 0x00, 0x00, \
  0x44, 0x1a, 0x20, 0xc0, 0x00, 0x00

#define kMaskRandom26_22 \
  0x51, 0x22, 0x89, 0x00, 0x00, 0x00, \
  0x22, 0x11, 0x10, 0x80, 0x00, 0x00, \
  0x13, 0x40, 0x9a, 0x00, 0x00, 0x00, \
  0x25, 0x01, 0x28, 0x00, 0x00, 0x00, \
  0x18, 0x18, 0xc0, 0xc0, 0x00, 0x00, \
  0x0a, 0x20, 0x51, 0x00, 0x00, 0x00, \
  0x88, 0x8c, 0x44, 0x40, 0x00, 0x00, \
  0x06, 0x80, 0x34, 0x00, 0x00, 0x00, \
  0xe0, 0x27, 0x01, 0x00, 0x00, 0x00, \
  0x84, 0x44, 0x22, 0x00, 0x00, 0x00, \
  0x44, 0x1a, 0x20, 0xc0, 0x00, 0x00, \
  0xd1, 0x06, 0x88, 0x00, 0x00, 0x00, \
  0x44, 0x52, 0x22, 0x80, 0x00, 0x00, \
  0x10, 0x98, 0x84, 0xc0, 0x00, 0x00, \
  0xa0, 0x55, 0x02, 0x80, 0x00, 0x00, \
  0x4a, 0x0a, 0x50, 0x40, 0x00, 0x00, \
  0x40, 0x32, 0x01, 0x80, 0x00, 0x00, \
  0x80, 0x2c, 0x01, 0x40, 0x00, 0x00, \
  0x0c, 0x90, 0x64, 0x80, 0x00, 0x00, \
  0x05, 0x88, 0x2c, 0x40, 0x00, 0x00, \
  0x62, 0x23, 0x11, 0x00, 0x00, 0x00, \
  0x13, 0xc6, 0x6b, 0x40, 0x00, 0x00

#define kMaskRandom26_23 \
  0x51, 0x22, 0x89, 0x00, 0x00, 0x00, \
  0x22, 0x11, 0x10, 0x80, 0x00, 0x00, \
  0x13, 0x40, 0x9a, 0x00, 0x00, 0x00, \
  0x25, 0x01, 0x28, 0x00, 0x00, 0x00, \
  0x18, 0x18, 0xc0, 0xc0, 0x00, 0x00, \
  0x0a, 0x20, 0x51, 0x00, 0x00, 0x00, \
  0x88, 0x8c, 0x44, 0x40, 0x00, 0x00, \
  0x06, 0x80, 0x34, 0x00, 0x00, 0x00, \
  0xe0, 0x27, 0x01, 0x00, 0x00, 0x00, \
  0x84, 0x44, 0x22, 0x00, 0x00, 0x00, \
  0x44, 0x1a, 0x20, 0xc0, 0x00, 0x00, \
  0x28, 0x29, 0x41, 0x40, 0x00, 0x00, \
  0x84, 0x54, 0x22, 0x80, 0x00, 0x00, \
  0x60, 0x43, 0x02, 0x00, 0x00, 0x00, \
  0x05, 0x48, 0x2a, 0x40, 0x00, 0x00, \
  0x02, 0x98, 0x14, 0xc0, 0x00, 0x00, \
  0x01, 0x30, 0x09, 0x80, 0x00, 0x00, \
  0x48, 0x12, 0x40, 0x80, 0x00, 0x00, \
  0x24, 0x81, 0x24, 0x00, 0x00, 0x00, \
  0x94, 0x04, 0xa0, 0x00, 0x00, 0x00, \
  0x8a, 0x04, 0x50, 0x00, 0x00, 0x00, \
  0x11, 0x80, 0x8c, 0x00, 0x00, 0x00, \
  0x52, 0x22, 0x91, 0x00, 0x00, 0x00

#define kMaskRandom26_24 \
  0x28, 0x29, 0x41, 0x40, 0x00, 0x00, \
  0x84, 0x54, 0x22, 0x80, 0x00, 0x00, \
  0x60, 0x43, 0x02, 0x00, 0x00, 0x00, \
  0x05, 0x48, 0x2a, 0x40, 0x00, 0x00, \
  0x02, 0x98, 0x14, 0xc0, 0x00, 0x00, \
  0x01, 0x30, 0x09, 0x80, 0x00, 0x00, \
  0x48, 0x12, 0x40, 0x80, 0x00, 0x00, \
  0x24, 0x81, 0x24, 0x00, 0x00, 0x00, \
  0x94, 0x04, 0xa0, 0x00, 0x00, 0x00, \
  0x8a, 0x04, 0x50, 0x00, 0x00, 0x00, \
  0x11, 0x80, 0x8c, 0x00, 0x00, 0x00, \
  0x52, 0x22, 0x91, 0x00, 0x00, 0x00, \
  0x51, 0x22, 0x89, 0x00, 0x00, 0x00, \
  0x22, 0x11, 0x10, 0x80, 0x00, 0x00, \
  0x13, 0x40, 0x9a, 0x00, 0x00, 0x00, \
  0x25, 0x01, 0x28, 0x00, 0x00, 0x00, \
  0x18, 0x18, 0xc0, 0xc0, 0x00, 0x00, \
  0x0a, 0x20, 0x51, 0x00, 0x00, 0x00, \
  0x88, 0x8c, 0x44, 0x40, 0x00, 0x00, \
  0x06, 0x80, 0x34, 0x00, 0x00, 0x00, \
  0xe0, 0x27, 0x01, 0x00, 0x00, 0x00, \
  0x84, 0x44, 0x22, 0x00, 0x00, 0x00, \
  0x44, 0x1a, 0x20, 0xc0, 0x00, 0x00, \
  0xdb, 0x4d, 0xd8, 0x40, 0x00, 0x00

#define kMaskRandom26_25 \
  0x28, 0x29, 0x41, 0x40, 0x00, 0x00, \
  0x84, 0x54, 0x22, 0x80, 0x00, 0x00, \
  0x60, 0x43, 0x02, 0x00, 0x00, 0x00, \
  0x05, 0x48, 0x2a, 0x40, 0x00, 0x00, \
  0x02, 0x98, 0x14, 0xc0, 0x00, 0x00, \
  0x01, 0x30, 0x09, 0x80, 0x00, 0x00, \
  0x48, 0x12, 0x40, 0x80, 0x00, 0x00, \
  0x24, 0x81, 0x24, 0x00, 0x00, 0x00, \
  0x94, 0x04, 0xa0, 0x00, 0x00, 0x00, \
  0x8a, 0x04, 0x50, 0x00, 0x00, 0x00, \
  0x11, 0x80, 0x8c, 0x00, 0x00, 0x00, \
  0x52, 0x22, 0x91, 0x00, 0x00, 0x00, \
  0x51, 0x22, 0x89, 0x00, 0x00, 0x00, \
  0x66, 0x43, 0x32, 0x00, 0x00, 0x00, \
  0x05, 0x48, 0x2a, 0x40, 0x00, 0x00, \
  0x81, 0x24, 0x09, 0x00, 0x00, 0x00, \
  0x94, 0x04, 0xa0, 0x00, 0x00, 0x00, \
  0x30, 0x81, 0x84, 0x00, 0x00, 0x00, \
  0x21, 0x11, 0x08, 0x80, 0x00, 0x00, \
  0x03, 0xc0, 0x1e, 0x00, 0x00, 0x00, \
  0xe8, 0x07, 0x40, 0x00, 0x00, 0x00, \
  0x0a, 0x10, 0x50, 0x80, 0x00, 0x00, \
  0x80, 0x1c, 0x00, 0xc0, 0x00, 0x00, \
  0x04, 0x90, 0x24, 0x80, 0x00, 0x00, \
  0x08, 0xa8, 0x45, 0x40, 0x00, 0x00

#define kMaskRandom26_26 \
  0x51, 0x22, 0x89, 0x00, 0x00, 0x00, \
  0x66, 0x43, 0x32, 0x00, 0x00, 0x00, \
  0x05, 0x48, 0x2a, 0x40, 0x00, 0x00, \
  0x81, 0x24, 0x09, 0x00, 0x00, 0x00, \
  0x94, 0x04, 0xa0, 0x00, 0x00, 0x00, \
  0x30, 0x81, 0x84, 0x00, 0x00, 0x00, \
  0x21, 0x11, 0x08, 0x80, 0x00, 0x00, \
  0x03, 0xc0, 0x1e, 0x00, 0x00, 0x00, \
  0xe8, 0x07, 0x40, 0x00, 0x00, 0x00, \
  0x0a, 0x10, 0x50, 0x80, 0x00, 0x00, \
  0x80, 0x1c, 0x00, 0xc0, 0x00, 0x00, \
  0x04, 0x90, 0x24, 0x80, 0x00, 0x00, \
  0x08, 0xa8, 0x45, 0x40, 0x00, 0x00, \
  0x28, 0x29, 0x41, 0x40, 0x00, 0x00, \
  0x84, 0x54, 0x22, 0x80, 0x00, 0x00, \
  0x60, 0x43, 0x02, 0x00, 0x00, 0x00, \
  0x05, 0x48, 0x2a, 0x40, 0x00, 0x00, \
  0x02, 0x98, 0x14, 0xc0, 0x00, 0x00, \
  0x01, 0x30, 0x09, 0x80, 0x00, 0x00, \
  0x48, 0x12, 0x40, 0x80, 0x00, 0x00, \
  0x24, 0x81, 0x24, 0x00, 0x00, 0x00, \
  0x94, 0x04, 0xa0, 0x00, 0x00, 0x00, \
  0x8a, 0x04, 0x50, 0x00, 0x00, 0x00, \
  0x11, 0x80, 0x8c, 0x00, 0x00, 0x00, \
  0x52, 0x22, 0x91, 0x00, 0x00, 0x00, \
  0xf9, 0x13, 0x51, 0x80, 0x00, 0x00

#define kMaskRandom26_3 \
  0x99, 0xb4, 0xcd, 0x80, 0x00, 0x00, \
  0x46, 0xda, 0x36, 0xc0, 0x00, 0x00, \
  0x37, 0x29, 0xb9, 0x40, 0x00, 0x00

#define kMaskRandom26_4 \
  0x49, 0xb2, 0x4d, 0x80, 0x00, 0x00, \
  0x26, 0xd1, 0x36, 0x80, 0x00, 0x00, \
  0x85, 0x6c, 0x2b, 0x40, 0x00, 0x00, \
  0x52, 0x5a, 0x92, 0xc0, 0x00, 0x00

#define kMaskRandom26_5 \
  0x51, 0x32, 0x89, 0x80, 0x00, 0x00, \
  0x66, 0x43, 0x32, 0x00, 0x00, 0x00, \
  0x0c, 0x68, 0x63, 0x40, 0x00, 0x00, \
  0xa1, 0xc5, 0x0e, 0x00, 0x00, 0x00, \
  0x22, 0x99, 0x14, 0xc0, 0x00, 0x00

#define kMaskRandom26_6 \
  0xd1, 0x26, 0x89, 0x00, 0x00, 0x00, \
  0x46, 0xd2, 0x36, 0x80, 0x00, 0x00, \
  0x15, 0x48, 0xaa, 0x40, 0x00, 0x00, \
  0x21, 0x71, 0x0b, 0x80, 0x00, 0x00, \
  0x28, 0xc9, 0x46, 0x40, 0x00, 0x00, \
  0xaa, 0x25, 0x51, 0x00, 0x00, 0x00

#define kMaskRandom26_7 \
  0x59, 0x22, 0xc9, 0x00, 0x00, 0x00, \
  0x26, 0x51, 0x32, 0x80, 0x00, 0x00, \
  0xb1, 0x45, 0x8a, 0x00, 0x00, 0x00, \
  0x2b, 0x09, 0x58, 0x40, 0x00, 0x00, \
  0x14, 0xc8, 0xa6, 0x40, 0x00, 0x00, \
  0xc8, 0x8e, 0x44, 0x40, 0x00, 0x00, \
  0x84, 0xb4, 0x25, 0x80, 0x00, 0x00

#define kMaskRandom26_8 \
  0x80, 0xac, 0x05, 0x40, 0x00, 0x00, \
  0x30, 0x91, 0x84, 0x80, 0x00, 0x00, \
  0x16, 0x08, 0xb0, 0x40, 0x00, 0x00, \
  0x03, 0x30, 0x19, 0x80, 0x00, 0x00, \
  0x44, 0x62, 0x23, 0x00, 0x00, 0x00, \
  0x08, 0x18, 0x40, 0xc0, 0x00, 0x00, \
  0xd8, 0x06, 0xc0, 0x00, 0x00, 0x00, \
  0xa1, 0x45, 0x0a, 0x00, 0x00, 0x00

#define kMaskRandom26_9 \
  0x59, 0x22, 0xc9, 0x00, 0x00, 0x00, \
  0x66, 0x43, 0x32, 0x00, 0x00, 0x00, \
  0x14, 0x40, 0xa2, 0x00, 0x00, 0x00, \
  0x21, 0x49, 0x0a, 0x40, 0x00, 0x00, \
  0x02, 0xc8, 0x16, 0x40, 0x00, 0x00, \
  0x94, 0x14, 0xa0, 0x80, 0x00, 0x00, \
  0x80, 0xac, 0x05, 0x40, 0x00, 0x00, \
  0x0a, 0x90, 0x54, 0x80, 0x00, 0x00, \
  0x40, 0x1a, 0x00, 0xc0, 0x00, 0x00

#define kMaskRandom27_1 \
  0xff, 0xff, 0xff, 0xe0, 0x00, 0x00

#define kMaskRandom27_10 \
  0xd1, 0x06, 0x06, 0xa0, 0x00, 0x00, \
  0x44, 0x50, 0xea, 0x00, 0x00, 0x00, \
  0x10, 0x9e, 0xa0, 0x40, 0x00, 0x00, \
  0xa0, 0x50, 0x13, 0x00, 0x00, 0x00, \
  0x4a, 0x08, 0x21, 0x40, 0x00, 0x00, \
  0x40, 0x31, 0x04, 0xc0, 0x00, 0x00, \
  0x80, 0x2a, 0x02, 0x20, 0x00, 0x00, \
  0x0c, 0x90, 0x44, 0x20, 0x00, 0x00, \
  0x05, 0x8b, 0x40, 0x00, 0x00, 0x00, \
  0x62, 0x21, 0x18, 0x80, 0x00, 0x00

#define kMaskRandom27_11 \
  0x51, 0x23, 0x16, 0x80, 0x00, 0x00, \
  0x22, 0x11, 0xa9, 0x00, 0x00, 0x00, \
  0x13, 0x40, 0xa0, 0xa0, 0x00, 0x00, \
  0x25, 0x06, 0x28, 0x40, 0x00, 0x00, \
  0x18, 0x19, 0x10, 0x60, 0x00, 0x00, \
  0x0a, 0x24, 0x45, 0xc0, 0x00, 0x00, \
  0x88, 0x8a, 0x12, 0xa0, 0x00, 0x00, \
  0x06, 0x81, 0x45, 0x20, 0x00, 0x00, \
  0xe0, 0x24, 0xa1, 0x00, 0x00, 0x00, \
  0x84, 0x40, 0xd8, 0x20, 0x00, 0x00, \
  0x44, 0x19, 0x16, 0x00, 0x00, 0x00

#define kMaskRandom27_12 \
  0x28, 0x2c, 0x08, 0x20, 0x00, 0x00, \
  0x84, 0x52, 0x03, 0x40, 0x00, 0x00, \
  0x60, 0x44, 0x81, 0x20, 0x00, 0x00, \
  0x05, 0x49, 0x41, 0x40, 0x00, 0x00, \
  0x02, 0x9a, 0x90, 0x80, 0x00, 0x00, \
  0x01, 0x32, 0x0c, 0x40, 0x00, 0x00, \
  0x48, 0x10, 0x49, 0x80, 0x00, 0x00, \
  0x24, 0x82, 0x42, 0x20, 0x00, 0x00, \
  0x94, 0x00, 0x22, 0x20, 0x00, 0x00, \
  0x8a, 0x00, 0x74, 0x00, 0x00, 0x00, \
  0x11, 0x85, 0x2c, 0x80, 0x00, 0x00, \
  0x52, 0x20, 0x90, 0x60, 0x00, 0x00

#define kMaskRandom27_13 \
  0x51, 0x23, 0x12, 0xa0, 0x00, 0x00, \
  0x66, 0x41, 0xa3, 0x00, 0x00, 0x00, \
  0x05, 0x4a, 0x40, 0x20, 0x00, 0x00, \
  0x81, 0x20, 0x05, 0x60, 0x00, 0x00, \
  0x94, 0x01, 0x40, 0x40, 0x00, 0x00, \
  0x30, 0x84, 0x08, 0x40, 0x00, 0x00, \
  0x21, 0x11, 0x18, 0x20, 0x00, 0x00, \
  0x03, 0xc0, 0x34, 0x00, 0x00, 0x00, \
  0xe8, 0x04, 0x00, 0xa0, 0x00, 0x00, \
  0x0a, 0x11, 0x80, 0x80, 0x00, 0x00, \
  0x80, 0x1c, 0x61, 0x00, 0x00, 0x00, \
  0x04, 0x92, 0xa0, 0x00, 0x00, 0x00, \
  0x08, 0xac, 0x06, 0x00, 0x00, 0x00

#define kMaskRandom27_14 \
  0x59, 0x23, 0x12, 0xa0, 0x00, 0x00, \
  0x26, 0x55, 0xc9, 0x00, 0x00, 0x00, \
  0xb1, 0x40, 0xc5, 0xa0, 0x00, 0x00, \
  0x2b, 0x0a, 0xa4, 0xc0, 0x00, 0x00, \
  0x14, 0xc8, 0x33, 0x60, 0x00, 0x00, \
  0xc8, 0x8c, 0x2a, 0xa0, 0x00, 0x00, \
  0x84, 0xb5, 0x54, 0x40, 0x00, 0x00, \
  0xd1, 0x22, 0x52, 0xa0, 0x00, 0x00, \
  0x46, 0xd4, 0xaa, 0x40, 0x00, 0x00, \
  0x15, 0x48, 0xa5, 0xa0, 0x00, 0x00, \
  0x21, 0x72, 0x8d, 0x40, 0x00, 0x00, \
  0x28, 0xc9, 0x13, 0x60, 0x00, 0x00, \
  0xaa, 0x24, 0x44, 0x60, 0x00, 0x00, \
  0x0a, 0xe7, 0x3b, 0x20, 0x00, 0x00

#define kMaskRandom27_15 \
  0x59, 0x23, 0x12, 0xa0, 0x00, 0x00, \
  0x26, 0x55, 0xc9, 0x00, 0x00, 0x00, \
  0xb1, 0x40, 0xc5, 0xa0, 0x00, 0x00, \
  0x2b, 0x0a, 0xa4, 0xc0, 0x00, 0x00, \
  0x14, 0xc8, 0x33, 0x60, 0x00, 0x00, \
  0xc8, 0x8c, 0x2a, 0xa0, 0x00, 0x00, \
  0x84, 0xb5, 0x54, 0x40, 0x00, 0x00, \
  0x80, 0xae, 0x00, 0xa0, 0x00, 0x00, \
  0x30, 0x92, 0x0b, 0x00, 0x00, 0x00, \
  0x16, 0x0c, 0x41, 0x80, 0x00, 0x00, \
  0x03, 0x31, 0x05, 0x20, 0x00, 0x00, \
  0x44, 0x60, 0x52, 0x40, 0x00, 0x00, \
  0x08, 0x18, 0x24, 0xc0, 0x00, 0x00, \
  0xd8, 0x04, 0xa2, 0x00, 0x00, 0x00, \
  0xa1, 0x43, 0x90, 0x00, 0x00, 0x00

#define kMaskRandom27_16 \
  0x80, 0xae, 0x00, 0xa0, 0x00, 0x00, \
  0x30, 0x92, 0x0b, 0x00, 0x00, 0x00, \
  0x16, 0x0c, 0x41, 0x80, 0x00, 0x00, \
  0x03, 0x31, 0x05, 0x20, 0x00, 0x00, \
  0x44, 0x60, 0x52, 0x40, 0x00, 0x00, \
  0x08, 0x18, 0x24, 0xc0, 0x00, 0x00, \
  0xd8, 0x04, 0xa2, 0x00, 0x00, 0x00, \
  0xa1, 0x43, 0x90, 0x00, 0x00, 0x00, \
  0x59, 0x23, 0x12, 0xa0, 0x00, 0x00, \
  0x26, 0x55, 0xc9, 0x00, 0x00, 0x00, \
  0xb1, 0x40, 0xc5, 0xa0, 0x00, 0x00, \
  0x2b, 0x0a, 0xa4, 0xc0, 0x00, 0x00, \
  0x14, 0xc8, 0x33, 0x60, 0x00, 0x00, \
  0xc8, 0x8c, 0x2a, 0xa0, 0x00, 0x00, \
  0x84, 0xb5, 0x54, 0x40, 0x00, 0x00, \
  0x01, 0x50, 0xfb, 0xe0, 0x00, 0x00

#define kMaskRandom27_17 \
  0x80, 0xae, 0x00, 0xa0, 0x00, 0x00, \
  0x30, 0x92, 0x0b, 0x00, 0x00, 0x00, \
  0x16, 0x0c, 0x41, 0x80, 0x00, 0x00, \
  0x03, 0x31, 0x05, 0x20, 0x00, 0x00, \
  0x44, 0x60, 0x52, 0x40, 0x00, 0x00, \
  0x08, 0x18, 0x24, 0xc0, 0x00, 0x00, \
  0xd8, 0x04, 0xa2, 0x00, 0x00, 0x00, \
  0xa1, 0x43, 0x90, 0x00, 0x00, 0x00, \
  0x59, 0x25, 0x12, 0xa0, 0x00, 0x00, \
  0x66, 0x41, 0xa3, 0x00, 0x00, 0x00, \
  0x14, 0x42, 0x51, 0x20, 0x00, 0x00, \
  0x21, 0x49, 0x05, 0x40, 0x00, 0x00, \
  0x02, 0xc8, 0x8c, 0x20, 0x00, 0x00, \
  0x94, 0x12, 0x48, 0x40, 0x00, 0x00, \
  0x80, 0xac, 0x30, 0x60, 0x00, 0x00, \
  0x0a, 0x91, 0x06, 0xa0, 0x00, 0x00, \
  0x40, 0x1c, 0x42, 0x40, 0x00, 0x00

#define kMaskRandom27_18 \
  0x59, 0x25, 0x12, 0xa0, 0x00, 0x00, \
  0x66, 0x41, 0xa3, 0x00, 0x00, 0x00, \
  0x14, 0x42, 0x51, 0x20, 0x00, 0x00, \
  0x21, 0x49, 0x05, 0x40, 0x00, 0x00, \
  0x02, 0xc8, 0x8c, 0x20, 0x00, 0x00, \
  0x94, 0x12, 0x48, 0x40, 0x00, 0x00, \
  0x80, 0xac, 0x30, 0x60, 0x00, 0x00, \
  0x0a, 0x91, 0x06, 0xa0, 0x00, 0x00, \
  0x40, 0x1c, 0x42, 0x40, 0x00, 0x00, \
  0x80, 0xae, 0x00, 0xa0, 0x00, 0x00, \
  0x30, 0x92, 0x0b, 0x00, 0x00, 0x00, \
  0x16, 0x0c, 0x41, 0x80, 0x00, 0x00, \
  0x03, 0x31, 0x05, 0x20, 0x00, 0x00, \
  0x44, 0x60, 0x52, 0x40, 0x00, 0x00, \
  0x08, 0x18, 0x24, 0xc0, 0x00, 0x00, \
  0xd8, 0x04, 0xa2, 0x00, 0x00, 0x00, \
  0xa1, 0x43, 0x90, 0x00, 0x00, 0x00, \
  0x53, 0xc3, 0x33, 0x80, 0x00, 0x00

#define kMaskRandom27_19 \
  0x59, 0x25, 0x12, 0xa0, 0x00, 0x00, \
  0x66, 0x41, 0xa3, 0x00, 0x00, 0x00, \
  0x14, 0x42, 0x51, 0x20, 0x00, 0x00, \
  0x21, 0x49, 0x05, 0x40, 0x00, 0x00, \
  0x02, 0xc8, 0x8c, 0x20, 0x00, 0x00, \
  0x94, 0x12, 0x48, 0x40, 0x00, 0x00, \
  0x80, 0xac, 0x30, 0x60, 0x00, 0x00, \
  0x0a, 0x91, 0x06, 0xa0, 0x00, 0x00, \
  0x40, 0x1c, 0x42, 0x40, 0x00, 0x00, \
  0xd1, 0x06, 0x06, 0xa0, 0x00, 0x00, \
  0x44, 0x50, 0xea, 0x00, 0x00, 0x00, \
  0x10, 0x9e, 0xa0, 0x40, 0x00, 0x00, \
  0xa0, 0x50, 0x13, 0x00, 0x00, 0x00, \
  0x4a, 0x08, 0x21, 0x40, 0x00, 0x00, \
  0x40, 0x31, 0x04, 0xc0, 0x00, 0x00, \
  0x80, 0x2a, 0x02, 0x20, 0x00, 0x00, \
  0x0c, 0x90, 0x44, 0x20, 0x00, 0x00, \
  0x05, 0x8b, 0x40, 0x00, 0x00, 0x00, \
  0x62, 0x21, 0x18, 0x80, 0x00, 0x00

#define kMaskRandom27_2 \
  0xec, 0xc7, 0x67, 0x40, 0x00, 0x00, \
  0x1b, 0x39, 0xdc, 0xe0, 0x00, 0x00

#define kMaskRandom27_20 \
  0xd1, 0x06, 0x06, 0xa0, 0x00, 0x00, \
  0x44, 0x50, 0xea, 0x00, 0x00, 0x00, \
  0x10, 0x9e, 0xa0, 0x40, 0x00, 0x00, \
  0xa0, 0x50, 0x13, 0x00, 0x00, 0x00, \
  0x4a, 0x08, 0x21, 0x40, 0x00, 0x00, \
  0x40, 0x31, 0x04, 0xc0, 0x00, 0x00, \
  0x80, 0x2a, 0x02, 0x20, 0x00, 0x00, \
  0x0c, 0x90, 0x44, 0x20, 0x00, 0x00, \
  0x05, 0x8b, 0x40, 0x00, 0x00, 0x00, \
  0x62, 0x21, 0x18, 0x80, 0x00, 0x00, \
  0x59, 0x25, 0x12, 0xa0, 0x00, 0x00, \
  0x66, 0x41, 0xa3, 0x00, 0x00, 0x00, \
  0x14, 0x42, 0x51, 0x20, 0x00, 0x00, \
  0x21, 0x49, 0x05, 0x40, 0x00, 0x00, \
  0x02, 0xc8, 0x8c, 0x20, 0x00, 0x00, \
  0x94, 0x12, 0x48, 0x40, 0x00, 0x00, \
  0x80, 0xac, 0x30, 0x60, 0x00, 0x00, \
  0x0a, 0x91, 0x06, 0xa0, 0x00, 0x00, \
  0x40, 0x1c, 0x42, 0x40, 0x00, 0x00, \
  0xcb, 0xff, 0x6f, 0xc0, 0x00, 0x00

#define kMaskRandom27_21 \
  0xd1, 0x06, 0x06, 0xa0, 0x00, 0x00, \
  0x44, 0x50, 0xea, 0x00, 0x00, 0x00, \
  0x10, 0x9e, 0xa0, 0x40, 0x00, 0x00, \
  0xa0, 0x50, 0x13, 0x00, 0x00, 0x00, \
  0x4a, 0x08, 0x21, 0x40, 0x00, 0x00, \
  0x40, 0x31, 0x04, 0xc0, 0x00, 0x00, \
  0x80, 0x2a, 0x02, 0x20, 0x00, 0x00, \
  0x0c, 0x90, 0x44, 0x20, 0x00, 0x00, \
  0x05, 0x8b, 0x40, 0x00, 0x00, 0x00, \
  0x62, 0x21, 0x18, 0x80, 0x00, 0x00, \
  0x51, 0x23, 0x16, 0x80, 0x00, 0x00, \
  0x22, 0x11, 0xa9, 0x00, 0x00, 0x00, \
  0x13, 0x40, 0xa0, 0xa0, 0x00, 0x00, \
  0x25, 0x06, 0x28, 0x40, 0x00, 0x00, \
  0x18, 0x19, 0x10, 0x60, 0x00, 0x00, \
  0x0a, 0x24, 0x45, 0xc0, 0x00, 0x00, \
  0x88, 0x8a, 0x12, 0xa0, 0x00, 0x00, \
  0x06, 0x81, 0x45, 0x20, 0x00, 0x00, \
  0xe0, 0x24, 0xa1, 0x00, 0x00, 0x00, \
  0x84, 0x40, 0xd8, 0x20, 0x00, 0x00, \
  0x44, 0x19, 0x16, 0x00, 0x00, 0x00

#define kMaskRandom27_22 \
  0x51, 0x23, 0x16, 0x80, 0x00, 0x00, \
  0x22, 0x11, 0xa9, 0x00, 0x00, 0x00, \
  0x13, 0x40, 0xa0, 0xa0, 0x00, 0x00, \
  0x25, 0x06, 0x28, 0x40, 0x00, 0x00, \
  0x18, 0x19, 0x10, 0x60, 0x00, 0x00, \
  0x0a, 0x24, 0x45, 0xc0, 0x00, 0x00, \
  0x88, 0x8a, 0x12, 0xa0, 0x00, 0x00, \
  0x06, 0x81, 0x45, 0x20, 0x00, 0x00, \
  0xe0, 0x24, 0xa1, 0x00, 0x00, 0x00, \
  0x84, 0x40, 0xd8, 0x20, 0x00, 0x00, \
  0x44, 0x19, 0x16, 0x00, 0x00, 0x00, \
  0xd1, 0x06, 0x06, 0xa0, 0x00, 0x00, \
  0x44, 0x50, 0xea, 0x00, 0x00, 0x00, \
  0x10, 0x9e, 0xa0, 0x40, 0x00, 0x00, \
  0xa0, 0x50, 0x13, 0x00, 0x00, 0x00, \
  0x4a, 0x08, 0x21, 0x40, 0x00, 0x00, \
  0x40, 0x31, 0x04, 0xc0, 0x00, 0x00, \
  0x80, 0x2a, 0x02, 0x20, 0x00, 0x00, \
  0x0c, 0x90, 0x44, 0x20, 0x00, 0x00, \
  0x05, 0x8b, 0x40, 0x00, 0x00, 0x00, \
  0x62, 0x21, 0x18, 0x80, 0x00, 0x00, \
  0xf5, 0x2d, 0x52, 0x40, 0x00, 0x00

#define kMaskRandom27_23 \
  0x51, 0x23, 0x16, 0x80, 0x00, 0x00, \
  0x22, 0x11, 0xa9, 0x00, 0x00, 0x00, \
  0x13, 0x40, 0xa0, 0xa0, 0x00, 0x00, \
  0x25, 0x06, 0x28, 0x40, 0x00, 0x00, \
  0x18, 0x19, 0x10, 0x60, 0x00, 0x00, \
  0x0a, 0x24, 0x45, 0xc0, 0x00, 0x00, \
  0x88, 0x8a, 0x12, 0xa0, 0x00, 0x00, \
  0x06, 0x81, 0x45, 0x20, 0x00, 0x00, \
  0xe0, 0x24, 0xa1, 0x00, 0x00, 0x00, \
  0x84, 0x40, 0xd8, 0x20, 0x00, 0x00, \
  0x44, 0x19, 0x16, 0x00, 0x00, 0x00, \
  0x28, 0x2c, 0x08, 0x20, 0x00, 0x00, \
  0x84, 0x52, 0x03, 0x40, 0x00, 0x00, \
  0x60, 0x44, 0x81, 0x20, 0x00, 0x00, \
  0x05, 0x49, 0x41, 0x40, 0x00, 0x00, \
  0x02, 0x9a, 0x90, 0x80, 0x00, 0x00, \
  0x01, 0x32, 0x0c, 0x40, 0x00, 0x00, \
  0x48, 0x10, 0x49, 0x80, 0x00, 0x00, \
  0x24, 0x82, 0x42, 0x20, 0x00, 0x00, \
  0x94, 0x00, 0x22, 0x20, 0x00, 0x00, \
  0x8a, 0x00, 0x74, 0x00, 0x00, 0x00, \
  0x11, 0x85, 0x2c, 0x80, 0x00, 0x00, \
  0x52, 0x20, 0x90, 0x60, 0x00, 0x00

#define kMaskRandom27_24 \
  0x28, 0x2c, 0x08, 0x20, 0x00, 0x00, \
  0x84, 0x52, 0x03, 0x40, 0x00, 0x00, \
  0x60, 0x44, 0x81, 0x20, 0x00, 0x00, \
  0x05, 0x49, 0x41, 0x40, 0x00, 0x00, \
  0x02, 0x9a, 0x90, 0x80, 0x00, 0x00, \
  0x01, 0x32, 0x0c, 0x40, 0x00, 0x00, \
  0x48, 0x10, 0x49, 0x80, 0x00, 0x00, \
  0x24, 0x82, 0x42, 0x20, 0x00, 0x00, \
  0x94, 0x00, 0x22, 0x20, 0x00, 0x00, \
  0x8a, 0x00, 0x74, 0x00, 0x00, 0x00, \
  0x11, 0x85, 0x2c, 0x80, 0x00, 0x00, \
  0x52, 0x20, 0x90, 0x60, 0x00, 0x00, \
  0x51, 0x23, 0x16, 0x80, 0x00, 0x00, \
  0x22, 0x11, 0xa9, 0x00, 0x00, 0x00, \
  0x13, 0x40, 0xa0, 0xa0, 0x00, 0x00, \
  0x25, 0x06, 0x28, 0x40, 0x00, 0x00, \
  0x18, 0x19, 0x10, 0x60, 0x00, 0x00, \
  0x0a, 0x24, 0x45, 0xc0, 0x00, 0x00, \
  0x88, 0x8a, 0x12, 0xa0, 0x00, 0x00, \
  0x06, 0x81, 0x45, 0x20, 0x00, 0x00, \
  0xe0, 0x24, 0xa1, 0x00, 0x00, 0x00, \
  0x84, 0x40, 0xd8, 0x20, 0x00, 0x00, \
  0x44, 0x19, 0x16, 0x00, 0x00, 0x00, \
  0xa2, 0x85, 0xdb, 0xa0, 0x00, 0x00

#define kMaskRandom27_25 \
  0x28, 0x2c, 0x08, 0x20, 0x00, 0x00, \
  0x84, 0x52, 0x03, 0x40, 0x00, 0x00, \
  0x60, 0x44, 0x81, 0x20, 0x00, 0x00, \
  0x05, 0x49, 0x41, 0x40, 0x00, 0x00, \
  0x02, 0x9a, 0x90, 0x80, 0x00, 0x00, \
  0x01, 0x32, 0x0c, 0x40, 0x00, 0x00, \
  0x48, 0x10, 0x49, 0x80, 0x00, 0x00, \
  0x24, 0x82, 0x42, 0x20, 0x00, 0x00, \
  0x94, 0x00, 0x22, 0x20, 0x00, 0x00, \
  0x8a, 0x00, 0x74, 0x00, 0x00, 0x00, \
  0x11, 0x85, 0x2c, 0x80, 0x00, 0x00, \
  0x52, 0x20, 0x90, 0x60, 0x00, 0x00, \
  0x51, 0x23, 0x12, 0xa0, 0x00, 0x00, \
  0x66, 0x41, 0xa3, 0x00, 0x00, 0x00, \
  0x05, 0x4a, 0x40, 0x20, 0x00, 0x00, \
  0x81, 0x20, 0x05, 0x60, 0x00, 0x00, \
  0x94, 0x01, 0x40, 0x40, 0x00, 0x00, \
  0x30, 0x84, 0x08, 0x40, 0x00, 0x00, \
  0x21, 0x11, 0x18, 0x20, 0x00, 0x00, \
  0x03, 0xc0, 0x34, 0x00, 0x00, 0x00, \
  0xe8, 0x04, 0x00, 0xa0, 0x00, 0x00, \
  0x0a, 0x11, 0x80, 0x80, 0x00, 0x00, \
  0x80, 0x1c, 0x61, 0x00, 0x00, 0x00, \
  0x04, 0x92, 0xa0, 0x00, 0x00, 0x00, \
  0x08, 0xac, 0x06, 0x00, 0x00, 0x00

#define kMaskRandom27_26 \
  0x51, 0x23, 0x12, 0xa0, 0x00, 0x00, \
  0x66, 0x41, 0xa3, 0x00, 0x00, 0x00, \
  0x05, 0x4a, 0x40, 0x20, 0x00, 0x00, \
  0x81, 0x20, 0x05, 0x60, 0x00, 0x00, \
  0x94, 0x01, 0x40, 0x40, 0x00, 0x00, \
  0x30, 0x84, 0x08, 0x40, 0x00, 0x00, \
  0x21, 0x11, 0x18, 0x20, 0x00, 0x00, \
  0x03, 0xc0, 0x34, 0x00, 0x00, 0x00, \
  0xe8, 0x04, 0x00, 0xa0, 0x00, 0x00, \
  0x0a, 0x11, 0x80, 0x80, 0x00, 0x00, \
  0x80, 0x1c, 0x61, 0x00, 0x00, 0x00, \
  0x04, 0x92, 0xa0, 0x00, 0x00, 0x00, \
  0x08, 0xac, 0x06, 0x00, 0x00, 0x00, \
  0x28, 0x2c, 0x08, 0x20, 0x00, 0x00, \
  0x84, 0x52, 0x03, 0x40, 0x00, 0x00, \
  0x60, 0x44, 0x81, 0x20, 0x00, 0x00, \
  0x05, 0x49, 0x41, 0x40, 0x00, 0x00, \
  0x02, 0x9a, 0x90, 0x80, 0x00, 0x00, \
  0x01, 0x32, 0x0c, 0x40, 0x00, 0x00, \
  0x48, 0x10, 0x49, 0x80, 0x00, 0x00, \
  0x24, 0x82, 0x42, 0x20, 0x00, 0x00, \
  0x94, 0x00, 0x22, 0x20, 0x00, 0x00, \
  0x8a, 0x00, 0x74, 0x00, 0x00, 0x00, \
  0x11, 0x85, 0x2c, 0x80, 0x00, 0x00, \
  0x52, 0x20, 0x90, 0x60, 0x00, 0x00, \
  0xcd, 0x41, 0xa2, 0x40, 0x00, 0x00

#define kMaskRandom27_27 \
  0x51, 0x23, 0x12, 0xa0, 0x00, 0x00, \
  0x66, 0x41, 0xa3, 0x00, 0x00, 0x00, \
  0x05, 0x4a, 0x40, 0x20, 0x00, 0x00, \
  0x81, 0x20, 0x05, 0x60, 0x00, 0x00, \
  0x94, 0x01, 0x40, 0x40, 0x00, 0x00, \
  0x30, 0x84, 0x08, 0x40, 0x00, 0x00, \
  0x21, 0x11, 0x18, 0x20, 0x00, 0x00, \
  0x03, 0xc0, 0x34, 0x00, 0x00, 0x00, \
  0xe8, 0x04, 0x00, 0xa0, 0x00, 0x00, \
  0x0a, 0x11, 0x80, 0x80, 0x00, 0x00, \
  0x80, 0x1c, 0x61, 0x00, 0x00, 0x00, \
  0x04, 0x92, 0xa0, 0x00, 0x00, 0x00, \
  0x08, 0xac, 0x06, 0x00, 0x00, 0x00, \
  0x51, 0x22, 0x02, 0xa0, 0x00, 0x00, \
  0x66, 0x40, 0xaa, 0x00, 0x00, 0x00, \
  0x05, 0x4e, 0x00, 0x20, 0x00, 0x00, \
  0x81, 0x21, 0x40, 0x80, 0x00, 0x00, \
  0x94, 0x00, 0x28, 0x60, 0x00, 0x00, \
  0x30, 0x83, 0x24, 0x00, 0x00, 0x00, \
  0x21, 0x14, 0x0c, 0x00, 0x00, 0x00, \
  0x03, 0xc0, 0x84, 0xc0, 0x00, 0x00, \
  0xe8, 0x04, 0x21, 0x00, 0x00, 0x00, \
  0x0a, 0x10, 0x91, 0x80, 0x00, 0x00, \
  0x80, 0x1b, 0x10, 0x00, 0x00, 0x00, \
  0x04, 0x91, 0x43, 0x00, 0x00, 0x00, \
  0x08, 0xa8, 0x70, 0x40, 0x00, 0x00, \
  0x9c, 0xc0, 0x84, 0x20, 0x00, 0x00

#define kMaskRandom27_3 \
  0x99, 0xb5, 0x66, 0xc0, 0x00, 0x00, \
  0x46, 0xda, 0xab, 0x60, 0x00, 0x00, \
  0x37, 0x29, 0x3d, 0xa0, 0x00, 0x00

#define kMaskRandom27_4 \
  0x49, 0xb1, 0x66, 0xc0, 0x00, 0x00, \
  0x26, 0xd4, 0x9b, 0x40, 0x00, 0x00, \
  0x85, 0x68, 0xd5, 0xa0, 0x00, 0x00, \
  0x52, 0x5a, 0x39, 0x60, 0x00, 0x00

#define kMaskRandom27_5 \
  0x51, 0x33, 0x26, 0xc0, 0x00, 0x00, \
  0x66, 0x45, 0x2b, 0x40, 0x00, 0x00, \
  0x0c, 0x6a, 0x95, 0xa0, 0x00, 0x00, \
  0xa1, 0xc0, 0xed, 0x40, 0x00, 0x00, \
  0x22, 0x9c, 0xe2, 0xa0, 0x00, 0x00

#define kMaskRandom27_6 \
  0xd1, 0x22, 0x52, 0xa0, 0x00, 0x00, \
  0x46, 0xd4, 0xaa, 0x40, 0x00, 0x00, \
  0x15, 0x48, 0xa5, 0xa0, 0x00, 0x00, \
  0x21, 0x72, 0x8d, 0x40, 0x00, 0x00, \
  0x28, 0xc9, 0x13, 0x60, 0x00, 0x00, \
  0xaa, 0x24, 0x44, 0x60, 0x00, 0x00

#define kMaskRandom27_7 \
  0x59, 0x23, 0x12, 0xa0, 0x00, 0x00, \
  0x26, 0x55, 0xc9, 0x00, 0x00, 0x00, \
  0xb1, 0x40, 0xc5, 0xa0, 0x00, 0x00, \
  0x2b, 0x0a, 0xa4, 0xc0, 0x00, 0x00, \
  0x14, 0xc8, 0x33, 0x60, 0x00, 0x00, \
  0xc8, 0x8c, 0x2a, 0xa0, 0x00, 0x00, \
  0x84, 0xb5, 0x54, 0x40, 0x00, 0x00

#define kMaskRandom27_8 \
  0x80, 0xae, 0x00, 0xa0, 0x00, 0x00, \
  0x30, 0x92, 0x0b, 0x00, 0x00, 0x00, \
  0x16, 0x0c, 0x41, 0x80, 0x00, 0x00, \
  0x03, 0x31, 0x05, 0x20, 0x00, 0x00, \
  0x44, 0x60, 0x52, 0x40, 0x00, 0x00, \
  0x08, 0x18, 0x24, 0xc0, 0x00, 0x00, \
  0xd8, 0x04, 0xa2, 0x00, 0x00, 0x00, \
  0xa1, 0x43, 0x90, 0x00, 0x00, 0x00

#define kMaskRandom27_9 \
  0x59, 0x25, 0x12, 0xa0, 0x00, 0x00, \
  0x66, 0x41, 0xa3, 0x00, 0x00, 0x00, \
  0x14, 0x42, 0x51, 0x20, 0x00, 0x00, \
  0x21, 0x49, 0x05, 0x40, 0x00, 0x00, \
  0x02, 0xc8, 0x8c, 0x20, 0x00, 0x00, \
  0x94, 0x12, 0x48, 0x40, 0x00, 0x00, \
  0x80, 0xac, 0x30, 0x60, 0x00, 0x00, \
  0x0a, 0x91, 0x06, 0xa0, 0x00, 0x00, \
  0x40, 0x1c, 0x42, 0x40, 0x00, 0x00

#define kMaskRandom28_1 \
  0xff, 0xff, 0xff, 0xf0, 0x00, 0x00

#define kMaskRandom28_10 \
  0xc0, 0xd7, 0x03, 0x50, 0x00, 0x00, \
  0x1d, 0x40, 0x75, 0x00, 0x00, 0x00, \
  0xd4, 0x0b, 0x50, 0x20, 0x00, 0x00, \
  0x02, 0x60, 0x09, 0x80, 0x00, 0x00, \
  0x04, 0x28, 0x10, 0xa0, 0x00, 0x00, \
  0x20, 0x98, 0x82, 0x60, 0x00, 0x00, \
  0x40, 0x45, 0x01, 0x10, 0x00, 0x00, \
  0x08, 0x84, 0x22, 0x10, 0x00, 0x00, \
  0x68, 0x01, 0xa0, 0x00, 0x00, 0x00, \
  0x23, 0x10, 0x8c, 0x40, 0x00, 0x00

#define kMaskRandom28_11 \
  0x62, 0xd1, 0x8b, 0x40, 0x00, 0x00, \
  0x35, 0x20, 0xd4, 0x80, 0x00, 0x00, \
  0x14, 0x14, 0x50, 0x50, 0x00, 0x00, \
  0xc5, 0x0b, 0x14, 0x20, 0x00, 0x00, \
  0x22, 0x0c, 0x88, 0x30, 0x00, 0x00, \
  0x88, 0xba, 0x22, 0xe0, 0x00, 0x00, \
  0x42, 0x55, 0x09, 0x50, 0x00, 0x00, \
  0x28, 0xa4, 0xa2, 0x90, 0x00, 0x00, \
  0x94, 0x22, 0x50, 0x80, 0x00, 0x00, \
  0x1b, 0x04, 0x6c, 0x10, 0x00, 0x00, \
  0x22, 0xc0, 0x8b, 0x00, 0x00, 0x00

#define kMaskRandom28_12 \
  0x81, 0x06, 0x04, 0x10, 0x00, 0x00, \
  0x40, 0x69, 0x01, 0xa0, 0x00, 0x00, \
  0x90, 0x26, 0x40, 0x90, 0x00, 0x00, \
  0x28, 0x28, 0xa0, 0xa0, 0x00, 0x00, \
  0x52, 0x11, 0x48, 0x40, 0x00, 0x00, \
  0x41, 0x89, 0x06, 0x20, 0x00, 0x00, \
  0x09, 0x30, 0x24, 0xc0, 0x00, 0x00, \
  0x48, 0x45, 0x21, 0x10, 0x00, 0x00, \
  0x04, 0x44, 0x11, 0x10, 0x00, 0x00, \
  0x0e, 0x80, 0x3a, 0x00, 0x00, 0x00, \
  0xa5, 0x92, 0x96, 0x40, 0x00, 0x00, \
  0x12, 0x0c, 0x48, 0x30, 0x00, 0x00

#define kMaskRandom28_13 \
  0x62, 0x55, 0x89, 0x50, 0x00, 0x00, \
  0x34, 0x60, 0xd1, 0x80, 0x00, 0x00, \
  0x48, 0x05, 0x20, 0x10, 0x00, 0x00, \
  0x00, 0xac, 0x02, 0xb0, 0x00, 0x00, \
  0x28, 0x08, 0xa0, 0x20, 0x00, 0x00, \
  0x81, 0x0a, 0x04, 0x20, 0x00, 0x00, \
  0x23, 0x04, 0x8c, 0x10, 0x00, 0x00, \
  0x06, 0x80, 0x1a, 0x00, 0x00, 0x00, \
  0x80, 0x16, 0x00, 0x50, 0x00, 0x00, \
  0x30, 0x10, 0xc0, 0x40, 0x00, 0x00, \
  0x8c, 0x22, 0x30, 0x80, 0x00, 0x00, \
  0x54, 0x01, 0x50, 0x00, 0x00, 0x00, \
  0x80, 0xc2, 0x03, 0x00, 0x00, 0x00

#define kMaskRandom28_14 \
  0x40, 0x55, 0x01, 0x50, 0x00, 0x00, \
  0x15, 0x40, 0x55, 0x00, 0x00, 0x00, \
  0xc0, 0x07, 0x00, 0x10, 0x00, 0x00, \
  0x28, 0x10, 0xa0, 0x40, 0x00, 0x00, \
  0x05, 0x0c, 0x14, 0x30, 0x00, 0x00, \
  0x64, 0x81, 0x92, 0x00, 0x00, 0x00, \
  0x81, 0x82, 0x06, 0x00, 0x00, 0x00, \
  0x10, 0x98, 0x42, 0x60, 0x00, 0x00, \
  0x84, 0x22, 0x10, 0x80, 0x00, 0x00, \
  0x12, 0x30, 0x48, 0xc0, 0x00, 0x00, \
  0x62, 0x01, 0x88, 0x00, 0x00, 0x00, \
  0x28, 0x60, 0xa1, 0x80, 0x00, 0x00, \
  0x0e, 0x08, 0x38, 0x20, 0x00, 0x00, \
  0x10, 0x84, 0x42, 0x10, 0x00, 0x00

#define kMaskRandom28_15 \
  0x62, 0x55, 0x89, 0x50, 0x00, 0x00, \
  0xb9, 0x22, 0xe4, 0x80, 0x00, 0x00, \
  0x18, 0xb4, 0x62, 0xd0, 0x00, 0x00, \
  0x54, 0x99, 0x52, 0x60, 0x00, 0x00, \
  0x06, 0x6c, 0x19, 0xb0, 0x00, 0x00, \
  0x85, 0x56, 0x15, 0x50, 0x00, 0x00, \
  0xaa, 0x8a, 0xaa, 0x20, 0x00, 0x00, \
  0xc0, 0x17, 0x00, 0x50, 0x00, 0x00, \
  0x41, 0x61, 0x05, 0x80, 0x00, 0x00, \
  0x88, 0x32, 0x20, 0xc0, 0x00, 0x00, \
  0x20, 0xa4, 0x82, 0x90, 0x00, 0x00, \
  0x0a, 0x48, 0x29, 0x20, 0x00, 0x00, \
  0x04, 0x98, 0x12, 0x60, 0x00, 0x00, \
  0x94, 0x42, 0x51, 0x00, 0x00, 0x00, \
  0x72, 0x01, 0xc8, 0x00, 0x00, 0x00

#define kMaskRandom28_16 \
  0xc0, 0x17, 0x00, 0x50, 0x00, 0x00, \
  0x41, 0x61, 0x05, 0x80, 0x00, 0x00, \
  0x88, 0x32, 0x20, 0xc0, 0x00, 0x00, \
  0x20, 0xa4, 0x82, 0x90, 0x00, 0x00, \
  0x0a, 0x48, 0x29, 0x20, 0x00, 0x00, \
  0x04, 0x98, 0x12, 0x60, 0x00, 0x00, \
  0x94, 0x42, 0x51, 0x00, 0x00, 0x00, \
  0x72, 0x01, 0xc8, 0x00, 0x00, 0x00, \
  0x62, 0x55, 0x89, 0x50, 0x00, 0x00, \
  0xb9, 0x22, 0xe4, 0x80, 0x00, 0x00, \
  0x18, 0xb4, 0x62, 0xd0, 0x00, 0x00, \
  0x54, 0x99, 0x52, 0x60, 0x00, 0x00, \
  0x06, 0x6c, 0x19, 0xb0, 0x00, 0x00, \
  0x85, 0x56, 0x15, 0x50, 0x00, 0x00, \
  0xaa, 0x8a, 0xaa, 0x20, 0x00, 0x00, \
  0xed, 0x76, 0x36, 0x50, 0x00, 0x00

#define kMaskRandom28_17 \
  0xc0, 0x17, 0x00, 0x50, 0x00, 0x00, \
  0x41, 0x61, 0x05, 0x80, 0x00, 0x00, \
  0x88, 0x32, 0x20, 0xc0, 0x00, 0x00, \
  0x20, 0xa4, 0x82, 0x90, 0x00, 0x00, \
  0x0a, 0x48, 0x29, 0x20, 0x00, 0x00, \
  0x04, 0x98, 0x12, 0x60, 0x00, 0x00, \
  0x94, 0x42, 0x51, 0x00, 0x00, 0x00, \
  0x72, 0x01, 0xc8, 0x00, 0x00, 0x00, \
  0xa2, 0x56, 0x89, 0x50, 0x00, 0x00, \
  0x34, 0x60, 0xd1, 0x80, 0x00, 0x00, \
  0x4a, 0x25, 0x28, 0x90, 0x00, 0x00, \
  0x20, 0xa8, 0x82, 0xa0, 0x00, 0x00, \
  0x11, 0x84, 0x46, 0x10, 0x00, 0x00, \
  0x49, 0x09, 0x24, 0x20, 0x00, 0x00, \
  0x86, 0x0e, 0x18, 0x30, 0x00, 0x00, \
  0x20, 0xd4, 0x83, 0x50, 0x00, 0x00, \
  0x88, 0x4a, 0x21, 0x20, 0x00, 0x00

#define kMaskRandom28_18 \
  0xa2, 0x56, 0x89, 0x50, 0x00, 0x00, \
  0x34, 0x60, 0xd1, 0x80, 0x00, 0x00, \
  0x4a, 0x25, 0x28, 0x90, 0x00, 0x00, \
  0x20, 0xa8, 0x82, 0xa0, 0x00, 0x00, \
  0x11, 0x84, 0x46, 0x10, 0x00, 0x00, \
  0x49, 0x09, 0x24, 0x20, 0x00, 0x00, \
  0x86, 0x0e, 0x18, 0x30, 0x00, 0x00, \
  0x20, 0xd4, 0x83, 0x50, 0x00, 0x00, \
  0x88, 0x4a, 0x21, 0x20, 0x00, 0x00, \
  0xc0, 0x17, 0x00, 0x50, 0x00, 0x00, \
  0x41, 0x61, 0x05, 0x80, 0x00, 0x00, \
  0x88, 0x32, 0x20, 0xc0, 0x00, 0x00, \
  0x20, 0xa4, 0x82, 0x90, 0x00, 0x00, \
  0x0a, 0x48, 0x29, 0x20, 0x00, 0x00, \
  0x04, 0x98, 0x12, 0x60, 0x00, 0x00, \
  0x94, 0x42, 0x51, 0x00, 0x00, 0x00, \
  0x72, 0x01, 0xc8, 0x00, 0x00, 0x00, \
  0x6e, 0x9f, 0x98, 0x10, 0x00, 0x00

#define kMaskRandom28_19 \
  0xa2, 0x56, 0x89, 0x50, 0x00, 0x00, \
  0x34, 0x60, 0xd1, 0x80, 0x00, 0x00, \
  0x4a, 0x25, 0x28, 0x90, 0x00, 0x00, \
  0x20, 0xa8, 0x82, 0xa0, 0x00, 0x00, \
  0x11, 0x84, 0x46, 0x10, 0x00, 0x00, \
  0x49, 0x09, 0x24, 0x20, 0x00, 0x00, \
  0x86, 0x0e, 0x18, 0x30, 0x00, 0x00, \
  0x20, 0xd4, 0x83, 0x50, 0x00, 0x00, \
  0x88, 0x4a, 0x21, 0x20, 0x00, 0x00, \
  0xc0, 0xd7, 0x03, 0x50, 0x00, 0x00, \
  0x1d, 0x40, 0x75, 0x00, 0x00, 0x00, \
  0xd4, 0x0b, 0x50, 0x20, 0x00, 0x00, \
  0x02, 0x60, 0x09, 0x80, 0x00, 0x00, \
  0x04, 0x28, 0x10, 0xa0, 0x00, 0x00, \
  0x20, 0x98, 0x82, 0x60, 0x00, 0x00, \
  0x40, 0x45, 0x01, 0x10, 0x00, 0x00, \
  0x08, 0x84, 0x22, 0x10, 0x00, 0x00, \
  0x68, 0x01, 0xa0, 0x00, 0x00, 0x00, \
  0x23, 0x10, 0x8c, 0x40, 0x00, 0x00

#define kMaskRandom28_2 \
  0xec, 0xeb, 0xb3, 0xa0, 0x00, 0x00, \
  0x3b, 0x9c, 0xee, 0x70, 0x00, 0x00

#define kMaskRandom28_20 \
  0xc0, 0xd7, 0x03, 0x50, 0x00, 0x00, \
  0x1d, 0x40, 0x75, 0x00, 0x00, 0x00, \
  0xd4, 0x0b, 0x50, 0x20, 0x00, 0x00, \
  0x02, 0x60, 0x09, 0x80, 0x00, 0x00, \
  0x04, 0x28, 0x10, 0xa0, 0x00, 0x00, \
  0x20, 0x98, 0x82, 0x60, 0x00, 0x00, \
  0x40, 0x45, 0x01, 0x10, 0x00, 0x00, \
  0x08, 0x84, 0x22, 0x10, 0x00, 0x00, \
  0x68, 0x01, 0xa0, 0x00, 0x00, 0x00, \
  0x23, 0x10, 0x8c, 0x40, 0x00, 0x00, \
  0xa2, 0x56, 0x89, 0x50, 0x00, 0x00, \
  0x34, 0x60, 0xd1, 0x80, 0x00, 0x00, \
  0x4a, 0x25, 0x28, 0x90, 0x00, 0x00, \
  0x20, 0xa8, 0x82, 0xa0, 0x00, 0x00, \
  0x11, 0x84, 0x46, 0x10, 0x00, 0x00, \
  0x49, 0x09, 0x24, 0x20, 0x00, 0x00, \
  0x86, 0x0e, 0x18, 0x30, 0x00, 0x00, \
  0x20, 0xd4, 0x83, 0x50, 0x00, 0x00, \
  0x88, 0x4a, 0x21, 0x20, 0x00, 0x00, \
  0xea, 0x1b, 0x3a, 0x10, 0x00, 0x00

#define kMaskRandom28_21 \
  0xc0, 0xd7, 0x03, 0x50, 0x00, 0x00, \
  0x1d, 0x40, 0x75, 0x00, 0x00, 0x00, \
  0xd4, 0x0b, 0x50, 0x20, 0x00, 0x00, \
  0x02, 0x60, 0x09, 0x80, 0x00, 0x00, \
  0x04, 0x28, 0x10, 0xa0, 0x00, 0x00, \
  0x20, 0x98, 0x82, 0x60, 0x00, 0x00, \
  0x40, 0x45, 0x01, 0x10, 0x00, 0x00, \
  0x08, 0x84, 0x22, 0x10, 0x00, 0x00, \
  0x68, 0x01, 0xa0, 0x00, 0x00, 0x00, \
  0x23, 0x10, 0x8c, 0x40, 0x00, 0x00, \
  0x62, 0xd1, 0x8b, 0x40, 0x00, 0x00, \
  0x35, 0x20, 0xd4, 0x80, 0x00, 0x00, \
  0x14, 0x14, 0x50, 0x50, 0x00, 0x00, \
  0xc5, 0x0b, 0x14, 0x20, 0x00, 0x00, \
  0x22, 0x0c, 0x88, 0x30, 0x00, 0x00, \
  0x88, 0xba, 0x22, 0xe0, 0x00, 0x00, \
  0x42, 0x55, 0x09, 0x50, 0x00, 0x00, \
  0x28, 0xa4, 0xa2, 0x90, 0x00, 0x00, \
  0x94, 0x22, 0x50, 0x80, 0x00, 0x00, \
  0x1b, 0x04, 0x6c, 0x10, 0x00, 0x00, \
  0x22, 0xc0, 0x8b, 0x00, 0x00, 0x00

#define kMaskRandom28_22 \
  0x62, 0xd1, 0x8b, 0x40, 0x00, 0x00, \
  0x35, 0x20, 0xd4, 0x80, 0x00, 0x00, \
  0x14, 0x14, 0x50, 0x50, 0x00, 0x00, \
  0xc5, 0x0b, 0x14, 0x20, 0x00, 0x00, \
  0x22, 0x0c, 0x88, 0x30, 0x00, 0x00, \
  0x88, 0xba, 0x22, 0xe0, 0x00, 0x00, \
  0x42, 0x55, 0x09, 0x50, 0x00, 0x00, \
  0x28, 0xa4, 0xa2, 0x90, 0x00, 0x00, \
  0x94, 0x22, 0x50, 0x80, 0x00, 0x00, \
  0x1b, 0x04, 0x6c, 0x10, 0x00, 0x00, \
  0x22, 0xc0, 0x8b, 0x00, 0x00, 0x00, \
  0xc0, 0xd7, 0x03, 0x50, 0x00, 0x00, \
  0x1d, 0x40, 0x75, 0x00, 0x00, 0x00, \
  0xd4, 0x0b, 0x50, 0x20, 0x00, 0x00, \
  0x02, 0x60, 0x09, 0x80, 0x00, 0x00, \
  0x04, 0x28, 0x10, 0xa0, 0x00, 0x00, \
  0x20, 0x98, 0x82, 0x60, 0x00, 0x00, \
  0x40, 0x45, 0x01, 0x10, 0x00, 0x00, \
  0x08, 0x84, 0x22, 0x10, 0x00, 0x00, \
  0x68, 0x01, 0xa0, 0x00, 0x00, 0x00, \
  0x23, 0x10, 0x8c, 0x40, 0x00, 0x00, \
  0x45, 0x05, 0x10, 0x00, 0x00, 0x00

#define kMaskRandom28_23 \
  0x62, 0xd1, 0x8b, 0x40, 0x00, 0x00, \
  0x35, 0x20, 0xd4, 0x80, 0x00, 0x00, \
  0x14, 0x14, 0x50, 0x50, 0x00, 0x00, \
  0xc5, 0x0b, 0x14, 0x20, 0x00, 0x00, \
  0x22, 0x0c, 0x88, 0x30, 0x00, 0x00, \
  0x88, 0xba, 0x22, 0xe0, 0x00, 0x00, \
  0x42, 0x55, 0x09, 0x50, 0x00, 0x00, \
  0x28, 0xa4, 0xa2, 0x90, 0x00, 0x00, \
  0x94, 0x22, 0x50, 0x80, 0x00, 0x00, \
  0x1b, 0x04, 0x6c, 0x10, 0x00, 0x00, \
  0x22, 0xc0, 0x8b, 0x00, 0x00, 0x00, \
  0x81, 0x06, 0x04, 0x10, 0x00, 0x00, \
  0x40, 0x69, 0x01, 0xa0, 0x00, 0x00, \
  0x90, 0x26, 0x40, 0x90, 0x00, 0x00, \
  0x28, 0x28, 0xa0, 0xa0, 0x00, 0x00, \
  0x52, 0x11, 0x48, 0x40, 0x00, 0x00, \
  0x41, 0x89, 0x06, 0x20, 0x00, 0x00, \
  0x09, 0x30, 0x24, 0xc0, 0x00, 0x00, \
  0x48, 0x45, 0x21, 0x10, 0x00, 0x00, \
  0x04, 0x44, 0x11, 0x10, 0x00, 0x00, \
  0x0e, 0x80, 0x3a, 0x00, 0x00, 0x00, \
  0xa5, 0x92, 0x96, 0x40, 0x00, 0x00, \
  0x12, 0x0c, 0x48, 0x30, 0x00, 0x00

#define kMaskRandom28_24 \
  0x81, 0x06, 0x04, 0x10, 0x00, 0x00, \
  0x40, 0x69, 0x01, 0xa0, 0x00, 0x00, \
  0x90, 0x26, 0x40, 0x90, 0x00, 0x00, \
  0x28, 0x28, 0xa0, 0xa0, 0x00, 0x00, \
  0x52, 0x11, 0x48, 0x40, 0x00, 0x00, \
  0x41, 0x89, 0x06, 0x20, 0x00, 0x00, \
  0x09, 0x30, 0x24, 0xc0, 0x00, 0x00, \
  0x48, 0x45, 0x21, 0x10, 0x00, 0x00, \
  0x04, 0x44, 0x11, 0x10, 0x00, 0x00, \
  0x0e, 0x80, 0x3a, 0x00, 0x00, 0x00, \
  0xa5, 0x92, 0x96, 0x40, 0x00, 0x00, \
  0x12, 0x0c, 0x48, 0x30, 0x00, 0x00, \
  0x62, 0xd1, 0x8b, 0x40, 0x00, 0x00, \
  0x35, 0x20, 0xd4, 0x80, 0x00, 0x00, \
  0x14, 0x14, 0x50, 0x50, 0x00, 0x00, \
  0xc5, 0x0b, 0x14, 0x20, 0x00, 0x00, \
  0x22, 0x0c, 0x88, 0x30, 0x00, 0x00, \
  0x88, 0xba, 0x22, 0xe0, 0x00, 0x00, \
  0x42, 0x55, 0x09, 0x50, 0x00, 0x00, \
  0x28, 0xa4, 0xa2, 0x90, 0x00, 0x00, \
  0x94, 0x22, 0x50, 0x80, 0x00, 0x00, \
  0x1b, 0x04, 0x6c, 0x10, 0x00, 0x00, \
  0x22, 0xc0, 0x8b, 0x00, 0x00, 0x00, \
  0x6f, 0xd8, 0xee, 0xa0, 0x00, 0x00

#define kMaskRandom28_25 \
  0x81, 0x06, 0x04, 0x10, 0x00, 0x00, \
  0x40, 0x69, 0x01, 0xa0, 0x00, 0x00, \
  0x90, 0x26, 0x40, 0x90, 0x00, 0x00, \
  0x28, 0x28, 0xa0, 0xa0, 0x00, 0x00, \
  0x52, 0x11, 0x48, 0x40, 0x00, 0x00, \
  0x41, 0x89, 0x06, 0x20, 0x00, 0x00, \
  0x09, 0x30, 0x24, 0xc0, 0x00, 0x00, \
  0x48, 0x45, 0x21, 0x10, 0x00, 0x00, \
  0x04, 0x44, 0x11, 0x10, 0x00, 0x00, \
  0x0e, 0x80, 0x3a, 0x00, 0x00, 0x00, \
  0xa5, 0x92, 0x96, 0x40, 0x00, 0x00, \
  0x12, 0x0c, 0x48, 0x30, 0x00, 0x00, \
  0x62, 0x55, 0x89, 0x50, 0x00, 0x00, \
  0x34, 0x60, 0xd1, 0x80, 0x00, 0x00, \
  0x48, 0x05, 0x20, 0x10, 0x00, 0x00, \
  0x00, 0xac, 0x02, 0xb0, 0x00, 0x00, \
  0x28, 0x08, 0xa0, 0x20, 0x00, 0x00, \
  0x81, 0x0a, 0x04, 0x20, 0x00, 0x00, \
  0x23, 0x04, 0x8c, 0x10, 0x00, 0x00, \
  0x06, 0x80, 0x1a, 0x00, 0x00, 0x00, \
  0x80, 0x16, 0x00, 0x50, 0x00, 0x00, \
  0x30, 0x10, 0xc0, 0x40, 0x00, 0x00, \
  0x8c, 0x22, 0x30, 0x80, 0x00, 0x00, \
  0x54, 0x01, 0x50, 0x00, 0x00, 0x00, \
  0x80, 0xc2, 0x03, 0x00, 0x00, 0x00

#define kMaskRandom28_26 \
  0x62, 0x55, 0x89, 0x50, 0x00, 0x00, \
  0x34, 0x60, 0xd1, 0x80, 0x00, 0x00, \
  0x48, 0x05, 0x20, 0x10, 0x00, 0x00, \
  0x00, 0xac, 0x02, 0xb0, 0x00, 0x00, \
  0x28, 0x08, 0xa0, 0x20, 0x00, 0x00, \
  0x81, 0x0a, 0x04, 0x20, 0x00, 0x00, \
  0x23, 0x04, 0x8c, 0x10, 0x00, 0x00, \
  0x06, 0x80, 0x1a, 0x00, 0x00, 0x00, \
  0x80, 0x16, 0x00, 0x50, 0x00, 0x00, \
  0x30, 0x10, 0xc0, 0x40, 0x00, 0x00, \
  0x8c, 0x22, 0x30, 0x80, 0x00, 0x00, \
  0x54, 0x01, 0x50, 0x00, 0x00, 0x00, \
  0x80, 0xc2, 0x03, 0x00, 0x00, 0x00, \
  0x81, 0x06, 0x04, 0x10, 0x00, 0x00, \
  0x40, 0x69, 0x01, 0xa0, 0x00, 0x00, \
  0x90, 0x26, 0x40, 0x90, 0x00, 0x00, \
  0x28, 0x28, 0xa0, 0xa0, 0x00, 0x00, \
  0x52, 0x11, 0x48, 0x40, 0x00, 0x00, \
  0x41, 0x89, 0x06, 0x20, 0x00, 0x00, \
  0x09, 0x30, 0x24, 0xc0, 0x00, 0x00, \
  0x48, 0x45, 0x21, 0x10, 0x00, 0x00, \
  0x04, 0x44, 0x11, 0x10, 0x00, 0x00, \
  0x0e, 0x80, 0x3a, 0x00, 0x00, 0x00, \
  0xa5, 0x92, 0x96, 0x40, 0x00, 0x00, \
  0x12, 0x0c, 0x48, 0x30, 0x00, 0x00, \
  0xf1, 0x64, 0xbe, 0x40, 0x00, 0x00

#define kMaskRandom28_27 \
  0x62, 0x55, 0x89, 0x50, 0x00, 0x00, \
  0x34, 0x60, 0xd1, 0x80, 0x00, 0x00, \
  0x48, 0x05, 0x20, 0x10, 0x00, 0x00, \
  0x00, 0xac, 0x02, 0xb0, 0x00, 0x00, \
  0x28, 0x08, 0xa0, 0x20, 0x00, 0x00, \
  0x81, 0x0a, 0x04, 0x20, 0x00, 0x00, \
  0x23, 0x04, 0x8c, 0x10, 0x00, 0x00, \
  0x06, 0x80, 0x1a, 0x00, 0x00, 0x00, \
  0x80, 0x16, 0x00, 0x50, 0x00, 0x00, \
  0x30, 0x10, 0xc0, 0x40, 0x00, 0x00, \
  0x8c, 0x22, 0x30, 0x80, 0x00, 0x00, \
  0x54, 0x01, 0x50, 0x00, 0x00, 0x00, \
  0x80, 0xc2, 0x03, 0x00, 0x00, 0x00, \
  0x40, 0x55, 0x01, 0x50, 0x00, 0x00, \
  0x15, 0x40, 0x55, 0x00, 0x00, 0x00, \
  0xc0, 0x07, 0x00, 0x10, 0x00, 0x00, \
  0x28, 0x10, 0xa0, 0x40, 0x00, 0x00, \
  0x05, 0x0c, 0x14, 0x30, 0x00, 0x00, \
  0x64, 0x81, 0x92, 0x00, 0x00, 0x00, \
  0x81, 0x82, 0x06, 0x00, 0x00, 0x00, \
  0x10, 0x98, 0x42, 0x60, 0x00, 0x00, \
  0x84, 0x22, 0x10, 0x80, 0x00, 0x00, \
  0x12, 0x30, 0x48, 0xc0, 0x00, 0x00, \
  0x62, 0x01, 0x88, 0x00, 0x00, 0x00, \
  0x28, 0x60, 0xa1, 0x80, 0x00, 0x00, \
  0x0e, 0x08, 0x38, 0x20, 0x00, 0x00, \
  0x10, 0x84, 0x42, 0x10, 0x00, 0x00

#define kMaskRandom28_28 \
  0x40, 0x55, 0x01, 0x50, 0x00, 0x00, \
  0x15, 0x40, 0x55, 0x00, 0x00, 0x00, \
  0xc0, 0x07, 0x00, 0x10, 0x00, 0x00, \
  0x28, 0x10, 0xa0, 0x40, 0x00, 0x00, \
  0x05, 0x0c, 0x14, 0x30, 0x00, 0x00, \
  0x64, 0x81, 0x92, 0x00, 0x00, 0x00, \
  0x81, 0x82, 0x06, 0x00, 0x00, 0x00, \
  0x10, 0x98, 0x42, 0x60, 0x00, 0x00, \
  0x84, 0x22, 0x10, 0x80, 0x00, 0x00, \
  0x12, 0x30, 0x48, 0xc0, 0x00, 0x00, \
  0x62, 0x01, 0x88, 0x00, 0x00, 0x00, \
  0x28, 0x60, 0xa1, 0x80, 0x00, 0x00, \
  0x0e, 0x08, 0x38, 0x20, 0x00, 0x00, \
  0x10, 0x84, 0x42, 0x10, 0x00, 0x00, \
  0x62, 0x55, 0x89, 0x50, 0x00, 0x00, \
  0x34, 0x60, 0xd1, 0x80, 0x00, 0x00, \
  0x48, 0x05, 0x20, 0x10, 0x00, 0x00, \
  0x00, 0xac, 0x02, 0xb0, 0x00, 0x00, \
  0x28, 0x08, 0xa0, 0x20, 0x00, 0x00, \
  0x81, 0x0a, 0x04, 0x20, 0x00, 0x00, \
  0x23, 0x04, 0x8c, 0x10, 0x00, 0x00, \
  0x06, 0x80, 0x1a, 0x00, 0x00, 0x00, \
  0x80, 0x16, 0x00, 0x50, 0x00, 0x00, \
  0x30, 0x10, 0xc0, 0x40, 0x00, 0x00, \
  0x8c, 0x22, 0x30, 0x80, 0x00, 0x00, \
  0x54, 0x01, 0x50, 0x00, 0x00, 0x00, \
  0x80, 0xc2, 0x03, 0x00, 0x00, 0x00, \
  0x36, 0x4f, 0x1f, 0xb0, 0x00, 0x00

#define kMaskRandom28_3 \
  0xac, 0xda, 0xb3, 0x60, 0x00, 0x00, \
  0x55, 0x6d, 0x55, 0xb0, 0x00, 0x00, \
  0x27, 0xb4, 0x9e, 0xd0, 0x00, 0x00

#define kMaskRandom28_4 \
  0x2c, 0xd8, 0xb3, 0x60, 0x00, 0x00, \
  0x93, 0x6a, 0x4d, 0xa0, 0x00, 0x00, \
  0x1a, 0xb4, 0x6a, 0xd0, 0x00, 0x00, \
  0x47, 0x2d, 0x1c, 0xb0, 0x00, 0x00

#define kMaskRandom28_5 \
  0x64, 0xd9, 0x93, 0x60, 0x00, 0x00, \
  0xa5, 0x6a, 0x95, 0xa0, 0x00, 0x00, \
  0x52, 0xb5, 0x4a, 0xd0, 0x00, 0x00, \
  0x1d, 0xa8, 0x76, 0xa0, 0x00, 0x00, \
  0x9c, 0x56, 0x71, 0x50, 0x00, 0x00

#define kMaskRandom28_6 \
  0x4a, 0x55, 0x29, 0x50, 0x00, 0x00, \
  0x95, 0x4a, 0x55, 0x20, 0x00, 0x00, \
  0x14, 0xb4, 0x52, 0xd0, 0x00, 0x00, \
  0x51, 0xa9, 0x46, 0xa0, 0x00, 0x00, \
  0x22, 0x6c, 0x89, 0xb0, 0x00, 0x00, \
  0x88, 0x8e, 0x22, 0x30, 0x00, 0x00

#define kMaskRandom28_7 \
  0x62, 0x55, 0x89, 0x50, 0x00, 0x00, \
  0xb9, 0x22, 0xe4, 0x80, 0x00, 0x00, \
  0x18, 0xb4, 0x62, 0xd0, 0x00, 0x00, \
  0x54, 0x99, 0x52, 0x60, 0x00, 0x00, \
  0x06, 0x6c, 0x19, 0xb0, 0x00, 0x00, \
  0x85, 0x56, 0x15, 0x50, 0x00, 0x00, \
  0xaa, 0x8a, 0xaa, 0x20, 0x00, 0x00

#define kMaskRandom28_8 \
  0xc0, 0x17, 0x00, 0x50, 0x00, 0x00, \
  0x41, 0x61, 0x05, 0x80, 0x00, 0x00, \
  0x88, 0x32, 0x20, 0xc0, 0x00, 0x00, \
  0x20, 0xa4, 0x82, 0x90, 0x00, 0x00, \
  0x0a, 0x48, 0x29, 0x20, 0x00, 0x00, \
  0x04, 0x98, 0x12, 0x60, 0x00, 0x00, \
  0x94, 0x42, 0x51, 0x00, 0x00, 0x00, \
  0x72, 0x01, 0xc8, 0x00, 0x00, 0x00

#define kMaskRandom28_9 \
  0xa2, 0x56, 0x89, 0x50, 0x00, 0x00, \
  0x34, 0x60, 0xd1, 0x80, 0x00, 0x00, \
  0x4a, 0x25, 0x28, 0x90, 0x00, 0x00, \
  0x20, 0xa8, 0x82, 0xa0, 0x00, 0x00, \
  0x11, 0x84, 0x46, 0x10, 0x00, 0x00, \
  0x49, 0x09, 0x24, 0x20, 0x00, 0x00, \
  0x86, 0x0e, 0x18, 0x30, 0x00, 0x00, \
  0x20, 0xd4, 0x83, 0x50, 0x00, 0x00, \
  0x88, 0x4a, 0x21, 0x20, 0x00, 0x00

#define kMaskRandom29_1 \
  0xff, 0xff, 0xff, 0xf8, 0x00, 0x00

#define kMaskRandom29_10 \
  0xc0, 0xd7, 0x02, 0x80, 0x00, 0x00, \
  0x1d, 0x40, 0x55, 0x58, 0x00, 0x00, \
  0xd4, 0x09, 0xd1, 0x00, 0x00, 0x00, \
  0x02, 0x60, 0x02, 0x70, 0x00, 0x00, \
  0x04, 0x28, 0x04, 0xb0, 0x00, 0x00, \
  0x20, 0x99, 0x12, 0x48, 0x00, 0x00, \
  0x40, 0x46, 0x21, 0x40, 0x00, 0x00, \
  0x08, 0x84, 0x82, 0x90, 0x00, 0x00, \
  0x68, 0x02, 0xa8, 0x10, 0x00, 0x00, \
  0x23, 0x10, 0x09, 0x88, 0x00, 0x00

#define kMaskRandom29_11 \
  0x62, 0xd1, 0x88, 0x88, 0x00, 0x00, \
  0x35, 0x23, 0xc4, 0x40, 0x00, 0x00, \
  0x14, 0x14, 0x40, 0x38, 0x00, 0x00, \
  0xc5, 0x08, 0x42, 0xc0, 0x00, 0x00, \
  0x22, 0x0c, 0x90, 0x90, 0x00, 0x00, \
  0x88, 0xb8, 0x04, 0x48, 0x00, 0x00, \
  0x42, 0x54, 0x03, 0x10, 0x00, 0x00, \
  0x28, 0xa4, 0x12, 0x88, 0x00, 0x00, \
  0x94, 0x20, 0x09, 0x60, 0x00, 0x00, \
  0x1b, 0x04, 0xac, 0x00, 0x00, 0x00, \
  0x22, 0xc2, 0x61, 0x00, 0x00, 0x00

#define kMaskRandom29_12 \
  0x81, 0x06, 0x22, 0x40, 0x00, 0x00, \
  0x40, 0x69, 0x01, 0x50, 0x00, 0x00, \
  0x90, 0x26, 0x09, 0x88, 0x00, 0x00, \
  0x28, 0x28, 0x86, 0x90, 0x00, 0x00, \
  0x52, 0x10, 0x41, 0x90, 0x00, 0x00, \
  0x41, 0x89, 0x10, 0x28, 0x00, 0x00, \
  0x09, 0x30, 0x43, 0x20, 0x00, 0x00, \
  0x48, 0x45, 0x34, 0xa8, 0x00, 0x00, \
  0x04, 0x44, 0xe0, 0x08, 0x00, 0x00, \
  0x0e, 0x80, 0x5d, 0x20, 0x00, 0x00, \
  0xa5, 0x92, 0x42, 0x10, 0x00, 0x00, \
  0x12, 0x0d, 0xc8, 0x50, 0x00, 0x00

#define kMaskRandom29_13 \
  0x62, 0x55, 0x8a, 0x88, 0x00, 0x00, \
  0x34, 0x60, 0xd1, 0x10, 0x00, 0x00, \
  0x48, 0x05, 0x01, 0x28, 0x00, 0x00, \
  0x00, 0xaf, 0x10, 0x10, 0x00, 0x00, \
  0x28, 0x08, 0x21, 0x80, 0x00, 0x00, \
  0x81, 0x0a, 0x50, 0x48, 0x00, 0x00, \
  0x23, 0x06, 0x23, 0x00, 0x00, 0x00, \
  0x06, 0x80, 0x84, 0xc8, 0x00, 0x00, \
  0x80, 0x17, 0x05, 0x00, 0x00, 0x00, \
  0x30, 0x10, 0x41, 0xa0, 0x00, 0x00, \
  0x8c, 0x20, 0x1a, 0x40, 0x00, 0x00, \
  0x54, 0x01, 0x64, 0x00, 0x00, 0x00, \
  0x80, 0xc0, 0x28, 0x30, 0x00, 0x00

#define kMaskRandom29_14 \
  0x40, 0x55, 0x02, 0x08, 0x00, 0x00, \
  0x15, 0x40, 0x55, 0x50, 0x00, 0x00, \
  0xc0, 0x06, 0x20, 0x48, 0x00, 0x00, \
  0x28, 0x13, 0x00, 0x40, 0x00, 0x00, \
  0x05, 0x0e, 0x02, 0x80, 0x00, 0x00, \
  0x64, 0x80, 0x04, 0x88, 0x00, 0x00, \
  0x81, 0x81, 0x00, 0xb0, 0x00, 0x00, \
  0x10, 0x98, 0x88, 0x08, 0x00, 0x00, \
  0x84, 0x22, 0x40, 0x10, 0x00, 0x00, \
  0x12, 0x30, 0x49, 0x00, 0x00, 0x00, \
  0x62, 0x01, 0x74, 0x00, 0x00, 0x00, \
  0x28, 0x60, 0x81, 0x50, 0x00, 0x00, \
  0x0e, 0x0a, 0x18, 0x20, 0x00, 0x00, \
  0x10, 0x84, 0xa2, 0x20, 0x00, 0x00

#define kMaskRandom29_15 \
  0x62, 0x55, 0x8a, 0x88, 0x00, 0x00, \
  0xb9, 0x22, 0xc4, 0x50, 0x00, 0x00, \
  0x18, 0xb4, 0x61, 0xa8, 0x00, 0x00, \
  0x54, 0x99, 0x13, 0x50, 0x00, 0x00, \
  0x06, 0x6c, 0x4d, 0x90, 0x00, 0x00, \
  0x85, 0x55, 0x24, 0x68, 0x00, 0x00, \
  0xaa, 0x8a, 0x1a, 0x30, 0x00, 0x00, \
  0xc0, 0x16, 0x40, 0x88, 0x00, 0x00, \
  0x41, 0x60, 0x25, 0x40, 0x00, 0x00, \
  0x88, 0x30, 0x01, 0xa8, 0x00, 0x00, \
  0x20, 0xa4, 0x80, 0xd0, 0x00, 0x00, \
  0x0a, 0x48, 0x51, 0x10, 0x00, 0x00, \
  0x04, 0x9b, 0x08, 0x40, 0x00, 0x00, \
  0x94, 0x40, 0x03, 0x18, 0x00, 0x00, \
  0x72, 0x01, 0x96, 0x00, 0x00, 0x00

#define kMaskRandom29_16 \
  0xc0, 0x16, 0x40, 0x88, 0x00, 0x00, \
  0x41, 0x60, 0x25, 0x40, 0x00, 0x00, \
  0x88, 0x30, 0x01, 0xa8, 0x00, 0x00, \
  0x20, 0xa4, 0x80, 0xd0, 0x00, 0x00, \
  0x0a, 0x48, 0x51, 0x10, 0x00, 0x00, \
  0x04, 0x9b, 0x08, 0x40, 0x00, 0x00, \
  0x94, 0x40, 0x03, 0x18, 0x00, 0x00, \
  0x72, 0x01, 0x96, 0x00, 0x00, 0x00, \
  0x62, 0x55, 0x8a, 0x88, 0x00, 0x00, \
  0xb9, 0x22, 0xc4, 0x50, 0x00, 0x00, \
  0x18, 0xb4, 0x61, 0xa8, 0x00, 0x00, \
  0x54, 0x99, 0x13, 0x50, 0x00, 0x00, \
  0x06, 0x6c, 0x4d, 0x90, 0x00, 0x00, \
  0x85, 0x55, 0x24, 0x68, 0x00, 0x00, \
  0xaa, 0x8a, 0x1a, 0x30, 0x00, 0x00, \
  0x0d, 0x2c, 0xf2, 0x20, 0x00, 0x00

#define kMaskRandom29_17 \
  0xc0, 0x16, 0x40, 0x88, 0x00, 0x00, \
  0x41, 0x60, 0x25, 0x40, 0x00, 0x00, \
  0x88, 0x30, 0x01, 0xa8, 0x00, 0x00, \
  0x20, 0xa4, 0x80, 0xd0, 0x00, 0x00, \
  0x0a, 0x48, 0x51, 0x10, 0x00, 0x00, \
  0x04, 0x9b, 0x08, 0x40, 0x00, 0x00, \
  0x94, 0x40, 0x03, 0x18, 0x00, 0x00, \
  0x72, 0x01, 0x96, 0x00, 0x00, 0x00, \
  0xa2, 0x55, 0x88, 0x88, 0x00, 0x00, \
  0x34, 0x60, 0x91, 0x10, 0x00, 0x00, \
  0x4a, 0x27, 0x01, 0x40, 0x00, 0x00, \
  0x20, 0xa8, 0x0c, 0x30, 0x00, 0x00, \
  0x11, 0x84, 0x58, 0xa0, 0x00, 0x00, \
  0x49, 0x0a, 0x24, 0x00, 0x00, 0x00, \
  0x86, 0x0e, 0x0a, 0x40, 0x00, 0x00, \
  0x20, 0xd4, 0x22, 0x90, 0x00, 0x00, \
  0x88, 0x4a, 0x41, 0x20, 0x00, 0x00

#define kMaskRandom29_18 \
  0xa2, 0x55, 0x88, 0x88, 0x00, 0x00, \
  0x34, 0x60, 0x91, 0x10, 0x00, 0x00, \
  0x4a, 0x27, 0x01, 0x40, 0x00, 0x00, \
  0x20, 0xa8, 0x0c, 0x30, 0x00, 0x00, \
  0x11, 0x84, 0x58, 0xa0, 0x00, 0x00, \
  0x49, 0x0a, 0x24, 0x00, 0x00, 0x00, \
  0x86, 0x0e, 0x0a, 0x40, 0x00, 0x00, \
  0x20, 0xd4, 0x22, 0x90, 0x00, 0x00, \
  0x88, 0x4a, 0x41, 0x20, 0x00, 0x00, \
  0xc0, 0x16, 0x40, 0x88, 0x00, 0x00, \
  0x41, 0x60, 0x25, 0x40, 0x00, 0x00, \
  0x88, 0x30, 0x01, 0xa8, 0x00, 0x00, \
  0x20, 0xa4, 0x80, 0xd0, 0x00, 0x00, \
  0x0a, 0x48, 0x51, 0x10, 0x00, 0x00, \
  0x04, 0x9b, 0x08, 0x40, 0x00, 0x00, \
  0x94, 0x40, 0x03, 0x18, 0x00, 0x00, \
  0x72, 0x01, 0x96, 0x00, 0x00, 0x00, \
  0x71, 0x36, 0xf2, 0xb0, 0x00, 0x00

#define kMaskRandom29_19 \
  0xa2, 0x55, 0x88, 0x88, 0x00, 0x00, \
  0x34, 0x60, 0x91, 0x10, 0x00, 0x00, \
  0x4a, 0x27, 0x01, 0x40, 0x00, 0x00, \
  0x20, 0xa8, 0x0c, 0x30, 0x00, 0x00, \
  0x11, 0x84, 0x58, 0xa0, 0x00, 0x00, \
  0x49, 0x0a, 0x24, 0x00, 0x00, 0x00, \
  0x86, 0x0e, 0x0a, 0x40, 0x00, 0x00, \
  0x20, 0xd4, 0x22, 0x90, 0x00, 0x00, \
  0x88, 0x4a, 0x41, 0x20, 0x00, 0x00, \
  0xc0, 0xd7, 0x02, 0x80, 0x00, 0x00, \
  0x1d, 0x40, 0x55, 0x58, 0x00, 0x00, \
  0xd4, 0x09, 0xd1, 0x00, 0x00, 0x00, \
  0x02, 0x60, 0x02, 0x70, 0x00, 0x00, \
  0x04, 0x28, 0x04, 0xb0, 0x00, 0x00, \
  0x20, 0x99, 0x12, 0x48, 0x00, 0x00, \
  0x40, 0x46, 0x21, 0x40, 0x00, 0x00, \
  0x08, 0x84, 0x82, 0x90, 0x00, 0x00, \
  0x68, 0x02, 0xa8, 0x10, 0x00, 0x00, \
  0x23, 0x10, 0x09, 0x88, 0x00, 0x00

#define kMaskRandom29_2 \
  0xec, 0xeb, 0xb3, 0xa8, 0x00, 0x00, \
  0x3b, 0x9e, 0xee, 0x70, 0x00, 0x00

#define kMaskRandom29_20 \
  0xc0, 0xd7, 0x02, 0x80, 0x00, 0x00, \
  0x1d, 0x40, 0x55, 0x58, 0x00, 0x00, \
  0xd4, 0x09, 0xd1, 0x00, 0x00, 0x00, \
  0x02, 0x60, 0x02, 0x70, 0x00, 0x00, \
  0x04, 0x28, 0x04, 0xb0, 0x00, 0x00, \
  0x20, 0x99, 0x12, 0x48, 0x00, 0x00, \
  0x40, 0x46, 0x21, 0x40, 0x00, 0x00, \
  0x08, 0x84, 0x82, 0x90, 0x00, 0x00, \
  0x68, 0x02, 0xa8, 0x10, 0x00, 0x00, \
  0x23, 0x10, 0x09, 0x88, 0x00, 0x00, \
  0xa2, 0x55, 0x88, 0x88, 0x00, 0x00, \
  0x34, 0x60, 0x91, 0x10, 0x00, 0x00, \
  0x4a, 0x27, 0x01, 0x40, 0x00, 0x00, \
  0x20, 0xa8, 0x0c, 0x30, 0x00, 0x00, \
  0x11, 0x84, 0x58, 0xa0, 0x00, 0x00, \
  0x49, 0x0a, 0x24, 0x00, 0x00, 0x00, \
  0x86, 0x0e, 0x0a, 0x40, 0x00, 0x00, \
  0x20, 0xd4, 0x22, 0x90, 0x00, 0x00, \
  0x88, 0x4a, 0x41, 0x20, 0x00, 0x00, \
  0xe7, 0xec, 0xdc, 0xb0, 0x00, 0x00

#define kMaskRandom29_21 \
  0xc0, 0xd7, 0x02, 0x80, 0x00, 0x00, \
  0x1d, 0x40, 0x55, 0x58, 0x00, 0x00, \
  0xd4, 0x09, 0xd1, 0x00, 0x00, 0x00, \
  0x02, 0x60, 0x02, 0x70, 0x00, 0x00, \
  0x04, 0x28, 0x04, 0xb0, 0x00, 0x00, \
  0x20, 0x99, 0x12, 0x48, 0x00, 0x00, \
  0x40, 0x46, 0x21, 0x40, 0x00, 0x00, \
  0x08, 0x84, 0x82, 0x90, 0x00, 0x00, \
  0x68, 0x02, 0xa8, 0x10, 0x00, 0x00, \
  0x23, 0x10, 0x09, 0x88, 0x00, 0x00, \
  0x62, 0xd1, 0x88, 0x88, 0x00, 0x00, \
  0x35, 0x23, 0xc4, 0x40, 0x00, 0x00, \
  0x14, 0x14, 0x40, 0x38, 0x00, 0x00, \
  0xc5, 0x08, 0x42, 0xc0, 0x00, 0x00, \
  0x22, 0x0c, 0x90, 0x90, 0x00, 0x00, \
  0x88, 0xb8, 0x04, 0x48, 0x00, 0x00, \
  0x42, 0x54, 0x03, 0x10, 0x00, 0x00, \
  0x28, 0xa4, 0x12, 0x88, 0x00, 0x00, \
  0x94, 0x20, 0x09, 0x60, 0x00, 0x00, \
  0x1b, 0x04, 0xac, 0x00, 0x00, 0x00, \
  0x22, 0xc2, 0x61, 0x00, 0x00, 0x00

#define kMaskRandom29_22 \
  0x62, 0xd1, 0x88, 0x88, 0x00, 0x00, \
  0x35, 0x23, 0xc4, 0x40, 0x00, 0x00, \
  0x14, 0x14, 0x40, 0x38, 0x00, 0x00, \
  0xc5, 0x08, 0x42, 0xc0, 0x00, 0x00, \
  0x22, 0x0c, 0x90, 0x90, 0x00, 0x00, \
  0x88, 0xb8, 0x04, 0x48, 0x00, 0x00, \
  0x42, 0x54, 0x03, 0x10, 0x00, 0x00, \
  0x28, 0xa4, 0x12, 0x88, 0x00, 0x00, \
  0x94, 0x20, 0x09, 0x60, 0x00, 0x00, \
  0x1b, 0x04, 0xac, 0x00, 0x00, 0x00, \
  0x22, 0xc2, 0x61, 0x00, 0x00, 0x00, \
  0xc0, 0xd7, 0x02, 0x80, 0x00, 0x00, \
  0x1d, 0x40, 0x55, 0x58, 0x00, 0x00, \
  0xd4, 0x09, 0xd1, 0x00, 0x00, 0x00, \
  0x02, 0x60, 0x02, 0x70, 0x00, 0x00, \
  0x04, 0x28, 0x04, 0xb0, 0x00, 0x00, \
  0x20, 0x99, 0x12, 0x48, 0x00, 0x00, \
  0x40, 0x46, 0x21, 0x40, 0x00, 0x00, \
  0x08, 0x84, 0x82, 0x90, 0x00, 0x00, \
  0x68, 0x02, 0xa8, 0x10, 0x00, 0x00, \
  0x23, 0x10, 0x09, 0x88, 0x00, 0x00, \
  0x1c, 0x90, 0xa9, 0xa0, 0x00, 0x00

#define kMaskRandom29_23 \
  0x62, 0xd1, 0x88, 0x88, 0x00, 0x00, \
  0x35, 0x23, 0xc4, 0x40, 0x00, 0x00, \
  0x14, 0x14, 0x40, 0x38, 0x00, 0x00, \
  0xc5, 0x08, 0x42, 0xc0, 0x00, 0x00, \
  0x22, 0x0c, 0x90, 0x90, 0x00, 0x00, \
  0x88, 0xb8, 0x04, 0x48, 0x00, 0x00, \
  0x42, 0x54, 0x03, 0x10, 0x00, 0x00, \
  0x28, 0xa4, 0x12, 0x88, 0x00, 0x00, \
  0x94, 0x20, 0x09, 0x60, 0x00, 0x00, \
  0x1b, 0x04, 0xac, 0x00, 0x00, 0x00, \
  0x22, 0xc2, 0x61, 0x00, 0x00, 0x00, \
  0x81, 0x06, 0x22, 0x40, 0x00, 0x00, \
  0x40, 0x69, 0x01, 0x50, 0x00, 0x00, \
  0x90, 0x26, 0x09, 0x88, 0x00, 0x00, \
  0x28, 0x28, 0x86, 0x90, 0x00, 0x00, \
  0x52, 0x10, 0x41, 0x90, 0x00, 0x00, \
  0x41, 0x89, 0x10, 0x28, 0x00, 0x00, \
  0x09, 0x30, 0x43, 0x20, 0x00, 0x00, \
  0x48, 0x45, 0x34, 0xa8, 0x00, 0x00, \
  0x04, 0x44, 0xe0, 0x08, 0x00, 0x00, \
  0x0e, 0x80, 0x5d, 0x20, 0x00, 0x00, \
  0xa5, 0x92, 0x42, 0x10, 0x00, 0x00, \
  0x12, 0x0d, 0xc8, 0x50, 0x00, 0x00

#define kMaskRandom29_24 \
  0x81, 0x06, 0x22, 0x40, 0x00, 0x00, \
  0x40, 0x69, 0x01, 0x50, 0x00, 0x00, \
  0x90, 0x26, 0x09, 0x88, 0x00, 0x00, \
  0x28, 0x28, 0x86, 0x90, 0x00, 0x00, \
  0x52, 0x10, 0x41, 0x90, 0x00, 0x00, \
  0x41, 0x89, 0x10, 0x28, 0x00, 0x00, \
  0x09, 0x30, 0x43, 0x20, 0x00, 0x00, \
  0x48, 0x45, 0x34, 0xa8, 0x00, 0x00, \
  0x04, 0x44, 0xe0, 0x08, 0x00, 0x00, \
  0x0e, 0x80, 0x5d, 0x20, 0x00, 0x00, \
  0xa5, 0x92, 0x42, 0x10, 0x00, 0x00, \
  0x12, 0x0d, 0xc8, 0x50, 0x00, 0x00, \
  0x62, 0xd1, 0x88, 0x88, 0x00, 0x00, \
  0x35, 0x23, 0xc4, 0x40, 0x00, 0x00, \
  0x14, 0x14, 0x40, 0x38, 0x00, 0x00, \
  0xc5, 0x08, 0x42, 0xc0, 0x00, 0x00, \
  0x22, 0x0c, 0x90, 0x90, 0x00, 0x00, \
  0x88, 0xb8, 0x04, 0x48, 0x00, 0x00, \
  0x42, 0x54, 0x03, 0x10, 0x00, 0x00, \
  0x28, 0xa4, 0x12, 0x88, 0x00, 0x00, \
  0x94, 0x20, 0x09, 0x60, 0x00, 0x00, \
  0x1b, 0x04, 0xac, 0x00, 0x00, 0x00, \
  0x22, 0xc2, 0x61, 0x00, 0x00, 0x00, \
  0xbd, 0x86, 0x97, 0xc0, 0x00, 0x00

#define kMaskRandom29_25 \
  0x81, 0x06, 0x22, 0x40, 0x00, 0x00, \
  0x40, 0x69, 0x01, 0x50, 0x00, 0x00, \
  0x90, 0x26, 0x09, 0x88, 0x00, 0x00, \
  0x28, 0x28, 0x86, 0x90, 0x00, 0x00, \
  0x52, 0x10, 0x41, 0x90, 0x00, 0x00, \
  0x41, 0x89, 0x10, 0x28, 0x00, 0x00, \
  0x09, 0x30, 0x43, 0x20, 0x00, 0x00, \
  0x48, 0x45, 0x34, 0xa8, 0x00, 0x00, \
  0x04, 0x44, 0xe0, 0x08, 0x00, 0x00, \
  0x0e, 0x80, 0x5d, 0x20, 0x00, 0x00, \
  0xa5, 0x92, 0x42, 0x10, 0x00, 0x00, \
  0x12, 0x0d, 0xc8, 0x50, 0x00, 0x00, \
  0x62, 0x55, 0x8a, 0x88, 0x00, 0x00, \
  0x34, 0x60, 0xd1, 0x10, 0x00, 0x00, \
  0x48, 0x05, 0x01, 0x28, 0x00, 0x00, \
  0x00, 0xaf, 0x10, 0x10, 0x00, 0x00, \
  0x28, 0x08, 0x21, 0x80, 0x00, 0x00, \
  0x81, 0x0a, 0x50, 0x48, 0x00, 0x00, \
  0x23, 0x06, 0x23, 0x00, 0x00, 0x00, \
  0x06, 0x80, 0x84, 0xc8, 0x00, 0x00, \
  0x80, 0x17, 0x05, 0x00, 0x00, 0x00, \
  0x30, 0x10, 0x41, 0xa0, 0x00, 0x00, \
  0x8c, 0x20, 0x1a, 0x40, 0x00, 0x00, \
  0x54, 0x01, 0x64, 0x00, 0x00, 0x00, \
  0x80, 0xc0, 0x28, 0x30, 0x00, 0x00

#define kMaskRandom29_26 \
  0x62, 0x55, 0x8a, 0x88, 0x00, 0x00, \
  0x34, 0x60, 0xd1, 0x10, 0x00, 0x00, \
  0x48, 0x05, 0x01, 0x28, 0x00, 0x00, \
  0x00, 0xaf, 0x10, 0x10, 0x00, 0x00, \
  0x28, 0x08, 0x21, 0x80, 0x00, 0x00, \
  0x81, 0x0a, 0x50, 0x48, 0x00, 0x00, \
  0x23, 0x06, 0x23, 0x00, 0x00, 0x00, \
  0x06, 0x80, 0x84, 0xc8, 0x00, 0x00, \
  0x80, 0x17, 0x05, 0x00, 0x00, 0x00, \
  0x30, 0x10, 0x41, 0xa0, 0x00, 0x00, \
  0x8c, 0x20, 0x1a, 0x40, 0x00, 0x00, \
  0x54, 0x01, 0x64, 0x00, 0x00, 0x00, \
  0x80, 0xc0, 0x28, 0x30, 0x00, 0x00, \
  0x81, 0x06, 0x22, 0x40, 0x00, 0x00, \
  0x40, 0x69, 0x01, 0x50, 0x00, 0x00, \
  0x90, 0x26, 0x09, 0x88, 0x00, 0x00, \
  0x28, 0x28, 0x86, 0x90, 0x00, 0x00, \
  0x52, 0x10, 0x41, 0x90, 0x00, 0x00, \
  0x41, 0x89, 0x10, 0x28, 0x00, 0x00, \
  0x09, 0x30, 0x43, 0x20, 0x00, 0x00, \
  0x48, 0x45, 0x34, 0xa8, 0x00, 0x00, \
  0x04, 0x44, 0xe0, 0x08, 0x00, 0x00, \
  0x0e, 0x80, 0x5d, 0x20, 0x00, 0x00, \
  0xa5, 0x92, 0x42, 0x10, 0x00, 0x00, \
  0x12, 0x0d, 0xc8, 0x50, 0x00, 0x00, \
  0xb5, 0x4c, 0xa9, 0x70, 0x00, 0x00

#define kMaskRandom29_27 \
  0x62, 0x55, 0x8a, 0x88, 0x00, 0x00, \
  0x34, 0x60, 0xd1, 0x10, 0x00, 0x00, \
  0x48, 0x05, 0x01, 0x28, 0x00, 0x00, \
  0x00, 0xaf, 0x10, 0x10, 0x00, 0x00, \
  0x28, 0x08, 0x21, 0x80, 0x00, 0x00, \
  0x81, 0x0a, 0x50, 0x48, 0x00, 0x00, \
  0x23, 0x06, 0x23, 0x00, 0x00, 0x00, \
  0x06, 0x80, 0x84, 0xc8, 0x00, 0x00, \
  0x80, 0x17, 0x05, 0x00, 0x00, 0x00, \
  0x30, 0x10, 0x41, 0xa0, 0x00, 0x00, \
  0x8c, 0x20, 0x1a, 0x40, 0x00, 0x00, \
  0x54, 0x01, 0x64, 0x00, 0x00, 0x00, \
  0x80, 0xc0, 0x28, 0x30, 0x00, 0x00, \
  0x40, 0x55, 0x02, 0x08, 0x00, 0x00, \
  0x15, 0x40, 0x55, 0x50, 0x00, 0x00, \
  0xc0, 0x06, 0x20, 0x48, 0x00, 0x00, \
  0x28, 0x13, 0x00, 0x40, 0x00, 0x00, \
  0x05, 0x0e, 0x02, 0x80, 0x00, 0x00, \
  0x64, 0x80, 0x04, 0x88, 0x00, 0x00, \
  0x81, 0x81, 0x00, 0xb0, 0x00, 0x00, \
  0x10, 0x98, 0x88, 0x08, 0x00, 0x00, \
  0x84, 0x22, 0x40, 0x10, 0x00, 0x00, \
  0x12, 0x30, 0x49, 0x00, 0x00, 0x00, \
  0x62, 0x01, 0x74, 0x00, 0x00, 0x00, \
  0x28, 0x60, 0x81, 0x50, 0x00, 0x00, \
  0x0e, 0x0a, 0x18, 0x20, 0x00, 0x00, \
  0x10, 0x84, 0xa2, 0x20, 0x00, 0x00

#define kMaskRandom29_28 \
  0x40, 0x55, 0x02, 0x08, 0x00, 0x00, \
  0x15, 0x40, 0x55, 0x50, 0x00, 0x00, \
  0xc0, 0x06, 0x20, 0x48, 0x00, 0x00, \
  0x28, 0x13, 0x00, 0x40, 0x00, 0x00, \
  0x05, 0x0e, 0x02, 0x80, 0x00, 0x00, \
  0x64, 0x80, 0x04, 0x88, 0x00, 0x00, \
  0x81, 0x81, 0x00, 0xb0, 0x00, 0x00, \
  0x10, 0x98, 0x88, 0x08, 0x00, 0x00, \
  0x84, 0x22, 0x40, 0x10, 0x00, 0x00, \
  0x12, 0x30, 0x49, 0x00, 0x00, 0x00, \
  0x62, 0x01, 0x74, 0x00, 0x00, 0x00, \
  0x28, 0x60, 0x81, 0x50, 0x00, 0x00, \
  0x0e, 0x0a, 0x18, 0x20, 0x00, 0x00, \
  0x10, 0x84, 0xa2, 0x20, 0x00, 0x00, \
  0x62, 0x55, 0x8a, 0x88, 0x00, 0x00, \
  0x34, 0x60, 0xd1, 0x10, 0x00, 0x00, \
  0x48, 0x05, 0x01, 0x28, 0x00, 0x00, \
  0x00, 0xaf, 0x10, 0x10, 0x00, 0x00, \
  0x28, 0x08, 0x21, 0x80, 0x00, 0x00, \
  0x81, 0x0a, 0x50, 0x48, 0x00, 0x00, \
  0x23, 0x06, 0x23, 0x00, 0x00, 0x00, \
  0x06, 0x80, 0x84, 0xc8, 0x00, 0x00, \
  0x80, 0x17, 0x05, 0x00, 0x00, 0x00, \
  0x30, 0x10, 0x41, 0xa0, 0x00, 0x00, \
  0x8c, 0x20, 0x1a, 0x40, 0x00, 0x00, \
  0x54, 0x01, 0x64, 0x00, 0x00, 0x00, \
  0x80, 0xc0, 0x28, 0x30, 0x00, 0x00, \
  0xbe, 0x1f, 0x99, 0xb0, 0x00, 0x00

#define kMaskRandom29_29 \
  0x40, 0x55, 0x02, 0x08, 0x00, 0x00, \
  0x15, 0x40, 0x55, 0x50, 0x00, 0x00, \
  0xc0, 0x06, 0x20, 0x48, 0x00, 0x00, \
  0x28, 0x13, 0x00, 0x40, 0x00, 0x00, \
  0x05, 0x0e, 0x02, 0x80, 0x00, 0x00, \
  0x64, 0x80, 0x04, 0x88, 0x00, 0x00, \
  0x81, 0x81, 0x00, 0xb0, 0x00, 0x00, \
  0x10, 0x98, 0x88, 0x08, 0x00, 0x00, \
  0x84, 0x22, 0x40, 0x10, 0x00, 0x00, \
  0x12, 0x30, 0x49, 0x00, 0x00, 0x00, \
  0x62, 0x01, 0x74, 0x00, 0x00, 0x00, \
  0x28, 0x60, 0x81, 0x50, 0x00, 0x00, \
  0x0e, 0x0a, 0x18, 0x20, 0x00, 0x00, \
  0x10, 0x84, 0xa2, 0x20, 0x00, 0x00, \
  0x40, 0x55, 0x88, 0x88, 0x00, 0x00, \
  0x15, 0x40, 0xc4, 0x40, 0x00, 0x00, \
  0xc0, 0x05, 0x60, 0x00, 0x00, 0x00, \
  0x28, 0x10, 0x04, 0x48, 0x00, 0x00, \
  0x05, 0x0e, 0x20, 0x80, 0x00, 0x00, \
  0x64, 0x81, 0x10, 0x08, 0x00, 0x00, \
  0x81, 0x80, 0xa4, 0x10, 0x00, 0x00, \
  0x10, 0x9a, 0x0a, 0x80, 0x00, 0x00, \
  0x84, 0x20, 0x28, 0x68, 0x00, 0x00, \
  0x12, 0x30, 0x47, 0x80, 0x00, 0x00, \
  0x62, 0x02, 0x10, 0x10, 0x00, 0x00, \
  0x28, 0x62, 0x19, 0x00, 0x00, 0x00, \
  0x0e, 0x08, 0x02, 0x18, 0x00, 0x00, \
  0x10, 0x85, 0x11, 0x20, 0x00, 0x00, \
  0x29, 0x50, 0x42, 0x60, 0x00, 0x00

#define kMaskRandom29_3 \
  0xac, 0xda, 0xb2, 0x48, 0x00, 0x00, \
  0x55, 0x6d, 0x55, 0x28, 0x00, 0x00, \
  0x27, 0xb5, 0x0c, 0xd8, 0x00, 0x00

#define kMaskRandom29_4 \
  0x2c, 0xd8, 0x96, 0xa8, 0x00, 0x00, \
  0x93, 0x6a, 0x55, 0x50, 0x00, 0x00, \
  0x1a, 0xb4, 0x69, 0xa8, 0x00, 0x00, \
  0x47, 0x2d, 0x0f, 0x50, 0x00, 0x00

#define kMaskRandom29_5 \
  0x64, 0xd9, 0x92, 0x88, 0x00, 0x00, \
  0xa5, 0x68, 0x95, 0x50, 0x00, 0x00, \
  0x52, 0xb5, 0x25, 0xa0, 0x00, 0x00, \
  0x1d, 0xa9, 0x4e, 0x40, 0x00, 0x00, \
  0x9c, 0x56, 0x38, 0xc0, 0x00, 0x00

#define kMaskRandom29_6 \
  0x4a, 0x55, 0x8a, 0x28, 0x00, 0x00, \
  0x95, 0x48, 0x55, 0x50, 0x00, 0x00, \
  0x14, 0xb5, 0x31, 0x18, 0x00, 0x00, \
  0x51, 0xa9, 0x4a, 0x50, 0x00, 0x00, \
  0x22, 0x6c, 0x8d, 0x90, 0x00, 0x00, \
  0x88, 0x8e, 0x29, 0x60, 0x00, 0x00

#define kMaskRandom29_7 \
  0x62, 0x55, 0x8a, 0x88, 0x00, 0x00, \
  0xb9, 0x22, 0xc4, 0x50, 0x00, 0x00, \
  0x18, 0xb4, 0x61, 0xa8, 0x00, 0x00, \
  0x54, 0x99, 0x13, 0x50, 0x00, 0x00, \
  0x06, 0x6c, 0x4d, 0x90, 0x00, 0x00, \
  0x85, 0x55, 0x24, 0x68, 0x00, 0x00, \
  0xaa, 0x8a, 0x1a, 0x30, 0x00, 0x00

#define kMaskRandom29_8 \
  0xc0, 0x16, 0x40, 0x88, 0x00, 0x00, \
  0x41, 0x60, 0x25, 0x40, 0x00, 0x00, \
  0x88, 0x30, 0x01, 0xa8, 0x00, 0x00, \
  0x20, 0xa4, 0x80, 0xd0, 0x00, 0x00, \
  0x0a, 0x48, 0x51, 0x10, 0x00, 0x00, \
  0x04, 0x9b, 0x08, 0x40, 0x00, 0x00, \
  0x94, 0x40, 0x03, 0x18, 0x00, 0x00, \
  0x72, 0x01, 0x96, 0x00, 0x00, 0x00

#define kMaskRandom29_9 \
  0xa2, 0x55, 0x88, 0x88, 0x00, 0x00, \
  0x34, 0x60, 0x91, 0x10, 0x00, 0x00, \
  0x4a, 0x27, 0x01, 0x40, 0x00, 0x00, \
  0x20, 0xa8, 0x0c, 0x30, 0x00, 0x00, \
  0x11, 0x84, 0x58, 0xa0, 0x00, 0x00, \
  0x49, 0x0a, 0x24, 0x00, 0x00, 0x00, \
  0x86, 0x0e, 0x0a, 0x40, 0x00, 0x00, \
  0x20, 0xd4, 0x22, 0x90, 0x00, 0x00, \
  0x88, 0x4a, 0x41, 0x20, 0x00, 0x00

#define kMaskRandom30_1 \
  0xff, 0xff, 0xff, 0xfc, 0x00, 0x00

#define kMaskRandom30_10 \
  0xc0, 0xa1, 0x81, 0x40, 0x00, 0x00, \
  0x15, 0x56, 0x2a, 0xac, 0x00, 0x00, \
  0x74, 0x40, 0xe8, 0x80, 0x00, 0x00, \
  0x00, 0x9c, 0x01, 0x38, 0x00, 0x00, \
  0x01, 0x2c, 0x02, 0x58, 0x00, 0x00, \
  0x44, 0x92, 0x89, 0x24, 0x00, 0x00, \
  0x88, 0x51, 0x10, 0xa0, 0x00, 0x00, \
  0x20, 0xa4, 0x41, 0x48, 0x00, 0x00, \
  0xaa, 0x05, 0x54, 0x08, 0x00, 0x00, \
  0x02, 0x62, 0x04, 0xc4, 0x00, 0x00

#define kMaskRandom30_11 \
  0x62, 0x22, 0xc4, 0x44, 0x00, 0x00, \
  0xf1, 0x11, 0xe2, 0x20, 0x00, 0x00, \
  0x10, 0x0e, 0x20, 0x1c, 0x00, 0x00, \
  0x10, 0xb0, 0x21, 0x60, 0x00, 0x00, \
  0x24, 0x24, 0x48, 0x48, 0x00, 0x00, \
  0x01, 0x12, 0x02, 0x24, 0x00, 0x00, \
  0x00, 0xc4, 0x01, 0x88, 0x00, 0x00, \
  0x04, 0xa2, 0x09, 0x44, 0x00, 0x00, \
  0x02, 0x58, 0x04, 0xb0, 0x00, 0x00, \
  0x2b, 0x00, 0x56, 0x00, 0x00, 0x00, \
  0x98, 0x41, 0x30, 0x80, 0x00, 0x00

#define kMaskRandom30_12 \
  0x88, 0x91, 0x11, 0x20, 0x00, 0x00, \
  0x40, 0x54, 0x80, 0xa8, 0x00, 0x00, \
  0x82, 0x63, 0x04, 0xc4, 0x00, 0x00, \
  0x21, 0xa4, 0x43, 0x48, 0x00, 0x00, \
  0x10, 0x64, 0x20, 0xc8, 0x00, 0x00, \
  0x44, 0x0a, 0x88, 0x14, 0x00, 0x00, \
  0x10, 0xc8, 0x21, 0x90, 0x00, 0x00, \
  0x4d, 0x2a, 0x9a, 0x54, 0x00, 0x00, \
  0x38, 0x02, 0x70, 0x04, 0x00, 0x00, \
  0x17, 0x48, 0x2e, 0x90, 0x00, 0x00, \
  0x90, 0x85, 0x21, 0x08, 0x00, 0x00, \
  0x72, 0x14, 0xe4, 0x28, 0x00, 0x00

#define kMaskRandom30_13 \
  0x62, 0xa2, 0xc5, 0x44, 0x00, 0x00, \
  0x34, 0x44, 0x68, 0x88, 0x00, 0x00, \
  0x40, 0x4a, 0x80, 0x94, 0x00, 0x00, \
  0xc4, 0x05, 0x88, 0x08, 0x00, 0x00, \
  0x08, 0x60, 0x10, 0xc0, 0x00, 0x00, \
  0x94, 0x13, 0x28, 0x24, 0x00, 0x00, \
  0x88, 0xc1, 0x11, 0x80, 0x00, 0x00, \
  0x21, 0x32, 0x42, 0x64, 0x00, 0x00, \
  0xc1, 0x41, 0x82, 0x80, 0x00, 0x00, \
  0x10, 0x68, 0x20, 0xd0, 0x00, 0x00, \
  0x06, 0x90, 0x0d, 0x20, 0x00, 0x00, \
  0x59, 0x00, 0xb2, 0x00, 0x00, 0x00, \
  0x0a, 0x0c, 0x14, 0x18, 0x00, 0x00

#define kMaskRandom30_14 \
  0x40, 0x82, 0x81, 0x04, 0x00, 0x00, \
  0x15, 0x54, 0x2a, 0xa8, 0x00, 0x00, \
  0x88, 0x13, 0x10, 0x24, 0x00, 0x00, \
  0xc0, 0x11, 0x80, 0x20, 0x00, 0x00, \
  0x80, 0xa1, 0x01, 0x40, 0x00, 0x00, \
  0x01, 0x22, 0x02, 0x44, 0x00, 0x00, \
  0x40, 0x2c, 0x80, 0x58, 0x00, 0x00, \
  0x22, 0x02, 0x44, 0x04, 0x00, 0x00, \
  0x90, 0x05, 0x20, 0x08, 0x00, 0x00, \
  0x12, 0x40, 0x24, 0x80, 0x00, 0x00, \
  0x5d, 0x00, 0xba, 0x00, 0x00, 0x00, \
  0x20, 0x54, 0x40, 0xa8, 0x00, 0x00, \
  0x86, 0x09, 0x0c, 0x10, 0x00, 0x00, \
  0x28, 0x88, 0x51, 0x10, 0x00, 0x00

#define kMaskRandom30_15 \
  0x62, 0x22, 0xc4, 0x44, 0x00, 0x00, \
  0x31, 0x10, 0x62, 0x20, 0x00, 0x00, \
  0x58, 0x00, 0xb0, 0x00, 0x00, 0x00, \
  0x01, 0x12, 0x02, 0x24, 0x00, 0x00, \
  0x88, 0x21, 0x10, 0x40, 0x00, 0x00, \
  0x44, 0x02, 0x88, 0x04, 0x00, 0x00, \
  0x29, 0x04, 0x52, 0x08, 0x00, 0x00, \
  0x82, 0xa1, 0x05, 0x40, 0x00, 0x00, \
  0x0a, 0x1a, 0x14, 0x34, 0x00, 0x00, \
  0x11, 0xe0, 0x23, 0xc0, 0x00, 0x00, \
  0x84, 0x05, 0x08, 0x08, 0x00, 0x00, \
  0x86, 0x41, 0x0c, 0x80, 0x00, 0x00, \
  0x00, 0x86, 0x01, 0x0c, 0x00, 0x00, \
  0x44, 0x48, 0x88, 0x90, 0x00, 0x00, \
  0x10, 0x98, 0x21, 0x30, 0x00, 0x00

#define kMaskRandom30_16 \
  0x90, 0x23, 0x20, 0x44, 0x00, 0x00, \
  0x09, 0x50, 0x12, 0xa0, 0x00, 0x00, \
  0x00, 0x6a, 0x00, 0xd4, 0x00, 0x00, \
  0x20, 0x34, 0x40, 0x68, 0x00, 0x00, \
  0x14, 0x44, 0x28, 0x88, 0x00, 0x00, \
  0xc2, 0x11, 0x84, 0x20, 0x00, 0x00, \
  0x00, 0xc6, 0x01, 0x8c, 0x00, 0x00, \
  0x65, 0x80, 0xcb, 0x00, 0x00, 0x00, \
  0x62, 0xa2, 0xc5, 0x44, 0x00, 0x00, \
  0xb1, 0x15, 0x62, 0x28, 0x00, 0x00, \
  0x18, 0x6a, 0x30, 0xd4, 0x00, 0x00, \
  0x44, 0xd4, 0x89, 0xa8, 0x00, 0x00, \
  0x13, 0x64, 0x26, 0xc8, 0x00, 0x00, \
  0x49, 0x1a, 0x92, 0x34, 0x00, 0x00, \
  0x86, 0x8d, 0x0d, 0x18, 0x00, 0x00, \
  0xce, 0x58, 0xa0, 0x14, 0x00, 0x00

#define kMaskRandom30_17 \
  0x90, 0x23, 0x20, 0x44, 0x00, 0x00, \
  0x09, 0x50, 0x12, 0xa0, 0x00, 0x00, \
  0x00, 0x6a, 0x00, 0xd4, 0x00, 0x00, \
  0x20, 0x34, 0x40, 0x68, 0x00, 0x00, \
  0x14, 0x44, 0x28, 0x88, 0x00, 0x00, \
  0xc2, 0x11, 0x84, 0x20, 0x00, 0x00, \
  0x00, 0xc6, 0x01, 0x8c, 0x00, 0x00, \
  0x65, 0x80, 0xcb, 0x00, 0x00, 0x00, \
  0x62, 0x22, 0xc4, 0x44, 0x00, 0x00, \
  0x24, 0x44, 0x48, 0x88, 0x00, 0x00, \
  0xc0, 0x51, 0x80, 0xa0, 0x00, 0x00, \
  0x03, 0x0c, 0x06, 0x18, 0x00, 0x00, \
  0x16, 0x28, 0x2c, 0x50, 0x00, 0x00, \
  0x89, 0x01, 0x12, 0x00, 0x00, 0x00, \
  0x82, 0x91, 0x05, 0x20, 0x00, 0x00, \
  0x08, 0xa4, 0x11, 0x48, 0x00, 0x00, \
  0x90, 0x49, 0x20, 0x90, 0x00, 0x00

#define kMaskRandom30_18 \
  0x62, 0x22, 0xc4, 0x44, 0x00, 0x00, \
  0x24, 0x44, 0x48, 0x88, 0x00, 0x00, \
  0xc0, 0x51, 0x80, 0xa0, 0x00, 0x00, \
  0x03, 0x0c, 0x06, 0x18, 0x00, 0x00, \
  0x16, 0x28, 0x2c, 0x50, 0x00, 0x00, \
  0x89, 0x01, 0x12, 0x00, 0x00, 0x00, \
  0x82, 0x91, 0x05, 0x20, 0x00, 0x00, \
  0x08, 0xa4, 0x11, 0x48, 0x00, 0x00, \
  0x90, 0x49, 0x20, 0x90, 0x00, 0x00, \
  0x90, 0x23, 0x20, 0x44, 0x00, 0x00, \
  0x09, 0x50, 0x12, 0xa0, 0x00, 0x00, \
  0x00, 0x6a, 0x00, 0xd4, 0x00, 0x00, \
  0x20, 0x34, 0x40, 0x68, 0x00, 0x00, \
  0x14, 0x44, 0x28, 0x88, 0x00, 0x00, \
  0xc2, 0x11, 0x84, 0x20, 0x00, 0x00, \
  0x00, 0xc6, 0x01, 0x8c, 0x00, 0x00, \
  0x65, 0x80, 0xcb, 0x00, 0x00, 0x00, \
  0x00, 0xb2, 0x47, 0x80, 0x00, 0x00

#define kMaskRandom30_19 \
  0x62, 0x22, 0xc4, 0x44, 0x00, 0x00, \
  0x24, 0x44, 0x48, 0x88, 0x00, 0x00, \
  0xc0, 0x51, 0x80, 0xa0, 0x00, 0x00, \
  0x03, 0x0c, 0x06, 0x18, 0x00, 0x00, \
  0x16, 0x28, 0x2c, 0x50, 0x00, 0x00, \
  0x89, 0x01, 0x12, 0x00, 0x00, 0x00, \
  0x82, 0x91, 0x05, 0x20, 0x00, 0x00, \
  0x08, 0xa4, 0x11, 0x48, 0x00, 0x00, \
  0x90, 0x49, 0x20, 0x90, 0x00, 0x00, \
  0xc0, 0xa1, 0x81, 0x40, 0x00, 0x00, \
  0x15, 0x56, 0x2a, 0xac, 0x00, 0x00, \
  0x74, 0x40, 0xe8, 0x80, 0x00, 0x00, \
  0x00, 0x9c, 0x01, 0x38, 0x00, 0x00, \
  0x01, 0x2c, 0x02, 0x58, 0x00, 0x00, \
  0x44, 0x92, 0x89, 0x24, 0x00, 0x00, \
  0x88, 0x51, 0x10, 0xa0, 0x00, 0x00, \
  0x20, 0xa4, 0x41, 0x48, 0x00, 0x00, \
  0xaa, 0x05, 0x54, 0x08, 0x00, 0x00, \
  0x02, 0x62, 0x04, 0xc4, 0x00, 0x00

#define kMaskRandom30_2 \
  0xec, 0xeb, 0xd9, 0xd4, 0x00, 0x00, \
  0xbb, 0x9d, 0x77, 0x38, 0x00, 0x00

#define kMaskRandom30_20 \
  0xc0, 0xa1, 0x81, 0x40, 0x00, 0x00, \
  0x15, 0x56, 0x2a, 0xac, 0x00, 0x00, \
  0x74, 0x40, 0xe8, 0x80, 0x00, 0x00, \
  0x00, 0x9c, 0x01, 0x38, 0x00, 0x00, \
  0x01, 0x2c, 0x02, 0x58, 0x00, 0x00, \
  0x44, 0x92, 0x89, 0x24, 0x00, 0x00, \
  0x88, 0x51, 0x10, 0xa0, 0x00, 0x00, \
  0x20, 0xa4, 0x41, 0x48, 0x00, 0x00, \
  0xaa, 0x05, 0x54, 0x08, 0x00, 0x00, \
  0x02, 0x62, 0x04, 0xc4, 0x00, 0x00, \
  0x62, 0x22, 0xc4, 0x44, 0x00, 0x00, \
  0x24, 0x44, 0x48, 0x88, 0x00, 0x00, \
  0xc0, 0x51, 0x80, 0xa0, 0x00, 0x00, \
  0x03, 0x0c, 0x06, 0x18, 0x00, 0x00, \
  0x16, 0x28, 0x2c, 0x50, 0x00, 0x00, \
  0x89, 0x01, 0x12, 0x00, 0x00, 0x00, \
  0x82, 0x91, 0x05, 0x20, 0x00, 0x00, \
  0x08, 0xa4, 0x11, 0x48, 0x00, 0x00, \
  0x90, 0x49, 0x20, 0x90, 0x00, 0x00, \
  0x51, 0x88, 0xd1, 0x78, 0x00, 0x00

#define kMaskRandom30_21 \
  0xc0, 0xa1, 0x81, 0x40, 0x00, 0x00, \
  0x15, 0x56, 0x2a, 0xac, 0x00, 0x00, \
  0x74, 0x40, 0xe8, 0x80, 0x00, 0x00, \
  0x00, 0x9c, 0x01, 0x38, 0x00, 0x00, \
  0x01, 0x2c, 0x02, 0x58, 0x00, 0x00, \
  0x44, 0x92, 0x89, 0x24, 0x00, 0x00, \
  0x88, 0x51, 0x10, 0xa0, 0x00, 0x00, \
  0x20, 0xa4, 0x41, 0x48, 0x00, 0x00, \
  0xaa, 0x05, 0x54, 0x08, 0x00, 0x00, \
  0x02, 0x62, 0x04, 0xc4, 0x00, 0x00, \
  0x62, 0x22, 0xc4, 0x44, 0x00, 0x00, \
  0xf1, 0x11, 0xe2, 0x20, 0x00, 0x00, \
  0x10, 0x0e, 0x20, 0x1c, 0x00, 0x00, \
  0x10, 0xb0, 0x21, 0x60, 0x00, 0x00, \
  0x24, 0x24, 0x48, 0x48, 0x00, 0x00, \
  0x01, 0x12, 0x02, 0x24, 0x00, 0x00, \
  0x00, 0xc4, 0x01, 0x88, 0x00, 0x00, \
  0x04, 0xa2, 0x09, 0x44, 0x00, 0x00, \
  0x02, 0x58, 0x04, 0xb0, 0x00, 0x00, \
  0x2b, 0x00, 0x56, 0x00, 0x00, 0x00, \
  0x98, 0x41, 0x30, 0x80, 0x00, 0x00

#define kMaskRandom30_22 \
  0x62, 0x22, 0xc4, 0x44, 0x00, 0x00, \
  0xf1, 0x11, 0xe2, 0x20, 0x00, 0x00, \
  0x10, 0x0e, 0x20, 0x1c, 0x00, 0x00, \
  0x10, 0xb0, 0x21, 0x60, 0x00, 0x00, \
  0x24, 0x24, 0x48, 0x48, 0x00, 0x00, \
  0x01, 0x12, 0x02, 0x24, 0x00, 0x00, \
  0x00, 0xc4, 0x01, 0x88, 0x00, 0x00, \
  0x04, 0xa2, 0x09, 0x44, 0x00, 0x00, \
  0x02, 0x58, 0x04, 0xb0, 0x00, 0x00, \
  0x2b, 0x00, 0x56, 0x00, 0x00, 0x00, \
  0x98, 0x41, 0x30, 0x80, 0x00, 0x00, \
  0xc0, 0xa1, 0x81, 0x40, 0x00, 0x00, \
  0x15, 0x56, 0x2a, 0xac, 0x00, 0x00, \
  0x74, 0x40, 0xe8, 0x80, 0x00, 0x00, \
  0x00, 0x9c, 0x01, 0x38, 0x00, 0x00, \
  0x01, 0x2c, 0x02, 0x58, 0x00, 0x00, \
  0x44, 0x92, 0x89, 0x24, 0x00, 0x00, \
  0x88, 0x51, 0x10, 0xa0, 0x00, 0x00, \
  0x20, 0xa4, 0x41, 0x48, 0x00, 0x00, \
  0xaa, 0x05, 0x54, 0x08, 0x00, 0x00, \
  0x02, 0x62, 0x04, 0xc4, 0x00, 0x00, \
  0x03, 0x10, 0x18, 0x74, 0x00, 0x00

#define kMaskRandom30_23 \
  0x62, 0x22, 0xc4, 0x44, 0x00, 0x00, \
  0xf1, 0x11, 0xe2, 0x20, 0x00, 0x00, \
  0x10, 0x0e, 0x20, 0x1c, 0x00, 0x00, \
  0x10, 0xb0, 0x21, 0x60, 0x00, 0x00, \
  0x24, 0x24, 0x48, 0x48, 0x00, 0x00, \
  0x01, 0x12, 0x02, 0x24, 0x00, 0x00, \
  0x00, 0xc4, 0x01, 0x88, 0x00, 0x00, \
  0x04, 0xa2, 0x09, 0x44, 0x00, 0x00, \
  0x02, 0x58, 0x04, 0xb0, 0x00, 0x00, \
  0x2b, 0x00, 0x56, 0x00, 0x00, 0x00, \
  0x98, 0x41, 0x30, 0x80, 0x00, 0x00, \
  0x88, 0x91, 0x11, 0x20, 0x00, 0x00, \
  0x40, 0x54, 0x80, 0xa8, 0x00, 0x00, \
  0x82, 0x63, 0x04, 0xc4, 0x00, 0x00, \
  0x21, 0xa4, 0x43, 0x48, 0x00, 0x00, \
  0x10, 0x64, 0x20, 0xc8, 0x00, 0x00, \
  0x44, 0x0a, 0x88, 0x14, 0x00, 0x00, \
  0x10, 0xc8, 0x21, 0x90, 0x00, 0x00, \
  0x4d, 0x2a, 0x9a, 0x54, 0x00, 0x00, \
  0x38, 0x02, 0x70, 0x04, 0x00, 0x00, \
  0x17, 0x48, 0x2e, 0x90, 0x00, 0x00, \
  0x90, 0x85, 0x21, 0x08, 0x00, 0x00, \
  0x72, 0x14, 0xe4, 0x28, 0x00, 0x00

#define kMaskRandom30_24 \
  0x88, 0x91, 0x11, 0x20, 0x00, 0x00, \
  0x40, 0x54, 0x80, 0xa8, 0x00, 0x00, \
  0x82, 0x63, 0x04, 0xc4, 0x00, 0x00, \
  0x21, 0xa4, 0x43, 0x48, 0x00, 0x00, \
  0x10, 0x64, 0x20, 0xc8, 0x00, 0x00, \
  0x44, 0x0a, 0x88, 0x14, 0x00, 0x00, \
  0x10, 0xc8, 0x21, 0x90, 0x00, 0x00, \
  0x4d, 0x2a, 0x9a, 0x54, 0x00, 0x00, \
  0x38, 0x02, 0x70, 0x04, 0x00, 0x00, \
  0x17, 0x48, 0x2e, 0x90, 0x00, 0x00, \
  0x90, 0x85, 0x21, 0x08, 0x00, 0x00, \
  0x72, 0x14, 0xe4, 0x28, 0x00, 0x00, \
  0x62, 0x22, 0xc4, 0x44, 0x00, 0x00, \
  0xf1, 0x11, 0xe2, 0x20, 0x00, 0x00, \
  0x10, 0x0e, 0x20, 0x1c, 0x00, 0x00, \
  0x10, 0xb0, 0x21, 0x60, 0x00, 0x00, \
  0x24, 0x24, 0x48, 0x48, 0x00, 0x00, \
  0x01, 0x12, 0x02, 0x24, 0x00, 0x00, \
  0x00, 0xc4, 0x01, 0x88, 0x00, 0x00, \
  0x04, 0xa2, 0x09, 0x44, 0x00, 0x00, \
  0x02, 0x58, 0x04, 0xb0, 0x00, 0x00, \
  0x2b, 0x00, 0x56, 0x00, 0x00, 0x00, \
  0x98, 0x41, 0x30, 0x80, 0x00, 0x00, \
  0xf3, 0x4d, 0x1c, 0x70, 0x00, 0x00

#define kMaskRandom30_25 \
  0x88, 0x91, 0x11, 0x20, 0x00, 0x00, \
  0x40, 0x54, 0x80, 0xa8, 0x00, 0x00, \
  0x82, 0x63, 0x04, 0xc4, 0x00, 0x00, \
  0x21, 0xa4, 0x43, 0x48, 0x00, 0x00, \
  0x10, 0x64, 0x20, 0xc8, 0x00, 0x00, \
  0x44, 0x0a, 0x88, 0x14, 0x00, 0x00, \
  0x10, 0xc8, 0x21, 0x90, 0x00, 0x00, \
  0x4d, 0x2a, 0x9a, 0x54, 0x00, 0x00, \
  0x38, 0x02, 0x70, 0x04, 0x00, 0x00, \
  0x17, 0x48, 0x2e, 0x90, 0x00, 0x00, \
  0x90, 0x85, 0x21, 0x08, 0x00, 0x00, \
  0x72, 0x14, 0xe4, 0x28, 0x00, 0x00, \
  0x62, 0xa2, 0xc5, 0x44, 0x00, 0x00, \
  0x34, 0x44, 0x68, 0x88, 0x00, 0x00, \
  0x40, 0x4a, 0x80, 0x94, 0x00, 0x00, \
  0xc4, 0x05, 0x88, 0x08, 0x00, 0x00, \
  0x08, 0x60, 0x10, 0xc0, 0x00, 0x00, \
  0x94, 0x13, 0x28, 0x24, 0x00, 0x00, \
  0x88, 0xc1, 0x11, 0x80, 0x00, 0x00, \
  0x21, 0x32, 0x42, 0x64, 0x00, 0x00, \
  0xc1, 0x41, 0x82, 0x80, 0x00, 0x00, \
  0x10, 0x68, 0x20, 0xd0, 0x00, 0x00, \
  0x06, 0x90, 0x0d, 0x20, 0x00, 0x00, \
  0x59, 0x00, 0xb2, 0x00, 0x00, 0x00, \
  0x0a, 0x0c, 0x14, 0x18, 0x00, 0x00

#define kMaskRandom30_26 \
  0x62, 0xa2, 0xc5, 0x44, 0x00, 0x00, \
  0x34, 0x44, 0x68, 0x88, 0x00, 0x00, \
  0x40, 0x4a, 0x80, 0x94, 0x00, 0x00, \
  0xc4, 0x05, 0x88, 0x08, 0x00, 0x00, \
  0x08, 0x60, 0x10, 0xc0, 0x00, 0x00, \
  0x94, 0x13, 0x28, 0x24, 0x00, 0x00, \
  0x88, 0xc1, 0x11, 0x80, 0x00, 0x00, \
  0x21, 0x32, 0x42, 0x64, 0x00, 0x00, \
  0xc1, 0x41, 0x82, 0x80, 0x00, 0x00, \
  0x10, 0x68, 0x20, 0xd0, 0x00, 0x00, \
  0x06, 0x90, 0x0d, 0x20, 0x00, 0x00, \
  0x59, 0x00, 0xb2, 0x00, 0x00, 0x00, \
  0x0a, 0x0c, 0x14, 0x18, 0x00, 0x00, \
  0x88, 0x91, 0x11, 0x20, 0x00, 0x00, \
  0x40, 0x54, 0x80, 0xa8, 0x00, 0x00, \
  0x82, 0x63, 0x04, 0xc4, 0x00, 0x00, \
  0x21, 0xa4, 0x43, 0x48, 0x00, 0x00, \
  0x10, 0x64, 0x20, 0xc8, 0x00, 0x00, \
  0x44, 0x0a, 0x88, 0x14, 0x00, 0x00, \
  0x10, 0xc8, 0x21, 0x90, 0x00, 0x00, \
  0x4d, 0x2a, 0x9a, 0x54, 0x00, 0x00, \
  0x38, 0x02, 0x70, 0x04, 0x00, 0x00, \
  0x17, 0x48, 0x2e, 0x90, 0x00, 0x00, \
  0x90, 0x85, 0x21, 0x08, 0x00, 0x00, \
  0x72, 0x14, 0xe4, 0x28, 0x00, 0x00, \
  0x83, 0x11, 0xad, 0xe8, 0x00, 0x00

#define kMaskRandom30_27 \
  0x62, 0xa2, 0xc5, 0x44, 0x00, 0x00, \
  0x34, 0x44, 0x68, 0x88, 0x00, 0x00, \
  0x40, 0x4a, 0x80, 0x94, 0x00, 0x00, \
  0xc4, 0x05, 0x88, 0x08, 0x00, 0x00, \
  0x08, 0x60, 0x10, 0xc0, 0x00, 0x00, \
  0x94, 0x13, 0x28, 0x24, 0x00, 0x00, \
  0x88, 0xc1, 0x11, 0x80, 0x00, 0x00, \
  0x21, 0x32, 0x42, 0x64, 0x00, 0x00, \
  0xc1, 0x41, 0x82, 0x80, 0x00, 0x00, \
  0x10, 0x68, 0x20, 0xd0, 0x00, 0x00, \
  0x06, 0x90, 0x0d, 0x20, 0x00, 0x00, \
  0x59, 0x00, 0xb2, 0x00, 0x00, 0x00, \
  0x0a, 0x0c, 0x14, 0x18, 0x00, 0x00, \
  0x40, 0x82, 0x81, 0x04, 0x00, 0x00, \
  0x15, 0x54, 0x2a, 0xa8, 0x00, 0x00, \
  0x88, 0x13, 0x10, 0x24, 0x00, 0x00, \
  0xc0, 0x11, 0x80, 0x20, 0x00, 0x00, \
  0x80, 0xa1, 0x01, 0x40, 0x00, 0x00, \
  0x01, 0x22, 0x02, 0x44, 0x00, 0x00, \
  0x40, 0x2c, 0x80, 0x58, 0x00, 0x00, \
  0x22, 0x02, 0x44, 0x04, 0x00, 0x00, \
  0x90, 0x05, 0x20, 0x08, 0x00, 0x00, \
  0x12, 0x40, 0x24, 0x80, 0x00, 0x00, \
  0x5d, 0x00, 0xba, 0x00, 0x00, 0x00, \
  0x20, 0x54, 0x40, 0xa8, 0x00, 0x00, \
  0x86, 0x09, 0x0c, 0x10, 0x00, 0x00, \
  0x28, 0x88, 0x51, 0x10, 0x00, 0x00

#define kMaskRandom30_28 \
  0x40, 0x82, 0x81, 0x04, 0x00, 0x00, \
  0x15, 0x54, 0x2a, 0xa8, 0x00, 0x00, \
  0x88, 0x13, 0x10, 0x24, 0x00, 0x00, \
  0xc0, 0x11, 0x80, 0x20, 0x00, 0x00, \
  0x80, 0xa1, 0x01, 0x40, 0x00, 0x00, \
  0x01, 0x22, 0x02, 0x44, 0x00, 0x00, \
  0x40, 0x2c, 0x80, 0x58, 0x00, 0x00, \
  0x22, 0x02, 0x44, 0x04, 0x00, 0x00, \
  0x90, 0x05, 0x20, 0x08, 0x00, 0x00, \
  0x12, 0x40, 0x24, 0x80, 0x00, 0x00, \
  0x5d, 0x00, 0xba, 0x00, 0x00, 0x00, \
  0x20, 0x54, 0x40, 0xa8, 0x00, 0x00, \
  0x86, 0x09, 0x0c, 0x10, 0x00, 0x00, \
  0x28, 0x88, 0x51, 0x10, 0x00, 0x00, \
  0x62, 0xa2, 0xc5, 0x44, 0x00, 0x00, \
  0x34, 0x44, 0x68, 0x88, 0x00, 0x00, \
  0x40, 0x4a, 0x80, 0x94, 0x00, 0x00, \
  0xc4, 0x05, 0x88, 0x08, 0x00, 0x00, \
  0x08, 0x60, 0x10, 0xc0, 0x00, 0x00, \
  0x94, 0x13, 0x28, 0x24, 0x00, 0x00, \
  0x88, 0xc1, 0x11, 0x80, 0x00, 0x00, \
  0x21, 0x32, 0x42, 0x64, 0x00, 0x00, \
  0xc1, 0x41, 0x82, 0x80, 0x00, 0x00, \
  0x10, 0x68, 0x20, 0xd0, 0x00, 0x00, \
  0x06, 0x90, 0x0d, 0x20, 0x00, 0x00, \
  0x59, 0x00, 0xb2, 0x00, 0x00, 0x00, \
  0x0a, 0x0c, 0x14, 0x18, 0x00, 0x00, \
  0x94, 0x59, 0x03, 0x18, 0x00, 0x00

#define kMaskRandom30_29 \
  0x40, 0x82, 0x81, 0x04, 0x00, 0x00, \
  0x15, 0x54, 0x2a, 0xa8, 0x00, 0x00, \
  0x88, 0x13, 0x10, 0x24, 0x00, 0x00, \
  0xc0, 0x11, 0x80, 0x20, 0x00, 0x00, \
  0x80, 0xa1, 0x01, 0x40, 0x00, 0x00, \
  0x01, 0x22, 0x02, 0x44, 0x00, 0x00, \
  0x40, 0x2c, 0x80, 0x58, 0x00, 0x00, \
  0x22, 0x02, 0x44, 0x04, 0x00, 0x00, \
  0x90, 0x05, 0x20, 0x08, 0x00, 0x00, \
  0x12, 0x40, 0x24, 0x80, 0x00, 0x00, \
  0x5d, 0x00, 0xba, 0x00, 0x00, 0x00, \
  0x20, 0x54, 0x40, 0xa8, 0x00, 0x00, \
  0x86, 0x09, 0x0c, 0x10, 0x00, 0x00, \
  0x28, 0x88, 0x51, 0x10, 0x00, 0x00, \
  0x62, 0x22, 0xc4, 0x44, 0x00, 0x00, \
  0x31, 0x10, 0x62, 0x20, 0x00, 0x00, \
  0x58, 0x00, 0xb0, 0x00, 0x00, 0x00, \
  0x01, 0x12, 0x02, 0x24, 0x00, 0x00, \
  0x88, 0x21, 0x10, 0x40, 0x00, 0x00, \
  0x44, 0x02, 0x88, 0x04, 0x00, 0x00, \
  0x29, 0x04, 0x52, 0x08, 0x00, 0x00, \
  0x82, 0xa1, 0x05, 0x40, 0x00, 0x00, \
  0x0a, 0x1a, 0x14, 0x34, 0x00, 0x00, \
  0x11, 0xe0, 0x23, 0xc0, 0x00, 0x00, \
  0x84, 0x05, 0x08, 0x08, 0x00, 0x00, \
  0x86, 0x41, 0x0c, 0x80, 0x00, 0x00, \
  0x00, 0x86, 0x01, 0x0c, 0x00, 0x00, \
  0x44, 0x48, 0x88, 0x90, 0x00, 0x00, \
  0x10, 0x98, 0x21, 0x30, 0x00, 0x00

#define kMaskRandom30_3 \
  0xac, 0x93, 0x59, 0x24, 0x00, 0x00, \
  0x55, 0x4a, 0xaa, 0x94, 0x00, 0x00, \
  0x43, 0x36, 0x86, 0x6c, 0x00, 0x00

#define kMaskRandom30_30 \
  0x62, 0x22, 0xc4, 0x44, 0x00, 0x00, \
  0x31, 0x10, 0x62, 0x20, 0x00, 0x00, \
  0x58, 0x00, 0xb0, 0x00, 0x00, 0x00, \
  0x01, 0x12, 0x02, 0x24, 0x00, 0x00, \
  0x88, 0x21, 0x10, 0x40, 0x00, 0x00, \
  0x44, 0x02, 0x88, 0x04, 0x00, 0x00, \
  0x29, 0x04, 0x52, 0x08, 0x00, 0x00, \
  0x82, 0xa1, 0x05, 0x40, 0x00, 0x00, \
  0x0a, 0x1a, 0x14, 0x34, 0x00, 0x00, \
  0x11, 0xe0, 0x23, 0xc0, 0x00, 0x00, \
  0x84, 0x05, 0x08, 0x08, 0x00, 0x00, \
  0x86, 0x41, 0x0c, 0x80, 0x00, 0x00, \
  0x00, 0x86, 0x01, 0x0c, 0x00, 0x00, \
  0x44, 0x48, 0x88, 0x90, 0x00, 0x00, \
  0x10, 0x98, 0x21, 0x30, 0x00, 0x00, \
  0x40, 0x82, 0x81, 0x04, 0x00, 0x00, \
  0x15, 0x54, 0x2a, 0xa8, 0x00, 0x00, \
  0x88, 0x13, 0x10, 0x24, 0x00, 0x00, \
  0xc0, 0x11, 0x80, 0x20, 0x00, 0x00, \
  0x80, 0xa1, 0x01, 0x40, 0x00, 0x00, \
  0x01, 0x22, 0x02, 0x44, 0x00, 0x00, \
  0x40, 0x2c, 0x80, 0x58, 0x00, 0x00, \
  0x22, 0x02, 0x44, 0x04, 0x00, 0x00, \
  0x90, 0x05, 0x20, 0x08, 0x00, 0x00, \
  0x12, 0x40, 0x24, 0x80, 0x00, 0x00, \
  0x5d, 0x00, 0xba, 0x00, 0x00, 0x00, \
  0x20, 0x54, 0x40, 0xa8, 0x00, 0x00, \
  0x86, 0x09, 0x0c, 0x10, 0x00, 0x00, \
  0x28, 0x88, 0x51, 0x10, 0x00, 0x00, \
  0x46, 0xf1, 0xef, 0xec, 0x00, 0x00

#define kMaskRandom30_4 \
  0x25, 0xaa, 0x4b, 0x54, 0x00, 0x00, \
  0x95, 0x55, 0x2a, 0xa8, 0x00, 0x00, \
  0x1a, 0x6a, 0x34, 0xd4, 0x00, 0x00, \
  0x43, 0xd4, 0x87, 0xa8, 0x00, 0x00

#define kMaskRandom30_5 \
  0x64, 0xa2, 0xc9, 0x44, 0x00, 0x00, \
  0x25, 0x54, 0x4a, 0xa8, 0x00, 0x00, \
  0x49, 0x68, 0x92, 0xd0, 0x00, 0x00, \
  0x53, 0x90, 0xa7, 0x20, 0x00, 0x00, \
  0x8e, 0x31, 0x1c, 0x60, 0x00, 0x00

#define kMaskRandom30_6 \
  0x62, 0x8a, 0xc5, 0x14, 0x00, 0x00, \
  0x15, 0x54, 0x2a, 0xa8, 0x00, 0x00, \
  0x4c, 0x46, 0x98, 0x8c, 0x00, 0x00, \
  0x52, 0x94, 0xa5, 0x28, 0x00, 0x00, \
  0x23, 0x64, 0x46, 0xc8, 0x00, 0x00, \
  0x8a, 0x59, 0x14, 0xb0, 0x00, 0x00

#define kMaskRandom30_7 \
  0x62, 0xa2, 0xc5, 0x44, 0x00, 0x00, \
  0xb1, 0x15, 0x62, 0x28, 0x00, 0x00, \
  0x18, 0x6a, 0x30, 0xd4, 0x00, 0x00, \
  0x44, 0xd4, 0x89, 0xa8, 0x00, 0x00, \
  0x13, 0x64, 0x26, 0xc8, 0x00, 0x00, \
  0x49, 0x1a, 0x92, 0x34, 0x00, 0x00, \
  0x86, 0x8d, 0x0d, 0x18, 0x00, 0x00

#define kMaskRandom30_8 \
  0x90, 0x23, 0x20, 0x44, 0x00, 0x00, \
  0x09, 0x50, 0x12, 0xa0, 0x00, 0x00, \
  0x00, 0x6a, 0x00, 0xd4, 0x00, 0x00, \
  0x20, 0x34, 0x40, 0x68, 0x00, 0x00, \
  0x14, 0x44, 0x28, 0x88, 0x00, 0x00, \
  0xc2, 0x11, 0x84, 0x20, 0x00, 0x00, \
  0x00, 0xc6, 0x01, 0x8c, 0x00, 0x00, \
  0x65, 0x80, 0xcb, 0x00, 0x00, 0x00

#define kMaskRandom30_9 \
  0x62, 0x22, 0xc4, 0x44, 0x00, 0x00, \
  0x24, 0x44, 0x48, 0x88, 0x00, 0x00, \
  0xc0, 0x51, 0x80, 0xa0, 0x00, 0x00, \
  0x03, 0x0c, 0x06, 0x18, 0x00, 0x00, \
  0x16, 0x28, 0x2c, 0x50, 0x00, 0x00, \
  0x89, 0x01, 0x12, 0x00, 0x00, 0x00, \
  0x82, 0x91, 0x05, 0x20, 0x00, 0x00, \
  0x08, 0xa4, 0x11, 0x48, 0x00, 0x00, \
  0x90, 0x49, 0x20, 0x90, 0x00, 0x00

#define kMaskRandom31_1 \
  0xff, 0xff, 0xff, 0xfe, 0x00, 0x00

#define kMaskRandom31_10 \
  0xc0, 0xa0, 0x8a, 0xa2, 0x00, 0x00, \
  0x15, 0x56, 0x21, 0x44, 0x00, 0x00, \
  0x74, 0x40, 0x02, 0x4a, 0x00, 0x00, \
  0x00, 0x9c, 0x16, 0x84, 0x00, 0x00, \
  0x01, 0x2d, 0xb0, 0x40, 0x00, 0x00, \
  0x44, 0x93, 0x05, 0x18, 0x00, 0x00, \
  0x88, 0x50, 0x48, 0x94, 0x00, 0x00, \
  0x20, 0xa4, 0x70, 0x30, 0x00, 0x00, \
  0xaa, 0x04, 0x54, 0x4a, 0x00, 0x00, \
  0x02, 0x63, 0x09, 0x24, 0x00, 0x00

#define kMaskRandom31_11 \
  0x62, 0x22, 0xaa, 0xaa, 0x00, 0x00, \
  0xf1, 0x10, 0x54, 0x44, 0x00, 0x00, \
  0x10, 0x0e, 0x62, 0x22, 0x00, 0x00, \
  0x10, 0xb1, 0x06, 0x84, 0x00, 0x00, \
  0x24, 0x24, 0x0d, 0x30, 0x00, 0x00, \
  0x01, 0x12, 0x81, 0xc2, 0x00, 0x00, \
  0x00, 0xc4, 0x58, 0x88, 0x00, 0x00, \
  0x04, 0xa3, 0xb0, 0x50, 0x00, 0x00, \
  0x02, 0x59, 0x25, 0x02, 0x00, 0x00, \
  0x2b, 0x01, 0x08, 0x64, 0x00, 0x00, \
  0x98, 0x40, 0xd0, 0x18, 0x00, 0x00

#define kMaskRandom31_12 \
  0x88, 0x91, 0x08, 0x62, 0x00, 0x00, \
  0x40, 0x54, 0x31, 0x44, 0x00, 0x00, \
  0x82, 0x62, 0x9c, 0x02, 0x00, 0x00, \
  0x21, 0xa4, 0x89, 0x90, 0x00, 0x00, \
  0x10, 0x64, 0x1d, 0x20, 0x00, 0x00, \
  0x44, 0x0a, 0x41, 0x98, 0x00, 0x00, \
  0x10, 0xc9, 0x26, 0x80, 0x00, 0x00, \
  0x4d, 0x2a, 0x5a, 0x20, 0x00, 0x00, \
  0x38, 0x02, 0x62, 0x88, 0x00, 0x00, \
  0x17, 0x49, 0x80, 0x46, 0x00, 0x00, \
  0x90, 0x84, 0x22, 0x4a, 0x00, 0x00, \
  0x72, 0x15, 0xd1, 0x00, 0x00, 0x00

#define kMaskRandom31_13 \
  0x62, 0xa2, 0x8a, 0x2a, 0x00, 0x00, \
  0x34, 0x44, 0x44, 0x44, 0x00, 0x00, \
  0x40, 0x4b, 0x2c, 0x18, 0x00, 0x00, \
  0xc4, 0x04, 0x18, 0xa0, 0x00, 0x00, \
  0x08, 0x60, 0xc4, 0x08, 0x00, 0x00, \
  0x94, 0x12, 0x92, 0x0c, 0x00, 0x00, \
  0x88, 0xc0, 0x23, 0x04, 0x00, 0x00, \
  0x21, 0x32, 0x24, 0x70, 0x00, 0x00, \
  0xc1, 0x40, 0x80, 0xe2, 0x00, 0x00, \
  0x10, 0x69, 0x51, 0x14, 0x00, 0x00, \
  0x06, 0x90, 0x11, 0x42, 0x00, 0x00, \
  0x59, 0x01, 0x41, 0x80, 0x00, 0x00, \
  0x0a, 0x0d, 0x8a, 0x20, 0x00, 0x00

#define kMaskRandom31_14 \
  0x40, 0x82, 0x8a, 0xa2, 0x00, 0x00, \
  0x15, 0x54, 0x44, 0x14, 0x00, 0x00, \
  0x88, 0x13, 0x09, 0xa0, 0x00, 0x00, \
  0xc0, 0x10, 0x19, 0x14, 0x00, 0x00, \
  0x80, 0xa0, 0x30, 0x0c, 0x00, 0x00, \
  0x01, 0x22, 0x60, 0x06, 0x00, 0x00, \
  0x40, 0x2c, 0xc2, 0x10, 0x00, 0x00, \
  0x22, 0x02, 0x80, 0x22, 0x00, 0x00, \
  0x90, 0x04, 0x20, 0x58, 0x00, 0x00, \
  0x12, 0x40, 0x12, 0xc0, 0x00, 0x00, \
  0x5d, 0x00, 0x01, 0x28, 0x00, 0x00, \
  0x20, 0x54, 0xa4, 0x80, 0x00, 0x00, \
  0x86, 0x09, 0x48, 0x48, 0x00, 0x00, \
  0x28, 0x89, 0x05, 0x10, 0x00, 0x00

#define kMaskRandom31_15 \
  0x62, 0x22, 0xaa, 0x22, 0x00, 0x00, \
  0x31, 0x10, 0x44, 0x44, 0x00, 0x00, \
  0x58, 0x00, 0x22, 0x22, 0x00, 0x00, \
  0x01, 0x13, 0x00, 0x8a, 0x00, 0x00, \
  0x88, 0x20, 0x40, 0x34, 0x00, 0x00, \
  0x44, 0x02, 0x10, 0xd0, 0x00, 0x00, \
  0x29, 0x04, 0x45, 0x08, 0x00, 0x00, \
  0x82, 0xa0, 0x90, 0x12, 0x00, 0x00, \
  0x0a, 0x1a, 0x0e, 0x02, 0x00, 0x00, \
  0x11, 0xe1, 0x28, 0x40, 0x00, 0x00, \
  0x84, 0x05, 0x04, 0x0c, 0x00, 0x00, \
  0x86, 0x40, 0xc0, 0x90, 0x00, 0x00, \
  0x00, 0x87, 0x13, 0x00, 0x00, 0x00, \
  0x44, 0x48, 0x01, 0x1c, 0x00, 0x00, \
  0x10, 0x98, 0x30, 0x44, 0x00, 0x00

#define kMaskRandom31_16 \
  0x90, 0x22, 0x40, 0xa8, 0x00, 0x00, \
  0x09, 0x50, 0x31, 0x10, 0x00, 0x00, \
  0x00, 0x6b, 0x08, 0x0e, 0x00, 0x00, \
  0x20, 0x34, 0xc0, 0x90, 0x00, 0x00, \
  0x14, 0x44, 0x25, 0x04, 0x00, 0x00, \
  0xc2, 0x11, 0x02, 0x82, 0x00, 0x00, \
  0x00, 0xc6, 0x80, 0xc4, 0x00, 0x00, \
  0x65, 0x80, 0x2c, 0x60, 0x00, 0x00, \
  0x62, 0xa2, 0x8a, 0xa2, 0x00, 0x00, \
  0xb1, 0x14, 0x44, 0x54, 0x00, 0x00, \
  0x18, 0x6b, 0x22, 0x22, 0x00, 0x00, \
  0x44, 0xd4, 0x5c, 0x10, 0x00, 0x00, \
  0x13, 0x64, 0x90, 0x68, 0x00, 0x00, \
  0x49, 0x1b, 0x20, 0x52, 0x00, 0x00, \
  0x86, 0x8c, 0x13, 0x0c, 0x00, 0x00, \
  0x8d, 0x94, 0xa9, 0xe0, 0x00, 0x00

#define kMaskRandom31_17 \
  0x90, 0x22, 0x40, 0xa8, 0x00, 0x00, \
  0x09, 0x50, 0x31, 0x10, 0x00, 0x00, \
  0x00, 0x6b, 0x08, 0x0e, 0x00, 0x00, \
  0x20, 0x34, 0xc0, 0x90, 0x00, 0x00, \
  0x14, 0x44, 0x25, 0x04, 0x00, 0x00, \
  0xc2, 0x11, 0x02, 0x82, 0x00, 0x00, \
  0x00, 0xc6, 0x80, 0xc4, 0x00, 0x00, \
  0x65, 0x80, 0x2c, 0x60, 0x00, 0x00, \
  0x62, 0x22, 0xaa, 0xa2, 0x00, 0x00, \
  0x24, 0x44, 0x44, 0x54, 0x00, 0x00, \
  0xc0, 0x50, 0x0b, 0x0a, 0x00, 0x00, \
  0x03, 0x0c, 0x12, 0x94, 0x00, 0x00, \
  0x16, 0x29, 0x08, 0x64, 0x00, 0x00, \
  0x89, 0x01, 0x80, 0x1a, 0x00, 0x00, \
  0x82, 0x90, 0x41, 0x4c, 0x00, 0x00, \
  0x08, 0xa4, 0x34, 0x12, 0x00, 0x00, \
  0x90, 0x48, 0x88, 0xc8, 0x00, 0x00

#define kMaskRandom31_18 \
  0x62, 0x22, 0xaa, 0xa2, 0x00, 0x00, \
  0x24, 0x44, 0x44, 0x54, 0x00, 0x00, \
  0xc0, 0x50, 0x0b, 0x0a, 0x00, 0x00, \
  0x03, 0x0c, 0x12, 0x94, 0x00, 0x00, \
  0x16, 0x29, 0x08, 0x64, 0x00, 0x00, \
  0x89, 0x01, 0x80, 0x1a, 0x00, 0x00, \
  0x82, 0x90, 0x41, 0x4c, 0x00, 0x00, \
  0x08, 0xa4, 0x34, 0x12, 0x00, 0x00, \
  0x90, 0x48, 0x88, 0xc8, 0x00, 0x00, \
  0x90, 0x22, 0x40, 0xa8, 0x00, 0x00, \
  0x09, 0x50, 0x31, 0x10, 0x00, 0x00, \
  0x00, 0x6b, 0x08, 0x0e, 0x00, 0x00, \
  0x20, 0x34, 0xc0, 0x90, 0x00, 0x00, \
  0x14, 0x44, 0x25, 0x04, 0x00, 0x00, \
  0xc2, 0x11, 0x02, 0x82, 0x00, 0x00, \
  0x00, 0xc6, 0x80, 0xc4, 0x00, 0x00, \
  0x65, 0x80, 0x2c, 0x60, 0x00, 0x00, \
  0xe3, 0xd1, 0x2e, 0x00, 0x00, 0x00

#define kMaskRandom31_19 \
  0x62, 0x22, 0xaa, 0xa2, 0x00, 0x00, \
  0x24, 0x44, 0x44, 0x54, 0x00, 0x00, \
  0xc0, 0x50, 0x0b, 0x0a, 0x00, 0x00, \
  0x03, 0x0c, 0x12, 0x94, 0x00, 0x00, \
  0x16, 0x29, 0x08, 0x64, 0x00, 0x00, \
  0x89, 0x01, 0x80, 0x1a, 0x00, 0x00, \
  0x82, 0x90, 0x41, 0x4c, 0x00, 0x00, \
  0x08, 0xa4, 0x34, 0x12, 0x00, 0x00, \
  0x90, 0x48, 0x88, 0xc8, 0x00, 0x00, \
  0xc0, 0xa0, 0x8a, 0xa2, 0x00, 0x00, \
  0x15, 0x56, 0x21, 0x44, 0x00, 0x00, \
  0x74, 0x40, 0x02, 0x4a, 0x00, 0x00, \
  0x00, 0x9c, 0x16, 0x84, 0x00, 0x00, \
  0x01, 0x2d, 0xb0, 0x40, 0x00, 0x00, \
  0x44, 0x93, 0x05, 0x18, 0x00, 0x00, \
  0x88, 0x50, 0x48, 0x94, 0x00, 0x00, \
  0x20, 0xa4, 0x70, 0x30, 0x00, 0x00, \
  0xaa, 0x04, 0x54, 0x4a, 0x00, 0x00, \
  0x02, 0x63, 0x09, 0x24, 0x00, 0x00

#define kMaskRandom31_2 \
  0xec, 0xeb, 0x5d, 0x5c, 0x00, 0x00, \
  0xbb, 0x9c, 0xf2, 0xf2, 0x00, 0x00

#define kMaskRandom31_20 \
  0xc0, 0xa0, 0x8a, 0xa2, 0x00, 0x00, \
  0x15, 0x56, 0x21, 0x44, 0x00, 0x00, \
  0x74, 0x40, 0x02, 0x4a, 0x00, 0x00, \
  0x00, 0x9c, 0x16, 0x84, 0x00, 0x00, \
  0x01, 0x2d, 0xb0, 0x40, 0x00, 0x00, \
  0x44, 0x93, 0x05, 0x18, 0x00, 0x00, \
  0x88, 0x50, 0x48, 0x94, 0x00, 0x00, \
  0x20, 0xa4, 0x70, 0x30, 0x00, 0x00, \
  0xaa, 0x04, 0x54, 0x4a, 0x00, 0x00, \
  0x02, 0x63, 0x09, 0x24, 0x00, 0x00, \
  0x62, 0x22, 0xaa, 0xa2, 0x00, 0x00, \
  0x24, 0x44, 0x44, 0x54, 0x00, 0x00, \
  0xc0, 0x50, 0x0b, 0x0a, 0x00, 0x00, \
  0x03, 0x0c, 0x12, 0x94, 0x00, 0x00, \
  0x16, 0x29, 0x08, 0x64, 0x00, 0x00, \
  0x89, 0x01, 0x80, 0x1a, 0x00, 0x00, \
  0x82, 0x90, 0x41, 0x4c, 0x00, 0x00, \
  0x08, 0xa4, 0x34, 0x12, 0x00, 0x00, \
  0x90, 0x48, 0x88, 0xc8, 0x00, 0x00, \
  0x9a, 0xd4, 0x6a, 0x36, 0x00, 0x00

#define kMaskRandom31_21 \
  0xc0, 0xa0, 0x8a, 0xa2, 0x00, 0x00, \
  0x15, 0x56, 0x21, 0x44, 0x00, 0x00, \
  0x74, 0x40, 0x02, 0x4a, 0x00, 0x00, \
  0x00, 0x9c, 0x16, 0x84, 0x00, 0x00, \
  0x01, 0x2d, 0xb0, 0x40, 0x00, 0x00, \
  0x44, 0x93, 0x05, 0x18, 0x00, 0x00, \
  0x88, 0x50, 0x48, 0x94, 0x00, 0x00, \
  0x20, 0xa4, 0x70, 0x30, 0x00, 0x00, \
  0xaa, 0x04, 0x54, 0x4a, 0x00, 0x00, \
  0x02, 0x63, 0x09, 0x24, 0x00, 0x00, \
  0x62, 0x22, 0xaa, 0xaa, 0x00, 0x00, \
  0xf1, 0x10, 0x54, 0x44, 0x00, 0x00, \
  0x10, 0x0e, 0x62, 0x22, 0x00, 0x00, \
  0x10, 0xb1, 0x06, 0x84, 0x00, 0x00, \
  0x24, 0x24, 0x0d, 0x30, 0x00, 0x00, \
  0x01, 0x12, 0x81, 0xc2, 0x00, 0x00, \
  0x00, 0xc4, 0x58, 0x88, 0x00, 0x00, \
  0x04, 0xa3, 0xb0, 0x50, 0x00, 0x00, \
  0x02, 0x59, 0x25, 0x02, 0x00, 0x00, \
  0x2b, 0x01, 0x08, 0x64, 0x00, 0x00, \
  0x98, 0x40, 0xd0, 0x18, 0x00, 0x00

#define kMaskRandom31_22 \
  0x62, 0x22, 0xaa, 0xaa, 0x00, 0x00, \
  0xf1, 0x10, 0x54, 0x44, 0x00, 0x00, \
  0x10, 0x0e, 0x62, 0x22, 0x00, 0x00, \
  0x10, 0xb1, 0x06, 0x84, 0x00, 0x00, \
  0x24, 0x24, 0x0d, 0x30, 0x00, 0x00, \
  0x01, 0x12, 0x81, 0xc2, 0x00, 0x00, \
  0x00, 0xc4, 0x58, 0x88, 0x00, 0x00, \
  0x04, 0xa3, 0xb0, 0x50, 0x00, 0x00, \
  0x02, 0x59, 0x25, 0x02, 0x00, 0x00, \
  0x2b, 0x01, 0x08, 0x64, 0x00, 0x00, \
  0x98, 0x40, 0xd0, 0x18, 0x00, 0x00, \
  0xc0, 0xa0, 0x8a, 0xa2, 0x00, 0x00, \
  0x15, 0x56, 0x21, 0x44, 0x00, 0x00, \
  0x74, 0x40, 0x02, 0x4a, 0x00, 0x00, \
  0x00, 0x9c, 0x16, 0x84, 0x00, 0x00, \
  0x01, 0x2d, 0xb0, 0x40, 0x00, 0x00, \
  0x44, 0x93, 0x05, 0x18, 0x00, 0x00, \
  0x88, 0x50, 0x48, 0x94, 0x00, 0x00, \
  0x20, 0xa4, 0x70, 0x30, 0x00, 0x00, \
  0xaa, 0x04, 0x54, 0x4a, 0x00, 0x00, \
  0x02, 0x63, 0x09, 0x24, 0x00, 0x00, \
  0x32, 0x23, 0x73, 0x8e, 0x00, 0x00

#define kMaskRandom31_23 \
  0x62, 0x22, 0xaa, 0xaa, 0x00, 0x00, \
  0xf1, 0x10, 0x54, 0x44, 0x00, 0x00, \
  0x10, 0x0e, 0x62, 0x22, 0x00, 0x00, \
  0x10, 0xb1, 0x06, 0x84, 0x00, 0x00, \
  0x24, 0x24, 0x0d, 0x30, 0x00, 0x00, \
  0x01, 0x12, 0x81, 0xc2, 0x00, 0x00, \
  0x00, 0xc4, 0x58, 0x88, 0x00, 0x00, \
  0x04, 0xa3, 0xb0, 0x50, 0x00, 0x00, \
  0x02, 0x59, 0x25, 0x02, 0x00, 0x00, \
  0x2b, 0x01, 0x08, 0x64, 0x00, 0x00, \
  0x98, 0x40, 0xd0, 0x18, 0x00, 0x00, \
  0x88, 0x91, 0x08, 0x62, 0x00, 0x00, \
  0x40, 0x54, 0x31, 0x44, 0x00, 0x00, \
  0x82, 0x62, 0x9c, 0x02, 0x00, 0x00, \
  0x21, 0xa4, 0x89, 0x90, 0x00, 0x00, \
  0x10, 0x64, 0x1d, 0x20, 0x00, 0x00, \
  0x44, 0x0a, 0x41, 0x98, 0x00, 0x00, \
  0x10, 0xc9, 0x26, 0x80, 0x00, 0x00, \
  0x4d, 0x2a, 0x5a, 0x20, 0x00, 0x00, \
  0x38, 0x02, 0x62, 0x88, 0x00, 0x00, \
  0x17, 0x49, 0x80, 0x46, 0x00, 0x00, \
  0x90, 0x84, 0x22, 0x4a, 0x00, 0x00, \
  0x72, 0x15, 0xd1, 0x00, 0x00, 0x00

#define kMaskRandom31_24 \
  0x88, 0x91, 0x08, 0x62, 0x00, 0x00, \
  0x40, 0x54, 0x31, 0x44, 0x00, 0x00, \
  0x82, 0x62, 0x9c, 0x02, 0x00, 0x00, \
  0x21, 0xa4, 0x89, 0x90, 0x00, 0x00, \
  0x10, 0x64, 0x1d, 0x20, 0x00, 0x00, \
  0x44, 0x0a, 0x41, 0x98, 0x00, 0x00, \
  0x10, 0xc9, 0x26, 0x80, 0x00, 0x00, \
  0x4d, 0x2a, 0x5a, 0x20, 0x00, 0x00, \
  0x38, 0x02, 0x62, 0x88, 0x00, 0x00, \
  0x17, 0x49, 0x80, 0x46, 0x00, 0x00, \
  0x90, 0x84, 0x22, 0x4a, 0x00, 0x00, \
  0x72, 0x15, 0xd1, 0x00, 0x00, 0x00, \
  0x62, 0x22, 0xaa, 0xaa, 0x00, 0x00, \
  0xf1, 0x10, 0x54, 0x44, 0x00, 0x00, \
  0x10, 0x0e, 0x62, 0x22, 0x00, 0x00, \
  0x10, 0xb1, 0x06, 0x84, 0x00, 0x00, \
  0x24, 0x24, 0x0d, 0x30, 0x00, 0x00, \
  0x01, 0x12, 0x81, 0xc2, 0x00, 0x00, \
  0x00, 0xc4, 0x58, 0x88, 0x00, 0x00, \
  0x04, 0xa3, 0xb0, 0x50, 0x00, 0x00, \
  0x02, 0x59, 0x25, 0x02, 0x00, 0x00, \
  0x2b, 0x01, 0x08, 0x64, 0x00, 0x00, \
  0x98, 0x40, 0xd0, 0x18, 0x00, 0x00, \
  0xf0, 0xdf, 0x91, 0xb6, 0x00, 0x00

#define kMaskRandom31_25 \
  0x88, 0x91, 0x08, 0x62, 0x00, 0x00, \
  0x40, 0x54, 0x31, 0x44, 0x00, 0x00, \
  0x82, 0x62, 0x9c, 0x02, 0x00, 0x00, \
  0x21, 0xa4, 0x89, 0x90, 0x00, 0x00, \
  0x10, 0x64, 0x1d, 0x20, 0x00, 0x00, \
  0x44, 0x0a, 0x41, 0x98, 0x00, 0x00, \
  0x10, 0xc9, 0x26, 0x80, 0x00, 0x00, \
  0x4d, 0x2a, 0x5a, 0x20, 0x00, 0x00, \
  0x38, 0x02, 0x62, 0x88, 0x00, 0x00, \
  0x17, 0x49, 0x80, 0x46, 0x00, 0x00, \
  0x90, 0x84, 0x22, 0x4a, 0x00, 0x00, \
  0x72, 0x15, 0xd1, 0x00, 0x00, 0x00, \
  0x62, 0xa2, 0x8a, 0x2a, 0x00, 0x00, \
  0x34, 0x44, 0x44, 0x44, 0x00, 0x00, \
  0x40, 0x4b, 0x2c, 0x18, 0x00, 0x00, \
  0xc4, 0x04, 0x18, 0xa0, 0x00, 0x00, \
  0x08, 0x60, 0xc4, 0x08, 0x00, 0x00, \
  0x94, 0x12, 0x92, 0x0c, 0x00, 0x00, \
  0x88, 0xc0, 0x23, 0x04, 0x00, 0x00, \
  0x21, 0x32, 0x24, 0x70, 0x00, 0x00, \
  0xc1, 0x40, 0x80, 0xe2, 0x00, 0x00, \
  0x10, 0x69, 0x51, 0x14, 0x00, 0x00, \
  0x06, 0x90, 0x11, 0x42, 0x00, 0x00, \
  0x59, 0x01, 0x41, 0x80, 0x00, 0x00, \
  0x0a, 0x0d, 0x8a, 0x20, 0x00, 0x00

#define kMaskRandom31_26 \
  0x62, 0xa2, 0x8a, 0x2a, 0x00, 0x00, \
  0x34, 0x44, 0x44, 0x44, 0x00, 0x00, \
  0x40, 0x4b, 0x2c, 0x18, 0x00, 0x00, \
  0xc4, 0x04, 0x18, 0xa0, 0x00, 0x00, \
  0x08, 0x60, 0xc4, 0x08, 0x00, 0x00, \
  0x94, 0x12, 0x92, 0x0c, 0x00, 0x00, \
  0x88, 0xc0, 0x23, 0x04, 0x00, 0x00, \
  0x21, 0x32, 0x24, 0x70, 0x00, 0x00, \
  0xc1, 0x40, 0x80, 0xe2, 0x00, 0x00, \
  0x10, 0x69, 0x51, 0x14, 0x00, 0x00, \
  0x06, 0x90, 0x11, 0x42, 0x00, 0x00, \
  0x59, 0x01, 0x41, 0x80, 0x00, 0x00, \
  0x0a, 0x0d, 0x8a, 0x20, 0x00, 0x00, \
  0x88, 0x91, 0x08, 0x62, 0x00, 0x00, \
  0x40, 0x54, 0x31, 0x44, 0x00, 0x00, \
  0x82, 0x62, 0x9c, 0x02, 0x00, 0x00, \
  0x21, 0xa4, 0x89, 0x90, 0x00, 0x00, \
  0x10, 0x64, 0x1d, 0x20, 0x00, 0x00, \
  0x44, 0x0a, 0x41, 0x98, 0x00, 0x00, \
  0x10, 0xc9, 0x26, 0x80, 0x00, 0x00, \
  0x4d, 0x2a, 0x5a, 0x20, 0x00, 0x00, \
  0x38, 0x02, 0x62, 0x88, 0x00, 0x00, \
  0x17, 0x49, 0x80, 0x46, 0x00, 0x00, \
  0x90, 0x84, 0x22, 0x4a, 0x00, 0x00, \
  0x72, 0x15, 0xd1, 0x00, 0x00, 0x00, \
  0xc5, 0x75, 0x48, 0xba, 0x00, 0x00

#define kMaskRandom31_27 \
  0x62, 0xa2, 0x8a, 0x2a, 0x00, 0x00, \
  0x34, 0x44, 0x44, 0x44, 0x00, 0x00, \
  0x40, 0x4b, 0x2c, 0x18, 0x00, 0x00, \
  0xc4, 0x04, 0x18, 0xa0, 0x00, 0x00, \
  0x08, 0x60, 0xc4, 0x08, 0x00, 0x00, \
  0x94, 0x12, 0x92, 0x0c, 0x00, 0x00, \
  0x88, 0xc0, 0x23, 0x04, 0x00, 0x00, \
  0x21, 0x32, 0x24, 0x70, 0x00, 0x00, \
  0xc1, 0x40, 0x80, 0xe2, 0x00, 0x00, \
  0x10, 0x69, 0x51, 0x14, 0x00, 0x00, \
  0x06, 0x90, 0x11, 0x42, 0x00, 0x00, \
  0x59, 0x01, 0x41, 0x80, 0x00, 0x00, \
  0x0a, 0x0d, 0x8a, 0x20, 0x00, 0x00, \
  0x40, 0x82, 0x8a, 0xa2, 0x00, 0x00, \
  0x15, 0x54, 0x44, 0x14, 0x00, 0x00, \
  0x88, 0x13, 0x09, 0xa0, 0x00, 0x00, \
  0xc0, 0x10, 0x19, 0x14, 0x00, 0x00, \
  0x80, 0xa0, 0x30, 0x0c, 0x00, 0x00, \
  0x01, 0x22, 0x60, 0x06, 0x00, 0x00, \
  0x40, 0x2c, 0xc2, 0x10, 0x00, 0x00, \
  0x22, 0x02, 0x80, 0x22, 0x00, 0x00, \
  0x90, 0x04, 0x20, 0x58, 0x00, 0x00, \
  0x12, 0x40, 0x12, 0xc0, 0x00, 0x00, \
  0x5d, 0x00, 0x01, 0x28, 0x00, 0x00, \
  0x20, 0x54, 0xa4, 0x80, 0x00, 0x00, \
  0x86, 0x09, 0x48, 0x48, 0x00, 0x00, \
  0x28, 0x89, 0x05, 0x10, 0x00, 0x00

#define kMaskRandom31_28 \
  0x40, 0x82, 0x8a, 0xa2, 0x00, 0x00, \
  0x15, 0x54, 0x44, 0x14, 0x00, 0x00, \
  0x88, 0x13, 0x09, 0xa0, 0x00, 0x00, \
  0xc0, 0x10, 0x19, 0x14, 0x00, 0x00, \
  0x80, 0xa0, 0x30, 0x0c, 0x00, 0x00, \
  0x01, 0x22, 0x60, 0x06, 0x00, 0x00, \
  0x40, 0x2c, 0xc2, 0x10, 0x00, 0x00, \
  0x22, 0x02, 0x80, 0x22, 0x00, 0x00, \
  0x90, 0x04, 0x20, 0x58, 0x00, 0x00, \
  0x12, 0x40, 0x12, 0xc0, 0x00, 0x00, \
  0x5d, 0x00, 0x01, 0x28, 0x00, 0x00, \
  0x20, 0x54, 0xa4, 0x80, 0x00, 0x00, \
  0x86, 0x09, 0x48, 0x48, 0x00, 0x00, \
  0x28, 0x89, 0x05, 0x10, 0x00, 0x00, \
  0x62, 0xa2, 0x8a, 0x2a, 0x00, 0x00, \
  0x34, 0x44, 0x44, 0x44, 0x00, 0x00, \
  0x40, 0x4b, 0x2c, 0x18, 0x00, 0x00, \
  0xc4, 0x04, 0x18, 0xa0, 0x00, 0x00, \
  0x08, 0x60, 0xc4, 0x08, 0x00, 0x00, \
  0x94, 0x12, 0x92, 0x0c, 0x00, 0x00, \
  0x88, 0xc0, 0x23, 0x04, 0x00, 0x00, \
  0x21, 0x32, 0x24, 0x70, 0x00, 0x00, \
  0xc1, 0x40, 0x80, 0xe2, 0x00, 0x00, \
  0x10, 0x69, 0x51, 0x14, 0x00, 0x00, \
  0x06, 0x90, 0x11, 0x42, 0x00, 0x00, \
  0x59, 0x01, 0x41, 0x80, 0x00, 0x00, \
  0x0a, 0x0d, 0x8a, 0x20, 0x00, 0x00, \
  0xbc, 0x0d, 0xca, 0x28, 0x00, 0x00

#define kMaskRandom31_29 \
  0x40, 0x82, 0x8a, 0xa2, 0x00, 0x00, \
  0x15, 0x54, 0x44, 0x14, 0x00, 0x00, \
  0x88, 0x13, 0x09, 0xa0, 0x00, 0x00, \
  0xc0, 0x10, 0x19, 0x14, 0x00, 0x00, \
  0x80, 0xa0, 0x30, 0x0c, 0x00, 0x00, \
  0x01, 0x22, 0x60, 0x06, 0x00, 0x00, \
  0x40, 0x2c, 0xc2, 0x10, 0x00, 0x00, \
  0x22, 0x02, 0x80, 0x22, 0x00, 0x00, \
  0x90, 0x04, 0x20, 0x58, 0x00, 0x00, \
  0x12, 0x40, 0x12, 0xc0, 0x00, 0x00, \
  0x5d, 0x00, 0x01, 0x28, 0x00, 0x00, \
  0x20, 0x54, 0xa4, 0x80, 0x00, 0x00, \
  0x86, 0x09, 0x48, 0x48, 0x00, 0x00, \
  0x28, 0x89, 0x05, 0x10, 0x00, 0x00, \
  0x62, 0x22, 0xaa, 0x22, 0x00, 0x00, \
  0x31, 0x10, 0x44, 0x44, 0x00, 0x00, \
  0x58, 0x00, 0x22, 0x22, 0x00, 0x00, \
  0x01, 0x13, 0x00, 0x8a, 0x00, 0x00, \
  0x88, 0x20, 0x40, 0x34, 0x00, 0x00, \
  0x44, 0x02, 0x10, 0xd0, 0x00, 0x00, \
  0x29, 0x04, 0x45, 0x08, 0x00, 0x00, \
  0x82, 0xa0, 0x90, 0x12, 0x00, 0x00, \
  0x0a, 0x1a, 0x0e, 0x02, 0x00, 0x00, \
  0x11, 0xe1, 0x28, 0x40, 0x00, 0x00, \
  0x84, 0x05, 0x04, 0x0c, 0x00, 0x00, \
  0x86, 0x40, 0xc0, 0x90, 0x00, 0x00, \
  0x00, 0x87, 0x13, 0x00, 0x00, 0x00, \
  0x44, 0x48, 0x01, 0x1c, 0x00, 0x00, \
  0x10, 0x98, 0x30, 0x44, 0x00, 0x00

#define kMaskRandom31_3 \
  0xac, 0x93, 0x5a, 0x5a, 0x00, 0x00, \
  0x55, 0x4a, 0xec, 0x6c, 0x00, 0x00, \
  0x43, 0x36, 0x4d, 0xb6, 0x00, 0x00

#define kMaskRandom31_30 \
  0x62, 0x22, 0xaa, 0x22, 0x00, 0x00, \
  0x31, 0x10, 0x44, 0x44, 0x00, 0x00, \
  0x58, 0x00, 0x22, 0x22, 0x00, 0x00, \
  0x01, 0x13, 0x00, 0x8a, 0x00, 0x00, \
  0x88, 0x20, 0x40, 0x34, 0x00, 0x00, \
  0x44, 0x02, 0x10, 0xd0, 0x00, 0x00, \
  0x29, 0x04, 0x45, 0x08, 0x00, 0x00, \
  0x82, 0xa0, 0x90, 0x12, 0x00, 0x00, \
  0x0a, 0x1a, 0x0e, 0x02, 0x00, 0x00, \
  0x11, 0xe1, 0x28, 0x40, 0x00, 0x00, \
  0x84, 0x05, 0x04, 0x0c, 0x00, 0x00, \
  0x86, 0x40, 0xc0, 0x90, 0x00, 0x00, \
  0x00, 0x87, 0x13, 0x00, 0x00, 0x00, \
  0x44, 0x48, 0x01, 0x1c, 0x00, 0x00, \
  0x10, 0x98, 0x30, 0x44, 0x00, 0x00, \
  0x40, 0x82, 0x8a, 0xa2, 0x00, 0x00, \
  0x15, 0x54, 0x44, 0x14, 0x00, 0x00, \
  0x88, 0x13, 0x09, 0xa0, 0x00, 0x00, \
  0xc0, 0x10, 0x19, 0x14, 0x00, 0x00, \
  0x80, 0xa0, 0x30, 0x0c, 0x00, 0x00, \
  0x01, 0x22, 0x60, 0x06, 0x00, 0x00, \
  0x40, 0x2c, 0xc2, 0x10, 0x00, 0x00, \
  0x22, 0x02, 0x80, 0x22, 0x00, 0x00, \
  0x90, 0x04, 0x20, 0x58, 0x00, 0x00, \
  0x12, 0x40, 0x12, 0xc0, 0x00, 0x00, \
  0x5d, 0x00, 0x01, 0x28, 0x00, 0x00, \
  0x20, 0x54, 0xa4, 0x80, 0x00, 0x00, \
  0x86, 0x09, 0x48, 0x48, 0x00, 0x00, \
  0x28, 0x89, 0x05, 0x10, 0x00, 0x00, \
  0xe1, 0x4f, 0xe0, 0x80, 0x00, 0x00

#define kMaskRandom31_31 \
  0x62, 0x22, 0xaa, 0x22, 0x00, 0x00, \
  0x31, 0x10, 0x44, 0x44, 0x00, 0x00, \
  0x58, 0x00, 0x22, 0x22, 0x00, 0x00, \
  0x01, 0x13, 0x00, 0x8a, 0x00, 0x00, \
  0x88, 0x20, 0x40, 0x34, 0x00, 0x00, \
  0x44, 0x02, 0x10, 0xd0, 0x00, 0x00, \
  0x29, 0x04, 0x45, 0x08, 0x00, 0x00, \
  0x82, 0xa0, 0x90, 0x12, 0x00, 0x00, \
  0x0a, 0x1a, 0x0e, 0x02, 0x00, 0x00, \
  0x11, 0xe1, 0x28, 0x40, 0x00, 0x00, \
  0x84, 0x05, 0x04, 0x0c, 0x00, 0x00, \
  0x86, 0x40, 0xc0, 0x90, 0x00, 0x00, \
  0x00, 0x87, 0x13, 0x00, 0x00, 0x00, \
  0x44, 0x48, 0x01, 0x1c, 0x00, 0x00, \
  0x10, 0x98, 0x30, 0x44, 0x00, 0x00, \
  0x62, 0x23, 0x48, 0x20, 0x00, 0x00, \
  0x31, 0x10, 0x02, 0x54, 0x00, 0x00, \
  0x58, 0x00, 0x0c, 0x84, 0x00, 0x00, \
  0x01, 0x12, 0x10, 0xd0, 0x00, 0x00, \
  0x88, 0x21, 0x03, 0x20, 0x00, 0x00, \
  0x44, 0x02, 0x01, 0xe0, 0x00, 0x00, \
  0x29, 0x04, 0xa0, 0x0a, 0x00, 0x00, \
  0x82, 0xa0, 0x40, 0xa2, 0x00, 0x00, \
  0x0a, 0x1a, 0x86, 0x10, 0x00, 0x00, \
  0x11, 0xe0, 0xd1, 0x00, 0x00, 0x00, \
  0x84, 0x05, 0x00, 0x16, 0x00, 0x00, \
  0x86, 0x40, 0x20, 0x98, 0x00, 0x00, \
  0x00, 0x86, 0x24, 0x60, 0x00, 0x00, \
  0x44, 0x48, 0x81, 0x0a, 0x00, 0x00, \
  0x10, 0x98, 0x1c, 0x08, 0x00, 0x00, \
  0x87, 0x74, 0x30, 0x24, 0x00, 0x00

#define kMaskRandom31_4 \
  0x25, 0xaa, 0xaa, 0xaa, 0x00, 0x00, \
  0x95, 0x55, 0x55, 0x54, 0x00, 0x00, \
  0x1a, 0x6a, 0x6a, 0x6a, 0x00, 0x00, \
  0x43, 0xd5, 0x95, 0x94, 0x00, 0x00

#define kMaskRandom31_5 \
  0x64, 0xa2, 0xaa, 0xaa, 0x00, 0x00, \
  0x25, 0x54, 0x54, 0x54, 0x00, 0x00, \
  0x49, 0x68, 0x48, 0x4a, 0x00, 0x00, \
  0x53, 0x91, 0x09, 0x90, 0x00, 0x00, \
  0x8e, 0x30, 0x21, 0x6c, 0x00, 0x00

#define kMaskRandom31_6 \
  0x62, 0x8a, 0xa2, 0xa2, 0x00, 0x00, \
  0x15, 0x54, 0x14, 0x54, 0x00, 0x00, \
  0x4c, 0x47, 0x44, 0x2a, 0x00, 0x00, \
  0x52, 0x95, 0x08, 0x94, 0x00, 0x00, \
  0x23, 0x64, 0x61, 0x24, 0x00, 0x00, \
  0x8a, 0x58, 0x09, 0x58, 0x00, 0x00

#define kMaskRandom31_7 \
  0x62, 0xa2, 0x8a, 0xa2, 0x00, 0x00, \
  0xb1, 0x14, 0x44, 0x54, 0x00, 0x00, \
  0x18, 0x6b, 0x22, 0x22, 0x00, 0x00, \
  0x44, 0xd4, 0x5c, 0x10, 0x00, 0x00, \
  0x13, 0x64, 0x90, 0x68, 0x00, 0x00, \
  0x49, 0x1b, 0x20, 0x52, 0x00, 0x00, \
  0x86, 0x8c, 0x13, 0x0c, 0x00, 0x00

#define kMaskRandom31_8 \
  0x90, 0x22, 0x40, 0xa8, 0x00, 0x00, \
  0x09, 0x50, 0x31, 0x10, 0x00, 0x00, \
  0x00, 0x6b, 0x08, 0x0e, 0x00, 0x00, \
  0x20, 0x34, 0xc0, 0x90, 0x00, 0x00, \
  0x14, 0x44, 0x25, 0x04, 0x00, 0x00, \
  0xc2, 0x11, 0x02, 0x82, 0x00, 0x00, \
  0x00, 0xc6, 0x80, 0xc4, 0x00, 0x00, \
  0x65, 0x80, 0x2c, 0x60, 0x00, 0x00

#define kMaskRandom31_9 \
  0x62, 0x22, 0xaa, 0xa2, 0x00, 0x00, \
  0x24, 0x44, 0x44, 0x54, 0x00, 0x00, \
  0xc0, 0x50, 0x0b, 0x0a, 0x00, 0x00, \
  0x03, 0x0c, 0x12, 0x94, 0x00, 0x00, \
  0x16, 0x29, 0x08, 0x64, 0x00, 0x00, \
  0x89, 0x01, 0x80, 0x1a, 0x00, 0x00, \
  0x82, 0x90, 0x41, 0x4c, 0x00, 0x00, \
  0x08, 0xa4, 0x34, 0x12, 0x00, 0x00, \
  0x90, 0x48, 0x88, 0xc8, 0x00, 0x00

#define kMaskRandom32_1 \
  0xff, 0xff, 0xff, 0xff, 0x00, 0x00

#define kMaskRandom32_10 \
  0x45, 0x51, 0x45, 0x51, 0x00, 0x00, \
  0x10, 0xa2, 0x10, 0xa2, 0x00, 0x00, \
  0x01, 0x25, 0x01, 0x25, 0x00, 0x00, \
  0x0b, 0x42, 0x0b, 0x42, 0x00, 0x00, \
  0xd8, 0x20, 0xd8, 0x20, 0x00, 0x00, \
  0x82, 0x8c, 0x82, 0x8c, 0x00, 0x00, \
  0x24, 0x4a, 0x24, 0x4a, 0x00, 0x00, \
  0x38, 0x18, 0x38, 0x18, 0x00, 0x00, \
  0x2a, 0x25, 0x2a, 0x25, 0x00, 0x00, \
  0x84, 0x92, 0x84, 0x92, 0x00, 0x00

#define kMaskRandom32_11 \
  0x55, 0x55, 0x55, 0x55, 0x00, 0x00, \
  0x2a, 0x22, 0x2a, 0x22, 0x00, 0x00, \
  0x31, 0x11, 0x31, 0x11, 0x00, 0x00, \
  0x83, 0x42, 0x83, 0x42, 0x00, 0x00, \
  0x06, 0x98, 0x06, 0x98, 0x00, 0x00, \
  0x40, 0xe1, 0x40, 0xe1, 0x00, 0x00, \
  0x2c, 0x44, 0x2c, 0x44, 0x00, 0x00, \
  0xd8, 0x28, 0xd8, 0x28, 0x00, 0x00, \
  0x92, 0x81, 0x92, 0x81, 0x00, 0x00, \
  0x84, 0x32, 0x84, 0x32, 0x00, 0x00, \
  0x68, 0x0c, 0x68, 0x0c, 0x00, 0x00

#define kMaskRandom32_12 \
  0x84, 0x31, 0x84, 0x31, 0x00, 0x00, \
  0x18, 0xa2, 0x18, 0xa2, 0x00, 0x00, \
  0x4e, 0x01, 0x4e, 0x01, 0x00, 0x00, \
  0x44, 0xc8, 0x44, 0xc8, 0x00, 0x00, \
  0x0e, 0x90, 0x0e, 0x90, 0x00, 0x00, \
  0x20, 0xcc, 0x20, 0xcc, 0x00, 0x00, \
  0x93, 0x40, 0x93, 0x40, 0x00, 0x00, \
  0x2d, 0x10, 0x2d, 0x10, 0x00, 0x00, \
  0x31, 0x44, 0x31, 0x44, 0x00, 0x00, \
  0xc0, 0x23, 0xc0, 0x23, 0x00, 0x00, \
  0x11, 0x25, 0x11, 0x25, 0x00, 0x00, \
  0xe8, 0x80, 0xe8, 0x80, 0x00, 0x00

#define kMaskRandom32_13 \
  0x45, 0x15, 0x45, 0x15, 0x00, 0x00, \
  0x22, 0x22, 0x22, 0x22, 0x00, 0x00, \
  0x96, 0x0c, 0x96, 0x0c, 0x00, 0x00, \
  0x0c, 0x50, 0x0c, 0x50, 0x00, 0x00, \
  0x62, 0x04, 0x62, 0x04, 0x00, 0x00, \
  0x49, 0x06, 0x49, 0x06, 0x00, 0x00, \
  0x11, 0x82, 0x11, 0x82, 0x00, 0x00, \
  0x12, 0x38, 0x12, 0x38, 0x00, 0x00, \
  0x40, 0x71, 0x40, 0x71, 0x00, 0x00, \
  0xa8, 0x8a, 0xa8, 0x8a, 0x00, 0x00, \
  0x08, 0xa1, 0x08, 0xa1, 0x00, 0x00, \
  0xa0, 0xc0, 0xa0, 0xc0, 0x00, 0x00, \
  0xc5, 0x10, 0xc5, 0x10, 0x00, 0x00

#define kMaskRandom32_14 \
  0x45, 0x51, 0x45, 0x51, 0x00, 0x00, \
  0x22, 0x0a, 0x22, 0x0a, 0x00, 0x00, \
  0x84, 0xd0, 0x84, 0xd0, 0x00, 0x00, \
  0x0c, 0x8a, 0x0c, 0x8a, 0x00, 0x00, \
  0x18, 0x06, 0x18, 0x06, 0x00, 0x00, \
  0x30, 0x03, 0x30, 0x03, 0x00, 0x00, \
  0x61, 0x08, 0x61, 0x08, 0x00, 0x00, \
  0x40, 0x11, 0x40, 0x11, 0x00, 0x00, \
  0x10, 0x2c, 0x10, 0x2c, 0x00, 0x00, \
  0x09, 0x60, 0x09, 0x60, 0x00, 0x00, \
  0x00, 0x94, 0x00, 0x94, 0x00, 0x00, \
  0x52, 0x40, 0x52, 0x40, 0x00, 0x00, \
  0xa4, 0x24, 0xa4, 0x24, 0x00, 0x00, \
  0x82, 0x88, 0x82, 0x88, 0x00, 0x00

#define kMaskRandom32_15 \
  0x55, 0x11, 0x55, 0x11, 0x00, 0x00, \
  0x22, 0x22, 0x22, 0x22, 0x00, 0x00, \
  0x11, 0x11, 0x11, 0x11, 0x00, 0x00, \
  0x80, 0x45, 0x80, 0x45, 0x00, 0x00, \
  0x20, 0x1a, 0x20, 0x1a, 0x00, 0x00, \
  0x08, 0x68, 0x08, 0x68, 0x00, 0x00, \
  0x22, 0x84, 0x22, 0x84, 0x00, 0x00, \
  0x48, 0x09, 0x48, 0x09, 0x00, 0x00, \
  0x07, 0x01, 0x07, 0x01, 0x00, 0x00, \
  0x94, 0x20, 0x94, 0x20, 0x00, 0x00, \
  0x82, 0x06, 0x82, 0x06, 0x00, 0x00, \
  0x60, 0x48, 0x60, 0x48, 0x00, 0x00, \
  0x89, 0x80, 0x89, 0x80, 0x00, 0x00, \
  0x00, 0x8e, 0x00, 0x8e, 0x00, 0x00, \
  0x18, 0x22, 0x18, 0x22, 0x00, 0x00

#define kMaskRandom32_16 \
  0xa4, 0x10, 0xa4, 0x10, 0x00, 0x00, \
  0x01, 0x2a, 0x01, 0x2a, 0x00, 0x00, \
  0x06, 0x42, 0x06, 0x42, 0x00, 0x00, \
  0x08, 0x68, 0x08, 0x68, 0x00, 0x00, \
  0x81, 0x90, 0x81, 0x90, 0x00, 0x00, \
  0x00, 0xf0, 0x00, 0xf0, 0x00, 0x00, \
  0x50, 0x05, 0x50, 0x05, 0x00, 0x00, \
  0x20, 0x51, 0x20, 0x51, 0x00, 0x00, \
  0x43, 0x08, 0x43, 0x08, 0x00, 0x00, \
  0x68, 0x80, 0x68, 0x80, 0x00, 0x00, \
  0x80, 0x0b, 0x80, 0x0b, 0x00, 0x00, \
  0x10, 0x4c, 0x10, 0x4c, 0x00, 0x00, \
  0x12, 0x30, 0x12, 0x30, 0x00, 0x00, \
  0x40, 0x85, 0x40, 0x85, 0x00, 0x00, \
  0x0e, 0x04, 0x0e, 0x04, 0x00, 0x00, \
  0x18, 0x12, 0x18, 0x12, 0x00, 0x00

#define kMaskRandom32_17 \
  0x20, 0x54, 0x20, 0x54, 0x00, 0x00, \
  0x18, 0x88, 0x18, 0x88, 0x00, 0x00, \
  0x84, 0x07, 0x84, 0x07, 0x00, 0x00, \
  0x60, 0x48, 0x60, 0x48, 0x00, 0x00, \
  0x12, 0x82, 0x12, 0x82, 0x00, 0x00, \
  0x81, 0x41, 0x81, 0x41, 0x00, 0x00, \
  0x40, 0x62, 0x40, 0x62, 0x00, 0x00, \
  0x16, 0x30, 0x16, 0x30, 0x00, 0x00, \
  0x55, 0x51, 0x55, 0x51, 0x00, 0x00, \
  0x22, 0x2a, 0x22, 0x2a, 0x00, 0x00, \
  0x05, 0x85, 0x05, 0x85, 0x00, 0x00, \
  0x09, 0x4a, 0x09, 0x4a, 0x00, 0x00, \
  0x84, 0x32, 0x84, 0x32, 0x00, 0x00, \
  0xc0, 0x0d, 0xc0, 0x0d, 0x00, 0x00, \
  0x20, 0xa6, 0x20, 0xa6, 0x00, 0x00, \
  0x1a, 0x09, 0x1a, 0x09, 0x00, 0x00, \
  0x44, 0x64, 0x44, 0x64, 0x00, 0x00

#define kMaskRandom32_18 \
  0x55, 0x51, 0x55, 0x51, 0x00, 0x00, \
  0x22, 0x2a, 0x22, 0x2a, 0x00, 0x00, \
  0x05, 0x85, 0x05, 0x85, 0x00, 0x00, \
  0x09, 0x4a, 0x09, 0x4a, 0x00, 0x00, \
  0x84, 0x32, 0x84, 0x32, 0x00, 0x00, \
  0xc0, 0x0d, 0xc0, 0x0d, 0x00, 0x00, \
  0x20, 0xa6, 0x20, 0xa6, 0x00, 0x00, \
  0x1a, 0x09, 0x1a, 0x09, 0x00, 0x00, \
  0x44, 0x64, 0x44, 0x64, 0x00, 0x00, \
  0x20, 0x54, 0x20, 0x54, 0x00, 0x00, \
  0x18, 0x88, 0x18, 0x88, 0x00, 0x00, \
  0x84, 0x07, 0x84, 0x07, 0x00, 0x00, \
  0x60, 0x48, 0x60, 0x48, 0x00, 0x00, \
  0x12, 0x82, 0x12, 0x82, 0x00, 0x00, \
  0x81, 0x41, 0x81, 0x41, 0x00, 0x00, \
  0x40, 0x62, 0x40, 0x62, 0x00, 0x00, \
  0x16, 0x30, 0x16, 0x30, 0x00, 0x00, \
  0x1e, 0xb2, 0xd8, 0x53, 0x00, 0x00

#define kMaskRandom32_19 \
  0x55, 0x51, 0x55, 0x51, 0x00, 0x00, \
  0x22, 0x2a, 0x22, 0x2a, 0x00, 0x00, \
  0x05, 0x85, 0x05, 0x85, 0x00, 0x00, \
  0x09, 0x4a, 0x09, 0x4a, 0x00, 0x00, \
  0x84, 0x32, 0x84, 0x32, 0x00, 0x00, \
  0xc0, 0x0d, 0xc0, 0x0d, 0x00, 0x00, \
  0x20, 0xa6, 0x20, 0xa6, 0x00, 0x00, \
  0x1a, 0x09, 0x1a, 0x09, 0x00, 0x00, \
  0x44, 0x64, 0x44, 0x64, 0x00, 0x00, \
  0x45, 0x51, 0x45, 0x51, 0x00, 0x00, \
  0x10, 0xa2, 0x10, 0xa2, 0x00, 0x00, \
  0x01, 0x25, 0x01, 0x25, 0x00, 0x00, \
  0x0b, 0x42, 0x0b, 0x42, 0x00, 0x00, \
  0xd8, 0x20, 0xd8, 0x20, 0x00, 0x00, \
  0x82, 0x8c, 0x82, 0x8c, 0x00, 0x00, \
  0x24, 0x4a, 0x24, 0x4a, 0x00, 0x00, \
  0x38, 0x18, 0x38, 0x18, 0x00, 0x00, \
  0x2a, 0x25, 0x2a, 0x25, 0x00, 0x00, \
  0x84, 0x92, 0x84, 0x92, 0x00, 0x00

#define kMaskRandom32_2 \
  0xae, 0xae, 0xae, 0xae, 0x00, 0x00, \
  0x79, 0x79, 0x79, 0x79, 0x00, 0x00

#define kMaskRandom32_20 \
  0x45, 0x51, 0x45, 0x51, 0x00, 0x00, \
  0x10, 0xa2, 0x10, 0xa2, 0x00, 0x00, \
  0x01, 0x25, 0x01, 0x25, 0x00, 0x00, \
  0x0b, 0x42, 0x0b, 0x42, 0x00, 0x00, \
  0xd8, 0x20, 0xd8, 0x20, 0x00, 0x00, \
  0x82, 0x8c, 0x82, 0x8c, 0x00, 0x00, \
  0x24, 0x4a, 0x24, 0x4a, 0x00, 0x00, \
  0x38, 0x18, 0x38, 0x18, 0x00, 0x00, \
  0x2a, 0x25, 0x2a, 0x25, 0x00, 0x00, \
  0x84, 0x92, 0x84, 0x92, 0x00, 0x00, \
  0x55, 0x51, 0x55, 0x51, 0x00, 0x00, \
  0x22, 0x2a, 0x22, 0x2a, 0x00, 0x00, \
  0x05, 0x85, 0x05, 0x85, 0x00, 0x00, \
  0x09, 0x4a, 0x09, 0x4a, 0x00, 0x00, \
  0x84, 0x32, 0x84, 0x32, 0x00, 0x00, \
  0xc0, 0x0d, 0xc0, 0x0d, 0x00, 0x00, \
  0x20, 0xa6, 0x20, 0xa6, 0x00, 0x00, \
  0x1a, 0x09, 0x1a, 0x09, 0x00, 0x00, \
  0x44, 0x64, 0x44, 0x64, 0x00, 0x00, \
  0x96, 0xd3, 0xf6, 0xac, 0x00, 0x00

#define kMaskRandom32_21 \
  0x45, 0x51, 0x45, 0x51, 0x00, 0x00, \
  0x10, 0xa2, 0x10, 0xa2, 0x00, 0x00, \
  0x01, 0x25, 0x01, 0x25, 0x00, 0x00, \
  0x0b, 0x42, 0x0b, 0x42, 0x00, 0x00, \
  0xd8, 0x20, 0xd8, 0x20, 0x00, 0x00, \
  0x82, 0x8c, 0x82, 0x8c, 0x00, 0x00, \
  0x24, 0x4a, 0x24, 0x4a, 0x00, 0x00, \
  0x38, 0x18, 0x38, 0x18, 0x00, 0x00, \
  0x2a, 0x25, 0x2a, 0x25, 0x00, 0x00, \
  0x84, 0x92, 0x84, 0x92, 0x00, 0x00, \
  0x55, 0x55, 0x55, 0x55, 0x00, 0x00, \
  0x2a, 0x22, 0x2a, 0x22, 0x00, 0x00, \
  0x31, 0x11, 0x31, 0x11, 0x00, 0x00, \
  0x83, 0x42, 0x83, 0x42, 0x00, 0x00, \
  0x06, 0x98, 0x06, 0x98, 0x00, 0x00, \
  0x40, 0xe1, 0x40, 0xe1, 0x00, 0x00, \
  0x2c, 0x44, 0x2c, 0x44, 0x00, 0x00, \
  0xd8, 0x28, 0xd8, 0x28, 0x00, 0x00, \
  0x92, 0x81, 0x92, 0x81, 0x00, 0x00, \
  0x84, 0x32, 0x84, 0x32, 0x00, 0x00, \
  0x68, 0x0c, 0x68, 0x0c, 0x00, 0x00

#define kMaskRandom32_22 \
  0x55, 0x55, 0x55, 0x55, 0x00, 0x00, \
  0x2a, 0x22, 0x2a, 0x22, 0x00, 0x00, \
  0x31, 0x11, 0x31, 0x11, 0x00, 0x00, \
  0x83, 0x42, 0x83, 0x42, 0x00, 0x00, \
  0x06, 0x98, 0x06, 0x98, 0x00, 0x00, \
  0x40, 0xe1, 0x40, 0xe1, 0x00, 0x00, \
  0x2c, 0x44, 0x2c, 0x44, 0x00, 0x00, \
  0xd8, 0x28, 0xd8, 0x28, 0x00, 0x00, \
  0x92, 0x81, 0x92, 0x81, 0x00, 0x00, \
  0x84, 0x32, 0x84, 0x32, 0x00, 0x00, \
  0x68, 0x0c, 0x68, 0x0c, 0x00, 0x00, \
  0x45, 0x51, 0x45, 0x51, 0x00, 0x00, \
  0x10, 0xa2, 0x10, 0xa2, 0x00, 0x00, \
  0x01, 0x25, 0x01, 0x25, 0x00, 0x00, \
  0x0b, 0x42, 0x0b, 0x42, 0x00, 0x00, \
  0xd8, 0x20, 0xd8, 0x20, 0x00, 0x00, \
  0x82, 0x8c, 0x82, 0x8c, 0x00, 0x00, \
  0x24, 0x4a, 0x24, 0x4a, 0x00, 0x00, \
  0x38, 0x18, 0x38, 0x18, 0x00, 0x00, \
  0x2a, 0x25, 0x2a, 0x25, 0x00, 0x00, \
  0x84, 0x92, 0x84, 0x92, 0x00, 0x00, \
  0xeb, 0xb2, 0x22, 0x89, 0x00, 0x00

#define kMaskRandom32_23 \
  0x55, 0x55, 0x55, 0x55, 0x00, 0x00, \
  0x2a, 0x22, 0x2a, 0x22, 0x00, 0x00, \
  0x31, 0x11, 0x31, 0x11, 0x00, 0x00, \
  0x83, 0x42, 0x83, 0x42, 0x00, 0x00, \
  0x06, 0x98, 0x06, 0x98, 0x00, 0x00, \
  0x40, 0xe1, 0x40, 0xe1, 0x00, 0x00, \
  0x2c, 0x44, 0x2c, 0x44, 0x00, 0x00, \
  0xd8, 0x28, 0xd8, 0x28, 0x00, 0x00, \
  0x92, 0x81, 0x92, 0x81, 0x00, 0x00, \
  0x84, 0x32, 0x84, 0x32, 0x00, 0x00, \
  0x68, 0x0c, 0x68, 0x0c, 0x00, 0x00, \
  0x84, 0x31, 0x84, 0x31, 0x00, 0x00, \
  0x18, 0xa2, 0x18, 0xa2, 0x00, 0x00, \
  0x4e, 0x01, 0x4e, 0x01, 0x00, 0x00, \
  0x44, 0xc8, 0x44, 0xc8, 0x00, 0x00, \
  0x0e, 0x90, 0x0e, 0x90, 0x00, 0x00, \
  0x20, 0xcc, 0x20, 0xcc, 0x00, 0x00, \
  0x93, 0x40, 0x93, 0x40, 0x00, 0x00, \
  0x2d, 0x10, 0x2d, 0x10, 0x00, 0x00, \
  0x31, 0x44, 0x31, 0x44, 0x00, 0x00, \
  0xc0, 0x23, 0xc0, 0x23, 0x00, 0x00, \
  0x11, 0x25, 0x11, 0x25, 0x00, 0x00, \
  0xe8, 0x80, 0xe8, 0x80, 0x00, 0x00

#define kMaskRandom32_24 \
  0x84, 0x31, 0x84, 0x31, 0x00, 0x00, \
  0x18, 0xa2, 0x18, 0xa2, 0x00, 0x00, \
  0x4e, 0x01, 0x4e, 0x01, 0x00, 0x00, \
  0x44, 0xc8, 0x44, 0xc8, 0x00, 0x00, \
  0x0e, 0x90, 0x0e, 0x90, 0x00, 0x00, \
  0x20, 0xcc, 0x20, 0xcc, 0x00, 0x00, \
  0x93, 0x40, 0x93, 0x40, 0x00, 0x00, \
  0x2d, 0x10, 0x2d, 0x10, 0x00, 0x00, \
  0x31, 0x44, 0x31, 0x44, 0x00, 0x00, \
  0xc0, 0x23, 0xc0, 0x23, 0x00, 0x00, \
  0x11, 0x25, 0x11, 0x25, 0x00, 0x00, \
  0xe8, 0x80, 0xe8, 0x80, 0x00, 0x00, \
  0x55, 0x55, 0x55, 0x55, 0x00, 0x00, \
  0x2a, 0x22, 0x2a, 0x22, 0x00, 0x00, \
  0x31, 0x11, 0x31, 0x11, 0x00, 0x00, \
  0x83, 0x42, 0x83, 0x42, 0x00, 0x00, \
  0x06, 0x98, 0x06, 0x98, 0x00, 0x00, \
  0x40, 0xe1, 0x40, 0xe1, 0x00, 0x00, \
  0x2c, 0x44, 0x2c, 0x44, 0x00, 0x00, \
  0xd8, 0x28, 0xd8, 0x28, 0x00, 0x00, \
  0x92, 0x81, 0x92, 0x81, 0x00, 0x00, \
  0x84, 0x32, 0x84, 0x32, 0x00, 0x00, \
  0x68, 0x0c, 0x68, 0x0c, 0x00, 0x00, \
  0xf3, 0x5a, 0x2f, 0x5d, 0x00, 0x00

#define kMaskRandom32_25 \
  0x84, 0x31, 0x84, 0x31, 0x00, 0x00, \
  0x18, 0xa2, 0x18, 0xa2, 0x00, 0x00, \
  0x4e, 0x01, 0x4e, 0x01, 0x00, 0x00, \
  0x44, 0xc8, 0x44, 0xc8, 0x00, 0x00, \
  0x0e, 0x90, 0x0e, 0x90, 0x00, 0x00, \
  0x20, 0xcc, 0x20, 0xcc, 0x00, 0x00, \
  0x93, 0x40, 0x93, 0x40, 0x00, 0x00, \
  0x2d, 0x10, 0x2d, 0x10, 0x00, 0x00, \
  0x31, 0x44, 0x31, 0x44, 0x00, 0x00, \
  0xc0, 0x23, 0xc0, 0x23, 0x00, 0x00, \
  0x11, 0x25, 0x11, 0x25, 0x00, 0x00, \
  0xe8, 0x80, 0xe8, 0x80, 0x00, 0x00, \
  0x45, 0x15, 0x45, 0x15, 0x00, 0x00, \
  0x22, 0x22, 0x22, 0x22, 0x00, 0x00, \
  0x96, 0x0c, 0x96, 0x0c, 0x00, 0x00, \
  0x0c, 0x50, 0x0c, 0x50, 0x00, 0x00, \
  0x62, 0x04, 0x62, 0x04, 0x00, 0x00, \
  0x49, 0x06, 0x49, 0x06, 0x00, 0x00, \
  0x11, 0x82, 0x11, 0x82, 0x00, 0x00, \
  0x12, 0x38, 0x12, 0x38, 0x00, 0x00, \
  0x40, 0x71, 0x40, 0x71, 0x00, 0x00, \
  0xa8, 0x8a, 0xa8, 0x8a, 0x00, 0x00, \
  0x08, 0xa1, 0x08, 0xa1, 0x00, 0x00, \
  0xa0, 0xc0, 0xa0, 0xc0, 0x00, 0x00, \
  0xc5, 0x10, 0xc5, 0x10, 0x00, 0x00

#define kMaskRandom32_26 \
  0x45, 0x15, 0x45, 0x15, 0x00, 0x00, \
  0x22, 0x22, 0x22, 0x22, 0x00, 0x00, \
  0x96, 0x0c, 0x96, 0x0c, 0x00, 0x00, \
  0x0c, 0x50, 0x0c, 0x50, 0x00, 0x00, \
  0x62, 0x04, 0x62, 0x04, 0x00, 0x00, \
  0x49, 0x06, 0x49, 0x06, 0x00, 0x00, \
  0x11, 0x82, 0x11, 0x82, 0x00, 0x00, \
  0x12, 0x38, 0x12, 0x38, 0x00, 0x00, \
  0x40, 0x71, 0x40, 0x71, 0x00, 0x00, \
  0xa8, 0x8a, 0xa8, 0x8a, 0x00, 0x00, \
  0x08, 0xa1, 0x08, 0xa1, 0x00, 0x00, \
  0xa0, 0xc0, 0xa0, 0xc0, 0x00, 0x00, \
  0xc5, 0x10, 0xc5, 0x10, 0x00, 0x00, \
  0x84, 0x31, 0x84, 0x31, 0x00, 0x00, \
  0x18, 0xa2, 0x18, 0xa2, 0x00, 0x00, \
  0x4e, 0x01, 0x4e, 0x01, 0x00, 0x00, \
  0x44, 0xc8, 0x44, 0xc8, 0x00, 0x00, \
  0x0e, 0x90, 0x0e, 0x90, 0x00, 0x00, \
  0x20, 0xcc, 0x20, 0xcc, 0x00, 0x00, \
  0x93, 0x40, 0x93, 0x40, 0x00, 0x00, \
  0x2d, 0x10, 0x2d, 0x10, 0x00, 0x00, \
  0x31, 0x44, 0x31, 0x44, 0x00, 0x00, \
  0xc0, 0x23, 0xc0, 0x23, 0x00, 0x00, \
  0x11, 0x25, 0x11, 0x25, 0x00, 0x00, \
  0xe8, 0x80, 0xe8, 0x80, 0x00, 0x00, \
  0x52, 0x15, 0x62, 0x0a, 0x00, 0x00

#define kMaskRandom32_27 \
  0x45, 0x15, 0x45, 0x15, 0x00, 0x00, \
  0x22, 0x22, 0x22, 0x22, 0x00, 0x00, \
  0x96, 0x0c, 0x96, 0x0c, 0x00, 0x00, \
  0x0c, 0x50, 0x0c, 0x50, 0x00, 0x00, \
  0x62, 0x04, 0x62, 0x04, 0x00, 0x00, \
  0x49, 0x06, 0x49, 0x06, 0x00, 0x00, \
  0x11, 0x82, 0x11, 0x82, 0x00, 0x00, \
  0x12, 0x38, 0x12, 0x38, 0x00, 0x00, \
  0x40, 0x71, 0x40, 0x71, 0x00, 0x00, \
  0xa8, 0x8a, 0xa8, 0x8a, 0x00, 0x00, \
  0x08, 0xa1, 0x08, 0xa1, 0x00, 0x00, \
  0xa0, 0xc0, 0xa0, 0xc0, 0x00, 0x00, \
  0xc5, 0x10, 0xc5, 0x10, 0x00, 0x00, \
  0x45, 0x51, 0x45, 0x51, 0x00, 0x00, \
  0x22, 0x0a, 0x22, 0x0a, 0x00, 0x00, \
  0x84, 0xd0, 0x84, 0xd0, 0x00, 0x00, \
  0x0c, 0x8a, 0x0c, 0x8a, 0x00, 0x00, \
  0x18, 0x06, 0x18, 0x06, 0x00, 0x00, \
  0x30, 0x03, 0x30, 0x03, 0x00, 0x00, \
  0x61, 0x08, 0x61, 0x08, 0x00, 0x00, \
  0x40, 0x11, 0x40, 0x11, 0x00, 0x00, \
  0x10, 0x2c, 0x10, 0x2c, 0x00, 0x00, \
  0x09, 0x60, 0x09, 0x60, 0x00, 0x00, \
  0x00, 0x94, 0x00, 0x94, 0x00, 0x00, \
  0x52, 0x40, 0x52, 0x40, 0x00, 0x00, \
  0xa4, 0x24, 0xa4, 0x24, 0x00, 0x00, \
  0x82, 0x88, 0x82, 0x88, 0x00, 0x00

#define kMaskRandom32_28 \
  0x45, 0x51, 0x45, 0x51, 0x00, 0x00, \
  0x22, 0x0a, 0x22, 0x0a, 0x00, 0x00, \
  0x84, 0xd0, 0x84, 0xd0, 0x00, 0x00, \
  0x0c, 0x8a, 0x0c, 0x8a, 0x00, 0x00, \
  0x18, 0x06, 0x18, 0x06, 0x00, 0x00, \
  0x30, 0x03, 0x30, 0x03, 0x00, 0x00, \
  0x61, 0x08, 0x61, 0x08, 0x00, 0x00, \
  0x40, 0x11, 0x40, 0x11, 0x00, 0x00, \
  0x10, 0x2c, 0x10, 0x2c, 0x00, 0x00, \
  0x09, 0x60, 0x09, 0x60, 0x00, 0x00, \
  0x00, 0x94, 0x00, 0x94, 0x00, 0x00, \
  0x52, 0x40, 0x52, 0x40, 0x00, 0x00, \
  0xa4, 0x24, 0xa4, 0x24, 0x00, 0x00, \
  0x82, 0x88, 0x82, 0x88, 0x00, 0x00, \
  0x45, 0x15, 0x45, 0x15, 0x00, 0x00, \
  0x22, 0x22, 0x22, 0x22, 0x00, 0x00, \
  0x96, 0x0c, 0x96, 0x0c, 0x00, 0x00, \
  0x0c, 0x50, 0x0c, 0x50, 0x00, 0x00, \
  0x62, 0x04, 0x62, 0x04, 0x00, 0x00, \
  0x49, 0x06, 0x49, 0x06, 0x00, 0x00, \
  0x11, 0x82, 0x11, 0x82, 0x00, 0x00, \
  0x12, 0x38, 0x12, 0x38, 0x00, 0x00, \
  0x40, 0x71, 0x40, 0x71, 0x00, 0x00, \
  0xa8, 0x8a, 0xa8, 0x8a, 0x00, 0x00, \
  0x08, 0xa1, 0x08, 0xa1, 0x00, 0x00, \
  0xa0, 0xc0, 0xa0, 0xc0, 0x00, 0x00, \
  0xc5, 0x10, 0xc5, 0x10, 0x00, 0x00, \
  0x7f, 0xe2, 0xbc, 0x01, 0x00, 0x00

#define kMaskRandom32_29 \
  0x45, 0x51, 0x45, 0x51, 0x00, 0x00, \
  0x22, 0x0a, 0x22, 0x0a, 0x00, 0x00, \
  0x84, 0xd0, 0x84, 0xd0, 0x00, 0x00, \
  0x0c, 0x8a, 0x0c, 0x8a, 0x00, 0x00, \
  0x18, 0x06, 0x18, 0x06, 0x00, 0x00, \
  0x30, 0x03, 0x30, 0x03, 0x00, 0x00, \
  0x61, 0x08, 0x61, 0x08, 0x00, 0x00, \
  0x40, 0x11, 0x40, 0x11, 0x00, 0x00, \
  0x10, 0x2c, 0x10, 0x2c, 0x00, 0x00, \
  0x09, 0x60, 0x09, 0x60, 0x00, 0x00, \
  0x00, 0x94, 0x00, 0x94, 0x00, 0x00, \
  0x52, 0x40, 0x52, 0x40, 0x00, 0x00, \
  0xa4, 0x24, 0xa4, 0x24, 0x00, 0x00, \
  0x82, 0x88, 0x82, 0x88, 0x00, 0x00, \
  0x55, 0x11, 0x55, 0x11, 0x00, 0x00, \
  0x22, 0x22, 0x22, 0x22, 0x00, 0x00, \
  0x11, 0x11, 0x11, 0x11, 0x00, 0x00, \
  0x80, 0x45, 0x80, 0x45, 0x00, 0x00, \
  0x20, 0x1a, 0x20, 0x1a, 0x00, 0x00, \
  0x08, 0x68, 0x08, 0x68, 0x00, 0x00, \
  0x22, 0x84, 0x22, 0x84, 0x00, 0x00, \
  0x48, 0x09, 0x48, 0x09, 0x00, 0x00, \
  0x07, 0x01, 0x07, 0x01, 0x00, 0x00, \
  0x94, 0x20, 0x94, 0x20, 0x00, 0x00, \
  0x82, 0x06, 0x82, 0x06, 0x00, 0x00, \
  0x60, 0x48, 0x60, 0x48, 0x00, 0x00, \
  0x89, 0x80, 0x89, 0x80, 0x00, 0x00, \
  0x00, 0x8e, 0x00, 0x8e, 0x00, 0x00, \
  0x18, 0x22, 0x18, 0x22, 0x00, 0x00

#define kMaskRandom32_3 \
  0xad, 0x2d, 0xad, 0x2d, 0x00, 0x00, \
  0x76, 0x36, 0x76, 0x36, 0x00, 0x00, \
  0x26, 0xdb, 0x26, 0xdb, 0x00, 0x00

#define kMaskRandom32_30 \
  0x55, 0x11, 0x55, 0x11, 0x00, 0x00, \
  0x22, 0x22, 0x22, 0x22, 0x00, 0x00, \
  0x11, 0x11, 0x11, 0x11, 0x00, 0x00, \
  0x80, 0x45, 0x80, 0x45, 0x00, 0x00, \
  0x20, 0x1a, 0x20, 0x1a, 0x00, 0x00, \
  0x08, 0x68, 0x08, 0x68, 0x00, 0x00, \
  0x22, 0x84, 0x22, 0x84, 0x00, 0x00, \
  0x48, 0x09, 0x48, 0x09, 0x00, 0x00, \
  0x07, 0x01, 0x07, 0x01, 0x00, 0x00, \
  0x94, 0x20, 0x94, 0x20, 0x00, 0x00, \
  0x82, 0x06, 0x82, 0x06, 0x00, 0x00, \
  0x60, 0x48, 0x60, 0x48, 0x00, 0x00, \
  0x89, 0x80, 0x89, 0x80, 0x00, 0x00, \
  0x00, 0x8e, 0x00, 0x8e, 0x00, 0x00, \
  0x18, 0x22, 0x18, 0x22, 0x00, 0x00, \
  0x45, 0x51, 0x45, 0x51, 0x00, 0x00, \
  0x22, 0x0a, 0x22, 0x0a, 0x00, 0x00, \
  0x84, 0xd0, 0x84, 0xd0, 0x00, 0x00, \
  0x0c, 0x8a, 0x0c, 0x8a, 0x00, 0x00, \
  0x18, 0x06, 0x18, 0x06, 0x00, 0x00, \
  0x30, 0x03, 0x30, 0x03, 0x00, 0x00, \
  0x61, 0x08, 0x61, 0x08, 0x00, 0x00, \
  0x40, 0x11, 0x40, 0x11, 0x00, 0x00, \
  0x10, 0x2c, 0x10, 0x2c, 0x00, 0x00, \
  0x09, 0x60, 0x09, 0x60, 0x00, 0x00, \
  0x00, 0x94, 0x00, 0x94, 0x00, 0x00, \
  0x52, 0x40, 0x52, 0x40, 0x00, 0x00, \
  0xa4, 0x24, 0xa4, 0x24, 0x00, 0x00, \
  0x82, 0x88, 0x82, 0x88, 0x00, 0x00, \
  0x1e, 0x27, 0xe2, 0xd8, 0x00, 0x00

#define kMaskRandom32_31 \
  0x55, 0x11, 0x55, 0x11, 0x00, 0x00, \
  0x22, 0x22, 0x22, 0x22, 0x00, 0x00, \
  0x11, 0x11, 0x11, 0x11, 0x00, 0x00, \
  0x80, 0x45, 0x80, 0x45, 0x00, 0x00, \
  0x20, 0x1a, 0x20, 0x1a, 0x00, 0x00, \
  0x08, 0x68, 0x08, 0x68, 0x00, 0x00, \
  0x22, 0x84, 0x22, 0x84, 0x00, 0x00, \
  0x48, 0x09, 0x48, 0x09, 0x00, 0x00, \
  0x07, 0x01, 0x07, 0x01, 0x00, 0x00, \
  0x94, 0x20, 0x94, 0x20, 0x00, 0x00, \
  0x82, 0x06, 0x82, 0x06, 0x00, 0x00, \
  0x60, 0x48, 0x60, 0x48, 0x00, 0x00, \
  0x89, 0x80, 0x89, 0x80, 0x00, 0x00, \
  0x00, 0x8e, 0x00, 0x8e, 0x00, 0x00, \
  0x18, 0x22, 0x18, 0x22, 0x00, 0x00, \
  0xa4, 0x10, 0xa4, 0x10, 0x00, 0x00, \
  0x01, 0x2a, 0x01, 0x2a, 0x00, 0x00, \
  0x06, 0x42, 0x06, 0x42, 0x00, 0x00, \
  0x08, 0x68, 0x08, 0x68, 0x00, 0x00, \
  0x81, 0x90, 0x81, 0x90, 0x00, 0x00, \
  0x00, 0xf0, 0x00, 0xf0, 0x00, 0x00, \
  0x50, 0x05, 0x50, 0x05, 0x00, 0x00, \
  0x20, 0x51, 0x20, 0x51, 0x00, 0x00, \
  0x43, 0x08, 0x43, 0x08, 0x00, 0x00, \
  0x68, 0x80, 0x68, 0x80, 0x00, 0x00, \
  0x80, 0x0b, 0x80, 0x0b, 0x00, 0x00, \
  0x10, 0x4c, 0x10, 0x4c, 0x00, 0x00, \
  0x12, 0x30, 0x12, 0x30, 0x00, 0x00, \
  0x40, 0x85, 0x40, 0x85, 0x00, 0x00, \
  0x0e, 0x04, 0x0e, 0x04, 0x00, 0x00, \
  0x18, 0x12, 0x18, 0x12, 0x00, 0x00

#define kMaskRandom32_32 \
  0xa4, 0x10, 0xa4, 0x10, 0x00, 0x00, \
  0x01, 0x2a, 0x01, 0x2a, 0x00, 0x00, \
  0x06, 0x42, 0x06, 0x42, 0x00, 0x00, \
  0x08, 0x68, 0x08, 0x68, 0x00, 0x00, \
  0x81, 0x90, 0x81, 0x90, 0x00, 0x00, \
  0x00, 0xf0, 0x00, 0xf0, 0x00, 0x00, \
  0x50, 0x05, 0x50, 0x05, 0x00, 0x00, \
  0x20, 0x51, 0x20, 0x51, 0x00, 0x00, \
  0x43, 0x08, 0x43, 0x08, 0x00, 0x00, \
  0x68, 0x80, 0x68, 0x80, 0x00, 0x00, \
  0x80, 0x0b, 0x80, 0x0b, 0x00, 0x00, \
  0x10, 0x4c, 0x10, 0x4c, 0x00, 0x00, \
  0x12, 0x30, 0x12, 0x30, 0x00, 0x00, \
  0x40, 0x85, 0x40, 0x85, 0x00, 0x00, \
  0x0e, 0x04, 0x0e, 0x04, 0x00, 0x00, \
  0x18, 0x12, 0x18, 0x12, 0x00, 0x00, \
  0x55, 0x11, 0x55, 0x11, 0x00, 0x00, \
  0x22, 0x22, 0x22, 0x22, 0x00, 0x00, \
  0x11, 0x11, 0x11, 0x11, 0x00, 0x00, \
  0x80, 0x45, 0x80, 0x45, 0x00, 0x00, \
  0x20, 0x1a, 0x20, 0x1a, 0x00, 0x00, \
  0x08, 0x68, 0x08, 0x68, 0x00, 0x00, \
  0x22, 0x84, 0x22, 0x84, 0x00, 0x00, \
  0x48, 0x09, 0x48, 0x09, 0x00, 0x00, \
  0x07, 0x01, 0x07, 0x01, 0x00, 0x00, \
  0x94, 0x20, 0x94, 0x20, 0x00, 0x00, \
  0x82, 0x06, 0x82, 0x06, 0x00, 0x00, \
  0x60, 0x48, 0x60, 0x48, 0x00, 0x00, \
  0x89, 0x80, 0x89, 0x80, 0x00, 0x00, \
  0x00, 0x8e, 0x00, 0x8e, 0x00, 0x00, \
  0x18, 0x22, 0x18, 0x22, 0x00, 0x00, \
  0x60, 0xc4, 0x02, 0x02, 0x00, 0x00

#define kMaskRandom32_4 \
  0x55, 0x55, 0x55, 0x55, 0x00, 0x00, \
  0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, \
  0x35, 0x35, 0x35, 0x35, 0x00, 0x00, \
  0xca, 0xca, 0xca, 0xca, 0x00, 0x00

#define kMaskRandom32_5 \
  0x55, 0x55, 0x55, 0x55, 0x00, 0x00, \
  0x2a, 0x2a, 0x2a, 0x2a, 0x00, 0x00, \
  0x24, 0x25, 0x24, 0x25, 0x00, 0x00, \
  0x84, 0xc8, 0x84, 0xc8, 0x00, 0x00, \
  0x10, 0xb6, 0x10, 0xb6, 0x00, 0x00

#define kMaskRandom32_6 \
  0x51, 0x51, 0x51, 0x51, 0x00, 0x00, \
  0x0a, 0x2a, 0x0a, 0x2a, 0x00, 0x00, \
  0xa2, 0x15, 0xa2, 0x15, 0x00, 0x00, \
  0x84, 0x4a, 0x84, 0x4a, 0x00, 0x00, \
  0x30, 0x92, 0x30, 0x92, 0x00, 0x00, \
  0x04, 0xac, 0x04, 0xac, 0x00, 0x00

#define kMaskRandom32_7 \
  0x45, 0x51, 0x45, 0x51, 0x00, 0x00, \
  0x22, 0x2a, 0x22, 0x2a, 0x00, 0x00, \
  0x91, 0x11, 0x91, 0x11, 0x00, 0x00, \
  0x2e, 0x08, 0x2e, 0x08, 0x00, 0x00, \
  0x48, 0x34, 0x48, 0x34, 0x00, 0x00, \
  0x90, 0x29, 0x90, 0x29, 0x00, 0x00, \
  0x09, 0x86, 0x09, 0x86, 0x00, 0x00

#define kMaskRandom32_8 \
  0x20, 0x54, 0x20, 0x54, 0x00, 0x00, \
  0x18, 0x88, 0x18, 0x88, 0x00, 0x00, \
  0x84, 0x07, 0x84, 0x07, 0x00, 0x00, \
  0x60, 0x48, 0x60, 0x48, 0x00, 0x00, \
  0x12, 0x82, 0x12, 0x82, 0x00, 0x00, \
  0x81, 0x41, 0x81, 0x41, 0x00, 0x00, \
  0x40, 0x62, 0x40, 0x62, 0x00, 0x00, \
  0x16, 0x30, 0x16, 0x30, 0x00, 0x00

#define kMaskRandom32_9 \
  0x55, 0x51, 0x55, 0x51, 0x00, 0x00, \
  0x22, 0x2a, 0x22, 0x2a, 0x00, 0x00, \
  0x05, 0x85, 0x05, 0x85, 0x00, 0x00, \
  0x09, 0x4a, 0x09, 0x4a, 0x00, 0x00, \
  0x84, 0x32, 0x84, 0x32, 0x00, 0x00, \
  0xc0, 0x0d, 0xc0, 0x0d, 0x00, 0x00, \
  0x20, 0xa6, 0x20, 0xa6, 0x00, 0x00, \
  0x1a, 0x09, 0x1a, 0x09, 0x00, 0x00, \
  0x44, 0x64, 0x44, 0x64, 0x00, 0x00

#define kMaskRandom33_1 \
  0xff, 0xff, 0xff, 0xff, 0x80, 0x00

#define kMaskRandom33_10 \
  0x45, 0x51, 0x55, 0x8c, 0x80, 0x00, \
  0x10, 0xa2, 0xaa, 0x27, 0x00, 0x00, \
  0x01, 0x25, 0xa5, 0x32, 0x80, 0x00, \
  0x0b, 0x42, 0x62, 0x61, 0x80, 0x00, \
  0xd8, 0x20, 0x3c, 0x5c, 0x00, 0x00, \
  0x82, 0x8c, 0x8e, 0xcc, 0x00, 0x00, \
  0x24, 0x4a, 0x6a, 0x2b, 0x00, 0x00, \
  0x38, 0x18, 0x36, 0x32, 0x80, 0x00, \
  0x2a, 0x25, 0xd1, 0x25, 0x80, 0x00, \
  0x84, 0x92, 0xc8, 0x02, 0x80, 0x00

#define kMaskRandom33_11 \
  0x55, 0x55, 0x55, 0x8c, 0x80, 0x00, \
  0x2a, 0x22, 0xaa, 0x27, 0x00, 0x00, \
  0x31, 0x11, 0xa5, 0x32, 0x80, 0x00, \
  0x83, 0x42, 0x62, 0x61, 0x80, 0x00, \
  0x06, 0x98, 0x3c, 0x5c, 0x00, 0x00, \
  0x40, 0xe1, 0x51, 0x84, 0x80, 0x00, \
  0x2c, 0x44, 0xa2, 0x27, 0x00, 0x00, \
  0xd8, 0x28, 0x95, 0x51, 0x80, 0x00, \
  0x92, 0x81, 0x4a, 0x1a, 0x00, 0x00, \
  0x84, 0x32, 0x30, 0x68, 0x00, 0x00, \
  0x68, 0x0c, 0x2c, 0x89, 0x00, 0x00

#define kMaskRandom33_12 \
  0x84, 0x31, 0x51, 0x84, 0x80, 0x00, \
  0x18, 0xa2, 0xa2, 0x27, 0x00, 0x00, \
  0x4e, 0x01, 0x95, 0x51, 0x80, 0x00, \
  0x44, 0xc8, 0x4a, 0x1a, 0x00, 0x00, \
  0x0e, 0x90, 0x30, 0x68, 0x00, 0x00, \
  0x20, 0xcc, 0x2c, 0x89, 0x00, 0x00, \
  0x93, 0x40, 0x55, 0x8c, 0x80, 0x00, \
  0x2d, 0x10, 0xaa, 0x27, 0x00, 0x00, \
  0x31, 0x44, 0xa5, 0x32, 0x80, 0x00, \
  0xc0, 0x23, 0x62, 0x61, 0x80, 0x00, \
  0x11, 0x25, 0x3c, 0x5c, 0x00, 0x00, \
  0xe8, 0x80, 0x51, 0x35, 0x00, 0x00

#define kMaskRandom33_13 \
  0x45, 0x15, 0x51, 0x84, 0x80, 0x00, \
  0x22, 0x22, 0xa2, 0x27, 0x00, 0x00, \
  0x96, 0x0c, 0x95, 0x51, 0x80, 0x00, \
  0x0c, 0x50, 0x4a, 0x1a, 0x00, 0x00, \
  0x62, 0x04, 0x30, 0x68, 0x00, 0x00, \
  0x49, 0x06, 0x2c, 0x89, 0x00, 0x00, \
  0x11, 0x82, 0x15, 0x8c, 0x00, 0x00, \
  0x12, 0x38, 0x8a, 0x47, 0x00, 0x00, \
  0x40, 0x71, 0x25, 0x81, 0x80, 0x00, \
  0xa8, 0x8a, 0x62, 0x12, 0x80, 0x00, \
  0x08, 0xa1, 0x58, 0x58, 0x00, 0x00, \
  0xa0, 0xc0, 0x0e, 0x28, 0x80, 0x00, \
  0xc5, 0x10, 0x83, 0x34, 0x00, 0x00

#define kMaskRandom33_14 \
  0x45, 0x51, 0x15, 0x8c, 0x00, 0x00, \
  0x22, 0x0a, 0x8a, 0x47, 0x00, 0x00, \
  0x84, 0xd0, 0x25, 0x81, 0x80, 0x00, \
  0x0c, 0x8a, 0x62, 0x12, 0x80, 0x00, \
  0x18, 0x06, 0x58, 0x58, 0x00, 0x00, \
  0x30, 0x03, 0x0e, 0x28, 0x80, 0x00, \
  0x61, 0x08, 0x83, 0x34, 0x00, 0x00, \
  0x40, 0x11, 0x51, 0x84, 0x80, 0x00, \
  0x10, 0x2c, 0xa2, 0x27, 0x00, 0x00, \
  0x09, 0x60, 0x95, 0x51, 0x80, 0x00, \
  0x00, 0x94, 0x4a, 0x1a, 0x00, 0x00, \
  0x52, 0x40, 0x30, 0x68, 0x00, 0x00, \
  0xa4, 0x24, 0x2c, 0x89, 0x00, 0x00, \
  0x82, 0x88, 0xb0, 0xde, 0x80, 0x00

#define kMaskRandom33_15 \
  0x55, 0x11, 0x15, 0x8c, 0x00, 0x00, \
  0x22, 0x22, 0x8a, 0x47, 0x00, 0x00, \
  0x11, 0x11, 0x25, 0x81, 0x80, 0x00, \
  0x80, 0x45, 0x62, 0x12, 0x80, 0x00, \
  0x20, 0x1a, 0x58, 0x58, 0x00, 0x00, \
  0x08, 0x68, 0x0e, 0x28, 0x80, 0x00, \
  0x22, 0x84, 0x83, 0x34, 0x00, 0x00, \
  0x48, 0x09, 0x25, 0x2c, 0x00, 0x00, \
  0x07, 0x01, 0x8a, 0x91, 0x00, 0x00, \
  0x94, 0x20, 0x91, 0xc0, 0x80, 0x00, \
  0x82, 0x06, 0x68, 0x06, 0x80, 0x00, \
  0x60, 0x48, 0x32, 0xc8, 0x00, 0x00, \
  0x89, 0x80, 0x43, 0x45, 0x00, 0x00, \
  0x00, 0x8e, 0xc4, 0x30, 0x80, 0x00, \
  0x18, 0x22, 0x1c, 0xa2, 0x00, 0x00

#define kMaskRandom33_16 \
  0xa4, 0x10, 0x25, 0x2c, 0x00, 0x00, \
  0x01, 0x2a, 0x8a, 0x91, 0x00, 0x00, \
  0x06, 0x42, 0x91, 0xc0, 0x80, 0x00, \
  0x08, 0x68, 0x68, 0x06, 0x80, 0x00, \
  0x81, 0x90, 0x32, 0xc8, 0x00, 0x00, \
  0x00, 0xf0, 0x43, 0x45, 0x00, 0x00, \
  0x50, 0x05, 0xc4, 0x30, 0x80, 0x00, \
  0x20, 0x51, 0x1c, 0xa2, 0x00, 0x00, \
  0x43, 0x08, 0x15, 0x8c, 0x00, 0x00, \
  0x68, 0x80, 0x8a, 0x47, 0x00, 0x00, \
  0x80, 0x0b, 0x25, 0x81, 0x80, 0x00, \
  0x10, 0x4c, 0x62, 0x12, 0x80, 0x00, \
  0x12, 0x30, 0x58, 0x58, 0x00, 0x00, \
  0x40, 0x85, 0x0e, 0x28, 0x80, 0x00, \
  0x0e, 0x04, 0x83, 0x34, 0x00, 0x00, \
  0x18, 0x12, 0x0a, 0x1c, 0x00, 0x00

#define kMaskRandom33_17 \
  0x20, 0x54, 0x64, 0x16, 0x00, 0x00, \
  0x18, 0x88, 0xa2, 0xc2, 0x00, 0x00, \
  0x84, 0x07, 0x51, 0x60, 0x80, 0x00, \
  0x60, 0x48, 0x4a, 0x85, 0x00, 0x00, \
  0x12, 0x82, 0x38, 0x4c, 0x00, 0x00, \
  0x81, 0x41, 0x89, 0x29, 0x00, 0x00, \
  0x40, 0x62, 0x07, 0x11, 0x80, 0x00, \
  0x16, 0x30, 0x94, 0xb0, 0x00, 0x00, \
  0x55, 0x51, 0x8e, 0xcc, 0x00, 0x00, \
  0x22, 0x2a, 0x6a, 0x2b, 0x00, 0x00, \
  0x05, 0x85, 0x36, 0x32, 0x80, 0x00, \
  0x09, 0x4a, 0xd1, 0x25, 0x80, 0x00, \
  0x84, 0x32, 0x55, 0x8c, 0x80, 0x00, \
  0xc0, 0x0d, 0xaa, 0x27, 0x00, 0x00, \
  0x20, 0xa6, 0xa5, 0x32, 0x80, 0x00, \
  0x1a, 0x09, 0x62, 0x61, 0x80, 0x00, \
  0x44, 0x64, 0x3c, 0x5c, 0x00, 0x00

#define kMaskRandom33_18 \
  0x55, 0x51, 0x8e, 0xcc, 0x00, 0x00, \
  0x22, 0x2a, 0x6a, 0x2b, 0x00, 0x00, \
  0x05, 0x85, 0x36, 0x32, 0x80, 0x00, \
  0x09, 0x4a, 0xd1, 0x25, 0x80, 0x00, \
  0x84, 0x32, 0x55, 0x8c, 0x80, 0x00, \
  0xc0, 0x0d, 0xaa, 0x27, 0x00, 0x00, \
  0x20, 0xa6, 0xa5, 0x32, 0x80, 0x00, \
  0x1a, 0x09, 0x62, 0x61, 0x80, 0x00, \
  0x44, 0x64, 0x3c, 0x5c, 0x00, 0x00, \
  0x20, 0x54, 0x64, 0x16, 0x00, 0x00, \
  0x18, 0x88, 0xa2, 0xc2, 0x00, 0x00, \
  0x84, 0x07, 0x51, 0x60, 0x80, 0x00, \
  0x60, 0x48, 0x4a, 0x85, 0x00, 0x00, \
  0x12, 0x82, 0x38, 0x4c, 0x00, 0x00, \
  0x81, 0x41, 0x89, 0x29, 0x00, 0x00, \
  0x40, 0x62, 0x07, 0x11, 0x80, 0x00, \
  0x16, 0x30, 0x94, 0xb0, 0x00, 0x00, \
  0x89, 0x53, 0x03, 0xad, 0x00, 0x00

#define kMaskRandom33_19 \
  0x55, 0x51, 0x8e, 0xcc, 0x00, 0x00, \
  0x22, 0x2a, 0x6a, 0x2b, 0x00, 0x00, \
  0x05, 0x85, 0x36, 0x32, 0x80, 0x00, \
  0x09, 0x4a, 0xd1, 0x25, 0x80, 0x00, \
  0x84, 0x32, 0x55, 0x8c, 0x80, 0x00, \
  0xc0, 0x0d, 0xaa, 0x27, 0x00, 0x00, \
  0x20, 0xa6, 0xa5, 0x32, 0x80, 0x00, \
  0x1a, 0x09, 0x62, 0x61, 0x80, 0x00, \
  0x44, 0x64, 0x3c, 0x5c, 0x00, 0x00, \
  0x45, 0x51, 0x55, 0x8c, 0x80, 0x00, \
  0x10, 0xa2, 0xaa, 0x27, 0x00, 0x00, \
  0x01, 0x25, 0xa5, 0x32, 0x80, 0x00, \
  0x0b, 0x42, 0x62, 0x61, 0x80, 0x00, \
  0xd8, 0x20, 0x3c, 0x5c, 0x00, 0x00, \
  0x82, 0x8c, 0x8e, 0xcc, 0x00, 0x00, \
  0x24, 0x4a, 0x6a, 0x2b, 0x00, 0x00, \
  0x38, 0x18, 0x36, 0x32, 0x80, 0x00, \
  0x2a, 0x25, 0xd1, 0x25, 0x80, 0x00, \
  0x84, 0x92, 0xc8, 0x02, 0x80, 0x00

#define kMaskRandom33_2 \
  0xae, 0xae, 0xce, 0xce, 0x00, 0x00, \
  0x79, 0x79, 0xb9, 0x39, 0x80, 0x00

#define kMaskRandom33_20 \
  0x45, 0x51, 0x55, 0x8c, 0x80, 0x00, \
  0x10, 0xa2, 0xaa, 0x27, 0x00, 0x00, \
  0x01, 0x25, 0xa5, 0x32, 0x80, 0x00, \
  0x0b, 0x42, 0x62, 0x61, 0x80, 0x00, \
  0xd8, 0x20, 0x3c, 0x5c, 0x00, 0x00, \
  0x82, 0x8c, 0x8e, 0xcc, 0x00, 0x00, \
  0x24, 0x4a, 0x6a, 0x2b, 0x00, 0x00, \
  0x38, 0x18, 0x36, 0x32, 0x80, 0x00, \
  0x2a, 0x25, 0xd1, 0x25, 0x80, 0x00, \
  0x84, 0x92, 0xc8, 0x02, 0x80, 0x00, \
  0x55, 0x51, 0x8e, 0xcc, 0x00, 0x00, \
  0x22, 0x2a, 0x6a, 0x2b, 0x00, 0x00, \
  0x05, 0x85, 0x36, 0x32, 0x80, 0x00, \
  0x09, 0x4a, 0xd1, 0x25, 0x80, 0x00, \
  0x84, 0x32, 0x55, 0x8c, 0x80, 0x00, \
  0xc0, 0x0d, 0xaa, 0x27, 0x00, 0x00, \
  0x20, 0xa6, 0xa5, 0x32, 0x80, 0x00, \
  0x1a, 0x09, 0x62, 0x61, 0x80, 0x00, \
  0x44, 0x64, 0x3c, 0x5c, 0x00, 0x00, \
  0x73, 0x5f, 0x5b, 0x0e, 0x80, 0x00

#define kMaskRandom33_21 \
  0x45, 0x51, 0x55, 0x8c, 0x80, 0x00, \
  0x10, 0xa2, 0xaa, 0x27, 0x00, 0x00, \
  0x01, 0x25, 0xa5, 0x32, 0x80, 0x00, \
  0x0b, 0x42, 0x62, 0x61, 0x80, 0x00, \
  0xd8, 0x20, 0x3c, 0x5c, 0x00, 0x00, \
  0x82, 0x8c, 0x8e, 0xcc, 0x00, 0x00, \
  0x24, 0x4a, 0x6a, 0x2b, 0x00, 0x00, \
  0x38, 0x18, 0x36, 0x32, 0x80, 0x00, \
  0x2a, 0x25, 0xd1, 0x25, 0x80, 0x00, \
  0x84, 0x92, 0xc8, 0x02, 0x80, 0x00, \
  0x55, 0x55, 0x55, 0x8c, 0x80, 0x00, \
  0x2a, 0x22, 0xaa, 0x27, 0x00, 0x00, \
  0x31, 0x11, 0xa5, 0x32, 0x80, 0x00, \
  0x83, 0x42, 0x62, 0x61, 0x80, 0x00, \
  0x06, 0x98, 0x3c, 0x5c, 0x00, 0x00, \
  0x40, 0xe1, 0x51, 0x84, 0x80, 0x00, \
  0x2c, 0x44, 0xa2, 0x27, 0x00, 0x00, \
  0xd8, 0x28, 0x95, 0x51, 0x80, 0x00, \
  0x92, 0x81, 0x4a, 0x1a, 0x00, 0x00, \
  0x84, 0x32, 0x30, 0x68, 0x00, 0x00, \
  0x68, 0x0c, 0x2c, 0x89, 0x00, 0x00

#define kMaskRandom33_22 \
  0x55, 0x55, 0x55, 0x8c, 0x80, 0x00, \
  0x2a, 0x22, 0xaa, 0x27, 0x00, 0x00, \
  0x31, 0x11, 0xa5, 0x32, 0x80, 0x00, \
  0x83, 0x42, 0x62, 0x61, 0x80, 0x00, \
  0x06, 0x98, 0x3c, 0x5c, 0x00, 0x00, \
  0x40, 0xe1, 0x51, 0x84, 0x80, 0x00, \
  0x2c, 0x44, 0xa2, 0x27, 0x00, 0x00, \
  0xd8, 0x28, 0x95, 0x51, 0x80, 0x00, \
  0x92, 0x81, 0x4a, 0x1a, 0x00, 0x00, \
  0x84, 0x32, 0x30, 0x68, 0x00, 0x00, \
  0x68, 0x0c, 0x2c, 0x89, 0x00, 0x00, \
  0x45, 0x51, 0x55, 0x8c, 0x80, 0x00, \
  0x10, 0xa2, 0xaa, 0x27, 0x00, 0x00, \
  0x01, 0x25, 0xa5, 0x32, 0x80, 0x00, \
  0x0b, 0x42, 0x62, 0x61, 0x80, 0x00, \
  0xd8, 0x20, 0x3c, 0x5c, 0x00, 0x00, \
  0x82, 0x8c, 0x8e, 0xcc, 0x00, 0x00, \
  0x24, 0x4a, 0x6a, 0x2b, 0x00, 0x00, \
  0x38, 0x18, 0x36, 0x32, 0x80, 0x00, \
  0x2a, 0x25, 0xd1, 0x25, 0x80, 0x00, \
  0x84, 0x92, 0xc8, 0x02, 0x80, 0x00, \
  0xcc, 0xe3, 0x42, 0x6b, 0x80, 0x00

#define kMaskRandom33_23 \
  0x55, 0x55, 0x55, 0x8c, 0x80, 0x00, \
  0x2a, 0x22, 0xaa, 0x27, 0x00, 0x00, \
  0x31, 0x11, 0xa5, 0x32, 0x80, 0x00, \
  0x83, 0x42, 0x62, 0x61, 0x80, 0x00, \
  0x06, 0x98, 0x3c, 0x5c, 0x00, 0x00, \
  0x40, 0xe1, 0x51, 0x84, 0x80, 0x00, \
  0x2c, 0x44, 0xa2, 0x27, 0x00, 0x00, \
  0xd8, 0x28, 0x95, 0x51, 0x80, 0x00, \
  0x92, 0x81, 0x4a, 0x1a, 0x00, 0x00, \
  0x84, 0x32, 0x30, 0x68, 0x00, 0x00, \
  0x68, 0x0c, 0x2c, 0x89, 0x00, 0x00, \
  0x84, 0x31, 0x51, 0x84, 0x80, 0x00, \
  0x18, 0xa2, 0xa2, 0x27, 0x00, 0x00, \
  0x4e, 0x01, 0x95, 0x51, 0x80, 0x00, \
  0x44, 0xc8, 0x4a, 0x1a, 0x00, 0x00, \
  0x0e, 0x90, 0x30, 0x68, 0x00, 0x00, \
  0x20, 0xcc, 0x2c, 0x89, 0x00, 0x00, \
  0x93, 0x40, 0x55, 0x8c, 0x80, 0x00, \
  0x2d, 0x10, 0xaa, 0x27, 0x00, 0x00, \
  0x31, 0x44, 0xa5, 0x32, 0x80, 0x00, \
  0xc0, 0x23, 0x62, 0x61, 0x80, 0x00, \
  0x11, 0x25, 0x3c, 0x5c, 0x00, 0x00, \
  0xe8, 0x80, 0x51, 0x35, 0x00, 0x00

#define kMaskRandom33_24 \
  0x84, 0x31, 0x51, 0x84, 0x80, 0x00, \
  0x18, 0xa2, 0xa2, 0x27, 0x00, 0x00, \
  0x4e, 0x01, 0x95, 0x51, 0x80, 0x00, \
  0x44, 0xc8, 0x4a, 0x1a, 0x00, 0x00, \
  0x0e, 0x90, 0x30, 0x68, 0x00, 0x00, \
  0x20, 0xcc, 0x2c, 0x89, 0x00, 0x00, \
  0x93, 0x40, 0x55, 0x8c, 0x80, 0x00, \
  0x2d, 0x10, 0xaa, 0x27, 0x00, 0x00, \
  0x31, 0x44, 0xa5, 0x32, 0x80, 0x00, \
  0xc0, 0x23, 0x62, 0x61, 0x80, 0x00, \
  0x11, 0x25, 0x3c, 0x5c, 0x00, 0x00, \
  0xe8, 0x80, 0x51, 0x35, 0x00, 0x00, \
  0x55, 0x55, 0x55, 0x8c, 0x80, 0x00, \
  0x2a, 0x22, 0xaa, 0x27, 0x00, 0x00, \
  0x31, 0x11, 0xa5, 0x32, 0x80, 0x00, \
  0x83, 0x42, 0x62, 0x61, 0x80, 0x00, \
  0x06, 0x98, 0x3c, 0x5c, 0x00, 0x00, \
  0x40, 0xe1, 0x51, 0x84, 0x80, 0x00, \
  0x2c, 0x44, 0xa2, 0x27, 0x00, 0x00, \
  0xd8, 0x28, 0x95, 0x51, 0x80, 0x00, \
  0x92, 0x81, 0x4a, 0x1a, 0x00, 0x00, \
  0x84, 0x32, 0x30, 0x68, 0x00, 0x00, \
  0x68, 0x0c, 0x2c, 0x89, 0x00, 0x00, \
  0xdc, 0x4e, 0xfc, 0x70, 0x00, 0x00

#define kMaskRandom33_25 \
  0x84, 0x31, 0x51, 0x84, 0x80, 0x00, \
  0x18, 0xa2, 0xa2, 0x27, 0x00, 0x00, \
  0x4e, 0x01, 0x95, 0x51, 0x80, 0x00, \
  0x44, 0xc8, 0x4a, 0x1a, 0x00, 0x00, \
  0x0e, 0x90, 0x30, 0x68, 0x00, 0x00, \
  0x20, 0xcc, 0x2c, 0x89, 0x00, 0x00, \
  0x93, 0x40, 0x55, 0x8c, 0x80, 0x00, \
  0x2d, 0x10, 0xaa, 0x27, 0x00, 0x00, \
  0x31, 0x44, 0xa5, 0x32, 0x80, 0x00, \
  0xc0, 0x23, 0x62, 0x61, 0x80, 0x00, \
  0x11, 0x25, 0x3c, 0x5c, 0x00, 0x00, \
  0xe8, 0x80, 0x51, 0x35, 0x00, 0x00, \
  0x45, 0x15, 0x51, 0x84, 0x80, 0x00, \
  0x22, 0x22, 0xa2, 0x27, 0x00, 0x00, \
  0x96, 0x0c, 0x95, 0x51, 0x80, 0x00, \
  0x0c, 0x50, 0x4a, 0x1a, 0x00, 0x00, \
  0x62, 0x04, 0x30, 0x68, 0x00, 0x00, \
  0x49, 0x06, 0x2c, 0x89, 0x00, 0x00, \
  0x11, 0x82, 0x15, 0x8c, 0x00, 0x00, \
  0x12, 0x38, 0x8a, 0x47, 0x00, 0x00, \
  0x40, 0x71, 0x25, 0x81, 0x80, 0x00, \
  0xa8, 0x8a, 0x62, 0x12, 0x80, 0x00, \
  0x08, 0xa1, 0x58, 0x58, 0x00, 0x00, \
  0xa0, 0xc0, 0x0e, 0x28, 0x80, 0x00, \
  0xc5, 0x10, 0x83, 0x34, 0x00, 0x00

#define kMaskRandom33_26 \
  0x45, 0x15, 0x51, 0x84, 0x80, 0x00, \
  0x22, 0x22, 0xa2, 0x27, 0x00, 0x00, \
  0x96, 0x0c, 0x95, 0x51, 0x80, 0x00, \
  0x0c, 0x50, 0x4a, 0x1a, 0x00, 0x00, \
  0x62, 0x04, 0x30, 0x68, 0x00, 0x00, \
  0x49, 0x06, 0x2c, 0x89, 0x00, 0x00, \
  0x11, 0x82, 0x15, 0x8c, 0x00, 0x00, \
  0x12, 0x38, 0x8a, 0x47, 0x00, 0x00, \
  0x40, 0x71, 0x25, 0x81, 0x80, 0x00, \
  0xa8, 0x8a, 0x62, 0x12, 0x80, 0x00, \
  0x08, 0xa1, 0x58, 0x58, 0x00, 0x00, \
  0xa0, 0xc0, 0x0e, 0x28, 0x80, 0x00, \
  0xc5, 0x10, 0x83, 0x34, 0x00, 0x00, \
  0x84, 0x31, 0x51, 0x84, 0x80, 0x00, \
  0x18, 0xa2, 0xa2, 0x27, 0x00, 0x00, \
  0x4e, 0x01, 0x95, 0x51, 0x80, 0x00, \
  0x44, 0xc8, 0x4a, 0x1a, 0x00, 0x00, \
  0x0e, 0x90, 0x30, 0x68, 0x00, 0x00, \
  0x20, 0xcc, 0x2c, 0x89, 0x00, 0x00, \
  0x93, 0x40, 0x55, 0x8c, 0x80, 0x00, \
  0x2d, 0x10, 0xaa, 0x27, 0x00, 0x00, \
  0x31, 0x44, 0xa5, 0x32, 0x80, 0x00, \
  0xc0, 0x23, 0x62, 0x61, 0x80, 0x00, \
  0x11, 0x25, 0x3c, 0x5c, 0x00, 0x00, \
  0xe8, 0x80, 0x51, 0x35, 0x00, 0x00, \
  0xa4, 0xa4, 0xfc, 0x91, 0x00, 0x00

#define kMaskRandom33_27 \
  0x45, 0x15, 0x51, 0x84, 0x80, 0x00, \
  0x22, 0x22, 0xa2, 0x27, 0x00, 0x00, \
  0x96, 0x0c, 0x95, 0x51, 0x80, 0x00, \
  0x0c, 0x50, 0x4a, 0x1a, 0x00, 0x00, \
  0x62, 0x04, 0x30, 0x68, 0x00, 0x00, \
  0x49, 0x06, 0x2c, 0x89, 0x00, 0x00, \
  0x11, 0x82, 0x15, 0x8c, 0x00, 0x00, \
  0x12, 0x38, 0x8a, 0x47, 0x00, 0x00, \
  0x40, 0x71, 0x25, 0x81, 0x80, 0x00, \
  0xa8, 0x8a, 0x62, 0x12, 0x80, 0x00, \
  0x08, 0xa1, 0x58, 0x58, 0x00, 0x00, \
  0xa0, 0xc0, 0x0e, 0x28, 0x80, 0x00, \
  0xc5, 0x10, 0x83, 0x34, 0x00, 0x00, \
  0x45, 0x51, 0x15, 0x8c, 0x00, 0x00, \
  0x22, 0x0a, 0x8a, 0x47, 0x00, 0x00, \
  0x84, 0xd0, 0x25, 0x81, 0x80, 0x00, \
  0x0c, 0x8a, 0x62, 0x12, 0x80, 0x00, \
  0x18, 0x06, 0x58, 0x58, 0x00, 0x00, \
  0x30, 0x03, 0x0e, 0x28, 0x80, 0x00, \
  0x61, 0x08, 0x83, 0x34, 0x00, 0x00, \
  0x40, 0x11, 0x51, 0x84, 0x80, 0x00, \
  0x10, 0x2c, 0xa2, 0x27, 0x00, 0x00, \
  0x09, 0x60, 0x95, 0x51, 0x80, 0x00, \
  0x00, 0x94, 0x4a, 0x1a, 0x00, 0x00, \
  0x52, 0x40, 0x30, 0x68, 0x00, 0x00, \
  0xa4, 0x24, 0x2c, 0x89, 0x00, 0x00, \
  0x82, 0x88, 0xb0, 0xde, 0x80, 0x00

#define kMaskRandom33_28 \
  0x45, 0x51, 0x15, 0x8c, 0x00, 0x00, \
  0x22, 0x0a, 0x8a, 0x47, 0x00, 0x00, \
  0x84, 0xd0, 0x25, 0x81, 0x80, 0x00, \
  0x0c, 0x8a, 0x62, 0x12, 0x80, 0x00, \
  0x18, 0x06, 0x58, 0x58, 0x00, 0x00, \
  0x30, 0x03, 0x0e, 0x28, 0x80, 0x00, \
  0x61, 0x08, 0x83, 0x34, 0x00, 0x00, \
  0x40, 0x11, 0x51, 0x84, 0x80, 0x00, \
  0x10, 0x2c, 0xa2, 0x27, 0x00, 0x00, \
  0x09, 0x60, 0x95, 0x51, 0x80, 0x00, \
  0x00, 0x94, 0x4a, 0x1a, 0x00, 0x00, \
  0x52, 0x40, 0x30, 0x68, 0x00, 0x00, \
  0xa4, 0x24, 0x2c, 0x89, 0x00, 0x00, \
  0x82, 0x88, 0xb0, 0xde, 0x80, 0x00, \
  0x45, 0x15, 0x51, 0x84, 0x80, 0x00, \
  0x22, 0x22, 0xa2, 0x27, 0x00, 0x00, \
  0x96, 0x0c, 0x95, 0x51, 0x80, 0x00, \
  0x0c, 0x50, 0x4a, 0x1a, 0x00, 0x00, \
  0x62, 0x04, 0x30, 0x68, 0x00, 0x00, \
  0x49, 0x06, 0x2c, 0x89, 0x00, 0x00, \
  0x11, 0x82, 0x15, 0x8c, 0x00, 0x00, \
  0x12, 0x38, 0x8a, 0x47, 0x00, 0x00, \
  0x40, 0x71, 0x25, 0x81, 0x80, 0x00, \
  0xa8, 0x8a, 0x62, 0x12, 0x80, 0x00, \
  0x08, 0xa1, 0x58, 0x58, 0x00, 0x00, \
  0xa0, 0xc0, 0x0e, 0x28, 0x80, 0x00, \
  0xc5, 0x10, 0x83, 0x34, 0x00, 0x00, \
  0x1b, 0xf4, 0xaa, 0xec, 0x80, 0x00

#define kMaskRandom33_29 \
  0x45, 0x51, 0x15, 0x8c, 0x00, 0x00, \
  0x22, 0x0a, 0x8a, 0x47, 0x00, 0x00, \
  0x84, 0xd0, 0x25, 0x81, 0x80, 0x00, \
  0x0c, 0x8a, 0x62, 0x12, 0x80, 0x00, \
  0x18, 0x06, 0x58, 0x58, 0x00, 0x00, \
  0x30, 0x03, 0x0e, 0x28, 0x80, 0x00, \
  0x61, 0x08, 0x83, 0x34, 0x00, 0x00, \
  0x40, 0x11, 0x51, 0x84, 0x80, 0x00, \
  0x10, 0x2c, 0xa2, 0x27, 0x00, 0x00, \
  0x09, 0x60, 0x95, 0x51, 0x80, 0x00, \
  0x00, 0x94, 0x4a, 0x1a, 0x00, 0x00, \
  0x52, 0x40, 0x30, 0x68, 0x00, 0x00, \
  0xa4, 0x24, 0x2c, 0x89, 0x00, 0x00, \
  0x82, 0x88, 0xb0, 0xde, 0x80, 0x00, \
  0x55, 0x11, 0x15, 0x8c, 0x00, 0x00, \
  0x22, 0x22, 0x8a, 0x47, 0x00, 0x00, \
  0x11, 0x11, 0x25, 0x81, 0x80, 0x00, \
  0x80, 0x45, 0x62, 0x12, 0x80, 0x00, \
  0x20, 0x1a, 0x58, 0x58, 0x00, 0x00, \
  0x08, 0x68, 0x0e, 0x28, 0x80, 0x00, \
  0x22, 0x84, 0x83, 0x34, 0x00, 0x00, \
  0x48, 0x09, 0x25, 0x2c, 0x00, 0x00, \
  0x07, 0x01, 0x8a, 0x91, 0x00, 0x00, \
  0x94, 0x20, 0x91, 0xc0, 0x80, 0x00, \
  0x82, 0x06, 0x68, 0x06, 0x80, 0x00, \
  0x60, 0x48, 0x32, 0xc8, 0x00, 0x00, \
  0x89, 0x80, 0x43, 0x45, 0x00, 0x00, \
  0x00, 0x8e, 0xc4, 0x30, 0x80, 0x00, \
  0x18, 0x22, 0x1c, 0xa2, 0x00, 0x00

#define kMaskRandom33_3 \
  0xad, 0x2d, 0xcd, 0xcc, 0x00, 0x00, \
  0x76, 0x36, 0x97, 0x27, 0x00, 0x00, \
  0x26, 0xdb, 0xb8, 0xd1, 0x80, 0x00

#define kMaskRandom33_30 \
  0x55, 0x11, 0x15, 0x8c, 0x00, 0x00, \
  0x22, 0x22, 0x8a, 0x47, 0x00, 0x00, \
  0x11, 0x11, 0x25, 0x81, 0x80, 0x00, \
  0x80, 0x45, 0x62, 0x12, 0x80, 0x00, \
  0x20, 0x1a, 0x58, 0x58, 0x00, 0x00, \
  0x08, 0x68, 0x0e, 0x28, 0x80, 0x00, \
  0x22, 0x84, 0x83, 0x34, 0x00, 0x00, \
  0x48, 0x09, 0x25, 0x2c, 0x00, 0x00, \
  0x07, 0x01, 0x8a, 0x91, 0x00, 0x00, \
  0x94, 0x20, 0x91, 0xc0, 0x80, 0x00, \
  0x82, 0x06, 0x68, 0x06, 0x80, 0x00, \
  0x60, 0x48, 0x32, 0xc8, 0x00, 0x00, \
  0x89, 0x80, 0x43, 0x45, 0x00, 0x00, \
  0x00, 0x8e, 0xc4, 0x30, 0x80, 0x00, \
  0x18, 0x22, 0x1c, 0xa2, 0x00, 0x00, \
  0x45, 0x51, 0x15, 0x8c, 0x00, 0x00, \
  0x22, 0x0a, 0x8a, 0x47, 0x00, 0x00, \
  0x84, 0xd0, 0x25, 0x81, 0x80, 0x00, \
  0x0c, 0x8a, 0x62, 0x12, 0x80, 0x00, \
  0x18, 0x06, 0x58, 0x58, 0x00, 0x00, \
  0x30, 0x03, 0x0e, 0x28, 0x80, 0x00, \
  0x61, 0x08, 0x83, 0x34, 0x00, 0x00, \
  0x40, 0x11, 0x51, 0x84, 0x80, 0x00, \
  0x10, 0x2c, 0xa2, 0x27, 0x00, 0x00, \
  0x09, 0x60, 0x95, 0x51, 0x80, 0x00, \
  0x00, 0x94, 0x4a, 0x1a, 0x00, 0x00, \
  0x52, 0x40, 0x30, 0x68, 0x00, 0x00, \
  0xa4, 0x24, 0x2c, 0x89, 0x00, 0x00, \
  0x82, 0x88, 0xb0, 0xde, 0x80, 0x00, \
  0x6d, 0xd2, 0x8c, 0x00, 0x00, 0x00

#define kMaskRandom33_31 \
  0x55, 0x11, 0x15, 0x8c, 0x00, 0x00, \
  0x22, 0x22, 0x8a, 0x47, 0x00, 0x00, \
  0x11, 0x11, 0x25, 0x81, 0x80, 0x00, \
  0x80, 0x45, 0x62, 0x12, 0x80, 0x00, \
  0x20, 0x1a, 0x58, 0x58, 0x00, 0x00, \
  0x08, 0x68, 0x0e, 0x28, 0x80, 0x00, \
  0x22, 0x84, 0x83, 0x34, 0x00, 0x00, \
  0x48, 0x09, 0x25, 0x2c, 0x00, 0x00, \
  0x07, 0x01, 0x8a, 0x91, 0x00, 0x00, \
  0x94, 0x20, 0x91, 0xc0, 0x80, 0x00, \
  0x82, 0x06, 0x68, 0x06, 0x80, 0x00, \
  0x60, 0x48, 0x32, 0xc8, 0x00, 0x00, \
  0x89, 0x80, 0x43, 0x45, 0x00, 0x00, \
  0x00, 0x8e, 0xc4, 0x30, 0x80, 0x00, \
  0x18, 0x22, 0x1c, 0xa2, 0x00, 0x00, \
  0xa4, 0x10, 0x25, 0x2c, 0x00, 0x00, \
  0x01, 0x2a, 0x8a, 0x91, 0x00, 0x00, \
  0x06, 0x42, 0x91, 0xc0, 0x80, 0x00, \
  0x08, 0x68, 0x68, 0x06, 0x80, 0x00, \
  0x81, 0x90, 0x32, 0xc8, 0x00, 0x00, \
  0x00, 0xf0, 0x43, 0x45, 0x00, 0x00, \
  0x50, 0x05, 0xc4, 0x30, 0x80, 0x00, \
  0x20, 0x51, 0x1c, 0xa2, 0x00, 0x00, \
  0x43, 0x08, 0x15, 0x8c, 0x00, 0x00, \
  0x68, 0x80, 0x8a, 0x47, 0x00, 0x00, \
  0x80, 0x0b, 0x25, 0x81, 0x80, 0x00, \
  0x10, 0x4c, 0x62, 0x12, 0x80, 0x00, \
  0x12, 0x30, 0x58, 0x58, 0x00, 0x00, \
  0x40, 0x85, 0x0e, 0x28, 0x80, 0x00, \
  0x0e, 0x04, 0x83, 0x34, 0x00, 0x00, \
  0x18, 0x12, 0x0a, 0x1c, 0x00, 0x00

#define kMaskRandom33_32 \
  0xa4, 0x10, 0x25, 0x2c, 0x00, 0x00, \
  0x01, 0x2a, 0x8a, 0x91, 0x00, 0x00, \
  0x06, 0x42, 0x91, 0xc0, 0x80, 0x00, \
  0x08, 0x68, 0x68, 0x06, 0x80, 0x00, \
  0x81, 0x90, 0x32, 0xc8, 0x00, 0x00, \
  0x00, 0xf0, 0x43, 0x45, 0x00, 0x00, \
  0x50, 0x05, 0xc4, 0x30, 0x80, 0x00, \
  0x20, 0x51, 0x1c, 0xa2, 0x00, 0x00, \
  0x43, 0x08, 0x15, 0x8c, 0x00, 0x00, \
  0x68, 0x80, 0x8a, 0x47, 0x00, 0x00, \
  0x80, 0x0b, 0x25, 0x81, 0x80, 0x00, \
  0x10, 0x4c, 0x62, 0x12, 0x80, 0x00, \
  0x12, 0x30, 0x58, 0x58, 0x00, 0x00, \
  0x40, 0x85, 0x0e, 0x28, 0x80, 0x00, \
  0x0e, 0x04, 0x83, 0x34, 0x00, 0x00, \
  0x18, 0x12, 0x0a, 0x1c, 0x00, 0x00, \
  0x55, 0x11, 0x15, 0x8c, 0x00, 0x00, \
  0x22, 0x22, 0x8a, 0x47, 0x00, 0x00, \
  0x11, 0x11, 0x25, 0x81, 0x80, 0x00, \
  0x80, 0x45, 0x62, 0x12, 0x80, 0x00, \
  0x20, 0x1a, 0x58, 0x58, 0x00, 0x00, \
  0x08, 0x68, 0x0e, 0x28, 0x80, 0x00, \
  0x22, 0x84, 0x83, 0x34, 0x00, 0x00, \
  0x48, 0x09, 0x25, 0x2c, 0x00, 0x00, \
  0x07, 0x01, 0x8a, 0x91, 0x00, 0x00, \
  0x94, 0x20, 0x91, 0xc0, 0x80, 0x00, \
  0x82, 0x06, 0x68, 0x06, 0x80, 0x00, \
  0x60, 0x48, 0x32, 0xc8, 0x00, 0x00, \
  0x89, 0x80, 0x43, 0x45, 0x00, 0x00, \
  0x00, 0x8e, 0xc4, 0x30, 0x80, 0x00, \
  0x18, 0x22, 0x1c, 0xa2, 0x00, 0x00, \
  0x73, 0x8e, 0x12, 0xca, 0x80, 0x00

#define kMaskRandom33_33 \
  0xa4, 0x10, 0x25, 0x2c, 0x00, 0x00, \
  0x01, 0x2a, 0x8a, 0x91, 0x00, 0x00, \
  0x06, 0x42, 0x91, 0xc0, 0x80, 0x00, \
  0x08, 0x68, 0x68, 0x06, 0x80, 0x00, \
  0x81, 0x90, 0x32, 0xc8, 0x00, 0x00, \
  0x00, 0xf0, 0x43, 0x45, 0x00, 0x00, \
  0x50, 0x05, 0xc4, 0x30, 0x80, 0x00, \
  0x20, 0x51, 0x1c, 0xa2, 0x00, 0x00, \
  0x43, 0x08, 0x15, 0x8c, 0x00, 0x00, \
  0x68, 0x80, 0x8a, 0x47, 0x00, 0x00, \
  0x80, 0x0b, 0x25, 0x81, 0x80, 0x00, \
  0x10, 0x4c, 0x62, 0x12, 0x80, 0x00, \
  0x12, 0x30, 0x58, 0x58, 0x00, 0x00, \
  0x40, 0x85, 0x0e, 0x28, 0x80, 0x00, \
  0x0e, 0x04, 0x83, 0x34, 0x00, 0x00, \
  0x18, 0x12, 0x0a, 0x1c, 0x00, 0x00, \
  0xa4, 0x10, 0x25, 0x2c, 0x00, 0x00, \
  0x01, 0x2a, 0x8a, 0x91, 0x00, 0x00, \
  0x06, 0x42, 0x91, 0xc0, 0x80, 0x00, \
  0x08, 0x68, 0x68, 0x06, 0x80, 0x00, \
  0x81, 0x90, 0x32, 0xc8, 0x00, 0x00, \
  0x00, 0xf0, 0x43, 0x45, 0x00, 0x00, \
  0x50, 0x05, 0xc4, 0x30, 0x80, 0x00, \
  0x20, 0x51, 0x1c, 0xa2, 0x00, 0x00, \
  0x43, 0x08, 0x25, 0x4c, 0x00, 0x00, \
  0x68, 0x80, 0x8a, 0x66, 0x00, 0x00, \
  0x80, 0x0b, 0x91, 0x91, 0x00, 0x00, \
  0x10, 0x4c, 0x68, 0x42, 0x80, 0x00, \
  0x12, 0x30, 0x32, 0xa4, 0x00, 0x00, \
  0x40, 0x85, 0x43, 0x13, 0x00, 0x00, \
  0x0e, 0x04, 0xc4, 0x30, 0x80, 0x00, \
  0x18, 0x12, 0x1c, 0x88, 0x80, 0x00, \
  0xdb, 0x10, 0x3c, 0x09, 0x00, 0x00

#define kMaskRandom33_4 \
  0x55, 0x55, 0xca, 0xec, 0x00, 0x00, \
  0xaa, 0xaa, 0xa9, 0x67, 0x00, 0x00, \
  0x35, 0x35, 0x3a, 0xb1, 0x80, 0x00, \
  0xca, 0xca, 0x55, 0x5a, 0x80, 0x00

#define kMaskRandom33_5 \
  0x55, 0x55, 0x55, 0x44, 0x80, 0x00, \
  0x2a, 0x2a, 0x2a, 0x66, 0x00, 0x00, \
  0x24, 0x25, 0x25, 0xa1, 0x80, 0x00, \
  0x84, 0xc8, 0xe2, 0x12, 0x80, 0x00, \
  0x10, 0xb6, 0x99, 0x98, 0x00, 0x00

#define kMaskRandom33_6 \
  0x51, 0x51, 0xd1, 0x4c, 0x00, 0x00, \
  0x0a, 0x2a, 0xa2, 0xc5, 0x00, 0x00, \
  0xa2, 0x15, 0x95, 0x30, 0x80, 0x00, \
  0x84, 0x4a, 0xca, 0x0a, 0x80, 0x00, \
  0x30, 0x92, 0xa4, 0xaa, 0x00, 0x00, \
  0x04, 0xac, 0x78, 0x15, 0x00, 0x00

#define kMaskRandom33_7 \
  0x45, 0x51, 0x15, 0x44, 0x80, 0x00, \
  0x22, 0x2a, 0x8a, 0x23, 0x00, 0x00, \
  0x91, 0x11, 0x85, 0x91, 0x00, 0x00, \
  0x2e, 0x08, 0x32, 0x0a, 0x80, 0x00, \
  0x48, 0x34, 0x58, 0x34, 0x00, 0x00, \
  0x90, 0x29, 0x2c, 0x0d, 0x00, 0x00, \
  0x09, 0x86, 0x43, 0xc8, 0x00, 0x00

#define kMaskRandom33_8 \
  0x20, 0x54, 0x64, 0x16, 0x00, 0x00, \
  0x18, 0x88, 0xa2, 0xc2, 0x00, 0x00, \
  0x84, 0x07, 0x51, 0x60, 0x80, 0x00, \
  0x60, 0x48, 0x4a, 0x85, 0x00, 0x00, \
  0x12, 0x82, 0x38, 0x4c, 0x00, 0x00, \
  0x81, 0x41, 0x89, 0x29, 0x00, 0x00, \
  0x40, 0x62, 0x07, 0x11, 0x80, 0x00, \
  0x16, 0x30, 0x94, 0xb0, 0x00, 0x00

#define kMaskRandom33_9 \
  0x55, 0x51, 0x8e, 0xcc, 0x00, 0x00, \
  0x22, 0x2a, 0x6a, 0x2b, 0x00, 0x00, \
  0x05, 0x85, 0x36, 0x32, 0x80, 0x00, \
  0x09, 0x4a, 0xd1, 0x25, 0x80, 0x00, \
  0x84, 0x32, 0x55, 0x8c, 0x80, 0x00, \
  0xc0, 0x0d, 0xaa, 0x27, 0x00, 0x00, \
  0x20, 0xa6, 0xa5, 0x32, 0x80, 0x00, \
  0x1a, 0x09, 0x62, 0x61, 0x80, 0x00, \
  0x44, 0x64, 0x3c, 0x5c, 0x00, 0x00

#define kMaskRandom34_1 \
  0xff, 0xff, 0xff, 0xff, 0xc0, 0x00

#define kMaskRandom34_10 \
  0x55, 0x8c, 0xaa, 0xc6, 0x40, 0x00, \
  0xaa, 0x27, 0x55, 0x13, 0x80, 0x00, \
  0xa5, 0x32, 0xd2, 0x99, 0x40, 0x00, \
  0x62, 0x61, 0xb1, 0x30, 0xc0, 0x00, \
  0x3c, 0x5c, 0x1e, 0x2e, 0x00, 0x00, \
  0x8e, 0xcc, 0x47, 0x66, 0x00, 0x00, \
  0x6a, 0x2b, 0x35, 0x15, 0x80, 0x00, \
  0x36, 0x32, 0x9b, 0x19, 0x40, 0x00, \
  0xd1, 0x25, 0xe8, 0x92, 0xc0, 0x00, \
  0xc8, 0x02, 0xe4, 0x01, 0x40, 0x00

#define kMaskRandom34_11 \
  0x55, 0x8c, 0xaa, 0xc6, 0x40, 0x00, \
  0xaa, 0x27, 0x55, 0x13, 0x80, 0x00, \
  0xa5, 0x32, 0xd2, 0x99, 0x40, 0x00, \
  0x62, 0x61, 0xb1, 0x30, 0xc0, 0x00, \
  0x3c, 0x5c, 0x1e, 0x2e, 0x00, 0x00, \
  0x51, 0x84, 0xa8, 0xc2, 0x40, 0x00, \
  0xa2, 0x27, 0x51, 0x13, 0x80, 0x00, \
  0x95, 0x51, 0xca, 0xa8, 0xc0, 0x00, \
  0x4a, 0x1a, 0x25, 0x0d, 0x00, 0x00, \
  0x30, 0x68, 0x18, 0x34, 0x00, 0x00, \
  0x2c, 0x89, 0x16, 0x44, 0x80, 0x00

#define kMaskRandom34_12 \
  0x51, 0x84, 0xa8, 0xc2, 0x40, 0x00, \
  0xa2, 0x27, 0x51, 0x13, 0x80, 0x00, \
  0x95, 0x51, 0xca, 0xa8, 0xc0, 0x00, \
  0x4a, 0x1a, 0x25, 0x0d, 0x00, 0x00, \
  0x30, 0x68, 0x18, 0x34, 0x00, 0x00, \
  0x2c, 0x89, 0x16, 0x44, 0x80, 0x00, \
  0x55, 0x8c, 0xaa, 0xc6, 0x40, 0x00, \
  0xaa, 0x27, 0x55, 0x13, 0x80, 0x00, \
  0xa5, 0x32, 0xd2, 0x99, 0x40, 0x00, \
  0x62, 0x61, 0xb1, 0x30, 0xc0, 0x00, \
  0x3c, 0x5c, 0x1e, 0x2e, 0x00, 0x00, \
  0x51, 0x35, 0x28, 0x9a, 0x80, 0x00

#define kMaskRandom34_13 \
  0x51, 0x84, 0xa8, 0xc2, 0x40, 0x00, \
  0xa2, 0x27, 0x51, 0x13, 0x80, 0x00, \
  0x95, 0x51, 0xca, 0xa8, 0xc0, 0x00, \
  0x4a, 0x1a, 0x25, 0x0d, 0x00, 0x00, \
  0x30, 0x68, 0x18, 0x34, 0x00, 0x00, \
  0x2c, 0x89, 0x16, 0x44, 0x80, 0x00, \
  0x15, 0x8c, 0x0a, 0xc6, 0x00, 0x00, \
  0x8a, 0x47, 0x45, 0x23, 0x80, 0x00, \
  0x25, 0x81, 0x92, 0xc0, 0xc0, 0x00, \
  0x62, 0x12, 0xb1, 0x09, 0x40, 0x00, \
  0x58, 0x58, 0x2c, 0x2c, 0x00, 0x00, \
  0x0e, 0x28, 0x87, 0x14, 0x40, 0x00, \
  0x83, 0x34, 0x41, 0x9a, 0x00, 0x00

#define kMaskRandom34_14 \
  0x15, 0x8c, 0x0a, 0xc6, 0x00, 0x00, \
  0x8a, 0x47, 0x45, 0x23, 0x80, 0x00, \
  0x25, 0x81, 0x92, 0xc0, 0xc0, 0x00, \
  0x62, 0x12, 0xb1, 0x09, 0x40, 0x00, \
  0x58, 0x58, 0x2c, 0x2c, 0x00, 0x00, \
  0x0e, 0x28, 0x87, 0x14, 0x40, 0x00, \
  0x83, 0x34, 0x41, 0x9a, 0x00, 0x00, \
  0x51, 0x84, 0xa8, 0xc2, 0x40, 0x00, \
  0xa2, 0x27, 0x51, 0x13, 0x80, 0x00, \
  0x95, 0x51, 0xca, 0xa8, 0xc0, 0x00, \
  0x4a, 0x1a, 0x25, 0x0d, 0x00, 0x00, \
  0x30, 0x68, 0x18, 0x34, 0x00, 0x00, \
  0x2c, 0x89, 0x16, 0x44, 0x80, 0x00, \
  0xb0, 0xde, 0xd8, 0x6f, 0x40, 0x00

#define kMaskRandom34_15 \
  0x15, 0x8c, 0x0a, 0xc6, 0x00, 0x00, \
  0x8a, 0x47, 0x45, 0x23, 0x80, 0x00, \
  0x25, 0x81, 0x92, 0xc0, 0xc0, 0x00, \
  0x62, 0x12, 0xb1, 0x09, 0x40, 0x00, \
  0x58, 0x58, 0x2c, 0x2c, 0x00, 0x00, \
  0x0e, 0x28, 0x87, 0x14, 0x40, 0x00, \
  0x83, 0x34, 0x41, 0x9a, 0x00, 0x00, \
  0x25, 0x2c, 0x12, 0x96, 0x00, 0x00, \
  0x8a, 0x91, 0x45, 0x48, 0x80, 0x00, \
  0x91, 0xc0, 0xc8, 0xe0, 0x40, 0x00, \
  0x68, 0x06, 0xb4, 0x03, 0x40, 0x00, \
  0x32, 0xc8, 0x19, 0x64, 0x00, 0x00, \
  0x43, 0x45, 0x21, 0xa2, 0x80, 0x00, \
  0xc4, 0x30, 0xe2, 0x18, 0x40, 0x00, \
  0x1c, 0xa2, 0x0e, 0x51, 0x00, 0x00

#define kMaskRandom34_16 \
  0x25, 0x2c, 0x12, 0x96, 0x00, 0x00, \
  0x8a, 0x91, 0x45, 0x48, 0x80, 0x00, \
  0x91, 0xc0, 0xc8, 0xe0, 0x40, 0x00, \
  0x68, 0x06, 0xb4, 0x03, 0x40, 0x00, \
  0x32, 0xc8, 0x19, 0x64, 0x00, 0x00, \
  0x43, 0x45, 0x21, 0xa2, 0x80, 0x00, \
  0xc4, 0x30, 0xe2, 0x18, 0x40, 0x00, \
  0x1c, 0xa2, 0x0e, 0x51, 0x00, 0x00, \
  0x15, 0x8c, 0x0a, 0xc6, 0x00, 0x00, \
  0x8a, 0x47, 0x45, 0x23, 0x80, 0x00, \
  0x25, 0x81, 0x92, 0xc0, 0xc0, 0x00, \
  0x62, 0x12, 0xb1, 0x09, 0x40, 0x00, \
  0x58, 0x58, 0x2c, 0x2c, 0x00, 0x00, \
  0x0e, 0x28, 0x87, 0x14, 0x40, 0x00, \
  0x83, 0x34, 0x41, 0x9a, 0x00, 0x00, \
  0x0a, 0x1c, 0x05, 0x0e, 0x00, 0x00

#define kMaskRandom34_17 \
  0x25, 0x2c, 0x12, 0x96, 0x00, 0x00, \
  0x8a, 0x91, 0x45, 0x48, 0x80, 0x00, \
  0x91, 0xc0, 0xc8, 0xe0, 0x40, 0x00, \
  0x68, 0x06, 0xb4, 0x03, 0x40, 0x00, \
  0x32, 0xc8, 0x19, 0x64, 0x00, 0x00, \
  0x43, 0x45, 0x21, 0xa2, 0x80, 0x00, \
  0xc4, 0x30, 0xe2, 0x18, 0x40, 0x00, \
  0x1c, 0xa2, 0x0e, 0x51, 0x00, 0x00, \
  0x25, 0x4c, 0x12, 0xa6, 0x00, 0x00, \
  0x8a, 0x66, 0x45, 0x33, 0x00, 0x00, \
  0x91, 0x91, 0x48, 0xc8, 0x80, 0x00, \
  0x68, 0x42, 0xb4, 0x21, 0x40, 0x00, \
  0x32, 0xa4, 0x19, 0x52, 0x00, 0x00, \
  0x43, 0x13, 0x21, 0x89, 0x80, 0x00, \
  0xc4, 0x30, 0xe2, 0x18, 0x40, 0x00, \
  0x1c, 0x88, 0x8e, 0x44, 0x40, 0x00, \
  0x3c, 0x09, 0x1e, 0x04, 0x80, 0x00

#define kMaskRandom34_18 \
  0x8e, 0xcc, 0x47, 0x66, 0x00, 0x00, \
  0x6a, 0x2b, 0x35, 0x15, 0x80, 0x00, \
  0x36, 0x32, 0x9b, 0x19, 0x40, 0x00, \
  0xd1, 0x25, 0xe8, 0x92, 0xc0, 0x00, \
  0x55, 0x8c, 0xaa, 0xc6, 0x40, 0x00, \
  0xaa, 0x27, 0x55, 0x13, 0x80, 0x00, \
  0xa5, 0x32, 0xd2, 0x99, 0x40, 0x00, \
  0x62, 0x61, 0xb1, 0x30, 0xc0, 0x00, \
  0x3c, 0x5c, 0x1e, 0x2e, 0x00, 0x00, \
  0x64, 0x16, 0x32, 0x0b, 0x00, 0x00, \
  0xa2, 0xc2, 0x51, 0x61, 0x00, 0x00, \
  0x51, 0x60, 0xa8, 0xb0, 0x40, 0x00, \
  0x4a, 0x85, 0x25, 0x42, 0x80, 0x00, \
  0x38, 0x4c, 0x1c, 0x26, 0x00, 0x00, \
  0x89, 0x29, 0x44, 0x94, 0x80, 0x00, \
  0x07, 0x11, 0x83, 0x88, 0xc0, 0x00, \
  0x94, 0xb0, 0x4a, 0x58, 0x00, 0x00, \
  0x89, 0x70, 0xf3, 0xf7, 0x40, 0x00

#define kMaskRandom34_19 \
  0x8e, 0xcc, 0x47, 0x66, 0x00, 0x00, \
  0x6a, 0x2b, 0x35, 0x15, 0x80, 0x00, \
  0x36, 0x32, 0x9b, 0x19, 0x40, 0x00, \
  0xd1, 0x25, 0xe8, 0x92, 0xc0, 0x00, \
  0x55, 0x8c, 0xaa, 0xc6, 0x40, 0x00, \
  0xaa, 0x27, 0x55, 0x13, 0x80, 0x00, \
  0xa5, 0x32, 0xd2, 0x99, 0x40, 0x00, \
  0x62, 0x61, 0xb1, 0x30, 0xc0, 0x00, \
  0x3c, 0x5c, 0x1e, 0x2e, 0x00, 0x00, \
  0x55, 0x8c, 0xaa, 0xc6, 0x40, 0x00, \
  0xaa, 0x27, 0x55, 0x13, 0x80, 0x00, \
  0xa5, 0x32, 0xd2, 0x99, 0x40, 0x00, \
  0x62, 0x61, 0xb1, 0x30, 0xc0, 0x00, \
  0x3c, 0x5c, 0x1e, 0x2e, 0x00, 0x00, \
  0x8e, 0xcc, 0x47, 0x66, 0x00, 0x00, \
  0x6a, 0x2b, 0x35, 0x15, 0x80, 0x00, \
  0x36, 0x32, 0x9b, 0x19, 0x40, 0x00, \
  0xd1, 0x25, 0xe8, 0x92, 0xc0, 0x00, \
  0xc8, 0x02, 0xe4, 0x01, 0x40, 0x00

#define kMaskRandom34_2 \
  0xce, 0xce, 0x67, 0x67, 0x00, 0x00, \
  0xb9, 0x39, 0xdc, 0x9c, 0xc0, 0x00

#define kMaskRandom34_20 \
  0x55, 0x8c, 0xaa, 0xc6, 0x40, 0x00, \
  0xaa, 0x27, 0x55, 0x13, 0x80, 0x00, \
  0xa5, 0x32, 0xd2, 0x99, 0x40, 0x00, \
  0x62, 0x61, 0xb1, 0x30, 0xc0, 0x00, \
  0x3c, 0x5c, 0x1e, 0x2e, 0x00, 0x00, \
  0x8e, 0xcc, 0x47, 0x66, 0x00, 0x00, \
  0x6a, 0x2b, 0x35, 0x15, 0x80, 0x00, \
  0x36, 0x32, 0x9b, 0x19, 0x40, 0x00, \
  0xd1, 0x25, 0xe8, 0x92, 0xc0, 0x00, \
  0xc8, 0x02, 0xe4, 0x01, 0x40, 0x00, \
  0x8e, 0xcc, 0x47, 0x66, 0x00, 0x00, \
  0x6a, 0x2b, 0x35, 0x15, 0x80, 0x00, \
  0x36, 0x32, 0x9b, 0x19, 0x40, 0x00, \
  0xd1, 0x25, 0xe8, 0x92, 0xc0, 0x00, \
  0x55, 0x8c, 0xaa, 0xc6, 0x40, 0x00, \
  0xaa, 0x27, 0x55, 0x13, 0x80, 0x00, \
  0xa5, 0x32, 0xd2, 0x99, 0x40, 0x00, \
  0x62, 0x61, 0xb1, 0x30, 0xc0, 0x00, \
  0x3c, 0x5c, 0x1e, 0x2e, 0x00, 0x00, \
  0x5d, 0xc5, 0xfe, 0xd8, 0x40, 0x00

#define kMaskRandom34_21 \
  0x55, 0x8c, 0xaa, 0xc6, 0x40, 0x00, \
  0xaa, 0x27, 0x55, 0x13, 0x80, 0x00, \
  0xa5, 0x32, 0xd2, 0x99, 0x40, 0x00, \
  0x62, 0x61, 0xb1, 0x30, 0xc0, 0x00, \
  0x3c, 0x5c, 0x1e, 0x2e, 0x00, 0x00, \
  0x8e, 0xcc, 0x47, 0x66, 0x00, 0x00, \
  0x6a, 0x2b, 0x35, 0x15, 0x80, 0x00, \
  0x36, 0x32, 0x9b, 0x19, 0x40, 0x00, \
  0xd1, 0x25, 0xe8, 0x92, 0xc0, 0x00, \
  0xc8, 0x02, 0xe4, 0x01, 0x40, 0x00, \
  0x55, 0x8c, 0xaa, 0xc6, 0x40, 0x00, \
  0xaa, 0x27, 0x55, 0x13, 0x80, 0x00, \
  0xa5, 0x32, 0xd2, 0x99, 0x40, 0x00, \
  0x62, 0x61, 0xb1, 0x30, 0xc0, 0x00, \
  0x3c, 0x5c, 0x1e, 0x2e, 0x00, 0x00, \
  0x51, 0x84, 0xa8, 0xc2, 0x40, 0x00, \
  0xa2, 0x27, 0x51, 0x13, 0x80, 0x00, \
  0x95, 0x51, 0xca, 0xa8, 0xc0, 0x00, \
  0x4a, 0x1a, 0x25, 0x0d, 0x00, 0x00, \
  0x30, 0x68, 0x18, 0x34, 0x00, 0x00, \
  0x2c, 0x89, 0x16, 0x44, 0x80, 0x00

#define kMaskRandom34_22 \
  0x55, 0x8c, 0xaa, 0xc6, 0x40, 0x00, \
  0xaa, 0x27, 0x55, 0x13, 0x80, 0x00, \
  0xa5, 0x32, 0xd2, 0x99, 0x40, 0x00, \
  0x62, 0x61, 0xb1, 0x30, 0xc0, 0x00, \
  0x3c, 0x5c, 0x1e, 0x2e, 0x00, 0x00, \
  0x51, 0x84, 0xa8, 0xc2, 0x40, 0x00, \
  0xa2, 0x27, 0x51, 0x13, 0x80, 0x00, \
  0x95, 0x51, 0xca, 0xa8, 0xc0, 0x00, \
  0x4a, 0x1a, 0x25, 0x0d, 0x00, 0x00, \
  0x30, 0x68, 0x18, 0x34, 0x00, 0x00, \
  0x2c, 0x89, 0x16, 0x44, 0x80, 0x00, \
  0x55, 0x8c, 0xaa, 0xc6, 0x40, 0x00, \
  0xaa, 0x27, 0x55, 0x13, 0x80, 0x00, \
  0xa5, 0x32, 0xd2, 0x99, 0x40, 0x00, \
  0x62, 0x61, 0xb1, 0x30, 0xc0, 0x00, \
  0x3c, 0x5c, 0x1e, 0x2e, 0x00, 0x00, \
  0x8e, 0xcc, 0x47, 0x66, 0x00, 0x00, \
  0x6a, 0x2b, 0x35, 0x15, 0x80, 0x00, \
  0x36, 0x32, 0x9b, 0x19, 0x40, 0x00, \
  0xd1, 0x25, 0xe8, 0x92, 0xc0, 0x00, \
  0xc8, 0x02, 0xe4, 0x01, 0x40, 0x00, \
  0x2a, 0xf7, 0x4f, 0xf5, 0x80, 0x00

#define kMaskRandom34_23 \
  0x55, 0x8c, 0xaa, 0xc6, 0x40, 0x00, \
  0xaa, 0x27, 0x55, 0x13, 0x80, 0x00, \
  0xa5, 0x32, 0xd2, 0x99, 0x40, 0x00, \
  0x62, 0x61, 0xb1, 0x30, 0xc0, 0x00, \
  0x3c, 0x5c, 0x1e, 0x2e, 0x00, 0x00, \
  0x51, 0x84, 0xa8, 0xc2, 0x40, 0x00, \
  0xa2, 0x27, 0x51, 0x13, 0x80, 0x00, \
  0x95, 0x51, 0xca, 0xa8, 0xc0, 0x00, \
  0x4a, 0x1a, 0x25, 0x0d, 0x00, 0x00, \
  0x30, 0x68, 0x18, 0x34, 0x00, 0x00, \
  0x2c, 0x89, 0x16, 0x44, 0x80, 0x00, \
  0x51, 0x84, 0xa8, 0xc2, 0x40, 0x00, \
  0xa2, 0x27, 0x51, 0x13, 0x80, 0x00, \
  0x95, 0x51, 0xca, 0xa8, 0xc0, 0x00, \
  0x4a, 0x1a, 0x25, 0x0d, 0x00, 0x00, \
  0x30, 0x68, 0x18, 0x34, 0x00, 0x00, \
  0x2c, 0x89, 0x16, 0x44, 0x80, 0x00, \
  0x55, 0x8c, 0xaa, 0xc6, 0x40, 0x00, \
  0xaa, 0x27, 0x55, 0x13, 0x80, 0x00, \
  0xa5, 0x32, 0xd2, 0x99, 0x40, 0x00, \
  0x62, 0x61, 0xb1, 0x30, 0xc0, 0x00, \
  0x3c, 0x5c, 0x1e, 0x2e, 0x00, 0x00, \
  0x51, 0x35, 0x28, 0x9a, 0x80, 0x00

#define kMaskRandom34_24 \
  0x51, 0x84, 0xa8, 0xc2, 0x40, 0x00, \
  0xa2, 0x27, 0x51, 0x13, 0x80, 0x00, \
  0x95, 0x51, 0xca, 0xa8, 0xc0, 0x00, \
  0x4a, 0x1a, 0x25, 0x0d, 0x00, 0x00, \
  0x30, 0x68, 0x18, 0x34, 0x00, 0x00, \
  0x2c, 0x89, 0x16, 0x44, 0x80, 0x00, \
  0x55, 0x8c, 0xaa, 0xc6, 0x40, 0x00, \
  0xaa, 0x27, 0x55, 0x13, 0x80, 0x00, \
  0xa5, 0x32, 0xd2, 0x99, 0x40, 0x00, \
  0x62, 0x61, 0xb1, 0x30, 0xc0, 0x00, \
  0x3c, 0x5c, 0x1e, 0x2e, 0x00, 0x00, \
  0x51, 0x35, 0x28, 0x9a, 0x80, 0x00, \
  0x55, 0x8c, 0xaa, 0xc6, 0x40, 0x00, \
  0xaa, 0x27, 0x55, 0x13, 0x80, 0x00, \
  0xa5, 0x32, 0xd2, 0x99, 0x40, 0x00, \
  0x62, 0x61, 0xb1, 0x30, 0xc0, 0x00, \
  0x3c, 0x5c, 0x1e, 0x2e, 0x00, 0x00, \
  0x51, 0x84, 0xa8, 0xc2, 0x40, 0x00, \
  0xa2, 0x27, 0x51, 0x13, 0x80, 0x00, \
  0x95, 0x51, 0xca, 0xa8, 0xc0, 0x00, \
  0x4a, 0x1a, 0x25, 0x0d, 0x00, 0x00, \
  0x30, 0x68, 0x18, 0x34, 0x00, 0x00, \
  0x2c, 0x89, 0x16, 0x44, 0x80, 0x00, \
  0x4c, 0xb8, 0x04, 0x74, 0xc0, 0x00

#define kMaskRandom34_25 \
  0x51, 0x84, 0xa8, 0xc2, 0x40, 0x00, \
  0xa2, 0x27, 0x51, 0x13, 0x80, 0x00, \
  0x95, 0x51, 0xca, 0xa8, 0xc0, 0x00, \
  0x4a, 0x1a, 0x25, 0x0d, 0x00, 0x00, \
  0x30, 0x68, 0x18, 0x34, 0x00, 0x00, \
  0x2c, 0x89, 0x16, 0x44, 0x80, 0x00, \
  0x55, 0x8c, 0xaa, 0xc6, 0x40, 0x00, \
  0xaa, 0x27, 0x55, 0x13, 0x80, 0x00, \
  0xa5, 0x32, 0xd2, 0x99, 0x40, 0x00, \
  0x62, 0x61, 0xb1, 0x30, 0xc0, 0x00, \
  0x3c, 0x5c, 0x1e, 0x2e, 0x00, 0x00, \
  0x51, 0x35, 0x28, 0x9a, 0x80, 0x00, \
  0x51, 0x84, 0xa8, 0xc2, 0x40, 0x00, \
  0xa2, 0x27, 0x51, 0x13, 0x80, 0x00, \
  0x95, 0x51, 0xca, 0xa8, 0xc0, 0x00, \
  0x4a, 0x1a, 0x25, 0x0d, 0x00, 0x00, \
  0x30, 0x68, 0x18, 0x34, 0x00, 0x00, \
  0x2c, 0x89, 0x16, 0x44, 0x80, 0x00, \
  0x15, 0x8c, 0x0a, 0xc6, 0x00, 0x00, \
  0x8a, 0x47, 0x45, 0x23, 0x80, 0x00, \
  0x25, 0x81, 0x92, 0xc0, 0xc0, 0x00, \
  0x62, 0x12, 0xb1, 0x09, 0x40, 0x00, \
  0x58, 0x58, 0x2c, 0x2c, 0x00, 0x00, \
  0x0e, 0x28, 0x87, 0x14, 0x40, 0x00, \
  0x83, 0x34, 0x41, 0x9a, 0x00, 0x00

#define kMaskRandom34_26 \
  0x51, 0x84, 0xa8, 0xc2, 0x40, 0x00, \
  0xa2, 0x27, 0x51, 0x13, 0x80, 0x00, \
  0x95, 0x51, 0xca, 0xa8, 0xc0, 0x00, \
  0x4a, 0x1a, 0x25, 0x0d, 0x00, 0x00, \
  0x30, 0x68, 0x18, 0x34, 0x00, 0x00, \
  0x2c, 0x89, 0x16, 0x44, 0x80, 0x00, \
  0x15, 0x8c, 0x0a, 0xc6, 0x00, 0x00, \
  0x8a, 0x47, 0x45, 0x23, 0x80, 0x00, \
  0x25, 0x81, 0x92, 0xc0, 0xc0, 0x00, \
  0x62, 0x12, 0xb1, 0x09, 0x40, 0x00, \
  0x58, 0x58, 0x2c, 0x2c, 0x00, 0x00, \
  0x0e, 0x28, 0x87, 0x14, 0x40, 0x00, \
  0x83, 0x34, 0x41, 0x9a, 0x00, 0x00, \
  0x51, 0x84, 0xa8, 0xc2, 0x40, 0x00, \
  0xa2, 0x27, 0x51, 0x13, 0x80, 0x00, \
  0x95, 0x51, 0xca, 0xa8, 0xc0, 0x00, \
  0x4a, 0x1a, 0x25, 0x0d, 0x00, 0x00, \
  0x30, 0x68, 0x18, 0x34, 0x00, 0x00, \
  0x2c, 0x89, 0x16, 0x44, 0x80, 0x00, \
  0x55, 0x8c, 0xaa, 0xc6, 0x40, 0x00, \
  0xaa, 0x27, 0x55, 0x13, 0x80, 0x00, \
  0xa5, 0x32, 0xd2, 0x99, 0x40, 0x00, \
  0x62, 0x61, 0xb1, 0x30, 0xc0, 0x00, \
  0x3c, 0x5c, 0x1e, 0x2e, 0x00, 0x00, \
  0x51, 0x35, 0x28, 0x9a, 0x80, 0x00, \
  0x95, 0x20, 0xe9, 0xef, 0xc0, 0x00

#define kMaskRandom34_27 \
  0x51, 0x84, 0xa8, 0xc2, 0x40, 0x00, \
  0xa2, 0x27, 0x51, 0x13, 0x80, 0x00, \
  0x95, 0x51, 0xca, 0xa8, 0xc0, 0x00, \
  0x4a, 0x1a, 0x25, 0x0d, 0x00, 0x00, \
  0x30, 0x68, 0x18, 0x34, 0x00, 0x00, \
  0x2c, 0x89, 0x16, 0x44, 0x80, 0x00, \
  0x15, 0x8c, 0x0a, 0xc6, 0x00, 0x00, \
  0x8a, 0x47, 0x45, 0x23, 0x80, 0x00, \
  0x25, 0x81, 0x92, 0xc0, 0xc0, 0x00, \
  0x62, 0x12, 0xb1, 0x09, 0x40, 0x00, \
  0x58, 0x58, 0x2c, 0x2c, 0x00, 0x00, \
  0x0e, 0x28, 0x87, 0x14, 0x40, 0x00, \
  0x83, 0x34, 0x41, 0x9a, 0x00, 0x00, \
  0x15, 0x8c, 0x0a, 0xc6, 0x00, 0x00, \
  0x8a, 0x47, 0x45, 0x23, 0x80, 0x00, \
  0x25, 0x81, 0x92, 0xc0, 0xc0, 0x00, \
  0x62, 0x12, 0xb1, 0x09, 0x40, 0x00, \
  0x58, 0x58, 0x2c, 0x2c, 0x00, 0x00, \
  0x0e, 0x28, 0x87, 0x14, 0x40, 0x00, \
  0x83, 0x34, 0x41, 0x9a, 0x00, 0x00, \
  0x51, 0x84, 0xa8, 0xc2, 0x40, 0x00, \
  0xa2, 0x27, 0x51, 0x13, 0x80, 0x00, \
  0x95, 0x51, 0xca, 0xa8, 0xc0, 0x00, \
  0x4a, 0x1a, 0x25, 0x0d, 0x00, 0x00, \
  0x30, 0x68, 0x18, 0x34, 0x00, 0x00, \
  0x2c, 0x89, 0x16, 0x44, 0x80, 0x00, \
  0xb0, 0xde, 0xd8, 0x6f, 0x40, 0x00

#define kMaskRandom34_28 \
  0x15, 0x8c, 0x0a, 0xc6, 0x00, 0x00, \
  0x8a, 0x47, 0x45, 0x23, 0x80, 0x00, \
  0x25, 0x81, 0x92, 0xc0, 0xc0, 0x00, \
  0x62, 0x12, 0xb1, 0x09, 0x40, 0x00, \
  0x58, 0x58, 0x2c, 0x2c, 0x00, 0x00, \
  0x0e, 0x28, 0x87, 0x14, 0x40, 0x00, \
  0x83, 0x34, 0x41, 0x9a, 0x00, 0x00, \
  0x51, 0x84, 0xa8, 0xc2, 0x40, 0x00, \
  0xa2, 0x27, 0x51, 0x13, 0x80, 0x00, \
  0x95, 0x51, 0xca, 0xa8, 0xc0, 0x00, \
  0x4a, 0x1a, 0x25, 0x0d, 0x00, 0x00, \
  0x30, 0x68, 0x18, 0x34, 0x00, 0x00, \
  0x2c, 0x89, 0x16, 0x44, 0x80, 0x00, \
  0xb0, 0xde, 0xd8, 0x6f, 0x40, 0x00, \
  0x51, 0x84, 0xa8, 0xc2, 0x40, 0x00, \
  0xa2, 0x27, 0x51, 0x13, 0x80, 0x00, \
  0x95, 0x51, 0xca, 0xa8, 0xc0, 0x00, \
  0x4a, 0x1a, 0x25, 0x0d, 0x00, 0x00, \
  0x30, 0x68, 0x18, 0x34, 0x00, 0x00, \
  0x2c, 0x89, 0x16, 0x44, 0x80, 0x00, \
  0x15, 0x8c, 0x0a, 0xc6, 0x00, 0x00, \
  0x8a, 0x47, 0x45, 0x23, 0x80, 0x00, \
  0x25, 0x81, 0x92, 0xc0, 0xc0, 0x00, \
  0x62, 0x12, 0xb1, 0x09, 0x40, 0x00, \
  0x58, 0x58, 0x2c, 0x2c, 0x00, 0x00, \
  0x0e, 0x28, 0x87, 0x14, 0x40, 0x00, \
  0x83, 0x34, 0x41, 0x9a, 0x00, 0x00, \
  0x10, 0x6c, 0xff, 0x60, 0x80, 0x00

#define kMaskRandom34_29 \
  0x15, 0x8c, 0x0a, 0xc6, 0x00, 0x00, \
  0x8a, 0x47, 0x45, 0x23, 0x80, 0x00, \
  0x25, 0x81, 0x92, 0xc0, 0xc0, 0x00, \
  0x62, 0x12, 0xb1, 0x09, 0x40, 0x00, \
  0x58, 0x58, 0x2c, 0x2c, 0x00, 0x00, \
  0x0e, 0x28, 0x87, 0x14, 0x40, 0x00, \
  0x83, 0x34, 0x41, 0x9a, 0x00, 0x00, \
  0x51, 0x84, 0xa8, 0xc2, 0x40, 0x00, \
  0xa2, 0x27, 0x51, 0x13, 0x80, 0x00, \
  0x95, 0x51, 0xca, 0xa8, 0xc0, 0x00, \
  0x4a, 0x1a, 0x25, 0x0d, 0x00, 0x00, \
  0x30, 0x68, 0x18, 0x34, 0x00, 0x00, \
  0x2c, 0x89, 0x16, 0x44, 0x80, 0x00, \
  0xb0, 0xde, 0xd8, 0x6f, 0x40, 0x00, \
  0x15, 0x8c, 0x0a, 0xc6, 0x00, 0x00, \
  0x8a, 0x47, 0x45, 0x23, 0x80, 0x00, \
  0x25, 0x81, 0x92, 0xc0, 0xc0, 0x00, \
  0x62, 0x12, 0xb1, 0x09, 0x40, 0x00, \
  0x58, 0x58, 0x2c, 0x2c, 0x00, 0x00, \
  0x0e, 0x28, 0x87, 0x14, 0x40, 0x00, \
  0x83, 0x34, 0x41, 0x9a, 0x00, 0x00, \
  0x25, 0x2c, 0x12, 0x96, 0x00, 0x00, \
  0x8a, 0x91, 0x45, 0x48, 0x80, 0x00, \
  0x91, 0xc0, 0xc8, 0xe0, 0x40, 0x00, \
  0x68, 0x06, 0xb4, 0x03, 0x40, 0x00, \
  0x32, 0xc8, 0x19, 0x64, 0x00, 0x00, \
  0x43, 0x45, 0x21, 0xa2, 0x80, 0x00, \
  0xc4, 0x30, 0xe2, 0x18, 0x40, 0x00, \
  0x1c, 0xa2, 0x0e, 0x51, 0x00, 0x00

#define kMaskRandom34_3 \
  0xcd, 0xcc, 0x66, 0xe6, 0x00, 0x00, \
  0x97, 0x27, 0x4b, 0x93, 0x80, 0x00, \
  0xb8, 0xd1, 0xdc, 0x68, 0xc0, 0x00

#define kMaskRandom34_30 \
  0x15, 0x8c, 0x0a, 0xc6, 0x00, 0x00, \
  0x8a, 0x47, 0x45, 0x23, 0x80, 0x00, \
  0x25, 0x81, 0x92, 0xc0, 0xc0, 0x00, \
  0x62, 0x12, 0xb1, 0x09, 0x40, 0x00, \
  0x58, 0x58, 0x2c, 0x2c, 0x00, 0x00, \
  0x0e, 0x28, 0x87, 0x14, 0x40, 0x00, \
  0x83, 0x34, 0x41, 0x9a, 0x00, 0x00, \
  0x25, 0x2c, 0x12, 0x96, 0x00, 0x00, \
  0x8a, 0x91, 0x45, 0x48, 0x80, 0x00, \
  0x91, 0xc0, 0xc8, 0xe0, 0x40, 0x00, \
  0x68, 0x06, 0xb4, 0x03, 0x40, 0x00, \
  0x32, 0xc8, 0x19, 0x64, 0x00, 0x00, \
  0x43, 0x45, 0x21, 0xa2, 0x80, 0x00, \
  0xc4, 0x30, 0xe2, 0x18, 0x40, 0x00, \
  0x1c, 0xa2, 0x0e, 0x51, 0x00, 0x00, \
  0x15, 0x8c, 0x0a, 0xc6, 0x00, 0x00, \
  0x8a, 0x47, 0x45, 0x23, 0x80, 0x00, \
  0x25, 0x81, 0x92, 0xc0, 0xc0, 0x00, \
  0x62, 0x12, 0xb1, 0x09, 0x40, 0x00, \
  0x58, 0x58, 0x2c, 0x2c, 0x00, 0x00, \
  0x0e, 0x28, 0x87, 0x14, 0x40, 0x00, \
  0x83, 0x34, 0x41, 0x9a, 0x00, 0x00, \
  0x51, 0x84, 0xa8, 0xc2, 0x40, 0x00, \
  0xa2, 0x27, 0x51, 0x13, 0x80, 0x00, \
  0x95, 0x51, 0xca, 0xa8, 0xc0, 0x00, \
  0x4a, 0x1a, 0x25, 0x0d, 0x00, 0x00, \
  0x30, 0x68, 0x18, 0x34, 0x00, 0x00, \
  0x2c, 0x89, 0x16, 0x44, 0x80, 0x00, \
  0xb0, 0xde, 0xd8, 0x6f, 0x40, 0x00, \
  0x87, 0x93, 0x96, 0xc7, 0x80, 0x00

#define kMaskRandom34_31 \
  0x15, 0x8c, 0x0a, 0xc6, 0x00, 0x00, \
  0x8a, 0x47, 0x45, 0x23, 0x80, 0x00, \
  0x25, 0x81, 0x92, 0xc0, 0xc0, 0x00, \
  0x62, 0x12, 0xb1, 0x09, 0x40, 0x00, \
  0x58, 0x58, 0x2c, 0x2c, 0x00, 0x00, \
  0x0e, 0x28, 0x87, 0x14, 0x40, 0x00, \
  0x83, 0x34, 0x41, 0x9a, 0x00, 0x00, \
  0x25, 0x2c, 0x12, 0x96, 0x00, 0x00, \
  0x8a, 0x91, 0x45, 0x48, 0x80, 0x00, \
  0x91, 0xc0, 0xc8, 0xe0, 0x40, 0x00, \
  0x68, 0x06, 0xb4, 0x03, 0x40, 0x00, \
  0x32, 0xc8, 0x19, 0x64, 0x00, 0x00, \
  0x43, 0x45, 0x21, 0xa2, 0x80, 0x00, \
  0xc4, 0x30, 0xe2, 0x18, 0x40, 0x00, \
  0x1c, 0xa2, 0x0e, 0x51, 0x00, 0x00, \
  0x25, 0x2c, 0x12, 0x96, 0x00, 0x00, \
  0x8a, 0x91, 0x45, 0x48, 0x80, 0x00, \
  0x91, 0xc0, 0xc8, 0xe0, 0x40, 0x00, \
  0x68, 0x06, 0xb4, 0x03, 0x40, 0x00, \
  0x32, 0xc8, 0x19, 0x64, 0x00, 0x00, \
  0x43, 0x45, 0x21, 0xa2, 0x80, 0x00, \
  0xc4, 0x30, 0xe2, 0x18, 0x40, 0x00, \
  0x1c, 0xa2, 0x0e, 0x51, 0x00, 0x00, \
  0x15, 0x8c, 0x0a, 0xc6, 0x00, 0x00, \
  0x8a, 0x47, 0x45, 0x23, 0x80, 0x00, \
  0x25, 0x81, 0x92, 0xc0, 0xc0, 0x00, \
  0x62, 0x12, 0xb1, 0x09, 0x40, 0x00, \
  0x58, 0x58, 0x2c, 0x2c, 0x00, 0x00, \
  0x0e, 0x28, 0x87, 0x14, 0x40, 0x00, \
  0x83, 0x34, 0x41, 0x9a, 0x00, 0x00, \
  0x0a, 0x1c, 0x05, 0x0e, 0x00, 0x00

#define kMaskRandom34_32 \
  0x25, 0x2c, 0x12, 0x96, 0x00, 0x00, \
  0x8a, 0x91, 0x45, 0x48, 0x80, 0x00, \
  0x91, 0xc0, 0xc8, 0xe0, 0x40, 0x00, \
  0x68, 0x06, 0xb4, 0x03, 0x40, 0x00, \
  0x32, 0xc8, 0x19, 0x64, 0x00, 0x00, \
  0x43, 0x45, 0x21, 0xa2, 0x80, 0x00, \
  0xc4, 0x30, 0xe2, 0x18, 0x40, 0x00, \
  0x1c, 0xa2, 0x0e, 0x51, 0x00, 0x00, \
  0x15, 0x8c, 0x0a, 0xc6, 0x00, 0x00, \
  0x8a, 0x47, 0x45, 0x23, 0x80, 0x00, \
  0x25, 0x81, 0x92, 0xc0, 0xc0, 0x00, \
  0x62, 0x12, 0xb1, 0x09, 0x40, 0x00, \
  0x58, 0x58, 0x2c, 0x2c, 0x00, 0x00, \
  0x0e, 0x28, 0x87, 0x14, 0x40, 0x00, \
  0x83, 0x34, 0x41, 0x9a, 0x00, 0x00, \
  0x0a, 0x1c, 0x05, 0x0e, 0x00, 0x00, \
  0x15, 0x8c, 0x0a, 0xc6, 0x00, 0x00, \
  0x8a, 0x47, 0x45, 0x23, 0x80, 0x00, \
  0x25, 0x81, 0x92, 0xc0, 0xc0, 0x00, \
  0x62, 0x12, 0xb1, 0x09, 0x40, 0x00, \
  0x58, 0x58, 0x2c, 0x2c, 0x00, 0x00, \
  0x0e, 0x28, 0x87, 0x14, 0x40, 0x00, \
  0x83, 0x34, 0x41, 0x9a, 0x00, 0x00, \
  0x25, 0x2c, 0x12, 0x96, 0x00, 0x00, \
  0x8a, 0x91, 0x45, 0x48, 0x80, 0x00, \
  0x91, 0xc0, 0xc8, 0xe0, 0x40, 0x00, \
  0x68, 0x06, 0xb4, 0x03, 0x40, 0x00, \
  0x32, 0xc8, 0x19, 0x64, 0x00, 0x00, \
  0x43, 0x45, 0x21, 0xa2, 0x80, 0x00, \
  0xc4, 0x30, 0xe2, 0x18, 0x40, 0x00, \
  0x1c, 0xa2, 0x0e, 0x51, 0x00, 0x00, \
  0xa6, 0x27, 0xa9, 0x4a, 0x40, 0x00

#define kMaskRandom34_33 \
  0x25, 0x2c, 0x12, 0x96, 0x00, 0x00, \
  0x8a, 0x91, 0x45, 0x48, 0x80, 0x00, \
  0x91, 0xc0, 0xc8, 0xe0, 0x40, 0x00, \
  0x68, 0x06, 0xb4, 0x03, 0x40, 0x00, \
  0x32, 0xc8, 0x19, 0x64, 0x00, 0x00, \
  0x43, 0x45, 0x21, 0xa2, 0x80, 0x00, \
  0xc4, 0x30, 0xe2, 0x18, 0x40, 0x00, \
  0x1c, 0xa2, 0x0e, 0x51, 0x00, 0x00, \
  0x15, 0x8c, 0x0a, 0xc6, 0x00, 0x00, \
  0x8a, 0x47, 0x45, 0x23, 0x80, 0x00, \
  0x25, 0x81, 0x92, 0xc0, 0xc0, 0x00, \
  0x62, 0x12, 0xb1, 0x09, 0x40, 0x00, \
  0x58, 0x58, 0x2c, 0x2c, 0x00, 0x00, \
  0x0e, 0x28, 0x87, 0x14, 0x40, 0x00, \
  0x83, 0x34, 0x41, 0x9a, 0x00, 0x00, \
  0x0a, 0x1c, 0x05, 0x0e, 0x00, 0x00, \
  0x25, 0x2c, 0x12, 0x96, 0x00, 0x00, \
  0x8a, 0x91, 0x45, 0x48, 0x80, 0x00, \
  0x91, 0xc0, 0xc8, 0xe0, 0x40, 0x00, \
  0x68, 0x06, 0xb4, 0x03, 0x40, 0x00, \
  0x32, 0xc8, 0x19, 0x64, 0x00, 0x00, \
  0x43, 0x45, 0x21, 0xa2, 0x80, 0x00, \
  0xc4, 0x30, 0xe2, 0x18, 0x40, 0x00, \
  0x1c, 0xa2, 0x0e, 0x51, 0x00, 0x00, \
  0x25, 0x4c, 0x12, 0xa6, 0x00, 0x00, \
  0x8a, 0x66, 0x45, 0x33, 0x00, 0x00, \
  0x91, 0x91, 0x48, 0xc8, 0x80, 0x00, \
  0x68, 0x42, 0xb4, 0x21, 0x40, 0x00, \
  0x32, 0xa4, 0x19, 0x52, 0x00, 0x00, \
  0x43, 0x13, 0x21, 0x89, 0x80, 0x00, \
  0xc4, 0x30, 0xe2, 0x18, 0x40, 0x00, \
  0x1c, 0x88, 0x8e, 0x44, 0x40, 0x00, \
  0x3c, 0x09, 0x1e, 0x04, 0x80, 0x00

#define kMaskRandom34_34 \
  0x25, 0x2c, 0x12, 0x96, 0x00, 0x00, \
  0x8a, 0x91, 0x45, 0x48, 0x80, 0x00, \
  0x91, 0xc0, 0xc8, 0xe0, 0x40, 0x00, \
  0x68, 0x06, 0xb4, 0x03, 0x40, 0x00, \
  0x32, 0xc8, 0x19, 0x64, 0x00, 0x00, \
  0x43, 0x45, 0x21, 0xa2, 0x80, 0x00, \
  0xc4, 0x30, 0xe2, 0x18, 0x40, 0x00, \
  0x1c, 0xa2, 0x0e, 0x51, 0x00, 0x00, \
  0x25, 0x4c, 0x12, 0xa6, 0x00, 0x00, \
  0x8a, 0x66, 0x45, 0x33, 0x00, 0x00, \
  0x91, 0x91, 0x48, 0xc8, 0x80, 0x00, \
  0x68, 0x42, 0xb4, 0x21, 0x40, 0x00, \
  0x32, 0xa4, 0x19, 0x52, 0x00, 0x00, \
  0x43, 0x13, 0x21, 0x89, 0x80, 0x00, \
  0xc4, 0x30, 0xe2, 0x18, 0x40, 0x00, \
  0x1c, 0x88, 0x8e, 0x44, 0x40, 0x00, \
  0x3c, 0x09, 0x1e, 0x04, 0x80, 0x00, \
  0x25, 0x2c, 0x12, 0x96, 0x00, 0x00, \
  0x8a, 0x91, 0x45, 0x48, 0x80, 0x00, \
  0x91, 0xc0, 0xc8, 0xe0, 0x40, 0x00, \
  0x68, 0x06, 0xb4, 0x03, 0x40, 0x00, \
  0x32, 0xc8, 0x19, 0x64, 0x00, 0x00, \
  0x43, 0x45, 0x21, 0xa2, 0x80, 0x00, \
  0xc4, 0x30, 0xe2, 0x18, 0x40, 0x00, \
  0x1c, 0xa2, 0x0e, 0x51, 0x00, 0x00, \
  0x15, 0x8c, 0x0a, 0xc6, 0x00, 0x00, \
  0x8a, 0x47, 0x45, 0x23, 0x80, 0x00, \
  0x25, 0x81, 0x92, 0xc0, 0xc0, 0x00, \
  0x62, 0x12, 0xb1, 0x09, 0x40, 0x00, \
  0x58, 0x58, 0x2c, 0x2c, 0x00, 0x00, \
  0x0e, 0x28, 0x87, 0x14, 0x40, 0x00, \
  0x83, 0x34, 0x41, 0x9a, 0x00, 0x00, \
  0x0a, 0x1c, 0x05, 0x0e, 0x00, 0x00, \
  0x30, 0x3c, 0xb3, 0xe6, 0x80, 0x00

#define kMaskRandom34_4 \
  0xca, 0xec, 0x65, 0x76, 0x00, 0x00, \
  0xa9, 0x67, 0x54, 0xb3, 0x80, 0x00, \
  0x3a, 0xb1, 0x9d, 0x58, 0xc0, 0x00, \
  0x55, 0x5a, 0xaa, 0xad, 0x40, 0x00

#define kMaskRandom34_5 \
  0x55, 0x44, 0xaa, 0xa2, 0x40, 0x00, \
  0x2a, 0x66, 0x15, 0x33, 0x00, 0x00, \
  0x25, 0xa1, 0x92, 0xd0, 0xc0, 0x00, \
  0xe2, 0x12, 0xf1, 0x09, 0x40, 0x00, \
  0x99, 0x98, 0x4c, 0xcc, 0x00, 0x00

#define kMaskRandom34_6 \
  0xd1, 0x4c, 0x68, 0xa6, 0x00, 0x00, \
  0xa2, 0xc5, 0x51, 0x62, 0x80, 0x00, \
  0x95, 0x30, 0xca, 0x98, 0x40, 0x00, \
  0xca, 0x0a, 0xe5, 0x05, 0x40, 0x00, \
  0xa4, 0xaa, 0x52, 0x55, 0x00, 0x00, \
  0x78, 0x15, 0x3c, 0x0a, 0x80, 0x00

#define kMaskRandom34_7 \
  0x15, 0x44, 0x8a, 0xa2, 0x40, 0x00, \
  0x8a, 0x23, 0x45, 0x11, 0x80, 0x00, \
  0x85, 0x91, 0x42, 0xc8, 0x80, 0x00, \
  0x32, 0x0a, 0x99, 0x05, 0x40, 0x00, \
  0x58, 0x34, 0x2c, 0x1a, 0x00, 0x00, \
  0x2c, 0x0d, 0x16, 0x06, 0x80, 0x00, \
  0x43, 0xc8, 0x21, 0xe4, 0x00, 0x00

#define kMaskRandom34_8 \
  0x64, 0x16, 0x32, 0x0b, 0x00, 0x00, \
  0xa2, 0xc2, 0x51, 0x61, 0x00, 0x00, \
  0x51, 0x60, 0xa8, 0xb0, 0x40, 0x00, \
  0x4a, 0x85, 0x25, 0x42, 0x80, 0x00, \
  0x38, 0x4c, 0x1c, 0x26, 0x00, 0x00, \
  0x89, 0x29, 0x44, 0x94, 0x80, 0x00, \
  0x07, 0x11, 0x83, 0x88, 0xc0, 0x00, \
  0x94, 0xb0, 0x4a, 0x58, 0x00, 0x00

#define kMaskRandom34_9 \
  0x8e, 0xcc, 0x47, 0x66, 0x00, 0x00, \
  0x6a, 0x2b, 0x35, 0x15, 0x80, 0x00, \
  0x36, 0x32, 0x9b, 0x19, 0x40, 0x00, \
  0xd1, 0x25, 0xe8, 0x92, 0xc0, 0x00, \
  0x55, 0x8c, 0xaa, 0xc6, 0x40, 0x00, \
  0xaa, 0x27, 0x55, 0x13, 0x80, 0x00, \
  0xa5, 0x32, 0xd2, 0x99, 0x40, 0x00, \
  0x62, 0x61, 0xb1, 0x30, 0xc0, 0x00, \
  0x3c, 0x5c, 0x1e, 0x2e, 0x00, 0x00

#define kMaskRandom35_1 \
  0xff, 0xff, 0xff, 0xff, 0xe0, 0x00

#define kMaskRandom35_10 \
  0x55, 0x8c, 0xc6, 0x63, 0x20, 0x00, \
  0xaa, 0x27, 0x13, 0x89, 0xc0, 0x00, \
  0xa5, 0x32, 0x99, 0x4c, 0xa0, 0x00, \
  0x62, 0x61, 0xb0, 0xd8, 0x60, 0x00, \
  0x3c, 0x5c, 0x2e, 0x17, 0x00, 0x00, \
  0x8e, 0xcc, 0x66, 0x33, 0x00, 0x00, \
  0x6a, 0x2b, 0x15, 0x8a, 0xc0, 0x00, \
  0x36, 0x32, 0x99, 0x4c, 0xa0, 0x00, \
  0xd1, 0x25, 0x92, 0xc9, 0x60, 0x00, \
  0xc8, 0x02, 0xfe, 0xce, 0xe0, 0x00

#define kMaskRandom35_11 \
  0x55, 0x8c, 0xc6, 0x63, 0x20, 0x00, \
  0xaa, 0x27, 0x13, 0x89, 0xc0, 0x00, \
  0xa5, 0x32, 0x99, 0x4c, 0xa0, 0x00, \
  0x62, 0x61, 0xb0, 0xd8, 0x60, 0x00, \
  0x3c, 0x5c, 0x2e, 0x17, 0x00, 0x00, \
  0x51, 0x84, 0xc2, 0x61, 0x20, 0x00, \
  0xa2, 0x27, 0x13, 0x89, 0xc0, 0x00, \
  0x95, 0x51, 0xa8, 0xd4, 0x60, 0x00, \
  0x4a, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0x30, 0x68, 0x34, 0x1a, 0x00, 0x00, \
  0x2c, 0x89, 0x44, 0xa2, 0x40, 0x00

#define kMaskRandom35_12 \
  0x51, 0x84, 0xc2, 0x61, 0x20, 0x00, \
  0xa2, 0x27, 0x13, 0x89, 0xc0, 0x00, \
  0x95, 0x51, 0xa8, 0xd4, 0x60, 0x00, \
  0x4a, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0x30, 0x68, 0x34, 0x1a, 0x00, 0x00, \
  0x2c, 0x89, 0x44, 0xa2, 0x40, 0x00, \
  0x55, 0x8c, 0xc6, 0x63, 0x20, 0x00, \
  0xaa, 0x27, 0x13, 0x89, 0xc0, 0x00, \
  0xa5, 0x32, 0x99, 0x4c, 0xa0, 0x00, \
  0x62, 0x61, 0xb0, 0xd8, 0x60, 0x00, \
  0x3c, 0x5c, 0x2e, 0x17, 0x00, 0x00, \
  0x51, 0x35, 0x2d, 0x86, 0x20, 0x00

#define kMaskRandom35_13 \
  0x51, 0x84, 0xc2, 0x61, 0x20, 0x00, \
  0xa2, 0x27, 0x13, 0x89, 0xc0, 0x00, \
  0x95, 0x51, 0xa8, 0xd4, 0x60, 0x00, \
  0x4a, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0x30, 0x68, 0x34, 0x1a, 0x00, 0x00, \
  0x2c, 0x89, 0x44, 0xa2, 0x40, 0x00, \
  0x15, 0x8c, 0x46, 0x23, 0x00, 0x00, \
  0x8a, 0x47, 0x23, 0x91, 0xc0, 0x00, \
  0x25, 0x81, 0xc0, 0xe0, 0x60, 0x00, \
  0x62, 0x12, 0x89, 0x44, 0xa0, 0x00, \
  0x58, 0x58, 0x2c, 0x16, 0x00, 0x00, \
  0x0e, 0x28, 0x94, 0x4a, 0x20, 0x00, \
  0x83, 0x34, 0x1a, 0x0d, 0x00, 0x00

#define kMaskRandom35_14 \
  0x15, 0x8c, 0x46, 0x23, 0x00, 0x00, \
  0x8a, 0x47, 0x23, 0x91, 0xc0, 0x00, \
  0x25, 0x81, 0xc0, 0xe0, 0x60, 0x00, \
  0x62, 0x12, 0x89, 0x44, 0xa0, 0x00, \
  0x58, 0x58, 0x2c, 0x16, 0x00, 0x00, \
  0x0e, 0x28, 0x94, 0x4a, 0x20, 0x00, \
  0x83, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x51, 0x84, 0xc2, 0x61, 0x20, 0x00, \
  0xa2, 0x27, 0x13, 0x89, 0xc0, 0x00, \
  0x95, 0x51, 0xa8, 0xd4, 0x60, 0x00, \
  0x4a, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0x30, 0x68, 0x34, 0x1a, 0x00, 0x00, \
  0x2c, 0x89, 0x44, 0xa2, 0x40, 0x00, \
  0xb0, 0xde, 0xbf, 0xa7, 0xe0, 0x00

#define kMaskRandom35_15 \
  0x15, 0x8c, 0x46, 0x23, 0x00, 0x00, \
  0x8a, 0x47, 0x23, 0x91, 0xc0, 0x00, \
  0x25, 0x81, 0xc0, 0xe0, 0x60, 0x00, \
  0x62, 0x12, 0x89, 0x44, 0xa0, 0x00, \
  0x58, 0x58, 0x2c, 0x16, 0x00, 0x00, \
  0x0e, 0x28, 0x94, 0x4a, 0x20, 0x00, \
  0x83, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x25, 0x2c, 0x16, 0x0b, 0x00, 0x00, \
  0x8a, 0x91, 0x48, 0xa4, 0x40, 0x00, \
  0x91, 0xc0, 0xe0, 0x70, 0x20, 0x00, \
  0x68, 0x06, 0x83, 0x41, 0xa0, 0x00, \
  0x32, 0xc8, 0x64, 0x32, 0x00, 0x00, \
  0x43, 0x45, 0x22, 0x91, 0x40, 0x00, \
  0xc4, 0x30, 0x98, 0x4c, 0x20, 0x00, \
  0x1c, 0xa2, 0x51, 0x28, 0x80, 0x00

#define kMaskRandom35_16 \
  0x25, 0x2c, 0x16, 0x0b, 0x00, 0x00, \
  0x8a, 0x91, 0x48, 0xa4, 0x40, 0x00, \
  0x91, 0xc0, 0xe0, 0x70, 0x20, 0x00, \
  0x68, 0x06, 0x83, 0x41, 0xa0, 0x00, \
  0x32, 0xc8, 0x64, 0x32, 0x00, 0x00, \
  0x43, 0x45, 0x22, 0x91, 0x40, 0x00, \
  0xc4, 0x30, 0x98, 0x4c, 0x20, 0x00, \
  0x1c, 0xa2, 0x51, 0x28, 0x80, 0x00, \
  0x15, 0x8c, 0x46, 0x23, 0x00, 0x00, \
  0x8a, 0x47, 0x23, 0x91, 0xc0, 0x00, \
  0x25, 0x81, 0xc0, 0xe0, 0x60, 0x00, \
  0x62, 0x12, 0x89, 0x44, 0xa0, 0x00, \
  0x58, 0x58, 0x2c, 0x16, 0x00, 0x00, \
  0x0e, 0x28, 0x94, 0x4a, 0x20, 0x00, \
  0x83, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x0a, 0x1c, 0x77, 0xf9, 0x00, 0x00

#define kMaskRandom35_17 \
  0x25, 0x2c, 0x16, 0x0b, 0x00, 0x00, \
  0x8a, 0x91, 0x48, 0xa4, 0x40, 0x00, \
  0x91, 0xc0, 0xe0, 0x70, 0x20, 0x00, \
  0x68, 0x06, 0x83, 0x41, 0xa0, 0x00, \
  0x32, 0xc8, 0x64, 0x32, 0x00, 0x00, \
  0x43, 0x45, 0x22, 0x91, 0x40, 0x00, \
  0xc4, 0x30, 0x98, 0x4c, 0x20, 0x00, \
  0x1c, 0xa2, 0x51, 0x28, 0x80, 0x00, \
  0x25, 0x4c, 0x26, 0x13, 0x00, 0x00, \
  0x8a, 0x66, 0x33, 0x19, 0x80, 0x00, \
  0x91, 0x91, 0x48, 0xa4, 0x40, 0x00, \
  0x68, 0x42, 0xa1, 0x50, 0xa0, 0x00, \
  0x32, 0xa4, 0x52, 0x29, 0x00, 0x00, \
  0x43, 0x13, 0x09, 0x84, 0xc0, 0x00, \
  0xc4, 0x30, 0x98, 0x4c, 0x20, 0x00, \
  0x1c, 0x88, 0xc4, 0x62, 0x20, 0x00, \
  0x3c, 0x09, 0x04, 0x82, 0x40, 0x00

#define kMaskRandom35_18 \
  0x8e, 0xcc, 0x22, 0x51, 0x20, 0x00, \
  0x6a, 0x2b, 0x33, 0x13, 0x00, 0x00, \
  0x36, 0x32, 0xc8, 0x24, 0xa0, 0x00, \
  0xd1, 0x25, 0x80, 0xd2, 0xc0, 0x00, \
  0x55, 0x8c, 0x87, 0x09, 0x40, 0x00, \
  0xaa, 0x27, 0x09, 0x85, 0x80, 0x00, \
  0xa5, 0x32, 0x90, 0x68, 0x20, 0x00, \
  0x62, 0x61, 0xe1, 0x28, 0x80, 0x00, \
  0x3c, 0x5c, 0x14, 0x86, 0x40, 0x00, \
  0x64, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0xa2, 0xc2, 0x61, 0x30, 0x80, 0x00, \
  0x51, 0x60, 0xb0, 0x58, 0x20, 0x00, \
  0x4a, 0x85, 0x42, 0xa1, 0x40, 0x00, \
  0x38, 0x4c, 0x26, 0x13, 0x00, 0x00, \
  0x89, 0x29, 0x14, 0x8a, 0x40, 0x00, \
  0x07, 0x11, 0x88, 0xc4, 0x60, 0x00, \
  0x94, 0xb0, 0x58, 0x2c, 0x00, 0x00, \
  0x40, 0xc9, 0x65, 0xbe, 0xc0, 0x00

#define kMaskRandom35_19 \
  0x8e, 0xcc, 0x22, 0x51, 0x20, 0x00, \
  0x6a, 0x2b, 0x33, 0x13, 0x00, 0x00, \
  0x36, 0x32, 0xc8, 0x24, 0xa0, 0x00, \
  0xd1, 0x25, 0x80, 0xd2, 0xc0, 0x00, \
  0x55, 0x8c, 0x87, 0x09, 0x40, 0x00, \
  0xaa, 0x27, 0x09, 0x85, 0x80, 0x00, \
  0xa5, 0x32, 0x90, 0x68, 0x20, 0x00, \
  0x62, 0x61, 0xe1, 0x28, 0x80, 0x00, \
  0x3c, 0x5c, 0x14, 0x86, 0x40, 0x00, \
  0x55, 0x8c, 0xc6, 0x63, 0x20, 0x00, \
  0xaa, 0x27, 0x13, 0x89, 0xc0, 0x00, \
  0xa5, 0x32, 0x99, 0x4c, 0xa0, 0x00, \
  0x62, 0x61, 0xb0, 0xd8, 0x60, 0x00, \
  0x3c, 0x5c, 0x2e, 0x17, 0x00, 0x00, \
  0x8e, 0xcc, 0x66, 0x33, 0x00, 0x00, \
  0x6a, 0x2b, 0x15, 0x8a, 0xc0, 0x00, \
  0x36, 0x32, 0x99, 0x4c, 0xa0, 0x00, \
  0xd1, 0x25, 0x92, 0xc9, 0x60, 0x00, \
  0xc8, 0x02, 0xfe, 0xce, 0xe0, 0x00

#define kMaskRandom35_2 \
  0xce, 0xce, 0x67, 0x33, 0x80, 0x00, \
  0xb9, 0x39, 0x9c, 0xce, 0x60, 0x00

#define kMaskRandom35_20 \
  0x55, 0x8c, 0xc6, 0x63, 0x20, 0x00, \
  0xaa, 0x27, 0x13, 0x89, 0xc0, 0x00, \
  0xa5, 0x32, 0x99, 0x4c, 0xa0, 0x00, \
  0x62, 0x61, 0xb0, 0xd8, 0x60, 0x00, \
  0x3c, 0x5c, 0x2e, 0x17, 0x00, 0x00, \
  0x8e, 0xcc, 0x66, 0x33, 0x00, 0x00, \
  0x6a, 0x2b, 0x15, 0x8a, 0xc0, 0x00, \
  0x36, 0x32, 0x99, 0x4c, 0xa0, 0x00, \
  0xd1, 0x25, 0x92, 0xc9, 0x60, 0x00, \
  0xc8, 0x02, 0xfe, 0xce, 0xe0, 0x00, \
  0x8e, 0xcc, 0x22, 0x51, 0x20, 0x00, \
  0x6a, 0x2b, 0x33, 0x13, 0x00, 0x00, \
  0x36, 0x32, 0xc8, 0x24, 0xa0, 0x00, \
  0xd1, 0x25, 0x80, 0xd2, 0xc0, 0x00, \
  0x55, 0x8c, 0x87, 0x09, 0x40, 0x00, \
  0xaa, 0x27, 0x09, 0x85, 0x80, 0x00, \
  0xa5, 0x32, 0x90, 0x68, 0x20, 0x00, \
  0x62, 0x61, 0xe1, 0x28, 0x80, 0x00, \
  0x3c, 0x5c, 0x14, 0x86, 0x40, 0x00, \
  0x63, 0x36, 0x5c, 0xd3, 0x80, 0x00

#define kMaskRandom35_21 \
  0x55, 0x8c, 0xc6, 0x63, 0x20, 0x00, \
  0xaa, 0x27, 0x13, 0x89, 0xc0, 0x00, \
  0xa5, 0x32, 0x99, 0x4c, 0xa0, 0x00, \
  0x62, 0x61, 0xb0, 0xd8, 0x60, 0x00, \
  0x3c, 0x5c, 0x2e, 0x17, 0x00, 0x00, \
  0x8e, 0xcc, 0x66, 0x33, 0x00, 0x00, \
  0x6a, 0x2b, 0x15, 0x8a, 0xc0, 0x00, \
  0x36, 0x32, 0x99, 0x4c, 0xa0, 0x00, \
  0xd1, 0x25, 0x92, 0xc9, 0x60, 0x00, \
  0xc8, 0x02, 0xfe, 0xce, 0xe0, 0x00, \
  0x55, 0x8c, 0xc6, 0x63, 0x20, 0x00, \
  0xaa, 0x27, 0x13, 0x89, 0xc0, 0x00, \
  0xa5, 0x32, 0x99, 0x4c, 0xa0, 0x00, \
  0x62, 0x61, 0xb0, 0xd8, 0x60, 0x00, \
  0x3c, 0x5c, 0x2e, 0x17, 0x00, 0x00, \
  0x51, 0x84, 0xc2, 0x61, 0x20, 0x00, \
  0xa2, 0x27, 0x13, 0x89, 0xc0, 0x00, \
  0x95, 0x51, 0xa8, 0xd4, 0x60, 0x00, \
  0x4a, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0x30, 0x68, 0x34, 0x1a, 0x00, 0x00, \
  0x2c, 0x89, 0x44, 0xa2, 0x40, 0x00

#define kMaskRandom35_22 \
  0x55, 0x8c, 0xc6, 0x63, 0x20, 0x00, \
  0xaa, 0x27, 0x13, 0x89, 0xc0, 0x00, \
  0xa5, 0x32, 0x99, 0x4c, 0xa0, 0x00, \
  0x62, 0x61, 0xb0, 0xd8, 0x60, 0x00, \
  0x3c, 0x5c, 0x2e, 0x17, 0x00, 0x00, \
  0x51, 0x84, 0xc2, 0x61, 0x20, 0x00, \
  0xa2, 0x27, 0x13, 0x89, 0xc0, 0x00, \
  0x95, 0x51, 0xa8, 0xd4, 0x60, 0x00, \
  0x4a, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0x30, 0x68, 0x34, 0x1a, 0x00, 0x00, \
  0x2c, 0x89, 0x44, 0xa2, 0x40, 0x00, \
  0x55, 0x8c, 0xc6, 0x63, 0x20, 0x00, \
  0xaa, 0x27, 0x13, 0x89, 0xc0, 0x00, \
  0xa5, 0x32, 0x99, 0x4c, 0xa0, 0x00, \
  0x62, 0x61, 0xb0, 0xd8, 0x60, 0x00, \
  0x3c, 0x5c, 0x2e, 0x17, 0x00, 0x00, \
  0x8e, 0xcc, 0x66, 0x33, 0x00, 0x00, \
  0x6a, 0x2b, 0x15, 0x8a, 0xc0, 0x00, \
  0x36, 0x32, 0x99, 0x4c, 0xa0, 0x00, \
  0xd1, 0x25, 0x92, 0xc9, 0x60, 0x00, \
  0xc8, 0x02, 0xfe, 0xce, 0xe0, 0x00, \
  0x84, 0xc7, 0xbc, 0xcc, 0x00, 0x00

#define kMaskRandom35_23 \
  0x55, 0x8c, 0xc6, 0x63, 0x20, 0x00, \
  0xaa, 0x27, 0x13, 0x89, 0xc0, 0x00, \
  0xa5, 0x32, 0x99, 0x4c, 0xa0, 0x00, \
  0x62, 0x61, 0xb0, 0xd8, 0x60, 0x00, \
  0x3c, 0x5c, 0x2e, 0x17, 0x00, 0x00, \
  0x51, 0x84, 0xc2, 0x61, 0x20, 0x00, \
  0xa2, 0x27, 0x13, 0x89, 0xc0, 0x00, \
  0x95, 0x51, 0xa8, 0xd4, 0x60, 0x00, \
  0x4a, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0x30, 0x68, 0x34, 0x1a, 0x00, 0x00, \
  0x2c, 0x89, 0x44, 0xa2, 0x40, 0x00, \
  0x51, 0x84, 0xc2, 0x61, 0x20, 0x00, \
  0xa2, 0x27, 0x13, 0x89, 0xc0, 0x00, \
  0x95, 0x51, 0xa8, 0xd4, 0x60, 0x00, \
  0x4a, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0x30, 0x68, 0x34, 0x1a, 0x00, 0x00, \
  0x2c, 0x89, 0x44, 0xa2, 0x40, 0x00, \
  0x55, 0x8c, 0xc6, 0x63, 0x20, 0x00, \
  0xaa, 0x27, 0x13, 0x89, 0xc0, 0x00, \
  0xa5, 0x32, 0x99, 0x4c, 0xa0, 0x00, \
  0x62, 0x61, 0xb0, 0xd8, 0x60, 0x00, \
  0x3c, 0x5c, 0x2e, 0x17, 0x00, 0x00, \
  0x51, 0x35, 0x2d, 0x86, 0x20, 0x00

#define kMaskRandom35_24 \
  0x51, 0x84, 0xc2, 0x61, 0x20, 0x00, \
  0xa2, 0x27, 0x13, 0x89, 0xc0, 0x00, \
  0x95, 0x51, 0xa8, 0xd4, 0x60, 0x00, \
  0x4a, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0x30, 0x68, 0x34, 0x1a, 0x00, 0x00, \
  0x2c, 0x89, 0x44, 0xa2, 0x40, 0x00, \
  0x55, 0x8c, 0xc6, 0x63, 0x20, 0x00, \
  0xaa, 0x27, 0x13, 0x89, 0xc0, 0x00, \
  0xa5, 0x32, 0x99, 0x4c, 0xa0, 0x00, \
  0x62, 0x61, 0xb0, 0xd8, 0x60, 0x00, \
  0x3c, 0x5c, 0x2e, 0x17, 0x00, 0x00, \
  0x51, 0x35, 0x2d, 0x86, 0x20, 0x00, \
  0x55, 0x8c, 0xc6, 0x63, 0x20, 0x00, \
  0xaa, 0x27, 0x13, 0x89, 0xc0, 0x00, \
  0xa5, 0x32, 0x99, 0x4c, 0xa0, 0x00, \
  0x62, 0x61, 0xb0, 0xd8, 0x60, 0x00, \
  0x3c, 0x5c, 0x2e, 0x17, 0x00, 0x00, \
  0x51, 0x84, 0xc2, 0x61, 0x20, 0x00, \
  0xa2, 0x27, 0x13, 0x89, 0xc0, 0x00, \
  0x95, 0x51, 0xa8, 0xd4, 0x60, 0x00, \
  0x4a, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0x30, 0x68, 0x34, 0x1a, 0x00, 0x00, \
  0x2c, 0x89, 0x44, 0xa2, 0x40, 0x00, \
  0x0d, 0xfb, 0x06, 0x89, 0x00, 0x00

#define kMaskRandom35_25 \
  0x51, 0x84, 0xc2, 0x61, 0x20, 0x00, \
  0xa2, 0x27, 0x13, 0x89, 0xc0, 0x00, \
  0x95, 0x51, 0xa8, 0xd4, 0x60, 0x00, \
  0x4a, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0x30, 0x68, 0x34, 0x1a, 0x00, 0x00, \
  0x2c, 0x89, 0x44, 0xa2, 0x40, 0x00, \
  0x55, 0x8c, 0xc6, 0x63, 0x20, 0x00, \
  0xaa, 0x27, 0x13, 0x89, 0xc0, 0x00, \
  0xa5, 0x32, 0x99, 0x4c, 0xa0, 0x00, \
  0x62, 0x61, 0xb0, 0xd8, 0x60, 0x00, \
  0x3c, 0x5c, 0x2e, 0x17, 0x00, 0x00, \
  0x51, 0x35, 0x2d, 0x86, 0x20, 0x00, \
  0x51, 0x84, 0xc2, 0x61, 0x20, 0x00, \
  0xa2, 0x27, 0x13, 0x89, 0xc0, 0x00, \
  0x95, 0x51, 0xa8, 0xd4, 0x60, 0x00, \
  0x4a, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0x30, 0x68, 0x34, 0x1a, 0x00, 0x00, \
  0x2c, 0x89, 0x44, 0xa2, 0x40, 0x00, \
  0x15, 0x8c, 0x46, 0x23, 0x00, 0x00, \
  0x8a, 0x47, 0x23, 0x91, 0xc0, 0x00, \
  0x25, 0x81, 0xc0, 0xe0, 0x60, 0x00, \
  0x62, 0x12, 0x89, 0x44, 0xa0, 0x00, \
  0x58, 0x58, 0x2c, 0x16, 0x00, 0x00, \
  0x0e, 0x28, 0x94, 0x4a, 0x20, 0x00, \
  0x83, 0x34, 0x1a, 0x0d, 0x00, 0x00

#define kMaskRandom35_26 \
  0x51, 0x84, 0xc2, 0x61, 0x20, 0x00, \
  0xa2, 0x27, 0x13, 0x89, 0xc0, 0x00, \
  0x95, 0x51, 0xa8, 0xd4, 0x60, 0x00, \
  0x4a, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0x30, 0x68, 0x34, 0x1a, 0x00, 0x00, \
  0x2c, 0x89, 0x44, 0xa2, 0x40, 0x00, \
  0x15, 0x8c, 0x46, 0x23, 0x00, 0x00, \
  0x8a, 0x47, 0x23, 0x91, 0xc0, 0x00, \
  0x25, 0x81, 0xc0, 0xe0, 0x60, 0x00, \
  0x62, 0x12, 0x89, 0x44, 0xa0, 0x00, \
  0x58, 0x58, 0x2c, 0x16, 0x00, 0x00, \
  0x0e, 0x28, 0x94, 0x4a, 0x20, 0x00, \
  0x83, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x51, 0x84, 0xc2, 0x61, 0x20, 0x00, \
  0xa2, 0x27, 0x13, 0x89, 0xc0, 0x00, \
  0x95, 0x51, 0xa8, 0xd4, 0x60, 0x00, \
  0x4a, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0x30, 0x68, 0x34, 0x1a, 0x00, 0x00, \
  0x2c, 0x89, 0x44, 0xa2, 0x40, 0x00, \
  0x55, 0x8c, 0xc6, 0x63, 0x20, 0x00, \
  0xaa, 0x27, 0x13, 0x89, 0xc0, 0x00, \
  0xa5, 0x32, 0x99, 0x4c, 0xa0, 0x00, \
  0x62, 0x61, 0xb0, 0xd8, 0x60, 0x00, \
  0x3c, 0x5c, 0x2e, 0x17, 0x00, 0x00, \
  0x51, 0x35, 0x2d, 0x86, 0x20, 0x00, \
  0xc4, 0x57, 0x70, 0x47, 0x40, 0x00

#define kMaskRandom35_27 \
  0x51, 0x84, 0xc2, 0x61, 0x20, 0x00, \
  0xa2, 0x27, 0x13, 0x89, 0xc0, 0x00, \
  0x95, 0x51, 0xa8, 0xd4, 0x60, 0x00, \
  0x4a, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0x30, 0x68, 0x34, 0x1a, 0x00, 0x00, \
  0x2c, 0x89, 0x44, 0xa2, 0x40, 0x00, \
  0x15, 0x8c, 0x46, 0x23, 0x00, 0x00, \
  0x8a, 0x47, 0x23, 0x91, 0xc0, 0x00, \
  0x25, 0x81, 0xc0, 0xe0, 0x60, 0x00, \
  0x62, 0x12, 0x89, 0x44, 0xa0, 0x00, \
  0x58, 0x58, 0x2c, 0x16, 0x00, 0x00, \
  0x0e, 0x28, 0x94, 0x4a, 0x20, 0x00, \
  0x83, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x15, 0x8c, 0x46, 0x23, 0x00, 0x00, \
  0x8a, 0x47, 0x23, 0x91, 0xc0, 0x00, \
  0x25, 0x81, 0xc0, 0xe0, 0x60, 0x00, \
  0x62, 0x12, 0x89, 0x44, 0xa0, 0x00, \
  0x58, 0x58, 0x2c, 0x16, 0x00, 0x00, \
  0x0e, 0x28, 0x94, 0x4a, 0x20, 0x00, \
  0x83, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x51, 0x84, 0xc2, 0x61, 0x20, 0x00, \
  0xa2, 0x27, 0x13, 0x89, 0xc0, 0x00, \
  0x95, 0x51, 0xa8, 0xd4, 0x60, 0x00, \
  0x4a, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0x30, 0x68, 0x34, 0x1a, 0x00, 0x00, \
  0x2c, 0x89, 0x44, 0xa2, 0x40, 0x00, \
  0xb0, 0xde, 0xbf, 0xa7, 0xe0, 0x00

#define kMaskRandom35_28 \
  0x15, 0x8c, 0x46, 0x23, 0x00, 0x00, \
  0x8a, 0x47, 0x23, 0x91, 0xc0, 0x00, \
  0x25, 0x81, 0xc0, 0xe0, 0x60, 0x00, \
  0x62, 0x12, 0x89, 0x44, 0xa0, 0x00, \
  0x58, 0x58, 0x2c, 0x16, 0x00, 0x00, \
  0x0e, 0x28, 0x94, 0x4a, 0x20, 0x00, \
  0x83, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x51, 0x84, 0xc2, 0x61, 0x20, 0x00, \
  0xa2, 0x27, 0x13, 0x89, 0xc0, 0x00, \
  0x95, 0x51, 0xa8, 0xd4, 0x60, 0x00, \
  0x4a, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0x30, 0x68, 0x34, 0x1a, 0x00, 0x00, \
  0x2c, 0x89, 0x44, 0xa2, 0x40, 0x00, \
  0xb0, 0xde, 0xbf, 0xa7, 0xe0, 0x00, \
  0x51, 0x84, 0xc2, 0x61, 0x20, 0x00, \
  0xa2, 0x27, 0x13, 0x89, 0xc0, 0x00, \
  0x95, 0x51, 0xa8, 0xd4, 0x60, 0x00, \
  0x4a, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0x30, 0x68, 0x34, 0x1a, 0x00, 0x00, \
  0x2c, 0x89, 0x44, 0xa2, 0x40, 0x00, \
  0x15, 0x8c, 0x46, 0x23, 0x00, 0x00, \
  0x8a, 0x47, 0x23, 0x91, 0xc0, 0x00, \
  0x25, 0x81, 0xc0, 0xe0, 0x60, 0x00, \
  0x62, 0x12, 0x89, 0x44, 0xa0, 0x00, \
  0x58, 0x58, 0x2c, 0x16, 0x00, 0x00, \
  0x0e, 0x28, 0x94, 0x4a, 0x20, 0x00, \
  0x83, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x34, 0x4a, 0x80, 0x94, 0x40, 0x00

#define kMaskRandom35_29 \
  0x15, 0x8c, 0x46, 0x23, 0x00, 0x00, \
  0x8a, 0x47, 0x23, 0x91, 0xc0, 0x00, \
  0x25, 0x81, 0xc0, 0xe0, 0x60, 0x00, \
  0x62, 0x12, 0x89, 0x44, 0xa0, 0x00, \
  0x58, 0x58, 0x2c, 0x16, 0x00, 0x00, \
  0x0e, 0x28, 0x94, 0x4a, 0x20, 0x00, \
  0x83, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x51, 0x84, 0xc2, 0x61, 0x20, 0x00, \
  0xa2, 0x27, 0x13, 0x89, 0xc0, 0x00, \
  0x95, 0x51, 0xa8, 0xd4, 0x60, 0x00, \
  0x4a, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0x30, 0x68, 0x34, 0x1a, 0x00, 0x00, \
  0x2c, 0x89, 0x44, 0xa2, 0x40, 0x00, \
  0xb0, 0xde, 0xbf, 0xa7, 0xe0, 0x00, \
  0x15, 0x8c, 0x46, 0x23, 0x00, 0x00, \
  0x8a, 0x47, 0x23, 0x91, 0xc0, 0x00, \
  0x25, 0x81, 0xc0, 0xe0, 0x60, 0x00, \
  0x62, 0x12, 0x89, 0x44, 0xa0, 0x00, \
  0x58, 0x58, 0x2c, 0x16, 0x00, 0x00, \
  0x0e, 0x28, 0x94, 0x4a, 0x20, 0x00, \
  0x83, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x25, 0x2c, 0x16, 0x0b, 0x00, 0x00, \
  0x8a, 0x91, 0x48, 0xa4, 0x40, 0x00, \
  0x91, 0xc0, 0xe0, 0x70, 0x20, 0x00, \
  0x68, 0x06, 0x83, 0x41, 0xa0, 0x00, \
  0x32, 0xc8, 0x64, 0x32, 0x00, 0x00, \
  0x43, 0x45, 0x22, 0x91, 0x40, 0x00, \
  0xc4, 0x30, 0x98, 0x4c, 0x20, 0x00, \
  0x1c, 0xa2, 0x51, 0x28, 0x80, 0x00

#define kMaskRandom35_3 \
  0xcd, 0xcc, 0x66, 0x33, 0x00, 0x00, \
  0x97, 0x27, 0x13, 0x8a, 0xc0, 0x00, \
  0xb8, 0xd1, 0xc9, 0x64, 0xa0, 0x00

#define kMaskRandom35_30 \
  0x15, 0x8c, 0x46, 0x23, 0x00, 0x00, \
  0x8a, 0x47, 0x23, 0x91, 0xc0, 0x00, \
  0x25, 0x81, 0xc0, 0xe0, 0x60, 0x00, \
  0x62, 0x12, 0x89, 0x44, 0xa0, 0x00, \
  0x58, 0x58, 0x2c, 0x16, 0x00, 0x00, \
  0x0e, 0x28, 0x94, 0x4a, 0x20, 0x00, \
  0x83, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x25, 0x2c, 0x16, 0x0b, 0x00, 0x00, \
  0x8a, 0x91, 0x48, 0xa4, 0x40, 0x00, \
  0x91, 0xc0, 0xe0, 0x70, 0x20, 0x00, \
  0x68, 0x06, 0x83, 0x41, 0xa0, 0x00, \
  0x32, 0xc8, 0x64, 0x32, 0x00, 0x00, \
  0x43, 0x45, 0x22, 0x91, 0x40, 0x00, \
  0xc4, 0x30, 0x98, 0x4c, 0x20, 0x00, \
  0x1c, 0xa2, 0x51, 0x28, 0x80, 0x00, \
  0x15, 0x8c, 0x46, 0x23, 0x00, 0x00, \
  0x8a, 0x47, 0x23, 0x91, 0xc0, 0x00, \
  0x25, 0x81, 0xc0, 0xe0, 0x60, 0x00, \
  0x62, 0x12, 0x89, 0x44, 0xa0, 0x00, \
  0x58, 0x58, 0x2c, 0x16, 0x00, 0x00, \
  0x0e, 0x28, 0x94, 0x4a, 0x20, 0x00, \
  0x83, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x51, 0x84, 0xc2, 0x61, 0x20, 0x00, \
  0xa2, 0x27, 0x13, 0x89, 0xc0, 0x00, \
  0x95, 0x51, 0xa8, 0xd4, 0x60, 0x00, \
  0x4a, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0x30, 0x68, 0x34, 0x1a, 0x00, 0x00, \
  0x2c, 0x89, 0x44, 0xa2, 0x40, 0x00, \
  0xb0, 0xde, 0xbf, 0xa7, 0xe0, 0x00, \
  0x32, 0x1b, 0x9f, 0x09, 0x20, 0x00

#define kMaskRandom35_31 \
  0x15, 0x8c, 0x46, 0x23, 0x00, 0x00, \
  0x8a, 0x47, 0x23, 0x91, 0xc0, 0x00, \
  0x25, 0x81, 0xc0, 0xe0, 0x60, 0x00, \
  0x62, 0x12, 0x89, 0x44, 0xa0, 0x00, \
  0x58, 0x58, 0x2c, 0x16, 0x00, 0x00, \
  0x0e, 0x28, 0x94, 0x4a, 0x20, 0x00, \
  0x83, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x25, 0x2c, 0x16, 0x0b, 0x00, 0x00, \
  0x8a, 0x91, 0x48, 0xa4, 0x40, 0x00, \
  0x91, 0xc0, 0xe0, 0x70, 0x20, 0x00, \
  0x68, 0x06, 0x83, 0x41, 0xa0, 0x00, \
  0x32, 0xc8, 0x64, 0x32, 0x00, 0x00, \
  0x43, 0x45, 0x22, 0x91, 0x40, 0x00, \
  0xc4, 0x30, 0x98, 0x4c, 0x20, 0x00, \
  0x1c, 0xa2, 0x51, 0x28, 0x80, 0x00, \
  0x25, 0x2c, 0x16, 0x0b, 0x00, 0x00, \
  0x8a, 0x91, 0x48, 0xa4, 0x40, 0x00, \
  0x91, 0xc0, 0xe0, 0x70, 0x20, 0x00, \
  0x68, 0x06, 0x83, 0x41, 0xa0, 0x00, \
  0x32, 0xc8, 0x64, 0x32, 0x00, 0x00, \
  0x43, 0x45, 0x22, 0x91, 0x40, 0x00, \
  0xc4, 0x30, 0x98, 0x4c, 0x20, 0x00, \
  0x1c, 0xa2, 0x51, 0x28, 0x80, 0x00, \
  0x15, 0x8c, 0x46, 0x23, 0x00, 0x00, \
  0x8a, 0x47, 0x23, 0x91, 0xc0, 0x00, \
  0x25, 0x81, 0xc0, 0xe0, 0x60, 0x00, \
  0x62, 0x12, 0x89, 0x44, 0xa0, 0x00, \
  0x58, 0x58, 0x2c, 0x16, 0x00, 0x00, \
  0x0e, 0x28, 0x94, 0x4a, 0x20, 0x00, \
  0x83, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x0a, 0x1c, 0x77, 0xf9, 0x00, 0x00

#define kMaskRandom35_32 \
  0x25, 0x2c, 0x16, 0x0b, 0x00, 0x00, \
  0x8a, 0x91, 0x48, 0xa4, 0x40, 0x00, \
  0x91, 0xc0, 0xe0, 0x70, 0x20, 0x00, \
  0x68, 0x06, 0x83, 0x41, 0xa0, 0x00, \
  0x32, 0xc8, 0x64, 0x32, 0x00, 0x00, \
  0x43, 0x45, 0x22, 0x91, 0x40, 0x00, \
  0xc4, 0x30, 0x98, 0x4c, 0x20, 0x00, \
  0x1c, 0xa2, 0x51, 0x28, 0x80, 0x00, \
  0x15, 0x8c, 0x46, 0x23, 0x00, 0x00, \
  0x8a, 0x47, 0x23, 0x91, 0xc0, 0x00, \
  0x25, 0x81, 0xc0, 0xe0, 0x60, 0x00, \
  0x62, 0x12, 0x89, 0x44, 0xa0, 0x00, \
  0x58, 0x58, 0x2c, 0x16, 0x00, 0x00, \
  0x0e, 0x28, 0x94, 0x4a, 0x20, 0x00, \
  0x83, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x0a, 0x1c, 0x77, 0xf9, 0x00, 0x00, \
  0x15, 0x8c, 0x46, 0x23, 0x00, 0x00, \
  0x8a, 0x47, 0x23, 0x91, 0xc0, 0x00, \
  0x25, 0x81, 0xc0, 0xe0, 0x60, 0x00, \
  0x62, 0x12, 0x89, 0x44, 0xa0, 0x00, \
  0x58, 0x58, 0x2c, 0x16, 0x00, 0x00, \
  0x0e, 0x28, 0x94, 0x4a, 0x20, 0x00, \
  0x83, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x25, 0x2c, 0x16, 0x0b, 0x00, 0x00, \
  0x8a, 0x91, 0x48, 0xa4, 0x40, 0x00, \
  0x91, 0xc0, 0xe0, 0x70, 0x20, 0x00, \
  0x68, 0x06, 0x83, 0x41, 0xa0, 0x00, \
  0x32, 0xc8, 0x64, 0x32, 0x00, 0x00, \
  0x43, 0x45, 0x22, 0x91, 0x40, 0x00, \
  0xc4, 0x30, 0x98, 0x4c, 0x20, 0x00, \
  0x1c, 0xa2, 0x51, 0x28, 0x80, 0x00, \
  0xeb, 0x31, 0x7b, 0x80, 0x00, 0x00

#define kMaskRandom35_33 \
  0x25, 0x2c, 0x16, 0x0b, 0x00, 0x00, \
  0x8a, 0x91, 0x48, 0xa4, 0x40, 0x00, \
  0x91, 0xc0, 0xe0, 0x70, 0x20, 0x00, \
  0x68, 0x06, 0x83, 0x41, 0xa0, 0x00, \
  0x32, 0xc8, 0x64, 0x32, 0x00, 0x00, \
  0x43, 0x45, 0x22, 0x91, 0x40, 0x00, \
  0xc4, 0x30, 0x98, 0x4c, 0x20, 0x00, \
  0x1c, 0xa2, 0x51, 0x28, 0x80, 0x00, \
  0x15, 0x8c, 0x46, 0x23, 0x00, 0x00, \
  0x8a, 0x47, 0x23, 0x91, 0xc0, 0x00, \
  0x25, 0x81, 0xc0, 0xe0, 0x60, 0x00, \
  0x62, 0x12, 0x89, 0x44, 0xa0, 0x00, \
  0x58, 0x58, 0x2c, 0x16, 0x00, 0x00, \
  0x0e, 0x28, 0x94, 0x4a, 0x20, 0x00, \
  0x83, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x0a, 0x1c, 0x77, 0xf9, 0x00, 0x00, \
  0x25, 0x2c, 0x16, 0x0b, 0x00, 0x00, \
  0x8a, 0x91, 0x48, 0xa4, 0x40, 0x00, \
  0x91, 0xc0, 0xe0, 0x70, 0x20, 0x00, \
  0x68, 0x06, 0x83, 0x41, 0xa0, 0x00, \
  0x32, 0xc8, 0x64, 0x32, 0x00, 0x00, \
  0x43, 0x45, 0x22, 0x91, 0x40, 0x00, \
  0xc4, 0x30, 0x98, 0x4c, 0x20, 0x00, \
  0x1c, 0xa2, 0x51, 0x28, 0x80, 0x00, \
  0x25, 0x4c, 0x26, 0x13, 0x00, 0x00, \
  0x8a, 0x66, 0x33, 0x19, 0x80, 0x00, \
  0x91, 0x91, 0x48, 0xa4, 0x40, 0x00, \
  0x68, 0x42, 0xa1, 0x50, 0xa0, 0x00, \
  0x32, 0xa4, 0x52, 0x29, 0x00, 0x00, \
  0x43, 0x13, 0x09, 0x84, 0xc0, 0x00, \
  0xc4, 0x30, 0x98, 0x4c, 0x20, 0x00, \
  0x1c, 0x88, 0xc4, 0x62, 0x20, 0x00, \
  0x3c, 0x09, 0x04, 0x82, 0x40, 0x00

#define kMaskRandom35_34 \
  0x25, 0x2c, 0x16, 0x0b, 0x00, 0x00, \
  0x8a, 0x91, 0x48, 0xa4, 0x40, 0x00, \
  0x91, 0xc0, 0xe0, 0x70, 0x20, 0x00, \
  0x68, 0x06, 0x83, 0x41, 0xa0, 0x00, \
  0x32, 0xc8, 0x64, 0x32, 0x00, 0x00, \
  0x43, 0x45, 0x22, 0x91, 0x40, 0x00, \
  0xc4, 0x30, 0x98, 0x4c, 0x20, 0x00, \
  0x1c, 0xa2, 0x51, 0x28, 0x80, 0x00, \
  0x25, 0x4c, 0x26, 0x13, 0x00, 0x00, \
  0x8a, 0x66, 0x33, 0x19, 0x80, 0x00, \
  0x91, 0x91, 0x48, 0xa4, 0x40, 0x00, \
  0x68, 0x42, 0xa1, 0x50, 0xa0, 0x00, \
  0x32, 0xa4, 0x52, 0x29, 0x00, 0x00, \
  0x43, 0x13, 0x09, 0x84, 0xc0, 0x00, \
  0xc4, 0x30, 0x98, 0x4c, 0x20, 0x00, \
  0x1c, 0x88, 0xc4, 0x62, 0x20, 0x00, \
  0x3c, 0x09, 0x04, 0x82, 0x40, 0x00, \
  0x25, 0x2c, 0x16, 0x0b, 0x00, 0x00, \
  0x8a, 0x91, 0x48, 0xa4, 0x40, 0x00, \
  0x91, 0xc0, 0xe0, 0x70, 0x20, 0x00, \
  0x68, 0x06, 0x83, 0x41, 0xa0, 0x00, \
  0x32, 0xc8, 0x64, 0x32, 0x00, 0x00, \
  0x43, 0x45, 0x22, 0x91, 0x40, 0x00, \
  0xc4, 0x30, 0x98, 0x4c, 0x20, 0x00, \
  0x1c, 0xa2, 0x51, 0x28, 0x80, 0x00, \
  0x15, 0x8c, 0x46, 0x23, 0x00, 0x00, \
  0x8a, 0x47, 0x23, 0x91, 0xc0, 0x00, \
  0x25, 0x81, 0xc0, 0xe0, 0x60, 0x00, \
  0x62, 0x12, 0x89, 0x44, 0xa0, 0x00, \
  0x58, 0x58, 0x2c, 0x16, 0x00, 0x00, \
  0x0e, 0x28, 0x94, 0x4a, 0x20, 0x00, \
  0x83, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x0a, 0x1c, 0x77, 0xf9, 0x00, 0x00, \
  0x70, 0x07, 0xcd, 0x8c, 0xc0, 0x00

#define kMaskRandom35_35 \
  0x25, 0x2c, 0x16, 0x0b, 0x00, 0x00, \
  0x8a, 0x91, 0x48, 0xa4, 0x40, 0x00, \
  0x91, 0xc0, 0xe0, 0x70, 0x20, 0x00, \
  0x68, 0x06, 0x83, 0x41, 0xa0, 0x00, \
  0x32, 0xc8, 0x64, 0x32, 0x00, 0x00, \
  0x43, 0x45, 0x22, 0x91, 0x40, 0x00, \
  0xc4, 0x30, 0x98, 0x4c, 0x20, 0x00, \
  0x1c, 0xa2, 0x51, 0x28, 0x80, 0x00, \
  0x25, 0x4c, 0x26, 0x13, 0x00, 0x00, \
  0x8a, 0x66, 0x33, 0x19, 0x80, 0x00, \
  0x91, 0x91, 0x48, 0xa4, 0x40, 0x00, \
  0x68, 0x42, 0xa1, 0x50, 0xa0, 0x00, \
  0x32, 0xa4, 0x52, 0x29, 0x00, 0x00, \
  0x43, 0x13, 0x09, 0x84, 0xc0, 0x00, \
  0xc4, 0x30, 0x98, 0x4c, 0x20, 0x00, \
  0x1c, 0x88, 0xc4, 0x62, 0x20, 0x00, \
  0x3c, 0x09, 0x04, 0x82, 0x40, 0x00, \
  0x25, 0x2c, 0x26, 0x13, 0x00, 0x00, \
  0x8a, 0x91, 0x33, 0x19, 0x80, 0x00, \
  0x91, 0xc0, 0xc8, 0xa4, 0x40, 0x00, \
  0x68, 0x06, 0xa1, 0x50, 0xa0, 0x00, \
  0x32, 0xc8, 0x52, 0x29, 0x00, 0x00, \
  0x43, 0x45, 0x09, 0x84, 0xc0, 0x00, \
  0xc4, 0x30, 0x98, 0x4c, 0x20, 0x00, \
  0x1c, 0xa2, 0x44, 0x62, 0x20, 0x00, \
  0x25, 0x4c, 0x04, 0x82, 0x40, 0x00, \
  0x8a, 0x66, 0x16, 0x0b, 0x00, 0x00, \
  0x91, 0x91, 0x48, 0xa4, 0x40, 0x00, \
  0x68, 0x42, 0xe0, 0x70, 0x20, 0x00, \
  0x32, 0xa4, 0x03, 0x41, 0xa0, 0x00, \
  0x43, 0x13, 0x64, 0x32, 0x00, 0x00, \
  0xc4, 0x30, 0xa2, 0x91, 0x40, 0x00, \
  0x1c, 0x88, 0x98, 0x4c, 0x20, 0x00, \
  0x3c, 0x09, 0x51, 0x28, 0x80, 0x00, \
  0xc2, 0x1c, 0x68, 0x01, 0xa0, 0x00

#define kMaskRandom35_4 \
  0xca, 0xec, 0x76, 0x3b, 0x00, 0x00, \
  0xa9, 0x67, 0x33, 0x99, 0xc0, 0x00, \
  0x3a, 0xb1, 0xd8, 0xec, 0x60, 0x00, \
  0x55, 0x5a, 0xad, 0x56, 0xa0, 0x00

#define kMaskRandom35_5 \
  0x55, 0x44, 0xa6, 0x53, 0x20, 0x00, \
  0x2a, 0x66, 0x33, 0x19, 0x80, 0x00, \
  0x25, 0xa1, 0x8c, 0xe8, 0x60, 0x00, \
  0xe2, 0x12, 0xce, 0x44, 0xa0, 0x00, \
  0x99, 0x98, 0x71, 0xa6, 0x00, 0x00

#define kMaskRandom35_6 \
  0xd1, 0x4c, 0x66, 0x13, 0x00, 0x00, \
  0xa2, 0xc5, 0x22, 0xb1, 0x40, 0x00, \
  0x95, 0x30, 0xd8, 0x4c, 0x20, 0x00, \
  0xca, 0x0a, 0xc5, 0x42, 0xa0, 0x00, \
  0xa4, 0xaa, 0x14, 0xa9, 0x80, 0x00, \
  0x78, 0x15, 0x53, 0x05, 0x40, 0x00

#define kMaskRandom35_7 \
  0x15, 0x44, 0xa2, 0x51, 0x20, 0x00, \
  0x8a, 0x23, 0x11, 0x88, 0xc0, 0x00, \
  0x85, 0x91, 0x48, 0xa4, 0x40, 0x00, \
  0x32, 0x0a, 0x85, 0x42, 0xa0, 0x00, \
  0x58, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x2c, 0x0d, 0x05, 0x83, 0x40, 0x00, \
  0x43, 0xc8, 0x70, 0x32, 0x00, 0x00

#define kMaskRandom35_8 \
  0x64, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0xa2, 0xc2, 0x61, 0x30, 0x80, 0x00, \
  0x51, 0x60, 0xb0, 0x58, 0x20, 0x00, \
  0x4a, 0x85, 0x42, 0xa1, 0x40, 0x00, \
  0x38, 0x4c, 0x26, 0x13, 0x00, 0x00, \
  0x89, 0x29, 0x14, 0x8a, 0x40, 0x00, \
  0x07, 0x11, 0x88, 0xc4, 0x60, 0x00, \
  0x94, 0xb0, 0x58, 0x2c, 0x00, 0x00

#define kMaskRandom35_9 \
  0x8e, 0xcc, 0x22, 0x51, 0x20, 0x00, \
  0x6a, 0x2b, 0x33, 0x13, 0x00, 0x00, \
  0x36, 0x32, 0xc8, 0x24, 0xa0, 0x00, \
  0xd1, 0x25, 0x80, 0xd2, 0xc0, 0x00, \
  0x55, 0x8c, 0x87, 0x09, 0x40, 0x00, \
  0xaa, 0x27, 0x09, 0x85, 0x80, 0x00, \
  0xa5, 0x32, 0x90, 0x68, 0x20, 0x00, \
  0x62, 0x61, 0xe1, 0x28, 0x80, 0x00, \
  0x3c, 0x5c, 0x14, 0x86, 0x40, 0x00

#define kMaskRandom36_1 \
  0xff, 0xff, 0xff, 0xff, 0xf0, 0x00

#define kMaskRandom36_10 \
  0x8c, 0xc6, 0x63, 0x31, 0x90, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0xe0, 0x00, \
  0x32, 0x99, 0x4c, 0xa6, 0x50, 0x00, \
  0x61, 0xb0, 0xd8, 0x6c, 0x30, 0x00, \
  0x5c, 0x2e, 0x17, 0x0b, 0x80, 0x00, \
  0xcc, 0x66, 0x33, 0x19, 0x80, 0x00, \
  0x2b, 0x15, 0x8a, 0xc5, 0x60, 0x00, \
  0x32, 0x99, 0x4c, 0xa6, 0x50, 0x00, \
  0x25, 0x92, 0xc9, 0x64, 0xb0, 0x00, \
  0xfd, 0x9d, 0xff, 0x67, 0x70, 0x00

#define kMaskRandom36_11 \
  0x8c, 0xc6, 0x63, 0x31, 0x90, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0xe0, 0x00, \
  0x32, 0x99, 0x4c, 0xa6, 0x50, 0x00, \
  0x61, 0xb0, 0xd8, 0x6c, 0x30, 0x00, \
  0x5c, 0x2e, 0x17, 0x0b, 0x80, 0x00, \
  0x84, 0xc2, 0x61, 0x30, 0x90, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0xe0, 0x00, \
  0x51, 0xa8, 0xd4, 0x6a, 0x30, 0x00, \
  0x1a, 0x0d, 0x06, 0x83, 0x40, 0x00, \
  0x68, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x89, 0x44, 0xa2, 0x51, 0x20, 0x00

#define kMaskRandom36_12 \
  0x84, 0xc2, 0x61, 0x30, 0x90, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0xe0, 0x00, \
  0x51, 0xa8, 0xd4, 0x6a, 0x30, 0x00, \
  0x1a, 0x0d, 0x06, 0x83, 0x40, 0x00, \
  0x68, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x89, 0x44, 0xa2, 0x51, 0x20, 0x00, \
  0x8c, 0xc6, 0x63, 0x31, 0x90, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0xe0, 0x00, \
  0x32, 0x99, 0x4c, 0xa6, 0x50, 0x00, \
  0x61, 0xb0, 0xd8, 0x6c, 0x30, 0x00, \
  0x5c, 0x2e, 0x17, 0x0b, 0x80, 0x00, \
  0x5b, 0x0c, 0x56, 0xc3, 0x10, 0x00

#define kMaskRandom36_13 \
  0x84, 0xc2, 0x61, 0x30, 0x90, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0xe0, 0x00, \
  0x51, 0xa8, 0xd4, 0x6a, 0x30, 0x00, \
  0x1a, 0x0d, 0x06, 0x83, 0x40, 0x00, \
  0x68, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x89, 0x44, 0xa2, 0x51, 0x20, 0x00, \
  0x8c, 0x46, 0x23, 0x11, 0x80, 0x00, \
  0x47, 0x23, 0x91, 0xc8, 0xe0, 0x00, \
  0x81, 0xc0, 0xe0, 0x70, 0x30, 0x00, \
  0x12, 0x89, 0x44, 0xa2, 0x50, 0x00, \
  0x58, 0x2c, 0x16, 0x0b, 0x00, 0x00, \
  0x28, 0x94, 0x4a, 0x25, 0x10, 0x00, \
  0x34, 0x1a, 0x0d, 0x06, 0x80, 0x00

#define kMaskRandom36_14 \
  0x8c, 0x46, 0x23, 0x11, 0x80, 0x00, \
  0x47, 0x23, 0x91, 0xc8, 0xe0, 0x00, \
  0x81, 0xc0, 0xe0, 0x70, 0x30, 0x00, \
  0x12, 0x89, 0x44, 0xa2, 0x50, 0x00, \
  0x58, 0x2c, 0x16, 0x0b, 0x00, 0x00, \
  0x28, 0x94, 0x4a, 0x25, 0x10, 0x00, \
  0x34, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0x84, 0xc2, 0x61, 0x30, 0x90, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0xe0, 0x00, \
  0x51, 0xa8, 0xd4, 0x6a, 0x30, 0x00, \
  0x1a, 0x0d, 0x06, 0x83, 0x40, 0x00, \
  0x68, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x89, 0x44, 0xa2, 0x51, 0x20, 0x00, \
  0x7f, 0x4f, 0xdf, 0xd3, 0xf0, 0x00

#define kMaskRandom36_15 \
  0x8c, 0x46, 0x23, 0x11, 0x80, 0x00, \
  0x47, 0x23, 0x91, 0xc8, 0xe0, 0x00, \
  0x81, 0xc0, 0xe0, 0x70, 0x30, 0x00, \
  0x12, 0x89, 0x44, 0xa2, 0x50, 0x00, \
  0x58, 0x2c, 0x16, 0x0b, 0x00, 0x00, \
  0x28, 0x94, 0x4a, 0x25, 0x10, 0x00, \
  0x34, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0x2c, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0x91, 0x48, 0xa4, 0x52, 0x20, 0x00, \
  0xc0, 0xe0, 0x70, 0x38, 0x10, 0x00, \
  0x06, 0x83, 0x41, 0xa0, 0xd0, 0x00, \
  0xc8, 0x64, 0x32, 0x19, 0x00, 0x00, \
  0x45, 0x22, 0x91, 0x48, 0xa0, 0x00, \
  0x30, 0x98, 0x4c, 0x26, 0x10, 0x00, \
  0xa2, 0x51, 0x28, 0x94, 0x40, 0x00

#define kMaskRandom36_16 \
  0x2c, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0x91, 0x48, 0xa4, 0x52, 0x20, 0x00, \
  0xc0, 0xe0, 0x70, 0x38, 0x10, 0x00, \
  0x06, 0x83, 0x41, 0xa0, 0xd0, 0x00, \
  0xc8, 0x64, 0x32, 0x19, 0x00, 0x00, \
  0x45, 0x22, 0x91, 0x48, 0xa0, 0x00, \
  0x30, 0x98, 0x4c, 0x26, 0x10, 0x00, \
  0xa2, 0x51, 0x28, 0x94, 0x40, 0x00, \
  0x8c, 0x46, 0x23, 0x11, 0x80, 0x00, \
  0x47, 0x23, 0x91, 0xc8, 0xe0, 0x00, \
  0x81, 0xc0, 0xe0, 0x70, 0x30, 0x00, \
  0x12, 0x89, 0x44, 0xa2, 0x50, 0x00, \
  0x58, 0x2c, 0x16, 0x0b, 0x00, 0x00, \
  0x28, 0x94, 0x4a, 0x25, 0x10, 0x00, \
  0x34, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0xef, 0xf2, 0x3b, 0xfc, 0x80, 0x00

#define kMaskRandom36_17 \
  0x2c, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0x91, 0x48, 0xa4, 0x52, 0x20, 0x00, \
  0xc0, 0xe0, 0x70, 0x38, 0x10, 0x00, \
  0x06, 0x83, 0x41, 0xa0, 0xd0, 0x00, \
  0xc8, 0x64, 0x32, 0x19, 0x00, 0x00, \
  0x45, 0x22, 0x91, 0x48, 0xa0, 0x00, \
  0x30, 0x98, 0x4c, 0x26, 0x10, 0x00, \
  0xa2, 0x51, 0x28, 0x94, 0x40, 0x00, \
  0x4c, 0x26, 0x13, 0x09, 0x80, 0x00, \
  0x66, 0x33, 0x19, 0x8c, 0xc0, 0x00, \
  0x91, 0x48, 0xa4, 0x52, 0x20, 0x00, \
  0x42, 0xa1, 0x50, 0xa8, 0x50, 0x00, \
  0xa4, 0x52, 0x29, 0x14, 0x80, 0x00, \
  0x13, 0x09, 0x84, 0xc2, 0x60, 0x00, \
  0x30, 0x98, 0x4c, 0x26, 0x10, 0x00, \
  0x88, 0xc4, 0x62, 0x31, 0x10, 0x00, \
  0x09, 0x04, 0x82, 0x41, 0x20, 0x00

#define kMaskRandom36_18 \
  0x4c, 0x26, 0x13, 0x09, 0x80, 0x00, \
  0x66, 0x33, 0x19, 0x8c, 0xc0, 0x00, \
  0x91, 0x48, 0xa4, 0x52, 0x20, 0x00, \
  0x42, 0xa1, 0x50, 0xa8, 0x50, 0x00, \
  0xa4, 0x52, 0x29, 0x14, 0x80, 0x00, \
  0x13, 0x09, 0x84, 0xc2, 0x60, 0x00, \
  0x30, 0x98, 0x4c, 0x26, 0x10, 0x00, \
  0x88, 0xc4, 0x62, 0x31, 0x10, 0x00, \
  0x09, 0x04, 0x82, 0x41, 0x20, 0x00, \
  0x2c, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0x91, 0x48, 0xa4, 0x52, 0x20, 0x00, \
  0xc0, 0xe0, 0x70, 0x38, 0x10, 0x00, \
  0x06, 0x83, 0x41, 0xa0, 0xd0, 0x00, \
  0xc8, 0x64, 0x32, 0x19, 0x00, 0x00, \
  0x45, 0x22, 0x91, 0x48, 0xa0, 0x00, \
  0x30, 0x98, 0x4c, 0x26, 0x10, 0x00, \
  0xa2, 0x51, 0x28, 0x94, 0x40, 0x00, \
  0xd0, 0x03, 0x74, 0x00, 0xd0, 0x00

#define kMaskRandom36_19 \
  0x44, 0xa2, 0x51, 0x28, 0x90, 0x00, \
  0x66, 0x26, 0x19, 0x89, 0x80, 0x00, \
  0x90, 0x49, 0x64, 0x12, 0x50, 0x00, \
  0x01, 0xa5, 0x80, 0x69, 0x60, 0x00, \
  0x0e, 0x12, 0x83, 0x84, 0xa0, 0x00, \
  0x13, 0x0b, 0x04, 0xc2, 0xc0, 0x00, \
  0x20, 0xd0, 0x48, 0x34, 0x10, 0x00, \
  0xc2, 0x51, 0x30, 0x94, 0x40, 0x00, \
  0x29, 0x0c, 0x8a, 0x43, 0x20, 0x00, \
  0x8c, 0xc6, 0x63, 0x31, 0x90, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0xe0, 0x00, \
  0x32, 0x99, 0x4c, 0xa6, 0x50, 0x00, \
  0x61, 0xb0, 0xd8, 0x6c, 0x30, 0x00, \
  0x5c, 0x2e, 0x17, 0x0b, 0x80, 0x00, \
  0xcc, 0x66, 0x33, 0x19, 0x80, 0x00, \
  0x2b, 0x15, 0x8a, 0xc5, 0x60, 0x00, \
  0x32, 0x99, 0x4c, 0xa6, 0x50, 0x00, \
  0x25, 0x92, 0xc9, 0x64, 0xb0, 0x00, \
  0xfd, 0x9d, 0xff, 0x67, 0x70, 0x00

#define kMaskRandom36_2 \
  0xce, 0x67, 0x33, 0x99, 0xc0, 0x00, \
  0x39, 0x9c, 0xce, 0x67, 0x30, 0x00

#define kMaskRandom36_20 \
  0x8c, 0xc6, 0x63, 0x31, 0x90, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0xe0, 0x00, \
  0x32, 0x99, 0x4c, 0xa6, 0x50, 0x00, \
  0x61, 0xb0, 0xd8, 0x6c, 0x30, 0x00, \
  0x5c, 0x2e, 0x17, 0x0b, 0x80, 0x00, \
  0xcc, 0x66, 0x33, 0x19, 0x80, 0x00, \
  0x2b, 0x15, 0x8a, 0xc5, 0x60, 0x00, \
  0x32, 0x99, 0x4c, 0xa6, 0x50, 0x00, \
  0x25, 0x92, 0xc9, 0x64, 0xb0, 0x00, \
  0xfd, 0x9d, 0xff, 0x67, 0x70, 0x00, \
  0x44, 0xa2, 0x51, 0x28, 0x90, 0x00, \
  0x66, 0x26, 0x19, 0x89, 0x80, 0x00, \
  0x90, 0x49, 0x64, 0x12, 0x50, 0x00, \
  0x01, 0xa5, 0x80, 0x69, 0x60, 0x00, \
  0x0e, 0x12, 0x83, 0x84, 0xa0, 0x00, \
  0x13, 0x0b, 0x04, 0xc2, 0xc0, 0x00, \
  0x20, 0xd0, 0x48, 0x34, 0x10, 0x00, \
  0xc2, 0x51, 0x30, 0x94, 0x40, 0x00, \
  0x29, 0x0c, 0x8a, 0x43, 0x20, 0x00, \
  0x45, 0xb9, 0x08, 0x16, 0x30, 0x00

#define kMaskRandom36_21 \
  0x8c, 0xc6, 0x63, 0x31, 0x90, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0xe0, 0x00, \
  0x32, 0x99, 0x4c, 0xa6, 0x50, 0x00, \
  0x61, 0xb0, 0xd8, 0x6c, 0x30, 0x00, \
  0x5c, 0x2e, 0x17, 0x0b, 0x80, 0x00, \
  0xcc, 0x66, 0x33, 0x19, 0x80, 0x00, \
  0x2b, 0x15, 0x8a, 0xc5, 0x60, 0x00, \
  0x32, 0x99, 0x4c, 0xa6, 0x50, 0x00, \
  0x25, 0x92, 0xc9, 0x64, 0xb0, 0x00, \
  0xfd, 0x9d, 0xff, 0x67, 0x70, 0x00, \
  0x8c, 0xc6, 0x63, 0x31, 0x90, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0xe0, 0x00, \
  0x32, 0x99, 0x4c, 0xa6, 0x50, 0x00, \
  0x61, 0xb0, 0xd8, 0x6c, 0x30, 0x00, \
  0x5c, 0x2e, 0x17, 0x0b, 0x80, 0x00, \
  0x84, 0xc2, 0x61, 0x30, 0x90, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0xe0, 0x00, \
  0x51, 0xa8, 0xd4, 0x6a, 0x30, 0x00, \
  0x1a, 0x0d, 0x06, 0x83, 0x40, 0x00, \
  0x68, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x89, 0x44, 0xa2, 0x51, 0x20, 0x00

#define kMaskRandom36_22 \
  0x8c, 0xc6, 0x63, 0x31, 0x90, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0xe0, 0x00, \
  0x32, 0x99, 0x4c, 0xa6, 0x50, 0x00, \
  0x61, 0xb0, 0xd8, 0x6c, 0x30, 0x00, \
  0x5c, 0x2e, 0x17, 0x0b, 0x80, 0x00, \
  0x84, 0xc2, 0x61, 0x30, 0x90, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0xe0, 0x00, \
  0x51, 0xa8, 0xd4, 0x6a, 0x30, 0x00, \
  0x1a, 0x0d, 0x06, 0x83, 0x40, 0x00, \
  0x68, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x89, 0x44, 0xa2, 0x51, 0x20, 0x00, \
  0x8c, 0xc6, 0x63, 0x31, 0x90, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0xe0, 0x00, \
  0x32, 0x99, 0x4c, 0xa6, 0x50, 0x00, \
  0x61, 0xb0, 0xd8, 0x6c, 0x30, 0x00, \
  0x5c, 0x2e, 0x17, 0x0b, 0x80, 0x00, \
  0xcc, 0x66, 0x33, 0x19, 0x80, 0x00, \
  0x2b, 0x15, 0x8a, 0xc5, 0x60, 0x00, \
  0x32, 0x99, 0x4c, 0xa6, 0x50, 0x00, \
  0x25, 0x92, 0xc9, 0x64, 0xb0, 0x00, \
  0xfd, 0x9d, 0xff, 0x67, 0x70, 0x00, \
  0x71, 0x04, 0xba, 0x7b, 0xe0, 0x00

#define kMaskRandom36_23 \
  0x8c, 0xc6, 0x63, 0x31, 0x90, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0xe0, 0x00, \
  0x32, 0x99, 0x4c, 0xa6, 0x50, 0x00, \
  0x61, 0xb0, 0xd8, 0x6c, 0x30, 0x00, \
  0x5c, 0x2e, 0x17, 0x0b, 0x80, 0x00, \
  0x84, 0xc2, 0x61, 0x30, 0x90, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0xe0, 0x00, \
  0x51, 0xa8, 0xd4, 0x6a, 0x30, 0x00, \
  0x1a, 0x0d, 0x06, 0x83, 0x40, 0x00, \
  0x68, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x89, 0x44, 0xa2, 0x51, 0x20, 0x00, \
  0x84, 0xc2, 0x61, 0x30, 0x90, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0xe0, 0x00, \
  0x51, 0xa8, 0xd4, 0x6a, 0x30, 0x00, \
  0x1a, 0x0d, 0x06, 0x83, 0x40, 0x00, \
  0x68, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x89, 0x44, 0xa2, 0x51, 0x20, 0x00, \
  0x8c, 0xc6, 0x63, 0x31, 0x90, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0xe0, 0x00, \
  0x32, 0x99, 0x4c, 0xa6, 0x50, 0x00, \
  0x61, 0xb0, 0xd8, 0x6c, 0x30, 0x00, \
  0x5c, 0x2e, 0x17, 0x0b, 0x80, 0x00, \
  0x5b, 0x0c, 0x56, 0xc3, 0x10, 0x00

#define kMaskRandom36_24 \
  0x84, 0xc2, 0x61, 0x30, 0x90, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0xe0, 0x00, \
  0x51, 0xa8, 0xd4, 0x6a, 0x30, 0x00, \
  0x1a, 0x0d, 0x06, 0x83, 0x40, 0x00, \
  0x68, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x89, 0x44, 0xa2, 0x51, 0x20, 0x00, \
  0x8c, 0xc6, 0x63, 0x31, 0x90, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0xe0, 0x00, \
  0x32, 0x99, 0x4c, 0xa6, 0x50, 0x00, \
  0x61, 0xb0, 0xd8, 0x6c, 0x30, 0x00, \
  0x5c, 0x2e, 0x17, 0x0b, 0x80, 0x00, \
  0x5b, 0x0c, 0x56, 0xc3, 0x10, 0x00, \
  0x8c, 0xc6, 0x63, 0x31, 0x90, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0xe0, 0x00, \
  0x32, 0x99, 0x4c, 0xa6, 0x50, 0x00, \
  0x61, 0xb0, 0xd8, 0x6c, 0x30, 0x00, \
  0x5c, 0x2e, 0x17, 0x0b, 0x80, 0x00, \
  0x84, 0xc2, 0x61, 0x30, 0x90, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0xe0, 0x00, \
  0x51, 0xa8, 0xd4, 0x6a, 0x30, 0x00, \
  0x1a, 0x0d, 0x06, 0x83, 0x40, 0x00, \
  0x68, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x89, 0x44, 0xa2, 0x51, 0x20, 0x00, \
  0x76, 0x3a, 0xeb, 0x17, 0xc0, 0x00

#define kMaskRandom36_25 \
  0x84, 0xc2, 0x61, 0x30, 0x90, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0xe0, 0x00, \
  0x51, 0xa8, 0xd4, 0x6a, 0x30, 0x00, \
  0x1a, 0x0d, 0x06, 0x83, 0x40, 0x00, \
  0x68, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x89, 0x44, 0xa2, 0x51, 0x20, 0x00, \
  0x8c, 0xc6, 0x63, 0x31, 0x90, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0xe0, 0x00, \
  0x32, 0x99, 0x4c, 0xa6, 0x50, 0x00, \
  0x61, 0xb0, 0xd8, 0x6c, 0x30, 0x00, \
  0x5c, 0x2e, 0x17, 0x0b, 0x80, 0x00, \
  0x5b, 0x0c, 0x56, 0xc3, 0x10, 0x00, \
  0x84, 0xc2, 0x61, 0x30, 0x90, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0xe0, 0x00, \
  0x51, 0xa8, 0xd4, 0x6a, 0x30, 0x00, \
  0x1a, 0x0d, 0x06, 0x83, 0x40, 0x00, \
  0x68, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x89, 0x44, 0xa2, 0x51, 0x20, 0x00, \
  0x8c, 0x46, 0x23, 0x11, 0x80, 0x00, \
  0x47, 0x23, 0x91, 0xc8, 0xe0, 0x00, \
  0x81, 0xc0, 0xe0, 0x70, 0x30, 0x00, \
  0x12, 0x89, 0x44, 0xa2, 0x50, 0x00, \
  0x58, 0x2c, 0x16, 0x0b, 0x00, 0x00, \
  0x28, 0x94, 0x4a, 0x25, 0x10, 0x00, \
  0x34, 0x1a, 0x0d, 0x06, 0x80, 0x00

#define kMaskRandom36_26 \
  0x84, 0xc2, 0x61, 0x30, 0x90, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0xe0, 0x00, \
  0x51, 0xa8, 0xd4, 0x6a, 0x30, 0x00, \
  0x1a, 0x0d, 0x06, 0x83, 0x40, 0x00, \
  0x68, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x89, 0x44, 0xa2, 0x51, 0x20, 0x00, \
  0x8c, 0x46, 0x23, 0x11, 0x80, 0x00, \
  0x47, 0x23, 0x91, 0xc8, 0xe0, 0x00, \
  0x81, 0xc0, 0xe0, 0x70, 0x30, 0x00, \
  0x12, 0x89, 0x44, 0xa2, 0x50, 0x00, \
  0x58, 0x2c, 0x16, 0x0b, 0x00, 0x00, \
  0x28, 0x94, 0x4a, 0x25, 0x10, 0x00, \
  0x34, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0x84, 0xc2, 0x61, 0x30, 0x90, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0xe0, 0x00, \
  0x51, 0xa8, 0xd4, 0x6a, 0x30, 0x00, \
  0x1a, 0x0d, 0x06, 0x83, 0x40, 0x00, \
  0x68, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x89, 0x44, 0xa2, 0x51, 0x20, 0x00, \
  0x8c, 0xc6, 0x63, 0x31, 0x90, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0xe0, 0x00, \
  0x32, 0x99, 0x4c, 0xa6, 0x50, 0x00, \
  0x61, 0xb0, 0xd8, 0x6c, 0x30, 0x00, \
  0x5c, 0x2e, 0x17, 0x0b, 0x80, 0x00, \
  0x5b, 0x0c, 0x56, 0xc3, 0x10, 0x00, \
  0xec, 0x58, 0x0e, 0x6c, 0xe0, 0x00

#define kMaskRandom36_27 \
  0x84, 0xc2, 0x61, 0x30, 0x90, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0xe0, 0x00, \
  0x51, 0xa8, 0xd4, 0x6a, 0x30, 0x00, \
  0x1a, 0x0d, 0x06, 0x83, 0x40, 0x00, \
  0x68, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x89, 0x44, 0xa2, 0x51, 0x20, 0x00, \
  0x8c, 0x46, 0x23, 0x11, 0x80, 0x00, \
  0x47, 0x23, 0x91, 0xc8, 0xe0, 0x00, \
  0x81, 0xc0, 0xe0, 0x70, 0x30, 0x00, \
  0x12, 0x89, 0x44, 0xa2, 0x50, 0x00, \
  0x58, 0x2c, 0x16, 0x0b, 0x00, 0x00, \
  0x28, 0x94, 0x4a, 0x25, 0x10, 0x00, \
  0x34, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0x8c, 0x46, 0x23, 0x11, 0x80, 0x00, \
  0x47, 0x23, 0x91, 0xc8, 0xe0, 0x00, \
  0x81, 0xc0, 0xe0, 0x70, 0x30, 0x00, \
  0x12, 0x89, 0x44, 0xa2, 0x50, 0x00, \
  0x58, 0x2c, 0x16, 0x0b, 0x00, 0x00, \
  0x28, 0x94, 0x4a, 0x25, 0x10, 0x00, \
  0x34, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0x84, 0xc2, 0x61, 0x30, 0x90, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0xe0, 0x00, \
  0x51, 0xa8, 0xd4, 0x6a, 0x30, 0x00, \
  0x1a, 0x0d, 0x06, 0x83, 0x40, 0x00, \
  0x68, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x89, 0x44, 0xa2, 0x51, 0x20, 0x00, \
  0x7f, 0x4f, 0xdf, 0xd3, 0xf0, 0x00

#define kMaskRandom36_28 \
  0x8c, 0x46, 0x23, 0x11, 0x80, 0x00, \
  0x47, 0x23, 0x91, 0xc8, 0xe0, 0x00, \
  0x81, 0xc0, 0xe0, 0x70, 0x30, 0x00, \
  0x12, 0x89, 0x44, 0xa2, 0x50, 0x00, \
  0x58, 0x2c, 0x16, 0x0b, 0x00, 0x00, \
  0x28, 0x94, 0x4a, 0x25, 0x10, 0x00, \
  0x34, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0x84, 0xc2, 0x61, 0x30, 0x90, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0xe0, 0x00, \
  0x51, 0xa8, 0xd4, 0x6a, 0x30, 0x00, \
  0x1a, 0x0d, 0x06, 0x83, 0x40, 0x00, \
  0x68, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x89, 0x44, 0xa2, 0x51, 0x20, 0x00, \
  0x7f, 0x4f, 0xdf, 0xd3, 0xf0, 0x00, \
  0x84, 0xc2, 0x61, 0x30, 0x90, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0xe0, 0x00, \
  0x51, 0xa8, 0xd4, 0x6a, 0x30, 0x00, \
  0x1a, 0x0d, 0x06, 0x83, 0x40, 0x00, \
  0x68, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x89, 0x44, 0xa2, 0x51, 0x20, 0x00, \
  0x8c, 0x46, 0x23, 0x11, 0x80, 0x00, \
  0x47, 0x23, 0x91, 0xc8, 0xe0, 0x00, \
  0x81, 0xc0, 0xe0, 0x70, 0x30, 0x00, \
  0x12, 0x89, 0x44, 0xa2, 0x50, 0x00, \
  0x58, 0x2c, 0x16, 0x0b, 0x00, 0x00, \
  0x28, 0x94, 0x4a, 0x25, 0x10, 0x00, \
  0x34, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0x29, 0xfd, 0x91, 0x6f, 0xd0, 0x00

#define kMaskRandom36_29 \
  0x8c, 0x46, 0x23, 0x11, 0x80, 0x00, \
  0x47, 0x23, 0x91, 0xc8, 0xe0, 0x00, \
  0x81, 0xc0, 0xe0, 0x70, 0x30, 0x00, \
  0x12, 0x89, 0x44, 0xa2, 0x50, 0x00, \
  0x58, 0x2c, 0x16, 0x0b, 0x00, 0x00, \
  0x28, 0x94, 0x4a, 0x25, 0x10, 0x00, \
  0x34, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0x84, 0xc2, 0x61, 0x30, 0x90, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0xe0, 0x00, \
  0x51, 0xa8, 0xd4, 0x6a, 0x30, 0x00, \
  0x1a, 0x0d, 0x06, 0x83, 0x40, 0x00, \
  0x68, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x89, 0x44, 0xa2, 0x51, 0x20, 0x00, \
  0x7f, 0x4f, 0xdf, 0xd3, 0xf0, 0x00, \
  0x8c, 0x46, 0x23, 0x11, 0x80, 0x00, \
  0x47, 0x23, 0x91, 0xc8, 0xe0, 0x00, \
  0x81, 0xc0, 0xe0, 0x70, 0x30, 0x00, \
  0x12, 0x89, 0x44, 0xa2, 0x50, 0x00, \
  0x58, 0x2c, 0x16, 0x0b, 0x00, 0x00, \
  0x28, 0x94, 0x4a, 0x25, 0x10, 0x00, \
  0x34, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0x2c, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0x91, 0x48, 0xa4, 0x52, 0x20, 0x00, \
  0xc0, 0xe0, 0x70, 0x38, 0x10, 0x00, \
  0x06, 0x83, 0x41, 0xa0, 0xd0, 0x00, \
  0xc8, 0x64, 0x32, 0x19, 0x00, 0x00, \
  0x45, 0x22, 0x91, 0x48, 0xa0, 0x00, \
  0x30, 0x98, 0x4c, 0x26, 0x10, 0x00, \
  0xa2, 0x51, 0x28, 0x94, 0x40, 0x00

#define kMaskRandom36_3 \
  0xcc, 0x66, 0x33, 0x19, 0x80, 0x00, \
  0x27, 0x15, 0x89, 0xc5, 0x60, 0x00, \
  0x92, 0xc9, 0x64, 0xb2, 0x50, 0x00

#define kMaskRandom36_30 \
  0x8c, 0x46, 0x23, 0x11, 0x80, 0x00, \
  0x47, 0x23, 0x91, 0xc8, 0xe0, 0x00, \
  0x81, 0xc0, 0xe0, 0x70, 0x30, 0x00, \
  0x12, 0x89, 0x44, 0xa2, 0x50, 0x00, \
  0x58, 0x2c, 0x16, 0x0b, 0x00, 0x00, \
  0x28, 0x94, 0x4a, 0x25, 0x10, 0x00, \
  0x34, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0x2c, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0x91, 0x48, 0xa4, 0x52, 0x20, 0x00, \
  0xc0, 0xe0, 0x70, 0x38, 0x10, 0x00, \
  0x06, 0x83, 0x41, 0xa0, 0xd0, 0x00, \
  0xc8, 0x64, 0x32, 0x19, 0x00, 0x00, \
  0x45, 0x22, 0x91, 0x48, 0xa0, 0x00, \
  0x30, 0x98, 0x4c, 0x26, 0x10, 0x00, \
  0xa2, 0x51, 0x28, 0x94, 0x40, 0x00, \
  0x8c, 0x46, 0x23, 0x11, 0x80, 0x00, \
  0x47, 0x23, 0x91, 0xc8, 0xe0, 0x00, \
  0x81, 0xc0, 0xe0, 0x70, 0x30, 0x00, \
  0x12, 0x89, 0x44, 0xa2, 0x50, 0x00, \
  0x58, 0x2c, 0x16, 0x0b, 0x00, 0x00, \
  0x28, 0x94, 0x4a, 0x25, 0x10, 0x00, \
  0x34, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0x84, 0xc2, 0x61, 0x30, 0x90, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0xe0, 0x00, \
  0x51, 0xa8, 0xd4, 0x6a, 0x30, 0x00, \
  0x1a, 0x0d, 0x06, 0x83, 0x40, 0x00, \
  0x68, 0x34, 0x1a, 0x0d, 0x00, 0x00, \
  0x89, 0x44, 0xa2, 0x51, 0x20, 0x00, \
  0x7f, 0x4f, 0xdf, 0xd3, 0xf0, 0x00, \
  0xc5, 0x38, 0xbb, 0x98, 0x80, 0x00

#define kMaskRandom36_31 \
  0x8c, 0x46, 0x23, 0x11, 0x80, 0x00, \
  0x47, 0x23, 0x91, 0xc8, 0xe0, 0x00, \
  0x81, 0xc0, 0xe0, 0x70, 0x30, 0x00, \
  0x12, 0x89, 0x44, 0xa2, 0x50, 0x00, \
  0x58, 0x2c, 0x16, 0x0b, 0x00, 0x00, \
  0x28, 0x94, 0x4a, 0x25, 0x10, 0x00, \
  0x34, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0x2c, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0x91, 0x48, 0xa4, 0x52, 0x20, 0x00, \
  0xc0, 0xe0, 0x70, 0x38, 0x10, 0x00, \
  0x06, 0x83, 0x41, 0xa0, 0xd0, 0x00, \
  0xc8, 0x64, 0x32, 0x19, 0x00, 0x00, \
  0x45, 0x22, 0x91, 0x48, 0xa0, 0x00, \
  0x30, 0x98, 0x4c, 0x26, 0x10, 0x00, \
  0xa2, 0x51, 0x28, 0x94, 0x40, 0x00, \
  0x2c, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0x91, 0x48, 0xa4, 0x52, 0x20, 0x00, \
  0xc0, 0xe0, 0x70, 0x38, 0x10, 0x00, \
  0x06, 0x83, 0x41, 0xa0, 0xd0, 0x00, \
  0xc8, 0x64, 0x32, 0x19, 0x00, 0x00, \
  0x45, 0x22, 0x91, 0x48, 0xa0, 0x00, \
  0x30, 0x98, 0x4c, 0x26, 0x10, 0x00, \
  0xa2, 0x51, 0x28, 0x94, 0x40, 0x00, \
  0x8c, 0x46, 0x23, 0x11, 0x80, 0x00, \
  0x47, 0x23, 0x91, 0xc8, 0xe0, 0x00, \
  0x81, 0xc0, 0xe0, 0x70, 0x30, 0x00, \
  0x12, 0x89, 0x44, 0xa2, 0x50, 0x00, \
  0x58, 0x2c, 0x16, 0x0b, 0x00, 0x00, \
  0x28, 0x94, 0x4a, 0x25, 0x10, 0x00, \
  0x34, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0xef, 0xf2, 0x3b, 0xfc, 0x80, 0x00

#define kMaskRandom36_32 \
  0x2c, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0x91, 0x48, 0xa4, 0x52, 0x20, 0x00, \
  0xc0, 0xe0, 0x70, 0x38, 0x10, 0x00, \
  0x06, 0x83, 0x41, 0xa0, 0xd0, 0x00, \
  0xc8, 0x64, 0x32, 0x19, 0x00, 0x00, \
  0x45, 0x22, 0x91, 0x48, 0xa0, 0x00, \
  0x30, 0x98, 0x4c, 0x26, 0x10, 0x00, \
  0xa2, 0x51, 0x28, 0x94, 0x40, 0x00, \
  0x8c, 0x46, 0x23, 0x11, 0x80, 0x00, \
  0x47, 0x23, 0x91, 0xc8, 0xe0, 0x00, \
  0x81, 0xc0, 0xe0, 0x70, 0x30, 0x00, \
  0x12, 0x89, 0x44, 0xa2, 0x50, 0x00, \
  0x58, 0x2c, 0x16, 0x0b, 0x00, 0x00, \
  0x28, 0x94, 0x4a, 0x25, 0x10, 0x00, \
  0x34, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0xef, 0xf2, 0x3b, 0xfc, 0x80, 0x00, \
  0x8c, 0x46, 0x23, 0x11, 0x80, 0x00, \
  0x47, 0x23, 0x91, 0xc8, 0xe0, 0x00, \
  0x81, 0xc0, 0xe0, 0x70, 0x30, 0x00, \
  0x12, 0x89, 0x44, 0xa2, 0x50, 0x00, \
  0x58, 0x2c, 0x16, 0x0b, 0x00, 0x00, \
  0x28, 0x94, 0x4a, 0x25, 0x10, 0x00, \
  0x34, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0x2c, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0x91, 0x48, 0xa4, 0x52, 0x20, 0x00, \
  0xc0, 0xe0, 0x70, 0x38, 0x10, 0x00, \
  0x06, 0x83, 0x41, 0xa0, 0xd0, 0x00, \
  0xc8, 0x64, 0x32, 0x19, 0x00, 0x00, \
  0x45, 0x22, 0x91, 0x48, 0xa0, 0x00, \
  0x30, 0x98, 0x4c, 0x26, 0x10, 0x00, \
  0xa2, 0x51, 0x28, 0x94, 0x40, 0x00, \
  0x3a, 0x28, 0x9c, 0x2f, 0xc0, 0x00

#define kMaskRandom36_33 \
  0x2c, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0x91, 0x48, 0xa4, 0x52, 0x20, 0x00, \
  0xc0, 0xe0, 0x70, 0x38, 0x10, 0x00, \
  0x06, 0x83, 0x41, 0xa0, 0xd0, 0x00, \
  0xc8, 0x64, 0x32, 0x19, 0x00, 0x00, \
  0x45, 0x22, 0x91, 0x48, 0xa0, 0x00, \
  0x30, 0x98, 0x4c, 0x26, 0x10, 0x00, \
  0xa2, 0x51, 0x28, 0x94, 0x40, 0x00, \
  0x8c, 0x46, 0x23, 0x11, 0x80, 0x00, \
  0x47, 0x23, 0x91, 0xc8, 0xe0, 0x00, \
  0x81, 0xc0, 0xe0, 0x70, 0x30, 0x00, \
  0x12, 0x89, 0x44, 0xa2, 0x50, 0x00, \
  0x58, 0x2c, 0x16, 0x0b, 0x00, 0x00, \
  0x28, 0x94, 0x4a, 0x25, 0x10, 0x00, \
  0x34, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0xef, 0xf2, 0x3b, 0xfc, 0x80, 0x00, \
  0x2c, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0x91, 0x48, 0xa4, 0x52, 0x20, 0x00, \
  0xc0, 0xe0, 0x70, 0x38, 0x10, 0x00, \
  0x06, 0x83, 0x41, 0xa0, 0xd0, 0x00, \
  0xc8, 0x64, 0x32, 0x19, 0x00, 0x00, \
  0x45, 0x22, 0x91, 0x48, 0xa0, 0x00, \
  0x30, 0x98, 0x4c, 0x26, 0x10, 0x00, \
  0xa2, 0x51, 0x28, 0x94, 0x40, 0x00, \
  0x4c, 0x26, 0x13, 0x09, 0x80, 0x00, \
  0x66, 0x33, 0x19, 0x8c, 0xc0, 0x00, \
  0x91, 0x48, 0xa4, 0x52, 0x20, 0x00, \
  0x42, 0xa1, 0x50, 0xa8, 0x50, 0x00, \
  0xa4, 0x52, 0x29, 0x14, 0x80, 0x00, \
  0x13, 0x09, 0x84, 0xc2, 0x60, 0x00, \
  0x30, 0x98, 0x4c, 0x26, 0x10, 0x00, \
  0x88, 0xc4, 0x62, 0x31, 0x10, 0x00, \
  0x09, 0x04, 0x82, 0x41, 0x20, 0x00

#define kMaskRandom36_34 \
  0x2c, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0x91, 0x48, 0xa4, 0x52, 0x20, 0x00, \
  0xc0, 0xe0, 0x70, 0x38, 0x10, 0x00, \
  0x06, 0x83, 0x41, 0xa0, 0xd0, 0x00, \
  0xc8, 0x64, 0x32, 0x19, 0x00, 0x00, \
  0x45, 0x22, 0x91, 0x48, 0xa0, 0x00, \
  0x30, 0x98, 0x4c, 0x26, 0x10, 0x00, \
  0xa2, 0x51, 0x28, 0x94, 0x40, 0x00, \
  0x4c, 0x26, 0x13, 0x09, 0x80, 0x00, \
  0x66, 0x33, 0x19, 0x8c, 0xc0, 0x00, \
  0x91, 0x48, 0xa4, 0x52, 0x20, 0x00, \
  0x42, 0xa1, 0x50, 0xa8, 0x50, 0x00, \
  0xa4, 0x52, 0x29, 0x14, 0x80, 0x00, \
  0x13, 0x09, 0x84, 0xc2, 0x60, 0x00, \
  0x30, 0x98, 0x4c, 0x26, 0x10, 0x00, \
  0x88, 0xc4, 0x62, 0x31, 0x10, 0x00, \
  0x09, 0x04, 0x82, 0x41, 0x20, 0x00, \
  0x2c, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0x91, 0x48, 0xa4, 0x52, 0x20, 0x00, \
  0xc0, 0xe0, 0x70, 0x38, 0x10, 0x00, \
  0x06, 0x83, 0x41, 0xa0, 0xd0, 0x00, \
  0xc8, 0x64, 0x32, 0x19, 0x00, 0x00, \
  0x45, 0x22, 0x91, 0x48, 0xa0, 0x00, \
  0x30, 0x98, 0x4c, 0x26, 0x10, 0x00, \
  0xa2, 0x51, 0x28, 0x94, 0x40, 0x00, \
  0x8c, 0x46, 0x23, 0x11, 0x80, 0x00, \
  0x47, 0x23, 0x91, 0xc8, 0xe0, 0x00, \
  0x81, 0xc0, 0xe0, 0x70, 0x30, 0x00, \
  0x12, 0x89, 0x44, 0xa2, 0x50, 0x00, \
  0x58, 0x2c, 0x16, 0x0b, 0x00, 0x00, \
  0x28, 0x94, 0x4a, 0x25, 0x10, 0x00, \
  0x34, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0xef, 0xf2, 0x3b, 0xfc, 0x80, 0x00, \
  0xf7, 0x5e, 0x66, 0x5b, 0x60, 0x00

#define kMaskRandom36_35 \
  0x2c, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0x91, 0x48, 0xa4, 0x52, 0x20, 0x00, \
  0xc0, 0xe0, 0x70, 0x38, 0x10, 0x00, \
  0x06, 0x83, 0x41, 0xa0, 0xd0, 0x00, \
  0xc8, 0x64, 0x32, 0x19, 0x00, 0x00, \
  0x45, 0x22, 0x91, 0x48, 0xa0, 0x00, \
  0x30, 0x98, 0x4c, 0x26, 0x10, 0x00, \
  0xa2, 0x51, 0x28, 0x94, 0x40, 0x00, \
  0x4c, 0x26, 0x13, 0x09, 0x80, 0x00, \
  0x66, 0x33, 0x19, 0x8c, 0xc0, 0x00, \
  0x91, 0x48, 0xa4, 0x52, 0x20, 0x00, \
  0x42, 0xa1, 0x50, 0xa8, 0x50, 0x00, \
  0xa4, 0x52, 0x29, 0x14, 0x80, 0x00, \
  0x13, 0x09, 0x84, 0xc2, 0x60, 0x00, \
  0x30, 0x98, 0x4c, 0x26, 0x10, 0x00, \
  0x88, 0xc4, 0x62, 0x31, 0x10, 0x00, \
  0x09, 0x04, 0x82, 0x41, 0x20, 0x00, \
  0x4c, 0x26, 0x13, 0x09, 0x80, 0x00, \
  0x66, 0x33, 0x19, 0x8c, 0xc0, 0x00, \
  0x91, 0x48, 0xa4, 0x52, 0x20, 0x00, \
  0x42, 0xa1, 0x50, 0xa8, 0x50, 0x00, \
  0xa4, 0x52, 0x29, 0x14, 0x80, 0x00, \
  0x13, 0x09, 0x84, 0xc2, 0x60, 0x00, \
  0x30, 0x98, 0x4c, 0x26, 0x10, 0x00, \
  0x88, 0xc4, 0x62, 0x31, 0x10, 0x00, \
  0x09, 0x04, 0x82, 0x41, 0x20, 0x00, \
  0x2c, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0x91, 0x48, 0xa4, 0x52, 0x20, 0x00, \
  0xc0, 0xe0, 0x70, 0x38, 0x10, 0x00, \
  0x06, 0x83, 0x41, 0xa0, 0xd0, 0x00, \
  0xc8, 0x64, 0x32, 0x19, 0x00, 0x00, \
  0x45, 0x22, 0x91, 0x48, 0xa0, 0x00, \
  0x30, 0x98, 0x4c, 0x26, 0x10, 0x00, \
  0xa2, 0x51, 0x28, 0x94, 0x40, 0x00, \
  0xd0, 0x03, 0x74, 0x00, 0xd0, 0x00

#define kMaskRandom36_36 \
  0x4c, 0x26, 0x13, 0x09, 0x80, 0x00, \
  0x66, 0x33, 0x19, 0x8c, 0xc0, 0x00, \
  0x91, 0x48, 0xa4, 0x52, 0x20, 0x00, \
  0x42, 0xa1, 0x50, 0xa8, 0x50, 0x00, \
  0xa4, 0x52, 0x29, 0x14, 0x80, 0x00, \
  0x13, 0x09, 0x84, 0xc2, 0x60, 0x00, \
  0x30, 0x98, 0x4c, 0x26, 0x10, 0x00, \
  0x88, 0xc4, 0x62, 0x31, 0x10, 0x00, \
  0x09, 0x04, 0x82, 0x41, 0x20, 0x00, \
  0x2c, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0x91, 0x48, 0xa4, 0x52, 0x20, 0x00, \
  0xc0, 0xe0, 0x70, 0x38, 0x10, 0x00, \
  0x06, 0x83, 0x41, 0xa0, 0xd0, 0x00, \
  0xc8, 0x64, 0x32, 0x19, 0x00, 0x00, \
  0x45, 0x22, 0x91, 0x48, 0xa0, 0x00, \
  0x30, 0x98, 0x4c, 0x26, 0x10, 0x00, \
  0xa2, 0x51, 0x28, 0x94, 0x40, 0x00, \
  0xd0, 0x03, 0x74, 0x00, 0xd0, 0x00, \
  0x2c, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0x91, 0x48, 0xa4, 0x52, 0x20, 0x00, \
  0xc0, 0xe0, 0x70, 0x38, 0x10, 0x00, \
  0x06, 0x83, 0x41, 0xa0, 0xd0, 0x00, \
  0xc8, 0x64, 0x32, 0x19, 0x00, 0x00, \
  0x45, 0x22, 0x91, 0x48, 0xa0, 0x00, \
  0x30, 0x98, 0x4c, 0x26, 0x10, 0x00, \
  0xa2, 0x51, 0x28, 0x94, 0x40, 0x00, \
  0x4c, 0x26, 0x13, 0x09, 0x80, 0x00, \
  0x66, 0x33, 0x19, 0x8c, 0xc0, 0x00, \
  0x91, 0x48, 0xa4, 0x52, 0x20, 0x00, \
  0x42, 0xa1, 0x50, 0xa8, 0x50, 0x00, \
  0xa4, 0x52, 0x29, 0x14, 0x80, 0x00, \
  0x13, 0x09, 0x84, 0xc2, 0x60, 0x00, \
  0x30, 0x98, 0x4c, 0x26, 0x10, 0x00, \
  0x88, 0xc4, 0x62, 0x31, 0x10, 0x00, \
  0x09, 0x04, 0x82, 0x41, 0x20, 0x00, \
  0xa4, 0x9c, 0x31, 0x13, 0x80, 0x00

#define kMaskRandom36_4 \
  0xec, 0x76, 0x3b, 0x1d, 0x80, 0x00, \
  0x67, 0x33, 0x99, 0xcc, 0xe0, 0x00, \
  0xb1, 0xd8, 0xec, 0x76, 0x30, 0x00, \
  0x5a, 0xad, 0x56, 0xab, 0x50, 0x00

#define kMaskRandom36_5 \
  0x4c, 0xa6, 0x53, 0x29, 0x90, 0x00, \
  0x66, 0x33, 0x19, 0x8c, 0xc0, 0x00, \
  0x19, 0xd0, 0xc6, 0x74, 0x30, 0x00, \
  0x9c, 0x89, 0x67, 0x22, 0x50, 0x00, \
  0xe3, 0x4c, 0x38, 0xd3, 0x00, 0x00

#define kMaskRandom36_6 \
  0xcc, 0x26, 0x33, 0x09, 0x80, 0x00, \
  0x45, 0x62, 0x91, 0x58, 0xa0, 0x00, \
  0xb0, 0x98, 0x6c, 0x26, 0x10, 0x00, \
  0x8a, 0x85, 0x62, 0xa1, 0x50, 0x00, \
  0x29, 0x53, 0x0a, 0x54, 0xc0, 0x00, \
  0xa6, 0x0a, 0xa9, 0x82, 0xa0, 0x00

#define kMaskRandom36_7 \
  0x44, 0xa2, 0x51, 0x28, 0x90, 0x00, \
  0x23, 0x11, 0x88, 0xc4, 0x60, 0x00, \
  0x91, 0x48, 0xa4, 0x52, 0x20, 0x00, \
  0x0a, 0x85, 0x42, 0xa1, 0x50, 0x00, \
  0x34, 0x1a, 0x0d, 0x06, 0x80, 0x00, \
  0x0b, 0x06, 0x82, 0xc1, 0xa0, 0x00, \
  0xe0, 0x64, 0x38, 0x19, 0x00, 0x00

#define kMaskRandom36_8 \
  0x16, 0x0b, 0x05, 0x82, 0xc0, 0x00, \
  0xc2, 0x61, 0x30, 0x98, 0x40, 0x00, \
  0x60, 0xb0, 0x58, 0x2c, 0x10, 0x00, \
  0x85, 0x42, 0xa1, 0x50, 0xa0, 0x00, \
  0x4c, 0x26, 0x13, 0x09, 0x80, 0x00, \
  0x29, 0x14, 0x8a, 0x45, 0x20, 0x00, \
  0x11, 0x88, 0xc4, 0x62, 0x30, 0x00, \
  0xb0, 0x58, 0x2c, 0x16, 0x00, 0x00

#define kMaskRandom36_9 \
  0x44, 0xa2, 0x51, 0x28, 0x90, 0x00, \
  0x66, 0x26, 0x19, 0x89, 0x80, 0x00, \
  0x90, 0x49, 0x64, 0x12, 0x50, 0x00, \
  0x01, 0xa5, 0x80, 0x69, 0x60, 0x00, \
  0x0e, 0x12, 0x83, 0x84, 0xa0, 0x00, \
  0x13, 0x0b, 0x04, 0xc2, 0xc0, 0x00, \
  0x20, 0xd0, 0x48, 0x34, 0x10, 0x00, \
  0xc2, 0x51, 0x30, 0x94, 0x40, 0x00, \
  0x29, 0x0c, 0x8a, 0x43, 0x20, 0x00

#define kMaskRandom37_1 \
  0xff, 0xff, 0xff, 0xff, 0xf8, 0x00

#define kMaskRandom37_10 \
  0x8c, 0xc6, 0x63, 0x38, 0xc0, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0x70, 0x00, \
  0x32, 0x99, 0x4c, 0xa3, 0x48, 0x00, \
  0x61, 0xb0, 0xd8, 0x64, 0x98, 0x00, \
  0x5c, 0x2e, 0x17, 0x0e, 0x20, 0x00, \
  0xcc, 0x66, 0x33, 0x1d, 0x40, 0x00, \
  0x2b, 0x15, 0x8a, 0xc6, 0x70, 0x00, \
  0x32, 0x99, 0x4c, 0xb4, 0x98, 0x00, \
  0x25, 0x92, 0xc9, 0x63, 0xa8, 0x00, \
  0xfd, 0x9d, 0xd4, 0x22, 0x30, 0x00

#define kMaskRandom37_11 \
  0x8c, 0xc6, 0x63, 0x38, 0xc0, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0x70, 0x00, \
  0x32, 0x99, 0x4c, 0xa3, 0x48, 0x00, \
  0x61, 0xb0, 0xd8, 0x64, 0x98, 0x00, \
  0x5c, 0x2e, 0x17, 0x0e, 0x20, 0x00, \
  0x84, 0xc2, 0x61, 0x21, 0xc0, 0x00, \
  0x27, 0x13, 0x89, 0xc6, 0x60, 0x00, \
  0x51, 0xa8, 0xd4, 0x62, 0x18, 0x00, \
  0x1a, 0x0d, 0x06, 0x88, 0xa8, 0x00, \
  0x68, 0x34, 0x1a, 0x11, 0x10, 0x00, \
  0x89, 0x44, 0xa2, 0x5c, 0x00, 0x00

#define kMaskRandom37_12 \
  0x84, 0xc2, 0x61, 0x21, 0xc0, 0x00, \
  0x27, 0x13, 0x89, 0xc6, 0x60, 0x00, \
  0x51, 0xa8, 0xd4, 0x62, 0x18, 0x00, \
  0x1a, 0x0d, 0x06, 0x88, 0xa8, 0x00, \
  0x68, 0x34, 0x1a, 0x11, 0x10, 0x00, \
  0x89, 0x44, 0xa2, 0x5c, 0x00, 0x00, \
  0x8c, 0xc6, 0x63, 0x38, 0xc0, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0x70, 0x00, \
  0x32, 0x99, 0x4c, 0xa3, 0x48, 0x00, \
  0x61, 0xb0, 0xd8, 0x64, 0x98, 0x00, \
  0x5c, 0x2e, 0x17, 0x0e, 0x20, 0x00, \
  0x5b, 0x0c, 0x64, 0x32, 0x20, 0x00

#define kMaskRandom37_13 \
  0x84, 0xc2, 0x61, 0x21, 0xc0, 0x00, \
  0x27, 0x13, 0x89, 0xc6, 0x60, 0x00, \
  0x51, 0xa8, 0xd4, 0x62, 0x18, 0x00, \
  0x1a, 0x0d, 0x06, 0x88, 0xa8, 0x00, \
  0x68, 0x34, 0x1a, 0x11, 0x10, 0x00, \
  0x89, 0x44, 0xa2, 0x5c, 0x00, 0x00, \
  0x8c, 0x46, 0x23, 0x08, 0xc0, 0x00, \
  0x47, 0x23, 0x91, 0xc6, 0x60, 0x00, \
  0x81, 0xc0, 0xe0, 0x62, 0x18, 0x00, \
  0x12, 0x89, 0x44, 0xa1, 0x88, 0x00, \
  0x58, 0x2c, 0x16, 0x05, 0x10, 0x00, \
  0x28, 0x94, 0x4a, 0x32, 0x80, 0x00, \
  0x34, 0x1a, 0x0d, 0x18, 0x20, 0x00

#define kMaskRandom37_14 \
  0x8c, 0x46, 0x23, 0x08, 0xc0, 0x00, \
  0x47, 0x23, 0x91, 0xc6, 0x60, 0x00, \
  0x81, 0xc0, 0xe0, 0x62, 0x18, 0x00, \
  0x12, 0x89, 0x44, 0xa1, 0x88, 0x00, \
  0x58, 0x2c, 0x16, 0x05, 0x10, 0x00, \
  0x28, 0x94, 0x4a, 0x32, 0x80, 0x00, \
  0x34, 0x1a, 0x0d, 0x18, 0x20, 0x00, \
  0x84, 0xc2, 0x61, 0x21, 0xc0, 0x00, \
  0x27, 0x13, 0x89, 0xc6, 0x60, 0x00, \
  0x51, 0xa8, 0xd4, 0x62, 0x18, 0x00, \
  0x1a, 0x0d, 0x06, 0x88, 0xa8, 0x00, \
  0x68, 0x34, 0x1a, 0x11, 0x10, 0x00, \
  0x89, 0x44, 0xa2, 0x5c, 0x00, 0x00, \
  0x7f, 0x4f, 0xdb, 0x89, 0xd8, 0x00

#define kMaskRandom37_15 \
  0x8c, 0x46, 0x23, 0x08, 0xc0, 0x00, \
  0x47, 0x23, 0x91, 0xc6, 0x60, 0x00, \
  0x81, 0xc0, 0xe0, 0x62, 0x18, 0x00, \
  0x12, 0x89, 0x44, 0xa1, 0x88, 0x00, \
  0x58, 0x2c, 0x16, 0x05, 0x10, 0x00, \
  0x28, 0x94, 0x4a, 0x32, 0x80, 0x00, \
  0x34, 0x1a, 0x0d, 0x18, 0x20, 0x00, \
  0x2c, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0x91, 0x48, 0xa4, 0x50, 0x30, 0x00, \
  0xc0, 0xe0, 0x70, 0x34, 0x08, 0x00, \
  0x06, 0x83, 0x41, 0xa0, 0xa8, 0x00, \
  0xc8, 0x64, 0x32, 0x03, 0x10, 0x00, \
  0x45, 0x22, 0x91, 0x58, 0x40, 0x00, \
  0x30, 0x98, 0x4c, 0x24, 0x50, 0x00, \
  0xa2, 0x51, 0x28, 0x8a, 0x08, 0x00

#define kMaskRandom37_16 \
  0x2c, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0x91, 0x48, 0xa4, 0x50, 0x30, 0x00, \
  0xc0, 0xe0, 0x70, 0x34, 0x08, 0x00, \
  0x06, 0x83, 0x41, 0xa0, 0xa8, 0x00, \
  0xc8, 0x64, 0x32, 0x03, 0x10, 0x00, \
  0x45, 0x22, 0x91, 0x58, 0x40, 0x00, \
  0x30, 0x98, 0x4c, 0x24, 0x50, 0x00, \
  0xa2, 0x51, 0x28, 0x8a, 0x08, 0x00, \
  0x8c, 0x46, 0x23, 0x08, 0xc0, 0x00, \
  0x47, 0x23, 0x91, 0xc6, 0x60, 0x00, \
  0x81, 0xc0, 0xe0, 0x62, 0x18, 0x00, \
  0x12, 0x89, 0x44, 0xa1, 0x88, 0x00, \
  0x58, 0x2c, 0x16, 0x05, 0x10, 0x00, \
  0x28, 0x94, 0x4a, 0x32, 0x80, 0x00, \
  0x34, 0x1a, 0x0d, 0x18, 0x20, 0x00, \
  0xef, 0xf2, 0x1f, 0x9d, 0x78, 0x00

#define kMaskRandom37_17 \
  0x2c, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0x91, 0x48, 0xa4, 0x50, 0x30, 0x00, \
  0xc0, 0xe0, 0x70, 0x34, 0x08, 0x00, \
  0x06, 0x83, 0x41, 0xa0, 0xa8, 0x00, \
  0xc8, 0x64, 0x32, 0x03, 0x10, 0x00, \
  0x45, 0x22, 0x91, 0x58, 0x40, 0x00, \
  0x30, 0x98, 0x4c, 0x24, 0x50, 0x00, \
  0xa2, 0x51, 0x28, 0x8a, 0x08, 0x00, \
  0x4c, 0x26, 0x13, 0x09, 0xc0, 0x00, \
  0x66, 0x33, 0x19, 0x9c, 0x60, 0x00, \
  0x91, 0x48, 0xa4, 0x50, 0x38, 0x00, \
  0x42, 0xa1, 0x50, 0xa4, 0x28, 0x00, \
  0xa4, 0x52, 0x29, 0x0a, 0x50, 0x00, \
  0x13, 0x09, 0x84, 0xd6, 0x80, 0x00, \
  0x30, 0x98, 0x4c, 0x24, 0xd0, 0x00, \
  0x88, 0xc4, 0x62, 0x2b, 0x08, 0x00, \
  0x09, 0x04, 0x82, 0x43, 0x30, 0x00

#define kMaskRandom37_18 \
  0x4c, 0x26, 0x13, 0x09, 0xc0, 0x00, \
  0x66, 0x33, 0x19, 0x9c, 0x60, 0x00, \
  0x91, 0x48, 0xa4, 0x50, 0x38, 0x00, \
  0x42, 0xa1, 0x50, 0xa4, 0x28, 0x00, \
  0xa4, 0x52, 0x29, 0x0a, 0x50, 0x00, \
  0x13, 0x09, 0x84, 0xd6, 0x80, 0x00, \
  0x30, 0x98, 0x4c, 0x24, 0xd0, 0x00, \
  0x88, 0xc4, 0x62, 0x2b, 0x08, 0x00, \
  0x09, 0x04, 0x82, 0x43, 0x30, 0x00, \
  0x2c, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0x91, 0x48, 0xa4, 0x50, 0x30, 0x00, \
  0xc0, 0xe0, 0x70, 0x34, 0x08, 0x00, \
  0x06, 0x83, 0x41, 0xa0, 0xa8, 0x00, \
  0xc8, 0x64, 0x32, 0x03, 0x10, 0x00, \
  0x45, 0x22, 0x91, 0x58, 0x40, 0x00, \
  0x30, 0x98, 0x4c, 0x24, 0x50, 0x00, \
  0xa2, 0x51, 0x28, 0x8a, 0x08, 0x00, \
  0xd0, 0x03, 0x54, 0x65, 0xc8, 0x00

#define kMaskRandom37_19 \
  0x44, 0xa2, 0x51, 0x29, 0xc0, 0x00, \
  0x66, 0x26, 0x19, 0x9c, 0x20, 0x00, \
  0x90, 0x49, 0x44, 0xb0, 0x38, 0x00, \
  0x01, 0xa5, 0xb0, 0xc4, 0x28, 0x00, \
  0x0e, 0x12, 0xa3, 0x0a, 0x50, 0x00, \
  0x13, 0x0b, 0x04, 0x56, 0xc0, 0x00, \
  0x20, 0xd0, 0x48, 0x64, 0xd0, 0x00, \
  0xc2, 0x51, 0x28, 0x8b, 0x00, 0x00, \
  0x29, 0x0c, 0x86, 0x03, 0x38, 0x00, \
  0x8c, 0xc6, 0x63, 0x38, 0xc0, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0x70, 0x00, \
  0x32, 0x99, 0x4c, 0xa3, 0x48, 0x00, \
  0x61, 0xb0, 0xd8, 0x64, 0x98, 0x00, \
  0x5c, 0x2e, 0x17, 0x0e, 0x20, 0x00, \
  0xcc, 0x66, 0x33, 0x1d, 0x40, 0x00, \
  0x2b, 0x15, 0x8a, 0xc6, 0x70, 0x00, \
  0x32, 0x99, 0x4c, 0xb4, 0x98, 0x00, \
  0x25, 0x92, 0xc9, 0x63, 0xa8, 0x00, \
  0xfd, 0x9d, 0xd4, 0x22, 0x30, 0x00

#define kMaskRandom37_2 \
  0xce, 0x67, 0x33, 0x9d, 0xc0, 0x00, \
  0x39, 0x9c, 0xce, 0x73, 0x38, 0x00

#define kMaskRandom37_20 \
  0x8c, 0xc6, 0x63, 0x38, 0xc0, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0x70, 0x00, \
  0x32, 0x99, 0x4c, 0xa3, 0x48, 0x00, \
  0x61, 0xb0, 0xd8, 0x64, 0x98, 0x00, \
  0x5c, 0x2e, 0x17, 0x0e, 0x20, 0x00, \
  0xcc, 0x66, 0x33, 0x1d, 0x40, 0x00, \
  0x2b, 0x15, 0x8a, 0xc6, 0x70, 0x00, \
  0x32, 0x99, 0x4c, 0xb4, 0x98, 0x00, \
  0x25, 0x92, 0xc9, 0x63, 0xa8, 0x00, \
  0xfd, 0x9d, 0xd4, 0x22, 0x30, 0x00, \
  0x44, 0xa2, 0x51, 0x29, 0xc0, 0x00, \
  0x66, 0x26, 0x19, 0x9c, 0x20, 0x00, \
  0x90, 0x49, 0x44, 0xb0, 0x38, 0x00, \
  0x01, 0xa5, 0xb0, 0xc4, 0x28, 0x00, \
  0x0e, 0x12, 0xa3, 0x0a, 0x50, 0x00, \
  0x13, 0x0b, 0x04, 0x56, 0xc0, 0x00, \
  0x20, 0xd0, 0x48, 0x64, 0xd0, 0x00, \
  0xc2, 0x51, 0x28, 0x8b, 0x00, 0x00, \
  0x29, 0x0c, 0x86, 0x03, 0x38, 0x00, \
  0xe5, 0x44, 0xda, 0x3a, 0xc8, 0x00

#define kMaskRandom37_21 \
  0x8c, 0xc6, 0x63, 0x38, 0xc0, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0x70, 0x00, \
  0x32, 0x99, 0x4c, 0xa3, 0x48, 0x00, \
  0x61, 0xb0, 0xd8, 0x64, 0x98, 0x00, \
  0x5c, 0x2e, 0x17, 0x0e, 0x20, 0x00, \
  0xcc, 0x66, 0x33, 0x1d, 0x40, 0x00, \
  0x2b, 0x15, 0x8a, 0xc6, 0x70, 0x00, \
  0x32, 0x99, 0x4c, 0xb4, 0x98, 0x00, \
  0x25, 0x92, 0xc9, 0x63, 0xa8, 0x00, \
  0xfd, 0x9d, 0xd4, 0x22, 0x30, 0x00, \
  0x8c, 0xc6, 0x63, 0x38, 0xc0, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0x70, 0x00, \
  0x32, 0x99, 0x4c, 0xa3, 0x48, 0x00, \
  0x61, 0xb0, 0xd8, 0x64, 0x98, 0x00, \
  0x5c, 0x2e, 0x17, 0x0e, 0x20, 0x00, \
  0x84, 0xc2, 0x61, 0x21, 0xc0, 0x00, \
  0x27, 0x13, 0x89, 0xc6, 0x60, 0x00, \
  0x51, 0xa8, 0xd4, 0x62, 0x18, 0x00, \
  0x1a, 0x0d, 0x06, 0x88, 0xa8, 0x00, \
  0x68, 0x34, 0x1a, 0x11, 0x10, 0x00, \
  0x89, 0x44, 0xa2, 0x5c, 0x00, 0x00

#define kMaskRandom37_22 \
  0x8c, 0xc6, 0x63, 0x38, 0xc0, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0x70, 0x00, \
  0x32, 0x99, 0x4c, 0xa3, 0x48, 0x00, \
  0x61, 0xb0, 0xd8, 0x64, 0x98, 0x00, \
  0x5c, 0x2e, 0x17, 0x0e, 0x20, 0x00, \
  0x84, 0xc2, 0x61, 0x21, 0xc0, 0x00, \
  0x27, 0x13, 0x89, 0xc6, 0x60, 0x00, \
  0x51, 0xa8, 0xd4, 0x62, 0x18, 0x00, \
  0x1a, 0x0d, 0x06, 0x88, 0xa8, 0x00, \
  0x68, 0x34, 0x1a, 0x11, 0x10, 0x00, \
  0x89, 0x44, 0xa2, 0x5c, 0x00, 0x00, \
  0x8c, 0xc6, 0x63, 0x38, 0xc0, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0x70, 0x00, \
  0x32, 0x99, 0x4c, 0xa3, 0x48, 0x00, \
  0x61, 0xb0, 0xd8, 0x64, 0x98, 0x00, \
  0x5c, 0x2e, 0x17, 0x0e, 0x20, 0x00, \
  0xcc, 0x66, 0x33, 0x1d, 0x40, 0x00, \
  0x2b, 0x15, 0x8a, 0xc6, 0x70, 0x00, \
  0x32, 0x99, 0x4c, 0xb4, 0x98, 0x00, \
  0x25, 0x92, 0xc9, 0x63, 0xa8, 0x00, \
  0xfd, 0x9d, 0xd4, 0x22, 0x30, 0x00, \
  0xe4, 0xd3, 0xff, 0x5a, 0x28, 0x00

#define kMaskRandom37_23 \
  0x8c, 0xc6, 0x63, 0x38, 0xc0, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0x70, 0x00, \
  0x32, 0x99, 0x4c, 0xa3, 0x48, 0x00, \
  0x61, 0xb0, 0xd8, 0x64, 0x98, 0x00, \
  0x5c, 0x2e, 0x17, 0x0e, 0x20, 0x00, \
  0x84, 0xc2, 0x61, 0x21, 0xc0, 0x00, \
  0x27, 0x13, 0x89, 0xc6, 0x60, 0x00, \
  0x51, 0xa8, 0xd4, 0x62, 0x18, 0x00, \
  0x1a, 0x0d, 0x06, 0x88, 0xa8, 0x00, \
  0x68, 0x34, 0x1a, 0x11, 0x10, 0x00, \
  0x89, 0x44, 0xa2, 0x5c, 0x00, 0x00, \
  0x84, 0xc2, 0x61, 0x21, 0xc0, 0x00, \
  0x27, 0x13, 0x89, 0xc6, 0x60, 0x00, \
  0x51, 0xa8, 0xd4, 0x62, 0x18, 0x00, \
  0x1a, 0x0d, 0x06, 0x88, 0xa8, 0x00, \
  0x68, 0x34, 0x1a, 0x11, 0x10, 0x00, \
  0x89, 0x44, 0xa2, 0x5c, 0x00, 0x00, \
  0x8c, 0xc6, 0x63, 0x38, 0xc0, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0x70, 0x00, \
  0x32, 0x99, 0x4c, 0xa3, 0x48, 0x00, \
  0x61, 0xb0, 0xd8, 0x64, 0x98, 0x00, \
  0x5c, 0x2e, 0x17, 0x0e, 0x20, 0x00, \
  0x5b, 0x0c, 0x64, 0x32, 0x20, 0x00

#define kMaskRandom37_24 \
  0x84, 0xc2, 0x61, 0x21, 0xc0, 0x00, \
  0x27, 0x13, 0x89, 0xc6, 0x60, 0x00, \
  0x51, 0xa8, 0xd4, 0x62, 0x18, 0x00, \
  0x1a, 0x0d, 0x06, 0x88, 0xa8, 0x00, \
  0x68, 0x34, 0x1a, 0x11, 0x10, 0x00, \
  0x89, 0x44, 0xa2, 0x5c, 0x00, 0x00, \
  0x8c, 0xc6, 0x63, 0x38, 0xc0, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0x70, 0x00, \
  0x32, 0x99, 0x4c, 0xa3, 0x48, 0x00, \
  0x61, 0xb0, 0xd8, 0x64, 0x98, 0x00, \
  0x5c, 0x2e, 0x17, 0x0e, 0x20, 0x00, \
  0x5b, 0x0c, 0x64, 0x32, 0x20, 0x00, \
  0x8c, 0xc6, 0x63, 0x38, 0xc0, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0x70, 0x00, \
  0x32, 0x99, 0x4c, 0xa3, 0x48, 0x00, \
  0x61, 0xb0, 0xd8, 0x64, 0x98, 0x00, \
  0x5c, 0x2e, 0x17, 0x0e, 0x20, 0x00, \
  0x84, 0xc2, 0x61, 0x21, 0xc0, 0x00, \
  0x27, 0x13, 0x89, 0xc6, 0x60, 0x00, \
  0x51, 0xa8, 0xd4, 0x62, 0x18, 0x00, \
  0x1a, 0x0d, 0x06, 0x88, 0xa8, 0x00, \
  0x68, 0x34, 0x1a, 0x11, 0x10, 0x00, \
  0x89, 0x44, 0xa2, 0x5c, 0x00, 0x00, \
  0xad, 0x58, 0xb2, 0x36, 0x68, 0x00

#define kMaskRandom37_25 \
  0x84, 0xc2, 0x61, 0x21, 0xc0, 0x00, \
  0x27, 0x13, 0x89, 0xc6, 0x60, 0x00, \
  0x51, 0xa8, 0xd4, 0x62, 0x18, 0x00, \
  0x1a, 0x0d, 0x06, 0x88, 0xa8, 0x00, \
  0x68, 0x34, 0x1a, 0x11, 0x10, 0x00, \
  0x89, 0x44, 0xa2, 0x5c, 0x00, 0x00, \
  0x8c, 0xc6, 0x63, 0x38, 0xc0, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0x70, 0x00, \
  0x32, 0x99, 0x4c, 0xa3, 0x48, 0x00, \
  0x61, 0xb0, 0xd8, 0x64, 0x98, 0x00, \
  0x5c, 0x2e, 0x17, 0x0e, 0x20, 0x00, \
  0x5b, 0x0c, 0x64, 0x32, 0x20, 0x00, \
  0x84, 0xc2, 0x61, 0x21, 0xc0, 0x00, \
  0x27, 0x13, 0x89, 0xc6, 0x60, 0x00, \
  0x51, 0xa8, 0xd4, 0x62, 0x18, 0x00, \
  0x1a, 0x0d, 0x06, 0x88, 0xa8, 0x00, \
  0x68, 0x34, 0x1a, 0x11, 0x10, 0x00, \
  0x89, 0x44, 0xa2, 0x5c, 0x00, 0x00, \
  0x8c, 0x46, 0x23, 0x08, 0xc0, 0x00, \
  0x47, 0x23, 0x91, 0xc6, 0x60, 0x00, \
  0x81, 0xc0, 0xe0, 0x62, 0x18, 0x00, \
  0x12, 0x89, 0x44, 0xa1, 0x88, 0x00, \
  0x58, 0x2c, 0x16, 0x05, 0x10, 0x00, \
  0x28, 0x94, 0x4a, 0x32, 0x80, 0x00, \
  0x34, 0x1a, 0x0d, 0x18, 0x20, 0x00

#define kMaskRandom37_26 \
  0x84, 0xc2, 0x61, 0x21, 0xc0, 0x00, \
  0x27, 0x13, 0x89, 0xc6, 0x60, 0x00, \
  0x51, 0xa8, 0xd4, 0x62, 0x18, 0x00, \
  0x1a, 0x0d, 0x06, 0x88, 0xa8, 0x00, \
  0x68, 0x34, 0x1a, 0x11, 0x10, 0x00, \
  0x89, 0x44, 0xa2, 0x5c, 0x00, 0x00, \
  0x8c, 0x46, 0x23, 0x08, 0xc0, 0x00, \
  0x47, 0x23, 0x91, 0xc6, 0x60, 0x00, \
  0x81, 0xc0, 0xe0, 0x62, 0x18, 0x00, \
  0x12, 0x89, 0x44, 0xa1, 0x88, 0x00, \
  0x58, 0x2c, 0x16, 0x05, 0x10, 0x00, \
  0x28, 0x94, 0x4a, 0x32, 0x80, 0x00, \
  0x34, 0x1a, 0x0d, 0x18, 0x20, 0x00, \
  0x84, 0xc2, 0x61, 0x21, 0xc0, 0x00, \
  0x27, 0x13, 0x89, 0xc6, 0x60, 0x00, \
  0x51, 0xa8, 0xd4, 0x62, 0x18, 0x00, \
  0x1a, 0x0d, 0x06, 0x88, 0xa8, 0x00, \
  0x68, 0x34, 0x1a, 0x11, 0x10, 0x00, \
  0x89, 0x44, 0xa2, 0x5c, 0x00, 0x00, \
  0x8c, 0xc6, 0x63, 0x38, 0xc0, 0x00, \
  0x27, 0x13, 0x89, 0xc4, 0x70, 0x00, \
  0x32, 0x99, 0x4c, 0xa3, 0x48, 0x00, \
  0x61, 0xb0, 0xd8, 0x64, 0x98, 0x00, \
  0x5c, 0x2e, 0x17, 0x0e, 0x20, 0x00, \
  0x5b, 0x0c, 0x64, 0x32, 0x20, 0x00, \
  0x7f, 0xb2, 0x5a, 0xaa, 0x20, 0x00

#define kMaskRandom37_27 \
  0x84, 0xc2, 0x61, 0x21, 0xc0, 0x00, \
  0x27, 0x13, 0x89, 0xc6, 0x60, 0x00, \
  0x51, 0xa8, 0xd4, 0x62, 0x18, 0x00, \
  0x1a, 0x0d, 0x06, 0x88, 0xa8, 0x00, \
  0x68, 0x34, 0x1a, 0x11, 0x10, 0x00, \
  0x89, 0x44, 0xa2, 0x5c, 0x00, 0x00, \
  0x8c, 0x46, 0x23, 0x08, 0xc0, 0x00, \
  0x47, 0x23, 0x91, 0xc6, 0x60, 0x00, \
  0x81, 0xc0, 0xe0, 0x62, 0x18, 0x00, \
  0x12, 0x89, 0x44, 0xa1, 0x88, 0x00, \
  0x58, 0x2c, 0x16, 0x05, 0x10, 0x00, \
  0x28, 0x94, 0x4a, 0x32, 0x80, 0x00, \
  0x34, 0x1a, 0x0d, 0x18, 0x20, 0x00, \
  0x8c, 0x46, 0x23, 0x08, 0xc0, 0x00, \
  0x47, 0x23, 0x91, 0xc6, 0x60, 0x00, \
  0x81, 0xc0, 0xe0, 0x62, 0x18, 0x00, \
  0x12, 0x89, 0x44, 0xa1, 0x88, 0x00, \
  0x58, 0x2c, 0x16, 0x05, 0x10, 0x00, \
  0x28, 0x94, 0x4a, 0x32, 0x80, 0x00, \
  0x34, 0x1a, 0x0d, 0x18, 0x20, 0x00, \
  0x84, 0xc2, 0x61, 0x21, 0xc0, 0x00, \
  0x27, 0x13, 0x89, 0xc6, 0x60, 0x00, \
  0x51, 0xa8, 0xd4, 0x62, 0x18, 0x00, \
  0x1a, 0x0d, 0x06, 0x88, 0xa8, 0x00, \
  0x68, 0x34, 0x1a, 0x11, 0x10, 0x00, \
  0x89, 0x44, 0xa2, 0x5c, 0x00, 0x00, \
  0x7f, 0x4f, 0xdb, 0x89, 0xd8, 0x00

#define kMaskRandom37_28 \
  0x8c, 0x46, 0x23, 0x08, 0xc0, 0x00, \
  0x47, 0x23, 0x91, 0xc6, 0x60, 0x00, \
  0x81, 0xc0, 0xe0, 0x62, 0x18, 0x00, \
  0x12, 0x89, 0x44, 0xa1, 0x88, 0x00, \
  0x58, 0x2c, 0x16, 0x05, 0x10, 0x00, \
  0x28, 0x94, 0x4a, 0x32, 0x80, 0x00, \
  0x34, 0x1a, 0x0d, 0x18, 0x20, 0x00, \
  0x84, 0xc2, 0x61, 0x21, 0xc0, 0x00, \
  0x27, 0x13, 0x89, 0xc6, 0x60, 0x00, \
  0x51, 0xa8, 0xd4, 0x62, 0x18, 0x00, \
  0x1a, 0x0d, 0x06, 0x88, 0xa8, 0x00, \
  0x68, 0x34, 0x1a, 0x11, 0x10, 0x00, \
  0x89, 0x44, 0xa2, 0x5c, 0x00, 0x00, \
  0x7f, 0x4f, 0xdb, 0x89, 0xd8, 0x00, \
  0x84, 0xc2, 0x61, 0x21, 0xc0, 0x00, \
  0x27, 0x13, 0x89, 0xc6, 0x60, 0x00, \
  0x51, 0xa8, 0xd4, 0x62, 0x18, 0x00, \
  0x1a, 0x0d, 0x06, 0x88, 0xa8, 0x00, \
  0x68, 0x34, 0x1a, 0x11, 0x10, 0x00, \
  0x89, 0x44, 0xa2, 0x5c, 0x00, 0x00, \
  0x8c, 0x46, 0x23, 0x08, 0xc0, 0x00, \
  0x47, 0x23, 0x91, 0xc6, 0x60, 0x00, \
  0x81, 0xc0, 0xe0, 0x62, 0x18, 0x00, \
  0x12, 0x89, 0x44, 0xa1, 0x88, 0x00, \
  0x58, 0x2c, 0x16, 0x05, 0x10, 0x00, \
  0x28, 0x94, 0x4a, 0x32, 0x80, 0x00, \
  0x34, 0x1a, 0x0d, 0x18, 0x20, 0x00, \
  0x7b, 0xc4, 0x24, 0xbf, 0x10, 0x00

#define kMaskRandom37_29 \
  0x8c, 0x46, 0x23, 0x08, 0xc0, 0x00, \
  0x47, 0x23, 0x91, 0xc6, 0x60, 0x00, \
  0x81, 0xc0, 0xe0, 0x62, 0x18, 0x00, \
  0x12, 0x89, 0x44, 0xa1, 0x88, 0x00, \
  0x58, 0x2c, 0x16, 0x05, 0x10, 0x00, \
  0x28, 0x94, 0x4a, 0x32, 0x80, 0x00, \
  0x34, 0x1a, 0x0d, 0x18, 0x20, 0x00, \
  0x84, 0xc2, 0x61, 0x21, 0xc0, 0x00, \
  0x27, 0x13, 0x89, 0xc6, 0x60, 0x00, \
  0x51, 0xa8, 0xd4, 0x62, 0x18, 0x00, \
  0x1a, 0x0d, 0x06, 0x88, 0xa8, 0x00, \
  0x68, 0x34, 0x1a, 0x11, 0x10, 0x00, \
  0x89, 0x44, 0xa2, 0x5c, 0x00, 0x00, \
  0x7f, 0x4f, 0xdb, 0x89, 0xd8, 0x00, \
  0x8c, 0x46, 0x23, 0x08, 0xc0, 0x00, \
  0x47, 0x23, 0x91, 0xc6, 0x60, 0x00, \
  0x81, 0xc0, 0xe0, 0x62, 0x18, 0x00, \
  0x12, 0x89, 0x44, 0xa1, 0x88, 0x00, \
  0x58, 0x2c, 0x16, 0x05, 0x10, 0x00, \
  0x28, 0x94, 0x4a, 0x32, 0x80, 0x00, \
  0x34, 0x1a, 0x0d, 0x18, 0x20, 0x00, \
  0x2c, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0x91, 0x48, 0xa4, 0x50, 0x30, 0x00, \
  0xc0, 0xe0, 0x70, 0x34, 0x08, 0x00, \
  0x06, 0x83, 0x41, 0xa0, 0xa8, 0x00, \
  0xc8, 0x64, 0x32, 0x03, 0x10, 0x00, \
  0x45, 0x22, 0x91, 0x58, 0x40, 0x00, \
  0x30, 0x98, 0x4c, 0x24, 0x50, 0x00, \
  0xa2, 0x51, 0x28, 0x8a, 0x08, 0x00

#define kMaskRandom37_3 \
  0xcc, 0x66, 0x33, 0x19, 0xc0, 0x00, \
  0x27, 0x15, 0x89, 0xcb, 0x30, 0x00, \
  0x92, 0xc9, 0x64, 0xb4, 0x98, 0x00

#define kMaskRandom37_30 \
  0x8c, 0x46, 0x23, 0x08, 0xc0, 0x00, \
  0x47, 0x23, 0x91, 0xc6, 0x60, 0x00, \
  0x81, 0xc0, 0xe0, 0x62, 0x18, 0x00, \
  0x12, 0x89, 0x44, 0xa1, 0x88, 0x00, \
  0x58, 0x2c, 0x16, 0x05, 0x10, 0x00, \
  0x28, 0x94, 0x4a, 0x32, 0x80, 0x00, \
  0x34, 0x1a, 0x0d, 0x18, 0x20, 0x00, \
  0x2c, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0x91, 0x48, 0xa4, 0x50, 0x30, 0x00, \
  0xc0, 0xe0, 0x70, 0x34, 0x08, 0x00, \
  0x06, 0x83, 0x41, 0xa0, 0xa8, 0x00, \
  0xc8, 0x64, 0x32, 0x03, 0x10, 0x00, \
  0x45, 0x22, 0x91, 0x58, 0x40, 0x00, \
  0x30, 0x98, 0x4c, 0x24, 0x50, 0x00, \
  0xa2, 0x51, 0x28, 0x8a, 0x08, 0x00, \
  0x8c, 0x46, 0x23, 0x08, 0xc0, 0x00, \
  0x47, 0x23, 0x91, 0xc6, 0x60, 0x00, \
  0x81, 0xc0, 0xe0, 0x62, 0x18, 0x00, \
  0x12, 0x89, 0x44, 0xa1, 0x88, 0x00, \
  0x58, 0x2c, 0x16, 0x05, 0x10, 0x00, \
  0x28, 0x94, 0x4a, 0x32, 0x80, 0x00, \
  0x34, 0x1a, 0x0d, 0x18, 0x20, 0x00, \
  0x84, 0xc2, 0x61, 0x21, 0xc0, 0x00, \
  0x27, 0x13, 0x89, 0xc6, 0x60, 0x00, \
  0x51, 0xa8, 0xd4, 0x62, 0x18, 0x00, \
  0x1a, 0x0d, 0x06, 0x88, 0xa8, 0x00, \
  0x68, 0x34, 0x1a, 0x11, 0x10, 0x00, \
  0x89, 0x44, 0xa2, 0x5c, 0x00, 0x00, \
  0x7f, 0x4f, 0xdb, 0x89, 0xd8, 0x00, \
  0x1d, 0x8e, 0x11, 0xb0, 0xe8, 0x00

#define kMaskRandom37_31 \
  0x8c, 0x46, 0x23, 0x08, 0xc0, 0x00, \
  0x47, 0x23, 0x91, 0xc6, 0x60, 0x00, \
  0x81, 0xc0, 0xe0, 0x62, 0x18, 0x00, \
  0x12, 0x89, 0x44, 0xa1, 0x88, 0x00, \
  0x58, 0x2c, 0x16, 0x05, 0x10, 0x00, \
  0x28, 0x94, 0x4a, 0x32, 0x80, 0x00, \
  0x34, 0x1a, 0x0d, 0x18, 0x20, 0x00, \
  0x2c, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0x91, 0x48, 0xa4, 0x50, 0x30, 0x00, \
  0xc0, 0xe0, 0x70, 0x34, 0x08, 0x00, \
  0x06, 0x83, 0x41, 0xa0, 0xa8, 0x00, \
  0xc8, 0x64, 0x32, 0x03, 0x10, 0x00, \
  0x45, 0x22, 0x91, 0x58, 0x40, 0x00, \
  0x30, 0x98, 0x4c, 0x24, 0x50, 0x00, \
  0xa2, 0x51, 0x28, 0x8a, 0x08, 0x00, \
  0x2c, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0x91, 0x48, 0xa4, 0x50, 0x30, 0x00, \
  0xc0, 0xe0, 0x70, 0x34, 0x08, 0x00, \
  0x06, 0x83, 0x41, 0xa0, 0xa8, 0x00, \
  0xc8, 0x64, 0x32, 0x03, 0x10, 0x00, \
  0x45, 0x22, 0x91, 0x58, 0x40, 0x00, \
  0x30, 0x98, 0x4c, 0x24, 0x50, 0x00, \
  0xa2, 0x51, 0x28, 0x8a, 0x08, 0x00, \
  0x8c, 0x46, 0x23, 0x08, 0xc0, 0x00, \
  0x47, 0x23, 0x91, 0xc6, 0x60, 0x00, \
  0x81, 0xc0, 0xe0, 0x62, 0x18, 0x00, \
  0x12, 0x89, 0x44, 0xa1, 0x88, 0x00, \
  0x58, 0x2c, 0x16, 0x05, 0x10, 0x00, \
  0x28, 0x94, 0x4a, 0x32, 0x80, 0x00, \
  0x34, 0x1a, 0x0d, 0x18, 0x20, 0x00, \
  0xef, 0xf2, 0x1f, 0x9d, 0x78, 0x00

#define kMaskRandom37_32 \
  0x2c, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0x91, 0x48, 0xa4, 0x50, 0x30, 0x00, \
  0xc0, 0xe0, 0x70, 0x34, 0x08, 0x00, \
  0x06, 0x83, 0x41, 0xa0, 0xa8, 0x00, \
  0xc8, 0x64, 0x32, 0x03, 0x10, 0x00, \
  0x45, 0x22, 0x91, 0x58, 0x40, 0x00, \
  0x30, 0x98, 0x4c, 0x24, 0x50, 0x00, \
  0xa2, 0x51, 0x28, 0x8a, 0x08, 0x00, \
  0x8c, 0x46, 0x23, 0x08, 0xc0, 0x00, \
  0x47, 0x23, 0x91, 0xc6, 0x60, 0x00, \
  0x81, 0xc0, 0xe0, 0x62, 0x18, 0x00, \
  0x12, 0x89, 0x44, 0xa1, 0x88, 0x00, \
  0x58, 0x2c, 0x16, 0x05, 0x10, 0x00, \
  0x28, 0x94, 0x4a, 0x32, 0x80, 0x00, \
  0x34, 0x1a, 0x0d, 0x18, 0x20, 0x00, \
  0xef, 0xf2, 0x1f, 0x9d, 0x78, 0x00, \
  0x8c, 0x46, 0x23, 0x08, 0xc0, 0x00, \
  0x47, 0x23, 0x91, 0xc6, 0x60, 0x00, \
  0x81, 0xc0, 0xe0, 0x62, 0x18, 0x00, \
  0x12, 0x89, 0x44, 0xa1, 0x88, 0x00, \
  0x58, 0x2c, 0x16, 0x05, 0x10, 0x00, \
  0x28, 0x94, 0x4a, 0x32, 0x80, 0x00, \
  0x34, 0x1a, 0x0d, 0x18, 0x20, 0x00, \
  0x2c, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0x91, 0x48, 0xa4, 0x50, 0x30, 0x00, \
  0xc0, 0xe0, 0x70, 0x34, 0x08, 0x00, \
  0x06, 0x83, 0x41, 0xa0, 0xa8, 0x00, \
  0xc8, 0x64, 0x32, 0x03, 0x10, 0x00, \
  0x45, 0x22, 0x91, 0x58, 0x40, 0x00, \
  0x30, 0x98, 0x4c, 0x24, 0x50, 0x00, \
  0xa2, 0x51, 0x28, 0x8a, 0x08, 0x00, \
  0xf7, 0x95, 0x57, 0x8c, 0x40, 0x00

#define kMaskRandom37_33 \
  0x2c, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0x91, 0x48, 0xa4, 0x50, 0x30, 0x00, \
  0xc0, 0xe0, 0x70, 0x34, 0x08, 0x00, \
  0x06, 0x83, 0x41, 0xa0, 0xa8, 0x00, \
  0xc8, 0x64, 0x32, 0x03, 0x10, 0x00, \
  0x45, 0x22, 0x91, 0x58, 0x40, 0x00, \
  0x30, 0x98, 0x4c, 0x24, 0x50, 0x00, \
  0xa2, 0x51, 0x28, 0x8a, 0x08, 0x00, \
  0x8c, 0x46, 0x23, 0x08, 0xc0, 0x00, \
  0x47, 0x23, 0x91, 0xc6, 0x60, 0x00, \
  0x81, 0xc0, 0xe0, 0x62, 0x18, 0x00, \
  0x12, 0x89, 0x44, 0xa1, 0x88, 0x00, \
  0x58, 0x2c, 0x16, 0x05, 0x10, 0x00, \
  0x28, 0x94, 0x4a, 0x32, 0x80, 0x00, \
  0x34, 0x1a, 0x0d, 0x18, 0x20, 0x00, \
  0xef, 0xf2, 0x1f, 0x9d, 0x78, 0x00, \
  0x2c, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0x91, 0x48, 0xa4, 0x50, 0x30, 0x00, \
  0xc0, 0xe0, 0x70, 0x34, 0x08, 0x00, \
  0x06, 0x83, 0x41, 0xa0, 0xa8, 0x00, \
  0xc8, 0x64, 0x32, 0x03, 0x10, 0x00, \
  0x45, 0x22, 0x91, 0x58, 0x40, 0x00, \
  0x30, 0x98, 0x4c, 0x24, 0x50, 0x00, \
  0xa2, 0x51, 0x28, 0x8a, 0x08, 0x00, \
  0x4c, 0x26, 0x13, 0x09, 0xc0, 0x00, \
  0x66, 0x33, 0x19, 0x9c, 0x60, 0x00, \
  0x91, 0x48, 0xa4, 0x50, 0x38, 0x00, \
  0x42, 0xa1, 0x50, 0xa4, 0x28, 0x00, \
  0xa4, 0x52, 0x29, 0x0a, 0x50, 0x00, \
  0x13, 0x09, 0x84, 0xd6, 0x80, 0x00, \
  0x30, 0x98, 0x4c, 0x24, 0xd0, 0x00, \
  0x88, 0xc4, 0x62, 0x2b, 0x08, 0x00, \
  0x09, 0x04, 0x82, 0x43, 0x30, 0x00

#define kMaskRandom37_34 \
  0x2c, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0x91, 0x48, 0xa4, 0x50, 0x30, 0x00, \
  0xc0, 0xe0, 0x70, 0x34, 0x08, 0x00, \
  0x06, 0x83, 0x41, 0xa0, 0xa8, 0x00, \
  0xc8, 0x64, 0x32, 0x03, 0x10, 0x00, \
  0x45, 0x22, 0x91, 0x58, 0x40, 0x00, \
  0x30, 0x98, 0x4c, 0x24, 0x50, 0x00, \
  0xa2, 0x51, 0x28, 0x8a, 0x08, 0x00, \
  0x4c, 0x26, 0x13, 0x09, 0xc0, 0x00, \
  0x66, 0x33, 0x19, 0x9c, 0x60, 0x00, \
  0x91, 0x48, 0xa4, 0x50, 0x38, 0x00, \
  0x42, 0xa1, 0x50, 0xa4, 0x28, 0x00, \
  0xa4, 0x52, 0x29, 0x0a, 0x50, 0x00, \
  0x13, 0x09, 0x84, 0xd6, 0x80, 0x00, \
  0x30, 0x98, 0x4c, 0x24, 0xd0, 0x00, \
  0x88, 0xc4, 0x62, 0x2b, 0x08, 0x00, \
  0x09, 0x04, 0x82, 0x43, 0x30, 0x00, \
  0x2c, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0x91, 0x48, 0xa4, 0x50, 0x30, 0x00, \
  0xc0, 0xe0, 0x70, 0x34, 0x08, 0x00, \
  0x06, 0x83, 0x41, 0xa0, 0xa8, 0x00, \
  0xc8, 0x64, 0x32, 0x03, 0x10, 0x00, \
  0x45, 0x22, 0x91, 0x58, 0x40, 0x00, \
  0x30, 0x98, 0x4c, 0x24, 0x50, 0x00, \
  0xa2, 0x51, 0x28, 0x8a, 0x08, 0x00, \
  0x8c, 0x46, 0x23, 0x08, 0xc0, 0x00, \
  0x47, 0x23, 0x91, 0xc6, 0x60, 0x00, \
  0x81, 0xc0, 0xe0, 0x62, 0x18, 0x00, \
  0x12, 0x89, 0x44, 0xa1, 0x88, 0x00, \
  0x58, 0x2c, 0x16, 0x05, 0x10, 0x00, \
  0x28, 0x94, 0x4a, 0x32, 0x80, 0x00, \
  0x34, 0x1a, 0x0d, 0x18, 0x20, 0x00, \
  0xef, 0xf2, 0x1f, 0x9d, 0x78, 0x00, \
  0x31, 0x9c, 0xfb, 0x37, 0xc0, 0x00

#define kMaskRandom37_35 \
  0x2c, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0x91, 0x48, 0xa4, 0x50, 0x30, 0x00, \
  0xc0, 0xe0, 0x70, 0x34, 0x08, 0x00, \
  0x06, 0x83, 0x41, 0xa0, 0xa8, 0x00, \
  0xc8, 0x64, 0x32, 0x03, 0x10, 0x00, \
  0x45, 0x22, 0x91, 0x58, 0x40, 0x00, \
  0x30, 0x98, 0x4c, 0x24, 0x50, 0x00, \
  0xa2, 0x51, 0x28, 0x8a, 0x08, 0x00, \
  0x4c, 0x26, 0x13, 0x09, 0xc0, 0x00, \
  0x66, 0x33, 0x19, 0x9c, 0x60, 0x00, \
  0x91, 0x48, 0xa4, 0x50, 0x38, 0x00, \
  0x42, 0xa1, 0x50, 0xa4, 0x28, 0x00, \
  0xa4, 0x52, 0x29, 0x0a, 0x50, 0x00, \
  0x13, 0x09, 0x84, 0xd6, 0x80, 0x00, \
  0x30, 0x98, 0x4c, 0x24, 0xd0, 0x00, \
  0x88, 0xc4, 0x62, 0x2b, 0x08, 0x00, \
  0x09, 0x04, 0x82, 0x43, 0x30, 0x00, \
  0x4c, 0x26, 0x13, 0x09, 0xc0, 0x00, \
  0x66, 0x33, 0x19, 0x9c, 0x60, 0x00, \
  0x91, 0x48, 0xa4, 0x50, 0x38, 0x00, \
  0x42, 0xa1, 0x50, 0xa4, 0x28, 0x00, \
  0xa4, 0x52, 0x29, 0x0a, 0x50, 0x00, \
  0x13, 0x09, 0x84, 0xd6, 0x80, 0x00, \
  0x30, 0x98, 0x4c, 0x24, 0xd0, 0x00, \
  0x88, 0xc4, 0x62, 0x2b, 0x08, 0x00, \
  0x09, 0x04, 0x82, 0x43, 0x30, 0x00, \
  0x2c, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0x91, 0x48, 0xa4, 0x50, 0x30, 0x00, \
  0xc0, 0xe0, 0x70, 0x34, 0x08, 0x00, \
  0x06, 0x83, 0x41, 0xa0, 0xa8, 0x00, \
  0xc8, 0x64, 0x32, 0x03, 0x10, 0x00, \
  0x45, 0x22, 0x91, 0x58, 0x40, 0x00, \
  0x30, 0x98, 0x4c, 0x24, 0x50, 0x00, \
  0xa2, 0x51, 0x28, 0x8a, 0x08, 0x00, \
  0xd0, 0x03, 0x54, 0x65, 0xc8, 0x00

#define kMaskRandom37_36 \
  0x4c, 0x26, 0x13, 0x09, 0xc0, 0x00, \
  0x66, 0x33, 0x19, 0x9c, 0x60, 0x00, \
  0x91, 0x48, 0xa4, 0x50, 0x38, 0x00, \
  0x42, 0xa1, 0x50, 0xa4, 0x28, 0x00, \
  0xa4, 0x52, 0x29, 0x0a, 0x50, 0x00, \
  0x13, 0x09, 0x84, 0xd6, 0x80, 0x00, \
  0x30, 0x98, 0x4c, 0x24, 0xd0, 0x00, \
  0x88, 0xc4, 0x62, 0x2b, 0x08, 0x00, \
  0x09, 0x04, 0x82, 0x43, 0x30, 0x00, \
  0x2c, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0x91, 0x48, 0xa4, 0x50, 0x30, 0x00, \
  0xc0, 0xe0, 0x70, 0x34, 0x08, 0x00, \
  0x06, 0x83, 0x41, 0xa0, 0xa8, 0x00, \
  0xc8, 0x64, 0x32, 0x03, 0x10, 0x00, \
  0x45, 0x22, 0x91, 0x58, 0x40, 0x00, \
  0x30, 0x98, 0x4c, 0x24, 0x50, 0x00, \
  0xa2, 0x51, 0x28, 0x8a, 0x08, 0x00, \
  0xd0, 0x03, 0x54, 0x65, 0xc8, 0x00, \
  0x2c, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0x91, 0x48, 0xa4, 0x50, 0x30, 0x00, \
  0xc0, 0xe0, 0x70, 0x34, 0x08, 0x00, \
  0x06, 0x83, 0x41, 0xa0, 0xa8, 0x00, \
  0xc8, 0x64, 0x32, 0x03, 0x10, 0x00, \
  0x45, 0x22, 0x91, 0x58, 0x40, 0x00, \
  0x30, 0x98, 0x4c, 0x24, 0x50, 0x00, \
  0xa2, 0x51, 0x28, 0x8a, 0x08, 0x00, \
  0x4c, 0x26, 0x13, 0x09, 0xc0, 0x00, \
  0x66, 0x33, 0x19, 0x9c, 0x60, 0x00, \
  0x91, 0x48, 0xa4, 0x50, 0x38, 0x00, \
  0x42, 0xa1, 0x50, 0xa4, 0x28, 0x00, \
  0xa4, 0x52, 0x29, 0x0a, 0x50, 0x00, \
  0x13, 0x09, 0x84, 0xd6, 0x80, 0x00, \
  0x30, 0x98, 0x4c, 0x24, 0xd0, 0x00, \
  0x88, 0xc4, 0x62, 0x2b, 0x08, 0x00, \
  0x09, 0x04, 0x82, 0x43, 0x30, 0x00, \
  0xc3, 0xc7, 0xce, 0xd8, 0x50, 0x00

#define kMaskRandom37_37 \
  0x4c, 0x26, 0x13, 0x09, 0xc0, 0x00, \
  0x66, 0x33, 0x19, 0x9c, 0x60, 0x00, \
  0x91, 0x48, 0xa4, 0x50, 0x38, 0x00, \
  0x42, 0xa1, 0x50, 0xa4, 0x28, 0x00, \
  0xa4, 0x52, 0x29, 0x0a, 0x50, 0x00, \
  0x13, 0x09, 0x84, 0xd6, 0x80, 0x00, \
  0x30, 0x98, 0x4c, 0x24, 0xd0, 0x00, \
  0x88, 0xc4, 0x62, 0x2b, 0x08, 0x00, \
  0x09, 0x04, 0x82, 0x43, 0x30, 0x00, \
  0x2c, 0x16, 0x0b, 0x05, 0x80, 0x00, \
  0x91, 0x48, 0xa4, 0x50, 0x30, 0x00, \
  0xc0, 0xe0, 0x70, 0x34, 0x08, 0x00, \
  0x06, 0x83, 0x41, 0xa0, 0xa8, 0x00, \
  0xc8, 0x64, 0x32, 0x03, 0x10, 0x00, \
  0x45, 0x22, 0x91, 0x58, 0x40, 0x00, \
  0x30, 0x98, 0x4c, 0x24, 0x50, 0x00, \
  0xa2, 0x51, 0x28, 0x8a, 0x08, 0x00, \
  0xd0, 0x03, 0x54, 0x65, 0xc8, 0x00, \
  0x4c, 0x26, 0x13, 0x09, 0xc0, 0x00, \
  0x66, 0x33, 0x19, 0x9c, 0x60, 0x00, \
  0x91, 0x48, 0xa4, 0x50, 0x38, 0x00, \
  0x42, 0xa1, 0x50, 0xa4, 0x28, 0x00, \
  0xa4, 0x52, 0x29, 0x0a, 0x50, 0x00, \
  0x13, 0x09, 0x84, 0xd6, 0x80, 0x00, \
  0x30, 0x98, 0x4c, 0x24, 0xd0, 0x00, \
  0x88, 0xc4, 0x62, 0x2b, 0x08, 0x00, \
  0x09, 0x04, 0x82, 0x43, 0x30, 0x00, \
  0x2c, 0x16, 0x13, 0x09, 0x80, 0x00, \
  0x91, 0x48, 0x99, 0x8a, 0x20, 0x00, \
  0xc0, 0xe0, 0x64, 0x54, 0x08, 0x00, \
  0x06, 0x83, 0x50, 0xa0, 0x98, 0x00, \
  0xc8, 0x64, 0x29, 0x00, 0x70, 0x00, \
  0x45, 0x22, 0x84, 0xd0, 0xc0, 0x00, \
  0x30, 0x98, 0x4c, 0x25, 0x20, 0x00, \
  0xa2, 0x51, 0x22, 0x28, 0x48, 0x00, \
  0xd0, 0x03, 0x42, 0x53, 0x00, 0x00, \
  0xee, 0xf5, 0xb3, 0x66, 0x10, 0x00

#define kMaskRandom37_4 \
  0xec, 0x76, 0x3b, 0x1c, 0xc0, 0x00, \
  0x67, 0x33, 0x99, 0xc6, 0x70, 0x00, \
  0xb1, 0xd8, 0xec, 0x73, 0x18, 0x00, \
  0x5a, 0xad, 0x56, 0xa5, 0xa8, 0x00

#define kMaskRandom37_5 \
  0x4c, 0xa6, 0x53, 0x39, 0xc0, 0x00, \
  0x66, 0x33, 0x19, 0x8c, 0x70, 0x00, \
  0x19, 0xd0, 0xe8, 0x73, 0x18, 0x00, \
  0x9c, 0x89, 0x64, 0xa9, 0xa8, 0x00, \
  0xe3, 0x4c, 0x2e, 0x26, 0x60, 0x00

#define kMaskRandom37_6 \
  0xcc, 0x26, 0x13, 0x0d, 0x80, 0x00, \
  0x45, 0x62, 0x91, 0x5a, 0x20, 0x00, \
  0xb0, 0x98, 0x4c, 0x34, 0x18, 0x00, \
  0x8a, 0x85, 0x62, 0xa0, 0xa8, 0x00, \
  0x29, 0x53, 0x09, 0x82, 0xd0, 0x00, \
  0xa6, 0x0a, 0xa5, 0x51, 0x40, 0x00

#define kMaskRandom37_7 \
  0x44, 0xa2, 0x71, 0x28, 0xc0, 0x00, \
  0x23, 0x11, 0x88, 0xc6, 0x60, 0x00, \
  0x91, 0x48, 0xa4, 0x47, 0x08, 0x00, \
  0x0a, 0x85, 0x52, 0xa0, 0xa8, 0x00, \
  0x34, 0x1a, 0x0d, 0x12, 0x50, 0x00, \
  0x0b, 0x06, 0xa2, 0xd2, 0x80, 0x00, \
  0xe0, 0x64, 0x32, 0x09, 0x30, 0x00

#define kMaskRandom37_8 \
  0x16, 0x0b, 0x05, 0x84, 0xe0, 0x00, \
  0xc2, 0x61, 0x30, 0x91, 0x30, 0x00, \
  0x60, 0xb0, 0x58, 0x3a, 0x08, 0x00, \
  0x85, 0x42, 0xa1, 0x44, 0x98, 0x00, \
  0x4c, 0x26, 0x33, 0x08, 0x50, 0x00, \
  0x29, 0x14, 0x8a, 0x58, 0xc0, 0x00, \
  0x11, 0x88, 0xc4, 0x66, 0x30, 0x00, \
  0xb0, 0x58, 0x2c, 0x03, 0x18, 0x00

#define kMaskRandom37_9 \
  0x44, 0xa2, 0x51, 0x29, 0xc0, 0x00, \
  0x66, 0x26, 0x19, 0x9c, 0x20, 0x00, \
  0x90, 0x49, 0x44, 0xb0, 0x38, 0x00, \
  0x01, 0xa5, 0xb0, 0xc4, 0x28, 0x00, \
  0x0e, 0x12, 0xa3, 0x0a, 0x50, 0x00, \
  0x13, 0x0b, 0x04, 0x56, 0xc0, 0x00, \
  0x20, 0xd0, 0x48, 0x64, 0xd0, 0x00, \
  0xc2, 0x51, 0x28, 0x8b, 0x00, 0x00, \
  0x29, 0x0c, 0x86, 0x03, 0x38, 0x00

#define kMaskRandom38_1 \
  0xff, 0xff, 0xff, 0xff, 0xfc, 0x00

#define kMaskRandom38_10 \
  0x8c, 0xe3, 0x11, 0x9c, 0x60, 0x00, \
  0x27, 0x11, 0xc4, 0xe2, 0x38, 0x00, \
  0x32, 0x8d, 0x26, 0x51, 0xa4, 0x00, \
  0x61, 0x92, 0x6c, 0x32, 0x4c, 0x00, \
  0x5c, 0x38, 0x8b, 0x87, 0x10, 0x00, \
  0xcc, 0x75, 0x19, 0x8e, 0xa0, 0x00, \
  0x2b, 0x19, 0xc5, 0x63, 0x38, 0x00, \
  0x32, 0xd2, 0x66, 0x5a, 0x4c, 0x00, \
  0x25, 0x8e, 0xa4, 0xb1, 0xd4, 0x00, \
  0x50, 0x88, 0xca, 0x11, 0x18, 0x00

#define kMaskRandom38_11 \
  0x8c, 0xe3, 0x11, 0x9c, 0x60, 0x00, \
  0x27, 0x11, 0xc4, 0xe2, 0x38, 0x00, \
  0x32, 0x8d, 0x26, 0x51, 0xa4, 0x00, \
  0x61, 0x92, 0x6c, 0x32, 0x4c, 0x00, \
  0x5c, 0x38, 0x8b, 0x87, 0x10, 0x00, \
  0x84, 0x87, 0x10, 0x90, 0xe0, 0x00, \
  0x27, 0x19, 0x84, 0xe3, 0x30, 0x00, \
  0x51, 0x88, 0x6a, 0x31, 0x0c, 0x00, \
  0x1a, 0x22, 0xa3, 0x44, 0x54, 0x00, \
  0x68, 0x44, 0x4d, 0x08, 0x88, 0x00, \
  0x89, 0x70, 0x11, 0x2e, 0x00, 0x00

#define kMaskRandom38_12 \
  0x84, 0x87, 0x10, 0x90, 0xe0, 0x00, \
  0x27, 0x19, 0x84, 0xe3, 0x30, 0x00, \
  0x51, 0x88, 0x6a, 0x31, 0x0c, 0x00, \
  0x1a, 0x22, 0xa3, 0x44, 0x54, 0x00, \
  0x68, 0x44, 0x4d, 0x08, 0x88, 0x00, \
  0x89, 0x70, 0x11, 0x2e, 0x00, 0x00, \
  0x8c, 0xe3, 0x11, 0x9c, 0x60, 0x00, \
  0x27, 0x11, 0xc4, 0xe2, 0x38, 0x00, \
  0x32, 0x8d, 0x26, 0x51, 0xa4, 0x00, \
  0x61, 0x92, 0x6c, 0x32, 0x4c, 0x00, \
  0x5c, 0x38, 0x8b, 0x87, 0x10, 0x00, \
  0x90, 0xc8, 0x92, 0x19, 0x10, 0x00

#define kMaskRandom38_13 \
  0x84, 0x87, 0x10, 0x90, 0xe0, 0x00, \
  0x27, 0x19, 0x84, 0xe3, 0x30, 0x00, \
  0x51, 0x88, 0x6a, 0x31, 0x0c, 0x00, \
  0x1a, 0x22, 0xa3, 0x44, 0x54, 0x00, \
  0x68, 0x44, 0x4d, 0x08, 0x88, 0x00, \
  0x89, 0x70, 0x11, 0x2e, 0x00, 0x00, \
  0x8c, 0x23, 0x11, 0x84, 0x60, 0x00, \
  0x47, 0x19, 0x88, 0xe3, 0x30, 0x00, \
  0x81, 0x88, 0x70, 0x31, 0x0c, 0x00, \
  0x12, 0x86, 0x22, 0x50, 0xc4, 0x00, \
  0x58, 0x14, 0x4b, 0x02, 0x88, 0x00, \
  0x28, 0xca, 0x05, 0x19, 0x40, 0x00, \
  0x34, 0x60, 0x86, 0x8c, 0x10, 0x00

#define kMaskRandom38_14 \
  0x8c, 0x23, 0x11, 0x84, 0x60, 0x00, \
  0x47, 0x19, 0x88, 0xe3, 0x30, 0x00, \
  0x81, 0x88, 0x70, 0x31, 0x0c, 0x00, \
  0x12, 0x86, 0x22, 0x50, 0xc4, 0x00, \
  0x58, 0x14, 0x4b, 0x02, 0x88, 0x00, \
  0x28, 0xca, 0x05, 0x19, 0x40, 0x00, \
  0x34, 0x60, 0x86, 0x8c, 0x10, 0x00, \
  0x84, 0x87, 0x10, 0x90, 0xe0, 0x00, \
  0x27, 0x19, 0x84, 0xe3, 0x30, 0x00, \
  0x51, 0x88, 0x6a, 0x31, 0x0c, 0x00, \
  0x1a, 0x22, 0xa3, 0x44, 0x54, 0x00, \
  0x68, 0x44, 0x4d, 0x08, 0x88, 0x00, \
  0x89, 0x70, 0x11, 0x2e, 0x00, 0x00, \
  0x6e, 0x27, 0x6d, 0xc4, 0xec, 0x00

#define kMaskRandom38_15 \
  0x8c, 0x23, 0x11, 0x84, 0x60, 0x00, \
  0x47, 0x19, 0x88, 0xe3, 0x30, 0x00, \
  0x81, 0x88, 0x70, 0x31, 0x0c, 0x00, \
  0x12, 0x86, 0x22, 0x50, 0xc4, 0x00, \
  0x58, 0x14, 0x4b, 0x02, 0x88, 0x00, \
  0x28, 0xca, 0x05, 0x19, 0x40, 0x00, \
  0x34, 0x60, 0x86, 0x8c, 0x10, 0x00, \
  0x2c, 0x16, 0x05, 0x82, 0xc0, 0x00, \
  0x91, 0x40, 0xd2, 0x28, 0x18, 0x00, \
  0xc0, 0xd0, 0x38, 0x1a, 0x04, 0x00, \
  0x06, 0x82, 0xa0, 0xd0, 0x54, 0x00, \
  0xc8, 0x0c, 0x59, 0x01, 0x88, 0x00, \
  0x45, 0x61, 0x08, 0xac, 0x20, 0x00, \
  0x30, 0x91, 0x46, 0x12, 0x28, 0x00, \
  0xa2, 0x28, 0x34, 0x45, 0x04, 0x00

#define kMaskRandom38_16 \
  0x2c, 0x16, 0x05, 0x82, 0xc0, 0x00, \
  0x91, 0x40, 0xd2, 0x28, 0x18, 0x00, \
  0xc0, 0xd0, 0x38, 0x1a, 0x04, 0x00, \
  0x06, 0x82, 0xa0, 0xd0, 0x54, 0x00, \
  0xc8, 0x0c, 0x59, 0x01, 0x88, 0x00, \
  0x45, 0x61, 0x08, 0xac, 0x20, 0x00, \
  0x30, 0x91, 0x46, 0x12, 0x28, 0x00, \
  0xa2, 0x28, 0x34, 0x45, 0x04, 0x00, \
  0x8c, 0x23, 0x11, 0x84, 0x60, 0x00, \
  0x47, 0x19, 0x88, 0xe3, 0x30, 0x00, \
  0x81, 0x88, 0x70, 0x31, 0x0c, 0x00, \
  0x12, 0x86, 0x22, 0x50, 0xc4, 0x00, \
  0x58, 0x14, 0x4b, 0x02, 0x88, 0x00, \
  0x28, 0xca, 0x05, 0x19, 0x40, 0x00, \
  0x34, 0x60, 0x86, 0x8c, 0x10, 0x00, \
  0x7e, 0x75, 0xef, 0xce, 0xbc, 0x00

#define kMaskRandom38_17 \
  0x2c, 0x16, 0x05, 0x82, 0xc0, 0x00, \
  0x91, 0x40, 0xd2, 0x28, 0x18, 0x00, \
  0xc0, 0xd0, 0x38, 0x1a, 0x04, 0x00, \
  0x06, 0x82, 0xa0, 0xd0, 0x54, 0x00, \
  0xc8, 0x0c, 0x59, 0x01, 0x88, 0x00, \
  0x45, 0x61, 0x08, 0xac, 0x20, 0x00, \
  0x30, 0x91, 0x46, 0x12, 0x28, 0x00, \
  0xa2, 0x28, 0x34, 0x45, 0x04, 0x00, \
  0x4c, 0x27, 0x09, 0x84, 0xe0, 0x00, \
  0x66, 0x71, 0x8c, 0xce, 0x30, 0x00, \
  0x91, 0x40, 0xf2, 0x28, 0x1c, 0x00, \
  0x42, 0x90, 0xa8, 0x52, 0x14, 0x00, \
  0xa4, 0x29, 0x54, 0x85, 0x28, 0x00, \
  0x13, 0x5a, 0x02, 0x6b, 0x40, 0x00, \
  0x30, 0x93, 0x46, 0x12, 0x68, 0x00, \
  0x88, 0xac, 0x31, 0x15, 0x84, 0x00, \
  0x09, 0x0c, 0xc1, 0x21, 0x98, 0x00

#define kMaskRandom38_18 \
  0x4c, 0x27, 0x09, 0x84, 0xe0, 0x00, \
  0x66, 0x71, 0x8c, 0xce, 0x30, 0x00, \
  0x91, 0x40, 0xf2, 0x28, 0x1c, 0x00, \
  0x42, 0x90, 0xa8, 0x52, 0x14, 0x00, \
  0xa4, 0x29, 0x54, 0x85, 0x28, 0x00, \
  0x13, 0x5a, 0x02, 0x6b, 0x40, 0x00, \
  0x30, 0x93, 0x46, 0x12, 0x68, 0x00, \
  0x88, 0xac, 0x31, 0x15, 0x84, 0x00, \
  0x09, 0x0c, 0xc1, 0x21, 0x98, 0x00, \
  0x2c, 0x16, 0x05, 0x82, 0xc0, 0x00, \
  0x91, 0x40, 0xd2, 0x28, 0x18, 0x00, \
  0xc0, 0xd0, 0x38, 0x1a, 0x04, 0x00, \
  0x06, 0x82, 0xa0, 0xd0, 0x54, 0x00, \
  0xc8, 0x0c, 0x59, 0x01, 0x88, 0x00, \
  0x45, 0x61, 0x08, 0xac, 0x20, 0x00, \
  0x30, 0x91, 0x46, 0x12, 0x28, 0x00, \
  0xa2, 0x28, 0x34, 0x45, 0x04, 0x00, \
  0x51, 0x97, 0x2a, 0x32, 0xe4, 0x00

#define kMaskRandom38_19 \
  0x4c, 0x27, 0x09, 0x84, 0xe0, 0x00, \
  0x66, 0x71, 0x8c, 0xce, 0x30, 0x00, \
  0x91, 0x40, 0xf2, 0x28, 0x1c, 0x00, \
  0x42, 0x90, 0xa8, 0x52, 0x14, 0x00, \
  0xa4, 0x29, 0x54, 0x85, 0x28, 0x00, \
  0x13, 0x5a, 0x02, 0x6b, 0x40, 0x00, \
  0x30, 0x93, 0x46, 0x12, 0x68, 0x00, \
  0x88, 0xac, 0x31, 0x15, 0x84, 0x00, \
  0x09, 0x0c, 0xc1, 0x21, 0x98, 0x00, \
  0x4c, 0x26, 0x09, 0x84, 0xc0, 0x00, \
  0x66, 0x28, 0x8c, 0xc5, 0x10, 0x00, \
  0x91, 0x50, 0x32, 0x2a, 0x04, 0x00, \
  0x42, 0x82, 0x68, 0x50, 0x4c, 0x00, \
  0xa4, 0x01, 0xd4, 0x80, 0x38, 0x00, \
  0x13, 0x43, 0x02, 0x68, 0x60, 0x00, \
  0x30, 0x94, 0x86, 0x12, 0x90, 0x00, \
  0x88, 0xa1, 0x31, 0x14, 0x24, 0x00, \
  0x09, 0x4c, 0x01, 0x29, 0x80, 0x00, \
  0xcd, 0x98, 0x59, 0xb3, 0x08, 0x00

#define kMaskRandom38_2 \
  0xce, 0x77, 0x19, 0xce, 0xe0, 0x00, \
  0x39, 0xcc, 0xe7, 0x39, 0x9c, 0x00

#define kMaskRandom38_20 \
  0x8c, 0xe3, 0x11, 0x9c, 0x60, 0x00, \
  0x27, 0x11, 0xc4, 0xe2, 0x38, 0x00, \
  0x32, 0x8d, 0x26, 0x51, 0xa4, 0x00, \
  0x61, 0x92, 0x6c, 0x32, 0x4c, 0x00, \
  0x5c, 0x38, 0x8b, 0x87, 0x10, 0x00, \
  0xcc, 0x75, 0x19, 0x8e, 0xa0, 0x00, \
  0x2b, 0x19, 0xc5, 0x63, 0x38, 0x00, \
  0x32, 0xd2, 0x66, 0x5a, 0x4c, 0x00, \
  0x25, 0x8e, 0xa4, 0xb1, 0xd4, 0x00, \
  0x50, 0x88, 0xca, 0x11, 0x18, 0x00, \
  0x44, 0xa7, 0x08, 0x94, 0xe0, 0x00, \
  0x66, 0x70, 0x8c, 0xce, 0x10, 0x00, \
  0x12, 0xc0, 0xe2, 0x58, 0x1c, 0x00, \
  0xc3, 0x10, 0xb8, 0x62, 0x14, 0x00, \
  0x8c, 0x29, 0x51, 0x85, 0x28, 0x00, \
  0x11, 0x5b, 0x02, 0x2b, 0x60, 0x00, \
  0x21, 0x93, 0x44, 0x32, 0x68, 0x00, \
  0xa2, 0x2c, 0x14, 0x45, 0x80, 0x00, \
  0x18, 0x0c, 0xe3, 0x01, 0x9c, 0x00, \
  0xe6, 0xbc, 0x88, 0xe3, 0x78, 0x00

#define kMaskRandom38_21 \
  0x8c, 0xe3, 0x11, 0x9c, 0x60, 0x00, \
  0x27, 0x11, 0xc4, 0xe2, 0x38, 0x00, \
  0x32, 0x8d, 0x26, 0x51, 0xa4, 0x00, \
  0x61, 0x92, 0x6c, 0x32, 0x4c, 0x00, \
  0x5c, 0x38, 0x8b, 0x87, 0x10, 0x00, \
  0xcc, 0x75, 0x19, 0x8e, 0xa0, 0x00, \
  0x2b, 0x19, 0xc5, 0x63, 0x38, 0x00, \
  0x32, 0xd2, 0x66, 0x5a, 0x4c, 0x00, \
  0x25, 0x8e, 0xa4, 0xb1, 0xd4, 0x00, \
  0x50, 0x88, 0xca, 0x11, 0x18, 0x00, \
  0x8c, 0xe3, 0x11, 0x9c, 0x60, 0x00, \
  0x27, 0x11, 0xc4, 0xe2, 0x38, 0x00, \
  0x32, 0x8d, 0x26, 0x51, 0xa4, 0x00, \
  0x61, 0x92, 0x6c, 0x32, 0x4c, 0x00, \
  0x5c, 0x38, 0x8b, 0x87, 0x10, 0x00, \
  0x84, 0x87, 0x10, 0x90, 0xe0, 0x00, \
  0x27, 0x19, 0x84, 0xe3, 0x30, 0x00, \
  0x51, 0x88, 0x6a, 0x31, 0x0c, 0x00, \
  0x1a, 0x22, 0xa3, 0x44, 0x54, 0x00, \
  0x68, 0x44, 0x4d, 0x08, 0x88, 0x00, \
  0x89, 0x70, 0x11, 0x2e, 0x00, 0x00

#define kMaskRandom38_22 \
  0x8c, 0xe3, 0x11, 0x9c, 0x60, 0x00, \
  0x27, 0x11, 0xc4, 0xe2, 0x38, 0x00, \
  0x32, 0x8d, 0x26, 0x51, 0xa4, 0x00, \
  0x61, 0x92, 0x6c, 0x32, 0x4c, 0x00, \
  0x5c, 0x38, 0x8b, 0x87, 0x10, 0x00, \
  0x84, 0x87, 0x10, 0x90, 0xe0, 0x00, \
  0x27, 0x19, 0x84, 0xe3, 0x30, 0x00, \
  0x51, 0x88, 0x6a, 0x31, 0x0c, 0x00, \
  0x1a, 0x22, 0xa3, 0x44, 0x54, 0x00, \
  0x68, 0x44, 0x4d, 0x08, 0x88, 0x00, \
  0x89, 0x70, 0x11, 0x2e, 0x00, 0x00, \
  0x8c, 0xe3, 0x11, 0x9c, 0x60, 0x00, \
  0x27, 0x11, 0xc4, 0xe2, 0x38, 0x00, \
  0x32, 0x8d, 0x26, 0x51, 0xa4, 0x00, \
  0x61, 0x92, 0x6c, 0x32, 0x4c, 0x00, \
  0x5c, 0x38, 0x8b, 0x87, 0x10, 0x00, \
  0xcc, 0x75, 0x19, 0x8e, 0xa0, 0x00, \
  0x2b, 0x19, 0xc5, 0x63, 0x38, 0x00, \
  0x32, 0xd2, 0x66, 0x5a, 0x4c, 0x00, \
  0x25, 0x8e, 0xa4, 0xb1, 0xd4, 0x00, \
  0x50, 0x88, 0xca, 0x11, 0x18, 0x00, \
  0x0c, 0x3c, 0x48, 0x3d, 0x58, 0x00

#define kMaskRandom38_23 \
  0x8c, 0xe3, 0x11, 0x9c, 0x60, 0x00, \
  0x27, 0x11, 0xc4, 0xe2, 0x38, 0x00, \
  0x32, 0x8d, 0x26, 0x51, 0xa4, 0x00, \
  0x61, 0x92, 0x6c, 0x32, 0x4c, 0x00, \
  0x5c, 0x38, 0x8b, 0x87, 0x10, 0x00, \
  0x84, 0x87, 0x10, 0x90, 0xe0, 0x00, \
  0x27, 0x19, 0x84, 0xe3, 0x30, 0x00, \
  0x51, 0x88, 0x6a, 0x31, 0x0c, 0x00, \
  0x1a, 0x22, 0xa3, 0x44, 0x54, 0x00, \
  0x68, 0x44, 0x4d, 0x08, 0x88, 0x00, \
  0x89, 0x70, 0x11, 0x2e, 0x00, 0x00, \
  0x84, 0x87, 0x10, 0x90, 0xe0, 0x00, \
  0x27, 0x19, 0x84, 0xe3, 0x30, 0x00, \
  0x51, 0x88, 0x6a, 0x31, 0x0c, 0x00, \
  0x1a, 0x22, 0xa3, 0x44, 0x54, 0x00, \
  0x68, 0x44, 0x4d, 0x08, 0x88, 0x00, \
  0x89, 0x70, 0x11, 0x2e, 0x00, 0x00, \
  0x8c, 0xe3, 0x11, 0x9c, 0x60, 0x00, \
  0x27, 0x11, 0xc4, 0xe2, 0x38, 0x00, \
  0x32, 0x8d, 0x26, 0x51, 0xa4, 0x00, \
  0x61, 0x92, 0x6c, 0x32, 0x4c, 0x00, \
  0x5c, 0x38, 0x8b, 0x87, 0x10, 0x00, \
  0x90, 0xc8, 0x92, 0x19, 0x10, 0x00

#define kMaskRandom38_24 \
  0x84, 0x87, 0x10, 0x90, 0xe0, 0x00, \
  0x27, 0x19, 0x84, 0xe3, 0x30, 0x00, \
  0x51, 0x88, 0x6a, 0x31, 0x0c, 0x00, \
  0x1a, 0x22, 0xa3, 0x44, 0x54, 0x00, \
  0x68, 0x44, 0x4d, 0x08, 0x88, 0x00, \
  0x89, 0x70, 0x11, 0x2e, 0x00, 0x00, \
  0x8c, 0xe3, 0x11, 0x9c, 0x60, 0x00, \
  0x27, 0x11, 0xc4, 0xe2, 0x38, 0x00, \
  0x32, 0x8d, 0x26, 0x51, 0xa4, 0x00, \
  0x61, 0x92, 0x6c, 0x32, 0x4c, 0x00, \
  0x5c, 0x38, 0x8b, 0x87, 0x10, 0x00, \
  0x90, 0xc8, 0x92, 0x19, 0x10, 0x00, \
  0x8c, 0xe3, 0x11, 0x9c, 0x60, 0x00, \
  0x27, 0x11, 0xc4, 0xe2, 0x38, 0x00, \
  0x32, 0x8d, 0x26, 0x51, 0xa4, 0x00, \
  0x61, 0x92, 0x6c, 0x32, 0x4c, 0x00, \
  0x5c, 0x38, 0x8b, 0x87, 0x10, 0x00, \
  0x84, 0x87, 0x10, 0x90, 0xe0, 0x00, \
  0x27, 0x19, 0x84, 0xe3, 0x30, 0x00, \
  0x51, 0x88, 0x6a, 0x31, 0x0c, 0x00, \
  0x1a, 0x22, 0xa3, 0x44, 0x54, 0x00, \
  0x68, 0x44, 0x4d, 0x08, 0x88, 0x00, \
  0x89, 0x70, 0x11, 0x2e, 0x00, 0x00, \
  0x93, 0xc8, 0xb3, 0xbe, 0x5c, 0x00

#define kMaskRandom38_25 \
  0x84, 0x87, 0x10, 0x90, 0xe0, 0x00, \
  0x27, 0x19, 0x84, 0xe3, 0x30, 0x00, \
  0x51, 0x88, 0x6a, 0x31, 0x0c, 0x00, \
  0x1a, 0x22, 0xa3, 0x44, 0x54, 0x00, \
  0x68, 0x44, 0x4d, 0x08, 0x88, 0x00, \
  0x89, 0x70, 0x11, 0x2e, 0x00, 0x00, \
  0x8c, 0xe3, 0x11, 0x9c, 0x60, 0x00, \
  0x27, 0x11, 0xc4, 0xe2, 0x38, 0x00, \
  0x32, 0x8d, 0x26, 0x51, 0xa4, 0x00, \
  0x61, 0x92, 0x6c, 0x32, 0x4c, 0x00, \
  0x5c, 0x38, 0x8b, 0x87, 0x10, 0x00, \
  0x90, 0xc8, 0x92, 0x19, 0x10, 0x00, \
  0x84, 0x87, 0x10, 0x90, 0xe0, 0x00, \
  0x27, 0x19, 0x84, 0xe3, 0x30, 0x00, \
  0x51, 0x88, 0x6a, 0x31, 0x0c, 0x00, \
  0x1a, 0x22, 0xa3, 0x44, 0x54, 0x00, \
  0x68, 0x44, 0x4d, 0x08, 0x88, 0x00, \
  0x89, 0x70, 0x11, 0x2e, 0x00, 0x00, \
  0x8c, 0x23, 0x11, 0x84, 0x60, 0x00, \
  0x47, 0x19, 0x88, 0xe3, 0x30, 0x00, \
  0x81, 0x88, 0x70, 0x31, 0x0c, 0x00, \
  0x12, 0x86, 0x22, 0x50, 0xc4, 0x00, \
  0x58, 0x14, 0x4b, 0x02, 0x88, 0x00, \
  0x28, 0xca, 0x05, 0x19, 0x40, 0x00, \
  0x34, 0x60, 0x86, 0x8c, 0x10, 0x00

#define kMaskRandom38_26 \
  0x84, 0x87, 0x10, 0x90, 0xe0, 0x00, \
  0x27, 0x19, 0x84, 0xe3, 0x30, 0x00, \
  0x51, 0x88, 0x6a, 0x31, 0x0c, 0x00, \
  0x1a, 0x22, 0xa3, 0x44, 0x54, 0x00, \
  0x68, 0x44, 0x4d, 0x08, 0x88, 0x00, \
  0x89, 0x70, 0x11, 0x2e, 0x00, 0x00, \
  0x8c, 0x23, 0x11, 0x84, 0x60, 0x00, \
  0x47, 0x19, 0x88, 0xe3, 0x30, 0x00, \
  0x81, 0x88, 0x70, 0x31, 0x0c, 0x00, \
  0x12, 0x86, 0x22, 0x50, 0xc4, 0x00, \
  0x58, 0x14, 0x4b, 0x02, 0x88, 0x00, \
  0x28, 0xca, 0x05, 0x19, 0x40, 0x00, \
  0x34, 0x60, 0x86, 0x8c, 0x10, 0x00, \
  0x84, 0x87, 0x10, 0x90, 0xe0, 0x00, \
  0x27, 0x19, 0x84, 0xe3, 0x30, 0x00, \
  0x51, 0x88, 0x6a, 0x31, 0x0c, 0x00, \
  0x1a, 0x22, 0xa3, 0x44, 0x54, 0x00, \
  0x68, 0x44, 0x4d, 0x08, 0x88, 0x00, \
  0x89, 0x70, 0x11, 0x2e, 0x00, 0x00, \
  0x8c, 0xe3, 0x11, 0x9c, 0x60, 0x00, \
  0x27, 0x11, 0xc4, 0xe2, 0x38, 0x00, \
  0x32, 0x8d, 0x26, 0x51, 0xa4, 0x00, \
  0x61, 0x92, 0x6c, 0x32, 0x4c, 0x00, \
  0x5c, 0x38, 0x8b, 0x87, 0x10, 0x00, \
  0x90, 0xc8, 0x92, 0x19, 0x10, 0x00, \
  0x4b, 0xab, 0xfc, 0xe6, 0xe8, 0x00

#define kMaskRandom38_27 \
  0x84, 0x87, 0x10, 0x90, 0xe0, 0x00, \
  0x27, 0x19, 0x84, 0xe3, 0x30, 0x00, \
  0x51, 0x88, 0x6a, 0x31, 0x0c, 0x00, \
  0x1a, 0x22, 0xa3, 0x44, 0x54, 0x00, \
  0x68, 0x44, 0x4d, 0x08, 0x88, 0x00, \
  0x89, 0x70, 0x11, 0x2e, 0x00, 0x00, \
  0x8c, 0x23, 0x11, 0x84, 0x60, 0x00, \
  0x47, 0x19, 0x88, 0xe3, 0x30, 0x00, \
  0x81, 0x88, 0x70, 0x31, 0x0c, 0x00, \
  0x12, 0x86, 0x22, 0x50, 0xc4, 0x00, \
  0x58, 0x14, 0x4b, 0x02, 0x88, 0x00, \
  0x28, 0xca, 0x05, 0x19, 0x40, 0x00, \
  0x34, 0x60, 0x86, 0x8c, 0x10, 0x00, \
  0x8c, 0x23, 0x11, 0x84, 0x60, 0x00, \
  0x47, 0x19, 0x88, 0xe3, 0x30, 0x00, \
  0x81, 0x88, 0x70, 0x31, 0x0c, 0x00, \
  0x12, 0x86, 0x22, 0x50, 0xc4, 0x00, \
  0x58, 0x14, 0x4b, 0x02, 0x88, 0x00, \
  0x28, 0xca, 0x05, 0x19, 0x40, 0x00, \
  0x34, 0x60, 0x86, 0x8c, 0x10, 0x00, \
  0x84, 0x87, 0x10, 0x90, 0xe0, 0x00, \
  0x27, 0x19, 0x84, 0xe3, 0x30, 0x00, \
  0x51, 0x88, 0x6a, 0x31, 0x0c, 0x00, \
  0x1a, 0x22, 0xa3, 0x44, 0x54, 0x00, \
  0x68, 0x44, 0x4d, 0x08, 0x88, 0x00, \
  0x89, 0x70, 0x11, 0x2e, 0x00, 0x00, \
  0x6e, 0x27, 0x6d, 0xc4, 0xec, 0x00

#define kMaskRandom38_28 \
  0x8c, 0x23, 0x11, 0x84, 0x60, 0x00, \
  0x47, 0x19, 0x88, 0xe3, 0x30, 0x00, \
  0x81, 0x88, 0x70, 0x31, 0x0c, 0x00, \
  0x12, 0x86, 0x22, 0x50, 0xc4, 0x00, \
  0x58, 0x14, 0x4b, 0x02, 0x88, 0x00, \
  0x28, 0xca, 0x05, 0x19, 0x40, 0x00, \
  0x34, 0x60, 0x86, 0x8c, 0x10, 0x00, \
  0x84, 0x87, 0x10, 0x90, 0xe0, 0x00, \
  0x27, 0x19, 0x84, 0xe3, 0x30, 0x00, \
  0x51, 0x88, 0x6a, 0x31, 0x0c, 0x00, \
  0x1a, 0x22, 0xa3, 0x44, 0x54, 0x00, \
  0x68, 0x44, 0x4d, 0x08, 0x88, 0x00, \
  0x89, 0x70, 0x11, 0x2e, 0x00, 0x00, \
  0x6e, 0x27, 0x6d, 0xc4, 0xec, 0x00, \
  0x84, 0x87, 0x10, 0x90, 0xe0, 0x00, \
  0x27, 0x19, 0x84, 0xe3, 0x30, 0x00, \
  0x51, 0x88, 0x6a, 0x31, 0x0c, 0x00, \
  0x1a, 0x22, 0xa3, 0x44, 0x54, 0x00, \
  0x68, 0x44, 0x4d, 0x08, 0x88, 0x00, \
  0x89, 0x70, 0x11, 0x2e, 0x00, 0x00, \
  0x8c, 0x23, 0x11, 0x84, 0x60, 0x00, \
  0x47, 0x19, 0x88, 0xe3, 0x30, 0x00, \
  0x81, 0x88, 0x70, 0x31, 0x0c, 0x00, \
  0x12, 0x86, 0x22, 0x50, 0xc4, 0x00, \
  0x58, 0x14, 0x4b, 0x02, 0x88, 0x00, \
  0x28, 0xca, 0x05, 0x19, 0x40, 0x00, \
  0x34, 0x60, 0x86, 0x8c, 0x10, 0x00, \
  0x70, 0x1b, 0x5b, 0x2c, 0x0c, 0x00

#define kMaskRandom38_29 \
  0x8c, 0x23, 0x11, 0x84, 0x60, 0x00, \
  0x47, 0x19, 0x88, 0xe3, 0x30, 0x00, \
  0x81, 0x88, 0x70, 0x31, 0x0c, 0x00, \
  0x12, 0x86, 0x22, 0x50, 0xc4, 0x00, \
  0x58, 0x14, 0x4b, 0x02, 0x88, 0x00, \
  0x28, 0xca, 0x05, 0x19, 0x40, 0x00, \
  0x34, 0x60, 0x86, 0x8c, 0x10, 0x00, \
  0x84, 0x87, 0x10, 0x90, 0xe0, 0x00, \
  0x27, 0x19, 0x84, 0xe3, 0x30, 0x00, \
  0x51, 0x88, 0x6a, 0x31, 0x0c, 0x00, \
  0x1a, 0x22, 0xa3, 0x44, 0x54, 0x00, \
  0x68, 0x44, 0x4d, 0x08, 0x88, 0x00, \
  0x89, 0x70, 0x11, 0x2e, 0x00, 0x00, \
  0x6e, 0x27, 0x6d, 0xc4, 0xec, 0x00, \
  0x8c, 0x23, 0x11, 0x84, 0x60, 0x00, \
  0x47, 0x19, 0x88, 0xe3, 0x30, 0x00, \
  0x81, 0x88, 0x70, 0x31, 0x0c, 0x00, \
  0x12, 0x86, 0x22, 0x50, 0xc4, 0x00, \
  0x58, 0x14, 0x4b, 0x02, 0x88, 0x00, \
  0x28, 0xca, 0x05, 0x19, 0x40, 0x00, \
  0x34, 0x60, 0x86, 0x8c, 0x10, 0x00, \
  0x2c, 0x16, 0x05, 0x82, 0xc0, 0x00, \
  0x91, 0x40, 0xd2, 0x28, 0x18, 0x00, \
  0xc0, 0xd0, 0x38, 0x1a, 0x04, 0x00, \
  0x06, 0x82, 0xa0, 0xd0, 0x54, 0x00, \
  0xc8, 0x0c, 0x59, 0x01, 0x88, 0x00, \
  0x45, 0x61, 0x08, 0xac, 0x20, 0x00, \
  0x30, 0x91, 0x46, 0x12, 0x28, 0x00, \
  0xa2, 0x28, 0x34, 0x45, 0x04, 0x00

#define kMaskRandom38_3 \
  0xcc, 0x67, 0x19, 0x8c, 0xe0, 0x00, \
  0x27, 0x2c, 0xc4, 0xe5, 0x98, 0x00, \
  0x92, 0xd2, 0x72, 0x5a, 0x4c, 0x00

#define kMaskRandom38_30 \
  0x8c, 0x23, 0x11, 0x84, 0x60, 0x00, \
  0x47, 0x19, 0x88, 0xe3, 0x30, 0x00, \
  0x81, 0x88, 0x70, 0x31, 0x0c, 0x00, \
  0x12, 0x86, 0x22, 0x50, 0xc4, 0x00, \
  0x58, 0x14, 0x4b, 0x02, 0x88, 0x00, \
  0x28, 0xca, 0x05, 0x19, 0x40, 0x00, \
  0x34, 0x60, 0x86, 0x8c, 0x10, 0x00, \
  0x2c, 0x16, 0x05, 0x82, 0xc0, 0x00, \
  0x91, 0x40, 0xd2, 0x28, 0x18, 0x00, \
  0xc0, 0xd0, 0x38, 0x1a, 0x04, 0x00, \
  0x06, 0x82, 0xa0, 0xd0, 0x54, 0x00, \
  0xc8, 0x0c, 0x59, 0x01, 0x88, 0x00, \
  0x45, 0x61, 0x08, 0xac, 0x20, 0x00, \
  0x30, 0x91, 0x46, 0x12, 0x28, 0x00, \
  0xa2, 0x28, 0x34, 0x45, 0x04, 0x00, \
  0x8c, 0x23, 0x11, 0x84, 0x60, 0x00, \
  0x47, 0x19, 0x88, 0xe3, 0x30, 0x00, \
  0x81, 0x88, 0x70, 0x31, 0x0c, 0x00, \
  0x12, 0x86, 0x22, 0x50, 0xc4, 0x00, \
  0x58, 0x14, 0x4b, 0x02, 0x88, 0x00, \
  0x28, 0xca, 0x05, 0x19, 0x40, 0x00, \
  0x34, 0x60, 0x86, 0x8c, 0x10, 0x00, \
  0x84, 0x87, 0x10, 0x90, 0xe0, 0x00, \
  0x27, 0x19, 0x84, 0xe3, 0x30, 0x00, \
  0x51, 0x88, 0x6a, 0x31, 0x0c, 0x00, \
  0x1a, 0x22, 0xa3, 0x44, 0x54, 0x00, \
  0x68, 0x44, 0x4d, 0x08, 0x88, 0x00, \
  0x89, 0x70, 0x11, 0x2e, 0x00, 0x00, \
  0x6e, 0x27, 0x6d, 0xc4, 0xec, 0x00, \
  0x5b, 0x16, 0xdf, 0xb8, 0xd0, 0x00

#define kMaskRandom38_31 \
  0x8c, 0x23, 0x11, 0x84, 0x60, 0x00, \
  0x47, 0x19, 0x88, 0xe3, 0x30, 0x00, \
  0x81, 0x88, 0x70, 0x31, 0x0c, 0x00, \
  0x12, 0x86, 0x22, 0x50, 0xc4, 0x00, \
  0x58, 0x14, 0x4b, 0x02, 0x88, 0x00, \
  0x28, 0xca, 0x05, 0x19, 0x40, 0x00, \
  0x34, 0x60, 0x86, 0x8c, 0x10, 0x00, \
  0x2c, 0x16, 0x05, 0x82, 0xc0, 0x00, \
  0x91, 0x40, 0xd2, 0x28, 0x18, 0x00, \
  0xc0, 0xd0, 0x38, 0x1a, 0x04, 0x00, \
  0x06, 0x82, 0xa0, 0xd0, 0x54, 0x00, \
  0xc8, 0x0c, 0x59, 0x01, 0x88, 0x00, \
  0x45, 0x61, 0x08, 0xac, 0x20, 0x00, \
  0x30, 0x91, 0x46, 0x12, 0x28, 0x00, \
  0xa2, 0x28, 0x34, 0x45, 0x04, 0x00, \
  0x2c, 0x16, 0x05, 0x82, 0xc0, 0x00, \
  0x91, 0x40, 0xd2, 0x28, 0x18, 0x00, \
  0xc0, 0xd0, 0x38, 0x1a, 0x04, 0x00, \
  0x06, 0x82, 0xa0, 0xd0, 0x54, 0x00, \
  0xc8, 0x0c, 0x59, 0x01, 0x88, 0x00, \
  0x45, 0x61, 0x08, 0xac, 0x20, 0x00, \
  0x30, 0x91, 0x46, 0x12, 0x28, 0x00, \
  0xa2, 0x28, 0x34, 0x45, 0x04, 0x00, \
  0x8c, 0x23, 0x11, 0x84, 0x60, 0x00, \
  0x47, 0x19, 0x88, 0xe3, 0x30, 0x00, \
  0x81, 0x88, 0x70, 0x31, 0x0c, 0x00, \
  0x12, 0x86, 0x22, 0x50, 0xc4, 0x00, \
  0x58, 0x14, 0x4b, 0x02, 0x88, 0x00, \
  0x28, 0xca, 0x05, 0x19, 0x40, 0x00, \
  0x34, 0x60, 0x86, 0x8c, 0x10, 0x00, \
  0x7e, 0x75, 0xef, 0xce, 0xbc, 0x00

#define kMaskRandom38_32 \
  0x2c, 0x16, 0x05, 0x82, 0xc0, 0x00, \
  0x91, 0x40, 0xd2, 0x28, 0x18, 0x00, \
  0xc0, 0xd0, 0x38, 0x1a, 0x04, 0x00, \
  0x06, 0x82, 0xa0, 0xd0, 0x54, 0x00, \
  0xc8, 0x0c, 0x59, 0x01, 0x88, 0x00, \
  0x45, 0x61, 0x08, 0xac, 0x20, 0x00, \
  0x30, 0x91, 0x46, 0x12, 0x28, 0x00, \
  0xa2, 0x28, 0x34, 0x45, 0x04, 0x00, \
  0x8c, 0x23, 0x11, 0x84, 0x60, 0x00, \
  0x47, 0x19, 0x88, 0xe3, 0x30, 0x00, \
  0x81, 0x88, 0x70, 0x31, 0x0c, 0x00, \
  0x12, 0x86, 0x22, 0x50, 0xc4, 0x00, \
  0x58, 0x14, 0x4b, 0x02, 0x88, 0x00, \
  0x28, 0xca, 0x05, 0x19, 0x40, 0x00, \
  0x34, 0x60, 0x86, 0x8c, 0x10, 0x00, \
  0x7e, 0x75, 0xef, 0xce, 0xbc, 0x00, \
  0x8c, 0x23, 0x11, 0x84, 0x60, 0x00, \
  0x47, 0x19, 0x88, 0xe3, 0x30, 0x00, \
  0x81, 0x88, 0x70, 0x31, 0x0c, 0x00, \
  0x12, 0x86, 0x22, 0x50, 0xc4, 0x00, \
  0x58, 0x14, 0x4b, 0x02, 0x88, 0x00, \
  0x28, 0xca, 0x05, 0x19, 0x40, 0x00, \
  0x34, 0x60, 0x86, 0x8c, 0x10, 0x00, \
  0x2c, 0x16, 0x05, 0x82, 0xc0, 0x00, \
  0x91, 0x40, 0xd2, 0x28, 0x18, 0x00, \
  0xc0, 0xd0, 0x38, 0x1a, 0x04, 0x00, \
  0x06, 0x82, 0xa0, 0xd0, 0x54, 0x00, \
  0xc8, 0x0c, 0x59, 0x01, 0x88, 0x00, \
  0x45, 0x61, 0x08, 0xac, 0x20, 0x00, \
  0x30, 0x91, 0x46, 0x12, 0x28, 0x00, \
  0xa2, 0x28, 0x34, 0x45, 0x04, 0x00, \
  0x33, 0x10, 0x02, 0x4e, 0x54, 0x00

#define kMaskRandom38_33 \
  0x2c, 0x16, 0x05, 0x82, 0xc0, 0x00, \
  0x91, 0x40, 0xd2, 0x28, 0x18, 0x00, \
  0xc0, 0xd0, 0x38, 0x1a, 0x04, 0x00, \
  0x06, 0x82, 0xa0, 0xd0, 0x54, 0x00, \
  0xc8, 0x0c, 0x59, 0x01, 0x88, 0x00, \
  0x45, 0x61, 0x08, 0xac, 0x20, 0x00, \
  0x30, 0x91, 0x46, 0x12, 0x28, 0x00, \
  0xa2, 0x28, 0x34, 0x45, 0x04, 0x00, \
  0x8c, 0x23, 0x11, 0x84, 0x60, 0x00, \
  0x47, 0x19, 0x88, 0xe3, 0x30, 0x00, \
  0x81, 0x88, 0x70, 0x31, 0x0c, 0x00, \
  0x12, 0x86, 0x22, 0x50, 0xc4, 0x00, \
  0x58, 0x14, 0x4b, 0x02, 0x88, 0x00, \
  0x28, 0xca, 0x05, 0x19, 0x40, 0x00, \
  0x34, 0x60, 0x86, 0x8c, 0x10, 0x00, \
  0x7e, 0x75, 0xef, 0xce, 0xbc, 0x00, \
  0x2c, 0x16, 0x05, 0x82, 0xc0, 0x00, \
  0x91, 0x40, 0xd2, 0x28, 0x18, 0x00, \
  0xc0, 0xd0, 0x38, 0x1a, 0x04, 0x00, \
  0x06, 0x82, 0xa0, 0xd0, 0x54, 0x00, \
  0xc8, 0x0c, 0x59, 0x01, 0x88, 0x00, \
  0x45, 0x61, 0x08, 0xac, 0x20, 0x00, \
  0x30, 0x91, 0x46, 0x12, 0x28, 0x00, \
  0xa2, 0x28, 0x34, 0x45, 0x04, 0x00, \
  0x4c, 0x27, 0x09, 0x84, 0xe0, 0x00, \
  0x66, 0x71, 0x8c, 0xce, 0x30, 0x00, \
  0x91, 0x40, 0xf2, 0x28, 0x1c, 0x00, \
  0x42, 0x90, 0xa8, 0x52, 0x14, 0x00, \
  0xa4, 0x29, 0x54, 0x85, 0x28, 0x00, \
  0x13, 0x5a, 0x02, 0x6b, 0x40, 0x00, \
  0x30, 0x93, 0x46, 0x12, 0x68, 0x00, \
  0x88, 0xac, 0x31, 0x15, 0x84, 0x00, \
  0x09, 0x0c, 0xc1, 0x21, 0x98, 0x00

#define kMaskRandom38_34 \
  0x2c, 0x16, 0x05, 0x82, 0xc0, 0x00, \
  0x91, 0x40, 0xd2, 0x28, 0x18, 0x00, \
  0xc0, 0xd0, 0x38, 0x1a, 0x04, 0x00, \
  0x06, 0x82, 0xa0, 0xd0, 0x54, 0x00, \
  0xc8, 0x0c, 0x59, 0x01, 0x88, 0x00, \
  0x45, 0x61, 0x08, 0xac, 0x20, 0x00, \
  0x30, 0x91, 0x46, 0x12, 0x28, 0x00, \
  0xa2, 0x28, 0x34, 0x45, 0x04, 0x00, \
  0x4c, 0x27, 0x09, 0x84, 0xe0, 0x00, \
  0x66, 0x71, 0x8c, 0xce, 0x30, 0x00, \
  0x91, 0x40, 0xf2, 0x28, 0x1c, 0x00, \
  0x42, 0x90, 0xa8, 0x52, 0x14, 0x00, \
  0xa4, 0x29, 0x54, 0x85, 0x28, 0x00, \
  0x13, 0x5a, 0x02, 0x6b, 0x40, 0x00, \
  0x30, 0x93, 0x46, 0x12, 0x68, 0x00, \
  0x88, 0xac, 0x31, 0x15, 0x84, 0x00, \
  0x09, 0x0c, 0xc1, 0x21, 0x98, 0x00, \
  0x2c, 0x16, 0x05, 0x82, 0xc0, 0x00, \
  0x91, 0x40, 0xd2, 0x28, 0x18, 0x00, \
  0xc0, 0xd0, 0x38, 0x1a, 0x04, 0x00, \
  0x06, 0x82, 0xa0, 0xd0, 0x54, 0x00, \
  0xc8, 0x0c, 0x59, 0x01, 0x88, 0x00, \
  0x45, 0x61, 0x08, 0xac, 0x20, 0x00, \
  0x30, 0x91, 0x46, 0x12, 0x28, 0x00, \
  0xa2, 0x28, 0x34, 0x45, 0x04, 0x00, \
  0x8c, 0x23, 0x11, 0x84, 0x60, 0x00, \
  0x47, 0x19, 0x88, 0xe3, 0x30, 0x00, \
  0x81, 0x88, 0x70, 0x31, 0x0c, 0x00, \
  0x12, 0x86, 0x22, 0x50, 0xc4, 0x00, \
  0x58, 0x14, 0x4b, 0x02, 0x88, 0x00, \
  0x28, 0xca, 0x05, 0x19, 0x40, 0x00, \
  0x34, 0x60, 0x86, 0x8c, 0x10, 0x00, \
  0x7e, 0x75, 0xef, 0xce, 0xbc, 0x00, \
  0x91, 0x48, 0xfa, 0xf0, 0xd8, 0x00

#define kMaskRandom38_35 \
  0x2c, 0x16, 0x05, 0x82, 0xc0, 0x00, \
  0x91, 0x40, 0xd2, 0x28, 0x18, 0x00, \
  0xc0, 0xd0, 0x38, 0x1a, 0x04, 0x00, \
  0x06, 0x82, 0xa0, 0xd0, 0x54, 0x00, \
  0xc8, 0x0c, 0x59, 0x01, 0x88, 0x00, \
  0x45, 0x61, 0x08, 0xac, 0x20, 0x00, \
  0x30, 0x91, 0x46, 0x12, 0x28, 0x00, \
  0xa2, 0x28, 0x34, 0x45, 0x04, 0x00, \
  0x4c, 0x27, 0x09, 0x84, 0xe0, 0x00, \
  0x66, 0x71, 0x8c, 0xce, 0x30, 0x00, \
  0x91, 0x40, 0xf2, 0x28, 0x1c, 0x00, \
  0x42, 0x90, 0xa8, 0x52, 0x14, 0x00, \
  0xa4, 0x29, 0x54, 0x85, 0x28, 0x00, \
  0x13, 0x5a, 0x02, 0x6b, 0x40, 0x00, \
  0x30, 0x93, 0x46, 0x12, 0x68, 0x00, \
  0x88, 0xac, 0x31, 0x15, 0x84, 0x00, \
  0x09, 0x0c, 0xc1, 0x21, 0x98, 0x00, \
  0x4c, 0x27, 0x09, 0x84, 0xe0, 0x00, \
  0x66, 0x71, 0x8c, 0xce, 0x30, 0x00, \
  0x91, 0x40, 0xf2, 0x28, 0x1c, 0x00, \
  0x42, 0x90, 0xa8, 0x52, 0x14, 0x00, \
  0xa4, 0x29, 0x54, 0x85, 0x28, 0x00, \
  0x13, 0x5a, 0x02, 0x6b, 0x40, 0x00, \
  0x30, 0x93, 0x46, 0x12, 0x68, 0x00, \
  0x88, 0xac, 0x31, 0x15, 0x84, 0x00, \
  0x09, 0x0c, 0xc1, 0x21, 0x98, 0x00, \
  0x2c, 0x16, 0x05, 0x82, 0xc0, 0x00, \
  0x91, 0x40, 0xd2, 0x28, 0x18, 0x00, \
  0xc0, 0xd0, 0x38, 0x1a, 0x04, 0x00, \
  0x06, 0x82, 0xa0, 0xd0, 0x54, 0x00, \
  0xc8, 0x0c, 0x59, 0x01, 0x88, 0x00, \
  0x45, 0x61, 0x08, 0xac, 0x20, 0x00, \
  0x30, 0x91, 0x46, 0x12, 0x28, 0x00, \
  0xa2, 0x28, 0x34, 0x45, 0x04, 0x00, \
  0x51, 0x97, 0x2a, 0x32, 0xe4, 0x00

#define kMaskRandom38_36 \
  0x4c, 0x27, 0x09, 0x84, 0xe0, 0x00, \
  0x66, 0x71, 0x8c, 0xce, 0x30, 0x00, \
  0x91, 0x40, 0xf2, 0x28, 0x1c, 0x00, \
  0x42, 0x90, 0xa8, 0x52, 0x14, 0x00, \
  0xa4, 0x29, 0x54, 0x85, 0x28, 0x00, \
  0x13, 0x5a, 0x02, 0x6b, 0x40, 0x00, \
  0x30, 0x93, 0x46, 0x12, 0x68, 0x00, \
  0x88, 0xac, 0x31, 0x15, 0x84, 0x00, \
  0x09, 0x0c, 0xc1, 0x21, 0x98, 0x00, \
  0x2c, 0x16, 0x05, 0x82, 0xc0, 0x00, \
  0x91, 0x40, 0xd2, 0x28, 0x18, 0x00, \
  0xc0, 0xd0, 0x38, 0x1a, 0x04, 0x00, \
  0x06, 0x82, 0xa0, 0xd0, 0x54, 0x00, \
  0xc8, 0x0c, 0x59, 0x01, 0x88, 0x00, \
  0x45, 0x61, 0x08, 0xac, 0x20, 0x00, \
  0x30, 0x91, 0x46, 0x12, 0x28, 0x00, \
  0xa2, 0x28, 0x34, 0x45, 0x04, 0x00, \
  0x51, 0x97, 0x2a, 0x32, 0xe4, 0x00, \
  0x2c, 0x16, 0x05, 0x82, 0xc0, 0x00, \
  0x91, 0x40, 0xd2, 0x28, 0x18, 0x00, \
  0xc0, 0xd0, 0x38, 0x1a, 0x04, 0x00, \
  0x06, 0x82, 0xa0, 0xd0, 0x54, 0x00, \
  0xc8, 0x0c, 0x59, 0x01, 0x88, 0x00, \
  0x45, 0x61, 0x08, 0xac, 0x20, 0x00, \
  0x30, 0x91, 0x46, 0x12, 0x28, 0x00, \
  0xa2, 0x28, 0x34, 0x45, 0x04, 0x00, \
  0x4c, 0x27, 0x09, 0x84, 0xe0, 0x00, \
  0x66, 0x71, 0x8c, 0xce, 0x30, 0x00, \
  0x91, 0x40, 0xf2, 0x28, 0x1c, 0x00, \
  0x42, 0x90, 0xa8, 0x52, 0x14, 0x00, \
  0xa4, 0x29, 0x54, 0x85, 0x28, 0x00, \
  0x13, 0x5a, 0x02, 0x6b, 0x40, 0x00, \
  0x30, 0x93, 0x46, 0x12, 0x68, 0x00, \
  0x88, 0xac, 0x31, 0x15, 0x84, 0x00, \
  0x09, 0x0c, 0xc1, 0x21, 0x98, 0x00, \
  0x80, 0x95, 0xc2, 0x68, 0x28, 0x00

#define kMaskRandom38_37 \
  0x4c, 0x27, 0x09, 0x84, 0xe0, 0x00, \
  0x66, 0x71, 0x8c, 0xce, 0x30, 0x00, \
  0x91, 0x40, 0xf2, 0x28, 0x1c, 0x00, \
  0x42, 0x90, 0xa8, 0x52, 0x14, 0x00, \
  0xa4, 0x29, 0x54, 0x85, 0x28, 0x00, \
  0x13, 0x5a, 0x02, 0x6b, 0x40, 0x00, \
  0x30, 0x93, 0x46, 0x12, 0x68, 0x00, \
  0x88, 0xac, 0x31, 0x15, 0x84, 0x00, \
  0x09, 0x0c, 0xc1, 0x21, 0x98, 0x00, \
  0x2c, 0x16, 0x05, 0x82, 0xc0, 0x00, \
  0x91, 0x40, 0xd2, 0x28, 0x18, 0x00, \
  0xc0, 0xd0, 0x38, 0x1a, 0x04, 0x00, \
  0x06, 0x82, 0xa0, 0xd0, 0x54, 0x00, \
  0xc8, 0x0c, 0x59, 0x01, 0x88, 0x00, \
  0x45, 0x61, 0x08, 0xac, 0x20, 0x00, \
  0x30, 0x91, 0x46, 0x12, 0x28, 0x00, \
  0xa2, 0x28, 0x34, 0x45, 0x04, 0x00, \
  0x51, 0x97, 0x2a, 0x32, 0xe4, 0x00, \
  0x4c, 0x27, 0x09, 0x84, 0xe0, 0x00, \
  0x66, 0x71, 0x8c, 0xce, 0x30, 0x00, \
  0x91, 0x40, 0xf2, 0x28, 0x1c, 0x00, \
  0x42, 0x90, 0xa8, 0x52, 0x14, 0x00, \
  0xa4, 0x29, 0x54, 0x85, 0x28, 0x00, \
  0x13, 0x5a, 0x02, 0x6b, 0x40, 0x00, \
  0x30, 0x93, 0x46, 0x12, 0x68, 0x00, \
  0x88, 0xac, 0x31, 0x15, 0x84, 0x00, \
  0x09, 0x0c, 0xc1, 0x21, 0x98, 0x00, \
  0x4c, 0x26, 0x09, 0x84, 0xc0, 0x00, \
  0x66, 0x28, 0x8c, 0xc5, 0x10, 0x00, \
  0x91, 0x50, 0x32, 0x2a, 0x04, 0x00, \
  0x42, 0x82, 0x68, 0x50, 0x4c, 0x00, \
  0xa4, 0x01, 0xd4, 0x80, 0x38, 0x00, \
  0x13, 0x43, 0x02, 0x68, 0x60, 0x00, \
  0x30, 0x94, 0x86, 0x12, 0x90, 0x00, \
  0x88, 0xa1, 0x31, 0x14, 0x24, 0x00, \
  0x09, 0x4c, 0x01, 0x29, 0x80, 0x00, \
  0xcd, 0x98, 0x59, 0xb3, 0x08, 0x00

#define kMaskRandom38_38 \
  0x4c, 0x27, 0x09, 0x84, 0xe0, 0x00, \
  0x66, 0x71, 0x8c, 0xce, 0x30, 0x00, \
  0x91, 0x40, 0xf2, 0x28, 0x1c, 0x00, \
  0x42, 0x90, 0xa8, 0x52, 0x14, 0x00, \
  0xa4, 0x29, 0x54, 0x85, 0x28, 0x00, \
  0x13, 0x5a, 0x02, 0x6b, 0x40, 0x00, \
  0x30, 0x93, 0x46, 0x12, 0x68, 0x00, \
  0x88, 0xac, 0x31, 0x15, 0x84, 0x00, \
  0x09, 0x0c, 0xc1, 0x21, 0x98, 0x00, \
  0x4c, 0x26, 0x09, 0x84, 0xc0, 0x00, \
  0x66, 0x28, 0x8c, 0xc5, 0x10, 0x00, \
  0x91, 0x50, 0x32, 0x2a, 0x04, 0x00, \
  0x42, 0x82, 0x68, 0x50, 0x4c, 0x00, \
  0xa4, 0x01, 0xd4, 0x80, 0x38, 0x00, \
  0x13, 0x43, 0x02, 0x68, 0x60, 0x00, \
  0x30, 0x94, 0x86, 0x12, 0x90, 0x00, \
  0x88, 0xa1, 0x31, 0x14, 0x24, 0x00, \
  0x09, 0x4c, 0x01, 0x29, 0x80, 0x00, \
  0xcd, 0x98, 0x59, 0xb3, 0x08, 0x00, \
  0x4c, 0x27, 0x09, 0x84, 0xe0, 0x00, \
  0x66, 0x71, 0x8c, 0xce, 0x30, 0x00, \
  0x91, 0x40, 0xf2, 0x28, 0x1c, 0x00, \
  0x42, 0x90, 0xa8, 0x52, 0x14, 0x00, \
  0xa4, 0x29, 0x54, 0x85, 0x28, 0x00, \
  0x13, 0x5a, 0x02, 0x6b, 0x40, 0x00, \
  0x30, 0x93, 0x46, 0x12, 0x68, 0x00, \
  0x88, 0xac, 0x31, 0x15, 0x84, 0x00, \
  0x09, 0x0c, 0xc1, 0x21, 0x98, 0x00, \
  0x2c, 0x16, 0x05, 0x82, 0xc0, 0x00, \
  0x91, 0x40, 0xd2, 0x28, 0x18, 0x00, \
  0xc0, 0xd0, 0x38, 0x1a, 0x04, 0x00, \
  0x06, 0x82, 0xa0, 0xd0, 0x54, 0x00, \
  0xc8, 0x0c, 0x59, 0x01, 0x88, 0x00, \
  0x45, 0x61, 0x08, 0xac, 0x20, 0x00, \
  0x30, 0x91, 0x46, 0x12, 0x28, 0x00, \
  0xa2, 0x28, 0x34, 0x45, 0x04, 0x00, \
  0x51, 0x97, 0x2a, 0x32, 0xe4, 0x00, \
  0x8c, 0xed, 0x11, 0x5f, 0x24, 0x00

#define kMaskRandom38_4 \
  0xec, 0x73, 0x1d, 0x8e, 0x60, 0x00, \
  0x67, 0x19, 0xcc, 0xe3, 0x38, 0x00, \
  0xb1, 0xcc, 0x76, 0x39, 0x8c, 0x00, \
  0x5a, 0x96, 0xab, 0x52, 0xd4, 0x00

#define kMaskRandom38_5 \
  0x4c, 0xe7, 0x09, 0x9c, 0xe0, 0x00, \
  0x66, 0x31, 0xcc, 0xc6, 0x38, 0x00, \
  0xa1, 0xcc, 0x74, 0x39, 0x8c, 0x00, \
  0x92, 0xa6, 0xb2, 0x54, 0xd4, 0x00, \
  0xb8, 0x99, 0x97, 0x13, 0x30, 0x00

#define kMaskRandom38_6 \
  0x4c, 0x36, 0x09, 0x86, 0xc0, 0x00, \
  0x45, 0x68, 0x88, 0xad, 0x10, 0x00, \
  0x30, 0xd0, 0x66, 0x1a, 0x0c, 0x00, \
  0x8a, 0x82, 0xb1, 0x50, 0x54, 0x00, \
  0x26, 0x0b, 0x44, 0xc1, 0x68, 0x00, \
  0x95, 0x45, 0x12, 0xa8, 0xa0, 0x00

#define kMaskRandom38_7 \
  0xc4, 0xa3, 0x18, 0x94, 0x60, 0x00, \
  0x23, 0x19, 0x84, 0x63, 0x30, 0x00, \
  0x91, 0x1c, 0x32, 0x23, 0x84, 0x00, \
  0x4a, 0x82, 0xa9, 0x50, 0x54, 0x00, \
  0x34, 0x49, 0x46, 0x89, 0x28, 0x00, \
  0x8b, 0x4a, 0x11, 0x69, 0x40, 0x00, \
  0xc8, 0x24, 0xd9, 0x04, 0x98, 0x00

#define kMaskRandom38_8 \
  0x16, 0x13, 0x82, 0xc2, 0x70, 0x00, \
  0xc2, 0x44, 0xd8, 0x48, 0x98, 0x00, \
  0x60, 0xe8, 0x2c, 0x1d, 0x04, 0x00, \
  0x85, 0x12, 0x70, 0xa2, 0x4c, 0x00, \
  0xcc, 0x21, 0x59, 0x84, 0x28, 0x00, \
  0x29, 0x63, 0x05, 0x2c, 0x60, 0x00, \
  0x11, 0x98, 0xc2, 0x33, 0x18, 0x00, \
  0xb0, 0x0c, 0x76, 0x01, 0x8c, 0x00

#define kMaskRandom38_9 \
  0x44, 0xa7, 0x08, 0x94, 0xe0, 0x00, \
  0x66, 0x70, 0x8c, 0xce, 0x10, 0x00, \
  0x12, 0xc0, 0xe2, 0x58, 0x1c, 0x00, \
  0xc3, 0x10, 0xb8, 0x62, 0x14, 0x00, \
  0x8c, 0x29, 0x51, 0x85, 0x28, 0x00, \
  0x11, 0x5b, 0x02, 0x2b, 0x60, 0x00, \
  0x21, 0x93, 0x44, 0x32, 0x68, 0x00, \
  0xa2, 0x2c, 0x14, 0x45, 0x80, 0x00, \
  0x18, 0x0c, 0xe3, 0x01, 0x9c, 0x00

#define kMaskRandom39_1 \
  0xff, 0xff, 0xff, 0xff, 0xfe, 0x00

#define kMaskRandom39_10 \
  0x8c, 0xe3, 0x09, 0x82, 0x60, 0x00, \
  0x27, 0x11, 0xca, 0x22, 0x88, 0x00, \
  0x32, 0x8d, 0x34, 0x0d, 0x02, 0x00, \
  0x61, 0x92, 0x60, 0x98, 0x26, 0x00, \
  0x5c, 0x38, 0x80, 0x70, 0x1c, 0x00, \
  0xcc, 0x75, 0x10, 0xc4, 0x30, 0x00, \
  0x2b, 0x19, 0xc5, 0x21, 0x48, 0x00, \
  0x32, 0xd2, 0x68, 0x4a, 0x12, 0x00, \
  0x25, 0x8e, 0xb3, 0x04, 0xc0, 0x00, \
  0x50, 0x88, 0xc6, 0x11, 0x84, 0x00

#define kMaskRandom39_11 \
  0x8c, 0xe3, 0x18, 0xc6, 0x30, 0x00, \
  0x27, 0x11, 0xc4, 0x71, 0x1c, 0x00, \
  0x32, 0x8d, 0x23, 0x48, 0xd2, 0x00, \
  0x61, 0x92, 0x64, 0x99, 0x26, 0x00, \
  0x5c, 0x38, 0x8e, 0x23, 0x88, 0x00, \
  0x84, 0x87, 0x01, 0xc0, 0x70, 0x00, \
  0x27, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x51, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x1a, 0x22, 0xa8, 0xaa, 0x2a, 0x00, \
  0x68, 0x44, 0x51, 0x14, 0x44, 0x00, \
  0x89, 0x70, 0x1c, 0x07, 0x00, 0x00

#define kMaskRandom39_12 \
  0x84, 0x87, 0x01, 0xc0, 0x70, 0x00, \
  0x27, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x51, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x1a, 0x22, 0xa8, 0xaa, 0x2a, 0x00, \
  0x68, 0x44, 0x51, 0x14, 0x44, 0x00, \
  0x89, 0x70, 0x1c, 0x07, 0x00, 0x00, \
  0x8c, 0xe3, 0x18, 0xc6, 0x30, 0x00, \
  0x27, 0x11, 0xc4, 0x71, 0x1c, 0x00, \
  0x32, 0x8d, 0x23, 0x48, 0xd2, 0x00, \
  0x61, 0x92, 0x64, 0x99, 0x26, 0x00, \
  0x5c, 0x38, 0x8e, 0x23, 0x88, 0x00, \
  0x90, 0xc8, 0x9e, 0xbb, 0x88, 0x00

#define kMaskRandom39_13 \
  0x84, 0x87, 0x01, 0xc0, 0x70, 0x00, \
  0x27, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x51, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x1a, 0x22, 0xa8, 0xaa, 0x2a, 0x00, \
  0x68, 0x44, 0x51, 0x14, 0x44, 0x00, \
  0x89, 0x70, 0x1c, 0x07, 0x00, 0x00, \
  0x8c, 0x23, 0x08, 0xc2, 0x30, 0x00, \
  0x47, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x81, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x12, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0x58, 0x14, 0x45, 0x11, 0x44, 0x00, \
  0x28, 0xca, 0x12, 0x84, 0xa0, 0x00, \
  0x34, 0x60, 0x98, 0x26, 0x08, 0x00

#define kMaskRandom39_14 \
  0x8c, 0x23, 0x08, 0xc2, 0x30, 0x00, \
  0x47, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x81, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x12, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0x58, 0x14, 0x45, 0x11, 0x44, 0x00, \
  0x28, 0xca, 0x12, 0x84, 0xa0, 0x00, \
  0x34, 0x60, 0x98, 0x26, 0x08, 0x00, \
  0x84, 0x87, 0x01, 0xc0, 0x70, 0x00, \
  0x27, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x51, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x1a, 0x22, 0xa8, 0xaa, 0x2a, 0x00, \
  0x68, 0x44, 0x51, 0x14, 0x44, 0x00, \
  0x89, 0x70, 0x1c, 0x07, 0x00, 0x00, \
  0x6e, 0x27, 0x6a, 0xc7, 0xc4, 0x00

#define kMaskRandom39_15 \
  0x8c, 0x23, 0x08, 0xc2, 0x30, 0x00, \
  0x47, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x81, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x12, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0x58, 0x14, 0x45, 0x11, 0x44, 0x00, \
  0x28, 0xca, 0x12, 0x84, 0xa0, 0x00, \
  0x34, 0x60, 0x98, 0x26, 0x08, 0x00, \
  0x2c, 0x16, 0x05, 0x81, 0x60, 0x00, \
  0x91, 0x40, 0xd0, 0x34, 0x0c, 0x00, \
  0xc0, 0xd0, 0x34, 0x0d, 0x02, 0x00, \
  0x06, 0x82, 0xa0, 0xa8, 0x2a, 0x00, \
  0xc8, 0x0c, 0x43, 0x10, 0xc4, 0x00, \
  0x45, 0x61, 0x18, 0x46, 0x10, 0x00, \
  0x30, 0x91, 0x44, 0x51, 0x14, 0x00, \
  0xa2, 0x28, 0x2a, 0x0a, 0x82, 0x00

#define kMaskRandom39_16 \
  0x2c, 0x16, 0x05, 0x81, 0x60, 0x00, \
  0x91, 0x40, 0xd0, 0x34, 0x0c, 0x00, \
  0xc0, 0xd0, 0x34, 0x0d, 0x02, 0x00, \
  0x06, 0x82, 0xa0, 0xa8, 0x2a, 0x00, \
  0xc8, 0x0c, 0x43, 0x10, 0xc4, 0x00, \
  0x45, 0x61, 0x18, 0x46, 0x10, 0x00, \
  0x30, 0x91, 0x44, 0x51, 0x14, 0x00, \
  0xa2, 0x28, 0x2a, 0x0a, 0x82, 0x00, \
  0x8c, 0x23, 0x08, 0xc2, 0x30, 0x00, \
  0x47, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x81, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x12, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0x58, 0x14, 0x45, 0x11, 0x44, 0x00, \
  0x28, 0xca, 0x12, 0x84, 0xa0, 0x00, \
  0x34, 0x60, 0x98, 0x26, 0x08, 0x00, \
  0x7e, 0x75, 0xe5, 0x03, 0x8c, 0x00

#define kMaskRandom39_17 \
  0x2c, 0x16, 0x05, 0x81, 0x60, 0x00, \
  0x91, 0x40, 0xd0, 0x34, 0x0c, 0x00, \
  0xc0, 0xd0, 0x34, 0x0d, 0x02, 0x00, \
  0x06, 0x82, 0xa0, 0xa8, 0x2a, 0x00, \
  0xc8, 0x0c, 0x43, 0x10, 0xc4, 0x00, \
  0x45, 0x61, 0x18, 0x46, 0x10, 0x00, \
  0x30, 0x91, 0x44, 0x51, 0x14, 0x00, \
  0xa2, 0x28, 0x2a, 0x0a, 0x82, 0x00, \
  0x4c, 0x27, 0x09, 0xc2, 0x70, 0x00, \
  0x66, 0x71, 0x9c, 0x67, 0x18, 0x00, \
  0x91, 0x40, 0xf0, 0x3c, 0x0e, 0x00, \
  0x42, 0x90, 0xa4, 0x29, 0x0a, 0x00, \
  0xa4, 0x29, 0x4a, 0x52, 0x94, 0x00, \
  0x13, 0x5a, 0x16, 0x85, 0xa0, 0x00, \
  0x30, 0x93, 0x44, 0xd1, 0x34, 0x00, \
  0x88, 0xac, 0x2b, 0x0a, 0xc2, 0x00, \
  0x09, 0x0c, 0xc3, 0x30, 0xcc, 0x00

#define kMaskRandom39_18 \
  0x4c, 0x27, 0x09, 0xc2, 0x70, 0x00, \
  0x66, 0x71, 0x9c, 0x67, 0x18, 0x00, \
  0x91, 0x40, 0xf0, 0x3c, 0x0e, 0x00, \
  0x42, 0x90, 0xa4, 0x29, 0x0a, 0x00, \
  0xa4, 0x29, 0x4a, 0x52, 0x94, 0x00, \
  0x13, 0x5a, 0x16, 0x85, 0xa0, 0x00, \
  0x30, 0x93, 0x44, 0xd1, 0x34, 0x00, \
  0x88, 0xac, 0x2b, 0x0a, 0xc2, 0x00, \
  0x09, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x2c, 0x16, 0x05, 0x81, 0x60, 0x00, \
  0x91, 0x40, 0xd0, 0x34, 0x0c, 0x00, \
  0xc0, 0xd0, 0x34, 0x0d, 0x02, 0x00, \
  0x06, 0x82, 0xa0, 0xa8, 0x2a, 0x00, \
  0xc8, 0x0c, 0x43, 0x10, 0xc4, 0x00, \
  0x45, 0x61, 0x18, 0x46, 0x10, 0x00, \
  0x30, 0x91, 0x44, 0x51, 0x14, 0x00, \
  0xa2, 0x28, 0x2a, 0x0a, 0x82, 0x00, \
  0x51, 0x97, 0x24, 0x2f, 0x7e, 0x00

#define kMaskRandom39_19 \
  0x4c, 0x27, 0x09, 0xc2, 0x70, 0x00, \
  0x66, 0x71, 0x9c, 0x67, 0x18, 0x00, \
  0x91, 0x40, 0xf0, 0x3c, 0x0e, 0x00, \
  0x42, 0x90, 0xa4, 0x29, 0x0a, 0x00, \
  0xa4, 0x29, 0x4a, 0x52, 0x94, 0x00, \
  0x13, 0x5a, 0x16, 0x85, 0xa0, 0x00, \
  0x30, 0x93, 0x44, 0xd1, 0x34, 0x00, \
  0x88, 0xac, 0x2b, 0x0a, 0xc2, 0x00, \
  0x09, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x4c, 0x26, 0x09, 0x82, 0x60, 0x00, \
  0x66, 0x28, 0x8a, 0x22, 0x88, 0x00, \
  0x91, 0x50, 0x34, 0x0d, 0x02, 0x00, \
  0x42, 0x82, 0x60, 0x98, 0x26, 0x00, \
  0xa4, 0x01, 0xc0, 0x70, 0x1c, 0x00, \
  0x13, 0x43, 0x10, 0xc4, 0x30, 0x00, \
  0x30, 0x94, 0x85, 0x21, 0x48, 0x00, \
  0x88, 0xa1, 0x28, 0x4a, 0x12, 0x00, \
  0x09, 0x4c, 0x13, 0x04, 0xc0, 0x00, \
  0xcd, 0x98, 0x46, 0x11, 0x84, 0x00

#define kMaskRandom39_2 \
  0xce, 0x77, 0x1d, 0xc7, 0x70, 0x00, \
  0x39, 0xcc, 0xf3, 0x3c, 0xce, 0x00

#define kMaskRandom39_20 \
  0x8c, 0xe3, 0x09, 0x82, 0x60, 0x00, \
  0x27, 0x11, 0xca, 0x22, 0x88, 0x00, \
  0x32, 0x8d, 0x34, 0x0d, 0x02, 0x00, \
  0x61, 0x92, 0x60, 0x98, 0x26, 0x00, \
  0x5c, 0x38, 0x80, 0x70, 0x1c, 0x00, \
  0xcc, 0x75, 0x10, 0xc4, 0x30, 0x00, \
  0x2b, 0x19, 0xc5, 0x21, 0x48, 0x00, \
  0x32, 0xd2, 0x68, 0x4a, 0x12, 0x00, \
  0x25, 0x8e, 0xb3, 0x04, 0xc0, 0x00, \
  0x50, 0x88, 0xc6, 0x11, 0x84, 0x00, \
  0x44, 0xa7, 0x09, 0xc2, 0x70, 0x00, \
  0x66, 0x70, 0x8c, 0x47, 0x18, 0x00, \
  0x12, 0xc0, 0xf0, 0x3c, 0x0e, 0x00, \
  0xc3, 0x10, 0xbc, 0x29, 0x0a, 0x00, \
  0x8c, 0x29, 0x42, 0x72, 0x94, 0x00, \
  0x11, 0x5b, 0x16, 0x85, 0xa0, 0x00, \
  0x21, 0x93, 0x44, 0xd1, 0x34, 0x00, \
  0xa2, 0x2c, 0x0b, 0x0a, 0xc2, 0x00, \
  0x18, 0x0c, 0xe9, 0x30, 0xca, 0x00, \
  0x0d, 0xba, 0x52, 0x38, 0xbc, 0x00

#define kMaskRandom39_21 \
  0x8c, 0xe3, 0x09, 0x82, 0x60, 0x00, \
  0x27, 0x11, 0xca, 0x22, 0x88, 0x00, \
  0x32, 0x8d, 0x34, 0x0d, 0x02, 0x00, \
  0x61, 0x92, 0x60, 0x98, 0x26, 0x00, \
  0x5c, 0x38, 0x80, 0x70, 0x1c, 0x00, \
  0xcc, 0x75, 0x10, 0xc4, 0x30, 0x00, \
  0x2b, 0x19, 0xc5, 0x21, 0x48, 0x00, \
  0x32, 0xd2, 0x68, 0x4a, 0x12, 0x00, \
  0x25, 0x8e, 0xb3, 0x04, 0xc0, 0x00, \
  0x50, 0x88, 0xc6, 0x11, 0x84, 0x00, \
  0x8c, 0xe3, 0x18, 0xc6, 0x30, 0x00, \
  0x27, 0x11, 0xc4, 0x71, 0x1c, 0x00, \
  0x32, 0x8d, 0x23, 0x48, 0xd2, 0x00, \
  0x61, 0x92, 0x64, 0x99, 0x26, 0x00, \
  0x5c, 0x38, 0x8e, 0x23, 0x88, 0x00, \
  0x84, 0x87, 0x01, 0xc0, 0x70, 0x00, \
  0x27, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x51, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x1a, 0x22, 0xa8, 0xaa, 0x2a, 0x00, \
  0x68, 0x44, 0x51, 0x14, 0x44, 0x00, \
  0x89, 0x70, 0x1c, 0x07, 0x00, 0x00

#define kMaskRandom39_22 \
  0x8c, 0xe3, 0x18, 0xc6, 0x30, 0x00, \
  0x27, 0x11, 0xc4, 0x71, 0x1c, 0x00, \
  0x32, 0x8d, 0x23, 0x48, 0xd2, 0x00, \
  0x61, 0x92, 0x64, 0x99, 0x26, 0x00, \
  0x5c, 0x38, 0x8e, 0x23, 0x88, 0x00, \
  0x84, 0x87, 0x01, 0xc0, 0x70, 0x00, \
  0x27, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x51, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x1a, 0x22, 0xa8, 0xaa, 0x2a, 0x00, \
  0x68, 0x44, 0x51, 0x14, 0x44, 0x00, \
  0x89, 0x70, 0x1c, 0x07, 0x00, 0x00, \
  0x8c, 0xe3, 0x09, 0x82, 0x60, 0x00, \
  0x27, 0x11, 0xca, 0x22, 0x88, 0x00, \
  0x32, 0x8d, 0x34, 0x0d, 0x02, 0x00, \
  0x61, 0x92, 0x60, 0x98, 0x26, 0x00, \
  0x5c, 0x38, 0x80, 0x70, 0x1c, 0x00, \
  0xcc, 0x75, 0x10, 0xc4, 0x30, 0x00, \
  0x2b, 0x19, 0xc5, 0x21, 0x48, 0x00, \
  0x32, 0xd2, 0x68, 0x4a, 0x12, 0x00, \
  0x25, 0x8e, 0xb3, 0x04, 0xc0, 0x00, \
  0x50, 0x88, 0xc6, 0x11, 0x84, 0x00, \
  0xfc, 0x5a, 0xb2, 0x13, 0x12, 0x00

#define kMaskRandom39_23 \
  0x8c, 0xe3, 0x18, 0xc6, 0x30, 0x00, \
  0x27, 0x11, 0xc4, 0x71, 0x1c, 0x00, \
  0x32, 0x8d, 0x23, 0x48, 0xd2, 0x00, \
  0x61, 0x92, 0x64, 0x99, 0x26, 0x00, \
  0x5c, 0x38, 0x8e, 0x23, 0x88, 0x00, \
  0x84, 0x87, 0x01, 0xc0, 0x70, 0x00, \
  0x27, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x51, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x1a, 0x22, 0xa8, 0xaa, 0x2a, 0x00, \
  0x68, 0x44, 0x51, 0x14, 0x44, 0x00, \
  0x89, 0x70, 0x1c, 0x07, 0x00, 0x00, \
  0x84, 0x87, 0x01, 0xc0, 0x70, 0x00, \
  0x27, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x51, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x1a, 0x22, 0xa8, 0xaa, 0x2a, 0x00, \
  0x68, 0x44, 0x51, 0x14, 0x44, 0x00, \
  0x89, 0x70, 0x1c, 0x07, 0x00, 0x00, \
  0x8c, 0xe3, 0x18, 0xc6, 0x30, 0x00, \
  0x27, 0x11, 0xc4, 0x71, 0x1c, 0x00, \
  0x32, 0x8d, 0x23, 0x48, 0xd2, 0x00, \
  0x61, 0x92, 0x64, 0x99, 0x26, 0x00, \
  0x5c, 0x38, 0x8e, 0x23, 0x88, 0x00, \
  0x90, 0xc8, 0x9e, 0xbb, 0x88, 0x00

#define kMaskRandom39_24 \
  0x84, 0x87, 0x01, 0xc0, 0x70, 0x00, \
  0x27, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x51, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x1a, 0x22, 0xa8, 0xaa, 0x2a, 0x00, \
  0x68, 0x44, 0x51, 0x14, 0x44, 0x00, \
  0x89, 0x70, 0x1c, 0x07, 0x00, 0x00, \
  0x8c, 0xe3, 0x18, 0xc6, 0x30, 0x00, \
  0x27, 0x11, 0xc4, 0x71, 0x1c, 0x00, \
  0x32, 0x8d, 0x23, 0x48, 0xd2, 0x00, \
  0x61, 0x92, 0x64, 0x99, 0x26, 0x00, \
  0x5c, 0x38, 0x8e, 0x23, 0x88, 0x00, \
  0x90, 0xc8, 0x9e, 0xbb, 0x88, 0x00, \
  0x8c, 0xe3, 0x18, 0xc6, 0x30, 0x00, \
  0x27, 0x11, 0xc4, 0x71, 0x1c, 0x00, \
  0x32, 0x8d, 0x23, 0x48, 0xd2, 0x00, \
  0x61, 0x92, 0x64, 0x99, 0x26, 0x00, \
  0x5c, 0x38, 0x8e, 0x23, 0x88, 0x00, \
  0x84, 0x87, 0x01, 0xc0, 0x70, 0x00, \
  0x27, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x51, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x1a, 0x22, 0xa8, 0xaa, 0x2a, 0x00, \
  0x68, 0x44, 0x51, 0x14, 0x44, 0x00, \
  0x89, 0x70, 0x1c, 0x07, 0x00, 0x00, \
  0xac, 0xbc, 0xf0, 0xff, 0x62, 0x00

#define kMaskRandom39_25 \
  0x84, 0x87, 0x01, 0xc0, 0x70, 0x00, \
  0x27, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x51, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x1a, 0x22, 0xa8, 0xaa, 0x2a, 0x00, \
  0x68, 0x44, 0x51, 0x14, 0x44, 0x00, \
  0x89, 0x70, 0x1c, 0x07, 0x00, 0x00, \
  0x8c, 0xe3, 0x18, 0xc6, 0x30, 0x00, \
  0x27, 0x11, 0xc4, 0x71, 0x1c, 0x00, \
  0x32, 0x8d, 0x23, 0x48, 0xd2, 0x00, \
  0x61, 0x92, 0x64, 0x99, 0x26, 0x00, \
  0x5c, 0x38, 0x8e, 0x23, 0x88, 0x00, \
  0x90, 0xc8, 0x9e, 0xbb, 0x88, 0x00, \
  0x84, 0x87, 0x01, 0xc0, 0x70, 0x00, \
  0x27, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x51, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x1a, 0x22, 0xa8, 0xaa, 0x2a, 0x00, \
  0x68, 0x44, 0x51, 0x14, 0x44, 0x00, \
  0x89, 0x70, 0x1c, 0x07, 0x00, 0x00, \
  0x8c, 0x23, 0x08, 0xc2, 0x30, 0x00, \
  0x47, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x81, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x12, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0x58, 0x14, 0x45, 0x11, 0x44, 0x00, \
  0x28, 0xca, 0x12, 0x84, 0xa0, 0x00, \
  0x34, 0x60, 0x98, 0x26, 0x08, 0x00

#define kMaskRandom39_26 \
  0x84, 0x87, 0x01, 0xc0, 0x70, 0x00, \
  0x27, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x51, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x1a, 0x22, 0xa8, 0xaa, 0x2a, 0x00, \
  0x68, 0x44, 0x51, 0x14, 0x44, 0x00, \
  0x89, 0x70, 0x1c, 0x07, 0x00, 0x00, \
  0x8c, 0x23, 0x08, 0xc2, 0x30, 0x00, \
  0x47, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x81, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x12, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0x58, 0x14, 0x45, 0x11, 0x44, 0x00, \
  0x28, 0xca, 0x12, 0x84, 0xa0, 0x00, \
  0x34, 0x60, 0x98, 0x26, 0x08, 0x00, \
  0x84, 0x87, 0x01, 0xc0, 0x70, 0x00, \
  0x27, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x51, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x1a, 0x22, 0xa8, 0xaa, 0x2a, 0x00, \
  0x68, 0x44, 0x51, 0x14, 0x44, 0x00, \
  0x89, 0x70, 0x1c, 0x07, 0x00, 0x00, \
  0x8c, 0xe3, 0x18, 0xc6, 0x30, 0x00, \
  0x27, 0x11, 0xc4, 0x71, 0x1c, 0x00, \
  0x32, 0x8d, 0x23, 0x48, 0xd2, 0x00, \
  0x61, 0x92, 0x64, 0x99, 0x26, 0x00, \
  0x5c, 0x38, 0x8e, 0x23, 0x88, 0x00, \
  0x90, 0xc8, 0x9e, 0xbb, 0x88, 0x00, \
  0x10, 0x17, 0x44, 0x72, 0xec, 0x00

#define kMaskRandom39_27 \
  0x84, 0x87, 0x01, 0xc0, 0x70, 0x00, \
  0x27, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x51, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x1a, 0x22, 0xa8, 0xaa, 0x2a, 0x00, \
  0x68, 0x44, 0x51, 0x14, 0x44, 0x00, \
  0x89, 0x70, 0x1c, 0x07, 0x00, 0x00, \
  0x8c, 0x23, 0x08, 0xc2, 0x30, 0x00, \
  0x47, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x81, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x12, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0x58, 0x14, 0x45, 0x11, 0x44, 0x00, \
  0x28, 0xca, 0x12, 0x84, 0xa0, 0x00, \
  0x34, 0x60, 0x98, 0x26, 0x08, 0x00, \
  0x8c, 0x23, 0x08, 0xc2, 0x30, 0x00, \
  0x47, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x81, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x12, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0x58, 0x14, 0x45, 0x11, 0x44, 0x00, \
  0x28, 0xca, 0x12, 0x84, 0xa0, 0x00, \
  0x34, 0x60, 0x98, 0x26, 0x08, 0x00, \
  0x84, 0x87, 0x01, 0xc0, 0x70, 0x00, \
  0x27, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x51, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x1a, 0x22, 0xa8, 0xaa, 0x2a, 0x00, \
  0x68, 0x44, 0x51, 0x14, 0x44, 0x00, \
  0x89, 0x70, 0x1c, 0x07, 0x00, 0x00, \
  0x6e, 0x27, 0x6a, 0xc7, 0xc4, 0x00

#define kMaskRandom39_28 \
  0x8c, 0x23, 0x08, 0xc2, 0x30, 0x00, \
  0x47, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x81, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x12, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0x58, 0x14, 0x45, 0x11, 0x44, 0x00, \
  0x28, 0xca, 0x12, 0x84, 0xa0, 0x00, \
  0x34, 0x60, 0x98, 0x26, 0x08, 0x00, \
  0x84, 0x87, 0x01, 0xc0, 0x70, 0x00, \
  0x27, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x51, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x1a, 0x22, 0xa8, 0xaa, 0x2a, 0x00, \
  0x68, 0x44, 0x51, 0x14, 0x44, 0x00, \
  0x89, 0x70, 0x1c, 0x07, 0x00, 0x00, \
  0x6e, 0x27, 0x6a, 0xc7, 0xc4, 0x00, \
  0x84, 0x87, 0x01, 0xc0, 0x70, 0x00, \
  0x27, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x51, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x1a, 0x22, 0xa8, 0xaa, 0x2a, 0x00, \
  0x68, 0x44, 0x51, 0x14, 0x44, 0x00, \
  0x89, 0x70, 0x1c, 0x07, 0x00, 0x00, \
  0x8c, 0x23, 0x08, 0xc2, 0x30, 0x00, \
  0x47, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x81, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x12, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0x58, 0x14, 0x45, 0x11, 0x44, 0x00, \
  0x28, 0xca, 0x12, 0x84, 0xa0, 0x00, \
  0x34, 0x60, 0x98, 0x26, 0x08, 0x00, \
  0x86, 0xb6, 0x04, 0xbc, 0x1e, 0x00

#define kMaskRandom39_29 \
  0x8c, 0x23, 0x08, 0xc2, 0x30, 0x00, \
  0x47, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x81, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x12, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0x58, 0x14, 0x45, 0x11, 0x44, 0x00, \
  0x28, 0xca, 0x12, 0x84, 0xa0, 0x00, \
  0x34, 0x60, 0x98, 0x26, 0x08, 0x00, \
  0x84, 0x87, 0x01, 0xc0, 0x70, 0x00, \
  0x27, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x51, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x1a, 0x22, 0xa8, 0xaa, 0x2a, 0x00, \
  0x68, 0x44, 0x51, 0x14, 0x44, 0x00, \
  0x89, 0x70, 0x1c, 0x07, 0x00, 0x00, \
  0x6e, 0x27, 0x6a, 0xc7, 0xc4, 0x00, \
  0x8c, 0x23, 0x08, 0xc2, 0x30, 0x00, \
  0x47, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x81, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x12, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0x58, 0x14, 0x45, 0x11, 0x44, 0x00, \
  0x28, 0xca, 0x12, 0x84, 0xa0, 0x00, \
  0x34, 0x60, 0x98, 0x26, 0x08, 0x00, \
  0x2c, 0x16, 0x05, 0x81, 0x60, 0x00, \
  0x91, 0x40, 0xd0, 0x34, 0x0c, 0x00, \
  0xc0, 0xd0, 0x34, 0x0d, 0x02, 0x00, \
  0x06, 0x82, 0xa0, 0xa8, 0x2a, 0x00, \
  0xc8, 0x0c, 0x43, 0x10, 0xc4, 0x00, \
  0x45, 0x61, 0x18, 0x46, 0x10, 0x00, \
  0x30, 0x91, 0x44, 0x51, 0x14, 0x00, \
  0xa2, 0x28, 0x2a, 0x0a, 0x82, 0x00

#define kMaskRandom39_3 \
  0xcc, 0x67, 0x19, 0xc6, 0x70, 0x00, \
  0x27, 0x2c, 0xca, 0xb2, 0xac, 0x00, \
  0x92, 0xd2, 0x76, 0x2d, 0x46, 0x00

#define kMaskRandom39_30 \
  0x8c, 0x23, 0x08, 0xc2, 0x30, 0x00, \
  0x47, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x81, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x12, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0x58, 0x14, 0x45, 0x11, 0x44, 0x00, \
  0x28, 0xca, 0x12, 0x84, 0xa0, 0x00, \
  0x34, 0x60, 0x98, 0x26, 0x08, 0x00, \
  0x2c, 0x16, 0x05, 0x81, 0x60, 0x00, \
  0x91, 0x40, 0xd0, 0x34, 0x0c, 0x00, \
  0xc0, 0xd0, 0x34, 0x0d, 0x02, 0x00, \
  0x06, 0x82, 0xa0, 0xa8, 0x2a, 0x00, \
  0xc8, 0x0c, 0x43, 0x10, 0xc4, 0x00, \
  0x45, 0x61, 0x18, 0x46, 0x10, 0x00, \
  0x30, 0x91, 0x44, 0x51, 0x14, 0x00, \
  0xa2, 0x28, 0x2a, 0x0a, 0x82, 0x00, \
  0x8c, 0x23, 0x08, 0xc2, 0x30, 0x00, \
  0x47, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x81, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x12, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0x58, 0x14, 0x45, 0x11, 0x44, 0x00, \
  0x28, 0xca, 0x12, 0x84, 0xa0, 0x00, \
  0x34, 0x60, 0x98, 0x26, 0x08, 0x00, \
  0x84, 0x87, 0x01, 0xc0, 0x70, 0x00, \
  0x27, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x51, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x1a, 0x22, 0xa8, 0xaa, 0x2a, 0x00, \
  0x68, 0x44, 0x51, 0x14, 0x44, 0x00, \
  0x89, 0x70, 0x1c, 0x07, 0x00, 0x00, \
  0x6e, 0x27, 0x6a, 0xc7, 0xc4, 0x00, \
  0xb3, 0x1d, 0x13, 0x03, 0x5a, 0x00

#define kMaskRandom39_31 \
  0x8c, 0x23, 0x08, 0xc2, 0x30, 0x00, \
  0x47, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x81, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x12, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0x58, 0x14, 0x45, 0x11, 0x44, 0x00, \
  0x28, 0xca, 0x12, 0x84, 0xa0, 0x00, \
  0x34, 0x60, 0x98, 0x26, 0x08, 0x00, \
  0x2c, 0x16, 0x05, 0x81, 0x60, 0x00, \
  0x91, 0x40, 0xd0, 0x34, 0x0c, 0x00, \
  0xc0, 0xd0, 0x34, 0x0d, 0x02, 0x00, \
  0x06, 0x82, 0xa0, 0xa8, 0x2a, 0x00, \
  0xc8, 0x0c, 0x43, 0x10, 0xc4, 0x00, \
  0x45, 0x61, 0x18, 0x46, 0x10, 0x00, \
  0x30, 0x91, 0x44, 0x51, 0x14, 0x00, \
  0xa2, 0x28, 0x2a, 0x0a, 0x82, 0x00, \
  0x2c, 0x16, 0x05, 0x81, 0x60, 0x00, \
  0x91, 0x40, 0xd0, 0x34, 0x0c, 0x00, \
  0xc0, 0xd0, 0x34, 0x0d, 0x02, 0x00, \
  0x06, 0x82, 0xa0, 0xa8, 0x2a, 0x00, \
  0xc8, 0x0c, 0x43, 0x10, 0xc4, 0x00, \
  0x45, 0x61, 0x18, 0x46, 0x10, 0x00, \
  0x30, 0x91, 0x44, 0x51, 0x14, 0x00, \
  0xa2, 0x28, 0x2a, 0x0a, 0x82, 0x00, \
  0x8c, 0x23, 0x08, 0xc2, 0x30, 0x00, \
  0x47, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x81, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x12, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0x58, 0x14, 0x45, 0x11, 0x44, 0x00, \
  0x28, 0xca, 0x12, 0x84, 0xa0, 0x00, \
  0x34, 0x60, 0x98, 0x26, 0x08, 0x00, \
  0x7e, 0x75, 0xe5, 0x03, 0x8c, 0x00

#define kMaskRandom39_32 \
  0x2c, 0x16, 0x05, 0x81, 0x60, 0x00, \
  0x91, 0x40, 0xd0, 0x34, 0x0c, 0x00, \
  0xc0, 0xd0, 0x34, 0x0d, 0x02, 0x00, \
  0x06, 0x82, 0xa0, 0xa8, 0x2a, 0x00, \
  0xc8, 0x0c, 0x43, 0x10, 0xc4, 0x00, \
  0x45, 0x61, 0x18, 0x46, 0x10, 0x00, \
  0x30, 0x91, 0x44, 0x51, 0x14, 0x00, \
  0xa2, 0x28, 0x2a, 0x0a, 0x82, 0x00, \
  0x8c, 0x23, 0x08, 0xc2, 0x30, 0x00, \
  0x47, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x81, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x12, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0x58, 0x14, 0x45, 0x11, 0x44, 0x00, \
  0x28, 0xca, 0x12, 0x84, 0xa0, 0x00, \
  0x34, 0x60, 0x98, 0x26, 0x08, 0x00, \
  0x7e, 0x75, 0xe5, 0x03, 0x8c, 0x00, \
  0x8c, 0x23, 0x08, 0xc2, 0x30, 0x00, \
  0x47, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x81, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x12, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0x58, 0x14, 0x45, 0x11, 0x44, 0x00, \
  0x28, 0xca, 0x12, 0x84, 0xa0, 0x00, \
  0x34, 0x60, 0x98, 0x26, 0x08, 0x00, \
  0x2c, 0x16, 0x05, 0x81, 0x60, 0x00, \
  0x91, 0x40, 0xd0, 0x34, 0x0c, 0x00, \
  0xc0, 0xd0, 0x34, 0x0d, 0x02, 0x00, \
  0x06, 0x82, 0xa0, 0xa8, 0x2a, 0x00, \
  0xc8, 0x0c, 0x43, 0x10, 0xc4, 0x00, \
  0x45, 0x61, 0x18, 0x46, 0x10, 0x00, \
  0x30, 0x91, 0x44, 0x51, 0x14, 0x00, \
  0xa2, 0x28, 0x2a, 0x0a, 0x82, 0x00, \
  0x83, 0x1a, 0x3c, 0x2a, 0x7a, 0x00

#define kMaskRandom39_33 \
  0x2c, 0x16, 0x05, 0x81, 0x60, 0x00, \
  0x91, 0x40, 0xd0, 0x34, 0x0c, 0x00, \
  0xc0, 0xd0, 0x34, 0x0d, 0x02, 0x00, \
  0x06, 0x82, 0xa0, 0xa8, 0x2a, 0x00, \
  0xc8, 0x0c, 0x43, 0x10, 0xc4, 0x00, \
  0x45, 0x61, 0x18, 0x46, 0x10, 0x00, \
  0x30, 0x91, 0x44, 0x51, 0x14, 0x00, \
  0xa2, 0x28, 0x2a, 0x0a, 0x82, 0x00, \
  0x8c, 0x23, 0x08, 0xc2, 0x30, 0x00, \
  0x47, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x81, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x12, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0x58, 0x14, 0x45, 0x11, 0x44, 0x00, \
  0x28, 0xca, 0x12, 0x84, 0xa0, 0x00, \
  0x34, 0x60, 0x98, 0x26, 0x08, 0x00, \
  0x7e, 0x75, 0xe5, 0x03, 0x8c, 0x00, \
  0x2c, 0x16, 0x05, 0x81, 0x60, 0x00, \
  0x91, 0x40, 0xd0, 0x34, 0x0c, 0x00, \
  0xc0, 0xd0, 0x34, 0x0d, 0x02, 0x00, \
  0x06, 0x82, 0xa0, 0xa8, 0x2a, 0x00, \
  0xc8, 0x0c, 0x43, 0x10, 0xc4, 0x00, \
  0x45, 0x61, 0x18, 0x46, 0x10, 0x00, \
  0x30, 0x91, 0x44, 0x51, 0x14, 0x00, \
  0xa2, 0x28, 0x2a, 0x0a, 0x82, 0x00, \
  0x4c, 0x27, 0x09, 0xc2, 0x70, 0x00, \
  0x66, 0x71, 0x9c, 0x67, 0x18, 0x00, \
  0x91, 0x40, 0xf0, 0x3c, 0x0e, 0x00, \
  0x42, 0x90, 0xa4, 0x29, 0x0a, 0x00, \
  0xa4, 0x29, 0x4a, 0x52, 0x94, 0x00, \
  0x13, 0x5a, 0x16, 0x85, 0xa0, 0x00, \
  0x30, 0x93, 0x44, 0xd1, 0x34, 0x00, \
  0x88, 0xac, 0x2b, 0x0a, 0xc2, 0x00, \
  0x09, 0x0c, 0xc3, 0x30, 0xcc, 0x00

#define kMaskRandom39_34 \
  0x2c, 0x16, 0x05, 0x81, 0x60, 0x00, \
  0x91, 0x40, 0xd0, 0x34, 0x0c, 0x00, \
  0xc0, 0xd0, 0x34, 0x0d, 0x02, 0x00, \
  0x06, 0x82, 0xa0, 0xa8, 0x2a, 0x00, \
  0xc8, 0x0c, 0x43, 0x10, 0xc4, 0x00, \
  0x45, 0x61, 0x18, 0x46, 0x10, 0x00, \
  0x30, 0x91, 0x44, 0x51, 0x14, 0x00, \
  0xa2, 0x28, 0x2a, 0x0a, 0x82, 0x00, \
  0x4c, 0x27, 0x09, 0xc2, 0x70, 0x00, \
  0x66, 0x71, 0x9c, 0x67, 0x18, 0x00, \
  0x91, 0x40, 0xf0, 0x3c, 0x0e, 0x00, \
  0x42, 0x90, 0xa4, 0x29, 0x0a, 0x00, \
  0xa4, 0x29, 0x4a, 0x52, 0x94, 0x00, \
  0x13, 0x5a, 0x16, 0x85, 0xa0, 0x00, \
  0x30, 0x93, 0x44, 0xd1, 0x34, 0x00, \
  0x88, 0xac, 0x2b, 0x0a, 0xc2, 0x00, \
  0x09, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x2c, 0x16, 0x05, 0x81, 0x60, 0x00, \
  0x91, 0x40, 0xd0, 0x34, 0x0c, 0x00, \
  0xc0, 0xd0, 0x34, 0x0d, 0x02, 0x00, \
  0x06, 0x82, 0xa0, 0xa8, 0x2a, 0x00, \
  0xc8, 0x0c, 0x43, 0x10, 0xc4, 0x00, \
  0x45, 0x61, 0x18, 0x46, 0x10, 0x00, \
  0x30, 0x91, 0x44, 0x51, 0x14, 0x00, \
  0xa2, 0x28, 0x2a, 0x0a, 0x82, 0x00, \
  0x8c, 0x23, 0x08, 0xc2, 0x30, 0x00, \
  0x47, 0x19, 0x86, 0x61, 0x98, 0x00, \
  0x81, 0x88, 0x62, 0x18, 0x86, 0x00, \
  0x12, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0x58, 0x14, 0x45, 0x11, 0x44, 0x00, \
  0x28, 0xca, 0x12, 0x84, 0xa0, 0x00, \
  0x34, 0x60, 0x98, 0x26, 0x08, 0x00, \
  0x7e, 0x75, 0xe5, 0x03, 0x8c, 0x00, \
  0xc6, 0xbb, 0x7e, 0xd9, 0x80, 0x00

#define kMaskRandom39_35 \
  0x2c, 0x16, 0x05, 0x81, 0x60, 0x00, \
  0x91, 0x40, 0xd0, 0x34, 0x0c, 0x00, \
  0xc0, 0xd0, 0x34, 0x0d, 0x02, 0x00, \
  0x06, 0x82, 0xa0, 0xa8, 0x2a, 0x00, \
  0xc8, 0x0c, 0x43, 0x10, 0xc4, 0x00, \
  0x45, 0x61, 0x18, 0x46, 0x10, 0x00, \
  0x30, 0x91, 0x44, 0x51, 0x14, 0x00, \
  0xa2, 0x28, 0x2a, 0x0a, 0x82, 0x00, \
  0x4c, 0x27, 0x09, 0xc2, 0x70, 0x00, \
  0x66, 0x71, 0x9c, 0x67, 0x18, 0x00, \
  0x91, 0x40, 0xf0, 0x3c, 0x0e, 0x00, \
  0x42, 0x90, 0xa4, 0x29, 0x0a, 0x00, \
  0xa4, 0x29, 0x4a, 0x52, 0x94, 0x00, \
  0x13, 0x5a, 0x16, 0x85, 0xa0, 0x00, \
  0x30, 0x93, 0x44, 0xd1, 0x34, 0x00, \
  0x88, 0xac, 0x2b, 0x0a, 0xc2, 0x00, \
  0x09, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x4c, 0x27, 0x09, 0xc2, 0x70, 0x00, \
  0x66, 0x71, 0x9c, 0x67, 0x18, 0x00, \
  0x91, 0x40, 0xf0, 0x3c, 0x0e, 0x00, \
  0x42, 0x90, 0xa4, 0x29, 0x0a, 0x00, \
  0xa4, 0x29, 0x4a, 0x52, 0x94, 0x00, \
  0x13, 0x5a, 0x16, 0x85, 0xa0, 0x00, \
  0x30, 0x93, 0x44, 0xd1, 0x34, 0x00, \
  0x88, 0xac, 0x2b, 0x0a, 0xc2, 0x00, \
  0x09, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x2c, 0x16, 0x05, 0x81, 0x60, 0x00, \
  0x91, 0x40, 0xd0, 0x34, 0x0c, 0x00, \
  0xc0, 0xd0, 0x34, 0x0d, 0x02, 0x00, \
  0x06, 0x82, 0xa0, 0xa8, 0x2a, 0x00, \
  0xc8, 0x0c, 0x43, 0x10, 0xc4, 0x00, \
  0x45, 0x61, 0x18, 0x46, 0x10, 0x00, \
  0x30, 0x91, 0x44, 0x51, 0x14, 0x00, \
  0xa2, 0x28, 0x2a, 0x0a, 0x82, 0x00, \
  0x51, 0x97, 0x24, 0x2f, 0x7e, 0x00

#define kMaskRandom39_36 \
  0x4c, 0x27, 0x09, 0xc2, 0x70, 0x00, \
  0x66, 0x71, 0x9c, 0x67, 0x18, 0x00, \
  0x91, 0x40, 0xf0, 0x3c, 0x0e, 0x00, \
  0x42, 0x90, 0xa4, 0x29, 0x0a, 0x00, \
  0xa4, 0x29, 0x4a, 0x52, 0x94, 0x00, \
  0x13, 0x5a, 0x16, 0x85, 0xa0, 0x00, \
  0x30, 0x93, 0x44, 0xd1, 0x34, 0x00, \
  0x88, 0xac, 0x2b, 0x0a, 0xc2, 0x00, \
  0x09, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x2c, 0x16, 0x05, 0x81, 0x60, 0x00, \
  0x91, 0x40, 0xd0, 0x34, 0x0c, 0x00, \
  0xc0, 0xd0, 0x34, 0x0d, 0x02, 0x00, \
  0x06, 0x82, 0xa0, 0xa8, 0x2a, 0x00, \
  0xc8, 0x0c, 0x43, 0x10, 0xc4, 0x00, \
  0x45, 0x61, 0x18, 0x46, 0x10, 0x00, \
  0x30, 0x91, 0x44, 0x51, 0x14, 0x00, \
  0xa2, 0x28, 0x2a, 0x0a, 0x82, 0x00, \
  0x51, 0x97, 0x24, 0x2f, 0x7e, 0x00, \
  0x2c, 0x16, 0x05, 0x81, 0x60, 0x00, \
  0x91, 0x40, 0xd0, 0x34, 0x0c, 0x00, \
  0xc0, 0xd0, 0x34, 0x0d, 0x02, 0x00, \
  0x06, 0x82, 0xa0, 0xa8, 0x2a, 0x00, \
  0xc8, 0x0c, 0x43, 0x10, 0xc4, 0x00, \
  0x45, 0x61, 0x18, 0x46, 0x10, 0x00, \
  0x30, 0x91, 0x44, 0x51, 0x14, 0x00, \
  0xa2, 0x28, 0x2a, 0x0a, 0x82, 0x00, \
  0x4c, 0x27, 0x09, 0xc2, 0x70, 0x00, \
  0x66, 0x71, 0x9c, 0x67, 0x18, 0x00, \
  0x91, 0x40, 0xf0, 0x3c, 0x0e, 0x00, \
  0x42, 0x90, 0xa4, 0x29, 0x0a, 0x00, \
  0xa4, 0x29, 0x4a, 0x52, 0x94, 0x00, \
  0x13, 0x5a, 0x16, 0x85, 0xa0, 0x00, \
  0x30, 0x93, 0x44, 0xd1, 0x34, 0x00, \
  0x88, 0xac, 0x2b, 0x0a, 0xc2, 0x00, \
  0x09, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x11, 0x78, 0xfe, 0x43, 0xd6, 0x00

#define kMaskRandom39_37 \
  0x4c, 0x27, 0x09, 0xc2, 0x70, 0x00, \
  0x66, 0x71, 0x9c, 0x67, 0x18, 0x00, \
  0x91, 0x40, 0xf0, 0x3c, 0x0e, 0x00, \
  0x42, 0x90, 0xa4, 0x29, 0x0a, 0x00, \
  0xa4, 0x29, 0x4a, 0x52, 0x94, 0x00, \
  0x13, 0x5a, 0x16, 0x85, 0xa0, 0x00, \
  0x30, 0x93, 0x44, 0xd1, 0x34, 0x00, \
  0x88, 0xac, 0x2b, 0x0a, 0xc2, 0x00, \
  0x09, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x2c, 0x16, 0x05, 0x81, 0x60, 0x00, \
  0x91, 0x40, 0xd0, 0x34, 0x0c, 0x00, \
  0xc0, 0xd0, 0x34, 0x0d, 0x02, 0x00, \
  0x06, 0x82, 0xa0, 0xa8, 0x2a, 0x00, \
  0xc8, 0x0c, 0x43, 0x10, 0xc4, 0x00, \
  0x45, 0x61, 0x18, 0x46, 0x10, 0x00, \
  0x30, 0x91, 0x44, 0x51, 0x14, 0x00, \
  0xa2, 0x28, 0x2a, 0x0a, 0x82, 0x00, \
  0x51, 0x97, 0x24, 0x2f, 0x7e, 0x00, \
  0x4c, 0x27, 0x09, 0xc2, 0x70, 0x00, \
  0x66, 0x71, 0x9c, 0x67, 0x18, 0x00, \
  0x91, 0x40, 0xf0, 0x3c, 0x0e, 0x00, \
  0x42, 0x90, 0xa4, 0x29, 0x0a, 0x00, \
  0xa4, 0x29, 0x4a, 0x52, 0x94, 0x00, \
  0x13, 0x5a, 0x16, 0x85, 0xa0, 0x00, \
  0x30, 0x93, 0x44, 0xd1, 0x34, 0x00, \
  0x88, 0xac, 0x2b, 0x0a, 0xc2, 0x00, \
  0x09, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x4c, 0x26, 0x09, 0x82, 0x60, 0x00, \
  0x66, 0x28, 0x8a, 0x22, 0x88, 0x00, \
  0x91, 0x50, 0x34, 0x0d, 0x02, 0x00, \
  0x42, 0x82, 0x60, 0x98, 0x26, 0x00, \
  0xa4, 0x01, 0xc0, 0x70, 0x1c, 0x00, \
  0x13, 0x43, 0x10, 0xc4, 0x30, 0x00, \
  0x30, 0x94, 0x85, 0x21, 0x48, 0x00, \
  0x88, 0xa1, 0x28, 0x4a, 0x12, 0x00, \
  0x09, 0x4c, 0x13, 0x04, 0xc0, 0x00, \
  0xcd, 0x98, 0x46, 0x11, 0x84, 0x00

#define kMaskRandom39_38 \
  0x4c, 0x27, 0x09, 0xc2, 0x70, 0x00, \
  0x66, 0x71, 0x9c, 0x67, 0x18, 0x00, \
  0x91, 0x40, 0xf0, 0x3c, 0x0e, 0x00, \
  0x42, 0x90, 0xa4, 0x29, 0x0a, 0x00, \
  0xa4, 0x29, 0x4a, 0x52, 0x94, 0x00, \
  0x13, 0x5a, 0x16, 0x85, 0xa0, 0x00, \
  0x30, 0x93, 0x44, 0xd1, 0x34, 0x00, \
  0x88, 0xac, 0x2b, 0x0a, 0xc2, 0x00, \
  0x09, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x4c, 0x26, 0x09, 0x82, 0x60, 0x00, \
  0x66, 0x28, 0x8a, 0x22, 0x88, 0x00, \
  0x91, 0x50, 0x34, 0x0d, 0x02, 0x00, \
  0x42, 0x82, 0x60, 0x98, 0x26, 0x00, \
  0xa4, 0x01, 0xc0, 0x70, 0x1c, 0x00, \
  0x13, 0x43, 0x10, 0xc4, 0x30, 0x00, \
  0x30, 0x94, 0x85, 0x21, 0x48, 0x00, \
  0x88, 0xa1, 0x28, 0x4a, 0x12, 0x00, \
  0x09, 0x4c, 0x13, 0x04, 0xc0, 0x00, \
  0xcd, 0x98, 0x46, 0x11, 0x84, 0x00, \
  0x4c, 0x27, 0x09, 0xc2, 0x70, 0x00, \
  0x66, 0x71, 0x9c, 0x67, 0x18, 0x00, \
  0x91, 0x40, 0xf0, 0x3c, 0x0e, 0x00, \
  0x42, 0x90, 0xa4, 0x29, 0x0a, 0x00, \
  0xa4, 0x29, 0x4a, 0x52, 0x94, 0x00, \
  0x13, 0x5a, 0x16, 0x85, 0xa0, 0x00, \
  0x30, 0x93, 0x44, 0xd1, 0x34, 0x00, \
  0x88, 0xac, 0x2b, 0x0a, 0xc2, 0x00, \
  0x09, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x2c, 0x16, 0x05, 0x81, 0x60, 0x00, \
  0x91, 0x40, 0xd0, 0x34, 0x0c, 0x00, \
  0xc0, 0xd0, 0x34, 0x0d, 0x02, 0x00, \
  0x06, 0x82, 0xa0, 0xa8, 0x2a, 0x00, \
  0xc8, 0x0c, 0x43, 0x10, 0xc4, 0x00, \
  0x45, 0x61, 0x18, 0x46, 0x10, 0x00, \
  0x30, 0x91, 0x44, 0x51, 0x14, 0x00, \
  0xa2, 0x28, 0x2a, 0x0a, 0x82, 0x00, \
  0x51, 0x97, 0x24, 0x2f, 0x7e, 0x00, \
  0x9e, 0xd8, 0x3c, 0x7e, 0x2e, 0x00

#define kMaskRandom39_39 \
  0x4c, 0x27, 0x09, 0xc2, 0x70, 0x00, \
  0x66, 0x71, 0x9c, 0x67, 0x18, 0x00, \
  0x91, 0x40, 0xf0, 0x3c, 0x0e, 0x00, \
  0x42, 0x90, 0xa4, 0x29, 0x0a, 0x00, \
  0xa4, 0x29, 0x4a, 0x52, 0x94, 0x00, \
  0x13, 0x5a, 0x16, 0x85, 0xa0, 0x00, \
  0x30, 0x93, 0x44, 0xd1, 0x34, 0x00, \
  0x88, 0xac, 0x2b, 0x0a, 0xc2, 0x00, \
  0x09, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x4c, 0x26, 0x09, 0x82, 0x60, 0x00, \
  0x66, 0x28, 0x8a, 0x22, 0x88, 0x00, \
  0x91, 0x50, 0x34, 0x0d, 0x02, 0x00, \
  0x42, 0x82, 0x60, 0x98, 0x26, 0x00, \
  0xa4, 0x01, 0xc0, 0x70, 0x1c, 0x00, \
  0x13, 0x43, 0x10, 0xc4, 0x30, 0x00, \
  0x30, 0x94, 0x85, 0x21, 0x48, 0x00, \
  0x88, 0xa1, 0x28, 0x4a, 0x12, 0x00, \
  0x09, 0x4c, 0x13, 0x04, 0xc0, 0x00, \
  0xcd, 0x98, 0x46, 0x11, 0x84, 0x00, \
  0x4c, 0x27, 0x09, 0x82, 0x60, 0x00, \
  0x66, 0x71, 0x8a, 0x22, 0x88, 0x00, \
  0x91, 0x40, 0xf4, 0x0d, 0x02, 0x00, \
  0x42, 0x90, 0xa0, 0x98, 0x26, 0x00, \
  0xa4, 0x29, 0x40, 0x70, 0x1c, 0x00, \
  0x13, 0x5a, 0x10, 0xc4, 0x30, 0x00, \
  0x30, 0x93, 0x45, 0x21, 0x48, 0x00, \
  0x88, 0xac, 0x28, 0x4a, 0x12, 0x00, \
  0x09, 0x0c, 0xd3, 0x04, 0xc0, 0x00, \
  0x4c, 0x26, 0x06, 0x11, 0x84, 0x00, \
  0x66, 0x28, 0x89, 0xc2, 0x70, 0x00, \
  0x91, 0x50, 0x3c, 0x67, 0x18, 0x00, \
  0x42, 0x82, 0x70, 0x3c, 0x0e, 0x00, \
  0xa4, 0x01, 0xc4, 0x29, 0x0a, 0x00, \
  0x13, 0x43, 0x0a, 0x52, 0x94, 0x00, \
  0x30, 0x94, 0x96, 0x85, 0xa0, 0x00, \
  0x88, 0xa1, 0x24, 0xd1, 0x34, 0x00, \
  0x09, 0x4c, 0x0b, 0x0a, 0xc2, 0x00, \
  0xcd, 0x98, 0x43, 0x30, 0xcc, 0x00, \
  0x1d, 0x04, 0x3e, 0xf1, 0xb4, 0x00

#define kMaskRandom39_4 \
  0xec, 0x73, 0x1c, 0xc7, 0x30, 0x00, \
  0x67, 0x19, 0xc6, 0x71, 0x9c, 0x00, \
  0xb1, 0xcc, 0x73, 0x1c, 0xc6, 0x00, \
  0x5a, 0x96, 0xa5, 0xa9, 0x6a, 0x00

#define kMaskRandom39_5 \
  0x4c, 0xe7, 0x19, 0xc6, 0x70, 0x00, \
  0x66, 0x31, 0xcc, 0x73, 0x1c, 0x00, \
  0xa1, 0xcc, 0x73, 0x1c, 0xa6, 0x00, \
  0x92, 0xa6, 0xa5, 0x6a, 0x6a, 0x00, \
  0xb8, 0x99, 0x96, 0x8b, 0x94, 0x00

#define kMaskRandom39_6 \
  0x4c, 0x36, 0x09, 0x83, 0x60, 0x00, \
  0x45, 0x68, 0x8a, 0x26, 0x88, 0x00, \
  0x30, 0xd0, 0x64, 0x1d, 0x06, 0x00, \
  0x8a, 0x82, 0xb0, 0xa8, 0x2a, 0x00, \
  0x26, 0x0b, 0x40, 0xd0, 0xd4, 0x00, \
  0x95, 0x45, 0x13, 0x44, 0x30, 0x00

#define kMaskRandom39_7 \
  0xc4, 0xa3, 0x09, 0xc2, 0x30, 0x00, \
  0x23, 0x19, 0x86, 0x65, 0x80, 0x00, \
  0x91, 0x1c, 0x22, 0x01, 0xd6, 0x00, \
  0x4a, 0x82, 0xb0, 0x2a, 0x2a, 0x00, \
  0x34, 0x49, 0x44, 0x98, 0x94, 0x00, \
  0x8b, 0x4a, 0x1a, 0x84, 0x60, 0x00, \
  0xc8, 0x24, 0xc1, 0x94, 0x4c, 0x00

#define kMaskRandom39_8 \
  0x16, 0x13, 0x84, 0xe1, 0x38, 0x00, \
  0xc2, 0x44, 0xd1, 0x34, 0x4c, 0x00, \
  0x60, 0xe8, 0x3a, 0x0e, 0x82, 0x00, \
  0x85, 0x12, 0x64, 0x99, 0x26, 0x00, \
  0xcc, 0x21, 0x5c, 0x52, 0x14, 0x00, \
  0x29, 0x63, 0x18, 0xc6, 0x30, 0x00, \
  0x11, 0x98, 0xc6, 0x31, 0x8c, 0x00, \
  0xb0, 0x0c, 0x63, 0x18, 0xc6, 0x00

#define kMaskRandom39_9 \
  0x44, 0xa7, 0x09, 0xc2, 0x70, 0x00, \
  0x66, 0x70, 0x8c, 0x47, 0x18, 0x00, \
  0x12, 0xc0, 0xf0, 0x3c, 0x0e, 0x00, \
  0xc3, 0x10, 0xbc, 0x29, 0x0a, 0x00, \
  0x8c, 0x29, 0x42, 0x72, 0x94, 0x00, \
  0x11, 0x5b, 0x16, 0x85, 0xa0, 0x00, \
  0x21, 0x93, 0x44, 0xd1, 0x34, 0x00, \
  0xa2, 0x2c, 0x0b, 0x0a, 0xc2, 0x00, \
  0x18, 0x0c, 0xe9, 0x30, 0xca, 0x00

#define kMaskRandom40_1 \
  0xff, 0xff, 0xff, 0xff, 0xff, 0x00

#define kMaskRandom40_10 \
  0x4c, 0x13, 0x04, 0xc1, 0x30, 0x00, \
  0x51, 0x14, 0x45, 0x11, 0x44, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0x81, 0x00, \
  0x04, 0xc1, 0x30, 0x4c, 0x13, 0x00, \
  0x03, 0x80, 0xe0, 0x38, 0x0e, 0x00, \
  0x86, 0x21, 0x88, 0x62, 0x18, 0x00, \
  0x29, 0x0a, 0x42, 0x90, 0xa4, 0x00, \
  0x42, 0x50, 0x94, 0x25, 0x09, 0x00, \
  0x98, 0x26, 0x09, 0x82, 0x60, 0x00, \
  0x30, 0x8c, 0x23, 0x08, 0xc2, 0x00

#define kMaskRandom40_11 \
  0xc6, 0x31, 0x8c, 0x63, 0x18, 0x00, \
  0x23, 0x88, 0xe2, 0x38, 0x8e, 0x00, \
  0x1a, 0x46, 0x91, 0xa4, 0x69, 0x00, \
  0x24, 0xc9, 0x32, 0x4c, 0x93, 0x00, \
  0x71, 0x1c, 0x47, 0x11, 0xc4, 0x00, \
  0x0e, 0x03, 0x80, 0xe0, 0x38, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x45, 0x51, 0x54, 0x55, 0x15, 0x00, \
  0x88, 0xa2, 0x28, 0x8a, 0x22, 0x00, \
  0xe0, 0x38, 0x0e, 0x03, 0x80, 0x00

#define kMaskRandom40_12 \
  0x0e, 0x03, 0x80, 0xe0, 0x38, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x45, 0x51, 0x54, 0x55, 0x15, 0x00, \
  0x88, 0xa2, 0x28, 0x8a, 0x22, 0x00, \
  0xe0, 0x38, 0x0e, 0x03, 0x80, 0x00, \
  0xc6, 0x31, 0x8c, 0x63, 0x18, 0x00, \
  0x23, 0x88, 0xe2, 0x38, 0x8e, 0x00, \
  0x1a, 0x46, 0x91, 0xa4, 0x69, 0x00, \
  0x24, 0xc9, 0x32, 0x4c, 0x93, 0x00, \
  0x71, 0x1c, 0x47, 0x11, 0xc4, 0x00, \
  0xf5, 0xdc, 0x4f, 0x5d, 0xc4, 0x00

#define kMaskRandom40_13 \
  0x0e, 0x03, 0x80, 0xe0, 0x38, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x45, 0x51, 0x54, 0x55, 0x15, 0x00, \
  0x88, 0xa2, 0x28, 0x8a, 0x22, 0x00, \
  0xe0, 0x38, 0x0e, 0x03, 0x80, 0x00, \
  0x46, 0x11, 0x84, 0x61, 0x18, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x0c, 0x43, 0x10, 0xc4, 0x31, 0x00, \
  0x28, 0x8a, 0x22, 0x88, 0xa2, 0x00, \
  0x94, 0x25, 0x09, 0x42, 0x50, 0x00, \
  0xc1, 0x30, 0x4c, 0x13, 0x04, 0x00

#define kMaskRandom40_14 \
  0x46, 0x11, 0x84, 0x61, 0x18, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x0c, 0x43, 0x10, 0xc4, 0x31, 0x00, \
  0x28, 0x8a, 0x22, 0x88, 0xa2, 0x00, \
  0x94, 0x25, 0x09, 0x42, 0x50, 0x00, \
  0xc1, 0x30, 0x4c, 0x13, 0x04, 0x00, \
  0x0e, 0x03, 0x80, 0xe0, 0x38, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x45, 0x51, 0x54, 0x55, 0x15, 0x00, \
  0x88, 0xa2, 0x28, 0x8a, 0x22, 0x00, \
  0xe0, 0x38, 0x0e, 0x03, 0x80, 0x00, \
  0x56, 0x3e, 0x25, 0x63, 0xe2, 0x00

#define kMaskRandom40_15 \
  0x46, 0x11, 0x84, 0x61, 0x18, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x0c, 0x43, 0x10, 0xc4, 0x31, 0x00, \
  0x28, 0x8a, 0x22, 0x88, 0xa2, 0x00, \
  0x94, 0x25, 0x09, 0x42, 0x50, 0x00, \
  0xc1, 0x30, 0x4c, 0x13, 0x04, 0x00, \
  0x2c, 0x0b, 0x02, 0xc0, 0xb0, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x06, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0x81, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0xc2, 0x30, 0x8c, 0x23, 0x08, 0x00, \
  0x22, 0x88, 0xa2, 0x28, 0x8a, 0x00, \
  0x50, 0x54, 0x15, 0x05, 0x41, 0x00

#define kMaskRandom40_16 \
  0x2c, 0x0b, 0x02, 0xc0, 0xb0, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x06, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0x81, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0xc2, 0x30, 0x8c, 0x23, 0x08, 0x00, \
  0x22, 0x88, 0xa2, 0x28, 0x8a, 0x00, \
  0x50, 0x54, 0x15, 0x05, 0x41, 0x00, \
  0x46, 0x11, 0x84, 0x61, 0x18, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x0c, 0x43, 0x10, 0xc4, 0x31, 0x00, \
  0x28, 0x8a, 0x22, 0x88, 0xa2, 0x00, \
  0x94, 0x25, 0x09, 0x42, 0x50, 0x00, \
  0xc1, 0x30, 0x4c, 0x13, 0x04, 0x00, \
  0x28, 0x1c, 0x62, 0x81, 0xc6, 0x00

#define kMaskRandom40_17 \
  0x2c, 0x0b, 0x02, 0xc0, 0xb0, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x06, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0x81, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0xc2, 0x30, 0x8c, 0x23, 0x08, 0x00, \
  0x22, 0x88, 0xa2, 0x28, 0x8a, 0x00, \
  0x50, 0x54, 0x15, 0x05, 0x41, 0x00, \
  0x4e, 0x13, 0x84, 0xe1, 0x38, 0x00, \
  0xe3, 0x38, 0xce, 0x33, 0x8c, 0x00, \
  0x81, 0xe0, 0x78, 0x1e, 0x07, 0x00, \
  0x21, 0x48, 0x52, 0x14, 0x85, 0x00, \
  0x52, 0x94, 0xa5, 0x29, 0x4a, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0xd0, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x9a, 0x00, \
  0x58, 0x56, 0x15, 0x85, 0x61, 0x00, \
  0x19, 0x86, 0x61, 0x98, 0x66, 0x00

#define kMaskRandom40_18 \
  0x4e, 0x13, 0x84, 0xe1, 0x38, 0x00, \
  0xe3, 0x38, 0xce, 0x33, 0x8c, 0x00, \
  0x81, 0xe0, 0x78, 0x1e, 0x07, 0x00, \
  0x21, 0x48, 0x52, 0x14, 0x85, 0x00, \
  0x52, 0x94, 0xa5, 0x29, 0x4a, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0xd0, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x9a, 0x00, \
  0x58, 0x56, 0x15, 0x85, 0x61, 0x00, \
  0x19, 0x86, 0x61, 0x98, 0x66, 0x00, \
  0x2c, 0x0b, 0x02, 0xc0, 0xb0, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x06, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0x81, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0xc2, 0x30, 0x8c, 0x23, 0x08, 0x00, \
  0x22, 0x88, 0xa2, 0x28, 0x8a, 0x00, \
  0x50, 0x54, 0x15, 0x05, 0x41, 0x00, \
  0x21, 0x7b, 0xf2, 0x17, 0xbf, 0x00

#define kMaskRandom40_19 \
  0x4e, 0x13, 0x84, 0xe1, 0x38, 0x00, \
  0xe3, 0x38, 0xce, 0x33, 0x8c, 0x00, \
  0x81, 0xe0, 0x78, 0x1e, 0x07, 0x00, \
  0x21, 0x48, 0x52, 0x14, 0x85, 0x00, \
  0x52, 0x94, 0xa5, 0x29, 0x4a, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0xd0, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x9a, 0x00, \
  0x58, 0x56, 0x15, 0x85, 0x61, 0x00, \
  0x19, 0x86, 0x61, 0x98, 0x66, 0x00, \
  0x4c, 0x13, 0x04, 0xc1, 0x30, 0x00, \
  0x51, 0x14, 0x45, 0x11, 0x44, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0x81, 0x00, \
  0x04, 0xc1, 0x30, 0x4c, 0x13, 0x00, \
  0x03, 0x80, 0xe0, 0x38, 0x0e, 0x00, \
  0x86, 0x21, 0x88, 0x62, 0x18, 0x00, \
  0x29, 0x0a, 0x42, 0x90, 0xa4, 0x00, \
  0x42, 0x50, 0x94, 0x25, 0x09, 0x00, \
  0x98, 0x26, 0x09, 0x82, 0x60, 0x00, \
  0x30, 0x8c, 0x23, 0x08, 0xc2, 0x00

#define kMaskRandom40_2 \
  0xee, 0x3b, 0x8e, 0xe3, 0xb8, 0x00, \
  0x99, 0xe6, 0x79, 0x9e, 0x67, 0x00

#define kMaskRandom40_20 \
  0x4c, 0x13, 0x04, 0xc1, 0x30, 0x00, \
  0x51, 0x14, 0x45, 0x11, 0x44, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0x81, 0x00, \
  0x04, 0xc1, 0x30, 0x4c, 0x13, 0x00, \
  0x03, 0x80, 0xe0, 0x38, 0x0e, 0x00, \
  0x86, 0x21, 0x88, 0x62, 0x18, 0x00, \
  0x29, 0x0a, 0x42, 0x90, 0xa4, 0x00, \
  0x42, 0x50, 0x94, 0x25, 0x09, 0x00, \
  0x98, 0x26, 0x09, 0x82, 0x60, 0x00, \
  0x30, 0x8c, 0x23, 0x08, 0xc2, 0x00, \
  0x4e, 0x13, 0x84, 0xe1, 0x38, 0x00, \
  0xe3, 0x38, 0xce, 0x33, 0x8c, 0x00, \
  0x81, 0xe0, 0x78, 0x1e, 0x07, 0x00, \
  0x21, 0x48, 0x52, 0x14, 0x85, 0x00, \
  0x52, 0x94, 0xa5, 0x29, 0x4a, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0xd0, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x9a, 0x00, \
  0x58, 0x56, 0x15, 0x85, 0x61, 0x00, \
  0x19, 0x86, 0x61, 0x98, 0x66, 0x00, \
  0xf7, 0x8d, 0xaf, 0x78, 0xda, 0x00

#define kMaskRandom40_21 \
  0x4c, 0x13, 0x04, 0xc1, 0x30, 0x00, \
  0x51, 0x14, 0x45, 0x11, 0x44, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0x81, 0x00, \
  0x04, 0xc1, 0x30, 0x4c, 0x13, 0x00, \
  0x03, 0x80, 0xe0, 0x38, 0x0e, 0x00, \
  0x86, 0x21, 0x88, 0x62, 0x18, 0x00, \
  0x29, 0x0a, 0x42, 0x90, 0xa4, 0x00, \
  0x42, 0x50, 0x94, 0x25, 0x09, 0x00, \
  0x98, 0x26, 0x09, 0x82, 0x60, 0x00, \
  0x30, 0x8c, 0x23, 0x08, 0xc2, 0x00, \
  0xc6, 0x31, 0x8c, 0x63, 0x18, 0x00, \
  0x23, 0x88, 0xe2, 0x38, 0x8e, 0x00, \
  0x1a, 0x46, 0x91, 0xa4, 0x69, 0x00, \
  0x24, 0xc9, 0x32, 0x4c, 0x93, 0x00, \
  0x71, 0x1c, 0x47, 0x11, 0xc4, 0x00, \
  0x0e, 0x03, 0x80, 0xe0, 0x38, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x45, 0x51, 0x54, 0x55, 0x15, 0x00, \
  0x88, 0xa2, 0x28, 0x8a, 0x22, 0x00, \
  0xe0, 0x38, 0x0e, 0x03, 0x80, 0x00

#define kMaskRandom40_22 \
  0xc6, 0x31, 0x8c, 0x63, 0x18, 0x00, \
  0x23, 0x88, 0xe2, 0x38, 0x8e, 0x00, \
  0x1a, 0x46, 0x91, 0xa4, 0x69, 0x00, \
  0x24, 0xc9, 0x32, 0x4c, 0x93, 0x00, \
  0x71, 0x1c, 0x47, 0x11, 0xc4, 0x00, \
  0x0e, 0x03, 0x80, 0xe0, 0x38, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x45, 0x51, 0x54, 0x55, 0x15, 0x00, \
  0x88, 0xa2, 0x28, 0x8a, 0x22, 0x00, \
  0xe0, 0x38, 0x0e, 0x03, 0x80, 0x00, \
  0x4c, 0x13, 0x04, 0xc1, 0x30, 0x00, \
  0x51, 0x14, 0x45, 0x11, 0x44, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0x81, 0x00, \
  0x04, 0xc1, 0x30, 0x4c, 0x13, 0x00, \
  0x03, 0x80, 0xe0, 0x38, 0x0e, 0x00, \
  0x86, 0x21, 0x88, 0x62, 0x18, 0x00, \
  0x29, 0x0a, 0x42, 0x90, 0xa4, 0x00, \
  0x42, 0x50, 0x94, 0x25, 0x09, 0x00, \
  0x98, 0x26, 0x09, 0x82, 0x60, 0x00, \
  0x30, 0x8c, 0x23, 0x08, 0xc2, 0x00, \
  0x89, 0xee, 0x1f, 0x38, 0xca, 0x00

#define kMaskRandom40_23 \
  0xc6, 0x31, 0x8c, 0x63, 0x18, 0x00, \
  0x23, 0x88, 0xe2, 0x38, 0x8e, 0x00, \
  0x1a, 0x46, 0x91, 0xa4, 0x69, 0x00, \
  0x24, 0xc9, 0x32, 0x4c, 0x93, 0x00, \
  0x71, 0x1c, 0x47, 0x11, 0xc4, 0x00, \
  0x0e, 0x03, 0x80, 0xe0, 0x38, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x45, 0x51, 0x54, 0x55, 0x15, 0x00, \
  0x88, 0xa2, 0x28, 0x8a, 0x22, 0x00, \
  0xe0, 0x38, 0x0e, 0x03, 0x80, 0x00, \
  0x0e, 0x03, 0x80, 0xe0, 0x38, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x45, 0x51, 0x54, 0x55, 0x15, 0x00, \
  0x88, 0xa2, 0x28, 0x8a, 0x22, 0x00, \
  0xe0, 0x38, 0x0e, 0x03, 0x80, 0x00, \
  0xc6, 0x31, 0x8c, 0x63, 0x18, 0x00, \
  0x23, 0x88, 0xe2, 0x38, 0x8e, 0x00, \
  0x1a, 0x46, 0x91, 0xa4, 0x69, 0x00, \
  0x24, 0xc9, 0x32, 0x4c, 0x93, 0x00, \
  0x71, 0x1c, 0x47, 0x11, 0xc4, 0x00, \
  0xf5, 0xdc, 0x4f, 0x5d, 0xc4, 0x00

#define kMaskRandom40_24 \
  0x0e, 0x03, 0x80, 0xe0, 0x38, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x45, 0x51, 0x54, 0x55, 0x15, 0x00, \
  0x88, 0xa2, 0x28, 0x8a, 0x22, 0x00, \
  0xe0, 0x38, 0x0e, 0x03, 0x80, 0x00, \
  0xc6, 0x31, 0x8c, 0x63, 0x18, 0x00, \
  0x23, 0x88, 0xe2, 0x38, 0x8e, 0x00, \
  0x1a, 0x46, 0x91, 0xa4, 0x69, 0x00, \
  0x24, 0xc9, 0x32, 0x4c, 0x93, 0x00, \
  0x71, 0x1c, 0x47, 0x11, 0xc4, 0x00, \
  0xf5, 0xdc, 0x4f, 0x5d, 0xc4, 0x00, \
  0xc6, 0x31, 0x8c, 0x63, 0x18, 0x00, \
  0x23, 0x88, 0xe2, 0x38, 0x8e, 0x00, \
  0x1a, 0x46, 0x91, 0xa4, 0x69, 0x00, \
  0x24, 0xc9, 0x32, 0x4c, 0x93, 0x00, \
  0x71, 0x1c, 0x47, 0x11, 0xc4, 0x00, \
  0x0e, 0x03, 0x80, 0xe0, 0x38, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x45, 0x51, 0x54, 0x55, 0x15, 0x00, \
  0x88, 0xa2, 0x28, 0x8a, 0x22, 0x00, \
  0xe0, 0x38, 0x0e, 0x03, 0x80, 0x00, \
  0x68, 0xde, 0x83, 0xa9, 0xcf, 0x00

#define kMaskRandom40_25 \
  0x0e, 0x03, 0x80, 0xe0, 0x38, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x45, 0x51, 0x54, 0x55, 0x15, 0x00, \
  0x88, 0xa2, 0x28, 0x8a, 0x22, 0x00, \
  0xe0, 0x38, 0x0e, 0x03, 0x80, 0x00, \
  0xc6, 0x31, 0x8c, 0x63, 0x18, 0x00, \
  0x23, 0x88, 0xe2, 0x38, 0x8e, 0x00, \
  0x1a, 0x46, 0x91, 0xa4, 0x69, 0x00, \
  0x24, 0xc9, 0x32, 0x4c, 0x93, 0x00, \
  0x71, 0x1c, 0x47, 0x11, 0xc4, 0x00, \
  0xf5, 0xdc, 0x4f, 0x5d, 0xc4, 0x00, \
  0x0e, 0x03, 0x80, 0xe0, 0x38, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x45, 0x51, 0x54, 0x55, 0x15, 0x00, \
  0x88, 0xa2, 0x28, 0x8a, 0x22, 0x00, \
  0xe0, 0x38, 0x0e, 0x03, 0x80, 0x00, \
  0x46, 0x11, 0x84, 0x61, 0x18, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x0c, 0x43, 0x10, 0xc4, 0x31, 0x00, \
  0x28, 0x8a, 0x22, 0x88, 0xa2, 0x00, \
  0x94, 0x25, 0x09, 0x42, 0x50, 0x00, \
  0xc1, 0x30, 0x4c, 0x13, 0x04, 0x00

#define kMaskRandom40_26 \
  0x0e, 0x03, 0x80, 0xe0, 0x38, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x45, 0x51, 0x54, 0x55, 0x15, 0x00, \
  0x88, 0xa2, 0x28, 0x8a, 0x22, 0x00, \
  0xe0, 0x38, 0x0e, 0x03, 0x80, 0x00, \
  0x46, 0x11, 0x84, 0x61, 0x18, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x0c, 0x43, 0x10, 0xc4, 0x31, 0x00, \
  0x28, 0x8a, 0x22, 0x88, 0xa2, 0x00, \
  0x94, 0x25, 0x09, 0x42, 0x50, 0x00, \
  0xc1, 0x30, 0x4c, 0x13, 0x04, 0x00, \
  0x0e, 0x03, 0x80, 0xe0, 0x38, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x45, 0x51, 0x54, 0x55, 0x15, 0x00, \
  0x88, 0xa2, 0x28, 0x8a, 0x22, 0x00, \
  0xe0, 0x38, 0x0e, 0x03, 0x80, 0x00, \
  0xc6, 0x31, 0x8c, 0x63, 0x18, 0x00, \
  0x23, 0x88, 0xe2, 0x38, 0x8e, 0x00, \
  0x1a, 0x46, 0x91, 0xa4, 0x69, 0x00, \
  0x24, 0xc9, 0x32, 0x4c, 0x93, 0x00, \
  0x71, 0x1c, 0x47, 0x11, 0xc4, 0x00, \
  0xf5, 0xdc, 0x4f, 0x5d, 0xc4, 0x00, \
  0x06, 0x8e, 0x8c, 0x1a, 0xd2, 0x00

#define kMaskRandom40_27 \
  0x0e, 0x03, 0x80, 0xe0, 0x38, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x45, 0x51, 0x54, 0x55, 0x15, 0x00, \
  0x88, 0xa2, 0x28, 0x8a, 0x22, 0x00, \
  0xe0, 0x38, 0x0e, 0x03, 0x80, 0x00, \
  0x46, 0x11, 0x84, 0x61, 0x18, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x0c, 0x43, 0x10, 0xc4, 0x31, 0x00, \
  0x28, 0x8a, 0x22, 0x88, 0xa2, 0x00, \
  0x94, 0x25, 0x09, 0x42, 0x50, 0x00, \
  0xc1, 0x30, 0x4c, 0x13, 0x04, 0x00, \
  0x46, 0x11, 0x84, 0x61, 0x18, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x0c, 0x43, 0x10, 0xc4, 0x31, 0x00, \
  0x28, 0x8a, 0x22, 0x88, 0xa2, 0x00, \
  0x94, 0x25, 0x09, 0x42, 0x50, 0x00, \
  0xc1, 0x30, 0x4c, 0x13, 0x04, 0x00, \
  0x0e, 0x03, 0x80, 0xe0, 0x38, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x45, 0x51, 0x54, 0x55, 0x15, 0x00, \
  0x88, 0xa2, 0x28, 0x8a, 0x22, 0x00, \
  0xe0, 0x38, 0x0e, 0x03, 0x80, 0x00, \
  0x56, 0x3e, 0x25, 0x63, 0xe2, 0x00

#define kMaskRandom40_28 \
  0x46, 0x11, 0x84, 0x61, 0x18, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x0c, 0x43, 0x10, 0xc4, 0x31, 0x00, \
  0x28, 0x8a, 0x22, 0x88, 0xa2, 0x00, \
  0x94, 0x25, 0x09, 0x42, 0x50, 0x00, \
  0xc1, 0x30, 0x4c, 0x13, 0x04, 0x00, \
  0x0e, 0x03, 0x80, 0xe0, 0x38, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x45, 0x51, 0x54, 0x55, 0x15, 0x00, \
  0x88, 0xa2, 0x28, 0x8a, 0x22, 0x00, \
  0xe0, 0x38, 0x0e, 0x03, 0x80, 0x00, \
  0x56, 0x3e, 0x25, 0x63, 0xe2, 0x00, \
  0x0e, 0x03, 0x80, 0xe0, 0x38, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x45, 0x51, 0x54, 0x55, 0x15, 0x00, \
  0x88, 0xa2, 0x28, 0x8a, 0x22, 0x00, \
  0xe0, 0x38, 0x0e, 0x03, 0x80, 0x00, \
  0x46, 0x11, 0x84, 0x61, 0x18, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x0c, 0x43, 0x10, 0xc4, 0x31, 0x00, \
  0x28, 0x8a, 0x22, 0x88, 0xa2, 0x00, \
  0x94, 0x25, 0x09, 0x42, 0x50, 0x00, \
  0xc1, 0x30, 0x4c, 0x13, 0x04, 0x00, \
  0x68, 0x0e, 0x9b, 0x52, 0xb6, 0x00

#define kMaskRandom40_29 \
  0x46, 0x11, 0x84, 0x61, 0x18, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x0c, 0x43, 0x10, 0xc4, 0x31, 0x00, \
  0x28, 0x8a, 0x22, 0x88, 0xa2, 0x00, \
  0x94, 0x25, 0x09, 0x42, 0x50, 0x00, \
  0xc1, 0x30, 0x4c, 0x13, 0x04, 0x00, \
  0x0e, 0x03, 0x80, 0xe0, 0x38, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x45, 0x51, 0x54, 0x55, 0x15, 0x00, \
  0x88, 0xa2, 0x28, 0x8a, 0x22, 0x00, \
  0xe0, 0x38, 0x0e, 0x03, 0x80, 0x00, \
  0x56, 0x3e, 0x25, 0x63, 0xe2, 0x00, \
  0x46, 0x11, 0x84, 0x61, 0x18, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x0c, 0x43, 0x10, 0xc4, 0x31, 0x00, \
  0x28, 0x8a, 0x22, 0x88, 0xa2, 0x00, \
  0x94, 0x25, 0x09, 0x42, 0x50, 0x00, \
  0xc1, 0x30, 0x4c, 0x13, 0x04, 0x00, \
  0x2c, 0x0b, 0x02, 0xc0, 0xb0, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x06, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0x81, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0xc2, 0x30, 0x8c, 0x23, 0x08, 0x00, \
  0x22, 0x88, 0xa2, 0x28, 0x8a, 0x00, \
  0x50, 0x54, 0x15, 0x05, 0x41, 0x00

#define kMaskRandom40_3 \
  0xce, 0x33, 0x8c, 0xe3, 0x38, 0x00, \
  0x55, 0x95, 0x65, 0x59, 0x56, 0x00, \
  0xb1, 0x6a, 0x3b, 0x16, 0xa3, 0x00

#define kMaskRandom40_30 \
  0x46, 0x11, 0x84, 0x61, 0x18, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x0c, 0x43, 0x10, 0xc4, 0x31, 0x00, \
  0x28, 0x8a, 0x22, 0x88, 0xa2, 0x00, \
  0x94, 0x25, 0x09, 0x42, 0x50, 0x00, \
  0xc1, 0x30, 0x4c, 0x13, 0x04, 0x00, \
  0x2c, 0x0b, 0x02, 0xc0, 0xb0, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x06, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0x81, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0xc2, 0x30, 0x8c, 0x23, 0x08, 0x00, \
  0x22, 0x88, 0xa2, 0x28, 0x8a, 0x00, \
  0x50, 0x54, 0x15, 0x05, 0x41, 0x00, \
  0x46, 0x11, 0x84, 0x61, 0x18, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x0c, 0x43, 0x10, 0xc4, 0x31, 0x00, \
  0x28, 0x8a, 0x22, 0x88, 0xa2, 0x00, \
  0x94, 0x25, 0x09, 0x42, 0x50, 0x00, \
  0xc1, 0x30, 0x4c, 0x13, 0x04, 0x00, \
  0x0e, 0x03, 0x80, 0xe0, 0x38, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x45, 0x51, 0x54, 0x55, 0x15, 0x00, \
  0x88, 0xa2, 0x28, 0x8a, 0x22, 0x00, \
  0xe0, 0x38, 0x0e, 0x03, 0x80, 0x00, \
  0x56, 0x3e, 0x25, 0x63, 0xe2, 0x00, \
  0xe1, 0x47, 0x04, 0x05, 0x47, 0x00

#define kMaskRandom40_31 \
  0x46, 0x11, 0x84, 0x61, 0x18, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x0c, 0x43, 0x10, 0xc4, 0x31, 0x00, \
  0x28, 0x8a, 0x22, 0x88, 0xa2, 0x00, \
  0x94, 0x25, 0x09, 0x42, 0x50, 0x00, \
  0xc1, 0x30, 0x4c, 0x13, 0x04, 0x00, \
  0x2c, 0x0b, 0x02, 0xc0, 0xb0, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x06, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0x81, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0xc2, 0x30, 0x8c, 0x23, 0x08, 0x00, \
  0x22, 0x88, 0xa2, 0x28, 0x8a, 0x00, \
  0x50, 0x54, 0x15, 0x05, 0x41, 0x00, \
  0x2c, 0x0b, 0x02, 0xc0, 0xb0, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x06, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0x81, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0xc2, 0x30, 0x8c, 0x23, 0x08, 0x00, \
  0x22, 0x88, 0xa2, 0x28, 0x8a, 0x00, \
  0x50, 0x54, 0x15, 0x05, 0x41, 0x00, \
  0x46, 0x11, 0x84, 0x61, 0x18, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x0c, 0x43, 0x10, 0xc4, 0x31, 0x00, \
  0x28, 0x8a, 0x22, 0x88, 0xa2, 0x00, \
  0x94, 0x25, 0x09, 0x42, 0x50, 0x00, \
  0xc1, 0x30, 0x4c, 0x13, 0x04, 0x00, \
  0x28, 0x1c, 0x62, 0x81, 0xc6, 0x00

#define kMaskRandom40_32 \
  0x2c, 0x0b, 0x02, 0xc0, 0xb0, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x06, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0x81, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0xc2, 0x30, 0x8c, 0x23, 0x08, 0x00, \
  0x22, 0x88, 0xa2, 0x28, 0x8a, 0x00, \
  0x50, 0x54, 0x15, 0x05, 0x41, 0x00, \
  0x46, 0x11, 0x84, 0x61, 0x18, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x0c, 0x43, 0x10, 0xc4, 0x31, 0x00, \
  0x28, 0x8a, 0x22, 0x88, 0xa2, 0x00, \
  0x94, 0x25, 0x09, 0x42, 0x50, 0x00, \
  0xc1, 0x30, 0x4c, 0x13, 0x04, 0x00, \
  0x28, 0x1c, 0x62, 0x81, 0xc6, 0x00, \
  0x46, 0x11, 0x84, 0x61, 0x18, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x0c, 0x43, 0x10, 0xc4, 0x31, 0x00, \
  0x28, 0x8a, 0x22, 0x88, 0xa2, 0x00, \
  0x94, 0x25, 0x09, 0x42, 0x50, 0x00, \
  0xc1, 0x30, 0x4c, 0x13, 0x04, 0x00, \
  0x2c, 0x0b, 0x02, 0xc0, 0xb0, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x06, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0x81, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0xc2, 0x30, 0x8c, 0x23, 0x08, 0x00, \
  0x22, 0x88, 0xa2, 0x28, 0x8a, 0x00, \
  0x50, 0x54, 0x15, 0x05, 0x41, 0x00, \
  0x03, 0x0c, 0x46, 0x10, 0xc5, 0x00

#define kMaskRandom40_33 \
  0x2c, 0x0b, 0x02, 0xc0, 0xb0, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x06, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0x81, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0xc2, 0x30, 0x8c, 0x23, 0x08, 0x00, \
  0x22, 0x88, 0xa2, 0x28, 0x8a, 0x00, \
  0x50, 0x54, 0x15, 0x05, 0x41, 0x00, \
  0x46, 0x11, 0x84, 0x61, 0x18, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x0c, 0x43, 0x10, 0xc4, 0x31, 0x00, \
  0x28, 0x8a, 0x22, 0x88, 0xa2, 0x00, \
  0x94, 0x25, 0x09, 0x42, 0x50, 0x00, \
  0xc1, 0x30, 0x4c, 0x13, 0x04, 0x00, \
  0x28, 0x1c, 0x62, 0x81, 0xc6, 0x00, \
  0x2c, 0x0b, 0x02, 0xc0, 0xb0, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x06, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0x81, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0xc2, 0x30, 0x8c, 0x23, 0x08, 0x00, \
  0x22, 0x88, 0xa2, 0x28, 0x8a, 0x00, \
  0x50, 0x54, 0x15, 0x05, 0x41, 0x00, \
  0x4e, 0x13, 0x84, 0xe1, 0x38, 0x00, \
  0xe3, 0x38, 0xce, 0x33, 0x8c, 0x00, \
  0x81, 0xe0, 0x78, 0x1e, 0x07, 0x00, \
  0x21, 0x48, 0x52, 0x14, 0x85, 0x00, \
  0x52, 0x94, 0xa5, 0x29, 0x4a, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0xd0, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x9a, 0x00, \
  0x58, 0x56, 0x15, 0x85, 0x61, 0x00, \
  0x19, 0x86, 0x61, 0x98, 0x66, 0x00

#define kMaskRandom40_34 \
  0x2c, 0x0b, 0x02, 0xc0, 0xb0, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x06, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0x81, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0xc2, 0x30, 0x8c, 0x23, 0x08, 0x00, \
  0x22, 0x88, 0xa2, 0x28, 0x8a, 0x00, \
  0x50, 0x54, 0x15, 0x05, 0x41, 0x00, \
  0x4e, 0x13, 0x84, 0xe1, 0x38, 0x00, \
  0xe3, 0x38, 0xce, 0x33, 0x8c, 0x00, \
  0x81, 0xe0, 0x78, 0x1e, 0x07, 0x00, \
  0x21, 0x48, 0x52, 0x14, 0x85, 0x00, \
  0x52, 0x94, 0xa5, 0x29, 0x4a, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0xd0, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x9a, 0x00, \
  0x58, 0x56, 0x15, 0x85, 0x61, 0x00, \
  0x19, 0x86, 0x61, 0x98, 0x66, 0x00, \
  0x2c, 0x0b, 0x02, 0xc0, 0xb0, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x06, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0x81, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0xc2, 0x30, 0x8c, 0x23, 0x08, 0x00, \
  0x22, 0x88, 0xa2, 0x28, 0x8a, 0x00, \
  0x50, 0x54, 0x15, 0x05, 0x41, 0x00, \
  0x46, 0x11, 0x84, 0x61, 0x18, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x43, 0x00, \
  0x0c, 0x43, 0x10, 0xc4, 0x31, 0x00, \
  0x28, 0x8a, 0x22, 0x88, 0xa2, 0x00, \
  0x94, 0x25, 0x09, 0x42, 0x50, 0x00, \
  0xc1, 0x30, 0x4c, 0x13, 0x04, 0x00, \
  0x28, 0x1c, 0x62, 0x81, 0xc6, 0x00, \
  0x87, 0x3c, 0x08, 0x19, 0x31, 0x00

#define kMaskRandom40_35 \
  0x2c, 0x0b, 0x02, 0xc0, 0xb0, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x06, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0x81, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0xc2, 0x30, 0x8c, 0x23, 0x08, 0x00, \
  0x22, 0x88, 0xa2, 0x28, 0x8a, 0x00, \
  0x50, 0x54, 0x15, 0x05, 0x41, 0x00, \
  0x4e, 0x13, 0x84, 0xe1, 0x38, 0x00, \
  0xe3, 0x38, 0xce, 0x33, 0x8c, 0x00, \
  0x81, 0xe0, 0x78, 0x1e, 0x07, 0x00, \
  0x21, 0x48, 0x52, 0x14, 0x85, 0x00, \
  0x52, 0x94, 0xa5, 0x29, 0x4a, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0xd0, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x9a, 0x00, \
  0x58, 0x56, 0x15, 0x85, 0x61, 0x00, \
  0x19, 0x86, 0x61, 0x98, 0x66, 0x00, \
  0x4e, 0x13, 0x84, 0xe1, 0x38, 0x00, \
  0xe3, 0x38, 0xce, 0x33, 0x8c, 0x00, \
  0x81, 0xe0, 0x78, 0x1e, 0x07, 0x00, \
  0x21, 0x48, 0x52, 0x14, 0x85, 0x00, \
  0x52, 0x94, 0xa5, 0x29, 0x4a, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0xd0, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x9a, 0x00, \
  0x58, 0x56, 0x15, 0x85, 0x61, 0x00, \
  0x19, 0x86, 0x61, 0x98, 0x66, 0x00, \
  0x2c, 0x0b, 0x02, 0xc0, 0xb0, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x06, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0x81, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0xc2, 0x30, 0x8c, 0x23, 0x08, 0x00, \
  0x22, 0x88, 0xa2, 0x28, 0x8a, 0x00, \
  0x50, 0x54, 0x15, 0x05, 0x41, 0x00, \
  0x21, 0x7b, 0xf2, 0x17, 0xbf, 0x00

#define kMaskRandom40_36 \
  0x4e, 0x13, 0x84, 0xe1, 0x38, 0x00, \
  0xe3, 0x38, 0xce, 0x33, 0x8c, 0x00, \
  0x81, 0xe0, 0x78, 0x1e, 0x07, 0x00, \
  0x21, 0x48, 0x52, 0x14, 0x85, 0x00, \
  0x52, 0x94, 0xa5, 0x29, 0x4a, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0xd0, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x9a, 0x00, \
  0x58, 0x56, 0x15, 0x85, 0x61, 0x00, \
  0x19, 0x86, 0x61, 0x98, 0x66, 0x00, \
  0x2c, 0x0b, 0x02, 0xc0, 0xb0, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x06, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0x81, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0xc2, 0x30, 0x8c, 0x23, 0x08, 0x00, \
  0x22, 0x88, 0xa2, 0x28, 0x8a, 0x00, \
  0x50, 0x54, 0x15, 0x05, 0x41, 0x00, \
  0x21, 0x7b, 0xf2, 0x17, 0xbf, 0x00, \
  0x2c, 0x0b, 0x02, 0xc0, 0xb0, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x06, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0x81, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0xc2, 0x30, 0x8c, 0x23, 0x08, 0x00, \
  0x22, 0x88, 0xa2, 0x28, 0x8a, 0x00, \
  0x50, 0x54, 0x15, 0x05, 0x41, 0x00, \
  0x4e, 0x13, 0x84, 0xe1, 0x38, 0x00, \
  0xe3, 0x38, 0xce, 0x33, 0x8c, 0x00, \
  0x81, 0xe0, 0x78, 0x1e, 0x07, 0x00, \
  0x21, 0x48, 0x52, 0x14, 0x85, 0x00, \
  0x52, 0x94, 0xa5, 0x29, 0x4a, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0xd0, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x9a, 0x00, \
  0x58, 0x56, 0x15, 0x85, 0x61, 0x00, \
  0x19, 0x86, 0x61, 0x98, 0x66, 0x00, \
  0x1e, 0xb9, 0x3d, 0x25, 0xcc, 0x00

#define kMaskRandom40_37 \
  0x4e, 0x13, 0x84, 0xe1, 0x38, 0x00, \
  0xe3, 0x38, 0xce, 0x33, 0x8c, 0x00, \
  0x81, 0xe0, 0x78, 0x1e, 0x07, 0x00, \
  0x21, 0x48, 0x52, 0x14, 0x85, 0x00, \
  0x52, 0x94, 0xa5, 0x29, 0x4a, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0xd0, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x9a, 0x00, \
  0x58, 0x56, 0x15, 0x85, 0x61, 0x00, \
  0x19, 0x86, 0x61, 0x98, 0x66, 0x00, \
  0x2c, 0x0b, 0x02, 0xc0, 0xb0, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x06, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0x81, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0xc2, 0x30, 0x8c, 0x23, 0x08, 0x00, \
  0x22, 0x88, 0xa2, 0x28, 0x8a, 0x00, \
  0x50, 0x54, 0x15, 0x05, 0x41, 0x00, \
  0x21, 0x7b, 0xf2, 0x17, 0xbf, 0x00, \
  0x4e, 0x13, 0x84, 0xe1, 0x38, 0x00, \
  0xe3, 0x38, 0xce, 0x33, 0x8c, 0x00, \
  0x81, 0xe0, 0x78, 0x1e, 0x07, 0x00, \
  0x21, 0x48, 0x52, 0x14, 0x85, 0x00, \
  0x52, 0x94, 0xa5, 0x29, 0x4a, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0xd0, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x9a, 0x00, \
  0x58, 0x56, 0x15, 0x85, 0x61, 0x00, \
  0x19, 0x86, 0x61, 0x98, 0x66, 0x00, \
  0x4c, 0x13, 0x04, 0xc1, 0x30, 0x00, \
  0x51, 0x14, 0x45, 0x11, 0x44, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0x81, 0x00, \
  0x04, 0xc1, 0x30, 0x4c, 0x13, 0x00, \
  0x03, 0x80, 0xe0, 0x38, 0x0e, 0x00, \
  0x86, 0x21, 0x88, 0x62, 0x18, 0x00, \
  0x29, 0x0a, 0x42, 0x90, 0xa4, 0x00, \
  0x42, 0x50, 0x94, 0x25, 0x09, 0x00, \
  0x98, 0x26, 0x09, 0x82, 0x60, 0x00, \
  0x30, 0x8c, 0x23, 0x08, 0xc2, 0x00

#define kMaskRandom40_38 \
  0x4e, 0x13, 0x84, 0xe1, 0x38, 0x00, \
  0xe3, 0x38, 0xce, 0x33, 0x8c, 0x00, \
  0x81, 0xe0, 0x78, 0x1e, 0x07, 0x00, \
  0x21, 0x48, 0x52, 0x14, 0x85, 0x00, \
  0x52, 0x94, 0xa5, 0x29, 0x4a, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0xd0, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x9a, 0x00, \
  0x58, 0x56, 0x15, 0x85, 0x61, 0x00, \
  0x19, 0x86, 0x61, 0x98, 0x66, 0x00, \
  0x4c, 0x13, 0x04, 0xc1, 0x30, 0x00, \
  0x51, 0x14, 0x45, 0x11, 0x44, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0x81, 0x00, \
  0x04, 0xc1, 0x30, 0x4c, 0x13, 0x00, \
  0x03, 0x80, 0xe0, 0x38, 0x0e, 0x00, \
  0x86, 0x21, 0x88, 0x62, 0x18, 0x00, \
  0x29, 0x0a, 0x42, 0x90, 0xa4, 0x00, \
  0x42, 0x50, 0x94, 0x25, 0x09, 0x00, \
  0x98, 0x26, 0x09, 0x82, 0x60, 0x00, \
  0x30, 0x8c, 0x23, 0x08, 0xc2, 0x00, \
  0x4e, 0x13, 0x84, 0xe1, 0x38, 0x00, \
  0xe3, 0x38, 0xce, 0x33, 0x8c, 0x00, \
  0x81, 0xe0, 0x78, 0x1e, 0x07, 0x00, \
  0x21, 0x48, 0x52, 0x14, 0x85, 0x00, \
  0x52, 0x94, 0xa5, 0x29, 0x4a, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0xd0, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x9a, 0x00, \
  0x58, 0x56, 0x15, 0x85, 0x61, 0x00, \
  0x19, 0x86, 0x61, 0x98, 0x66, 0x00, \
  0x2c, 0x0b, 0x02, 0xc0, 0xb0, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x06, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0x81, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x88, 0x62, 0x00, \
  0xc2, 0x30, 0x8c, 0x23, 0x08, 0x00, \
  0x22, 0x88, 0xa2, 0x28, 0x8a, 0x00, \
  0x50, 0x54, 0x15, 0x05, 0x41, 0x00, \
  0x21, 0x7b, 0xf2, 0x17, 0xbf, 0x00, \
  0xea, 0xaa, 0x20, 0xa2, 0x1b, 0x00

#define kMaskRandom40_39 \
  0x4e, 0x13, 0x84, 0xe1, 0x38, 0x00, \
  0xe3, 0x38, 0xce, 0x33, 0x8c, 0x00, \
  0x81, 0xe0, 0x78, 0x1e, 0x07, 0x00, \
  0x21, 0x48, 0x52, 0x14, 0x85, 0x00, \
  0x52, 0x94, 0xa5, 0x29, 0x4a, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0xd0, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x9a, 0x00, \
  0x58, 0x56, 0x15, 0x85, 0x61, 0x00, \
  0x19, 0x86, 0x61, 0x98, 0x66, 0x00, \
  0x4c, 0x13, 0x04, 0xc1, 0x30, 0x00, \
  0x51, 0x14, 0x45, 0x11, 0x44, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0x81, 0x00, \
  0x04, 0xc1, 0x30, 0x4c, 0x13, 0x00, \
  0x03, 0x80, 0xe0, 0x38, 0x0e, 0x00, \
  0x86, 0x21, 0x88, 0x62, 0x18, 0x00, \
  0x29, 0x0a, 0x42, 0x90, 0xa4, 0x00, \
  0x42, 0x50, 0x94, 0x25, 0x09, 0x00, \
  0x98, 0x26, 0x09, 0x82, 0x60, 0x00, \
  0x30, 0x8c, 0x23, 0x08, 0xc2, 0x00, \
  0x4c, 0x13, 0x04, 0xc1, 0x30, 0x00, \
  0x51, 0x14, 0x45, 0x11, 0x44, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0x81, 0x00, \
  0x04, 0xc1, 0x30, 0x4c, 0x13, 0x00, \
  0x03, 0x80, 0xe0, 0x38, 0x0e, 0x00, \
  0x86, 0x21, 0x88, 0x62, 0x18, 0x00, \
  0x29, 0x0a, 0x42, 0x90, 0xa4, 0x00, \
  0x42, 0x50, 0x94, 0x25, 0x09, 0x00, \
  0x98, 0x26, 0x09, 0x82, 0x60, 0x00, \
  0x30, 0x8c, 0x23, 0x08, 0xc2, 0x00, \
  0x4e, 0x13, 0x84, 0xe1, 0x38, 0x00, \
  0xe3, 0x38, 0xce, 0x33, 0x8c, 0x00, \
  0x81, 0xe0, 0x78, 0x1e, 0x07, 0x00, \
  0x21, 0x48, 0x52, 0x14, 0x85, 0x00, \
  0x52, 0x94, 0xa5, 0x29, 0x4a, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0xd0, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x9a, 0x00, \
  0x58, 0x56, 0x15, 0x85, 0x61, 0x00, \
  0x19, 0x86, 0x61, 0x98, 0x66, 0x00, \
  0xf7, 0x8d, 0xaf, 0x78, 0xda, 0x00

#define kMaskRandom40_4 \
  0xe6, 0x39, 0x8e, 0x63, 0x98, 0x00, \
  0x33, 0x8c, 0xe3, 0x38, 0xce, 0x00, \
  0x98, 0xe6, 0x39, 0x8e, 0x63, 0x00, \
  0x2d, 0x4b, 0x52, 0xd4, 0xb5, 0x00

#define kMaskRandom40_40 \
  0x4c, 0x13, 0x04, 0xc1, 0x30, 0x00, \
  0x51, 0x14, 0x45, 0x11, 0x44, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0x81, 0x00, \
  0x04, 0xc1, 0x30, 0x4c, 0x13, 0x00, \
  0x03, 0x80, 0xe0, 0x38, 0x0e, 0x00, \
  0x86, 0x21, 0x88, 0x62, 0x18, 0x00, \
  0x29, 0x0a, 0x42, 0x90, 0xa4, 0x00, \
  0x42, 0x50, 0x94, 0x25, 0x09, 0x00, \
  0x98, 0x26, 0x09, 0x82, 0x60, 0x00, \
  0x30, 0x8c, 0x23, 0x08, 0xc2, 0x00, \
  0x4e, 0x13, 0x84, 0xe1, 0x38, 0x00, \
  0xe3, 0x38, 0xce, 0x33, 0x8c, 0x00, \
  0x81, 0xe0, 0x78, 0x1e, 0x07, 0x00, \
  0x21, 0x48, 0x52, 0x14, 0x85, 0x00, \
  0x52, 0x94, 0xa5, 0x29, 0x4a, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0xd0, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x9a, 0x00, \
  0x58, 0x56, 0x15, 0x85, 0x61, 0x00, \
  0x19, 0x86, 0x61, 0x98, 0x66, 0x00, \
  0xf7, 0x8d, 0xaf, 0x78, 0xda, 0x00, \
  0x4e, 0x13, 0x84, 0xe1, 0x38, 0x00, \
  0xe3, 0x38, 0xce, 0x33, 0x8c, 0x00, \
  0x81, 0xe0, 0x78, 0x1e, 0x07, 0x00, \
  0x21, 0x48, 0x52, 0x14, 0x85, 0x00, \
  0x52, 0x94, 0xa5, 0x29, 0x4a, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0xd0, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x9a, 0x00, \
  0x58, 0x56, 0x15, 0x85, 0x61, 0x00, \
  0x19, 0x86, 0x61, 0x98, 0x66, 0x00, \
  0x4c, 0x13, 0x04, 0xc1, 0x30, 0x00, \
  0x51, 0x14, 0x45, 0x11, 0x44, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0x81, 0x00, \
  0x04, 0xc1, 0x30, 0x4c, 0x13, 0x00, \
  0x03, 0x80, 0xe0, 0x38, 0x0e, 0x00, \
  0x86, 0x21, 0x88, 0x62, 0x18, 0x00, \
  0x29, 0x0a, 0x42, 0x90, 0xa4, 0x00, \
  0x42, 0x50, 0x94, 0x25, 0x09, 0x00, \
  0x98, 0x26, 0x09, 0x82, 0x60, 0x00, \
  0x30, 0x8c, 0x23, 0x08, 0xc2, 0x00, \
  0xa6, 0xf3, 0xab, 0x1b, 0x87, 0x00

#define kMaskRandom40_5 \
  0xce, 0x33, 0x8c, 0xe3, 0x38, 0x00, \
  0x63, 0x98, 0xe6, 0x39, 0x8e, 0x00, \
  0x98, 0xe5, 0x39, 0x8e, 0x53, 0x00, \
  0x2b, 0x53, 0x52, 0xb5, 0x35, 0x00, \
  0xb4, 0x5c, 0xab, 0x45, 0xca, 0x00

#define kMaskRandom40_6 \
  0x4c, 0x1b, 0x04, 0xc1, 0xb0, 0x00, \
  0x51, 0x34, 0x45, 0x13, 0x44, 0x00, \
  0x20, 0xe8, 0x32, 0x0e, 0x83, 0x00, \
  0x85, 0x41, 0x58, 0x54, 0x15, 0x00, \
  0x06, 0x86, 0xa0, 0x68, 0x6a, 0x00, \
  0x9a, 0x21, 0x89, 0xa2, 0x18, 0x00

#define kMaskRandom40_7 \
  0x4e, 0x11, 0x84, 0xe1, 0x18, 0x00, \
  0x33, 0x2c, 0x03, 0x32, 0xc0, 0x00, \
  0x10, 0x0e, 0xb1, 0x00, 0xeb, 0x00, \
  0x81, 0x51, 0x58, 0x15, 0x15, 0x00, \
  0x24, 0xc4, 0xa2, 0x4c, 0x4a, 0x00, \
  0xd4, 0x23, 0x0d, 0x42, 0x30, 0x00, \
  0x0c, 0xa2, 0x60, 0xca, 0x26, 0x00

#define kMaskRandom40_8 \
  0x27, 0x09, 0xc2, 0x70, 0x9c, 0x00, \
  0x89, 0xa2, 0x68, 0x9a, 0x26, 0x00, \
  0xd0, 0x74, 0x1d, 0x07, 0x41, 0x00, \
  0x24, 0xc9, 0x32, 0x4c, 0x93, 0x00, \
  0xe2, 0x90, 0xae, 0x29, 0x0a, 0x00, \
  0xc6, 0x31, 0x8c, 0x63, 0x18, 0x00, \
  0x31, 0x8c, 0x63, 0x18, 0xc6, 0x00, \
  0x18, 0xc6, 0x31, 0x8c, 0x63, 0x00

#define kMaskRandom40_9 \
  0x4e, 0x13, 0x84, 0xe1, 0x38, 0x00, \
  0x62, 0x38, 0xc6, 0x23, 0x8c, 0x00, \
  0x81, 0xe0, 0x78, 0x1e, 0x07, 0x00, \
  0xe1, 0x48, 0x5e, 0x14, 0x85, 0x00, \
  0x13, 0x94, 0xa1, 0x39, 0x4a, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0xd0, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x9a, 0x00, \
  0x58, 0x56, 0x15, 0x85, 0x61, 0x00, \
  0x49, 0x86, 0x54, 0x98, 0x65, 0x00

#define kMaskRandom41_1 \
  0xff, 0xff, 0xff, 0xff, 0xff, 0x80

#define kMaskRandom41_10 \
  0x4c, 0x13, 0x04, 0xc1, 0x91, 0x00, \
  0x51, 0x14, 0x45, 0x11, 0x45, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0xa4, 0x00, \
  0x04, 0xc1, 0x30, 0x4c, 0x13, 0x00, \
  0x03, 0x80, 0xe0, 0x3b, 0x40, 0x00, \
  0x86, 0x21, 0x88, 0x62, 0x09, 0x00, \
  0x29, 0x0a, 0x42, 0x90, 0x84, 0x80, \
  0x42, 0x50, 0x94, 0x24, 0x30, 0x80, \
  0x98, 0x26, 0x09, 0x81, 0x28, 0x00, \
  0x30, 0x8c, 0x23, 0x08, 0x4a, 0x80

#define kMaskRandom41_11 \
  0xc6, 0x31, 0x8c, 0x62, 0x1a, 0x00, \
  0x23, 0x88, 0xe2, 0x38, 0x8c, 0x80, \
  0x1a, 0x46, 0x91, 0xa4, 0x58, 0x80, \
  0x24, 0xc9, 0x32, 0x4d, 0x30, 0x80, \
  0x71, 0x1c, 0x47, 0x11, 0x07, 0x00, \
  0x0e, 0x03, 0x80, 0xe1, 0x91, 0x00, \
  0x33, 0x0c, 0xc3, 0x31, 0x45, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x32, 0x80, \
  0x45, 0x51, 0x54, 0x56, 0x84, 0x80, \
  0x88, 0xa2, 0x28, 0x88, 0x4a, 0x80, \
  0xe0, 0x38, 0x0e, 0x02, 0x29, 0x00

#define kMaskRandom41_12 \
  0x0e, 0x03, 0x80, 0xe1, 0x91, 0x00, \
  0x33, 0x0c, 0xc3, 0x31, 0x45, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x32, 0x80, \
  0x45, 0x51, 0x54, 0x56, 0x84, 0x80, \
  0x88, 0xa2, 0x28, 0x88, 0x4a, 0x80, \
  0xe0, 0x38, 0x0e, 0x02, 0x29, 0x00, \
  0xc6, 0x31, 0x8c, 0x62, 0x1a, 0x00, \
  0x23, 0x88, 0xe2, 0x38, 0x8c, 0x80, \
  0x1a, 0x46, 0x91, 0xa4, 0x58, 0x80, \
  0x24, 0xc9, 0x32, 0x4d, 0x30, 0x80, \
  0x71, 0x1c, 0x47, 0x11, 0x07, 0x00, \
  0xf5, 0xdc, 0x4a, 0x06, 0x51, 0x80

#define kMaskRandom41_13 \
  0x0e, 0x03, 0x80, 0xe1, 0x91, 0x00, \
  0x33, 0x0c, 0xc3, 0x31, 0x45, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x32, 0x80, \
  0x45, 0x51, 0x54, 0x56, 0x84, 0x80, \
  0x88, 0xa2, 0x28, 0x88, 0x4a, 0x80, \
  0xe0, 0x38, 0x0e, 0x02, 0x29, 0x00, \
  0x46, 0x11, 0x84, 0x61, 0x19, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x80, \
  0x10, 0xc4, 0x31, 0x0e, 0x46, 0x00, \
  0x0c, 0x43, 0x10, 0xc6, 0x90, 0x80, \
  0x28, 0x8a, 0x22, 0x89, 0x42, 0x80, \
  0x94, 0x25, 0x09, 0x42, 0x13, 0x00, \
  0xc1, 0x30, 0x4c, 0x10, 0x25, 0x80

#define kMaskRandom41_14 \
  0x46, 0x11, 0x84, 0x61, 0x19, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x80, \
  0x10, 0xc4, 0x31, 0x0e, 0x46, 0x00, \
  0x0c, 0x43, 0x10, 0xc6, 0x90, 0x80, \
  0x28, 0x8a, 0x22, 0x89, 0x42, 0x80, \
  0x94, 0x25, 0x09, 0x42, 0x13, 0x00, \
  0xc1, 0x30, 0x4c, 0x10, 0x25, 0x80, \
  0x0e, 0x03, 0x80, 0xe1, 0x91, 0x00, \
  0x33, 0x0c, 0xc3, 0x31, 0x45, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x32, 0x80, \
  0x45, 0x51, 0x54, 0x56, 0x84, 0x80, \
  0x88, 0xa2, 0x28, 0x88, 0x4a, 0x80, \
  0xe0, 0x38, 0x0e, 0x02, 0x29, 0x00, \
  0x56, 0x3e, 0x24, 0xdd, 0x0c, 0x00

#define kMaskRandom41_15 \
  0x46, 0x11, 0x84, 0x61, 0x19, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x80, \
  0x10, 0xc4, 0x31, 0x0e, 0x46, 0x00, \
  0x0c, 0x43, 0x10, 0xc6, 0x90, 0x80, \
  0x28, 0x8a, 0x22, 0x89, 0x42, 0x80, \
  0x94, 0x25, 0x09, 0x42, 0x13, 0x00, \
  0xc1, 0x30, 0x4c, 0x10, 0x25, 0x80, \
  0x2c, 0x0b, 0x02, 0xc0, 0x32, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x01, 0x80, \
  0xa0, 0x68, 0x1a, 0x06, 0x82, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x89, 0x0c, 0x00, \
  0xc2, 0x30, 0x8c, 0x20, 0x68, 0x00, \
  0x22, 0x88, 0xa2, 0x29, 0x80, 0x80, \
  0x50, 0x54, 0x15, 0x04, 0x50, 0x80

#define kMaskRandom41_16 \
  0x2c, 0x0b, 0x02, 0xc0, 0x32, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x01, 0x80, \
  0xa0, 0x68, 0x1a, 0x06, 0x82, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x89, 0x0c, 0x00, \
  0xc2, 0x30, 0x8c, 0x20, 0x68, 0x00, \
  0x22, 0x88, 0xa2, 0x29, 0x80, 0x80, \
  0x50, 0x54, 0x15, 0x04, 0x50, 0x80, \
  0x46, 0x11, 0x84, 0x61, 0x19, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x80, \
  0x10, 0xc4, 0x31, 0x0e, 0x46, 0x00, \
  0x0c, 0x43, 0x10, 0xc6, 0x90, 0x80, \
  0x28, 0x8a, 0x22, 0x89, 0x42, 0x80, \
  0x94, 0x25, 0x09, 0x42, 0x13, 0x00, \
  0xc1, 0x30, 0x4c, 0x10, 0x25, 0x80, \
  0x28, 0x1c, 0x63, 0xbf, 0x53, 0x80

#define kMaskRandom41_17 \
  0x2c, 0x0b, 0x02, 0xc0, 0x32, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x01, 0x80, \
  0xa0, 0x68, 0x1a, 0x06, 0x82, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x89, 0x0c, 0x00, \
  0xc2, 0x30, 0x8c, 0x20, 0x68, 0x00, \
  0x22, 0x88, 0xa2, 0x29, 0x80, 0x80, \
  0x50, 0x54, 0x15, 0x04, 0x50, 0x80, \
  0x4e, 0x13, 0x84, 0xe1, 0x19, 0x00, \
  0xe3, 0x38, 0xce, 0x31, 0x89, 0x80, \
  0x81, 0xe0, 0x78, 0x1e, 0x30, 0x00, \
  0x21, 0x48, 0x52, 0x14, 0x05, 0x80, \
  0x52, 0x94, 0xa5, 0x28, 0x1e, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0x82, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x62, 0x80, \
  0x58, 0x56, 0x15, 0x86, 0x44, 0x00, \
  0x19, 0x86, 0x61, 0x99, 0xe0, 0x00

#define kMaskRandom41_18 \
  0x4e, 0x13, 0x84, 0xe1, 0x19, 0x00, \
  0xe3, 0x38, 0xce, 0x31, 0x89, 0x80, \
  0x81, 0xe0, 0x78, 0x1e, 0x30, 0x00, \
  0x21, 0x48, 0x52, 0x14, 0x05, 0x80, \
  0x52, 0x94, 0xa5, 0x28, 0x1e, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0x82, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x62, 0x80, \
  0x58, 0x56, 0x15, 0x86, 0x44, 0x00, \
  0x19, 0x86, 0x61, 0x99, 0xe0, 0x00, \
  0x2c, 0x0b, 0x02, 0xc0, 0x32, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x01, 0x80, \
  0xa0, 0x68, 0x1a, 0x06, 0x82, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x89, 0x0c, 0x00, \
  0xc2, 0x30, 0x8c, 0x20, 0x68, 0x00, \
  0x22, 0x88, 0xa2, 0x29, 0x80, 0x80, \
  0x50, 0x54, 0x15, 0x04, 0x50, 0x80, \
  0x21, 0x7b, 0xf5, 0xa5, 0x65, 0x80

#define kMaskRandom41_19 \
  0x4e, 0x13, 0x84, 0xe1, 0x19, 0x00, \
  0xe3, 0x38, 0xce, 0x31, 0x89, 0x80, \
  0x81, 0xe0, 0x78, 0x1e, 0x30, 0x00, \
  0x21, 0x48, 0x52, 0x14, 0x05, 0x80, \
  0x52, 0x94, 0xa5, 0x28, 0x1e, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0x82, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x62, 0x80, \
  0x58, 0x56, 0x15, 0x86, 0x44, 0x00, \
  0x19, 0x86, 0x61, 0x99, 0xe0, 0x00, \
  0x4c, 0x13, 0x04, 0xc1, 0x91, 0x00, \
  0x51, 0x14, 0x45, 0x11, 0x45, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0xa4, 0x00, \
  0x04, 0xc1, 0x30, 0x4c, 0x13, 0x00, \
  0x03, 0x80, 0xe0, 0x3b, 0x40, 0x00, \
  0x86, 0x21, 0x88, 0x62, 0x09, 0x00, \
  0x29, 0x0a, 0x42, 0x90, 0x84, 0x80, \
  0x42, 0x50, 0x94, 0x24, 0x30, 0x80, \
  0x98, 0x26, 0x09, 0x81, 0x28, 0x00, \
  0x30, 0x8c, 0x23, 0x08, 0x4a, 0x80

#define kMaskRandom41_2 \
  0xee, 0x3b, 0x8e, 0xe3, 0xb3, 0x00, \
  0x99, 0xe6, 0x79, 0x9e, 0x6e, 0x80

#define kMaskRandom41_20 \
  0x4c, 0x13, 0x04, 0xc1, 0x91, 0x00, \
  0x51, 0x14, 0x45, 0x11, 0x45, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0xa4, 0x00, \
  0x04, 0xc1, 0x30, 0x4c, 0x13, 0x00, \
  0x03, 0x80, 0xe0, 0x3b, 0x40, 0x00, \
  0x86, 0x21, 0x88, 0x62, 0x09, 0x00, \
  0x29, 0x0a, 0x42, 0x90, 0x84, 0x80, \
  0x42, 0x50, 0x94, 0x24, 0x30, 0x80, \
  0x98, 0x26, 0x09, 0x81, 0x28, 0x00, \
  0x30, 0x8c, 0x23, 0x08, 0x4a, 0x80, \
  0x4e, 0x13, 0x84, 0xe1, 0x19, 0x00, \
  0xe3, 0x38, 0xce, 0x31, 0x89, 0x80, \
  0x81, 0xe0, 0x78, 0x1e, 0x30, 0x00, \
  0x21, 0x48, 0x52, 0x14, 0x05, 0x80, \
  0x52, 0x94, 0xa5, 0x28, 0x1e, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0x82, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x62, 0x80, \
  0x58, 0x56, 0x15, 0x86, 0x44, 0x00, \
  0x19, 0x86, 0x61, 0x99, 0xe0, 0x00, \
  0xf7, 0x8d, 0xa2, 0xa0, 0x33, 0x00

#define kMaskRandom41_21 \
  0x4c, 0x13, 0x04, 0xc1, 0x91, 0x00, \
  0x51, 0x14, 0x45, 0x11, 0x45, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0xa4, 0x00, \
  0x04, 0xc1, 0x30, 0x4c, 0x13, 0x00, \
  0x03, 0x80, 0xe0, 0x3b, 0x40, 0x00, \
  0x86, 0x21, 0x88, 0x62, 0x09, 0x00, \
  0x29, 0x0a, 0x42, 0x90, 0x84, 0x80, \
  0x42, 0x50, 0x94, 0x24, 0x30, 0x80, \
  0x98, 0x26, 0x09, 0x81, 0x28, 0x00, \
  0x30, 0x8c, 0x23, 0x08, 0x4a, 0x80, \
  0xc6, 0x31, 0x8c, 0x62, 0x1a, 0x00, \
  0x23, 0x88, 0xe2, 0x38, 0x8c, 0x80, \
  0x1a, 0x46, 0x91, 0xa4, 0x58, 0x80, \
  0x24, 0xc9, 0x32, 0x4d, 0x30, 0x80, \
  0x71, 0x1c, 0x47, 0x11, 0x07, 0x00, \
  0x0e, 0x03, 0x80, 0xe1, 0x91, 0x00, \
  0x33, 0x0c, 0xc3, 0x31, 0x45, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x32, 0x80, \
  0x45, 0x51, 0x54, 0x56, 0x84, 0x80, \
  0x88, 0xa2, 0x28, 0x88, 0x4a, 0x80, \
  0xe0, 0x38, 0x0e, 0x02, 0x29, 0x00

#define kMaskRandom41_22 \
  0xc6, 0x31, 0x8c, 0x62, 0x1a, 0x00, \
  0x23, 0x88, 0xe2, 0x38, 0x8c, 0x80, \
  0x1a, 0x46, 0x91, 0xa4, 0x58, 0x80, \
  0x24, 0xc9, 0x32, 0x4d, 0x30, 0x80, \
  0x71, 0x1c, 0x47, 0x11, 0x07, 0x00, \
  0x0e, 0x03, 0x80, 0xe1, 0x91, 0x00, \
  0x33, 0x0c, 0xc3, 0x31, 0x45, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x32, 0x80, \
  0x45, 0x51, 0x54, 0x56, 0x84, 0x80, \
  0x88, 0xa2, 0x28, 0x88, 0x4a, 0x80, \
  0xe0, 0x38, 0x0e, 0x02, 0x29, 0x00, \
  0x4c, 0x13, 0x04, 0xc1, 0x91, 0x00, \
  0x51, 0x14, 0x45, 0x11, 0x45, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0xa4, 0x00, \
  0x04, 0xc1, 0x30, 0x4c, 0x13, 0x00, \
  0x03, 0x80, 0xe0, 0x3b, 0x40, 0x00, \
  0x86, 0x21, 0x88, 0x62, 0x09, 0x00, \
  0x29, 0x0a, 0x42, 0x90, 0x84, 0x80, \
  0x42, 0x50, 0x94, 0x24, 0x30, 0x80, \
  0x98, 0x26, 0x09, 0x81, 0x28, 0x00, \
  0x30, 0x8c, 0x23, 0x08, 0x4a, 0x80, \
  0x33, 0x09, 0x6e, 0x49, 0x6b, 0x80

#define kMaskRandom41_23 \
  0xc6, 0x31, 0x8c, 0x62, 0x1a, 0x00, \
  0x23, 0x88, 0xe2, 0x38, 0x8c, 0x80, \
  0x1a, 0x46, 0x91, 0xa4, 0x58, 0x80, \
  0x24, 0xc9, 0x32, 0x4d, 0x30, 0x80, \
  0x71, 0x1c, 0x47, 0x11, 0x07, 0x00, \
  0x0e, 0x03, 0x80, 0xe1, 0x91, 0x00, \
  0x33, 0x0c, 0xc3, 0x31, 0x45, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x32, 0x80, \
  0x45, 0x51, 0x54, 0x56, 0x84, 0x80, \
  0x88, 0xa2, 0x28, 0x88, 0x4a, 0x80, \
  0xe0, 0x38, 0x0e, 0x02, 0x29, 0x00, \
  0x0e, 0x03, 0x80, 0xe1, 0x91, 0x00, \
  0x33, 0x0c, 0xc3, 0x31, 0x45, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x32, 0x80, \
  0x45, 0x51, 0x54, 0x56, 0x84, 0x80, \
  0x88, 0xa2, 0x28, 0x88, 0x4a, 0x80, \
  0xe0, 0x38, 0x0e, 0x02, 0x29, 0x00, \
  0xc6, 0x31, 0x8c, 0x62, 0x1a, 0x00, \
  0x23, 0x88, 0xe2, 0x38, 0x8c, 0x80, \
  0x1a, 0x46, 0x91, 0xa4, 0x58, 0x80, \
  0x24, 0xc9, 0x32, 0x4d, 0x30, 0x80, \
  0x71, 0x1c, 0x47, 0x11, 0x07, 0x00, \
  0xf5, 0xdc, 0x4a, 0x06, 0x51, 0x80

#define kMaskRandom41_24 \
  0x0e, 0x03, 0x80, 0xe1, 0x91, 0x00, \
  0x33, 0x0c, 0xc3, 0x31, 0x45, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x32, 0x80, \
  0x45, 0x51, 0x54, 0x56, 0x84, 0x80, \
  0x88, 0xa2, 0x28, 0x88, 0x4a, 0x80, \
  0xe0, 0x38, 0x0e, 0x02, 0x29, 0x00, \
  0xc6, 0x31, 0x8c, 0x62, 0x1a, 0x00, \
  0x23, 0x88, 0xe2, 0x38, 0x8c, 0x80, \
  0x1a, 0x46, 0x91, 0xa4, 0x58, 0x80, \
  0x24, 0xc9, 0x32, 0x4d, 0x30, 0x80, \
  0x71, 0x1c, 0x47, 0x11, 0x07, 0x00, \
  0xf5, 0xdc, 0x4a, 0x06, 0x51, 0x80, \
  0xc6, 0x31, 0x8c, 0x62, 0x1a, 0x00, \
  0x23, 0x88, 0xe2, 0x38, 0x8c, 0x80, \
  0x1a, 0x46, 0x91, 0xa4, 0x58, 0x80, \
  0x24, 0xc9, 0x32, 0x4d, 0x30, 0x80, \
  0x71, 0x1c, 0x47, 0x11, 0x07, 0x00, \
  0x0e, 0x03, 0x80, 0xe1, 0x91, 0x00, \
  0x33, 0x0c, 0xc3, 0x31, 0x45, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x32, 0x80, \
  0x45, 0x51, 0x54, 0x56, 0x84, 0x80, \
  0x88, 0xa2, 0x28, 0x88, 0x4a, 0x80, \
  0xe0, 0x38, 0x0e, 0x02, 0x29, 0x00, \
  0x45, 0xa6, 0xef, 0xc9, 0xc3, 0x00

#define kMaskRandom41_25 \
  0x0e, 0x03, 0x80, 0xe1, 0x91, 0x00, \
  0x33, 0x0c, 0xc3, 0x31, 0x45, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x32, 0x80, \
  0x45, 0x51, 0x54, 0x56, 0x84, 0x80, \
  0x88, 0xa2, 0x28, 0x88, 0x4a, 0x80, \
  0xe0, 0x38, 0x0e, 0x02, 0x29, 0x00, \
  0xc6, 0x31, 0x8c, 0x62, 0x1a, 0x00, \
  0x23, 0x88, 0xe2, 0x38, 0x8c, 0x80, \
  0x1a, 0x46, 0x91, 0xa4, 0x58, 0x80, \
  0x24, 0xc9, 0x32, 0x4d, 0x30, 0x80, \
  0x71, 0x1c, 0x47, 0x11, 0x07, 0x00, \
  0xf5, 0xdc, 0x4a, 0x06, 0x51, 0x80, \
  0x0e, 0x03, 0x80, 0xe1, 0x91, 0x00, \
  0x33, 0x0c, 0xc3, 0x31, 0x45, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x32, 0x80, \
  0x45, 0x51, 0x54, 0x56, 0x84, 0x80, \
  0x88, 0xa2, 0x28, 0x88, 0x4a, 0x80, \
  0xe0, 0x38, 0x0e, 0x02, 0x29, 0x00, \
  0x46, 0x11, 0x84, 0x61, 0x19, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x80, \
  0x10, 0xc4, 0x31, 0x0e, 0x46, 0x00, \
  0x0c, 0x43, 0x10, 0xc6, 0x90, 0x80, \
  0x28, 0x8a, 0x22, 0x89, 0x42, 0x80, \
  0x94, 0x25, 0x09, 0x42, 0x13, 0x00, \
  0xc1, 0x30, 0x4c, 0x10, 0x25, 0x80

#define kMaskRandom41_26 \
  0x0e, 0x03, 0x80, 0xe1, 0x91, 0x00, \
  0x33, 0x0c, 0xc3, 0x31, 0x45, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x32, 0x80, \
  0x45, 0x51, 0x54, 0x56, 0x84, 0x80, \
  0x88, 0xa2, 0x28, 0x88, 0x4a, 0x80, \
  0xe0, 0x38, 0x0e, 0x02, 0x29, 0x00, \
  0x46, 0x11, 0x84, 0x61, 0x19, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x80, \
  0x10, 0xc4, 0x31, 0x0e, 0x46, 0x00, \
  0x0c, 0x43, 0x10, 0xc6, 0x90, 0x80, \
  0x28, 0x8a, 0x22, 0x89, 0x42, 0x80, \
  0x94, 0x25, 0x09, 0x42, 0x13, 0x00, \
  0xc1, 0x30, 0x4c, 0x10, 0x25, 0x80, \
  0x0e, 0x03, 0x80, 0xe1, 0x91, 0x00, \
  0x33, 0x0c, 0xc3, 0x31, 0x45, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x32, 0x80, \
  0x45, 0x51, 0x54, 0x56, 0x84, 0x80, \
  0x88, 0xa2, 0x28, 0x88, 0x4a, 0x80, \
  0xe0, 0x38, 0x0e, 0x02, 0x29, 0x00, \
  0xc6, 0x31, 0x8c, 0x62, 0x1a, 0x00, \
  0x23, 0x88, 0xe2, 0x38, 0x8c, 0x80, \
  0x1a, 0x46, 0x91, 0xa4, 0x58, 0x80, \
  0x24, 0xc9, 0x32, 0x4d, 0x30, 0x80, \
  0x71, 0x1c, 0x47, 0x11, 0x07, 0x00, \
  0xf5, 0xdc, 0x4a, 0x06, 0x51, 0x80, \
  0x6f, 0x72, 0xf1, 0xe7, 0x1a, 0x80

#define kMaskRandom41_27 \
  0x0e, 0x03, 0x80, 0xe1, 0x91, 0x00, \
  0x33, 0x0c, 0xc3, 0x31, 0x45, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x32, 0x80, \
  0x45, 0x51, 0x54, 0x56, 0x84, 0x80, \
  0x88, 0xa2, 0x28, 0x88, 0x4a, 0x80, \
  0xe0, 0x38, 0x0e, 0x02, 0x29, 0x00, \
  0x46, 0x11, 0x84, 0x61, 0x19, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x80, \
  0x10, 0xc4, 0x31, 0x0e, 0x46, 0x00, \
  0x0c, 0x43, 0x10, 0xc6, 0x90, 0x80, \
  0x28, 0x8a, 0x22, 0x89, 0x42, 0x80, \
  0x94, 0x25, 0x09, 0x42, 0x13, 0x00, \
  0xc1, 0x30, 0x4c, 0x10, 0x25, 0x80, \
  0x46, 0x11, 0x84, 0x61, 0x19, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x80, \
  0x10, 0xc4, 0x31, 0x0e, 0x46, 0x00, \
  0x0c, 0x43, 0x10, 0xc6, 0x90, 0x80, \
  0x28, 0x8a, 0x22, 0x89, 0x42, 0x80, \
  0x94, 0x25, 0x09, 0x42, 0x13, 0x00, \
  0xc1, 0x30, 0x4c, 0x10, 0x25, 0x80, \
  0x0e, 0x03, 0x80, 0xe1, 0x91, 0x00, \
  0x33, 0x0c, 0xc3, 0x31, 0x45, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x32, 0x80, \
  0x45, 0x51, 0x54, 0x56, 0x84, 0x80, \
  0x88, 0xa2, 0x28, 0x88, 0x4a, 0x80, \
  0xe0, 0x38, 0x0e, 0x02, 0x29, 0x00, \
  0x56, 0x3e, 0x24, 0xdd, 0x0c, 0x00

#define kMaskRandom41_28 \
  0x46, 0x11, 0x84, 0x61, 0x19, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x80, \
  0x10, 0xc4, 0x31, 0x0e, 0x46, 0x00, \
  0x0c, 0x43, 0x10, 0xc6, 0x90, 0x80, \
  0x28, 0x8a, 0x22, 0x89, 0x42, 0x80, \
  0x94, 0x25, 0x09, 0x42, 0x13, 0x00, \
  0xc1, 0x30, 0x4c, 0x10, 0x25, 0x80, \
  0x0e, 0x03, 0x80, 0xe1, 0x91, 0x00, \
  0x33, 0x0c, 0xc3, 0x31, 0x45, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x32, 0x80, \
  0x45, 0x51, 0x54, 0x56, 0x84, 0x80, \
  0x88, 0xa2, 0x28, 0x88, 0x4a, 0x80, \
  0xe0, 0x38, 0x0e, 0x02, 0x29, 0x00, \
  0x56, 0x3e, 0x24, 0xdd, 0x0c, 0x00, \
  0x0e, 0x03, 0x80, 0xe1, 0x91, 0x00, \
  0x33, 0x0c, 0xc3, 0x31, 0x45, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x32, 0x80, \
  0x45, 0x51, 0x54, 0x56, 0x84, 0x80, \
  0x88, 0xa2, 0x28, 0x88, 0x4a, 0x80, \
  0xe0, 0x38, 0x0e, 0x02, 0x29, 0x00, \
  0x46, 0x11, 0x84, 0x61, 0x19, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x80, \
  0x10, 0xc4, 0x31, 0x0e, 0x46, 0x00, \
  0x0c, 0x43, 0x10, 0xc6, 0x90, 0x80, \
  0x28, 0x8a, 0x22, 0x89, 0x42, 0x80, \
  0x94, 0x25, 0x09, 0x42, 0x13, 0x00, \
  0xc1, 0x30, 0x4c, 0x10, 0x25, 0x80, \
  0x61, 0x2c, 0xfa, 0x25, 0x38, 0x00

#define kMaskRandom41_29 \
  0x46, 0x11, 0x84, 0x61, 0x19, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x80, \
  0x10, 0xc4, 0x31, 0x0e, 0x46, 0x00, \
  0x0c, 0x43, 0x10, 0xc6, 0x90, 0x80, \
  0x28, 0x8a, 0x22, 0x89, 0x42, 0x80, \
  0x94, 0x25, 0x09, 0x42, 0x13, 0x00, \
  0xc1, 0x30, 0x4c, 0x10, 0x25, 0x80, \
  0x0e, 0x03, 0x80, 0xe1, 0x91, 0x00, \
  0x33, 0x0c, 0xc3, 0x31, 0x45, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x32, 0x80, \
  0x45, 0x51, 0x54, 0x56, 0x84, 0x80, \
  0x88, 0xa2, 0x28, 0x88, 0x4a, 0x80, \
  0xe0, 0x38, 0x0e, 0x02, 0x29, 0x00, \
  0x56, 0x3e, 0x24, 0xdd, 0x0c, 0x00, \
  0x46, 0x11, 0x84, 0x61, 0x19, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x80, \
  0x10, 0xc4, 0x31, 0x0e, 0x46, 0x00, \
  0x0c, 0x43, 0x10, 0xc6, 0x90, 0x80, \
  0x28, 0x8a, 0x22, 0x89, 0x42, 0x80, \
  0x94, 0x25, 0x09, 0x42, 0x13, 0x00, \
  0xc1, 0x30, 0x4c, 0x10, 0x25, 0x80, \
  0x2c, 0x0b, 0x02, 0xc0, 0x32, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x01, 0x80, \
  0xa0, 0x68, 0x1a, 0x06, 0x82, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x89, 0x0c, 0x00, \
  0xc2, 0x30, 0x8c, 0x20, 0x68, 0x00, \
  0x22, 0x88, 0xa2, 0x29, 0x80, 0x80, \
  0x50, 0x54, 0x15, 0x04, 0x50, 0x80

#define kMaskRandom41_3 \
  0xce, 0x33, 0x8c, 0xe3, 0x2b, 0x00, \
  0x55, 0x95, 0x65, 0x5d, 0xc5, 0x00, \
  0xb1, 0x6a, 0x3a, 0x8e, 0xd8, 0x80

#define kMaskRandom41_30 \
  0x46, 0x11, 0x84, 0x61, 0x19, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x80, \
  0x10, 0xc4, 0x31, 0x0e, 0x46, 0x00, \
  0x0c, 0x43, 0x10, 0xc6, 0x90, 0x80, \
  0x28, 0x8a, 0x22, 0x89, 0x42, 0x80, \
  0x94, 0x25, 0x09, 0x42, 0x13, 0x00, \
  0xc1, 0x30, 0x4c, 0x10, 0x25, 0x80, \
  0x2c, 0x0b, 0x02, 0xc0, 0x32, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x01, 0x80, \
  0xa0, 0x68, 0x1a, 0x06, 0x82, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x89, 0x0c, 0x00, \
  0xc2, 0x30, 0x8c, 0x20, 0x68, 0x00, \
  0x22, 0x88, 0xa2, 0x29, 0x80, 0x80, \
  0x50, 0x54, 0x15, 0x04, 0x50, 0x80, \
  0x46, 0x11, 0x84, 0x61, 0x19, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x80, \
  0x10, 0xc4, 0x31, 0x0e, 0x46, 0x00, \
  0x0c, 0x43, 0x10, 0xc6, 0x90, 0x80, \
  0x28, 0x8a, 0x22, 0x89, 0x42, 0x80, \
  0x94, 0x25, 0x09, 0x42, 0x13, 0x00, \
  0xc1, 0x30, 0x4c, 0x10, 0x25, 0x80, \
  0x0e, 0x03, 0x80, 0xe1, 0x91, 0x00, \
  0x33, 0x0c, 0xc3, 0x31, 0x45, 0x00, \
  0x10, 0xc4, 0x31, 0x0c, 0x32, 0x80, \
  0x45, 0x51, 0x54, 0x56, 0x84, 0x80, \
  0x88, 0xa2, 0x28, 0x88, 0x4a, 0x80, \
  0xe0, 0x38, 0x0e, 0x02, 0x29, 0x00, \
  0x56, 0x3e, 0x24, 0xdd, 0x0c, 0x00, \
  0x59, 0x53, 0x31, 0x62, 0x15, 0x00

#define kMaskRandom41_31 \
  0x46, 0x11, 0x84, 0x61, 0x19, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x80, \
  0x10, 0xc4, 0x31, 0x0e, 0x46, 0x00, \
  0x0c, 0x43, 0x10, 0xc6, 0x90, 0x80, \
  0x28, 0x8a, 0x22, 0x89, 0x42, 0x80, \
  0x94, 0x25, 0x09, 0x42, 0x13, 0x00, \
  0xc1, 0x30, 0x4c, 0x10, 0x25, 0x80, \
  0x2c, 0x0b, 0x02, 0xc0, 0x32, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x01, 0x80, \
  0xa0, 0x68, 0x1a, 0x06, 0x82, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x89, 0x0c, 0x00, \
  0xc2, 0x30, 0x8c, 0x20, 0x68, 0x00, \
  0x22, 0x88, 0xa2, 0x29, 0x80, 0x80, \
  0x50, 0x54, 0x15, 0x04, 0x50, 0x80, \
  0x2c, 0x0b, 0x02, 0xc0, 0x32, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x01, 0x80, \
  0xa0, 0x68, 0x1a, 0x06, 0x82, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x89, 0x0c, 0x00, \
  0xc2, 0x30, 0x8c, 0x20, 0x68, 0x00, \
  0x22, 0x88, 0xa2, 0x29, 0x80, 0x80, \
  0x50, 0x54, 0x15, 0x04, 0x50, 0x80, \
  0x46, 0x11, 0x84, 0x61, 0x19, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x80, \
  0x10, 0xc4, 0x31, 0x0e, 0x46, 0x00, \
  0x0c, 0x43, 0x10, 0xc6, 0x90, 0x80, \
  0x28, 0x8a, 0x22, 0x89, 0x42, 0x80, \
  0x94, 0x25, 0x09, 0x42, 0x13, 0x00, \
  0xc1, 0x30, 0x4c, 0x10, 0x25, 0x80, \
  0x28, 0x1c, 0x63, 0xbf, 0x53, 0x80

#define kMaskRandom41_32 \
  0x2c, 0x0b, 0x02, 0xc0, 0x32, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x01, 0x80, \
  0xa0, 0x68, 0x1a, 0x06, 0x82, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x89, 0x0c, 0x00, \
  0xc2, 0x30, 0x8c, 0x20, 0x68, 0x00, \
  0x22, 0x88, 0xa2, 0x29, 0x80, 0x80, \
  0x50, 0x54, 0x15, 0x04, 0x50, 0x80, \
  0x46, 0x11, 0x84, 0x61, 0x19, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x80, \
  0x10, 0xc4, 0x31, 0x0e, 0x46, 0x00, \
  0x0c, 0x43, 0x10, 0xc6, 0x90, 0x80, \
  0x28, 0x8a, 0x22, 0x89, 0x42, 0x80, \
  0x94, 0x25, 0x09, 0x42, 0x13, 0x00, \
  0xc1, 0x30, 0x4c, 0x10, 0x25, 0x80, \
  0x28, 0x1c, 0x63, 0xbf, 0x53, 0x80, \
  0x46, 0x11, 0x84, 0x61, 0x19, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x80, \
  0x10, 0xc4, 0x31, 0x0e, 0x46, 0x00, \
  0x0c, 0x43, 0x10, 0xc6, 0x90, 0x80, \
  0x28, 0x8a, 0x22, 0x89, 0x42, 0x80, \
  0x94, 0x25, 0x09, 0x42, 0x13, 0x00, \
  0xc1, 0x30, 0x4c, 0x10, 0x25, 0x80, \
  0x2c, 0x0b, 0x02, 0xc0, 0x32, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x01, 0x80, \
  0xa0, 0x68, 0x1a, 0x06, 0x82, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x89, 0x0c, 0x00, \
  0xc2, 0x30, 0x8c, 0x20, 0x68, 0x00, \
  0x22, 0x88, 0xa2, 0x29, 0x80, 0x80, \
  0x50, 0x54, 0x15, 0x04, 0x50, 0x80, \
  0xca, 0xbb, 0xcb, 0x6d, 0xaa, 0x00

#define kMaskRandom41_33 \
  0x2c, 0x0b, 0x02, 0xc0, 0x32, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x01, 0x80, \
  0xa0, 0x68, 0x1a, 0x06, 0x82, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x89, 0x0c, 0x00, \
  0xc2, 0x30, 0x8c, 0x20, 0x68, 0x00, \
  0x22, 0x88, 0xa2, 0x29, 0x80, 0x80, \
  0x50, 0x54, 0x15, 0x04, 0x50, 0x80, \
  0x46, 0x11, 0x84, 0x61, 0x19, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x80, \
  0x10, 0xc4, 0x31, 0x0e, 0x46, 0x00, \
  0x0c, 0x43, 0x10, 0xc6, 0x90, 0x80, \
  0x28, 0x8a, 0x22, 0x89, 0x42, 0x80, \
  0x94, 0x25, 0x09, 0x42, 0x13, 0x00, \
  0xc1, 0x30, 0x4c, 0x10, 0x25, 0x80, \
  0x28, 0x1c, 0x63, 0xbf, 0x53, 0x80, \
  0x2c, 0x0b, 0x02, 0xc0, 0x32, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x01, 0x80, \
  0xa0, 0x68, 0x1a, 0x06, 0x82, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x89, 0x0c, 0x00, \
  0xc2, 0x30, 0x8c, 0x20, 0x68, 0x00, \
  0x22, 0x88, 0xa2, 0x29, 0x80, 0x80, \
  0x50, 0x54, 0x15, 0x04, 0x50, 0x80, \
  0x4e, 0x13, 0x84, 0xe1, 0x19, 0x00, \
  0xe3, 0x38, 0xce, 0x31, 0x89, 0x80, \
  0x81, 0xe0, 0x78, 0x1e, 0x30, 0x00, \
  0x21, 0x48, 0x52, 0x14, 0x05, 0x80, \
  0x52, 0x94, 0xa5, 0x28, 0x1e, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0x82, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x62, 0x80, \
  0x58, 0x56, 0x15, 0x86, 0x44, 0x00, \
  0x19, 0x86, 0x61, 0x99, 0xe0, 0x00

#define kMaskRandom41_34 \
  0x2c, 0x0b, 0x02, 0xc0, 0x32, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x01, 0x80, \
  0xa0, 0x68, 0x1a, 0x06, 0x82, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x89, 0x0c, 0x00, \
  0xc2, 0x30, 0x8c, 0x20, 0x68, 0x00, \
  0x22, 0x88, 0xa2, 0x29, 0x80, 0x80, \
  0x50, 0x54, 0x15, 0x04, 0x50, 0x80, \
  0x4e, 0x13, 0x84, 0xe1, 0x19, 0x00, \
  0xe3, 0x38, 0xce, 0x31, 0x89, 0x80, \
  0x81, 0xe0, 0x78, 0x1e, 0x30, 0x00, \
  0x21, 0x48, 0x52, 0x14, 0x05, 0x80, \
  0x52, 0x94, 0xa5, 0x28, 0x1e, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0x82, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x62, 0x80, \
  0x58, 0x56, 0x15, 0x86, 0x44, 0x00, \
  0x19, 0x86, 0x61, 0x99, 0xe0, 0x00, \
  0x2c, 0x0b, 0x02, 0xc0, 0x32, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x01, 0x80, \
  0xa0, 0x68, 0x1a, 0x06, 0x82, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x89, 0x0c, 0x00, \
  0xc2, 0x30, 0x8c, 0x20, 0x68, 0x00, \
  0x22, 0x88, 0xa2, 0x29, 0x80, 0x80, \
  0x50, 0x54, 0x15, 0x04, 0x50, 0x80, \
  0x46, 0x11, 0x84, 0x61, 0x19, 0x00, \
  0x33, 0x0c, 0xc3, 0x30, 0xcc, 0x80, \
  0x10, 0xc4, 0x31, 0x0e, 0x46, 0x00, \
  0x0c, 0x43, 0x10, 0xc6, 0x90, 0x80, \
  0x28, 0x8a, 0x22, 0x89, 0x42, 0x80, \
  0x94, 0x25, 0x09, 0x42, 0x13, 0x00, \
  0xc1, 0x30, 0x4c, 0x10, 0x25, 0x80, \
  0x28, 0x1c, 0x63, 0xbf, 0x53, 0x80, \
  0xbd, 0x37, 0x3f, 0x75, 0x36, 0x80

#define kMaskRandom41_35 \
  0x2c, 0x0b, 0x02, 0xc0, 0x32, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x01, 0x80, \
  0xa0, 0x68, 0x1a, 0x06, 0x82, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x89, 0x0c, 0x00, \
  0xc2, 0x30, 0x8c, 0x20, 0x68, 0x00, \
  0x22, 0x88, 0xa2, 0x29, 0x80, 0x80, \
  0x50, 0x54, 0x15, 0x04, 0x50, 0x80, \
  0x4e, 0x13, 0x84, 0xe1, 0x19, 0x00, \
  0xe3, 0x38, 0xce, 0x31, 0x89, 0x80, \
  0x81, 0xe0, 0x78, 0x1e, 0x30, 0x00, \
  0x21, 0x48, 0x52, 0x14, 0x05, 0x80, \
  0x52, 0x94, 0xa5, 0x28, 0x1e, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0x82, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x62, 0x80, \
  0x58, 0x56, 0x15, 0x86, 0x44, 0x00, \
  0x19, 0x86, 0x61, 0x99, 0xe0, 0x00, \
  0x4e, 0x13, 0x84, 0xe1, 0x19, 0x00, \
  0xe3, 0x38, 0xce, 0x31, 0x89, 0x80, \
  0x81, 0xe0, 0x78, 0x1e, 0x30, 0x00, \
  0x21, 0x48, 0x52, 0x14, 0x05, 0x80, \
  0x52, 0x94, 0xa5, 0x28, 0x1e, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0x82, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x62, 0x80, \
  0x58, 0x56, 0x15, 0x86, 0x44, 0x00, \
  0x19, 0x86, 0x61, 0x99, 0xe0, 0x00, \
  0x2c, 0x0b, 0x02, 0xc0, 0x32, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x01, 0x80, \
  0xa0, 0x68, 0x1a, 0x06, 0x82, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x89, 0x0c, 0x00, \
  0xc2, 0x30, 0x8c, 0x20, 0x68, 0x00, \
  0x22, 0x88, 0xa2, 0x29, 0x80, 0x80, \
  0x50, 0x54, 0x15, 0x04, 0x50, 0x80, \
  0x21, 0x7b, 0xf5, 0xa5, 0x65, 0x80

#define kMaskRandom41_36 \
  0x4e, 0x13, 0x84, 0xe1, 0x19, 0x00, \
  0xe3, 0x38, 0xce, 0x31, 0x89, 0x80, \
  0x81, 0xe0, 0x78, 0x1e, 0x30, 0x00, \
  0x21, 0x48, 0x52, 0x14, 0x05, 0x80, \
  0x52, 0x94, 0xa5, 0x28, 0x1e, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0x82, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x62, 0x80, \
  0x58, 0x56, 0x15, 0x86, 0x44, 0x00, \
  0x19, 0x86, 0x61, 0x99, 0xe0, 0x00, \
  0x2c, 0x0b, 0x02, 0xc0, 0x32, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x01, 0x80, \
  0xa0, 0x68, 0x1a, 0x06, 0x82, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x89, 0x0c, 0x00, \
  0xc2, 0x30, 0x8c, 0x20, 0x68, 0x00, \
  0x22, 0x88, 0xa2, 0x29, 0x80, 0x80, \
  0x50, 0x54, 0x15, 0x04, 0x50, 0x80, \
  0x21, 0x7b, 0xf5, 0xa5, 0x65, 0x80, \
  0x2c, 0x0b, 0x02, 0xc0, 0x32, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x01, 0x80, \
  0xa0, 0x68, 0x1a, 0x06, 0x82, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x89, 0x0c, 0x00, \
  0xc2, 0x30, 0x8c, 0x20, 0x68, 0x00, \
  0x22, 0x88, 0xa2, 0x29, 0x80, 0x80, \
  0x50, 0x54, 0x15, 0x04, 0x50, 0x80, \
  0x4e, 0x13, 0x84, 0xe1, 0x19, 0x00, \
  0xe3, 0x38, 0xce, 0x31, 0x89, 0x80, \
  0x81, 0xe0, 0x78, 0x1e, 0x30, 0x00, \
  0x21, 0x48, 0x52, 0x14, 0x05, 0x80, \
  0x52, 0x94, 0xa5, 0x28, 0x1e, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0x82, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x62, 0x80, \
  0x58, 0x56, 0x15, 0x86, 0x44, 0x00, \
  0x19, 0x86, 0x61, 0x99, 0xe0, 0x00, \
  0xc1, 0xb1, 0x80, 0xbe, 0x3e, 0x00

#define kMaskRandom41_37 \
  0x4e, 0x13, 0x84, 0xe1, 0x19, 0x00, \
  0xe3, 0x38, 0xce, 0x31, 0x89, 0x80, \
  0x81, 0xe0, 0x78, 0x1e, 0x30, 0x00, \
  0x21, 0x48, 0x52, 0x14, 0x05, 0x80, \
  0x52, 0x94, 0xa5, 0x28, 0x1e, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0x82, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x62, 0x80, \
  0x58, 0x56, 0x15, 0x86, 0x44, 0x00, \
  0x19, 0x86, 0x61, 0x99, 0xe0, 0x00, \
  0x2c, 0x0b, 0x02, 0xc0, 0x32, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x01, 0x80, \
  0xa0, 0x68, 0x1a, 0x06, 0x82, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x89, 0x0c, 0x00, \
  0xc2, 0x30, 0x8c, 0x20, 0x68, 0x00, \
  0x22, 0x88, 0xa2, 0x29, 0x80, 0x80, \
  0x50, 0x54, 0x15, 0x04, 0x50, 0x80, \
  0x21, 0x7b, 0xf5, 0xa5, 0x65, 0x80, \
  0x4e, 0x13, 0x84, 0xe1, 0x19, 0x00, \
  0xe3, 0x38, 0xce, 0x31, 0x89, 0x80, \
  0x81, 0xe0, 0x78, 0x1e, 0x30, 0x00, \
  0x21, 0x48, 0x52, 0x14, 0x05, 0x80, \
  0x52, 0x94, 0xa5, 0x28, 0x1e, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0x82, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x62, 0x80, \
  0x58, 0x56, 0x15, 0x86, 0x44, 0x00, \
  0x19, 0x86, 0x61, 0x99, 0xe0, 0x00, \
  0x4c, 0x13, 0x04, 0xc1, 0x91, 0x00, \
  0x51, 0x14, 0x45, 0x11, 0x45, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0xa4, 0x00, \
  0x04, 0xc1, 0x30, 0x4c, 0x13, 0x00, \
  0x03, 0x80, 0xe0, 0x3b, 0x40, 0x00, \
  0x86, 0x21, 0x88, 0x62, 0x09, 0x00, \
  0x29, 0x0a, 0x42, 0x90, 0x84, 0x80, \
  0x42, 0x50, 0x94, 0x24, 0x30, 0x80, \
  0x98, 0x26, 0x09, 0x81, 0x28, 0x00, \
  0x30, 0x8c, 0x23, 0x08, 0x4a, 0x80

#define kMaskRandom41_38 \
  0x4e, 0x13, 0x84, 0xe1, 0x19, 0x00, \
  0xe3, 0x38, 0xce, 0x31, 0x89, 0x80, \
  0x81, 0xe0, 0x78, 0x1e, 0x30, 0x00, \
  0x21, 0x48, 0x52, 0x14, 0x05, 0x80, \
  0x52, 0x94, 0xa5, 0x28, 0x1e, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0x82, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x62, 0x80, \
  0x58, 0x56, 0x15, 0x86, 0x44, 0x00, \
  0x19, 0x86, 0x61, 0x99, 0xe0, 0x00, \
  0x4c, 0x13, 0x04, 0xc1, 0x91, 0x00, \
  0x51, 0x14, 0x45, 0x11, 0x45, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0xa4, 0x00, \
  0x04, 0xc1, 0x30, 0x4c, 0x13, 0x00, \
  0x03, 0x80, 0xe0, 0x3b, 0x40, 0x00, \
  0x86, 0x21, 0x88, 0x62, 0x09, 0x00, \
  0x29, 0x0a, 0x42, 0x90, 0x84, 0x80, \
  0x42, 0x50, 0x94, 0x24, 0x30, 0x80, \
  0x98, 0x26, 0x09, 0x81, 0x28, 0x00, \
  0x30, 0x8c, 0x23, 0x08, 0x4a, 0x80, \
  0x4e, 0x13, 0x84, 0xe1, 0x19, 0x00, \
  0xe3, 0x38, 0xce, 0x31, 0x89, 0x80, \
  0x81, 0xe0, 0x78, 0x1e, 0x30, 0x00, \
  0x21, 0x48, 0x52, 0x14, 0x05, 0x80, \
  0x52, 0x94, 0xa5, 0x28, 0x1e, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0x82, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x62, 0x80, \
  0x58, 0x56, 0x15, 0x86, 0x44, 0x00, \
  0x19, 0x86, 0x61, 0x99, 0xe0, 0x00, \
  0x2c, 0x0b, 0x02, 0xc0, 0x32, 0x00, \
  0x81, 0xa0, 0x68, 0x1a, 0x01, 0x80, \
  0xa0, 0x68, 0x1a, 0x06, 0x82, 0x00, \
  0x05, 0x41, 0x50, 0x54, 0x15, 0x00, \
  0x18, 0x86, 0x21, 0x89, 0x0c, 0x00, \
  0xc2, 0x30, 0x8c, 0x20, 0x68, 0x00, \
  0x22, 0x88, 0xa2, 0x29, 0x80, 0x80, \
  0x50, 0x54, 0x15, 0x04, 0x50, 0x80, \
  0x21, 0x7b, 0xf5, 0xa5, 0x65, 0x80, \
  0xea, 0xc8, 0xbb, 0xd4, 0x5d, 0x00

#define kMaskRandom41_39 \
  0x4e, 0x13, 0x84, 0xe1, 0x19, 0x00, \
  0xe3, 0x38, 0xce, 0x31, 0x89, 0x80, \
  0x81, 0xe0, 0x78, 0x1e, 0x30, 0x00, \
  0x21, 0x48, 0x52, 0x14, 0x05, 0x80, \
  0x52, 0x94, 0xa5, 0x28, 0x1e, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0x82, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x62, 0x80, \
  0x58, 0x56, 0x15, 0x86, 0x44, 0x00, \
  0x19, 0x86, 0x61, 0x99, 0xe0, 0x00, \
  0x4c, 0x13, 0x04, 0xc1, 0x91, 0x00, \
  0x51, 0x14, 0x45, 0x11, 0x45, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0xa4, 0x00, \
  0x04, 0xc1, 0x30, 0x4c, 0x13, 0x00, \
  0x03, 0x80, 0xe0, 0x3b, 0x40, 0x00, \
  0x86, 0x21, 0x88, 0x62, 0x09, 0x00, \
  0x29, 0x0a, 0x42, 0x90, 0x84, 0x80, \
  0x42, 0x50, 0x94, 0x24, 0x30, 0x80, \
  0x98, 0x26, 0x09, 0x81, 0x28, 0x00, \
  0x30, 0x8c, 0x23, 0x08, 0x4a, 0x80, \
  0x4c, 0x13, 0x04, 0xc1, 0x91, 0x00, \
  0x51, 0x14, 0x45, 0x11, 0x45, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0xa4, 0x00, \
  0x04, 0xc1, 0x30, 0x4c, 0x13, 0x00, \
  0x03, 0x80, 0xe0, 0x3b, 0x40, 0x00, \
  0x86, 0x21, 0x88, 0x62, 0x09, 0x00, \
  0x29, 0x0a, 0x42, 0x90, 0x84, 0x80, \
  0x42, 0x50, 0x94, 0x24, 0x30, 0x80, \
  0x98, 0x26, 0x09, 0x81, 0x28, 0x00, \
  0x30, 0x8c, 0x23, 0x08, 0x4a, 0x80, \
  0x4e, 0x13, 0x84, 0xe1, 0x19, 0x00, \
  0xe3, 0x38, 0xce, 0x31, 0x89, 0x80, \
  0x81, 0xe0, 0x78, 0x1e, 0x30, 0x00, \
  0x21, 0x48, 0x52, 0x14, 0x05, 0x80, \
  0x52, 0x94, 0xa5, 0x28, 0x1e, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0x82, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x62, 0x80, \
  0x58, 0x56, 0x15, 0x86, 0x44, 0x00, \
  0x19, 0x86, 0x61, 0x99, 0xe0, 0x00, \
  0xf7, 0x8d, 0xa2, 0xa0, 0x33, 0x00

#define kMaskRandom41_4 \
  0xe6, 0x39, 0x8e, 0x63, 0x13, 0x00, \
  0x33, 0x8c, 0xe3, 0x38, 0xc5, 0x80, \
  0x98, 0xe6, 0x39, 0x8d, 0x2c, 0x80, \
  0x2d, 0x4b, 0x52, 0xd4, 0xb2, 0x80

#define kMaskRandom41_40 \
  0x4c, 0x13, 0x04, 0xc1, 0x91, 0x00, \
  0x51, 0x14, 0x45, 0x11, 0x45, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0xa4, 0x00, \
  0x04, 0xc1, 0x30, 0x4c, 0x13, 0x00, \
  0x03, 0x80, 0xe0, 0x3b, 0x40, 0x00, \
  0x86, 0x21, 0x88, 0x62, 0x09, 0x00, \
  0x29, 0x0a, 0x42, 0x90, 0x84, 0x80, \
  0x42, 0x50, 0x94, 0x24, 0x30, 0x80, \
  0x98, 0x26, 0x09, 0x81, 0x28, 0x00, \
  0x30, 0x8c, 0x23, 0x08, 0x4a, 0x80, \
  0x4e, 0x13, 0x84, 0xe1, 0x19, 0x00, \
  0xe3, 0x38, 0xce, 0x31, 0x89, 0x80, \
  0x81, 0xe0, 0x78, 0x1e, 0x30, 0x00, \
  0x21, 0x48, 0x52, 0x14, 0x05, 0x80, \
  0x52, 0x94, 0xa5, 0x28, 0x1e, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0x82, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x62, 0x80, \
  0x58, 0x56, 0x15, 0x86, 0x44, 0x00, \
  0x19, 0x86, 0x61, 0x99, 0xe0, 0x00, \
  0xf7, 0x8d, 0xa2, 0xa0, 0x33, 0x00, \
  0x4e, 0x13, 0x84, 0xe1, 0x19, 0x00, \
  0xe3, 0x38, 0xce, 0x31, 0x89, 0x80, \
  0x81, 0xe0, 0x78, 0x1e, 0x30, 0x00, \
  0x21, 0x48, 0x52, 0x14, 0x05, 0x80, \
  0x52, 0x94, 0xa5, 0x28, 0x1e, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0x82, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x62, 0x80, \
  0x58, 0x56, 0x15, 0x86, 0x44, 0x00, \
  0x19, 0x86, 0x61, 0x99, 0xe0, 0x00, \
  0x4c, 0x13, 0x04, 0xc1, 0x91, 0x00, \
  0x51, 0x14, 0x45, 0x11, 0x45, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0xa4, 0x00, \
  0x04, 0xc1, 0x30, 0x4c, 0x13, 0x00, \
  0x03, 0x80, 0xe0, 0x3b, 0x40, 0x00, \
  0x86, 0x21, 0x88, 0x62, 0x09, 0x00, \
  0x29, 0x0a, 0x42, 0x90, 0x84, 0x80, \
  0x42, 0x50, 0x94, 0x24, 0x30, 0x80, \
  0x98, 0x26, 0x09, 0x81, 0x28, 0x00, \
  0x30, 0x8c, 0x23, 0x08, 0x4a, 0x80, \
  0xe8, 0x07, 0x18, 0x9a, 0x02, 0x00

#define kMaskRandom41_41 \
  0x4c, 0x13, 0x04, 0xc1, 0x91, 0x00, \
  0x51, 0x14, 0x45, 0x11, 0x45, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0xa4, 0x00, \
  0x04, 0xc1, 0x30, 0x4c, 0x13, 0x00, \
  0x03, 0x80, 0xe0, 0x3b, 0x40, 0x00, \
  0x86, 0x21, 0x88, 0x62, 0x09, 0x00, \
  0x29, 0x0a, 0x42, 0x90, 0x84, 0x80, \
  0x42, 0x50, 0x94, 0x24, 0x30, 0x80, \
  0x98, 0x26, 0x09, 0x81, 0x28, 0x00, \
  0x30, 0x8c, 0x23, 0x08, 0x4a, 0x80, \
  0x4e, 0x13, 0x84, 0xe1, 0x19, 0x00, \
  0xe3, 0x38, 0xce, 0x31, 0x89, 0x80, \
  0x81, 0xe0, 0x78, 0x1e, 0x30, 0x00, \
  0x21, 0x48, 0x52, 0x14, 0x05, 0x80, \
  0x52, 0x94, 0xa5, 0x28, 0x1e, 0x00, \
  0xb4, 0x2d, 0x0b, 0x42, 0x82, 0x00, \
  0x26, 0x89, 0xa2, 0x68, 0x62, 0x80, \
  0x58, 0x56, 0x15, 0x86, 0x44, 0x00, \
  0x19, 0x86, 0x61, 0x99, 0xe0, 0x00, \
  0xf7, 0x8d, 0xa2, 0xa0, 0x33, 0x00, \
  0x4c, 0x13, 0x04, 0xc1, 0x91, 0x00, \
  0x51, 0x14, 0x45, 0x11, 0x45, 0x00, \
  0xa0, 0x68, 0x1a, 0x06, 0xa4, 0x00, \
  0x04, 0xc1, 0x30, 0x4c, 0x13, 0x00, \
  0x03, 0x80, 0xe0, 0x3b, 0x40, 0x00, \
  0x86, 0x21, 0x88, 0x62, 0x09, 0x00, \
  0x29, 0x0a, 0x42, 0x90, 0x84, 0x80, \
  0x42, 0x50, 0x94, 0x24, 0x30, 0x80, \
  0x98, 0x26, 0x09, 0x81, 0x28, 0x00, \
  0x30, 0x8c, 0x23, 0x08, 0x4a, 0x80, \
  0x4e, 0x13, 0x84, 0xc1, 0x19, 0x00, \
  0xe3, 0x38, 0xc5, 0x10, 0xcc, 0x80, \
  0x81, 0xe0, 0x7a, 0x06, 0x64, 0x00, \
  0x21, 0x48, 0x50, 0x4c, 0x16, 0x00, \
  0x52, 0x94, 0xa0, 0x3a, 0x02, 0x80, \
  0xb4, 0x2d, 0x08, 0x62, 0x11, 0x00, \
  0x26, 0x89, 0xa2, 0x91, 0x01, 0x80, \
  0x58, 0x56, 0x14, 0x24, 0x2a, 0x00, \
  0x19, 0x86, 0x69, 0x81, 0xa0, 0x00, \
  0xf7, 0x8d, 0xa3, 0x08, 0x40, 0x80, \
  0x2b, 0xea, 0x4d, 0xf4, 0xc1, 0x00

#define kMaskRandom41_5 \
  0xce, 0x33, 0x8c, 0xe3, 0x1b, 0x00, \
  0x63, 0x98, 0xe6, 0x39, 0x8d, 0x80, \
  0x98, 0xe5, 0x39, 0x8c, 0x76, 0x80, \
  0x2b, 0x53, 0x54, 0xd6, 0xb5, 0x00, \
  0xb4, 0x5c, 0xab, 0x26, 0xca, 0x80

#define kMaskRandom41_6 \
  0x4c, 0x1b, 0x04, 0xc1, 0x91, 0x00, \
  0x51, 0x34, 0x45, 0x11, 0x45, 0x00, \
  0x20, 0xe8, 0x32, 0x0e, 0xa0, 0x80, \
  0x85, 0x41, 0x58, 0x54, 0x12, 0x80, \
  0x06, 0x86, 0xa0, 0x68, 0x0d, 0x80, \
  0x9a, 0x21, 0x88, 0xa2, 0x43, 0x00

#define kMaskRandom41_7 \
  0x4e, 0x11, 0x8c, 0x61, 0x19, 0x00, \
  0x33, 0x2c, 0x03, 0x30, 0x4c, 0x80, \
  0x10, 0x0e, 0xb1, 0x86, 0x74, 0x00, \
  0x81, 0x51, 0x54, 0x54, 0x2d, 0x00, \
  0x24, 0xc4, 0xa1, 0x2d, 0x42, 0x80, \
  0xd4, 0x23, 0x0b, 0x42, 0x83, 0x00, \
  0x0c, 0xa2, 0x62, 0x99, 0x21, 0x80

#define kMaskRandom41_8 \
  0x27, 0x09, 0xc0, 0x70, 0xa7, 0x00, \
  0x89, 0xa2, 0x64, 0x9a, 0x82, 0x80, \
  0xd0, 0x74, 0x1b, 0x07, 0xa0, 0x00, \
  0x24, 0xc9, 0x32, 0x4c, 0x5c, 0x00, \
  0xe2, 0x90, 0xa5, 0x28, 0x0e, 0x80, \
  0xc6, 0x31, 0x8c, 0x63, 0x18, 0x00, \
  0x31, 0x8c, 0x63, 0x19, 0x41, 0x80, \
  0x18, 0xc6, 0x31, 0x8c, 0x70, 0x80

#define kMaskRandom41_9 \
  0x4e, 0x13, 0x84, 0xe1, 0x11, 0x00, \
  0x62, 0x38, 0xc6, 0x21, 0xa0, 0x80, \
  0x81, 0xe0, 0x78, 0x0e, 0x94, 0x00, \
  0xe1, 0x48, 0x5a, 0x15, 0x05, 0x00, \
  0x13, 0x94, 0xa5, 0x30, 0x06, 0x80, \
  0xb4, 0x2d, 0x0a, 0x42, 0x43, 0x00, \
  0x26, 0x89, 0xa1, 0x6a, 0x08, 0x80, \
  0x58, 0x56, 0x15, 0x84, 0x52, 0x00, \
  0x49, 0x86, 0x52, 0x98, 0x68, 0x00

#define kMaskRandom42_1 \
  0xff, 0xff, 0xff, 0xff, 0xff, 0xc0

#define kMaskRandom42_10 \
  0x4c, 0x19, 0x12, 0x60, 0xc8, 0x80, \
  0x51, 0x14, 0x52, 0x88, 0xa2, 0x80, \
  0xa0, 0x6a, 0x45, 0x03, 0x52, 0x00, \
  0x04, 0xc1, 0x30, 0x26, 0x09, 0x80, \
  0x03, 0xb4, 0x00, 0x1d, 0xa0, 0x00, \
  0x86, 0x20, 0x94, 0x31, 0x04, 0x80, \
  0x29, 0x08, 0x49, 0x48, 0x42, 0x40, \
  0x42, 0x43, 0x0a, 0x12, 0x18, 0x40, \
  0x98, 0x12, 0x84, 0xc0, 0x94, 0x00, \
  0x30, 0x84, 0xa9, 0x84, 0x25, 0x40

#define kMaskRandom42_11 \
  0xc6, 0x21, 0xa6, 0x31, 0x0d, 0x00, \
  0x23, 0x88, 0xc9, 0x1c, 0x46, 0x40, \
  0x1a, 0x45, 0x88, 0xd2, 0x2c, 0x40, \
  0x24, 0xd3, 0x09, 0x26, 0x98, 0x40, \
  0x71, 0x10, 0x73, 0x88, 0x83, 0x80, \
  0x0e, 0x19, 0x10, 0x70, 0xc8, 0x80, \
  0x33, 0x14, 0x51, 0x98, 0xa2, 0x80, \
  0x10, 0xc3, 0x28, 0x86, 0x19, 0x40, \
  0x45, 0x68, 0x4a, 0x2b, 0x42, 0x40, \
  0x88, 0x84, 0xac, 0x44, 0x25, 0x40, \
  0xe0, 0x22, 0x97, 0x01, 0x14, 0x80

#define kMaskRandom42_12 \
  0x0e, 0x19, 0x10, 0x70, 0xc8, 0x80, \
  0x33, 0x14, 0x51, 0x98, 0xa2, 0x80, \
  0x10, 0xc3, 0x28, 0x86, 0x19, 0x40, \
  0x45, 0x68, 0x4a, 0x2b, 0x42, 0x40, \
  0x88, 0x84, 0xac, 0x44, 0x25, 0x40, \
  0xe0, 0x22, 0x97, 0x01, 0x14, 0x80, \
  0xc6, 0x21, 0xa6, 0x31, 0x0d, 0x00, \
  0x23, 0x88, 0xc9, 0x1c, 0x46, 0x40, \
  0x1a, 0x45, 0x88, 0xd2, 0x2c, 0x40, \
  0x24, 0xd3, 0x09, 0x26, 0x98, 0x40, \
  0x71, 0x10, 0x73, 0x88, 0x83, 0x80, \
  0xa0, 0x65, 0x1d, 0x03, 0x28, 0xc0

#define kMaskRandom42_13 \
  0x0e, 0x19, 0x10, 0x70, 0xc8, 0x80, \
  0x33, 0x14, 0x51, 0x98, 0xa2, 0x80, \
  0x10, 0xc3, 0x28, 0x86, 0x19, 0x40, \
  0x45, 0x68, 0x4a, 0x2b, 0x42, 0x40, \
  0x88, 0x84, 0xac, 0x44, 0x25, 0x40, \
  0xe0, 0x22, 0x97, 0x01, 0x14, 0x80, \
  0x46, 0x11, 0x92, 0x30, 0x8c, 0x80, \
  0x33, 0x0c, 0xc9, 0x98, 0x66, 0x40, \
  0x10, 0xe4, 0x60, 0x87, 0x23, 0x00, \
  0x0c, 0x69, 0x08, 0x63, 0x48, 0x40, \
  0x28, 0x94, 0x29, 0x44, 0xa1, 0x40, \
  0x94, 0x21, 0x34, 0xa1, 0x09, 0x80, \
  0xc1, 0x02, 0x5e, 0x08, 0x12, 0xc0

#define kMaskRandom42_14 \
  0x46, 0x11, 0x92, 0x30, 0x8c, 0x80, \
  0x33, 0x0c, 0xc9, 0x98, 0x66, 0x40, \
  0x10, 0xe4, 0x60, 0x87, 0x23, 0x00, \
  0x0c, 0x69, 0x08, 0x63, 0x48, 0x40, \
  0x28, 0x94, 0x29, 0x44, 0xa1, 0x40, \
  0x94, 0x21, 0x34, 0xa1, 0x09, 0x80, \
  0xc1, 0x02, 0x5e, 0x08, 0x12, 0xc0, \
  0x0e, 0x19, 0x10, 0x70, 0xc8, 0x80, \
  0x33, 0x14, 0x51, 0x98, 0xa2, 0x80, \
  0x10, 0xc3, 0x28, 0x86, 0x19, 0x40, \
  0x45, 0x68, 0x4a, 0x2b, 0x42, 0x40, \
  0x88, 0x84, 0xac, 0x44, 0x25, 0x40, \
  0xe0, 0x22, 0x97, 0x01, 0x14, 0x80, \
  0x4d, 0xd0, 0xc2, 0x6e, 0x86, 0x00

#define kMaskRandom42_15 \
  0x46, 0x11, 0x92, 0x30, 0x8c, 0x80, \
  0x33, 0x0c, 0xc9, 0x98, 0x66, 0x40, \
  0x10, 0xe4, 0x60, 0x87, 0x23, 0x00, \
  0x0c, 0x69, 0x08, 0x63, 0x48, 0x40, \
  0x28, 0x94, 0x29, 0x44, 0xa1, 0x40, \
  0x94, 0x21, 0x34, 0xa1, 0x09, 0x80, \
  0xc1, 0x02, 0x5e, 0x08, 0x12, 0xc0, \
  0x2c, 0x03, 0x21, 0x60, 0x19, 0x00, \
  0x81, 0xa0, 0x1c, 0x0d, 0x00, 0xc0, \
  0xa0, 0x68, 0x25, 0x03, 0x41, 0x00, \
  0x05, 0x41, 0x50, 0x2a, 0x0a, 0x80, \
  0x18, 0x90, 0xc0, 0xc4, 0x86, 0x00, \
  0xc2, 0x06, 0x86, 0x10, 0x34, 0x00, \
  0x22, 0x98, 0x09, 0x14, 0xc0, 0x40, \
  0x50, 0x45, 0x0a, 0x82, 0x28, 0x40

#define kMaskRandom42_16 \
  0x2c, 0x03, 0x21, 0x60, 0x19, 0x00, \
  0x81, 0xa0, 0x1c, 0x0d, 0x00, 0xc0, \
  0xa0, 0x68, 0x25, 0x03, 0x41, 0x00, \
  0x05, 0x41, 0x50, 0x2a, 0x0a, 0x80, \
  0x18, 0x90, 0xc0, 0xc4, 0x86, 0x00, \
  0xc2, 0x06, 0x86, 0x10, 0x34, 0x00, \
  0x22, 0x98, 0x09, 0x14, 0xc0, 0x40, \
  0x50, 0x45, 0x0a, 0x82, 0x28, 0x40, \
  0x46, 0x11, 0x92, 0x30, 0x8c, 0x80, \
  0x33, 0x0c, 0xc9, 0x98, 0x66, 0x40, \
  0x10, 0xe4, 0x60, 0x87, 0x23, 0x00, \
  0x0c, 0x69, 0x08, 0x63, 0x48, 0x40, \
  0x28, 0x94, 0x29, 0x44, 0xa1, 0x40, \
  0x94, 0x21, 0x34, 0xa1, 0x09, 0x80, \
  0xc1, 0x02, 0x5e, 0x08, 0x12, 0xc0, \
  0x3b, 0xf5, 0x39, 0xdf, 0xa9, 0xc0

#define kMaskRandom42_17 \
  0x2c, 0x03, 0x21, 0x60, 0x19, 0x00, \
  0x81, 0xa0, 0x1c, 0x0d, 0x00, 0xc0, \
  0xa0, 0x68, 0x25, 0x03, 0x41, 0x00, \
  0x05, 0x41, 0x50, 0x2a, 0x0a, 0x80, \
  0x18, 0x90, 0xc0, 0xc4, 0x86, 0x00, \
  0xc2, 0x06, 0x86, 0x10, 0x34, 0x00, \
  0x22, 0x98, 0x09, 0x14, 0xc0, 0x40, \
  0x50, 0x45, 0x0a, 0x82, 0x28, 0x40, \
  0x4e, 0x11, 0x92, 0x70, 0x8c, 0x80, \
  0xe3, 0x18, 0x9f, 0x18, 0xc4, 0xc0, \
  0x81, 0xe3, 0x04, 0x0f, 0x18, 0x00, \
  0x21, 0x40, 0x59, 0x0a, 0x02, 0xc0, \
  0x52, 0x81, 0xe2, 0x94, 0x0f, 0x00, \
  0xb4, 0x28, 0x25, 0xa1, 0x41, 0x00, \
  0x26, 0x86, 0x29, 0x34, 0x31, 0x40, \
  0x58, 0x64, 0x42, 0xc3, 0x22, 0x00, \
  0x19, 0x9e, 0x00, 0xcc, 0xf0, 0x00

#define kMaskRandom42_18 \
  0x4e, 0x11, 0x92, 0x70, 0x8c, 0x80, \
  0xe3, 0x18, 0x9f, 0x18, 0xc4, 0xc0, \
  0x81, 0xe3, 0x04, 0x0f, 0x18, 0x00, \
  0x21, 0x40, 0x59, 0x0a, 0x02, 0xc0, \
  0x52, 0x81, 0xe2, 0x94, 0x0f, 0x00, \
  0xb4, 0x28, 0x25, 0xa1, 0x41, 0x00, \
  0x26, 0x86, 0x29, 0x34, 0x31, 0x40, \
  0x58, 0x64, 0x42, 0xc3, 0x22, 0x00, \
  0x19, 0x9e, 0x00, 0xcc, 0xf0, 0x00, \
  0x2c, 0x03, 0x21, 0x60, 0x19, 0x00, \
  0x81, 0xa0, 0x1c, 0x0d, 0x00, 0xc0, \
  0xa0, 0x68, 0x25, 0x03, 0x41, 0x00, \
  0x05, 0x41, 0x50, 0x2a, 0x0a, 0x80, \
  0x18, 0x90, 0xc0, 0xc4, 0x86, 0x00, \
  0xc2, 0x06, 0x86, 0x10, 0x34, 0x00, \
  0x22, 0x98, 0x09, 0x14, 0xc0, 0x40, \
  0x50, 0x45, 0x0a, 0x82, 0x28, 0x40, \
  0x5a, 0x56, 0x5a, 0xd2, 0xb2, 0xc0

#define kMaskRandom42_19 \
  0x4e, 0x11, 0x92, 0x70, 0x8c, 0x80, \
  0xe3, 0x18, 0x9f, 0x18, 0xc4, 0xc0, \
  0x81, 0xe3, 0x04, 0x0f, 0x18, 0x00, \
  0x21, 0x40, 0x59, 0x0a, 0x02, 0xc0, \
  0x52, 0x81, 0xe2, 0x94, 0x0f, 0x00, \
  0xb4, 0x28, 0x25, 0xa1, 0x41, 0x00, \
  0x26, 0x86, 0x29, 0x34, 0x31, 0x40, \
  0x58, 0x64, 0x42, 0xc3, 0x22, 0x00, \
  0x19, 0x9e, 0x00, 0xcc, 0xf0, 0x00, \
  0x4c, 0x19, 0x12, 0x60, 0xc8, 0x80, \
  0x51, 0x14, 0x52, 0x88, 0xa2, 0x80, \
  0xa0, 0x6a, 0x45, 0x03, 0x52, 0x00, \
  0x04, 0xc1, 0x30, 0x26, 0x09, 0x80, \
  0x03, 0xb4, 0x00, 0x1d, 0xa0, 0x00, \
  0x86, 0x20, 0x94, 0x31, 0x04, 0x80, \
  0x29, 0x08, 0x49, 0x48, 0x42, 0x40, \
  0x42, 0x43, 0x0a, 0x12, 0x18, 0x40, \
  0x98, 0x12, 0x84, 0xc0, 0x94, 0x00, \
  0x30, 0x84, 0xa9, 0x84, 0x25, 0x40

#define kMaskRandom42_2 \
  0xee, 0x3b, 0x37, 0x71, 0xd9, 0x80, \
  0x99, 0xe6, 0xec, 0xcf, 0x37, 0x40

#define kMaskRandom42_20 \
  0x4c, 0x19, 0x12, 0x60, 0xc8, 0x80, \
  0x51, 0x14, 0x52, 0x88, 0xa2, 0x80, \
  0xa0, 0x6a, 0x45, 0x03, 0x52, 0x00, \
  0x04, 0xc1, 0x30, 0x26, 0x09, 0x80, \
  0x03, 0xb4, 0x00, 0x1d, 0xa0, 0x00, \
  0x86, 0x20, 0x94, 0x31, 0x04, 0x80, \
  0x29, 0x08, 0x49, 0x48, 0x42, 0x40, \
  0x42, 0x43, 0x0a, 0x12, 0x18, 0x40, \
  0x98, 0x12, 0x84, 0xc0, 0x94, 0x00, \
  0x30, 0x84, 0xa9, 0x84, 0x25, 0x40, \
  0x4e, 0x11, 0x92, 0x70, 0x8c, 0x80, \
  0xe3, 0x18, 0x9f, 0x18, 0xc4, 0xc0, \
  0x81, 0xe3, 0x04, 0x0f, 0x18, 0x00, \
  0x21, 0x40, 0x59, 0x0a, 0x02, 0xc0, \
  0x52, 0x81, 0xe2, 0x94, 0x0f, 0x00, \
  0xb4, 0x28, 0x25, 0xa1, 0x41, 0x00, \
  0x26, 0x86, 0x29, 0x34, 0x31, 0x40, \
  0x58, 0x64, 0x42, 0xc3, 0x22, 0x00, \
  0x19, 0x9e, 0x00, 0xcc, 0xf0, 0x00, \
  0x2a, 0x03, 0x31, 0x50, 0x19, 0x80

#define kMaskRandom42_21 \
  0x4c, 0x19, 0x12, 0x60, 0xc8, 0x80, \
  0x51, 0x14, 0x52, 0x88, 0xa2, 0x80, \
  0xa0, 0x6a, 0x45, 0x03, 0x52, 0x00, \
  0x04, 0xc1, 0x30, 0x26, 0x09, 0x80, \
  0x03, 0xb4, 0x00, 0x1d, 0xa0, 0x00, \
  0x86, 0x20, 0x94, 0x31, 0x04, 0x80, \
  0x29, 0x08, 0x49, 0x48, 0x42, 0x40, \
  0x42, 0x43, 0x0a, 0x12, 0x18, 0x40, \
  0x98, 0x12, 0x84, 0xc0, 0x94, 0x00, \
  0x30, 0x84, 0xa9, 0x84, 0x25, 0x40, \
  0x4c, 0x11, 0x92, 0x60, 0x8c, 0x80, \
  0x51, 0x0c, 0xca, 0x88, 0x66, 0x40, \
  0xa0, 0x66, 0x45, 0x03, 0x32, 0x00, \
  0x04, 0xc1, 0x60, 0x26, 0x0b, 0x00, \
  0x03, 0xa0, 0x28, 0x1d, 0x01, 0x40, \
  0x86, 0x21, 0x14, 0x31, 0x08, 0x80, \
  0x29, 0x10, 0x19, 0x48, 0x80, 0xc0, \
  0x42, 0x42, 0xa2, 0x12, 0x15, 0x00, \
  0x98, 0x1a, 0x04, 0xc0, 0xd0, 0x00, \
  0x30, 0x84, 0x09, 0x84, 0x20, 0x40, \
  0xdf, 0x4c, 0x16, 0xfa, 0x60, 0x80

#define kMaskRandom42_22 \
  0xc6, 0x21, 0xa6, 0x31, 0x0d, 0x00, \
  0x23, 0x88, 0xc9, 0x1c, 0x46, 0x40, \
  0x1a, 0x45, 0x88, 0xd2, 0x2c, 0x40, \
  0x24, 0xd3, 0x09, 0x26, 0x98, 0x40, \
  0x71, 0x10, 0x73, 0x88, 0x83, 0x80, \
  0x0e, 0x19, 0x10, 0x70, 0xc8, 0x80, \
  0x33, 0x14, 0x51, 0x98, 0xa2, 0x80, \
  0x10, 0xc3, 0x28, 0x86, 0x19, 0x40, \
  0x45, 0x68, 0x4a, 0x2b, 0x42, 0x40, \
  0x88, 0x84, 0xac, 0x44, 0x25, 0x40, \
  0xe0, 0x22, 0x97, 0x01, 0x14, 0x80, \
  0x4c, 0x19, 0x12, 0x60, 0xc8, 0x80, \
  0x51, 0x14, 0x52, 0x88, 0xa2, 0x80, \
  0xa0, 0x6a, 0x45, 0x03, 0x52, 0x00, \
  0x04, 0xc1, 0x30, 0x26, 0x09, 0x80, \
  0x03, 0xb4, 0x00, 0x1d, 0xa0, 0x00, \
  0x86, 0x20, 0x94, 0x31, 0x04, 0x80, \
  0x29, 0x08, 0x49, 0x48, 0x42, 0x40, \
  0x42, 0x43, 0x0a, 0x12, 0x18, 0x40, \
  0x98, 0x12, 0x84, 0xc0, 0x94, 0x00, \
  0x30, 0x84, 0xa9, 0x84, 0x25, 0x40, \
  0xdb, 0x36, 0xb0, 0x33, 0x14, 0x80

#define kMaskRandom42_23 \
  0xc6, 0x21, 0xa6, 0x31, 0x0d, 0x00, \
  0x23, 0x88, 0xc9, 0x1c, 0x46, 0x40, \
  0x1a, 0x45, 0x88, 0xd2, 0x2c, 0x40, \
  0x24, 0xd3, 0x09, 0x26, 0x98, 0x40, \
  0x71, 0x10, 0x73, 0x88, 0x83, 0x80, \
  0x0e, 0x19, 0x10, 0x70, 0xc8, 0x80, \
  0x33, 0x14, 0x51, 0x98, 0xa2, 0x80, \
  0x10, 0xc3, 0x28, 0x86, 0x19, 0x40, \
  0x45, 0x68, 0x4a, 0x2b, 0x42, 0x40, \
  0x88, 0x84, 0xac, 0x44, 0x25, 0x40, \
  0xe0, 0x22, 0x97, 0x01, 0x14, 0x80, \
  0x0e, 0x19, 0x10, 0x70, 0xc8, 0x80, \
  0x33, 0x14, 0x51, 0x98, 0xa2, 0x80, \
  0x10, 0xc3, 0x28, 0x86, 0x19, 0x40, \
  0x45, 0x68, 0x4a, 0x2b, 0x42, 0x40, \
  0x88, 0x84, 0xac, 0x44, 0x25, 0x40, \
  0xe0, 0x22, 0x97, 0x01, 0x14, 0x80, \
  0xc6, 0x21, 0xa6, 0x31, 0x0d, 0x00, \
  0x23, 0x88, 0xc9, 0x1c, 0x46, 0x40, \
  0x1a, 0x45, 0x88, 0xd2, 0x2c, 0x40, \
  0x24, 0xd3, 0x09, 0x26, 0x98, 0x40, \
  0x71, 0x10, 0x73, 0x88, 0x83, 0x80, \
  0xa0, 0x65, 0x1d, 0x03, 0x28, 0xc0

#define kMaskRandom42_24 \
  0x0e, 0x19, 0x10, 0x70, 0xc8, 0x80, \
  0x33, 0x14, 0x51, 0x98, 0xa2, 0x80, \
  0x10, 0xc3, 0x28, 0x86, 0x19, 0x40, \
  0x45, 0x68, 0x4a, 0x2b, 0x42, 0x40, \
  0x88, 0x84, 0xac, 0x44, 0x25, 0x40, \
  0xe0, 0x22, 0x97, 0x01, 0x14, 0x80, \
  0xc6, 0x21, 0xa6, 0x31, 0x0d, 0x00, \
  0x23, 0x88, 0xc9, 0x1c, 0x46, 0x40, \
  0x1a, 0x45, 0x88, 0xd2, 0x2c, 0x40, \
  0x24, 0xd3, 0x09, 0x26, 0x98, 0x40, \
  0x71, 0x10, 0x73, 0x88, 0x83, 0x80, \
  0xa0, 0x65, 0x1d, 0x03, 0x28, 0xc0, \
  0xc6, 0x21, 0xa6, 0x31, 0x0d, 0x00, \
  0x23, 0x88, 0xc9, 0x1c, 0x46, 0x40, \
  0x1a, 0x45, 0x88, 0xd2, 0x2c, 0x40, \
  0x24, 0xd3, 0x09, 0x26, 0x98, 0x40, \
  0x71, 0x10, 0x73, 0x88, 0x83, 0x80, \
  0x0e, 0x19, 0x10, 0x70, 0xc8, 0x80, \
  0x33, 0x14, 0x51, 0x98, 0xa2, 0x80, \
  0x10, 0xc3, 0x28, 0x86, 0x19, 0x40, \
  0x45, 0x68, 0x4a, 0x2b, 0x42, 0x40, \
  0x88, 0x84, 0xac, 0x44, 0x25, 0x40, \
  0xe0, 0x22, 0x97, 0x01, 0x14, 0x80, \
  0x2e, 0x1c, 0x92, 0xbb, 0x07, 0xc0

#define kMaskRandom42_25 \
  0x0e, 0x19, 0x10, 0x70, 0xc8, 0x80, \
  0x33, 0x14, 0x51, 0x98, 0xa2, 0x80, \
  0x10, 0xc3, 0x28, 0x86, 0x19, 0x40, \
  0x45, 0x68, 0x4a, 0x2b, 0x42, 0x40, \
  0x88, 0x84, 0xac, 0x44, 0x25, 0x40, \
  0xe0, 0x22, 0x97, 0x01, 0x14, 0x80, \
  0xc6, 0x21, 0xa6, 0x31, 0x0d, 0x00, \
  0x23, 0x88, 0xc9, 0x1c, 0x46, 0x40, \
  0x1a, 0x45, 0x88, 0xd2, 0x2c, 0x40, \
  0x24, 0xd3, 0x09, 0x26, 0x98, 0x40, \
  0x71, 0x10, 0x73, 0x88, 0x83, 0x80, \
  0xa0, 0x65, 0x1d, 0x03, 0x28, 0xc0, \
  0x0e, 0x19, 0x10, 0x70, 0xc8, 0x80, \
  0x33, 0x14, 0x51, 0x98, 0xa2, 0x80, \
  0x10, 0xc3, 0x28, 0x86, 0x19, 0x40, \
  0x45, 0x68, 0x4a, 0x2b, 0x42, 0x40, \
  0x88, 0x84, 0xac, 0x44, 0x25, 0x40, \
  0xe0, 0x22, 0x97, 0x01, 0x14, 0x80, \
  0x46, 0x11, 0x92, 0x30, 0x8c, 0x80, \
  0x33, 0x0c, 0xc9, 0x98, 0x66, 0x40, \
  0x10, 0xe4, 0x60, 0x87, 0x23, 0x00, \
  0x0c, 0x69, 0x08, 0x63, 0x48, 0x40, \
  0x28, 0x94, 0x29, 0x44, 0xa1, 0x40, \
  0x94, 0x21, 0x34, 0xa1, 0x09, 0x80, \
  0xc1, 0x02, 0x5e, 0x08, 0x12, 0xc0

#define kMaskRandom42_26 \
  0x0e, 0x19, 0x10, 0x70, 0xc8, 0x80, \
  0x33, 0x14, 0x51, 0x98, 0xa2, 0x80, \
  0x10, 0xc3, 0x28, 0x86, 0x19, 0x40, \
  0x45, 0x68, 0x4a, 0x2b, 0x42, 0x40, \
  0x88, 0x84, 0xac, 0x44, 0x25, 0x40, \
  0xe0, 0x22, 0x97, 0x01, 0x14, 0x80, \
  0x46, 0x11, 0x92, 0x30, 0x8c, 0x80, \
  0x33, 0x0c, 0xc9, 0x98, 0x66, 0x40, \
  0x10, 0xe4, 0x60, 0x87, 0x23, 0x00, \
  0x0c, 0x69, 0x08, 0x63, 0x48, 0x40, \
  0x28, 0x94, 0x29, 0x44, 0xa1, 0x40, \
  0x94, 0x21, 0x34, 0xa1, 0x09, 0x80, \
  0xc1, 0x02, 0x5e, 0x08, 0x12, 0xc0, \
  0x0e, 0x19, 0x10, 0x70, 0xc8, 0x80, \
  0x33, 0x14, 0x51, 0x98, 0xa2, 0x80, \
  0x10, 0xc3, 0x28, 0x86, 0x19, 0x40, \
  0x45, 0x68, 0x4a, 0x2b, 0x42, 0x40, \
  0x88, 0x84, 0xac, 0x44, 0x25, 0x40, \
  0xe0, 0x22, 0x97, 0x01, 0x14, 0x80, \
  0xc6, 0x21, 0xa6, 0x31, 0x0d, 0x00, \
  0x23, 0x88, 0xc9, 0x1c, 0x46, 0x40, \
  0x1a, 0x45, 0x88, 0xd2, 0x2c, 0x40, \
  0x24, 0xd3, 0x09, 0x26, 0x98, 0x40, \
  0x71, 0x10, 0x73, 0x88, 0x83, 0x80, \
  0xa0, 0x65, 0x1d, 0x03, 0x28, 0xc0, \
  0xb8, 0x41, 0xed, 0xa3, 0x77, 0xc0

#define kMaskRandom42_27 \
  0x0e, 0x19, 0x10, 0x70, 0xc8, 0x80, \
  0x33, 0x14, 0x51, 0x98, 0xa2, 0x80, \
  0x10, 0xc3, 0x28, 0x86, 0x19, 0x40, \
  0x45, 0x68, 0x4a, 0x2b, 0x42, 0x40, \
  0x88, 0x84, 0xac, 0x44, 0x25, 0x40, \
  0xe0, 0x22, 0x97, 0x01, 0x14, 0x80, \
  0x46, 0x11, 0x92, 0x30, 0x8c, 0x80, \
  0x33, 0x0c, 0xc9, 0x98, 0x66, 0x40, \
  0x10, 0xe4, 0x60, 0x87, 0x23, 0x00, \
  0x0c, 0x69, 0x08, 0x63, 0x48, 0x40, \
  0x28, 0x94, 0x29, 0x44, 0xa1, 0x40, \
  0x94, 0x21, 0x34, 0xa1, 0x09, 0x80, \
  0xc1, 0x02, 0x5e, 0x08, 0x12, 0xc0, \
  0x46, 0x11, 0x92, 0x30, 0x8c, 0x80, \
  0x33, 0x0c, 0xc9, 0x98, 0x66, 0x40, \
  0x10, 0xe4, 0x60, 0x87, 0x23, 0x00, \
  0x0c, 0x69, 0x08, 0x63, 0x48, 0x40, \
  0x28, 0x94, 0x29, 0x44, 0xa1, 0x40, \
  0x94, 0x21, 0x34, 0xa1, 0x09, 0x80, \
  0xc1, 0x02, 0x5e, 0x08, 0x12, 0xc0, \
  0x0e, 0x19, 0x10, 0x70, 0xc8, 0x80, \
  0x33, 0x14, 0x51, 0x98, 0xa2, 0x80, \
  0x10, 0xc3, 0x28, 0x86, 0x19, 0x40, \
  0x45, 0x68, 0x4a, 0x2b, 0x42, 0x40, \
  0x88, 0x84, 0xac, 0x44, 0x25, 0x40, \
  0xe0, 0x22, 0x97, 0x01, 0x14, 0x80, \
  0x4d, 0xd0, 0xc2, 0x6e, 0x86, 0x00

#define kMaskRandom42_28 \
  0x46, 0x11, 0x92, 0x30, 0x8c, 0x80, \
  0x33, 0x0c, 0xc9, 0x98, 0x66, 0x40, \
  0x10, 0xe4, 0x60, 0x87, 0x23, 0x00, \
  0x0c, 0x69, 0x08, 0x63, 0x48, 0x40, \
  0x28, 0x94, 0x29, 0x44, 0xa1, 0x40, \
  0x94, 0x21, 0x34, 0xa1, 0x09, 0x80, \
  0xc1, 0x02, 0x5e, 0x08, 0x12, 0xc0, \
  0x0e, 0x19, 0x10, 0x70, 0xc8, 0x80, \
  0x33, 0x14, 0x51, 0x98, 0xa2, 0x80, \
  0x10, 0xc3, 0x28, 0x86, 0x19, 0x40, \
  0x45, 0x68, 0x4a, 0x2b, 0x42, 0x40, \
  0x88, 0x84, 0xac, 0x44, 0x25, 0x40, \
  0xe0, 0x22, 0x97, 0x01, 0x14, 0x80, \
  0x4d, 0xd0, 0xc2, 0x6e, 0x86, 0x00, \
  0x0e, 0x19, 0x10, 0x70, 0xc8, 0x80, \
  0x33, 0x14, 0x51, 0x98, 0xa2, 0x80, \
  0x10, 0xc3, 0x28, 0x86, 0x19, 0x40, \
  0x45, 0x68, 0x4a, 0x2b, 0x42, 0x40, \
  0x88, 0x84, 0xac, 0x44, 0x25, 0x40, \
  0xe0, 0x22, 0x97, 0x01, 0x14, 0x80, \
  0x46, 0x11, 0x92, 0x30, 0x8c, 0x80, \
  0x33, 0x0c, 0xc9, 0x98, 0x66, 0x40, \
  0x10, 0xe4, 0x60, 0x87, 0x23, 0x00, \
  0x0c, 0x69, 0x08, 0x63, 0x48, 0x40, \
  0x28, 0x94, 0x29, 0x44, 0xa1, 0x40, \
  0x94, 0x21, 0x34, 0xa1, 0x09, 0x80, \
  0xc1, 0x02, 0x5e, 0x08, 0x12, 0xc0, \
  0xc3, 0x3c, 0x56, 0xc2, 0x30, 0x40

#define kMaskRandom42_29 \
  0x46, 0x11, 0x92, 0x30, 0x8c, 0x80, \
  0x33, 0x0c, 0xc9, 0x98, 0x66, 0x40, \
  0x10, 0xe4, 0x60, 0x87, 0x23, 0x00, \
  0x0c, 0x69, 0x08, 0x63, 0x48, 0x40, \
  0x28, 0x94, 0x29, 0x44, 0xa1, 0x40, \
  0x94, 0x21, 0x34, 0xa1, 0x09, 0x80, \
  0xc1, 0x02, 0x5e, 0x08, 0x12, 0xc0, \
  0x0e, 0x19, 0x10, 0x70, 0xc8, 0x80, \
  0x33, 0x14, 0x51, 0x98, 0xa2, 0x80, \
  0x10, 0xc3, 0x28, 0x86, 0x19, 0x40, \
  0x45, 0x68, 0x4a, 0x2b, 0x42, 0x40, \
  0x88, 0x84, 0xac, 0x44, 0x25, 0x40, \
  0xe0, 0x22, 0x97, 0x01, 0x14, 0x80, \
  0x4d, 0xd0, 0xc2, 0x6e, 0x86, 0x00, \
  0x46, 0x11, 0x92, 0x30, 0x8c, 0x80, \
  0x33, 0x0c, 0xc9, 0x98, 0x66, 0x40, \
  0x10, 0xe4, 0x60, 0x87, 0x23, 0x00, \
  0x0c, 0x69, 0x08, 0x63, 0x48, 0x40, \
  0x28, 0x94, 0x29, 0x44, 0xa1, 0x40, \
  0x94, 0x21, 0x34, 0xa1, 0x09, 0x80, \
  0xc1, 0x02, 0x5e, 0x08, 0x12, 0xc0, \
  0x2c, 0x03, 0x21, 0x60, 0x19, 0x00, \
  0x81, 0xa0, 0x1c, 0x0d, 0x00, 0xc0, \
  0xa0, 0x68, 0x25, 0x03, 0x41, 0x00, \
  0x05, 0x41, 0x50, 0x2a, 0x0a, 0x80, \
  0x18, 0x90, 0xc0, 0xc4, 0x86, 0x00, \
  0xc2, 0x06, 0x86, 0x10, 0x34, 0x00, \
  0x22, 0x98, 0x09, 0x14, 0xc0, 0x40, \
  0x50, 0x45, 0x0a, 0x82, 0x28, 0x40

#define kMaskRandom42_3 \
  0xce, 0x32, 0xb6, 0x71, 0x95, 0x80, \
  0x55, 0xdc, 0x52, 0xae, 0xe2, 0x80, \
  0xa8, 0xed, 0x8d, 0x47, 0x6c, 0x40

#define kMaskRandom42_30 \
  0x46, 0x11, 0x92, 0x30, 0x8c, 0x80, \
  0x33, 0x0c, 0xc9, 0x98, 0x66, 0x40, \
  0x10, 0xe4, 0x60, 0x87, 0x23, 0x00, \
  0x0c, 0x69, 0x08, 0x63, 0x48, 0x40, \
  0x28, 0x94, 0x29, 0x44, 0xa1, 0x40, \
  0x94, 0x21, 0x34, 0xa1, 0x09, 0x80, \
  0xc1, 0x02, 0x5e, 0x08, 0x12, 0xc0, \
  0x2c, 0x03, 0x21, 0x60, 0x19, 0x00, \
  0x81, 0xa0, 0x1c, 0x0d, 0x00, 0xc0, \
  0xa0, 0x68, 0x25, 0x03, 0x41, 0x00, \
  0x05, 0x41, 0x50, 0x2a, 0x0a, 0x80, \
  0x18, 0x90, 0xc0, 0xc4, 0x86, 0x00, \
  0xc2, 0x06, 0x86, 0x10, 0x34, 0x00, \
  0x22, 0x98, 0x09, 0x14, 0xc0, 0x40, \
  0x50, 0x45, 0x0a, 0x82, 0x28, 0x40, \
  0x46, 0x11, 0x92, 0x30, 0x8c, 0x80, \
  0x33, 0x0c, 0xc9, 0x98, 0x66, 0x40, \
  0x10, 0xe4, 0x60, 0x87, 0x23, 0x00, \
  0x0c, 0x69, 0x08, 0x63, 0x48, 0x40, \
  0x28, 0x94, 0x29, 0x44, 0xa1, 0x40, \
  0x94, 0x21, 0x34, 0xa1, 0x09, 0x80, \
  0xc1, 0x02, 0x5e, 0x08, 0x12, 0xc0, \
  0x0e, 0x19, 0x10, 0x70, 0xc8, 0x80, \
  0x33, 0x14, 0x51, 0x98, 0xa2, 0x80, \
  0x10, 0xc3, 0x28, 0x86, 0x19, 0x40, \
  0x45, 0x68, 0x4a, 0x2b, 0x42, 0x40, \
  0x88, 0x84, 0xac, 0x44, 0x25, 0x40, \
  0xe0, 0x22, 0x97, 0x01, 0x14, 0x80, \
  0x4d, 0xd0, 0xc2, 0x6e, 0x86, 0x00, \
  0xf5, 0xdd, 0x0d, 0x58, 0xeb, 0x00

#define kMaskRandom42_31 \
  0x46, 0x11, 0x92, 0x30, 0x8c, 0x80, \
  0x33, 0x0c, 0xc9, 0x98, 0x66, 0x40, \
  0x10, 0xe4, 0x60, 0x87, 0x23, 0x00, \
  0x0c, 0x69, 0x08, 0x63, 0x48, 0x40, \
  0x28, 0x94, 0x29, 0x44, 0xa1, 0x40, \
  0x94, 0x21, 0x34, 0xa1, 0x09, 0x80, \
  0xc1, 0x02, 0x5e, 0x08, 0x12, 0xc0, \
  0x2c, 0x03, 0x21, 0x60, 0x19, 0x00, \
  0x81, 0xa0, 0x1c, 0x0d, 0x00, 0xc0, \
  0xa0, 0x68, 0x25, 0x03, 0x41, 0x00, \
  0x05, 0x41, 0x50, 0x2a, 0x0a, 0x80, \
  0x18, 0x90, 0xc0, 0xc4, 0x86, 0x00, \
  0xc2, 0x06, 0x86, 0x10, 0x34, 0x00, \
  0x22, 0x98, 0x09, 0x14, 0xc0, 0x40, \
  0x50, 0x45, 0x0a, 0x82, 0x28, 0x40, \
  0x2c, 0x03, 0x21, 0x60, 0x19, 0x00, \
  0x81, 0xa0, 0x1c, 0x0d, 0x00, 0xc0, \
  0xa0, 0x68, 0x25, 0x03, 0x41, 0x00, \
  0x05, 0x41, 0x50, 0x2a, 0x0a, 0x80, \
  0x18, 0x90, 0xc0, 0xc4, 0x86, 0x00, \
  0xc2, 0x06, 0x86, 0x10, 0x34, 0x00, \
  0x22, 0x98, 0x09, 0x14, 0xc0, 0x40, \
  0x50, 0x45, 0x0a, 0x82, 0x28, 0x40, \
  0x46, 0x11, 0x92, 0x30, 0x8c, 0x80, \
  0x33, 0x0c, 0xc9, 0x98, 0x66, 0x40, \
  0x10, 0xe4, 0x60, 0x87, 0x23, 0x00, \
  0x0c, 0x69, 0x08, 0x63, 0x48, 0x40, \
  0x28, 0x94, 0x29, 0x44, 0xa1, 0x40, \
  0x94, 0x21, 0x34, 0xa1, 0x09, 0x80, \
  0xc1, 0x02, 0x5e, 0x08, 0x12, 0xc0, \
  0x3b, 0xf5, 0x39, 0xdf, 0xa9, 0xc0

#define kMaskRandom42_32 \
  0x2c, 0x03, 0x21, 0x60, 0x19, 0x00, \
  0x81, 0xa0, 0x1c, 0x0d, 0x00, 0xc0, \
  0xa0, 0x68, 0x25, 0x03, 0x41, 0x00, \
  0x05, 0x41, 0x50, 0x2a, 0x0a, 0x80, \
  0x18, 0x90, 0xc0, 0xc4, 0x86, 0x00, \
  0xc2, 0x06, 0x86, 0x10, 0x34, 0x00, \
  0x22, 0x98, 0x09, 0x14, 0xc0, 0x40, \
  0x50, 0x45, 0x0a, 0x82, 0x28, 0x40, \
  0x46, 0x11, 0x92, 0x30, 0x8c, 0x80, \
  0x33, 0x0c, 0xc9, 0x98, 0x66, 0x40, \
  0x10, 0xe4, 0x60, 0x87, 0x23, 0x00, \
  0x0c, 0x69, 0x08, 0x63, 0x48, 0x40, \
  0x28, 0x94, 0x29, 0x44, 0xa1, 0x40, \
  0x94, 0x21, 0x34, 0xa1, 0x09, 0x80, \
  0xc1, 0x02, 0x5e, 0x08, 0x12, 0xc0, \
  0x3b, 0xf5, 0x39, 0xdf, 0xa9, 0xc0, \
  0x46, 0x11, 0x92, 0x30, 0x8c, 0x80, \
  0x33, 0x0c, 0xc9, 0x98, 0x66, 0x40, \
  0x10, 0xe4, 0x60, 0x87, 0x23, 0x00, \
  0x0c, 0x69, 0x08, 0x63, 0x48, 0x40, \
  0x28, 0x94, 0x29, 0x44, 0xa1, 0x40, \
  0x94, 0x21, 0x34, 0xa1, 0x09, 0x80, \
  0xc1, 0x02, 0x5e, 0x08, 0x12, 0xc0, \
  0x2c, 0x03, 0x21, 0x60, 0x19, 0x00, \
  0x81, 0xa0, 0x1c, 0x0d, 0x00, 0xc0, \
  0xa0, 0x68, 0x25, 0x03, 0x41, 0x00, \
  0x05, 0x41, 0x50, 0x2a, 0x0a, 0x80, \
  0x18, 0x90, 0xc0, 0xc4, 0x86, 0x00, \
  0xc2, 0x06, 0x86, 0x10, 0x34, 0x00, \
  0x22, 0x98, 0x09, 0x14, 0xc0, 0x40, \
  0x50, 0x45, 0x0a, 0x82, 0x28, 0x40, \
  0xf9, 0x1f, 0xb6, 0xe1, 0x09, 0xc0

#define kMaskRandom42_33 \
  0x2c, 0x03, 0x21, 0x60, 0x19, 0x00, \
  0x81, 0xa0, 0x1c, 0x0d, 0x00, 0xc0, \
  0xa0, 0x68, 0x25, 0x03, 0x41, 0x00, \
  0x05, 0x41, 0x50, 0x2a, 0x0a, 0x80, \
  0x18, 0x90, 0xc0, 0xc4, 0x86, 0x00, \
  0xc2, 0x06, 0x86, 0x10, 0x34, 0x00, \
  0x22, 0x98, 0x09, 0x14, 0xc0, 0x40, \
  0x50, 0x45, 0x0a, 0x82, 0x28, 0x40, \
  0x46, 0x11, 0x92, 0x30, 0x8c, 0x80, \
  0x33, 0x0c, 0xc9, 0x98, 0x66, 0x40, \
  0x10, 0xe4, 0x60, 0x87, 0x23, 0x00, \
  0x0c, 0x69, 0x08, 0x63, 0x48, 0x40, \
  0x28, 0x94, 0x29, 0x44, 0xa1, 0x40, \
  0x94, 0x21, 0x34, 0xa1, 0x09, 0x80, \
  0xc1, 0x02, 0x5e, 0x08, 0x12, 0xc0, \
  0x3b, 0xf5, 0x39, 0xdf, 0xa9, 0xc0, \
  0x2c, 0x03, 0x21, 0x60, 0x19, 0x00, \
  0x81, 0xa0, 0x1c, 0x0d, 0x00, 0xc0, \
  0xa0, 0x68, 0x25, 0x03, 0x41, 0x00, \
  0x05, 0x41, 0x50, 0x2a, 0x0a, 0x80, \
  0x18, 0x90, 0xc0, 0xc4, 0x86, 0x00, \
  0xc2, 0x06, 0x86, 0x10, 0x34, 0x00, \
  0x22, 0x98, 0x09, 0x14, 0xc0, 0x40, \
  0x50, 0x45, 0x0a, 0x82, 0x28, 0x40, \
  0x4e, 0x11, 0x92, 0x70, 0x8c, 0x80, \
  0xe3, 0x18, 0x9f, 0x18, 0xc4, 0xc0, \
  0x81, 0xe3, 0x04, 0x0f, 0x18, 0x00, \
  0x21, 0x40, 0x59, 0x0a, 0x02, 0xc0, \
  0x52, 0x81, 0xe2, 0x94, 0x0f, 0x00, \
  0xb4, 0x28, 0x25, 0xa1, 0x41, 0x00, \
  0x26, 0x86, 0x29, 0x34, 0x31, 0x40, \
  0x58, 0x64, 0x42, 0xc3, 0x22, 0x00, \
  0x19, 0x9e, 0x00, 0xcc, 0xf0, 0x00

#define kMaskRandom42_34 \
  0x2c, 0x03, 0x21, 0x60, 0x19, 0x00, \
  0x81, 0xa0, 0x1c, 0x0d, 0x00, 0xc0, \
  0xa0, 0x68, 0x25, 0x03, 0x41, 0x00, \
  0x05, 0x41, 0x50, 0x2a, 0x0a, 0x80, \
  0x18, 0x90, 0xc0, 0xc4, 0x86, 0x00, \
  0xc2, 0x06, 0x86, 0x10, 0x34, 0x00, \
  0x22, 0x98, 0x09, 0x14, 0xc0, 0x40, \
  0x50, 0x45, 0x0a, 0x82, 0x28, 0x40, \
  0x4e, 0x11, 0x92, 0x70, 0x8c, 0x80, \
  0xe3, 0x18, 0x9f, 0x18, 0xc4, 0xc0, \
  0x81, 0xe3, 0x04, 0x0f, 0x18, 0x00, \
  0x21, 0x40, 0x59, 0x0a, 0x02, 0xc0, \
  0x52, 0x81, 0xe2, 0x94, 0x0f, 0x00, \
  0xb4, 0x28, 0x25, 0xa1, 0x41, 0x00, \
  0x26, 0x86, 0x29, 0x34, 0x31, 0x40, \
  0x58, 0x64, 0x42, 0xc3, 0x22, 0x00, \
  0x19, 0x9e, 0x00, 0xcc, 0xf0, 0x00, \
  0x2c, 0x03, 0x21, 0x60, 0x19, 0x00, \
  0x81, 0xa0, 0x1c, 0x0d, 0x00, 0xc0, \
  0xa0, 0x68, 0x25, 0x03, 0x41, 0x00, \
  0x05, 0x41, 0x50, 0x2a, 0x0a, 0x80, \
  0x18, 0x90, 0xc0, 0xc4, 0x86, 0x00, \
  0xc2, 0x06, 0x86, 0x10, 0x34, 0x00, \
  0x22, 0x98, 0x09, 0x14, 0xc0, 0x40, \
  0x50, 0x45, 0x0a, 0x82, 0x28, 0x40, \
  0x46, 0x11, 0x92, 0x30, 0x8c, 0x80, \
  0x33, 0x0c, 0xc9, 0x98, 0x66, 0x40, \
  0x10, 0xe4, 0x60, 0x87, 0x23, 0x00, \
  0x0c, 0x69, 0x08, 0x63, 0x48, 0x40, \
  0x28, 0x94, 0x29, 0x44, 0xa1, 0x40, \
  0x94, 0x21, 0x34, 0xa1, 0x09, 0x80, \
  0xc1, 0x02, 0x5e, 0x08, 0x12, 0xc0, \
  0x3b, 0xf5, 0x39, 0xdf, 0xa9, 0xc0, \
  0xf8, 0xbf, 0xf6, 0x76, 0x1b, 0x80

#define kMaskRandom42_35 \
  0x2c, 0x03, 0x21, 0x60, 0x19, 0x00, \
  0x81, 0xa0, 0x1c, 0x0d, 0x00, 0xc0, \
  0xa0, 0x68, 0x25, 0x03, 0x41, 0x00, \
  0x05, 0x41, 0x50, 0x2a, 0x0a, 0x80, \
  0x18, 0x90, 0xc0, 0xc4, 0x86, 0x00, \
  0xc2, 0x06, 0x86, 0x10, 0x34, 0x00, \
  0x22, 0x98, 0x09, 0x14, 0xc0, 0x40, \
  0x50, 0x45, 0x0a, 0x82, 0x28, 0x40, \
  0x4e, 0x11, 0x92, 0x70, 0x8c, 0x80, \
  0xe3, 0x18, 0x9f, 0x18, 0xc4, 0xc0, \
  0x81, 0xe3, 0x04, 0x0f, 0x18, 0x00, \
  0x21, 0x40, 0x59, 0x0a, 0x02, 0xc0, \
  0x52, 0x81, 0xe2, 0x94, 0x0f, 0x00, \
  0xb4, 0x28, 0x25, 0xa1, 0x41, 0x00, \
  0x26, 0x86, 0x29, 0x34, 0x31, 0x40, \
  0x58, 0x64, 0x42, 0xc3, 0x22, 0x00, \
  0x19, 0x9e, 0x00, 0xcc, 0xf0, 0x00, \
  0x4e, 0x11, 0x92, 0x70, 0x8c, 0x80, \
  0xe3, 0x18, 0x9f, 0x18, 0xc4, 0xc0, \
  0x81, 0xe3, 0x04, 0x0f, 0x18, 0x00, \
  0x21, 0x40, 0x59, 0x0a, 0x02, 0xc0, \
  0x52, 0x81, 0xe2, 0x94, 0x0f, 0x00, \
  0xb4, 0x28, 0x25, 0xa1, 0x41, 0x00, \
  0x26, 0x86, 0x29, 0x34, 0x31, 0x40, \
  0x58, 0x64, 0x42, 0xc3, 0x22, 0x00, \
  0x19, 0x9e, 0x00, 0xcc, 0xf0, 0x00, \
  0x2c, 0x03, 0x21, 0x60, 0x19, 0x00, \
  0x81, 0xa0, 0x1c, 0x0d, 0x00, 0xc0, \
  0xa0, 0x68, 0x25, 0x03, 0x41, 0x00, \
  0x05, 0x41, 0x50, 0x2a, 0x0a, 0x80, \
  0x18, 0x90, 0xc0, 0xc4, 0x86, 0x00, \
  0xc2, 0x06, 0x86, 0x10, 0x34, 0x00, \
  0x22, 0x98, 0x09, 0x14, 0xc0, 0x40, \
  0x50, 0x45, 0x0a, 0x82, 0x28, 0x40, \
  0x5a, 0x56, 0x5a, 0xd2, 0xb2, 0xc0

#define kMaskRandom42_36 \
  0x4e, 0x11, 0x92, 0x70, 0x8c, 0x80, \
  0xe3, 0x18, 0x9f, 0x18, 0xc4, 0xc0, \
  0x81, 0xe3, 0x04, 0x0f, 0x18, 0x00, \
  0x21, 0x40, 0x59, 0x0a, 0x02, 0xc0, \
  0x52, 0x81, 0xe2, 0x94, 0x0f, 0x00, \
  0xb4, 0x28, 0x25, 0xa1, 0x41, 0x00, \
  0x26, 0x86, 0x29, 0x34, 0x31, 0x40, \
  0x58, 0x64, 0x42, 0xc3, 0x22, 0x00, \
  0x19, 0x9e, 0x00, 0xcc, 0xf0, 0x00, \
  0x2c, 0x03, 0x21, 0x60, 0x19, 0x00, \
  0x81, 0xa0, 0x1c, 0x0d, 0x00, 0xc0, \
  0xa0, 0x68, 0x25, 0x03, 0x41, 0x00, \
  0x05, 0x41, 0x50, 0x2a, 0x0a, 0x80, \
  0x18, 0x90, 0xc0, 0xc4, 0x86, 0x00, \
  0xc2, 0x06, 0x86, 0x10, 0x34, 0x00, \
  0x22, 0x98, 0x09, 0x14, 0xc0, 0x40, \
  0x50, 0x45, 0x0a, 0x82, 0x28, 0x40, \
  0x5a, 0x56, 0x5a, 0xd2, 0xb2, 0xc0, \
  0x2c, 0x03, 0x21, 0x60, 0x19, 0x00, \
  0x81, 0xa0, 0x1c, 0x0d, 0x00, 0xc0, \
  0xa0, 0x68, 0x25, 0x03, 0x41, 0x00, \
  0x05, 0x41, 0x50, 0x2a, 0x0a, 0x80, \
  0x18, 0x90, 0xc0, 0xc4, 0x86, 0x00, \
  0xc2, 0x06, 0x86, 0x10, 0x34, 0x00, \
  0x22, 0x98, 0x09, 0x14, 0xc0, 0x40, \
  0x50, 0x45, 0x0a, 0x82, 0x28, 0x40, \
  0x4e, 0x11, 0x92, 0x70, 0x8c, 0x80, \
  0xe3, 0x18, 0x9f, 0x18, 0xc4, 0xc0, \
  0x81, 0xe3, 0x04, 0x0f, 0x18, 0x00, \
  0x21, 0x40, 0x59, 0x0a, 0x02, 0xc0, \
  0x52, 0x81, 0xe2, 0x94, 0x0f, 0x00, \
  0xb4, 0x28, 0x25, 0xa1, 0x41, 0x00, \
  0x26, 0x86, 0x29, 0x34, 0x31, 0x40, \
  0x58, 0x64, 0x42, 0xc3, 0x22, 0x00, \
  0x19, 0x9e, 0x00, 0xcc, 0xf0, 0x00, \
  0x57, 0xc7, 0x03, 0xf9, 0xc6, 0x00

#define kMaskRandom42_37 \
  0x4e, 0x11, 0x92, 0x70, 0x8c, 0x80, \
  0xe3, 0x18, 0x9f, 0x18, 0xc4, 0xc0, \
  0x81, 0xe3, 0x04, 0x0f, 0x18, 0x00, \
  0x21, 0x40, 0x59, 0x0a, 0x02, 0xc0, \
  0x52, 0x81, 0xe2, 0x94, 0x0f, 0x00, \
  0xb4, 0x28, 0x25, 0xa1, 0x41, 0x00, \
  0x26, 0x86, 0x29, 0x34, 0x31, 0x40, \
  0x58, 0x64, 0x42, 0xc3, 0x22, 0x00, \
  0x19, 0x9e, 0x00, 0xcc, 0xf0, 0x00, \
  0x2c, 0x03, 0x21, 0x60, 0x19, 0x00, \
  0x81, 0xa0, 0x1c, 0x0d, 0x00, 0xc0, \
  0xa0, 0x68, 0x25, 0x03, 0x41, 0x00, \
  0x05, 0x41, 0x50, 0x2a, 0x0a, 0x80, \
  0x18, 0x90, 0xc0, 0xc4, 0x86, 0x00, \
  0xc2, 0x06, 0x86, 0x10, 0x34, 0x00, \
  0x22, 0x98, 0x09, 0x14, 0xc0, 0x40, \
  0x50, 0x45, 0x0a, 0x82, 0x28, 0x40, \
  0x5a, 0x56, 0x5a, 0xd2, 0xb2, 0xc0, \
  0x4e, 0x11, 0x92, 0x70, 0x8c, 0x80, \
  0xe3, 0x18, 0x9f, 0x18, 0xc4, 0xc0, \
  0x81, 0xe3, 0x04, 0x0f, 0x18, 0x00, \
  0x21, 0x40, 0x59, 0x0a, 0x02, 0xc0, \
  0x52, 0x81, 0xe2, 0x94, 0x0f, 0x00, \
  0xb4, 0x28, 0x25, 0xa1, 0x41, 0x00, \
  0x26, 0x86, 0x29, 0x34, 0x31, 0x40, \
  0x58, 0x64, 0x42, 0xc3, 0x22, 0x00, \
  0x19, 0x9e, 0x00, 0xcc, 0xf0, 0x00, \
  0x4c, 0x19, 0x12, 0x60, 0xc8, 0x80, \
  0x51, 0x14, 0x52, 0x88, 0xa2, 0x80, \
  0xa0, 0x6a, 0x45, 0x03, 0x52, 0x00, \
  0x04, 0xc1, 0x30, 0x26, 0x09, 0x80, \
  0x03, 0xb4, 0x00, 0x1d, 0xa0, 0x00, \
  0x86, 0x20, 0x94, 0x31, 0x04, 0x80, \
  0x29, 0x08, 0x49, 0x48, 0x42, 0x40, \
  0x42, 0x43, 0x0a, 0x12, 0x18, 0x40, \
  0x98, 0x12, 0x84, 0xc0, 0x94, 0x00, \
  0x30, 0x84, 0xa9, 0x84, 0x25, 0x40

#define kMaskRandom42_38 \
  0x4e, 0x11, 0x92, 0x70, 0x8c, 0x80, \
  0xe3, 0x18, 0x9f, 0x18, 0xc4, 0xc0, \
  0x81, 0xe3, 0x04, 0x0f, 0x18, 0x00, \
  0x21, 0x40, 0x59, 0x0a, 0x02, 0xc0, \
  0x52, 0x81, 0xe2, 0x94, 0x0f, 0x00, \
  0xb4, 0x28, 0x25, 0xa1, 0x41, 0x00, \
  0x26, 0x86, 0x29, 0x34, 0x31, 0x40, \
  0x58, 0x64, 0x42, 0xc3, 0x22, 0x00, \
  0x19, 0x9e, 0x00, 0xcc, 0xf0, 0x00, \
  0x4c, 0x19, 0x12, 0x60, 0xc8, 0x80, \
  0x51, 0x14, 0x52, 0x88, 0xa2, 0x80, \
  0xa0, 0x6a, 0x45, 0x03, 0x52, 0x00, \
  0x04, 0xc1, 0x30, 0x26, 0x09, 0x80, \
  0x03, 0xb4, 0x00, 0x1d, 0xa0, 0x00, \
  0x86, 0x20, 0x94, 0x31, 0x04, 0x80, \
  0x29, 0x08, 0x49, 0x48, 0x42, 0x40, \
  0x42, 0x43, 0x0a, 0x12, 0x18, 0x40, \
  0x98, 0x12, 0x84, 0xc0, 0x94, 0x00, \
  0x30, 0x84, 0xa9, 0x84, 0x25, 0x40, \
  0x4e, 0x11, 0x92, 0x70, 0x8c, 0x80, \
  0xe3, 0x18, 0x9f, 0x18, 0xc4, 0xc0, \
  0x81, 0xe3, 0x04, 0x0f, 0x18, 0x00, \
  0x21, 0x40, 0x59, 0x0a, 0x02, 0xc0, \
  0x52, 0x81, 0xe2, 0x94, 0x0f, 0x00, \
  0xb4, 0x28, 0x25, 0xa1, 0x41, 0x00, \
  0x26, 0x86, 0x29, 0x34, 0x31, 0x40, \
  0x58, 0x64, 0x42, 0xc3, 0x22, 0x00, \
  0x19, 0x9e, 0x00, 0xcc, 0xf0, 0x00, \
  0x2c, 0x03, 0x21, 0x60, 0x19, 0x00, \
  0x81, 0xa0, 0x1c, 0x0d, 0x00, 0xc0, \
  0xa0, 0x68, 0x25, 0x03, 0x41, 0x00, \
  0x05, 0x41, 0x50, 0x2a, 0x0a, 0x80, \
  0x18, 0x90, 0xc0, 0xc4, 0x86, 0x00, \
  0xc2, 0x06, 0x86, 0x10, 0x34, 0x00, \
  0x22, 0x98, 0x09, 0x14, 0xc0, 0x40, \
  0x50, 0x45, 0x0a, 0x82, 0x28, 0x40, \
  0x5a, 0x56, 0x5a, 0xd2, 0xb2, 0xc0, \
  0x05, 0x19, 0x55, 0xee, 0xe2, 0xc0

#define kMaskRandom42_39 \
  0x4e, 0x11, 0x92, 0x70, 0x8c, 0x80, \
  0xe3, 0x18, 0x9f, 0x18, 0xc4, 0xc0, \
  0x81, 0xe3, 0x04, 0x0f, 0x18, 0x00, \
  0x21, 0x40, 0x59, 0x0a, 0x02, 0xc0, \
  0x52, 0x81, 0xe2, 0x94, 0x0f, 0x00, \
  0xb4, 0x28, 0x25, 0xa1, 0x41, 0x00, \
  0x26, 0x86, 0x29, 0x34, 0x31, 0x40, \
  0x58, 0x64, 0x42, 0xc3, 0x22, 0x00, \
  0x19, 0x9e, 0x00, 0xcc, 0xf0, 0x00, \
  0x4c, 0x19, 0x12, 0x60, 0xc8, 0x80, \
  0x51, 0x14, 0x52, 0x88, 0xa2, 0x80, \
  0xa0, 0x6a, 0x45, 0x03, 0x52, 0x00, \
  0x04, 0xc1, 0x30, 0x26, 0x09, 0x80, \
  0x03, 0xb4, 0x00, 0x1d, 0xa0, 0x00, \
  0x86, 0x20, 0x94, 0x31, 0x04, 0x80, \
  0x29, 0x08, 0x49, 0x48, 0x42, 0x40, \
  0x42, 0x43, 0x0a, 0x12, 0x18, 0x40, \
  0x98, 0x12, 0x84, 0xc0, 0x94, 0x00, \
  0x30, 0x84, 0xa9, 0x84, 0x25, 0x40, \
  0x4c, 0x19, 0x12, 0x60, 0xc8, 0x80, \
  0x51, 0x14, 0x52, 0x88, 0xa2, 0x80, \
  0xa0, 0x6a, 0x45, 0x03, 0x52, 0x00, \
  0x04, 0xc1, 0x30, 0x26, 0x09, 0x80, \
  0x03, 0xb4, 0x00, 0x1d, 0xa0, 0x00, \
  0x86, 0x20, 0x94, 0x31, 0x04, 0x80, \
  0x29, 0x08, 0x49, 0x48, 0x42, 0x40, \
  0x42, 0x43, 0x0a, 0x12, 0x18, 0x40, \
  0x98, 0x12, 0x84, 0xc0, 0x94, 0x00, \
  0x30, 0x84, 0xa9, 0x84, 0x25, 0x40, \
  0x4e, 0x11, 0x92, 0x70, 0x8c, 0x80, \
  0xe3, 0x18, 0x9f, 0x18, 0xc4, 0xc0, \
  0x81, 0xe3, 0x04, 0x0f, 0x18, 0x00, \
  0x21, 0x40, 0x59, 0x0a, 0x02, 0xc0, \
  0x52, 0x81, 0xe2, 0x94, 0x0f, 0x00, \
  0xb4, 0x28, 0x25, 0xa1, 0x41, 0x00, \
  0x26, 0x86, 0x29, 0x34, 0x31, 0x40, \
  0x58, 0x64, 0x42, 0xc3, 0x22, 0x00, \
  0x19, 0x9e, 0x00, 0xcc, 0xf0, 0x00, \
  0x2a, 0x03, 0x31, 0x50, 0x19, 0x80

#define kMaskRandom42_4 \
  0xe6, 0x31, 0x37, 0x31, 0x89, 0x80, \
  0x33, 0x8c, 0x59, 0x9c, 0x62, 0xc0, \
  0x98, 0xd2, 0xcc, 0xc6, 0x96, 0x40, \
  0x2d, 0x4b, 0x29, 0x6a, 0x59, 0x40

#define kMaskRandom42_40 \
  0x4c, 0x19, 0x12, 0x60, 0xc8, 0x80, \
  0x51, 0x14, 0x52, 0x88, 0xa2, 0x80, \
  0xa0, 0x6a, 0x45, 0x03, 0x52, 0x00, \
  0x04, 0xc1, 0x30, 0x26, 0x09, 0x80, \
  0x03, 0xb4, 0x00, 0x1d, 0xa0, 0x00, \
  0x86, 0x20, 0x94, 0x31, 0x04, 0x80, \
  0x29, 0x08, 0x49, 0x48, 0x42, 0x40, \
  0x42, 0x43, 0x0a, 0x12, 0x18, 0x40, \
  0x98, 0x12, 0x84, 0xc0, 0x94, 0x00, \
  0x30, 0x84, 0xa9, 0x84, 0x25, 0x40, \
  0x4e, 0x11, 0x92, 0x70, 0x8c, 0x80, \
  0xe3, 0x18, 0x9f, 0x18, 0xc4, 0xc0, \
  0x81, 0xe3, 0x04, 0x0f, 0x18, 0x00, \
  0x21, 0x40, 0x59, 0x0a, 0x02, 0xc0, \
  0x52, 0x81, 0xe2, 0x94, 0x0f, 0x00, \
  0xb4, 0x28, 0x25, 0xa1, 0x41, 0x00, \
  0x26, 0x86, 0x29, 0x34, 0x31, 0x40, \
  0x58, 0x64, 0x42, 0xc3, 0x22, 0x00, \
  0x19, 0x9e, 0x00, 0xcc, 0xf0, 0x00, \
  0x2a, 0x03, 0x31, 0x50, 0x19, 0x80, \
  0x4e, 0x11, 0x92, 0x70, 0x8c, 0x80, \
  0xe3, 0x18, 0x9f, 0x18, 0xc4, 0xc0, \
  0x81, 0xe3, 0x04, 0x0f, 0x18, 0x00, \
  0x21, 0x40, 0x59, 0x0a, 0x02, 0xc0, \
  0x52, 0x81, 0xe2, 0x94, 0x0f, 0x00, \
  0xb4, 0x28, 0x25, 0xa1, 0x41, 0x00, \
  0x26, 0x86, 0x29, 0x34, 0x31, 0x40, \
  0x58, 0x64, 0x42, 0xc3, 0x22, 0x00, \
  0x19, 0x9e, 0x00, 0xcc, 0xf0, 0x00, \
  0x4c, 0x19, 0x12, 0x60, 0xc8, 0x80, \
  0x51, 0x14, 0x52, 0x88, 0xa2, 0x80, \
  0xa0, 0x6a, 0x45, 0x03, 0x52, 0x00, \
  0x04, 0xc1, 0x30, 0x26, 0x09, 0x80, \
  0x03, 0xb4, 0x00, 0x1d, 0xa0, 0x00, \
  0x86, 0x20, 0x94, 0x31, 0x04, 0x80, \
  0x29, 0x08, 0x49, 0x48, 0x42, 0x40, \
  0x42, 0x43, 0x0a, 0x12, 0x18, 0x40, \
  0x98, 0x12, 0x84, 0xc0, 0x94, 0x00, \
  0x30, 0x84, 0xa9, 0x84, 0x25, 0x40, \
  0xf9, 0xdb, 0x5d, 0x7a, 0xd4, 0x40

#define kMaskRandom42_41 \
  0x4c, 0x19, 0x12, 0x60, 0xc8, 0x80, \
  0x51, 0x14, 0x52, 0x88, 0xa2, 0x80, \
  0xa0, 0x6a, 0x45, 0x03, 0x52, 0x00, \
  0x04, 0xc1, 0x30, 0x26, 0x09, 0x80, \
  0x03, 0xb4, 0x00, 0x1d, 0xa0, 0x00, \
  0x86, 0x20, 0x94, 0x31, 0x04, 0x80, \
  0x29, 0x08, 0x49, 0x48, 0x42, 0x40, \
  0x42, 0x43, 0x0a, 0x12, 0x18, 0x40, \
  0x98, 0x12, 0x84, 0xc0, 0x94, 0x00, \
  0x30, 0x84, 0xa9, 0x84, 0x25, 0x40, \
  0x4e, 0x11, 0x92, 0x70, 0x8c, 0x80, \
  0xe3, 0x18, 0x9f, 0x18, 0xc4, 0xc0, \
  0x81, 0xe3, 0x04, 0x0f, 0x18, 0x00, \
  0x21, 0x40, 0x59, 0x0a, 0x02, 0xc0, \
  0x52, 0x81, 0xe2, 0x94, 0x0f, 0x00, \
  0xb4, 0x28, 0x25, 0xa1, 0x41, 0x00, \
  0x26, 0x86, 0x29, 0x34, 0x31, 0x40, \
  0x58, 0x64, 0x42, 0xc3, 0x22, 0x00, \
  0x19, 0x9e, 0x00, 0xcc, 0xf0, 0x00, \
  0x2a, 0x03, 0x31, 0x50, 0x19, 0x80, \
  0x4c, 0x19, 0x12, 0x60, 0xc8, 0x80, \
  0x51, 0x14, 0x52, 0x88, 0xa2, 0x80, \
  0xa0, 0x6a, 0x45, 0x03, 0x52, 0x00, \
  0x04, 0xc1, 0x30, 0x26, 0x09, 0x80, \
  0x03, 0xb4, 0x00, 0x1d, 0xa0, 0x00, \
  0x86, 0x20, 0x94, 0x31, 0x04, 0x80, \
  0x29, 0x08, 0x49, 0x48, 0x42, 0x40, \
  0x42, 0x43, 0x0a, 0x12, 0x18, 0x40, \
  0x98, 0x12, 0x84, 0xc0, 0x94, 0x00, \
  0x30, 0x84, 0xa9, 0x84, 0x25, 0x40, \
  0x4c, 0x11, 0x92, 0x60, 0x8c, 0x80, \
  0x51, 0x0c, 0xca, 0x88, 0x66, 0x40, \
  0xa0, 0x66, 0x45, 0x03, 0x32, 0x00, \
  0x04, 0xc1, 0x60, 0x26, 0x0b, 0x00, \
  0x03, 0xa0, 0x28, 0x1d, 0x01, 0x40, \
  0x86, 0x21, 0x14, 0x31, 0x08, 0x80, \
  0x29, 0x10, 0x19, 0x48, 0x80, 0xc0, \
  0x42, 0x42, 0xa2, 0x12, 0x15, 0x00, \
  0x98, 0x1a, 0x04, 0xc0, 0xd0, 0x00, \
  0x30, 0x84, 0x09, 0x84, 0x20, 0x40, \
  0xdf, 0x4c, 0x16, 0xfa, 0x60, 0x80

#define kMaskRandom42_42 \
  0x4c, 0x19, 0x12, 0x60, 0xc8, 0x80, \
  0x51, 0x14, 0x52, 0x88, 0xa2, 0x80, \
  0xa0, 0x6a, 0x45, 0x03, 0x52, 0x00, \
  0x04, 0xc1, 0x30, 0x26, 0x09, 0x80, \
  0x03, 0xb4, 0x00, 0x1d, 0xa0, 0x00, \
  0x86, 0x20, 0x94, 0x31, 0x04, 0x80, \
  0x29, 0x08, 0x49, 0x48, 0x42, 0x40, \
  0x42, 0x43, 0x0a, 0x12, 0x18, 0x40, \
  0x98, 0x12, 0x84, 0xc0, 0x94, 0x00, \
  0x30, 0x84, 0xa9, 0x84, 0x25, 0x40, \
  0x4c, 0x11, 0x92, 0x60, 0x8c, 0x80, \
  0x51, 0x0c, 0xca, 0x88, 0x66, 0x40, \
  0xa0, 0x66, 0x45, 0x03, 0x32, 0x00, \
  0x04, 0xc1, 0x60, 0x26, 0x0b, 0x00, \
  0x03, 0xa0, 0x28, 0x1d, 0x01, 0x40, \
  0x86, 0x21, 0x14, 0x31, 0x08, 0x80, \
  0x29, 0x10, 0x19, 0x48, 0x80, 0xc0, \
  0x42, 0x42, 0xa2, 0x12, 0x15, 0x00, \
  0x98, 0x1a, 0x04, 0xc0, 0xd0, 0x00, \
  0x30, 0x84, 0x09, 0x84, 0x20, 0x40, \
  0xdf, 0x4c, 0x16, 0xfa, 0x60, 0x80, \
  0x4c, 0x19, 0x12, 0x60, 0xc8, 0x80, \
  0x51, 0x14, 0x52, 0x88, 0xa2, 0x80, \
  0xa0, 0x6a, 0x45, 0x03, 0x52, 0x00, \
  0x04, 0xc1, 0x30, 0x26, 0x09, 0x80, \
  0x03, 0xb4, 0x00, 0x1d, 0xa0, 0x00, \
  0x86, 0x20, 0x94, 0x31, 0x04, 0x80, \
  0x29, 0x08, 0x49, 0x48, 0x42, 0x40, \
  0x42, 0x43, 0x0a, 0x12, 0x18, 0x40, \
  0x98, 0x12, 0x84, 0xc0, 0x94, 0x00, \
  0x30, 0x84, 0xa9, 0x84, 0x25, 0x40, \
  0x4e, 0x11, 0x92, 0x70, 0x8c, 0x80, \
  0xe3, 0x18, 0x9f, 0x18, 0xc4, 0xc0, \
  0x81, 0xe3, 0x04, 0x0f, 0x18, 0x00, \
  0x21, 0x40, 0x59, 0x0a, 0x02, 0xc0, \
  0x52, 0x81, 0xe2, 0x94, 0x0f, 0x00, \
  0xb4, 0x28, 0x25, 0xa1, 0x41, 0x00, \
  0x26, 0x86, 0x29, 0x34, 0x31, 0x40, \
  0x58, 0x64, 0x42, 0xc3, 0x22, 0x00, \
  0x19, 0x9e, 0x00, 0xcc, 0xf0, 0x00, \
  0x2a, 0x03, 0x31, 0x50, 0x19, 0x80, \
  0xea, 0x9e, 0x23, 0xb3, 0x65, 0x00

#define kMaskRandom42_5 \
  0xce, 0x31, 0xb6, 0x71, 0x8d, 0x80, \
  0x63, 0x98, 0xdb, 0x1c, 0xc6, 0xc0, \
  0x98, 0xc7, 0x6c, 0xc6, 0x3b, 0x40, \
  0x4d, 0x6b, 0x52, 0x6b, 0x5a, 0x80, \
  0xb2, 0x6c, 0xad, 0x93, 0x65, 0x40

#define kMaskRandom42_6 \
  0x4c, 0x19, 0x12, 0x60, 0xc8, 0x80, \
  0x51, 0x14, 0x52, 0x88, 0xa2, 0x80, \
  0x20, 0xea, 0x09, 0x07, 0x50, 0x40, \
  0x85, 0x41, 0x2c, 0x2a, 0x09, 0x40, \
  0x06, 0x80, 0xd8, 0x34, 0x06, 0xc0, \
  0x8a, 0x24, 0x34, 0x51, 0x21, 0x80

#define kMaskRandom42_7 \
  0xc6, 0x11, 0x96, 0x30, 0x8c, 0x80, \
  0x33, 0x04, 0xc9, 0x98, 0x26, 0x40, \
  0x18, 0x67, 0x40, 0xc3, 0x3a, 0x00, \
  0x45, 0x42, 0xd2, 0x2a, 0x16, 0x80, \
  0x12, 0xd4, 0x28, 0x96, 0xa1, 0x40, \
  0xb4, 0x28, 0x35, 0xa1, 0x41, 0x80, \
  0x29, 0x92, 0x19, 0x4c, 0x90, 0xc0

#define kMaskRandom42_8 \
  0x07, 0x0a, 0x70, 0x38, 0x53, 0x80, \
  0x49, 0xa8, 0x2a, 0x4d, 0x41, 0x40, \
  0xb0, 0x7a, 0x05, 0x83, 0xd0, 0x00, \
  0x24, 0xc5, 0xc1, 0x26, 0x2e, 0x00, \
  0x52, 0x80, 0xea, 0x94, 0x07, 0x40, \
  0xc6, 0x31, 0x86, 0x31, 0x8c, 0x00, \
  0x31, 0x94, 0x19, 0x8c, 0xa0, 0xc0, \
  0x18, 0xc7, 0x08, 0xc6, 0x38, 0x40

#define kMaskRandom42_9 \
  0x4e, 0x11, 0x12, 0x70, 0x88, 0x80, \
  0x62, 0x1a, 0x0b, 0x10, 0xd0, 0x40, \
  0x80, 0xe9, 0x44, 0x07, 0x4a, 0x00, \
  0xa1, 0x50, 0x55, 0x0a, 0x82, 0x80, \
  0x53, 0x00, 0x6a, 0x98, 0x03, 0x40, \
  0xa4, 0x24, 0x35, 0x21, 0x21, 0x80, \
  0x16, 0xa0, 0x88, 0xb5, 0x04, 0x40, \
  0x58, 0x45, 0x22, 0xc2, 0x29, 0x00, \
  0x29, 0x86, 0x81, 0x4c, 0x34, 0x00

#define kMaskRandom43_1 \
  0xff, 0xff, 0xff, 0xff, 0xff, 0xe0

#define kMaskRandom43_10 \
  0x4c, 0x19, 0x16, 0x01, 0xc4, 0x40, \
  0x51, 0x14, 0x51, 0x80, 0x71, 0x40, \
  0xa0, 0x6a, 0x47, 0x40, 0x38, 0x00, \
  0x04, 0xc1, 0x34, 0x28, 0x45, 0x40, \
  0x03, 0xb4, 0x06, 0x84, 0x90, 0x80, \
  0x86, 0x20, 0x94, 0x32, 0x82, 0x40, \
  0x29, 0x08, 0x4a, 0x53, 0x40, 0x60, \
  0x42, 0x43, 0x08, 0x0d, 0x03, 0xa0, \
  0x98, 0x12, 0x82, 0x64, 0x0c, 0x80, \
  0x30, 0x84, 0xab, 0x11, 0x20, 0x20

#define kMaskRandom43_11 \
  0xc6, 0x21, 0xa2, 0x32, 0x46, 0x40, \
  0x23, 0x88, 0xc9, 0x99, 0x33, 0x20, \
  0x1a, 0x45, 0x8c, 0xc8, 0x99, 0x00, \
  0x24, 0xd3, 0x08, 0x2c, 0x05, 0x80, \
  0x71, 0x10, 0x74, 0x05, 0x80, 0xa0, \
  0x0e, 0x19, 0x14, 0x22, 0x84, 0x40, \
  0x33, 0x14, 0x52, 0x03, 0x40, 0x60, \
  0x10, 0xc3, 0x28, 0x54, 0x0a, 0x80, \
  0x45, 0x68, 0x4b, 0x40, 0x68, 0x00, \
  0x88, 0x84, 0xa8, 0x81, 0x10, 0x20, \
  0xe0, 0x22, 0x91, 0x82, 0x30, 0x40

#define kMaskRandom43_12 \
  0x0e, 0x19, 0x13, 0x22, 0x64, 0x40, \
  0x33, 0x14, 0x52, 0x8a, 0x51, 0x40, \
  0x10, 0xc3, 0x28, 0x65, 0x0c, 0xa0, \
  0x45, 0x68, 0x4d, 0x09, 0xa1, 0x20, \
  0x88, 0x84, 0xa8, 0x95, 0x12, 0xa0, \
  0xe0, 0x22, 0x94, 0x52, 0x8a, 0x40, \
  0xc6, 0x21, 0xa4, 0x34, 0x86, 0x80, \
  0x23, 0x88, 0xc9, 0x19, 0x23, 0x20, \
  0x1a, 0x45, 0x88, 0xb1, 0x16, 0x20, \
  0x24, 0xd3, 0x0a, 0x61, 0x4c, 0x20, \
  0x71, 0x10, 0x72, 0x0e, 0x41, 0xc0, \
  0xa0, 0x65, 0x1f, 0xa0, 0xc4, 0xe0

#define kMaskRandom43_13 \
  0x0e, 0x19, 0x13, 0x22, 0x64, 0x40, \
  0x33, 0x14, 0x52, 0x8a, 0x51, 0x40, \
  0x10, 0xc3, 0x28, 0x65, 0x0c, 0xa0, \
  0x45, 0x68, 0x4d, 0x09, 0xa1, 0x20, \
  0x88, 0x84, 0xa8, 0x95, 0x12, 0xa0, \
  0xe0, 0x22, 0x94, 0x52, 0x8a, 0x40, \
  0x46, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0x33, 0x0c, 0xc9, 0x99, 0x33, 0x20, \
  0x10, 0xe4, 0x64, 0x8c, 0x91, 0x80, \
  0x0c, 0x69, 0x0d, 0x21, 0xa4, 0x20, \
  0x28, 0x94, 0x2a, 0x85, 0x50, 0xa0, \
  0x94, 0x21, 0x34, 0x26, 0x84, 0xc0, \
  0xc1, 0x02, 0x58, 0x4b, 0x09, 0x60

#define kMaskRandom43_14 \
  0x46, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0x33, 0x0c, 0xc9, 0x99, 0x33, 0x20, \
  0x10, 0xe4, 0x64, 0x8c, 0x91, 0x80, \
  0x0c, 0x69, 0x0d, 0x21, 0xa4, 0x20, \
  0x28, 0x94, 0x2a, 0x85, 0x50, 0xa0, \
  0x94, 0x21, 0x34, 0x26, 0x84, 0xc0, \
  0xc1, 0x02, 0x58, 0x4b, 0x09, 0x60, \
  0x0e, 0x19, 0x13, 0x22, 0x64, 0x40, \
  0x33, 0x14, 0x52, 0x8a, 0x51, 0x40, \
  0x10, 0xc3, 0x28, 0x65, 0x0c, 0xa0, \
  0x45, 0x68, 0x4d, 0x09, 0xa1, 0x20, \
  0x88, 0x84, 0xa8, 0x95, 0x12, 0xa0, \
  0xe0, 0x22, 0x94, 0x52, 0x8a, 0x40, \
  0x4d, 0xd0, 0xc6, 0x36, 0x57, 0x40

#define kMaskRandom43_15 \
  0x46, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0x33, 0x0c, 0xc9, 0x99, 0x33, 0x20, \
  0x10, 0xe4, 0x64, 0x8c, 0x91, 0x80, \
  0x0c, 0x69, 0x0d, 0x21, 0xa4, 0x20, \
  0x28, 0x94, 0x2a, 0x85, 0x50, 0xa0, \
  0x94, 0x21, 0x34, 0x26, 0x84, 0xc0, \
  0xc1, 0x02, 0x58, 0x4b, 0x09, 0x60, \
  0x2c, 0x03, 0x20, 0x64, 0x0c, 0x80, \
  0x81, 0xa0, 0x1c, 0x03, 0x80, 0x60, \
  0xa0, 0x68, 0x25, 0x04, 0xa0, 0x80, \
  0x05, 0x41, 0x50, 0x2a, 0x05, 0x40, \
  0x18, 0x90, 0xc2, 0x18, 0x43, 0x00, \
  0xc2, 0x06, 0x80, 0xd0, 0x1a, 0x00, \
  0x22, 0x98, 0x0b, 0x01, 0x60, 0x20, \
  0x50, 0x45, 0x08, 0xa1, 0x14, 0x20

#define kMaskRandom43_16 \
  0x2c, 0x03, 0x20, 0x64, 0x0c, 0x80, \
  0x81, 0xa0, 0x1c, 0x03, 0x80, 0x60, \
  0xa0, 0x68, 0x25, 0x04, 0xa0, 0x80, \
  0x05, 0x41, 0x50, 0x2a, 0x05, 0x40, \
  0x18, 0x90, 0xc2, 0x18, 0x43, 0x00, \
  0xc2, 0x06, 0x80, 0xd0, 0x1a, 0x00, \
  0x22, 0x98, 0x0b, 0x01, 0x60, 0x20, \
  0x50, 0x45, 0x08, 0xa1, 0x14, 0x20, \
  0x46, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0x33, 0x0c, 0xc9, 0x99, 0x33, 0x20, \
  0x10, 0xe4, 0x64, 0x8c, 0x91, 0x80, \
  0x0c, 0x69, 0x0d, 0x21, 0xa4, 0x20, \
  0x28, 0x94, 0x2a, 0x85, 0x50, 0xa0, \
  0x94, 0x21, 0x34, 0x26, 0x84, 0xc0, \
  0xc1, 0x02, 0x58, 0x4b, 0x09, 0x60, \
  0x3b, 0xf5, 0x3c, 0x36, 0x0a, 0x20

#define kMaskRandom43_17 \
  0x2c, 0x03, 0x20, 0x64, 0x0c, 0x80, \
  0x81, 0xa0, 0x1c, 0x03, 0x80, 0x60, \
  0xa0, 0x68, 0x25, 0x04, 0xa0, 0x80, \
  0x05, 0x41, 0x50, 0x2a, 0x05, 0x40, \
  0x18, 0x90, 0xc2, 0x18, 0x43, 0x00, \
  0xc2, 0x06, 0x80, 0xd0, 0x1a, 0x00, \
  0x22, 0x98, 0x0b, 0x01, 0x60, 0x20, \
  0x50, 0x45, 0x08, 0xa1, 0x14, 0x20, \
  0x4e, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0xe3, 0x18, 0x9b, 0x13, 0x62, 0x60, \
  0x81, 0xe3, 0x04, 0x60, 0x8c, 0x00, \
  0x21, 0x40, 0x58, 0x0b, 0x01, 0x60, \
  0x52, 0x81, 0xe0, 0x3c, 0x07, 0x80, \
  0xb4, 0x28, 0x25, 0x04, 0xa0, 0x80, \
  0x26, 0x86, 0x28, 0xc5, 0x18, 0xa0, \
  0x58, 0x64, 0x44, 0x88, 0x91, 0x00, \
  0x19, 0x9e, 0x03, 0xc0, 0x78, 0x00

#define kMaskRandom43_18 \
  0x4e, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0xe3, 0x18, 0x9b, 0x13, 0x62, 0x60, \
  0x81, 0xe3, 0x04, 0x60, 0x8c, 0x00, \
  0x21, 0x40, 0x58, 0x0b, 0x01, 0x60, \
  0x52, 0x81, 0xe0, 0x3c, 0x07, 0x80, \
  0xb4, 0x28, 0x25, 0x04, 0xa0, 0x80, \
  0x26, 0x86, 0x28, 0xc5, 0x18, 0xa0, \
  0x58, 0x64, 0x44, 0x88, 0x91, 0x00, \
  0x19, 0x9e, 0x03, 0xc0, 0x78, 0x00, \
  0x2c, 0x03, 0x20, 0x64, 0x0c, 0x80, \
  0x81, 0xa0, 0x1c, 0x03, 0x80, 0x60, \
  0xa0, 0x68, 0x25, 0x04, 0xa0, 0x80, \
  0x05, 0x41, 0x50, 0x2a, 0x05, 0x40, \
  0x18, 0x90, 0xc2, 0x18, 0x43, 0x00, \
  0xc2, 0x06, 0x80, 0xd0, 0x1a, 0x00, \
  0x22, 0x98, 0x0b, 0x01, 0x60, 0x20, \
  0x50, 0x45, 0x08, 0xa1, 0x14, 0x20, \
  0x5a, 0x56, 0x5f, 0x26, 0xa3, 0x60

#define kMaskRandom43_19 \
  0x4e, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0xe3, 0x18, 0x9b, 0x13, 0x62, 0x60, \
  0x81, 0xe3, 0x04, 0x60, 0x8c, 0x00, \
  0x21, 0x40, 0x58, 0x0b, 0x01, 0x60, \
  0x52, 0x81, 0xe0, 0x3c, 0x07, 0x80, \
  0xb4, 0x28, 0x25, 0x04, 0xa0, 0x80, \
  0x26, 0x86, 0x28, 0xc5, 0x18, 0xa0, \
  0x58, 0x64, 0x44, 0x88, 0x91, 0x00, \
  0x19, 0x9e, 0x03, 0xc0, 0x78, 0x00, \
  0x4c, 0x19, 0x13, 0x22, 0x64, 0x40, \
  0x51, 0x14, 0x52, 0x8a, 0x51, 0x40, \
  0xa0, 0x6a, 0x45, 0x48, 0xa9, 0x00, \
  0x04, 0xc1, 0x30, 0x26, 0x04, 0xc0, \
  0x03, 0xb4, 0x06, 0x80, 0xd0, 0x00, \
  0x86, 0x20, 0x94, 0x12, 0x82, 0x40, \
  0x29, 0x08, 0x49, 0x09, 0x21, 0x20, \
  0x42, 0x43, 0x08, 0x61, 0x0c, 0x20, \
  0x98, 0x12, 0x82, 0x50, 0x4a, 0x00, \
  0x30, 0x84, 0xa8, 0x95, 0x12, 0xa0

#define kMaskRandom43_2 \
  0xee, 0x3b, 0x37, 0x66, 0xec, 0xc0, \
  0x99, 0xe6, 0xec, 0xdd, 0x9b, 0xa0

#define kMaskRandom43_20 \
  0x4c, 0x19, 0x13, 0x22, 0x64, 0x40, \
  0x51, 0x14, 0x52, 0x8a, 0x51, 0x40, \
  0xa0, 0x6a, 0x45, 0x48, 0xa9, 0x00, \
  0x04, 0xc1, 0x30, 0x26, 0x04, 0xc0, \
  0x03, 0xb4, 0x06, 0x80, 0xd0, 0x00, \
  0x86, 0x20, 0x94, 0x12, 0x82, 0x40, \
  0x29, 0x08, 0x49, 0x09, 0x21, 0x20, \
  0x42, 0x43, 0x08, 0x61, 0x0c, 0x20, \
  0x98, 0x12, 0x82, 0x50, 0x4a, 0x00, \
  0x30, 0x84, 0xa8, 0x95, 0x12, 0xa0, \
  0x4e, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0xe3, 0x18, 0x9b, 0x13, 0x62, 0x60, \
  0x81, 0xe3, 0x04, 0x60, 0x8c, 0x00, \
  0x21, 0x40, 0x58, 0x0b, 0x01, 0x60, \
  0x52, 0x81, 0xe0, 0x3c, 0x07, 0x80, \
  0xb4, 0x28, 0x25, 0x04, 0xa0, 0x80, \
  0x26, 0x86, 0x28, 0xc5, 0x18, 0xa0, \
  0x58, 0x64, 0x44, 0x88, 0x91, 0x00, \
  0x19, 0x9e, 0x03, 0xc0, 0x78, 0x00, \
  0x2a, 0x03, 0x31, 0xda, 0x46, 0x20

#define kMaskRandom43_21 \
  0x4c, 0x19, 0x13, 0x22, 0x64, 0x40, \
  0x51, 0x14, 0x52, 0x8a, 0x51, 0x40, \
  0xa0, 0x6a, 0x45, 0x48, 0xa9, 0x00, \
  0x04, 0xc1, 0x30, 0x26, 0x04, 0xc0, \
  0x03, 0xb4, 0x06, 0x80, 0xd0, 0x00, \
  0x86, 0x20, 0x94, 0x12, 0x82, 0x40, \
  0x29, 0x08, 0x49, 0x09, 0x21, 0x20, \
  0x42, 0x43, 0x08, 0x61, 0x0c, 0x20, \
  0x98, 0x12, 0x82, 0x50, 0x4a, 0x00, \
  0x30, 0x84, 0xa8, 0x95, 0x12, 0xa0, \
  0x4c, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0x51, 0x0c, 0xc9, 0x99, 0x33, 0x20, \
  0xa0, 0x66, 0x44, 0xc8, 0x99, 0x00, \
  0x04, 0xc1, 0x60, 0x2c, 0x05, 0x80, \
  0x03, 0xa0, 0x2c, 0x05, 0x80, 0xa0, \
  0x86, 0x21, 0x14, 0x22, 0x84, 0x40, \
  0x29, 0x10, 0x1a, 0x03, 0x40, 0x60, \
  0x42, 0x42, 0xa0, 0x54, 0x0a, 0x80, \
  0x98, 0x1a, 0x03, 0x40, 0x68, 0x00, \
  0x30, 0x84, 0x08, 0x81, 0x10, 0x20, \
  0xdf, 0x4c, 0x11, 0x82, 0x30, 0x40

#define kMaskRandom43_22 \
  0xc6, 0x21, 0xa2, 0x32, 0x46, 0x40, \
  0x23, 0x88, 0xc9, 0x99, 0x33, 0x20, \
  0x1a, 0x45, 0x8c, 0xc8, 0x99, 0x00, \
  0x24, 0xd3, 0x08, 0x2c, 0x05, 0x80, \
  0x71, 0x10, 0x74, 0x05, 0x80, 0xa0, \
  0x0e, 0x19, 0x14, 0x22, 0x84, 0x40, \
  0x33, 0x14, 0x52, 0x03, 0x40, 0x60, \
  0x10, 0xc3, 0x28, 0x54, 0x0a, 0x80, \
  0x45, 0x68, 0x4b, 0x40, 0x68, 0x00, \
  0x88, 0x84, 0xa8, 0x81, 0x10, 0x20, \
  0xe0, 0x22, 0x91, 0x82, 0x30, 0x40, \
  0x4c, 0x19, 0x16, 0x01, 0xc4, 0x40, \
  0x51, 0x14, 0x51, 0x80, 0x71, 0x40, \
  0xa0, 0x6a, 0x47, 0x40, 0x38, 0x00, \
  0x04, 0xc1, 0x34, 0x28, 0x45, 0x40, \
  0x03, 0xb4, 0x06, 0x84, 0x90, 0x80, \
  0x86, 0x20, 0x94, 0x32, 0x82, 0x40, \
  0x29, 0x08, 0x4a, 0x53, 0x40, 0x60, \
  0x42, 0x43, 0x08, 0x0d, 0x03, 0xa0, \
  0x98, 0x12, 0x82, 0x64, 0x0c, 0x80, \
  0x30, 0x84, 0xab, 0x11, 0x20, 0x20, \
  0xfe, 0x2c, 0x85, 0xcc, 0x24, 0x80

#define kMaskRandom43_23 \
  0xc6, 0x21, 0xa2, 0x32, 0x46, 0x40, \
  0x23, 0x88, 0xc9, 0x99, 0x33, 0x20, \
  0x1a, 0x45, 0x8c, 0xc8, 0x99, 0x00, \
  0x24, 0xd3, 0x08, 0x2c, 0x05, 0x80, \
  0x71, 0x10, 0x74, 0x05, 0x80, 0xa0, \
  0x0e, 0x19, 0x14, 0x22, 0x84, 0x40, \
  0x33, 0x14, 0x52, 0x03, 0x40, 0x60, \
  0x10, 0xc3, 0x28, 0x54, 0x0a, 0x80, \
  0x45, 0x68, 0x4b, 0x40, 0x68, 0x00, \
  0x88, 0x84, 0xa8, 0x81, 0x10, 0x20, \
  0xe0, 0x22, 0x91, 0x82, 0x30, 0x40, \
  0x0e, 0x19, 0x13, 0x22, 0x64, 0x40, \
  0x33, 0x14, 0x52, 0x8a, 0x51, 0x40, \
  0x10, 0xc3, 0x28, 0x65, 0x0c, 0xa0, \
  0x45, 0x68, 0x4d, 0x09, 0xa1, 0x20, \
  0x88, 0x84, 0xa8, 0x95, 0x12, 0xa0, \
  0xe0, 0x22, 0x94, 0x52, 0x8a, 0x40, \
  0xc6, 0x21, 0xa4, 0x34, 0x86, 0x80, \
  0x23, 0x88, 0xc9, 0x19, 0x23, 0x20, \
  0x1a, 0x45, 0x88, 0xb1, 0x16, 0x20, \
  0x24, 0xd3, 0x0a, 0x61, 0x4c, 0x20, \
  0x71, 0x10, 0x72, 0x0e, 0x41, 0xc0, \
  0xa0, 0x65, 0x1f, 0xa0, 0xc4, 0xe0

#define kMaskRandom43_24 \
  0x0e, 0x19, 0x13, 0x22, 0x64, 0x40, \
  0x33, 0x14, 0x52, 0x8a, 0x51, 0x40, \
  0x10, 0xc3, 0x28, 0x65, 0x0c, 0xa0, \
  0x45, 0x68, 0x4d, 0x09, 0xa1, 0x20, \
  0x88, 0x84, 0xa8, 0x95, 0x12, 0xa0, \
  0xe0, 0x22, 0x94, 0x52, 0x8a, 0x40, \
  0xc6, 0x21, 0xa4, 0x34, 0x86, 0x80, \
  0x23, 0x88, 0xc9, 0x19, 0x23, 0x20, \
  0x1a, 0x45, 0x88, 0xb1, 0x16, 0x20, \
  0x24, 0xd3, 0x0a, 0x61, 0x4c, 0x20, \
  0x71, 0x10, 0x72, 0x0e, 0x41, 0xc0, \
  0xa0, 0x65, 0x1f, 0xa0, 0xc4, 0xe0, \
  0xc6, 0x21, 0xa2, 0x32, 0x46, 0x40, \
  0x23, 0x88, 0xc9, 0x99, 0x33, 0x20, \
  0x1a, 0x45, 0x8c, 0xc8, 0x99, 0x00, \
  0x24, 0xd3, 0x08, 0x2c, 0x05, 0x80, \
  0x71, 0x10, 0x74, 0x05, 0x80, 0xa0, \
  0x0e, 0x19, 0x14, 0x22, 0x84, 0x40, \
  0x33, 0x14, 0x52, 0x03, 0x40, 0x60, \
  0x10, 0xc3, 0x28, 0x54, 0x0a, 0x80, \
  0x45, 0x68, 0x4b, 0x40, 0x68, 0x00, \
  0x88, 0x84, 0xa8, 0x81, 0x10, 0x20, \
  0xe0, 0x22, 0x91, 0x82, 0x30, 0x40, \
  0xf9, 0xb1, 0x26, 0x6c, 0x51, 0xe0

#define kMaskRandom43_25 \
  0x0e, 0x19, 0x13, 0x22, 0x64, 0x40, \
  0x33, 0x14, 0x52, 0x8a, 0x51, 0x40, \
  0x10, 0xc3, 0x28, 0x65, 0x0c, 0xa0, \
  0x45, 0x68, 0x4d, 0x09, 0xa1, 0x20, \
  0x88, 0x84, 0xa8, 0x95, 0x12, 0xa0, \
  0xe0, 0x22, 0x94, 0x52, 0x8a, 0x40, \
  0xc6, 0x21, 0xa4, 0x34, 0x86, 0x80, \
  0x23, 0x88, 0xc9, 0x19, 0x23, 0x20, \
  0x1a, 0x45, 0x88, 0xb1, 0x16, 0x20, \
  0x24, 0xd3, 0x0a, 0x61, 0x4c, 0x20, \
  0x71, 0x10, 0x72, 0x0e, 0x41, 0xc0, \
  0xa0, 0x65, 0x1f, 0xa0, 0xc4, 0xe0, \
  0x0e, 0x19, 0x13, 0x22, 0x64, 0x40, \
  0x33, 0x14, 0x52, 0x8a, 0x51, 0x40, \
  0x10, 0xc3, 0x28, 0x65, 0x0c, 0xa0, \
  0x45, 0x68, 0x4d, 0x09, 0xa1, 0x20, \
  0x88, 0x84, 0xa8, 0x95, 0x12, 0xa0, \
  0xe0, 0x22, 0x94, 0x52, 0x8a, 0x40, \
  0x46, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0x33, 0x0c, 0xc9, 0x99, 0x33, 0x20, \
  0x10, 0xe4, 0x64, 0x8c, 0x91, 0x80, \
  0x0c, 0x69, 0x0d, 0x21, 0xa4, 0x20, \
  0x28, 0x94, 0x2a, 0x85, 0x50, 0xa0, \
  0x94, 0x21, 0x34, 0x26, 0x84, 0xc0, \
  0xc1, 0x02, 0x58, 0x4b, 0x09, 0x60

#define kMaskRandom43_26 \
  0x0e, 0x19, 0x13, 0x22, 0x64, 0x40, \
  0x33, 0x14, 0x52, 0x8a, 0x51, 0x40, \
  0x10, 0xc3, 0x28, 0x65, 0x0c, 0xa0, \
  0x45, 0x68, 0x4d, 0x09, 0xa1, 0x20, \
  0x88, 0x84, 0xa8, 0x95, 0x12, 0xa0, \
  0xe0, 0x22, 0x94, 0x52, 0x8a, 0x40, \
  0x46, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0x33, 0x0c, 0xc9, 0x99, 0x33, 0x20, \
  0x10, 0xe4, 0x64, 0x8c, 0x91, 0x80, \
  0x0c, 0x69, 0x0d, 0x21, 0xa4, 0x20, \
  0x28, 0x94, 0x2a, 0x85, 0x50, 0xa0, \
  0x94, 0x21, 0x34, 0x26, 0x84, 0xc0, \
  0xc1, 0x02, 0x58, 0x4b, 0x09, 0x60, \
  0x0e, 0x19, 0x13, 0x22, 0x64, 0x40, \
  0x33, 0x14, 0x52, 0x8a, 0x51, 0x40, \
  0x10, 0xc3, 0x28, 0x65, 0x0c, 0xa0, \
  0x45, 0x68, 0x4d, 0x09, 0xa1, 0x20, \
  0x88, 0x84, 0xa8, 0x95, 0x12, 0xa0, \
  0xe0, 0x22, 0x94, 0x52, 0x8a, 0x40, \
  0xc6, 0x21, 0xa4, 0x34, 0x86, 0x80, \
  0x23, 0x88, 0xc9, 0x19, 0x23, 0x20, \
  0x1a, 0x45, 0x88, 0xb1, 0x16, 0x20, \
  0x24, 0xd3, 0x0a, 0x61, 0x4c, 0x20, \
  0x71, 0x10, 0x72, 0x0e, 0x41, 0xc0, \
  0xa0, 0x65, 0x1f, 0xa0, 0xc4, 0xe0, \
  0xef, 0x84, 0x77, 0xca, 0x0d, 0x40

#define kMaskRandom43_27 \
  0x0e, 0x19, 0x13, 0x22, 0x64, 0x40, \
  0x33, 0x14, 0x52, 0x8a, 0x51, 0x40, \
  0x10, 0xc3, 0x28, 0x65, 0x0c, 0xa0, \
  0x45, 0x68, 0x4d, 0x09, 0xa1, 0x20, \
  0x88, 0x84, 0xa8, 0x95, 0x12, 0xa0, \
  0xe0, 0x22, 0x94, 0x52, 0x8a, 0x40, \
  0x46, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0x33, 0x0c, 0xc9, 0x99, 0x33, 0x20, \
  0x10, 0xe4, 0x64, 0x8c, 0x91, 0x80, \
  0x0c, 0x69, 0x0d, 0x21, 0xa4, 0x20, \
  0x28, 0x94, 0x2a, 0x85, 0x50, 0xa0, \
  0x94, 0x21, 0x34, 0x26, 0x84, 0xc0, \
  0xc1, 0x02, 0x58, 0x4b, 0x09, 0x60, \
  0x46, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0x33, 0x0c, 0xc9, 0x99, 0x33, 0x20, \
  0x10, 0xe4, 0x64, 0x8c, 0x91, 0x80, \
  0x0c, 0x69, 0x0d, 0x21, 0xa4, 0x20, \
  0x28, 0x94, 0x2a, 0x85, 0x50, 0xa0, \
  0x94, 0x21, 0x34, 0x26, 0x84, 0xc0, \
  0xc1, 0x02, 0x58, 0x4b, 0x09, 0x60, \
  0x0e, 0x19, 0x13, 0x22, 0x64, 0x40, \
  0x33, 0x14, 0x52, 0x8a, 0x51, 0x40, \
  0x10, 0xc3, 0x28, 0x65, 0x0c, 0xa0, \
  0x45, 0x68, 0x4d, 0x09, 0xa1, 0x20, \
  0x88, 0x84, 0xa8, 0x95, 0x12, 0xa0, \
  0xe0, 0x22, 0x94, 0x52, 0x8a, 0x40, \
  0x4d, 0xd0, 0xc6, 0x36, 0x57, 0x40

#define kMaskRandom43_28 \
  0x46, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0x33, 0x0c, 0xc9, 0x99, 0x33, 0x20, \
  0x10, 0xe4, 0x64, 0x8c, 0x91, 0x80, \
  0x0c, 0x69, 0x0d, 0x21, 0xa4, 0x20, \
  0x28, 0x94, 0x2a, 0x85, 0x50, 0xa0, \
  0x94, 0x21, 0x34, 0x26, 0x84, 0xc0, \
  0xc1, 0x02, 0x58, 0x4b, 0x09, 0x60, \
  0x0e, 0x19, 0x13, 0x22, 0x64, 0x40, \
  0x33, 0x14, 0x52, 0x8a, 0x51, 0x40, \
  0x10, 0xc3, 0x28, 0x65, 0x0c, 0xa0, \
  0x45, 0x68, 0x4d, 0x09, 0xa1, 0x20, \
  0x88, 0x84, 0xa8, 0x95, 0x12, 0xa0, \
  0xe0, 0x22, 0x94, 0x52, 0x8a, 0x40, \
  0x4d, 0xd0, 0xc6, 0x36, 0x57, 0x40, \
  0x0e, 0x19, 0x13, 0x22, 0x64, 0x40, \
  0x33, 0x14, 0x52, 0x8a, 0x51, 0x40, \
  0x10, 0xc3, 0x28, 0x65, 0x0c, 0xa0, \
  0x45, 0x68, 0x4d, 0x09, 0xa1, 0x20, \
  0x88, 0x84, 0xa8, 0x95, 0x12, 0xa0, \
  0xe0, 0x22, 0x94, 0x52, 0x8a, 0x40, \
  0x46, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0x33, 0x0c, 0xc9, 0x99, 0x33, 0x20, \
  0x10, 0xe4, 0x64, 0x8c, 0x91, 0x80, \
  0x0c, 0x69, 0x0d, 0x21, 0xa4, 0x20, \
  0x28, 0x94, 0x2a, 0x85, 0x50, 0xa0, \
  0x94, 0x21, 0x34, 0x26, 0x84, 0xc0, \
  0xc1, 0x02, 0x58, 0x4b, 0x09, 0x60, \
  0x16, 0xc9, 0x53, 0x1e, 0xc4, 0x00

#define kMaskRandom43_29 \
  0x46, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0x33, 0x0c, 0xc9, 0x99, 0x33, 0x20, \
  0x10, 0xe4, 0x64, 0x8c, 0x91, 0x80, \
  0x0c, 0x69, 0x0d, 0x21, 0xa4, 0x20, \
  0x28, 0x94, 0x2a, 0x85, 0x50, 0xa0, \
  0x94, 0x21, 0x34, 0x26, 0x84, 0xc0, \
  0xc1, 0x02, 0x58, 0x4b, 0x09, 0x60, \
  0x0e, 0x19, 0x13, 0x22, 0x64, 0x40, \
  0x33, 0x14, 0x52, 0x8a, 0x51, 0x40, \
  0x10, 0xc3, 0x28, 0x65, 0x0c, 0xa0, \
  0x45, 0x68, 0x4d, 0x09, 0xa1, 0x20, \
  0x88, 0x84, 0xa8, 0x95, 0x12, 0xa0, \
  0xe0, 0x22, 0x94, 0x52, 0x8a, 0x40, \
  0x4d, 0xd0, 0xc6, 0x36, 0x57, 0x40, \
  0x46, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0x33, 0x0c, 0xc9, 0x99, 0x33, 0x20, \
  0x10, 0xe4, 0x64, 0x8c, 0x91, 0x80, \
  0x0c, 0x69, 0x0d, 0x21, 0xa4, 0x20, \
  0x28, 0x94, 0x2a, 0x85, 0x50, 0xa0, \
  0x94, 0x21, 0x34, 0x26, 0x84, 0xc0, \
  0xc1, 0x02, 0x58, 0x4b, 0x09, 0x60, \
  0x2c, 0x03, 0x20, 0x64, 0x0c, 0x80, \
  0x81, 0xa0, 0x1c, 0x03, 0x80, 0x60, \
  0xa0, 0x68, 0x25, 0x04, 0xa0, 0x80, \
  0x05, 0x41, 0x50, 0x2a, 0x05, 0x40, \
  0x18, 0x90, 0xc2, 0x18, 0x43, 0x00, \
  0xc2, 0x06, 0x80, 0xd0, 0x1a, 0x00, \
  0x22, 0x98, 0x0b, 0x01, 0x60, 0x20, \
  0x50, 0x45, 0x08, 0xa1, 0x14, 0x20

#define kMaskRandom43_3 \
  0xce, 0x32, 0xb6, 0x56, 0xca, 0xc0, \
  0x55, 0xdc, 0x57, 0x8a, 0xf1, 0x40, \
  0xa8, 0xed, 0x8d, 0xb1, 0xae, 0x20

#define kMaskRandom43_30 \
  0x46, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0x33, 0x0c, 0xc9, 0x99, 0x33, 0x20, \
  0x10, 0xe4, 0x64, 0x8c, 0x91, 0x80, \
  0x0c, 0x69, 0x0d, 0x21, 0xa4, 0x20, \
  0x28, 0x94, 0x2a, 0x85, 0x50, 0xa0, \
  0x94, 0x21, 0x34, 0x26, 0x84, 0xc0, \
  0xc1, 0x02, 0x58, 0x4b, 0x09, 0x60, \
  0x2c, 0x03, 0x20, 0x64, 0x0c, 0x80, \
  0x81, 0xa0, 0x1c, 0x03, 0x80, 0x60, \
  0xa0, 0x68, 0x25, 0x04, 0xa0, 0x80, \
  0x05, 0x41, 0x50, 0x2a, 0x05, 0x40, \
  0x18, 0x90, 0xc2, 0x18, 0x43, 0x00, \
  0xc2, 0x06, 0x80, 0xd0, 0x1a, 0x00, \
  0x22, 0x98, 0x0b, 0x01, 0x60, 0x20, \
  0x50, 0x45, 0x08, 0xa1, 0x14, 0x20, \
  0x46, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0x33, 0x0c, 0xc9, 0x99, 0x33, 0x20, \
  0x10, 0xe4, 0x64, 0x8c, 0x91, 0x80, \
  0x0c, 0x69, 0x0d, 0x21, 0xa4, 0x20, \
  0x28, 0x94, 0x2a, 0x85, 0x50, 0xa0, \
  0x94, 0x21, 0x34, 0x26, 0x84, 0xc0, \
  0xc1, 0x02, 0x58, 0x4b, 0x09, 0x60, \
  0x0e, 0x19, 0x13, 0x22, 0x64, 0x40, \
  0x33, 0x14, 0x52, 0x8a, 0x51, 0x40, \
  0x10, 0xc3, 0x28, 0x65, 0x0c, 0xa0, \
  0x45, 0x68, 0x4d, 0x09, 0xa1, 0x20, \
  0x88, 0x84, 0xa8, 0x95, 0x12, 0xa0, \
  0xe0, 0x22, 0x94, 0x52, 0x8a, 0x40, \
  0x4d, 0xd0, 0xc6, 0x36, 0x57, 0x40, \
  0x79, 0x4a, 0x8f, 0x42, 0x79, 0x40

#define kMaskRandom43_31 \
  0x46, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0x33, 0x0c, 0xc9, 0x99, 0x33, 0x20, \
  0x10, 0xe4, 0x64, 0x8c, 0x91, 0x80, \
  0x0c, 0x69, 0x0d, 0x21, 0xa4, 0x20, \
  0x28, 0x94, 0x2a, 0x85, 0x50, 0xa0, \
  0x94, 0x21, 0x34, 0x26, 0x84, 0xc0, \
  0xc1, 0x02, 0x58, 0x4b, 0x09, 0x60, \
  0x2c, 0x03, 0x20, 0x64, 0x0c, 0x80, \
  0x81, 0xa0, 0x1c, 0x03, 0x80, 0x60, \
  0xa0, 0x68, 0x25, 0x04, 0xa0, 0x80, \
  0x05, 0x41, 0x50, 0x2a, 0x05, 0x40, \
  0x18, 0x90, 0xc2, 0x18, 0x43, 0x00, \
  0xc2, 0x06, 0x80, 0xd0, 0x1a, 0x00, \
  0x22, 0x98, 0x0b, 0x01, 0x60, 0x20, \
  0x50, 0x45, 0x08, 0xa1, 0x14, 0x20, \
  0x2c, 0x03, 0x20, 0x64, 0x0c, 0x80, \
  0x81, 0xa0, 0x1c, 0x03, 0x80, 0x60, \
  0xa0, 0x68, 0x25, 0x04, 0xa0, 0x80, \
  0x05, 0x41, 0x50, 0x2a, 0x05, 0x40, \
  0x18, 0x90, 0xc2, 0x18, 0x43, 0x00, \
  0xc2, 0x06, 0x80, 0xd0, 0x1a, 0x00, \
  0x22, 0x98, 0x0b, 0x01, 0x60, 0x20, \
  0x50, 0x45, 0x08, 0xa1, 0x14, 0x20, \
  0x46, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0x33, 0x0c, 0xc9, 0x99, 0x33, 0x20, \
  0x10, 0xe4, 0x64, 0x8c, 0x91, 0x80, \
  0x0c, 0x69, 0x0d, 0x21, 0xa4, 0x20, \
  0x28, 0x94, 0x2a, 0x85, 0x50, 0xa0, \
  0x94, 0x21, 0x34, 0x26, 0x84, 0xc0, \
  0xc1, 0x02, 0x58, 0x4b, 0x09, 0x60, \
  0x3b, 0xf5, 0x3c, 0x36, 0x0a, 0x20

#define kMaskRandom43_32 \
  0x2c, 0x03, 0x20, 0x64, 0x0c, 0x80, \
  0x81, 0xa0, 0x1c, 0x03, 0x80, 0x60, \
  0xa0, 0x68, 0x25, 0x04, 0xa0, 0x80, \
  0x05, 0x41, 0x50, 0x2a, 0x05, 0x40, \
  0x18, 0x90, 0xc2, 0x18, 0x43, 0x00, \
  0xc2, 0x06, 0x80, 0xd0, 0x1a, 0x00, \
  0x22, 0x98, 0x0b, 0x01, 0x60, 0x20, \
  0x50, 0x45, 0x08, 0xa1, 0x14, 0x20, \
  0x46, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0x33, 0x0c, 0xc9, 0x99, 0x33, 0x20, \
  0x10, 0xe4, 0x64, 0x8c, 0x91, 0x80, \
  0x0c, 0x69, 0x0d, 0x21, 0xa4, 0x20, \
  0x28, 0x94, 0x2a, 0x85, 0x50, 0xa0, \
  0x94, 0x21, 0x34, 0x26, 0x84, 0xc0, \
  0xc1, 0x02, 0x58, 0x4b, 0x09, 0x60, \
  0x3b, 0xf5, 0x3c, 0x36, 0x0a, 0x20, \
  0x46, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0x33, 0x0c, 0xc9, 0x99, 0x33, 0x20, \
  0x10, 0xe4, 0x64, 0x8c, 0x91, 0x80, \
  0x0c, 0x69, 0x0d, 0x21, 0xa4, 0x20, \
  0x28, 0x94, 0x2a, 0x85, 0x50, 0xa0, \
  0x94, 0x21, 0x34, 0x26, 0x84, 0xc0, \
  0xc1, 0x02, 0x58, 0x4b, 0x09, 0x60, \
  0x2c, 0x03, 0x20, 0x64, 0x0c, 0x80, \
  0x81, 0xa0, 0x1c, 0x03, 0x80, 0x60, \
  0xa0, 0x68, 0x25, 0x04, 0xa0, 0x80, \
  0x05, 0x41, 0x50, 0x2a, 0x05, 0x40, \
  0x18, 0x90, 0xc2, 0x18, 0x43, 0x00, \
  0xc2, 0x06, 0x80, 0xd0, 0x1a, 0x00, \
  0x22, 0x98, 0x0b, 0x01, 0x60, 0x20, \
  0x50, 0x45, 0x08, 0xa1, 0x14, 0x20, \
  0xd1, 0xd1, 0x11, 0xa4, 0xed, 0xc0

#define kMaskRandom43_33 \
  0x2c, 0x03, 0x20, 0x64, 0x0c, 0x80, \
  0x81, 0xa0, 0x1c, 0x03, 0x80, 0x60, \
  0xa0, 0x68, 0x25, 0x04, 0xa0, 0x80, \
  0x05, 0x41, 0x50, 0x2a, 0x05, 0x40, \
  0x18, 0x90, 0xc2, 0x18, 0x43, 0x00, \
  0xc2, 0x06, 0x80, 0xd0, 0x1a, 0x00, \
  0x22, 0x98, 0x0b, 0x01, 0x60, 0x20, \
  0x50, 0x45, 0x08, 0xa1, 0x14, 0x20, \
  0x46, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0x33, 0x0c, 0xc9, 0x99, 0x33, 0x20, \
  0x10, 0xe4, 0x64, 0x8c, 0x91, 0x80, \
  0x0c, 0x69, 0x0d, 0x21, 0xa4, 0x20, \
  0x28, 0x94, 0x2a, 0x85, 0x50, 0xa0, \
  0x94, 0x21, 0x34, 0x26, 0x84, 0xc0, \
  0xc1, 0x02, 0x58, 0x4b, 0x09, 0x60, \
  0x3b, 0xf5, 0x3c, 0x36, 0x0a, 0x20, \
  0x2c, 0x03, 0x20, 0x64, 0x0c, 0x80, \
  0x81, 0xa0, 0x1c, 0x03, 0x80, 0x60, \
  0xa0, 0x68, 0x25, 0x04, 0xa0, 0x80, \
  0x05, 0x41, 0x50, 0x2a, 0x05, 0x40, \
  0x18, 0x90, 0xc2, 0x18, 0x43, 0x00, \
  0xc2, 0x06, 0x80, 0xd0, 0x1a, 0x00, \
  0x22, 0x98, 0x0b, 0x01, 0x60, 0x20, \
  0x50, 0x45, 0x08, 0xa1, 0x14, 0x20, \
  0x4e, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0xe3, 0x18, 0x9b, 0x13, 0x62, 0x60, \
  0x81, 0xe3, 0x04, 0x60, 0x8c, 0x00, \
  0x21, 0x40, 0x58, 0x0b, 0x01, 0x60, \
  0x52, 0x81, 0xe0, 0x3c, 0x07, 0x80, \
  0xb4, 0x28, 0x25, 0x04, 0xa0, 0x80, \
  0x26, 0x86, 0x28, 0xc5, 0x18, 0xa0, \
  0x58, 0x64, 0x44, 0x88, 0x91, 0x00, \
  0x19, 0x9e, 0x03, 0xc0, 0x78, 0x00

#define kMaskRandom43_34 \
  0x2c, 0x03, 0x20, 0x64, 0x0c, 0x80, \
  0x81, 0xa0, 0x1c, 0x03, 0x80, 0x60, \
  0xa0, 0x68, 0x25, 0x04, 0xa0, 0x80, \
  0x05, 0x41, 0x50, 0x2a, 0x05, 0x40, \
  0x18, 0x90, 0xc2, 0x18, 0x43, 0x00, \
  0xc2, 0x06, 0x80, 0xd0, 0x1a, 0x00, \
  0x22, 0x98, 0x0b, 0x01, 0x60, 0x20, \
  0x50, 0x45, 0x08, 0xa1, 0x14, 0x20, \
  0x4e, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0xe3, 0x18, 0x9b, 0x13, 0x62, 0x60, \
  0x81, 0xe3, 0x04, 0x60, 0x8c, 0x00, \
  0x21, 0x40, 0x58, 0x0b, 0x01, 0x60, \
  0x52, 0x81, 0xe0, 0x3c, 0x07, 0x80, \
  0xb4, 0x28, 0x25, 0x04, 0xa0, 0x80, \
  0x26, 0x86, 0x28, 0xc5, 0x18, 0xa0, \
  0x58, 0x64, 0x44, 0x88, 0x91, 0x00, \
  0x19, 0x9e, 0x03, 0xc0, 0x78, 0x00, \
  0x2c, 0x03, 0x20, 0x64, 0x0c, 0x80, \
  0x81, 0xa0, 0x1c, 0x03, 0x80, 0x60, \
  0xa0, 0x68, 0x25, 0x04, 0xa0, 0x80, \
  0x05, 0x41, 0x50, 0x2a, 0x05, 0x40, \
  0x18, 0x90, 0xc2, 0x18, 0x43, 0x00, \
  0xc2, 0x06, 0x80, 0xd0, 0x1a, 0x00, \
  0x22, 0x98, 0x0b, 0x01, 0x60, 0x20, \
  0x50, 0x45, 0x08, 0xa1, 0x14, 0x20, \
  0x46, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0x33, 0x0c, 0xc9, 0x99, 0x33, 0x20, \
  0x10, 0xe4, 0x64, 0x8c, 0x91, 0x80, \
  0x0c, 0x69, 0x0d, 0x21, 0xa4, 0x20, \
  0x28, 0x94, 0x2a, 0x85, 0x50, 0xa0, \
  0x94, 0x21, 0x34, 0x26, 0x84, 0xc0, \
  0xc1, 0x02, 0x58, 0x4b, 0x09, 0x60, \
  0x3b, 0xf5, 0x3c, 0x36, 0x0a, 0x20, \
  0x76, 0x81, 0x4d, 0x33, 0x66, 0x00

#define kMaskRandom43_35 \
  0x2c, 0x03, 0x20, 0x64, 0x0c, 0x80, \
  0x81, 0xa0, 0x1c, 0x03, 0x80, 0x60, \
  0xa0, 0x68, 0x25, 0x04, 0xa0, 0x80, \
  0x05, 0x41, 0x50, 0x2a, 0x05, 0x40, \
  0x18, 0x90, 0xc2, 0x18, 0x43, 0x00, \
  0xc2, 0x06, 0x80, 0xd0, 0x1a, 0x00, \
  0x22, 0x98, 0x0b, 0x01, 0x60, 0x20, \
  0x50, 0x45, 0x08, 0xa1, 0x14, 0x20, \
  0x4e, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0xe3, 0x18, 0x9b, 0x13, 0x62, 0x60, \
  0x81, 0xe3, 0x04, 0x60, 0x8c, 0x00, \
  0x21, 0x40, 0x58, 0x0b, 0x01, 0x60, \
  0x52, 0x81, 0xe0, 0x3c, 0x07, 0x80, \
  0xb4, 0x28, 0x25, 0x04, 0xa0, 0x80, \
  0x26, 0x86, 0x28, 0xc5, 0x18, 0xa0, \
  0x58, 0x64, 0x44, 0x88, 0x91, 0x00, \
  0x19, 0x9e, 0x03, 0xc0, 0x78, 0x00, \
  0x4e, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0xe3, 0x18, 0x9b, 0x13, 0x62, 0x60, \
  0x81, 0xe3, 0x04, 0x60, 0x8c, 0x00, \
  0x21, 0x40, 0x58, 0x0b, 0x01, 0x60, \
  0x52, 0x81, 0xe0, 0x3c, 0x07, 0x80, \
  0xb4, 0x28, 0x25, 0x04, 0xa0, 0x80, \
  0x26, 0x86, 0x28, 0xc5, 0x18, 0xa0, \
  0x58, 0x64, 0x44, 0x88, 0x91, 0x00, \
  0x19, 0x9e, 0x03, 0xc0, 0x78, 0x00, \
  0x2c, 0x03, 0x20, 0x64, 0x0c, 0x80, \
  0x81, 0xa0, 0x1c, 0x03, 0x80, 0x60, \
  0xa0, 0x68, 0x25, 0x04, 0xa0, 0x80, \
  0x05, 0x41, 0x50, 0x2a, 0x05, 0x40, \
  0x18, 0x90, 0xc2, 0x18, 0x43, 0x00, \
  0xc2, 0x06, 0x80, 0xd0, 0x1a, 0x00, \
  0x22, 0x98, 0x0b, 0x01, 0x60, 0x20, \
  0x50, 0x45, 0x08, 0xa1, 0x14, 0x20, \
  0x5a, 0x56, 0x5f, 0x26, 0xa3, 0x60

#define kMaskRandom43_36 \
  0x4e, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0xe3, 0x18, 0x9b, 0x13, 0x62, 0x60, \
  0x81, 0xe3, 0x04, 0x60, 0x8c, 0x00, \
  0x21, 0x40, 0x58, 0x0b, 0x01, 0x60, \
  0x52, 0x81, 0xe0, 0x3c, 0x07, 0x80, \
  0xb4, 0x28, 0x25, 0x04, 0xa0, 0x80, \
  0x26, 0x86, 0x28, 0xc5, 0x18, 0xa0, \
  0x58, 0x64, 0x44, 0x88, 0x91, 0x00, \
  0x19, 0x9e, 0x03, 0xc0, 0x78, 0x00, \
  0x2c, 0x03, 0x20, 0x64, 0x0c, 0x80, \
  0x81, 0xa0, 0x1c, 0x03, 0x80, 0x60, \
  0xa0, 0x68, 0x25, 0x04, 0xa0, 0x80, \
  0x05, 0x41, 0x50, 0x2a, 0x05, 0x40, \
  0x18, 0x90, 0xc2, 0x18, 0x43, 0x00, \
  0xc2, 0x06, 0x80, 0xd0, 0x1a, 0x00, \
  0x22, 0x98, 0x0b, 0x01, 0x60, 0x20, \
  0x50, 0x45, 0x08, 0xa1, 0x14, 0x20, \
  0x5a, 0x56, 0x5f, 0x26, 0xa3, 0x60, \
  0x2c, 0x03, 0x20, 0x64, 0x0c, 0x80, \
  0x81, 0xa0, 0x1c, 0x03, 0x80, 0x60, \
  0xa0, 0x68, 0x25, 0x04, 0xa0, 0x80, \
  0x05, 0x41, 0x50, 0x2a, 0x05, 0x40, \
  0x18, 0x90, 0xc2, 0x18, 0x43, 0x00, \
  0xc2, 0x06, 0x80, 0xd0, 0x1a, 0x00, \
  0x22, 0x98, 0x0b, 0x01, 0x60, 0x20, \
  0x50, 0x45, 0x08, 0xa1, 0x14, 0x20, \
  0x4e, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0xe3, 0x18, 0x9b, 0x13, 0x62, 0x60, \
  0x81, 0xe3, 0x04, 0x60, 0x8c, 0x00, \
  0x21, 0x40, 0x58, 0x0b, 0x01, 0x60, \
  0x52, 0x81, 0xe0, 0x3c, 0x07, 0x80, \
  0xb4, 0x28, 0x25, 0x04, 0xa0, 0x80, \
  0x26, 0x86, 0x28, 0xc5, 0x18, 0xa0, \
  0x58, 0x64, 0x44, 0x88, 0x91, 0x00, \
  0x19, 0x9e, 0x03, 0xc0, 0x78, 0x00, \
  0xa3, 0x85, 0x0a, 0xb5, 0x11, 0x60

#define kMaskRandom43_37 \
  0x4e, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0xe3, 0x18, 0x9b, 0x13, 0x62, 0x60, \
  0x81, 0xe3, 0x04, 0x60, 0x8c, 0x00, \
  0x21, 0x40, 0x58, 0x0b, 0x01, 0x60, \
  0x52, 0x81, 0xe0, 0x3c, 0x07, 0x80, \
  0xb4, 0x28, 0x25, 0x04, 0xa0, 0x80, \
  0x26, 0x86, 0x28, 0xc5, 0x18, 0xa0, \
  0x58, 0x64, 0x44, 0x88, 0x91, 0x00, \
  0x19, 0x9e, 0x03, 0xc0, 0x78, 0x00, \
  0x2c, 0x03, 0x20, 0x64, 0x0c, 0x80, \
  0x81, 0xa0, 0x1c, 0x03, 0x80, 0x60, \
  0xa0, 0x68, 0x25, 0x04, 0xa0, 0x80, \
  0x05, 0x41, 0x50, 0x2a, 0x05, 0x40, \
  0x18, 0x90, 0xc2, 0x18, 0x43, 0x00, \
  0xc2, 0x06, 0x80, 0xd0, 0x1a, 0x00, \
  0x22, 0x98, 0x0b, 0x01, 0x60, 0x20, \
  0x50, 0x45, 0x08, 0xa1, 0x14, 0x20, \
  0x5a, 0x56, 0x5f, 0x26, 0xa3, 0x60, \
  0x4e, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0xe3, 0x18, 0x9b, 0x13, 0x62, 0x60, \
  0x81, 0xe3, 0x04, 0x60, 0x8c, 0x00, \
  0x21, 0x40, 0x58, 0x0b, 0x01, 0x60, \
  0x52, 0x81, 0xe0, 0x3c, 0x07, 0x80, \
  0xb4, 0x28, 0x25, 0x04, 0xa0, 0x80, \
  0x26, 0x86, 0x28, 0xc5, 0x18, 0xa0, \
  0x58, 0x64, 0x44, 0x88, 0x91, 0x00, \
  0x19, 0x9e, 0x03, 0xc0, 0x78, 0x00, \
  0x4c, 0x19, 0x13, 0x22, 0x64, 0x40, \
  0x51, 0x14, 0x52, 0x8a, 0x51, 0x40, \
  0xa0, 0x6a, 0x45, 0x48, 0xa9, 0x00, \
  0x04, 0xc1, 0x30, 0x26, 0x04, 0xc0, \
  0x03, 0xb4, 0x06, 0x80, 0xd0, 0x00, \
  0x86, 0x20, 0x94, 0x12, 0x82, 0x40, \
  0x29, 0x08, 0x49, 0x09, 0x21, 0x20, \
  0x42, 0x43, 0x08, 0x61, 0x0c, 0x20, \
  0x98, 0x12, 0x82, 0x50, 0x4a, 0x00, \
  0x30, 0x84, 0xa8, 0x95, 0x12, 0xa0

#define kMaskRandom43_38 \
  0x4e, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0xe3, 0x18, 0x9b, 0x13, 0x62, 0x60, \
  0x81, 0xe3, 0x04, 0x60, 0x8c, 0x00, \
  0x21, 0x40, 0x58, 0x0b, 0x01, 0x60, \
  0x52, 0x81, 0xe0, 0x3c, 0x07, 0x80, \
  0xb4, 0x28, 0x25, 0x04, 0xa0, 0x80, \
  0x26, 0x86, 0x28, 0xc5, 0x18, 0xa0, \
  0x58, 0x64, 0x44, 0x88, 0x91, 0x00, \
  0x19, 0x9e, 0x03, 0xc0, 0x78, 0x00, \
  0x4c, 0x19, 0x13, 0x22, 0x64, 0x40, \
  0x51, 0x14, 0x52, 0x8a, 0x51, 0x40, \
  0xa0, 0x6a, 0x45, 0x48, 0xa9, 0x00, \
  0x04, 0xc1, 0x30, 0x26, 0x04, 0xc0, \
  0x03, 0xb4, 0x06, 0x80, 0xd0, 0x00, \
  0x86, 0x20, 0x94, 0x12, 0x82, 0x40, \
  0x29, 0x08, 0x49, 0x09, 0x21, 0x20, \
  0x42, 0x43, 0x08, 0x61, 0x0c, 0x20, \
  0x98, 0x12, 0x82, 0x50, 0x4a, 0x00, \
  0x30, 0x84, 0xa8, 0x95, 0x12, 0xa0, \
  0x4e, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0xe3, 0x18, 0x9b, 0x13, 0x62, 0x60, \
  0x81, 0xe3, 0x04, 0x60, 0x8c, 0x00, \
  0x21, 0x40, 0x58, 0x0b, 0x01, 0x60, \
  0x52, 0x81, 0xe0, 0x3c, 0x07, 0x80, \
  0xb4, 0x28, 0x25, 0x04, 0xa0, 0x80, \
  0x26, 0x86, 0x28, 0xc5, 0x18, 0xa0, \
  0x58, 0x64, 0x44, 0x88, 0x91, 0x00, \
  0x19, 0x9e, 0x03, 0xc0, 0x78, 0x00, \
  0x2c, 0x03, 0x20, 0x64, 0x0c, 0x80, \
  0x81, 0xa0, 0x1c, 0x03, 0x80, 0x60, \
  0xa0, 0x68, 0x25, 0x04, 0xa0, 0x80, \
  0x05, 0x41, 0x50, 0x2a, 0x05, 0x40, \
  0x18, 0x90, 0xc2, 0x18, 0x43, 0x00, \
  0xc2, 0x06, 0x80, 0xd0, 0x1a, 0x00, \
  0x22, 0x98, 0x0b, 0x01, 0x60, 0x20, \
  0x50, 0x45, 0x08, 0xa1, 0x14, 0x20, \
  0x5a, 0x56, 0x5f, 0x26, 0xa3, 0x60, \
  0x9a, 0x16, 0x97, 0x21, 0xb9, 0x80

#define kMaskRandom43_39 \
  0x4e, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0xe3, 0x18, 0x9b, 0x13, 0x62, 0x60, \
  0x81, 0xe3, 0x04, 0x60, 0x8c, 0x00, \
  0x21, 0x40, 0x58, 0x0b, 0x01, 0x60, \
  0x52, 0x81, 0xe0, 0x3c, 0x07, 0x80, \
  0xb4, 0x28, 0x25, 0x04, 0xa0, 0x80, \
  0x26, 0x86, 0x28, 0xc5, 0x18, 0xa0, \
  0x58, 0x64, 0x44, 0x88, 0x91, 0x00, \
  0x19, 0x9e, 0x03, 0xc0, 0x78, 0x00, \
  0x4c, 0x19, 0x13, 0x22, 0x64, 0x40, \
  0x51, 0x14, 0x52, 0x8a, 0x51, 0x40, \
  0xa0, 0x6a, 0x45, 0x48, 0xa9, 0x00, \
  0x04, 0xc1, 0x30, 0x26, 0x04, 0xc0, \
  0x03, 0xb4, 0x06, 0x80, 0xd0, 0x00, \
  0x86, 0x20, 0x94, 0x12, 0x82, 0x40, \
  0x29, 0x08, 0x49, 0x09, 0x21, 0x20, \
  0x42, 0x43, 0x08, 0x61, 0x0c, 0x20, \
  0x98, 0x12, 0x82, 0x50, 0x4a, 0x00, \
  0x30, 0x84, 0xa8, 0x95, 0x12, 0xa0, \
  0x4c, 0x19, 0x13, 0x22, 0x64, 0x40, \
  0x51, 0x14, 0x52, 0x8a, 0x51, 0x40, \
  0xa0, 0x6a, 0x45, 0x48, 0xa9, 0x00, \
  0x04, 0xc1, 0x30, 0x26, 0x04, 0xc0, \
  0x03, 0xb4, 0x06, 0x80, 0xd0, 0x00, \
  0x86, 0x20, 0x94, 0x12, 0x82, 0x40, \
  0x29, 0x08, 0x49, 0x09, 0x21, 0x20, \
  0x42, 0x43, 0x08, 0x61, 0x0c, 0x20, \
  0x98, 0x12, 0x82, 0x50, 0x4a, 0x00, \
  0x30, 0x84, 0xa8, 0x95, 0x12, 0xa0, \
  0x4e, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0xe3, 0x18, 0x9b, 0x13, 0x62, 0x60, \
  0x81, 0xe3, 0x04, 0x60, 0x8c, 0x00, \
  0x21, 0x40, 0x58, 0x0b, 0x01, 0x60, \
  0x52, 0x81, 0xe0, 0x3c, 0x07, 0x80, \
  0xb4, 0x28, 0x25, 0x04, 0xa0, 0x80, \
  0x26, 0x86, 0x28, 0xc5, 0x18, 0xa0, \
  0x58, 0x64, 0x44, 0x88, 0x91, 0x00, \
  0x19, 0x9e, 0x03, 0xc0, 0x78, 0x00, \
  0x2a, 0x03, 0x31, 0xda, 0x46, 0x20

#define kMaskRandom43_4 \
  0xe6, 0x31, 0x36, 0x26, 0xc4, 0xc0, \
  0x33, 0x8c, 0x59, 0x8b, 0x31, 0x60, \
  0x98, 0xd2, 0xca, 0x59, 0x4b, 0x20, \
  0x2d, 0x4b, 0x29, 0x65, 0x2c, 0xa0

#define kMaskRandom43_40 \
  0x4c, 0x19, 0x13, 0x22, 0x64, 0x40, \
  0x51, 0x14, 0x52, 0x8a, 0x51, 0x40, \
  0xa0, 0x6a, 0x45, 0x48, 0xa9, 0x00, \
  0x04, 0xc1, 0x30, 0x26, 0x04, 0xc0, \
  0x03, 0xb4, 0x06, 0x80, 0xd0, 0x00, \
  0x86, 0x20, 0x94, 0x12, 0x82, 0x40, \
  0x29, 0x08, 0x49, 0x09, 0x21, 0x20, \
  0x42, 0x43, 0x08, 0x61, 0x0c, 0x20, \
  0x98, 0x12, 0x82, 0x50, 0x4a, 0x00, \
  0x30, 0x84, 0xa8, 0x95, 0x12, 0xa0, \
  0x4e, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0xe3, 0x18, 0x9b, 0x13, 0x62, 0x60, \
  0x81, 0xe3, 0x04, 0x60, 0x8c, 0x00, \
  0x21, 0x40, 0x58, 0x0b, 0x01, 0x60, \
  0x52, 0x81, 0xe0, 0x3c, 0x07, 0x80, \
  0xb4, 0x28, 0x25, 0x04, 0xa0, 0x80, \
  0x26, 0x86, 0x28, 0xc5, 0x18, 0xa0, \
  0x58, 0x64, 0x44, 0x88, 0x91, 0x00, \
  0x19, 0x9e, 0x03, 0xc0, 0x78, 0x00, \
  0x2a, 0x03, 0x31, 0xda, 0x46, 0x20, \
  0x4e, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0xe3, 0x18, 0x9b, 0x13, 0x62, 0x60, \
  0x81, 0xe3, 0x04, 0x60, 0x8c, 0x00, \
  0x21, 0x40, 0x58, 0x0b, 0x01, 0x60, \
  0x52, 0x81, 0xe0, 0x3c, 0x07, 0x80, \
  0xb4, 0x28, 0x25, 0x04, 0xa0, 0x80, \
  0x26, 0x86, 0x28, 0xc5, 0x18, 0xa0, \
  0x58, 0x64, 0x44, 0x88, 0x91, 0x00, \
  0x19, 0x9e, 0x03, 0xc0, 0x78, 0x00, \
  0x4c, 0x19, 0x13, 0x22, 0x64, 0x40, \
  0x51, 0x14, 0x52, 0x8a, 0x51, 0x40, \
  0xa0, 0x6a, 0x45, 0x48, 0xa9, 0x00, \
  0x04, 0xc1, 0x30, 0x26, 0x04, 0xc0, \
  0x03, 0xb4, 0x06, 0x80, 0xd0, 0x00, \
  0x86, 0x20, 0x94, 0x12, 0x82, 0x40, \
  0x29, 0x08, 0x49, 0x09, 0x21, 0x20, \
  0x42, 0x43, 0x08, 0x61, 0x0c, 0x20, \
  0x98, 0x12, 0x82, 0x50, 0x4a, 0x00, \
  0x30, 0x84, 0xa8, 0x95, 0x12, 0xa0, \
  0x3a, 0xab, 0x77, 0x63, 0xef, 0x60

#define kMaskRandom43_41 \
  0x4c, 0x19, 0x13, 0x22, 0x64, 0x40, \
  0x51, 0x14, 0x52, 0x8a, 0x51, 0x40, \
  0xa0, 0x6a, 0x45, 0x48, 0xa9, 0x00, \
  0x04, 0xc1, 0x30, 0x26, 0x04, 0xc0, \
  0x03, 0xb4, 0x06, 0x80, 0xd0, 0x00, \
  0x86, 0x20, 0x94, 0x12, 0x82, 0x40, \
  0x29, 0x08, 0x49, 0x09, 0x21, 0x20, \
  0x42, 0x43, 0x08, 0x61, 0x0c, 0x20, \
  0x98, 0x12, 0x82, 0x50, 0x4a, 0x00, \
  0x30, 0x84, 0xa8, 0x95, 0x12, 0xa0, \
  0x4e, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0xe3, 0x18, 0x9b, 0x13, 0x62, 0x60, \
  0x81, 0xe3, 0x04, 0x60, 0x8c, 0x00, \
  0x21, 0x40, 0x58, 0x0b, 0x01, 0x60, \
  0x52, 0x81, 0xe0, 0x3c, 0x07, 0x80, \
  0xb4, 0x28, 0x25, 0x04, 0xa0, 0x80, \
  0x26, 0x86, 0x28, 0xc5, 0x18, 0xa0, \
  0x58, 0x64, 0x44, 0x88, 0x91, 0x00, \
  0x19, 0x9e, 0x03, 0xc0, 0x78, 0x00, \
  0x2a, 0x03, 0x31, 0xda, 0x46, 0x20, \
  0x4c, 0x19, 0x13, 0x22, 0x64, 0x40, \
  0x51, 0x14, 0x52, 0x8a, 0x51, 0x40, \
  0xa0, 0x6a, 0x45, 0x48, 0xa9, 0x00, \
  0x04, 0xc1, 0x30, 0x26, 0x04, 0xc0, \
  0x03, 0xb4, 0x06, 0x80, 0xd0, 0x00, \
  0x86, 0x20, 0x94, 0x12, 0x82, 0x40, \
  0x29, 0x08, 0x49, 0x09, 0x21, 0x20, \
  0x42, 0x43, 0x08, 0x61, 0x0c, 0x20, \
  0x98, 0x12, 0x82, 0x50, 0x4a, 0x00, \
  0x30, 0x84, 0xa8, 0x95, 0x12, 0xa0, \
  0x4c, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0x51, 0x0c, 0xc9, 0x99, 0x33, 0x20, \
  0xa0, 0x66, 0x44, 0xc8, 0x99, 0x00, \
  0x04, 0xc1, 0x60, 0x2c, 0x05, 0x80, \
  0x03, 0xa0, 0x2c, 0x05, 0x80, 0xa0, \
  0x86, 0x21, 0x14, 0x22, 0x84, 0x40, \
  0x29, 0x10, 0x1a, 0x03, 0x40, 0x60, \
  0x42, 0x42, 0xa0, 0x54, 0x0a, 0x80, \
  0x98, 0x1a, 0x03, 0x40, 0x68, 0x00, \
  0x30, 0x84, 0x08, 0x81, 0x10, 0x20, \
  0xdf, 0x4c, 0x11, 0x82, 0x30, 0x40

#define kMaskRandom43_42 \
  0x4c, 0x19, 0x13, 0x22, 0x64, 0x40, \
  0x51, 0x14, 0x52, 0x8a, 0x51, 0x40, \
  0xa0, 0x6a, 0x45, 0x48, 0xa9, 0x00, \
  0x04, 0xc1, 0x30, 0x26, 0x04, 0xc0, \
  0x03, 0xb4, 0x06, 0x80, 0xd0, 0x00, \
  0x86, 0x20, 0x94, 0x12, 0x82, 0x40, \
  0x29, 0x08, 0x49, 0x09, 0x21, 0x20, \
  0x42, 0x43, 0x08, 0x61, 0x0c, 0x20, \
  0x98, 0x12, 0x82, 0x50, 0x4a, 0x00, \
  0x30, 0x84, 0xa8, 0x95, 0x12, 0xa0, \
  0x4c, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0x51, 0x0c, 0xc9, 0x99, 0x33, 0x20, \
  0xa0, 0x66, 0x44, 0xc8, 0x99, 0x00, \
  0x04, 0xc1, 0x60, 0x2c, 0x05, 0x80, \
  0x03, 0xa0, 0x2c, 0x05, 0x80, 0xa0, \
  0x86, 0x21, 0x14, 0x22, 0x84, 0x40, \
  0x29, 0x10, 0x1a, 0x03, 0x40, 0x60, \
  0x42, 0x42, 0xa0, 0x54, 0x0a, 0x80, \
  0x98, 0x1a, 0x03, 0x40, 0x68, 0x00, \
  0x30, 0x84, 0x08, 0x81, 0x10, 0x20, \
  0xdf, 0x4c, 0x11, 0x82, 0x30, 0x40, \
  0x4c, 0x19, 0x13, 0x22, 0x64, 0x40, \
  0x51, 0x14, 0x52, 0x8a, 0x51, 0x40, \
  0xa0, 0x6a, 0x45, 0x48, 0xa9, 0x00, \
  0x04, 0xc1, 0x30, 0x26, 0x04, 0xc0, \
  0x03, 0xb4, 0x06, 0x80, 0xd0, 0x00, \
  0x86, 0x20, 0x94, 0x12, 0x82, 0x40, \
  0x29, 0x08, 0x49, 0x09, 0x21, 0x20, \
  0x42, 0x43, 0x08, 0x61, 0x0c, 0x20, \
  0x98, 0x12, 0x82, 0x50, 0x4a, 0x00, \
  0x30, 0x84, 0xa8, 0x95, 0x12, 0xa0, \
  0x4e, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0xe3, 0x18, 0x9b, 0x13, 0x62, 0x60, \
  0x81, 0xe3, 0x04, 0x60, 0x8c, 0x00, \
  0x21, 0x40, 0x58, 0x0b, 0x01, 0x60, \
  0x52, 0x81, 0xe0, 0x3c, 0x07, 0x80, \
  0xb4, 0x28, 0x25, 0x04, 0xa0, 0x80, \
  0x26, 0x86, 0x28, 0xc5, 0x18, 0xa0, \
  0x58, 0x64, 0x44, 0x88, 0x91, 0x00, \
  0x19, 0x9e, 0x03, 0xc0, 0x78, 0x00, \
  0x2a, 0x03, 0x31, 0xda, 0x46, 0x20, \
  0x26, 0x84, 0x10, 0xcd, 0xf7, 0x60

#define kMaskRandom43_43 \
  0x4c, 0x19, 0x13, 0x22, 0x64, 0x40, \
  0x51, 0x14, 0x52, 0x8a, 0x51, 0x40, \
  0xa0, 0x6a, 0x45, 0x48, 0xa9, 0x00, \
  0x04, 0xc1, 0x30, 0x26, 0x04, 0xc0, \
  0x03, 0xb4, 0x06, 0x80, 0xd0, 0x00, \
  0x86, 0x20, 0x94, 0x12, 0x82, 0x40, \
  0x29, 0x08, 0x49, 0x09, 0x21, 0x20, \
  0x42, 0x43, 0x08, 0x61, 0x0c, 0x20, \
  0x98, 0x12, 0x82, 0x50, 0x4a, 0x00, \
  0x30, 0x84, 0xa8, 0x95, 0x12, 0xa0, \
  0x4c, 0x11, 0x92, 0x32, 0x46, 0x40, \
  0x51, 0x0c, 0xc9, 0x99, 0x33, 0x20, \
  0xa0, 0x66, 0x44, 0xc8, 0x99, 0x00, \
  0x04, 0xc1, 0x60, 0x2c, 0x05, 0x80, \
  0x03, 0xa0, 0x2c, 0x05, 0x80, 0xa0, \
  0x86, 0x21, 0x14, 0x22, 0x84, 0x40, \
  0x29, 0x10, 0x1a, 0x03, 0x40, 0x60, \
  0x42, 0x42, 0xa0, 0x54, 0x0a, 0x80, \
  0x98, 0x1a, 0x03, 0x40, 0x68, 0x00, \
  0x30, 0x84, 0x08, 0x81, 0x10, 0x20, \
  0xdf, 0x4c, 0x11, 0x82, 0x30, 0x40, \
  0x4c, 0x19, 0x12, 0x32, 0x46, 0x40, \
  0x51, 0x14, 0x51, 0x99, 0x33, 0x20, \
  0xa0, 0x6a, 0x44, 0xc8, 0x99, 0x00, \
  0x04, 0xc1, 0x30, 0x2c, 0x05, 0x80, \
  0x03, 0xb4, 0x04, 0x05, 0x80, 0xa0, \
  0x86, 0x20, 0x94, 0x22, 0x84, 0x40, \
  0x29, 0x08, 0x4a, 0x03, 0x40, 0x60, \
  0x42, 0x43, 0x08, 0x54, 0x0a, 0x80, \
  0x98, 0x12, 0x83, 0x40, 0x68, 0x00, \
  0x30, 0x84, 0xa8, 0x81, 0x10, 0x20, \
  0x4c, 0x11, 0x91, 0x82, 0x30, 0x40, \
  0x51, 0x0c, 0xcb, 0x22, 0x64, 0x40, \
  0xa0, 0x66, 0x42, 0x8a, 0x51, 0x40, \
  0x04, 0xc1, 0x65, 0x48, 0xa9, 0x00, \
  0x03, 0xa0, 0x28, 0x26, 0x04, 0xc0, \
  0x86, 0x21, 0x16, 0x80, 0xd0, 0x00, \
  0x29, 0x10, 0x1c, 0x12, 0x82, 0x40, \
  0x42, 0x42, 0xa1, 0x09, 0x21, 0x20, \
  0x98, 0x1a, 0x00, 0x61, 0x0c, 0x20, \
  0x30, 0x84, 0x0a, 0x50, 0x4a, 0x00, \
  0xdf, 0x4c, 0x10, 0x95, 0x12, 0xa0, \
  0x72, 0x06, 0x94, 0xf6, 0x74, 0x40

#define kMaskRandom43_5 \
  0xce, 0x31, 0xb6, 0x36, 0xc6, 0xc0, \
  0x63, 0x98, 0xdb, 0x1b, 0x63, 0x60, \
  0x98, 0xc7, 0x68, 0xed, 0x1d, 0xa0, \
  0x4d, 0x6b, 0x55, 0x6a, 0xad, 0x40, \
  0xb2, 0x6c, 0xad, 0x95, 0xb2, 0xa0

#define kMaskRandom43_6 \
  0x4c, 0x19, 0x13, 0x22, 0x64, 0x40, \
  0x51, 0x14, 0x52, 0x8a, 0x51, 0x40, \
  0x20, 0xea, 0x0d, 0x41, 0xa8, 0x20, \
  0x85, 0x41, 0x2e, 0x25, 0x04, 0xa0, \
  0x06, 0x80, 0xd8, 0x1b, 0x03, 0x60, \
  0x8a, 0x24, 0x34, 0x86, 0x90, 0xc0

#define kMaskRandom43_7 \
  0xc6, 0x11, 0x96, 0x32, 0x46, 0x40, \
  0x33, 0x04, 0xc8, 0x99, 0x33, 0x20, \
  0x18, 0x67, 0x44, 0x68, 0x9d, 0x00, \
  0x45, 0x42, 0xd4, 0x5a, 0x0b, 0x40, \
  0x12, 0xd4, 0x2a, 0x95, 0x50, 0xa0, \
  0xb4, 0x28, 0x35, 0x16, 0xa0, 0xc0, \
  0x29, 0x92, 0x1b, 0x0d, 0x41, 0x60

#define kMaskRandom43_8 \
  0x07, 0x0a, 0x71, 0x44, 0x29, 0xc0, \
  0x49, 0xa8, 0x29, 0x0f, 0xa0, 0x20, \
  0xb0, 0x7a, 0x07, 0x48, 0xe8, 0x00, \
  0x24, 0xc5, 0xc0, 0xb8, 0x17, 0x00, \
  0x52, 0x80, 0xec, 0x1d, 0x02, 0xa0, \
  0xc6, 0x31, 0x82, 0x30, 0xc7, 0x40, \
  0x31, 0x94, 0x1a, 0x83, 0x50, 0x60, \
  0x18, 0xc7, 0x08, 0xe1, 0x1c, 0x20

#define kMaskRandom43_9 \
  0x4e, 0x11, 0x12, 0x22, 0x46, 0x40, \
  0x62, 0x1a, 0x09, 0x41, 0x68, 0x60, \
  0x80, 0xe9, 0x41, 0x28, 0xa5, 0x00, \
  0xa1, 0x50, 0x52, 0xc8, 0x51, 0x00, \
  0x53, 0x00, 0x68, 0x1d, 0x01, 0xa0, \
  0xa4, 0x24, 0x36, 0x06, 0x80, 0xc0, \
  0x16, 0xa0, 0x8d, 0x11, 0x82, 0x20, \
  0x58, 0x45, 0x20, 0xa4, 0x16, 0x80, \
  0x29, 0x86, 0x84, 0xd0, 0x1c, 0x00

#define kMaskRandom44_1 \
  0xff, 0xff, 0xff, 0xff, 0xff, 0xf0

#define kMaskRandom44_10 \
  0xc0, 0x38, 0x8b, 0x00, 0xe2, 0x20, \
  0x30, 0x0e, 0x28, 0xc0, 0x38, 0xa0, \
  0xe8, 0x07, 0x03, 0xa0, 0x1c, 0x00, \
  0x85, 0x08, 0xaa, 0x14, 0x22, 0xa0, \
  0xd0, 0x92, 0x13, 0x42, 0x48, 0x40, \
  0x86, 0x50, 0x4a, 0x19, 0x41, 0x20, \
  0x4a, 0x68, 0x0d, 0x29, 0xa0, 0x30, \
  0x01, 0xa0, 0x74, 0x06, 0x81, 0xd0, \
  0x4c, 0x81, 0x91, 0x32, 0x06, 0x40, \
  0x62, 0x24, 0x05, 0x88, 0x90, 0x10

#define kMaskRandom44_11 \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x33, 0x26, 0x64, 0xcc, 0x99, 0x90, \
  0x99, 0x13, 0x22, 0x64, 0x4c, 0x80, \
  0x05, 0x80, 0xb0, 0x16, 0x02, 0xc0, \
  0x80, 0xb0, 0x16, 0x02, 0xc0, 0x50, \
  0x84, 0x50, 0x8a, 0x11, 0x42, 0x20, \
  0x40, 0x68, 0x0d, 0x01, 0xa0, 0x30, \
  0x0a, 0x81, 0x50, 0x2a, 0x05, 0x40, \
  0x68, 0x0d, 0x01, 0xa0, 0x34, 0x00, \
  0x10, 0x22, 0x04, 0x40, 0x88, 0x10, \
  0x30, 0x46, 0x08, 0xc1, 0x18, 0x20

#define kMaskRandom44_12 \
  0x64, 0x4c, 0x89, 0x91, 0x32, 0x20, \
  0x51, 0x4a, 0x29, 0x45, 0x28, 0xa0, \
  0x0c, 0xa1, 0x94, 0x32, 0x86, 0x50, \
  0xa1, 0x34, 0x26, 0x84, 0xd0, 0x90, \
  0x12, 0xa2, 0x54, 0x4a, 0x89, 0x50, \
  0x8a, 0x51, 0x4a, 0x29, 0x45, 0x20, \
  0x86, 0x90, 0xd2, 0x1a, 0x43, 0x40, \
  0x23, 0x24, 0x64, 0x8c, 0x91, 0x90, \
  0x16, 0x22, 0xc4, 0x58, 0x8b, 0x10, \
  0x4c, 0x29, 0x85, 0x30, 0xa6, 0x10, \
  0x41, 0xc8, 0x39, 0x07, 0x20, 0xe0, \
  0xf4, 0x18, 0x9f, 0xd0, 0x62, 0x70

#define kMaskRandom44_13 \
  0x64, 0x4c, 0x89, 0x91, 0x32, 0x20, \
  0x51, 0x4a, 0x29, 0x45, 0x28, 0xa0, \
  0x0c, 0xa1, 0x94, 0x32, 0x86, 0x50, \
  0xa1, 0x34, 0x26, 0x84, 0xd0, 0x90, \
  0x12, 0xa2, 0x54, 0x4a, 0x89, 0x50, \
  0x8a, 0x51, 0x4a, 0x29, 0x45, 0x20, \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x33, 0x26, 0x64, 0xcc, 0x99, 0x90, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0xc0, \
  0xa4, 0x34, 0x86, 0x90, 0xd2, 0x10, \
  0x50, 0xaa, 0x15, 0x42, 0xa8, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x42, 0x60, \
  0x09, 0x61, 0x2c, 0x25, 0x84, 0xb0

#define kMaskRandom44_14 \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x33, 0x26, 0x64, 0xcc, 0x99, 0x90, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0xc0, \
  0xa4, 0x34, 0x86, 0x90, 0xd2, 0x10, \
  0x50, 0xaa, 0x15, 0x42, 0xa8, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x42, 0x60, \
  0x09, 0x61, 0x2c, 0x25, 0x84, 0xb0, \
  0x64, 0x4c, 0x89, 0x91, 0x32, 0x20, \
  0x51, 0x4a, 0x29, 0x45, 0x28, 0xa0, \
  0x0c, 0xa1, 0x94, 0x32, 0x86, 0x50, \
  0xa1, 0x34, 0x26, 0x84, 0xd0, 0x90, \
  0x12, 0xa2, 0x54, 0x4a, 0x89, 0x50, \
  0x8a, 0x51, 0x4a, 0x29, 0x45, 0x20, \
  0xc6, 0xca, 0xeb, 0x1b, 0x2b, 0xa0

#define kMaskRandom44_15 \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x33, 0x26, 0x64, 0xcc, 0x99, 0x90, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0xc0, \
  0xa4, 0x34, 0x86, 0x90, 0xd2, 0x10, \
  0x50, 0xaa, 0x15, 0x42, 0xa8, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x42, 0x60, \
  0x09, 0x61, 0x2c, 0x25, 0x84, 0xb0, \
  0x0c, 0x81, 0x90, 0x32, 0x06, 0x40, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x30, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x05, 0x40, 0xa8, 0x15, 0x02, 0xa0, \
  0x43, 0x08, 0x61, 0x0c, 0x21, 0x80, \
  0x1a, 0x03, 0x40, 0x68, 0x0d, 0x00, \
  0x60, 0x2c, 0x05, 0x80, 0xb0, 0x10, \
  0x14, 0x22, 0x84, 0x50, 0x8a, 0x10

#define kMaskRandom44_16 \
  0x0c, 0x81, 0x90, 0x32, 0x06, 0x40, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x30, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x05, 0x40, 0xa8, 0x15, 0x02, 0xa0, \
  0x43, 0x08, 0x61, 0x0c, 0x21, 0x80, \
  0x1a, 0x03, 0x40, 0x68, 0x0d, 0x00, \
  0x60, 0x2c, 0x05, 0x80, 0xb0, 0x10, \
  0x14, 0x22, 0x84, 0x50, 0x8a, 0x10, \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x33, 0x26, 0x64, 0xcc, 0x99, 0x90, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0xc0, \
  0xa4, 0x34, 0x86, 0x90, 0xd2, 0x10, \
  0x50, 0xaa, 0x15, 0x42, 0xa8, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x42, 0x60, \
  0x09, 0x61, 0x2c, 0x25, 0x84, 0xb0, \
  0x86, 0xc1, 0x46, 0x1b, 0x05, 0x10

#define kMaskRandom44_17 \
  0x0c, 0x81, 0x90, 0x32, 0x06, 0x40, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x30, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x05, 0x40, 0xa8, 0x15, 0x02, 0xa0, \
  0x43, 0x08, 0x61, 0x0c, 0x21, 0x80, \
  0x1a, 0x03, 0x40, 0x68, 0x0d, 0x00, \
  0x60, 0x2c, 0x05, 0x80, 0xb0, 0x10, \
  0x14, 0x22, 0x84, 0x50, 0x8a, 0x10, \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x62, 0x6c, 0x4d, 0x89, 0xb1, 0x30, \
  0x8c, 0x11, 0x82, 0x30, 0x46, 0x00, \
  0x01, 0x60, 0x2c, 0x05, 0x80, 0xb0, \
  0x07, 0x80, 0xf0, 0x1e, 0x03, 0xc0, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x18, 0xa3, 0x14, 0x62, 0x8c, 0x50, \
  0x91, 0x12, 0x22, 0x44, 0x48, 0x80, \
  0x78, 0x0f, 0x01, 0xe0, 0x3c, 0x00

#define kMaskRandom44_18 \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x62, 0x6c, 0x4d, 0x89, 0xb1, 0x30, \
  0x8c, 0x11, 0x82, 0x30, 0x46, 0x00, \
  0x01, 0x60, 0x2c, 0x05, 0x80, 0xb0, \
  0x07, 0x80, 0xf0, 0x1e, 0x03, 0xc0, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x18, 0xa3, 0x14, 0x62, 0x8c, 0x50, \
  0x91, 0x12, 0x22, 0x44, 0x48, 0x80, \
  0x78, 0x0f, 0x01, 0xe0, 0x3c, 0x00, \
  0x0c, 0x81, 0x90, 0x32, 0x06, 0x40, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x30, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x05, 0x40, 0xa8, 0x15, 0x02, 0xa0, \
  0x43, 0x08, 0x61, 0x0c, 0x21, 0x80, \
  0x1a, 0x03, 0x40, 0x68, 0x0d, 0x00, \
  0x60, 0x2c, 0x05, 0x80, 0xb0, 0x10, \
  0x14, 0x22, 0x84, 0x50, 0x8a, 0x10, \
  0xe4, 0xd4, 0x6f, 0x93, 0x51, 0xb0

#define kMaskRandom44_19 \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x62, 0x6c, 0x4d, 0x89, 0xb1, 0x30, \
  0x8c, 0x11, 0x82, 0x30, 0x46, 0x00, \
  0x01, 0x60, 0x2c, 0x05, 0x80, 0xb0, \
  0x07, 0x80, 0xf0, 0x1e, 0x03, 0xc0, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x18, 0xa3, 0x14, 0x62, 0x8c, 0x50, \
  0x91, 0x12, 0x22, 0x44, 0x48, 0x80, \
  0x78, 0x0f, 0x01, 0xe0, 0x3c, 0x00, \
  0x64, 0x4c, 0x89, 0x91, 0x32, 0x20, \
  0x51, 0x4a, 0x29, 0x45, 0x28, 0xa0, \
  0xa9, 0x15, 0x22, 0xa4, 0x54, 0x80, \
  0x04, 0xc0, 0x98, 0x13, 0x02, 0x60, \
  0xd0, 0x1a, 0x03, 0x40, 0x68, 0x00, \
  0x82, 0x50, 0x4a, 0x09, 0x41, 0x20, \
  0x21, 0x24, 0x24, 0x84, 0x90, 0x90, \
  0x0c, 0x21, 0x84, 0x30, 0x86, 0x10, \
  0x4a, 0x09, 0x41, 0x28, 0x25, 0x00, \
  0x12, 0xa2, 0x54, 0x4a, 0x89, 0x50

#define kMaskRandom44_2 \
  0xec, 0xdd, 0x9b, 0xb3, 0x76, 0x60, \
  0x9b, 0xb3, 0x76, 0x6e, 0xcd, 0xd0

#define kMaskRandom44_20 \
  0x64, 0x4c, 0x89, 0x91, 0x32, 0x20, \
  0x51, 0x4a, 0x29, 0x45, 0x28, 0xa0, \
  0xa9, 0x15, 0x22, 0xa4, 0x54, 0x80, \
  0x04, 0xc0, 0x98, 0x13, 0x02, 0x60, \
  0xd0, 0x1a, 0x03, 0x40, 0x68, 0x00, \
  0x82, 0x50, 0x4a, 0x09, 0x41, 0x20, \
  0x21, 0x24, 0x24, 0x84, 0x90, 0x90, \
  0x0c, 0x21, 0x84, 0x30, 0x86, 0x10, \
  0x4a, 0x09, 0x41, 0x28, 0x25, 0x00, \
  0x12, 0xa2, 0x54, 0x4a, 0x89, 0x50, \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x62, 0x6c, 0x4d, 0x89, 0xb1, 0x30, \
  0x8c, 0x11, 0x82, 0x30, 0x46, 0x00, \
  0x01, 0x60, 0x2c, 0x05, 0x80, 0xb0, \
  0x07, 0x80, 0xf0, 0x1e, 0x03, 0xc0, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x18, 0xa3, 0x14, 0x62, 0x8c, 0x50, \
  0x91, 0x12, 0x22, 0x44, 0x48, 0x80, \
  0x78, 0x0f, 0x01, 0xe0, 0x3c, 0x00, \
  0x3b, 0x48, 0xc4, 0xed, 0x23, 0x10

#define kMaskRandom44_21 \
  0x64, 0x4c, 0x89, 0x91, 0x32, 0x20, \
  0x51, 0x4a, 0x29, 0x45, 0x28, 0xa0, \
  0xa9, 0x15, 0x22, 0xa4, 0x54, 0x80, \
  0x04, 0xc0, 0x98, 0x13, 0x02, 0x60, \
  0xd0, 0x1a, 0x03, 0x40, 0x68, 0x00, \
  0x82, 0x50, 0x4a, 0x09, 0x41, 0x20, \
  0x21, 0x24, 0x24, 0x84, 0x90, 0x90, \
  0x0c, 0x21, 0x84, 0x30, 0x86, 0x10, \
  0x4a, 0x09, 0x41, 0x28, 0x25, 0x00, \
  0x12, 0xa2, 0x54, 0x4a, 0x89, 0x50, \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x33, 0x26, 0x64, 0xcc, 0x99, 0x90, \
  0x99, 0x13, 0x22, 0x64, 0x4c, 0x80, \
  0x05, 0x80, 0xb0, 0x16, 0x02, 0xc0, \
  0x80, 0xb0, 0x16, 0x02, 0xc0, 0x50, \
  0x84, 0x50, 0x8a, 0x11, 0x42, 0x20, \
  0x40, 0x68, 0x0d, 0x01, 0xa0, 0x30, \
  0x0a, 0x81, 0x50, 0x2a, 0x05, 0x40, \
  0x68, 0x0d, 0x01, 0xa0, 0x34, 0x00, \
  0x10, 0x22, 0x04, 0x40, 0x88, 0x10, \
  0x30, 0x46, 0x08, 0xc1, 0x18, 0x20

#define kMaskRandom44_22 \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x33, 0x26, 0x64, 0xcc, 0x99, 0x90, \
  0x99, 0x13, 0x22, 0x64, 0x4c, 0x80, \
  0x05, 0x80, 0xb0, 0x16, 0x02, 0xc0, \
  0x80, 0xb0, 0x16, 0x02, 0xc0, 0x50, \
  0x84, 0x50, 0x8a, 0x11, 0x42, 0x20, \
  0x40, 0x68, 0x0d, 0x01, 0xa0, 0x30, \
  0x0a, 0x81, 0x50, 0x2a, 0x05, 0x40, \
  0x68, 0x0d, 0x01, 0xa0, 0x34, 0x00, \
  0x10, 0x22, 0x04, 0x40, 0x88, 0x10, \
  0x30, 0x46, 0x08, 0xc1, 0x18, 0x20, \
  0x64, 0x4c, 0x89, 0x91, 0x32, 0x20, \
  0x51, 0x4a, 0x29, 0x45, 0x28, 0xa0, \
  0xa9, 0x15, 0x22, 0xa4, 0x54, 0x80, \
  0x04, 0xc0, 0x98, 0x13, 0x02, 0x60, \
  0xd0, 0x1a, 0x03, 0x40, 0x68, 0x00, \
  0x82, 0x50, 0x4a, 0x09, 0x41, 0x20, \
  0x21, 0x24, 0x24, 0x84, 0x90, 0x90, \
  0x0c, 0x21, 0x84, 0x30, 0x86, 0x10, \
  0x4a, 0x09, 0x41, 0x28, 0x25, 0x00, \
  0x12, 0xa2, 0x54, 0x4a, 0x89, 0x50, \
  0x9e, 0xce, 0x8a, 0x7b, 0x3a, 0x20

#define kMaskRandom44_23 \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x33, 0x26, 0x64, 0xcc, 0x99, 0x90, \
  0x99, 0x13, 0x22, 0x64, 0x4c, 0x80, \
  0x05, 0x80, 0xb0, 0x16, 0x02, 0xc0, \
  0x80, 0xb0, 0x16, 0x02, 0xc0, 0x50, \
  0x84, 0x50, 0x8a, 0x11, 0x42, 0x20, \
  0x40, 0x68, 0x0d, 0x01, 0xa0, 0x30, \
  0x0a, 0x81, 0x50, 0x2a, 0x05, 0x40, \
  0x68, 0x0d, 0x01, 0xa0, 0x34, 0x00, \
  0x10, 0x22, 0x04, 0x40, 0x88, 0x10, \
  0x30, 0x46, 0x08, 0xc1, 0x18, 0x20, \
  0x64, 0x4c, 0x89, 0x91, 0x32, 0x20, \
  0x51, 0x4a, 0x29, 0x45, 0x28, 0xa0, \
  0x0c, 0xa1, 0x94, 0x32, 0x86, 0x50, \
  0xa1, 0x34, 0x26, 0x84, 0xd0, 0x90, \
  0x12, 0xa2, 0x54, 0x4a, 0x89, 0x50, \
  0x8a, 0x51, 0x4a, 0x29, 0x45, 0x20, \
  0x86, 0x90, 0xd2, 0x1a, 0x43, 0x40, \
  0x23, 0x24, 0x64, 0x8c, 0x91, 0x90, \
  0x16, 0x22, 0xc4, 0x58, 0x8b, 0x10, \
  0x4c, 0x29, 0x85, 0x30, 0xa6, 0x10, \
  0x41, 0xc8, 0x39, 0x07, 0x20, 0xe0, \
  0xf4, 0x18, 0x9f, 0xd0, 0x62, 0x70

#define kMaskRandom44_24 \
  0x64, 0x4c, 0x89, 0x91, 0x32, 0x20, \
  0x51, 0x4a, 0x29, 0x45, 0x28, 0xa0, \
  0x0c, 0xa1, 0x94, 0x32, 0x86, 0x50, \
  0xa1, 0x34, 0x26, 0x84, 0xd0, 0x90, \
  0x12, 0xa2, 0x54, 0x4a, 0x89, 0x50, \
  0x8a, 0x51, 0x4a, 0x29, 0x45, 0x20, \
  0x86, 0x90, 0xd2, 0x1a, 0x43, 0x40, \
  0x23, 0x24, 0x64, 0x8c, 0x91, 0x90, \
  0x16, 0x22, 0xc4, 0x58, 0x8b, 0x10, \
  0x4c, 0x29, 0x85, 0x30, 0xa6, 0x10, \
  0x41, 0xc8, 0x39, 0x07, 0x20, 0xe0, \
  0xf4, 0x18, 0x9f, 0xd0, 0x62, 0x70, \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x33, 0x26, 0x64, 0xcc, 0x99, 0x90, \
  0x99, 0x13, 0x22, 0x64, 0x4c, 0x80, \
  0x05, 0x80, 0xb0, 0x16, 0x02, 0xc0, \
  0x80, 0xb0, 0x16, 0x02, 0xc0, 0x50, \
  0x84, 0x50, 0x8a, 0x11, 0x42, 0x20, \
  0x40, 0x68, 0x0d, 0x01, 0xa0, 0x30, \
  0x0a, 0x81, 0x50, 0x2a, 0x05, 0x40, \
  0x68, 0x0d, 0x01, 0xa0, 0x34, 0x00, \
  0x10, 0x22, 0x04, 0x40, 0x88, 0x10, \
  0x30, 0x46, 0x08, 0xc1, 0x18, 0x20, \
  0x15, 0x0f, 0x44, 0x6d, 0x9d, 0xa0

#define kMaskRandom44_25 \
  0x64, 0x4c, 0x89, 0x91, 0x32, 0x20, \
  0x51, 0x4a, 0x29, 0x45, 0x28, 0xa0, \
  0x0c, 0xa1, 0x94, 0x32, 0x86, 0x50, \
  0xa1, 0x34, 0x26, 0x84, 0xd0, 0x90, \
  0x12, 0xa2, 0x54, 0x4a, 0x89, 0x50, \
  0x8a, 0x51, 0x4a, 0x29, 0x45, 0x20, \
  0x86, 0x90, 0xd2, 0x1a, 0x43, 0x40, \
  0x23, 0x24, 0x64, 0x8c, 0x91, 0x90, \
  0x16, 0x22, 0xc4, 0x58, 0x8b, 0x10, \
  0x4c, 0x29, 0x85, 0x30, 0xa6, 0x10, \
  0x41, 0xc8, 0x39, 0x07, 0x20, 0xe0, \
  0xf4, 0x18, 0x9f, 0xd0, 0x62, 0x70, \
  0x64, 0x4c, 0x89, 0x91, 0x32, 0x20, \
  0x51, 0x4a, 0x29, 0x45, 0x28, 0xa0, \
  0x0c, 0xa1, 0x94, 0x32, 0x86, 0x50, \
  0xa1, 0x34, 0x26, 0x84, 0xd0, 0x90, \
  0x12, 0xa2, 0x54, 0x4a, 0x89, 0x50, \
  0x8a, 0x51, 0x4a, 0x29, 0x45, 0x20, \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x33, 0x26, 0x64, 0xcc, 0x99, 0x90, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0xc0, \
  0xa4, 0x34, 0x86, 0x90, 0xd2, 0x10, \
  0x50, 0xaa, 0x15, 0x42, 0xa8, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x42, 0x60, \
  0x09, 0x61, 0x2c, 0x25, 0x84, 0xb0

#define kMaskRandom44_26 \
  0x64, 0x4c, 0x89, 0x91, 0x32, 0x20, \
  0x51, 0x4a, 0x29, 0x45, 0x28, 0xa0, \
  0x0c, 0xa1, 0x94, 0x32, 0x86, 0x50, \
  0xa1, 0x34, 0x26, 0x84, 0xd0, 0x90, \
  0x12, 0xa2, 0x54, 0x4a, 0x89, 0x50, \
  0x8a, 0x51, 0x4a, 0x29, 0x45, 0x20, \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x33, 0x26, 0x64, 0xcc, 0x99, 0x90, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0xc0, \
  0xa4, 0x34, 0x86, 0x90, 0xd2, 0x10, \
  0x50, 0xaa, 0x15, 0x42, 0xa8, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x42, 0x60, \
  0x09, 0x61, 0x2c, 0x25, 0x84, 0xb0, \
  0x64, 0x4c, 0x89, 0x91, 0x32, 0x20, \
  0x51, 0x4a, 0x29, 0x45, 0x28, 0xa0, \
  0x0c, 0xa1, 0x94, 0x32, 0x86, 0x50, \
  0xa1, 0x34, 0x26, 0x84, 0xd0, 0x90, \
  0x12, 0xa2, 0x54, 0x4a, 0x89, 0x50, \
  0x8a, 0x51, 0x4a, 0x29, 0x45, 0x20, \
  0x86, 0x90, 0xd2, 0x1a, 0x43, 0x40, \
  0x23, 0x24, 0x64, 0x8c, 0x91, 0x90, \
  0x16, 0x22, 0xc4, 0x58, 0x8b, 0x10, \
  0x4c, 0x29, 0x85, 0x30, 0xa6, 0x10, \
  0x41, 0xc8, 0x39, 0x07, 0x20, 0xe0, \
  0xf4, 0x18, 0x9f, 0xd0, 0x62, 0x70, \
  0x02, 0xcb, 0x64, 0xb8, 0x55, 0x80

#define kMaskRandom44_27 \
  0x64, 0x4c, 0x89, 0x91, 0x32, 0x20, \
  0x51, 0x4a, 0x29, 0x45, 0x28, 0xa0, \
  0x0c, 0xa1, 0x94, 0x32, 0x86, 0x50, \
  0xa1, 0x34, 0x26, 0x84, 0xd0, 0x90, \
  0x12, 0xa2, 0x54, 0x4a, 0x89, 0x50, \
  0x8a, 0x51, 0x4a, 0x29, 0x45, 0x20, \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x33, 0x26, 0x64, 0xcc, 0x99, 0x90, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0xc0, \
  0xa4, 0x34, 0x86, 0x90, 0xd2, 0x10, \
  0x50, 0xaa, 0x15, 0x42, 0xa8, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x42, 0x60, \
  0x09, 0x61, 0x2c, 0x25, 0x84, 0xb0, \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x33, 0x26, 0x64, 0xcc, 0x99, 0x90, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0xc0, \
  0xa4, 0x34, 0x86, 0x90, 0xd2, 0x10, \
  0x50, 0xaa, 0x15, 0x42, 0xa8, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x42, 0x60, \
  0x09, 0x61, 0x2c, 0x25, 0x84, 0xb0, \
  0x64, 0x4c, 0x89, 0x91, 0x32, 0x20, \
  0x51, 0x4a, 0x29, 0x45, 0x28, 0xa0, \
  0x0c, 0xa1, 0x94, 0x32, 0x86, 0x50, \
  0xa1, 0x34, 0x26, 0x84, 0xd0, 0x90, \
  0x12, 0xa2, 0x54, 0x4a, 0x89, 0x50, \
  0x8a, 0x51, 0x4a, 0x29, 0x45, 0x20, \
  0xc6, 0xca, 0xeb, 0x1b, 0x2b, 0xa0

#define kMaskRandom44_28 \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x33, 0x26, 0x64, 0xcc, 0x99, 0x90, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0xc0, \
  0xa4, 0x34, 0x86, 0x90, 0xd2, 0x10, \
  0x50, 0xaa, 0x15, 0x42, 0xa8, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x42, 0x60, \
  0x09, 0x61, 0x2c, 0x25, 0x84, 0xb0, \
  0x64, 0x4c, 0x89, 0x91, 0x32, 0x20, \
  0x51, 0x4a, 0x29, 0x45, 0x28, 0xa0, \
  0x0c, 0xa1, 0x94, 0x32, 0x86, 0x50, \
  0xa1, 0x34, 0x26, 0x84, 0xd0, 0x90, \
  0x12, 0xa2, 0x54, 0x4a, 0x89, 0x50, \
  0x8a, 0x51, 0x4a, 0x29, 0x45, 0x20, \
  0xc6, 0xca, 0xeb, 0x1b, 0x2b, 0xa0, \
  0x64, 0x4c, 0x89, 0x91, 0x32, 0x20, \
  0x51, 0x4a, 0x29, 0x45, 0x28, 0xa0, \
  0x0c, 0xa1, 0x94, 0x32, 0x86, 0x50, \
  0xa1, 0x34, 0x26, 0x84, 0xd0, 0x90, \
  0x12, 0xa2, 0x54, 0x4a, 0x89, 0x50, \
  0x8a, 0x51, 0x4a, 0x29, 0x45, 0x20, \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x33, 0x26, 0x64, 0xcc, 0x99, 0x90, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0xc0, \
  0xa4, 0x34, 0x86, 0x90, 0xd2, 0x10, \
  0x50, 0xaa, 0x15, 0x42, 0xa8, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x42, 0x60, \
  0x09, 0x61, 0x2c, 0x25, 0x84, 0xb0, \
  0x66, 0x26, 0x6c, 0x91, 0xc7, 0x20

#define kMaskRandom44_29 \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x33, 0x26, 0x64, 0xcc, 0x99, 0x90, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0xc0, \
  0xa4, 0x34, 0x86, 0x90, 0xd2, 0x10, \
  0x50, 0xaa, 0x15, 0x42, 0xa8, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x42, 0x60, \
  0x09, 0x61, 0x2c, 0x25, 0x84, 0xb0, \
  0x64, 0x4c, 0x89, 0x91, 0x32, 0x20, \
  0x51, 0x4a, 0x29, 0x45, 0x28, 0xa0, \
  0x0c, 0xa1, 0x94, 0x32, 0x86, 0x50, \
  0xa1, 0x34, 0x26, 0x84, 0xd0, 0x90, \
  0x12, 0xa2, 0x54, 0x4a, 0x89, 0x50, \
  0x8a, 0x51, 0x4a, 0x29, 0x45, 0x20, \
  0xc6, 0xca, 0xeb, 0x1b, 0x2b, 0xa0, \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x33, 0x26, 0x64, 0xcc, 0x99, 0x90, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0xc0, \
  0xa4, 0x34, 0x86, 0x90, 0xd2, 0x10, \
  0x50, 0xaa, 0x15, 0x42, 0xa8, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x42, 0x60, \
  0x09, 0x61, 0x2c, 0x25, 0x84, 0xb0, \
  0x0c, 0x81, 0x90, 0x32, 0x06, 0x40, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x30, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x05, 0x40, 0xa8, 0x15, 0x02, 0xa0, \
  0x43, 0x08, 0x61, 0x0c, 0x21, 0x80, \
  0x1a, 0x03, 0x40, 0x68, 0x0d, 0x00, \
  0x60, 0x2c, 0x05, 0x80, 0xb0, 0x10, \
  0x14, 0x22, 0x84, 0x50, 0x8a, 0x10

#define kMaskRandom44_3 \
  0xca, 0xd9, 0x5b, 0x2b, 0x65, 0x60, \
  0xf1, 0x5e, 0x2b, 0xc5, 0x78, 0xa0, \
  0xb6, 0x35, 0xc6, 0xd8, 0xd7, 0x10

#define kMaskRandom44_30 \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x33, 0x26, 0x64, 0xcc, 0x99, 0x90, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0xc0, \
  0xa4, 0x34, 0x86, 0x90, 0xd2, 0x10, \
  0x50, 0xaa, 0x15, 0x42, 0xa8, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x42, 0x60, \
  0x09, 0x61, 0x2c, 0x25, 0x84, 0xb0, \
  0x0c, 0x81, 0x90, 0x32, 0x06, 0x40, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x30, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x05, 0x40, 0xa8, 0x15, 0x02, 0xa0, \
  0x43, 0x08, 0x61, 0x0c, 0x21, 0x80, \
  0x1a, 0x03, 0x40, 0x68, 0x0d, 0x00, \
  0x60, 0x2c, 0x05, 0x80, 0xb0, 0x10, \
  0x14, 0x22, 0x84, 0x50, 0x8a, 0x10, \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x33, 0x26, 0x64, 0xcc, 0x99, 0x90, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0xc0, \
  0xa4, 0x34, 0x86, 0x90, 0xd2, 0x10, \
  0x50, 0xaa, 0x15, 0x42, 0xa8, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x42, 0x60, \
  0x09, 0x61, 0x2c, 0x25, 0x84, 0xb0, \
  0x64, 0x4c, 0x89, 0x91, 0x32, 0x20, \
  0x51, 0x4a, 0x29, 0x45, 0x28, 0xa0, \
  0x0c, 0xa1, 0x94, 0x32, 0x86, 0x50, \
  0xa1, 0x34, 0x26, 0x84, 0xd0, 0x90, \
  0x12, 0xa2, 0x54, 0x4a, 0x89, 0x50, \
  0x8a, 0x51, 0x4a, 0x29, 0x45, 0x20, \
  0xc6, 0xca, 0xeb, 0x1b, 0x2b, 0xa0, \
  0x60, 0xf4, 0x75, 0x84, 0x90, 0xc0

#define kMaskRandom44_31 \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x33, 0x26, 0x64, 0xcc, 0x99, 0x90, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0xc0, \
  0xa4, 0x34, 0x86, 0x90, 0xd2, 0x10, \
  0x50, 0xaa, 0x15, 0x42, 0xa8, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x42, 0x60, \
  0x09, 0x61, 0x2c, 0x25, 0x84, 0xb0, \
  0x0c, 0x81, 0x90, 0x32, 0x06, 0x40, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x30, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x05, 0x40, 0xa8, 0x15, 0x02, 0xa0, \
  0x43, 0x08, 0x61, 0x0c, 0x21, 0x80, \
  0x1a, 0x03, 0x40, 0x68, 0x0d, 0x00, \
  0x60, 0x2c, 0x05, 0x80, 0xb0, 0x10, \
  0x14, 0x22, 0x84, 0x50, 0x8a, 0x10, \
  0x0c, 0x81, 0x90, 0x32, 0x06, 0x40, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x30, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x05, 0x40, 0xa8, 0x15, 0x02, 0xa0, \
  0x43, 0x08, 0x61, 0x0c, 0x21, 0x80, \
  0x1a, 0x03, 0x40, 0x68, 0x0d, 0x00, \
  0x60, 0x2c, 0x05, 0x80, 0xb0, 0x10, \
  0x14, 0x22, 0x84, 0x50, 0x8a, 0x10, \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x33, 0x26, 0x64, 0xcc, 0x99, 0x90, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0xc0, \
  0xa4, 0x34, 0x86, 0x90, 0xd2, 0x10, \
  0x50, 0xaa, 0x15, 0x42, 0xa8, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x42, 0x60, \
  0x09, 0x61, 0x2c, 0x25, 0x84, 0xb0, \
  0x86, 0xc1, 0x46, 0x1b, 0x05, 0x10

#define kMaskRandom44_32 \
  0x0c, 0x81, 0x90, 0x32, 0x06, 0x40, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x30, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x05, 0x40, 0xa8, 0x15, 0x02, 0xa0, \
  0x43, 0x08, 0x61, 0x0c, 0x21, 0x80, \
  0x1a, 0x03, 0x40, 0x68, 0x0d, 0x00, \
  0x60, 0x2c, 0x05, 0x80, 0xb0, 0x10, \
  0x14, 0x22, 0x84, 0x50, 0x8a, 0x10, \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x33, 0x26, 0x64, 0xcc, 0x99, 0x90, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0xc0, \
  0xa4, 0x34, 0x86, 0x90, 0xd2, 0x10, \
  0x50, 0xaa, 0x15, 0x42, 0xa8, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x42, 0x60, \
  0x09, 0x61, 0x2c, 0x25, 0x84, 0xb0, \
  0x86, 0xc1, 0x46, 0x1b, 0x05, 0x10, \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x33, 0x26, 0x64, 0xcc, 0x99, 0x90, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0xc0, \
  0xa4, 0x34, 0x86, 0x90, 0xd2, 0x10, \
  0x50, 0xaa, 0x15, 0x42, 0xa8, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x42, 0x60, \
  0x09, 0x61, 0x2c, 0x25, 0x84, 0xb0, \
  0x0c, 0x81, 0x90, 0x32, 0x06, 0x40, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x30, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x05, 0x40, 0xa8, 0x15, 0x02, 0xa0, \
  0x43, 0x08, 0x61, 0x0c, 0x21, 0x80, \
  0x1a, 0x03, 0x40, 0x68, 0x0d, 0x00, \
  0x60, 0x2c, 0x05, 0x80, 0xb0, 0x10, \
  0x14, 0x22, 0x84, 0x50, 0x8a, 0x10, \
  0x3e, 0x39, 0x86, 0x5c, 0xd9, 0xd0

#define kMaskRandom44_33 \
  0x0c, 0x81, 0x90, 0x32, 0x06, 0x40, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x30, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x05, 0x40, 0xa8, 0x15, 0x02, 0xa0, \
  0x43, 0x08, 0x61, 0x0c, 0x21, 0x80, \
  0x1a, 0x03, 0x40, 0x68, 0x0d, 0x00, \
  0x60, 0x2c, 0x05, 0x80, 0xb0, 0x10, \
  0x14, 0x22, 0x84, 0x50, 0x8a, 0x10, \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x33, 0x26, 0x64, 0xcc, 0x99, 0x90, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0xc0, \
  0xa4, 0x34, 0x86, 0x90, 0xd2, 0x10, \
  0x50, 0xaa, 0x15, 0x42, 0xa8, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x42, 0x60, \
  0x09, 0x61, 0x2c, 0x25, 0x84, 0xb0, \
  0x86, 0xc1, 0x46, 0x1b, 0x05, 0x10, \
  0x0c, 0x81, 0x90, 0x32, 0x06, 0x40, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x30, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x05, 0x40, 0xa8, 0x15, 0x02, 0xa0, \
  0x43, 0x08, 0x61, 0x0c, 0x21, 0x80, \
  0x1a, 0x03, 0x40, 0x68, 0x0d, 0x00, \
  0x60, 0x2c, 0x05, 0x80, 0xb0, 0x10, \
  0x14, 0x22, 0x84, 0x50, 0x8a, 0x10, \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x62, 0x6c, 0x4d, 0x89, 0xb1, 0x30, \
  0x8c, 0x11, 0x82, 0x30, 0x46, 0x00, \
  0x01, 0x60, 0x2c, 0x05, 0x80, 0xb0, \
  0x07, 0x80, 0xf0, 0x1e, 0x03, 0xc0, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x18, 0xa3, 0x14, 0x62, 0x8c, 0x50, \
  0x91, 0x12, 0x22, 0x44, 0x48, 0x80, \
  0x78, 0x0f, 0x01, 0xe0, 0x3c, 0x00

#define kMaskRandom44_34 \
  0x0c, 0x81, 0x90, 0x32, 0x06, 0x40, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x30, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x05, 0x40, 0xa8, 0x15, 0x02, 0xa0, \
  0x43, 0x08, 0x61, 0x0c, 0x21, 0x80, \
  0x1a, 0x03, 0x40, 0x68, 0x0d, 0x00, \
  0x60, 0x2c, 0x05, 0x80, 0xb0, 0x10, \
  0x14, 0x22, 0x84, 0x50, 0x8a, 0x10, \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x62, 0x6c, 0x4d, 0x89, 0xb1, 0x30, \
  0x8c, 0x11, 0x82, 0x30, 0x46, 0x00, \
  0x01, 0x60, 0x2c, 0x05, 0x80, 0xb0, \
  0x07, 0x80, 0xf0, 0x1e, 0x03, 0xc0, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x18, 0xa3, 0x14, 0x62, 0x8c, 0x50, \
  0x91, 0x12, 0x22, 0x44, 0x48, 0x80, \
  0x78, 0x0f, 0x01, 0xe0, 0x3c, 0x00, \
  0x0c, 0x81, 0x90, 0x32, 0x06, 0x40, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x30, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x05, 0x40, 0xa8, 0x15, 0x02, 0xa0, \
  0x43, 0x08, 0x61, 0x0c, 0x21, 0x80, \
  0x1a, 0x03, 0x40, 0x68, 0x0d, 0x00, \
  0x60, 0x2c, 0x05, 0x80, 0xb0, 0x10, \
  0x14, 0x22, 0x84, 0x50, 0x8a, 0x10, \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x33, 0x26, 0x64, 0xcc, 0x99, 0x90, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0xc0, \
  0xa4, 0x34, 0x86, 0x90, 0xd2, 0x10, \
  0x50, 0xaa, 0x15, 0x42, 0xa8, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x42, 0x60, \
  0x09, 0x61, 0x2c, 0x25, 0x84, 0xb0, \
  0x86, 0xc1, 0x46, 0x1b, 0x05, 0x10, \
  0xb5, 0xc7, 0xe8, 0x0c, 0xb9, 0x90

#define kMaskRandom44_35 \
  0x0c, 0x81, 0x90, 0x32, 0x06, 0x40, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x30, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x05, 0x40, 0xa8, 0x15, 0x02, 0xa0, \
  0x43, 0x08, 0x61, 0x0c, 0x21, 0x80, \
  0x1a, 0x03, 0x40, 0x68, 0x0d, 0x00, \
  0x60, 0x2c, 0x05, 0x80, 0xb0, 0x10, \
  0x14, 0x22, 0x84, 0x50, 0x8a, 0x10, \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x62, 0x6c, 0x4d, 0x89, 0xb1, 0x30, \
  0x8c, 0x11, 0x82, 0x30, 0x46, 0x00, \
  0x01, 0x60, 0x2c, 0x05, 0x80, 0xb0, \
  0x07, 0x80, 0xf0, 0x1e, 0x03, 0xc0, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x18, 0xa3, 0x14, 0x62, 0x8c, 0x50, \
  0x91, 0x12, 0x22, 0x44, 0x48, 0x80, \
  0x78, 0x0f, 0x01, 0xe0, 0x3c, 0x00, \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x62, 0x6c, 0x4d, 0x89, 0xb1, 0x30, \
  0x8c, 0x11, 0x82, 0x30, 0x46, 0x00, \
  0x01, 0x60, 0x2c, 0x05, 0x80, 0xb0, \
  0x07, 0x80, 0xf0, 0x1e, 0x03, 0xc0, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x18, 0xa3, 0x14, 0x62, 0x8c, 0x50, \
  0x91, 0x12, 0x22, 0x44, 0x48, 0x80, \
  0x78, 0x0f, 0x01, 0xe0, 0x3c, 0x00, \
  0x0c, 0x81, 0x90, 0x32, 0x06, 0x40, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x30, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x05, 0x40, 0xa8, 0x15, 0x02, 0xa0, \
  0x43, 0x08, 0x61, 0x0c, 0x21, 0x80, \
  0x1a, 0x03, 0x40, 0x68, 0x0d, 0x00, \
  0x60, 0x2c, 0x05, 0x80, 0xb0, 0x10, \
  0x14, 0x22, 0x84, 0x50, 0x8a, 0x10, \
  0xe4, 0xd4, 0x6f, 0x93, 0x51, 0xb0

#define kMaskRandom44_36 \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x62, 0x6c, 0x4d, 0x89, 0xb1, 0x30, \
  0x8c, 0x11, 0x82, 0x30, 0x46, 0x00, \
  0x01, 0x60, 0x2c, 0x05, 0x80, 0xb0, \
  0x07, 0x80, 0xf0, 0x1e, 0x03, 0xc0, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x18, 0xa3, 0x14, 0x62, 0x8c, 0x50, \
  0x91, 0x12, 0x22, 0x44, 0x48, 0x80, \
  0x78, 0x0f, 0x01, 0xe0, 0x3c, 0x00, \
  0x0c, 0x81, 0x90, 0x32, 0x06, 0x40, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x30, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x05, 0x40, 0xa8, 0x15, 0x02, 0xa0, \
  0x43, 0x08, 0x61, 0x0c, 0x21, 0x80, \
  0x1a, 0x03, 0x40, 0x68, 0x0d, 0x00, \
  0x60, 0x2c, 0x05, 0x80, 0xb0, 0x10, \
  0x14, 0x22, 0x84, 0x50, 0x8a, 0x10, \
  0xe4, 0xd4, 0x6f, 0x93, 0x51, 0xb0, \
  0x0c, 0x81, 0x90, 0x32, 0x06, 0x40, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x30, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x05, 0x40, 0xa8, 0x15, 0x02, 0xa0, \
  0x43, 0x08, 0x61, 0x0c, 0x21, 0x80, \
  0x1a, 0x03, 0x40, 0x68, 0x0d, 0x00, \
  0x60, 0x2c, 0x05, 0x80, 0xb0, 0x10, \
  0x14, 0x22, 0x84, 0x50, 0x8a, 0x10, \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x62, 0x6c, 0x4d, 0x89, 0xb1, 0x30, \
  0x8c, 0x11, 0x82, 0x30, 0x46, 0x00, \
  0x01, 0x60, 0x2c, 0x05, 0x80, 0xb0, \
  0x07, 0x80, 0xf0, 0x1e, 0x03, 0xc0, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x18, 0xa3, 0x14, 0x62, 0x8c, 0x50, \
  0x91, 0x12, 0x22, 0x44, 0x48, 0x80, \
  0x78, 0x0f, 0x01, 0xe0, 0x3c, 0x00, \
  0xa6, 0x92, 0x01, 0x65, 0x91, 0x20

#define kMaskRandom44_37 \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x62, 0x6c, 0x4d, 0x89, 0xb1, 0x30, \
  0x8c, 0x11, 0x82, 0x30, 0x46, 0x00, \
  0x01, 0x60, 0x2c, 0x05, 0x80, 0xb0, \
  0x07, 0x80, 0xf0, 0x1e, 0x03, 0xc0, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x18, 0xa3, 0x14, 0x62, 0x8c, 0x50, \
  0x91, 0x12, 0x22, 0x44, 0x48, 0x80, \
  0x78, 0x0f, 0x01, 0xe0, 0x3c, 0x00, \
  0x0c, 0x81, 0x90, 0x32, 0x06, 0x40, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x30, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x05, 0x40, 0xa8, 0x15, 0x02, 0xa0, \
  0x43, 0x08, 0x61, 0x0c, 0x21, 0x80, \
  0x1a, 0x03, 0x40, 0x68, 0x0d, 0x00, \
  0x60, 0x2c, 0x05, 0x80, 0xb0, 0x10, \
  0x14, 0x22, 0x84, 0x50, 0x8a, 0x10, \
  0xe4, 0xd4, 0x6f, 0x93, 0x51, 0xb0, \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x62, 0x6c, 0x4d, 0x89, 0xb1, 0x30, \
  0x8c, 0x11, 0x82, 0x30, 0x46, 0x00, \
  0x01, 0x60, 0x2c, 0x05, 0x80, 0xb0, \
  0x07, 0x80, 0xf0, 0x1e, 0x03, 0xc0, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x18, 0xa3, 0x14, 0x62, 0x8c, 0x50, \
  0x91, 0x12, 0x22, 0x44, 0x48, 0x80, \
  0x78, 0x0f, 0x01, 0xe0, 0x3c, 0x00, \
  0x64, 0x4c, 0x89, 0x91, 0x32, 0x20, \
  0x51, 0x4a, 0x29, 0x45, 0x28, 0xa0, \
  0xa9, 0x15, 0x22, 0xa4, 0x54, 0x80, \
  0x04, 0xc0, 0x98, 0x13, 0x02, 0x60, \
  0xd0, 0x1a, 0x03, 0x40, 0x68, 0x00, \
  0x82, 0x50, 0x4a, 0x09, 0x41, 0x20, \
  0x21, 0x24, 0x24, 0x84, 0x90, 0x90, \
  0x0c, 0x21, 0x84, 0x30, 0x86, 0x10, \
  0x4a, 0x09, 0x41, 0x28, 0x25, 0x00, \
  0x12, 0xa2, 0x54, 0x4a, 0x89, 0x50

#define kMaskRandom44_38 \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x62, 0x6c, 0x4d, 0x89, 0xb1, 0x30, \
  0x8c, 0x11, 0x82, 0x30, 0x46, 0x00, \
  0x01, 0x60, 0x2c, 0x05, 0x80, 0xb0, \
  0x07, 0x80, 0xf0, 0x1e, 0x03, 0xc0, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x18, 0xa3, 0x14, 0x62, 0x8c, 0x50, \
  0x91, 0x12, 0x22, 0x44, 0x48, 0x80, \
  0x78, 0x0f, 0x01, 0xe0, 0x3c, 0x00, \
  0x64, 0x4c, 0x89, 0x91, 0x32, 0x20, \
  0x51, 0x4a, 0x29, 0x45, 0x28, 0xa0, \
  0xa9, 0x15, 0x22, 0xa4, 0x54, 0x80, \
  0x04, 0xc0, 0x98, 0x13, 0x02, 0x60, \
  0xd0, 0x1a, 0x03, 0x40, 0x68, 0x00, \
  0x82, 0x50, 0x4a, 0x09, 0x41, 0x20, \
  0x21, 0x24, 0x24, 0x84, 0x90, 0x90, \
  0x0c, 0x21, 0x84, 0x30, 0x86, 0x10, \
  0x4a, 0x09, 0x41, 0x28, 0x25, 0x00, \
  0x12, 0xa2, 0x54, 0x4a, 0x89, 0x50, \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x62, 0x6c, 0x4d, 0x89, 0xb1, 0x30, \
  0x8c, 0x11, 0x82, 0x30, 0x46, 0x00, \
  0x01, 0x60, 0x2c, 0x05, 0x80, 0xb0, \
  0x07, 0x80, 0xf0, 0x1e, 0x03, 0xc0, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x18, 0xa3, 0x14, 0x62, 0x8c, 0x50, \
  0x91, 0x12, 0x22, 0x44, 0x48, 0x80, \
  0x78, 0x0f, 0x01, 0xe0, 0x3c, 0x00, \
  0x0c, 0x81, 0x90, 0x32, 0x06, 0x40, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x30, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x05, 0x40, 0xa8, 0x15, 0x02, 0xa0, \
  0x43, 0x08, 0x61, 0x0c, 0x21, 0x80, \
  0x1a, 0x03, 0x40, 0x68, 0x0d, 0x00, \
  0x60, 0x2c, 0x05, 0x80, 0xb0, 0x10, \
  0x14, 0x22, 0x84, 0x50, 0x8a, 0x10, \
  0xe4, 0xd4, 0x6f, 0x93, 0x51, 0xb0, \
  0x43, 0x64, 0xf2, 0xe5, 0x5d, 0x10

#define kMaskRandom44_39 \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x62, 0x6c, 0x4d, 0x89, 0xb1, 0x30, \
  0x8c, 0x11, 0x82, 0x30, 0x46, 0x00, \
  0x01, 0x60, 0x2c, 0x05, 0x80, 0xb0, \
  0x07, 0x80, 0xf0, 0x1e, 0x03, 0xc0, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x18, 0xa3, 0x14, 0x62, 0x8c, 0x50, \
  0x91, 0x12, 0x22, 0x44, 0x48, 0x80, \
  0x78, 0x0f, 0x01, 0xe0, 0x3c, 0x00, \
  0x64, 0x4c, 0x89, 0x91, 0x32, 0x20, \
  0x51, 0x4a, 0x29, 0x45, 0x28, 0xa0, \
  0xa9, 0x15, 0x22, 0xa4, 0x54, 0x80, \
  0x04, 0xc0, 0x98, 0x13, 0x02, 0x60, \
  0xd0, 0x1a, 0x03, 0x40, 0x68, 0x00, \
  0x82, 0x50, 0x4a, 0x09, 0x41, 0x20, \
  0x21, 0x24, 0x24, 0x84, 0x90, 0x90, \
  0x0c, 0x21, 0x84, 0x30, 0x86, 0x10, \
  0x4a, 0x09, 0x41, 0x28, 0x25, 0x00, \
  0x12, 0xa2, 0x54, 0x4a, 0x89, 0x50, \
  0x64, 0x4c, 0x89, 0x91, 0x32, 0x20, \
  0x51, 0x4a, 0x29, 0x45, 0x28, 0xa0, \
  0xa9, 0x15, 0x22, 0xa4, 0x54, 0x80, \
  0x04, 0xc0, 0x98, 0x13, 0x02, 0x60, \
  0xd0, 0x1a, 0x03, 0x40, 0x68, 0x00, \
  0x82, 0x50, 0x4a, 0x09, 0x41, 0x20, \
  0x21, 0x24, 0x24, 0x84, 0x90, 0x90, \
  0x0c, 0x21, 0x84, 0x30, 0x86, 0x10, \
  0x4a, 0x09, 0x41, 0x28, 0x25, 0x00, \
  0x12, 0xa2, 0x54, 0x4a, 0x89, 0x50, \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x62, 0x6c, 0x4d, 0x89, 0xb1, 0x30, \
  0x8c, 0x11, 0x82, 0x30, 0x46, 0x00, \
  0x01, 0x60, 0x2c, 0x05, 0x80, 0xb0, \
  0x07, 0x80, 0xf0, 0x1e, 0x03, 0xc0, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x18, 0xa3, 0x14, 0x62, 0x8c, 0x50, \
  0x91, 0x12, 0x22, 0x44, 0x48, 0x80, \
  0x78, 0x0f, 0x01, 0xe0, 0x3c, 0x00, \
  0x3b, 0x48, 0xc4, 0xed, 0x23, 0x10

#define kMaskRandom44_4 \
  0xc4, 0xd8, 0x9b, 0x13, 0x62, 0x60, \
  0x31, 0x66, 0x2c, 0xc5, 0x98, 0xb0, \
  0x4b, 0x29, 0x65, 0x2c, 0xa5, 0x90, \
  0x2c, 0xa5, 0x94, 0xb2, 0x96, 0x50

#define kMaskRandom44_40 \
  0x64, 0x4c, 0x89, 0x91, 0x32, 0x20, \
  0x51, 0x4a, 0x29, 0x45, 0x28, 0xa0, \
  0xa9, 0x15, 0x22, 0xa4, 0x54, 0x80, \
  0x04, 0xc0, 0x98, 0x13, 0x02, 0x60, \
  0xd0, 0x1a, 0x03, 0x40, 0x68, 0x00, \
  0x82, 0x50, 0x4a, 0x09, 0x41, 0x20, \
  0x21, 0x24, 0x24, 0x84, 0x90, 0x90, \
  0x0c, 0x21, 0x84, 0x30, 0x86, 0x10, \
  0x4a, 0x09, 0x41, 0x28, 0x25, 0x00, \
  0x12, 0xa2, 0x54, 0x4a, 0x89, 0x50, \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x62, 0x6c, 0x4d, 0x89, 0xb1, 0x30, \
  0x8c, 0x11, 0x82, 0x30, 0x46, 0x00, \
  0x01, 0x60, 0x2c, 0x05, 0x80, 0xb0, \
  0x07, 0x80, 0xf0, 0x1e, 0x03, 0xc0, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x18, 0xa3, 0x14, 0x62, 0x8c, 0x50, \
  0x91, 0x12, 0x22, 0x44, 0x48, 0x80, \
  0x78, 0x0f, 0x01, 0xe0, 0x3c, 0x00, \
  0x3b, 0x48, 0xc4, 0xed, 0x23, 0x10, \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x62, 0x6c, 0x4d, 0x89, 0xb1, 0x30, \
  0x8c, 0x11, 0x82, 0x30, 0x46, 0x00, \
  0x01, 0x60, 0x2c, 0x05, 0x80, 0xb0, \
  0x07, 0x80, 0xf0, 0x1e, 0x03, 0xc0, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x18, 0xa3, 0x14, 0x62, 0x8c, 0x50, \
  0x91, 0x12, 0x22, 0x44, 0x48, 0x80, \
  0x78, 0x0f, 0x01, 0xe0, 0x3c, 0x00, \
  0x64, 0x4c, 0x89, 0x91, 0x32, 0x20, \
  0x51, 0x4a, 0x29, 0x45, 0x28, 0xa0, \
  0xa9, 0x15, 0x22, 0xa4, 0x54, 0x80, \
  0x04, 0xc0, 0x98, 0x13, 0x02, 0x60, \
  0xd0, 0x1a, 0x03, 0x40, 0x68, 0x00, \
  0x82, 0x50, 0x4a, 0x09, 0x41, 0x20, \
  0x21, 0x24, 0x24, 0x84, 0x90, 0x90, \
  0x0c, 0x21, 0x84, 0x30, 0x86, 0x10, \
  0x4a, 0x09, 0x41, 0x28, 0x25, 0x00, \
  0x12, 0xa2, 0x54, 0x4a, 0x89, 0x50, \
  0xd8, 0x2a, 0x16, 0x26, 0x51, 0x40

#define kMaskRandom44_41 \
  0x64, 0x4c, 0x89, 0x91, 0x32, 0x20, \
  0x51, 0x4a, 0x29, 0x45, 0x28, 0xa0, \
  0xa9, 0x15, 0x22, 0xa4, 0x54, 0x80, \
  0x04, 0xc0, 0x98, 0x13, 0x02, 0x60, \
  0xd0, 0x1a, 0x03, 0x40, 0x68, 0x00, \
  0x82, 0x50, 0x4a, 0x09, 0x41, 0x20, \
  0x21, 0x24, 0x24, 0x84, 0x90, 0x90, \
  0x0c, 0x21, 0x84, 0x30, 0x86, 0x10, \
  0x4a, 0x09, 0x41, 0x28, 0x25, 0x00, \
  0x12, 0xa2, 0x54, 0x4a, 0x89, 0x50, \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x62, 0x6c, 0x4d, 0x89, 0xb1, 0x30, \
  0x8c, 0x11, 0x82, 0x30, 0x46, 0x00, \
  0x01, 0x60, 0x2c, 0x05, 0x80, 0xb0, \
  0x07, 0x80, 0xf0, 0x1e, 0x03, 0xc0, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x18, 0xa3, 0x14, 0x62, 0x8c, 0x50, \
  0x91, 0x12, 0x22, 0x44, 0x48, 0x80, \
  0x78, 0x0f, 0x01, 0xe0, 0x3c, 0x00, \
  0x3b, 0x48, 0xc4, 0xed, 0x23, 0x10, \
  0x64, 0x4c, 0x89, 0x91, 0x32, 0x20, \
  0x51, 0x4a, 0x29, 0x45, 0x28, 0xa0, \
  0xa9, 0x15, 0x22, 0xa4, 0x54, 0x80, \
  0x04, 0xc0, 0x98, 0x13, 0x02, 0x60, \
  0xd0, 0x1a, 0x03, 0x40, 0x68, 0x00, \
  0x82, 0x50, 0x4a, 0x09, 0x41, 0x20, \
  0x21, 0x24, 0x24, 0x84, 0x90, 0x90, \
  0x0c, 0x21, 0x84, 0x30, 0x86, 0x10, \
  0x4a, 0x09, 0x41, 0x28, 0x25, 0x00, \
  0x12, 0xa2, 0x54, 0x4a, 0x89, 0x50, \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x33, 0x26, 0x64, 0xcc, 0x99, 0x90, \
  0x99, 0x13, 0x22, 0x64, 0x4c, 0x80, \
  0x05, 0x80, 0xb0, 0x16, 0x02, 0xc0, \
  0x80, 0xb0, 0x16, 0x02, 0xc0, 0x50, \
  0x84, 0x50, 0x8a, 0x11, 0x42, 0x20, \
  0x40, 0x68, 0x0d, 0x01, 0xa0, 0x30, \
  0x0a, 0x81, 0x50, 0x2a, 0x05, 0x40, \
  0x68, 0x0d, 0x01, 0xa0, 0x34, 0x00, \
  0x10, 0x22, 0x04, 0x40, 0x88, 0x10, \
  0x30, 0x46, 0x08, 0xc1, 0x18, 0x20

#define kMaskRandom44_42 \
  0x64, 0x4c, 0x89, 0x91, 0x32, 0x20, \
  0x51, 0x4a, 0x29, 0x45, 0x28, 0xa0, \
  0xa9, 0x15, 0x22, 0xa4, 0x54, 0x80, \
  0x04, 0xc0, 0x98, 0x13, 0x02, 0x60, \
  0xd0, 0x1a, 0x03, 0x40, 0x68, 0x00, \
  0x82, 0x50, 0x4a, 0x09, 0x41, 0x20, \
  0x21, 0x24, 0x24, 0x84, 0x90, 0x90, \
  0x0c, 0x21, 0x84, 0x30, 0x86, 0x10, \
  0x4a, 0x09, 0x41, 0x28, 0x25, 0x00, \
  0x12, 0xa2, 0x54, 0x4a, 0x89, 0x50, \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x33, 0x26, 0x64, 0xcc, 0x99, 0x90, \
  0x99, 0x13, 0x22, 0x64, 0x4c, 0x80, \
  0x05, 0x80, 0xb0, 0x16, 0x02, 0xc0, \
  0x80, 0xb0, 0x16, 0x02, 0xc0, 0x50, \
  0x84, 0x50, 0x8a, 0x11, 0x42, 0x20, \
  0x40, 0x68, 0x0d, 0x01, 0xa0, 0x30, \
  0x0a, 0x81, 0x50, 0x2a, 0x05, 0x40, \
  0x68, 0x0d, 0x01, 0xa0, 0x34, 0x00, \
  0x10, 0x22, 0x04, 0x40, 0x88, 0x10, \
  0x30, 0x46, 0x08, 0xc1, 0x18, 0x20, \
  0x64, 0x4c, 0x89, 0x91, 0x32, 0x20, \
  0x51, 0x4a, 0x29, 0x45, 0x28, 0xa0, \
  0xa9, 0x15, 0x22, 0xa4, 0x54, 0x80, \
  0x04, 0xc0, 0x98, 0x13, 0x02, 0x60, \
  0xd0, 0x1a, 0x03, 0x40, 0x68, 0x00, \
  0x82, 0x50, 0x4a, 0x09, 0x41, 0x20, \
  0x21, 0x24, 0x24, 0x84, 0x90, 0x90, \
  0x0c, 0x21, 0x84, 0x30, 0x86, 0x10, \
  0x4a, 0x09, 0x41, 0x28, 0x25, 0x00, \
  0x12, 0xa2, 0x54, 0x4a, 0x89, 0x50, \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x62, 0x6c, 0x4d, 0x89, 0xb1, 0x30, \
  0x8c, 0x11, 0x82, 0x30, 0x46, 0x00, \
  0x01, 0x60, 0x2c, 0x05, 0x80, 0xb0, \
  0x07, 0x80, 0xf0, 0x1e, 0x03, 0xc0, \
  0xa0, 0x94, 0x12, 0x82, 0x50, 0x40, \
  0x18, 0xa3, 0x14, 0x62, 0x8c, 0x50, \
  0x91, 0x12, 0x22, 0x44, 0x48, 0x80, \
  0x78, 0x0f, 0x01, 0xe0, 0x3c, 0x00, \
  0x3b, 0x48, 0xc4, 0xed, 0x23, 0x10, \
  0xd9, 0xc1, 0x6f, 0xa8, 0x1c, 0x90

#define kMaskRandom44_43 \
  0x64, 0x4c, 0x89, 0x91, 0x32, 0x20, \
  0x51, 0x4a, 0x29, 0x45, 0x28, 0xa0, \
  0xa9, 0x15, 0x22, 0xa4, 0x54, 0x80, \
  0x04, 0xc0, 0x98, 0x13, 0x02, 0x60, \
  0xd0, 0x1a, 0x03, 0x40, 0x68, 0x00, \
  0x82, 0x50, 0x4a, 0x09, 0x41, 0x20, \
  0x21, 0x24, 0x24, 0x84, 0x90, 0x90, \
  0x0c, 0x21, 0x84, 0x30, 0x86, 0x10, \
  0x4a, 0x09, 0x41, 0x28, 0x25, 0x00, \
  0x12, 0xa2, 0x54, 0x4a, 0x89, 0x50, \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x33, 0x26, 0x64, 0xcc, 0x99, 0x90, \
  0x99, 0x13, 0x22, 0x64, 0x4c, 0x80, \
  0x05, 0x80, 0xb0, 0x16, 0x02, 0xc0, \
  0x80, 0xb0, 0x16, 0x02, 0xc0, 0x50, \
  0x84, 0x50, 0x8a, 0x11, 0x42, 0x20, \
  0x40, 0x68, 0x0d, 0x01, 0xa0, 0x30, \
  0x0a, 0x81, 0x50, 0x2a, 0x05, 0x40, \
  0x68, 0x0d, 0x01, 0xa0, 0x34, 0x00, \
  0x10, 0x22, 0x04, 0x40, 0x88, 0x10, \
  0x30, 0x46, 0x08, 0xc1, 0x18, 0x20, \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x33, 0x26, 0x64, 0xcc, 0x99, 0x90, \
  0x99, 0x13, 0x22, 0x64, 0x4c, 0x80, \
  0x05, 0x80, 0xb0, 0x16, 0x02, 0xc0, \
  0x80, 0xb0, 0x16, 0x02, 0xc0, 0x50, \
  0x84, 0x50, 0x8a, 0x11, 0x42, 0x20, \
  0x40, 0x68, 0x0d, 0x01, 0xa0, 0x30, \
  0x0a, 0x81, 0x50, 0x2a, 0x05, 0x40, \
  0x68, 0x0d, 0x01, 0xa0, 0x34, 0x00, \
  0x10, 0x22, 0x04, 0x40, 0x88, 0x10, \
  0x30, 0x46, 0x08, 0xc1, 0x18, 0x20, \
  0x64, 0x4c, 0x89, 0x91, 0x32, 0x20, \
  0x51, 0x4a, 0x29, 0x45, 0x28, 0xa0, \
  0xa9, 0x15, 0x22, 0xa4, 0x54, 0x80, \
  0x04, 0xc0, 0x98, 0x13, 0x02, 0x60, \
  0xd0, 0x1a, 0x03, 0x40, 0x68, 0x00, \
  0x82, 0x50, 0x4a, 0x09, 0x41, 0x20, \
  0x21, 0x24, 0x24, 0x84, 0x90, 0x90, \
  0x0c, 0x21, 0x84, 0x30, 0x86, 0x10, \
  0x4a, 0x09, 0x41, 0x28, 0x25, 0x00, \
  0x12, 0xa2, 0x54, 0x4a, 0x89, 0x50, \
  0x9e, 0xce, 0x8a, 0x7b, 0x3a, 0x20

#define kMaskRandom44_44 \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x33, 0x26, 0x64, 0xcc, 0x99, 0x90, \
  0x99, 0x13, 0x22, 0x64, 0x4c, 0x80, \
  0x05, 0x80, 0xb0, 0x16, 0x02, 0xc0, \
  0x80, 0xb0, 0x16, 0x02, 0xc0, 0x50, \
  0x84, 0x50, 0x8a, 0x11, 0x42, 0x20, \
  0x40, 0x68, 0x0d, 0x01, 0xa0, 0x30, \
  0x0a, 0x81, 0x50, 0x2a, 0x05, 0x40, \
  0x68, 0x0d, 0x01, 0xa0, 0x34, 0x00, \
  0x10, 0x22, 0x04, 0x40, 0x88, 0x10, \
  0x30, 0x46, 0x08, 0xc1, 0x18, 0x20, \
  0x64, 0x4c, 0x89, 0x91, 0x32, 0x20, \
  0x51, 0x4a, 0x29, 0x45, 0x28, 0xa0, \
  0xa9, 0x15, 0x22, 0xa4, 0x54, 0x80, \
  0x04, 0xc0, 0x98, 0x13, 0x02, 0x60, \
  0xd0, 0x1a, 0x03, 0x40, 0x68, 0x00, \
  0x82, 0x50, 0x4a, 0x09, 0x41, 0x20, \
  0x21, 0x24, 0x24, 0x84, 0x90, 0x90, \
  0x0c, 0x21, 0x84, 0x30, 0x86, 0x10, \
  0x4a, 0x09, 0x41, 0x28, 0x25, 0x00, \
  0x12, 0xa2, 0x54, 0x4a, 0x89, 0x50, \
  0x9e, 0xce, 0x8a, 0x7b, 0x3a, 0x20, \
  0x64, 0x4c, 0x89, 0x91, 0x32, 0x20, \
  0x51, 0x4a, 0x29, 0x45, 0x28, 0xa0, \
  0xa9, 0x15, 0x22, 0xa4, 0x54, 0x80, \
  0x04, 0xc0, 0x98, 0x13, 0x02, 0x60, \
  0xd0, 0x1a, 0x03, 0x40, 0x68, 0x00, \
  0x82, 0x50, 0x4a, 0x09, 0x41, 0x20, \
  0x21, 0x24, 0x24, 0x84, 0x90, 0x90, \
  0x0c, 0x21, 0x84, 0x30, 0x86, 0x10, \
  0x4a, 0x09, 0x41, 0x28, 0x25, 0x00, \
  0x12, 0xa2, 0x54, 0x4a, 0x89, 0x50, \
  0x46, 0x48, 0xc9, 0x19, 0x23, 0x20, \
  0x33, 0x26, 0x64, 0xcc, 0x99, 0x90, \
  0x99, 0x13, 0x22, 0x64, 0x4c, 0x80, \
  0x05, 0x80, 0xb0, 0x16, 0x02, 0xc0, \
  0x80, 0xb0, 0x16, 0x02, 0xc0, 0x50, \
  0x84, 0x50, 0x8a, 0x11, 0x42, 0x20, \
  0x40, 0x68, 0x0d, 0x01, 0xa0, 0x30, \
  0x0a, 0x81, 0x50, 0x2a, 0x05, 0x40, \
  0x68, 0x0d, 0x01, 0xa0, 0x34, 0x00, \
  0x10, 0x22, 0x04, 0x40, 0x88, 0x10, \
  0x30, 0x46, 0x08, 0xc1, 0x18, 0x20, \
  0xb5, 0x1c, 0x1c, 0x21, 0xac, 0xa0

#define kMaskRandom44_5 \
  0xc6, 0xd8, 0xdb, 0x1b, 0x63, 0x60, \
  0x63, 0x6c, 0x6d, 0x8d, 0xb1, 0xb0, \
  0x1d, 0xa3, 0xb4, 0x76, 0x8e, 0xd0, \
  0xad, 0x55, 0xaa, 0xb5, 0x56, 0xa0, \
  0xb2, 0xb6, 0x56, 0xca, 0xd9, 0x50

#define kMaskRandom44_6 \
  0x64, 0x4c, 0x89, 0x91, 0x32, 0x20, \
  0x51, 0x4a, 0x29, 0x45, 0x28, 0xa0, \
  0xa8, 0x35, 0x06, 0xa0, 0xd4, 0x10, \
  0xc4, 0xa0, 0x97, 0x12, 0x82, 0x50, \
  0x03, 0x60, 0x6c, 0x0d, 0x81, 0xb0, \
  0x90, 0xd2, 0x1a, 0x43, 0x48, 0x60

#define kMaskRandom44_7 \
  0xc6, 0x48, 0xcb, 0x19, 0x23, 0x20, \
  0x13, 0x26, 0x64, 0x4c, 0x99, 0x90, \
  0x8d, 0x13, 0xa2, 0x34, 0x4e, 0x80, \
  0x8b, 0x41, 0x6a, 0x2d, 0x05, 0xa0, \
  0x52, 0xaa, 0x15, 0x4a, 0xa8, 0x50, \
  0xa2, 0xd4, 0x1a, 0x8b, 0x50, 0x60, \
  0x61, 0xa8, 0x2d, 0x86, 0xa0, 0xb0

#define kMaskRandom44_8 \
  0x28, 0x85, 0x38, 0xa2, 0x14, 0xe0, \
  0x21, 0xf4, 0x04, 0x87, 0xd0, 0x10, \
  0xe9, 0x1d, 0x03, 0xa4, 0x74, 0x00, \
  0x17, 0x02, 0xe0, 0x5c, 0x0b, 0x80, \
  0x83, 0xa0, 0x56, 0x0e, 0x81, 0x50, \
  0x46, 0x18, 0xe9, 0x18, 0x63, 0xa0, \
  0x50, 0x6a, 0x0d, 0x41, 0xa8, 0x30, \
  0x1c, 0x23, 0x84, 0x70, 0x8e, 0x10

#define kMaskRandom44_9 \
  0x44, 0x48, 0xc9, 0x11, 0x23, 0x20, \
  0x28, 0x2d, 0x0c, 0xa0, 0xb4, 0x30, \
  0x25, 0x14, 0xa0, 0x94, 0x52, 0x80, \
  0x59, 0x0a, 0x21, 0x64, 0x28, 0x80, \
  0x03, 0xa0, 0x34, 0x0e, 0x80, 0xd0, \
  0xc0, 0xd0, 0x1b, 0x03, 0x40, 0x60, \
  0xa2, 0x30, 0x46, 0x88, 0xc1, 0x10, \
  0x14, 0x82, 0xd0, 0x52, 0x0b, 0x40, \
  0x9a, 0x03, 0x82, 0x68, 0x0e, 0x00

#define kMaskRandom45_1 \
  0xff, 0xff, 0xff, 0xff, 0xff, 0xf8

#define kMaskRandom45_10 \
  0xc0, 0x38, 0x89, 0x91, 0x28, 0xa0, \
  0x30, 0x0e, 0x29, 0x45, 0x22, 0x88, \
  0xe8, 0x07, 0x02, 0xa4, 0x40, 0x68, \
  0x85, 0x08, 0xa8, 0x13, 0x12, 0x10, \
  0xd0, 0x92, 0x13, 0x40, 0x05, 0x10, \
  0x86, 0x50, 0x4a, 0x09, 0x00, 0x70, \
  0x4a, 0x68, 0x0c, 0x84, 0xdc, 0x00, \
  0x01, 0xa0, 0x74, 0x30, 0x84, 0x88, \
  0x4c, 0x81, 0x91, 0x28, 0x2b, 0x00, \
  0x62, 0x24, 0x04, 0x4a, 0xd1, 0x40

#define kMaskRandom45_11 \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x33, 0x26, 0x64, 0xcc, 0x90, 0x98, \
  0x99, 0x13, 0x22, 0x64, 0x08, 0x48, \
  0x05, 0x80, 0xb0, 0x16, 0x00, 0x38, \
  0x80, 0xb0, 0x16, 0x02, 0x86, 0x08, \
  0x84, 0x50, 0x8a, 0x11, 0x20, 0x60, \
  0x40, 0x68, 0x0d, 0x01, 0xb5, 0x00, \
  0x0a, 0x81, 0x50, 0x2a, 0x43, 0x00, \
  0x68, 0x0d, 0x01, 0xa0, 0x12, 0x40, \
  0x10, 0x22, 0x04, 0x40, 0xc4, 0x80, \
  0x30, 0x46, 0x08, 0xc1, 0x60, 0x10

#define kMaskRandom45_12 \
  0x64, 0x4c, 0x89, 0x91, 0x28, 0xa0, \
  0x51, 0x4a, 0x29, 0x45, 0x62, 0x88, \
  0x0c, 0xa1, 0x94, 0x32, 0x90, 0xc0, \
  0xa1, 0x34, 0x26, 0x84, 0x89, 0x18, \
  0x12, 0xa2, 0x54, 0x4a, 0x84, 0x70, \
  0x8a, 0x51, 0x4a, 0x29, 0x17, 0x00, \
  0x86, 0x90, 0xd2, 0x1a, 0x29, 0xb0, \
  0x23, 0x24, 0x64, 0x8c, 0xb2, 0x10, \
  0x16, 0x22, 0xc4, 0x58, 0x86, 0x60, \
  0x4c, 0x29, 0x85, 0x30, 0xc1, 0x50, \
  0x41, 0xc8, 0x39, 0x07, 0x04, 0x98, \
  0xf4, 0x18, 0x9c, 0x65, 0x5b, 0x90

#define kMaskRandom45_13 \
  0x64, 0x4c, 0x89, 0x91, 0x28, 0xa0, \
  0x51, 0x4a, 0x29, 0x45, 0x62, 0x88, \
  0x0c, 0xa1, 0x94, 0x32, 0x90, 0xc0, \
  0xa1, 0x34, 0x26, 0x84, 0x89, 0x18, \
  0x12, 0xa2, 0x54, 0x4a, 0x84, 0x70, \
  0x8a, 0x51, 0x4a, 0x29, 0x17, 0x00, \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x33, 0x26, 0x64, 0xcc, 0x90, 0x98, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0x48, \
  0xa4, 0x34, 0x86, 0x90, 0x81, 0x28, \
  0x50, 0xaa, 0x15, 0x42, 0x83, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x16, 0x00, \
  0x09, 0x61, 0x2c, 0x25, 0xc4, 0x30

#define kMaskRandom45_14 \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x33, 0x26, 0x64, 0xcc, 0x90, 0x98, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0x48, \
  0xa4, 0x34, 0x86, 0x90, 0x81, 0x28, \
  0x50, 0xaa, 0x15, 0x42, 0x83, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x16, 0x00, \
  0x09, 0x61, 0x2c, 0x25, 0xc4, 0x30, \
  0x64, 0x4c, 0x89, 0x91, 0x28, 0xa0, \
  0x51, 0x4a, 0x29, 0x45, 0x62, 0x88, \
  0x0c, 0xa1, 0x94, 0x32, 0x90, 0xc0, \
  0xa1, 0x34, 0x26, 0x84, 0x89, 0x18, \
  0x12, 0xa2, 0x54, 0x4a, 0x84, 0x70, \
  0x8a, 0x51, 0x4a, 0x29, 0x17, 0x00, \
  0xc6, 0xca, 0xea, 0x70, 0xfe, 0xc8

#define kMaskRandom45_15 \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x33, 0x26, 0x64, 0xcc, 0x90, 0x98, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0x48, \
  0xa4, 0x34, 0x86, 0x90, 0x81, 0x28, \
  0x50, 0xaa, 0x15, 0x42, 0x83, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x16, 0x00, \
  0x09, 0x61, 0x2c, 0x25, 0xc4, 0x30, \
  0x0c, 0x81, 0x90, 0x32, 0x10, 0x30, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x21, 0x20, \
  0x05, 0x40, 0xa8, 0x15, 0x00, 0xc8, \
  0x43, 0x08, 0x61, 0x0c, 0x0a, 0x08, \
  0x1a, 0x03, 0x40, 0x68, 0x05, 0x40, \
  0x60, 0x2c, 0x05, 0x80, 0x9c, 0x00, \
  0x14, 0x22, 0x84, 0x50, 0xe2, 0x80

#define kMaskRandom45_16 \
  0x0c, 0x81, 0x90, 0x32, 0x10, 0x30, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x21, 0x20, \
  0x05, 0x40, 0xa8, 0x15, 0x00, 0xc8, \
  0x43, 0x08, 0x61, 0x0c, 0x0a, 0x08, \
  0x1a, 0x03, 0x40, 0x68, 0x05, 0x40, \
  0x60, 0x2c, 0x05, 0x80, 0x9c, 0x00, \
  0x14, 0x22, 0x84, 0x50, 0xe2, 0x80, \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x33, 0x26, 0x64, 0xcc, 0x90, 0x98, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0x48, \
  0xa4, 0x34, 0x86, 0x90, 0x81, 0x28, \
  0x50, 0xaa, 0x15, 0x42, 0x83, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x16, 0x00, \
  0x09, 0x61, 0x2c, 0x25, 0xc4, 0x30, \
  0x86, 0xc1, 0x47, 0xeb, 0x67, 0xd0

#define kMaskRandom45_17 \
  0x0c, 0x81, 0x90, 0x32, 0x10, 0x30, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x21, 0x20, \
  0x05, 0x40, 0xa8, 0x15, 0x00, 0xc8, \
  0x43, 0x08, 0x61, 0x0c, 0x0a, 0x08, \
  0x1a, 0x03, 0x40, 0x68, 0x05, 0x40, \
  0x60, 0x2c, 0x05, 0x80, 0x9c, 0x00, \
  0x14, 0x22, 0x84, 0x50, 0xe2, 0x80, \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x62, 0x6c, 0x4d, 0x89, 0xf2, 0x10, \
  0x8c, 0x11, 0x82, 0x30, 0x12, 0x20, \
  0x01, 0x60, 0x2c, 0x05, 0xd0, 0x88, \
  0x07, 0x80, 0xf0, 0x1e, 0x0c, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x01, 0xc8, \
  0x18, 0xa3, 0x14, 0x62, 0xc5, 0x08, \
  0x91, 0x12, 0x22, 0x44, 0x02, 0x48, \
  0x78, 0x0f, 0x01, 0xe0, 0x00, 0x70

#define kMaskRandom45_18 \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x62, 0x6c, 0x4d, 0x89, 0xf2, 0x10, \
  0x8c, 0x11, 0x82, 0x30, 0x12, 0x20, \
  0x01, 0x60, 0x2c, 0x05, 0xd0, 0x88, \
  0x07, 0x80, 0xf0, 0x1e, 0x0c, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x01, 0xc8, \
  0x18, 0xa3, 0x14, 0x62, 0xc5, 0x08, \
  0x91, 0x12, 0x22, 0x44, 0x02, 0x48, \
  0x78, 0x0f, 0x01, 0xe0, 0x00, 0x70, \
  0x0c, 0x81, 0x90, 0x32, 0x10, 0x30, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x21, 0x20, \
  0x05, 0x40, 0xa8, 0x15, 0x00, 0xc8, \
  0x43, 0x08, 0x61, 0x0c, 0x0a, 0x08, \
  0x1a, 0x03, 0x40, 0x68, 0x05, 0x40, \
  0x60, 0x2c, 0x05, 0x80, 0x9c, 0x00, \
  0x14, 0x22, 0x84, 0x50, 0xe2, 0x80, \
  0xe4, 0xd4, 0x6e, 0x08, 0xc9, 0x58

#define kMaskRandom45_19 \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x62, 0x6c, 0x4d, 0x89, 0xf2, 0x10, \
  0x8c, 0x11, 0x82, 0x30, 0x12, 0x20, \
  0x01, 0x60, 0x2c, 0x05, 0xd0, 0x88, \
  0x07, 0x80, 0xf0, 0x1e, 0x0c, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x01, 0xc8, \
  0x18, 0xa3, 0x14, 0x62, 0xc5, 0x08, \
  0x91, 0x12, 0x22, 0x44, 0x02, 0x48, \
  0x78, 0x0f, 0x01, 0xe0, 0x00, 0x70, \
  0x64, 0x4c, 0x89, 0x91, 0x28, 0xa0, \
  0x51, 0x4a, 0x29, 0x45, 0x22, 0x88, \
  0xa9, 0x15, 0x22, 0xa4, 0x40, 0x68, \
  0x04, 0xc0, 0x98, 0x13, 0x12, 0x10, \
  0xd0, 0x1a, 0x03, 0x40, 0x05, 0x10, \
  0x82, 0x50, 0x4a, 0x09, 0x00, 0x70, \
  0x21, 0x24, 0x24, 0x84, 0xdc, 0x00, \
  0x0c, 0x21, 0x84, 0x30, 0x84, 0x88, \
  0x4a, 0x09, 0x41, 0x28, 0x2b, 0x00, \
  0x12, 0xa2, 0x54, 0x4a, 0xd1, 0x40

#define kMaskRandom45_2 \
  0xec, 0xdd, 0x9b, 0xb3, 0x76, 0x60, \
  0x9b, 0xb3, 0x76, 0x6e, 0xc9, 0xd8

#define kMaskRandom45_20 \
  0x64, 0x4c, 0x89, 0x91, 0x28, 0xa0, \
  0x51, 0x4a, 0x29, 0x45, 0x22, 0x88, \
  0xa9, 0x15, 0x22, 0xa4, 0x40, 0x68, \
  0x04, 0xc0, 0x98, 0x13, 0x12, 0x10, \
  0xd0, 0x1a, 0x03, 0x40, 0x05, 0x10, \
  0x82, 0x50, 0x4a, 0x09, 0x00, 0x70, \
  0x21, 0x24, 0x24, 0x84, 0xdc, 0x00, \
  0x0c, 0x21, 0x84, 0x30, 0x84, 0x88, \
  0x4a, 0x09, 0x41, 0x28, 0x2b, 0x00, \
  0x12, 0xa2, 0x54, 0x4a, 0xd1, 0x40, \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x62, 0x6c, 0x4d, 0x89, 0xf2, 0x10, \
  0x8c, 0x11, 0x82, 0x30, 0x12, 0x20, \
  0x01, 0x60, 0x2c, 0x05, 0xd0, 0x88, \
  0x07, 0x80, 0xf0, 0x1e, 0x0c, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x01, 0xc8, \
  0x18, 0xa3, 0x14, 0x62, 0xc5, 0x08, \
  0x91, 0x12, 0x22, 0x44, 0x02, 0x48, \
  0x78, 0x0f, 0x01, 0xe0, 0x00, 0x70, \
  0x3b, 0x48, 0xc7, 0x6d, 0x29, 0xe8

#define kMaskRandom45_21 \
  0x64, 0x4c, 0x89, 0x91, 0x28, 0xa0, \
  0x51, 0x4a, 0x29, 0x45, 0x22, 0x88, \
  0xa9, 0x15, 0x22, 0xa4, 0x40, 0x68, \
  0x04, 0xc0, 0x98, 0x13, 0x12, 0x10, \
  0xd0, 0x1a, 0x03, 0x40, 0x05, 0x10, \
  0x82, 0x50, 0x4a, 0x09, 0x00, 0x70, \
  0x21, 0x24, 0x24, 0x84, 0xdc, 0x00, \
  0x0c, 0x21, 0x84, 0x30, 0x84, 0x88, \
  0x4a, 0x09, 0x41, 0x28, 0x2b, 0x00, \
  0x12, 0xa2, 0x54, 0x4a, 0xd1, 0x40, \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x33, 0x26, 0x64, 0xcc, 0x90, 0x98, \
  0x99, 0x13, 0x22, 0x64, 0x08, 0x48, \
  0x05, 0x80, 0xb0, 0x16, 0x00, 0x38, \
  0x80, 0xb0, 0x16, 0x02, 0x86, 0x08, \
  0x84, 0x50, 0x8a, 0x11, 0x20, 0x60, \
  0x40, 0x68, 0x0d, 0x01, 0xb5, 0x00, \
  0x0a, 0x81, 0x50, 0x2a, 0x43, 0x00, \
  0x68, 0x0d, 0x01, 0xa0, 0x12, 0x40, \
  0x10, 0x22, 0x04, 0x40, 0xc4, 0x80, \
  0x30, 0x46, 0x08, 0xc1, 0x60, 0x10

#define kMaskRandom45_22 \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x33, 0x26, 0x64, 0xcc, 0x90, 0x98, \
  0x99, 0x13, 0x22, 0x64, 0x08, 0x48, \
  0x05, 0x80, 0xb0, 0x16, 0x00, 0x38, \
  0x80, 0xb0, 0x16, 0x02, 0x86, 0x08, \
  0x84, 0x50, 0x8a, 0x11, 0x20, 0x60, \
  0x40, 0x68, 0x0d, 0x01, 0xb5, 0x00, \
  0x0a, 0x81, 0x50, 0x2a, 0x43, 0x00, \
  0x68, 0x0d, 0x01, 0xa0, 0x12, 0x40, \
  0x10, 0x22, 0x04, 0x40, 0xc4, 0x80, \
  0x30, 0x46, 0x08, 0xc1, 0x60, 0x10, \
  0x64, 0x4c, 0x89, 0x91, 0x28, 0xa0, \
  0x51, 0x4a, 0x29, 0x45, 0x22, 0x88, \
  0xa9, 0x15, 0x22, 0xa4, 0x40, 0x68, \
  0x04, 0xc0, 0x98, 0x13, 0x12, 0x10, \
  0xd0, 0x1a, 0x03, 0x40, 0x05, 0x10, \
  0x82, 0x50, 0x4a, 0x09, 0x00, 0x70, \
  0x21, 0x24, 0x24, 0x84, 0xdc, 0x00, \
  0x0c, 0x21, 0x84, 0x30, 0x84, 0x88, \
  0x4a, 0x09, 0x41, 0x28, 0x2b, 0x00, \
  0x12, 0xa2, 0x54, 0x4a, 0xd1, 0x40, \
  0x9e, 0xce, 0x8b, 0xaa, 0x34, 0x68

#define kMaskRandom45_23 \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x33, 0x26, 0x64, 0xcc, 0x90, 0x98, \
  0x99, 0x13, 0x22, 0x64, 0x08, 0x48, \
  0x05, 0x80, 0xb0, 0x16, 0x00, 0x38, \
  0x80, 0xb0, 0x16, 0x02, 0x86, 0x08, \
  0x84, 0x50, 0x8a, 0x11, 0x20, 0x60, \
  0x40, 0x68, 0x0d, 0x01, 0xb5, 0x00, \
  0x0a, 0x81, 0x50, 0x2a, 0x43, 0x00, \
  0x68, 0x0d, 0x01, 0xa0, 0x12, 0x40, \
  0x10, 0x22, 0x04, 0x40, 0xc4, 0x80, \
  0x30, 0x46, 0x08, 0xc1, 0x60, 0x10, \
  0x64, 0x4c, 0x89, 0x91, 0x28, 0xa0, \
  0x51, 0x4a, 0x29, 0x45, 0x62, 0x88, \
  0x0c, 0xa1, 0x94, 0x32, 0x90, 0xc0, \
  0xa1, 0x34, 0x26, 0x84, 0x89, 0x18, \
  0x12, 0xa2, 0x54, 0x4a, 0x84, 0x70, \
  0x8a, 0x51, 0x4a, 0x29, 0x17, 0x00, \
  0x86, 0x90, 0xd2, 0x1a, 0x29, 0xb0, \
  0x23, 0x24, 0x64, 0x8c, 0xb2, 0x10, \
  0x16, 0x22, 0xc4, 0x58, 0x86, 0x60, \
  0x4c, 0x29, 0x85, 0x30, 0xc1, 0x50, \
  0x41, 0xc8, 0x39, 0x07, 0x04, 0x98, \
  0xf4, 0x18, 0x9c, 0x65, 0x5b, 0x90

#define kMaskRandom45_24 \
  0x64, 0x4c, 0x89, 0x91, 0x28, 0xa0, \
  0x51, 0x4a, 0x29, 0x45, 0x62, 0x88, \
  0x0c, 0xa1, 0x94, 0x32, 0x90, 0xc0, \
  0xa1, 0x34, 0x26, 0x84, 0x89, 0x18, \
  0x12, 0xa2, 0x54, 0x4a, 0x84, 0x70, \
  0x8a, 0x51, 0x4a, 0x29, 0x17, 0x00, \
  0x86, 0x90, 0xd2, 0x1a, 0x29, 0xb0, \
  0x23, 0x24, 0x64, 0x8c, 0xb2, 0x10, \
  0x16, 0x22, 0xc4, 0x58, 0x86, 0x60, \
  0x4c, 0x29, 0x85, 0x30, 0xc1, 0x50, \
  0x41, 0xc8, 0x39, 0x07, 0x04, 0x98, \
  0xf4, 0x18, 0x9c, 0x65, 0x5b, 0x90, \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x33, 0x26, 0x64, 0xcc, 0x90, 0x98, \
  0x99, 0x13, 0x22, 0x64, 0x08, 0x48, \
  0x05, 0x80, 0xb0, 0x16, 0x00, 0x38, \
  0x80, 0xb0, 0x16, 0x02, 0x86, 0x08, \
  0x84, 0x50, 0x8a, 0x11, 0x20, 0x60, \
  0x40, 0x68, 0x0d, 0x01, 0xb5, 0x00, \
  0x0a, 0x81, 0x50, 0x2a, 0x43, 0x00, \
  0x68, 0x0d, 0x01, 0xa0, 0x12, 0x40, \
  0x10, 0x22, 0x04, 0x40, 0xc4, 0x80, \
  0x30, 0x46, 0x08, 0xc1, 0x60, 0x10, \
  0x95, 0x91, 0xad, 0xd9, 0x86, 0x98

#define kMaskRandom45_25 \
  0x64, 0x4c, 0x89, 0x91, 0x28, 0xa0, \
  0x51, 0x4a, 0x29, 0x45, 0x62, 0x88, \
  0x0c, 0xa1, 0x94, 0x32, 0x90, 0xc0, \
  0xa1, 0x34, 0x26, 0x84, 0x89, 0x18, \
  0x12, 0xa2, 0x54, 0x4a, 0x84, 0x70, \
  0x8a, 0x51, 0x4a, 0x29, 0x17, 0x00, \
  0x86, 0x90, 0xd2, 0x1a, 0x29, 0xb0, \
  0x23, 0x24, 0x64, 0x8c, 0xb2, 0x10, \
  0x16, 0x22, 0xc4, 0x58, 0x86, 0x60, \
  0x4c, 0x29, 0x85, 0x30, 0xc1, 0x50, \
  0x41, 0xc8, 0x39, 0x07, 0x04, 0x98, \
  0xf4, 0x18, 0x9c, 0x65, 0x5b, 0x90, \
  0x64, 0x4c, 0x89, 0x91, 0x28, 0xa0, \
  0x51, 0x4a, 0x29, 0x45, 0x62, 0x88, \
  0x0c, 0xa1, 0x94, 0x32, 0x90, 0xc0, \
  0xa1, 0x34, 0x26, 0x84, 0x89, 0x18, \
  0x12, 0xa2, 0x54, 0x4a, 0x84, 0x70, \
  0x8a, 0x51, 0x4a, 0x29, 0x17, 0x00, \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x33, 0x26, 0x64, 0xcc, 0x90, 0x98, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0x48, \
  0xa4, 0x34, 0x86, 0x90, 0x81, 0x28, \
  0x50, 0xaa, 0x15, 0x42, 0x83, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x16, 0x00, \
  0x09, 0x61, 0x2c, 0x25, 0xc4, 0x30

#define kMaskRandom45_26 \
  0x64, 0x4c, 0x89, 0x91, 0x28, 0xa0, \
  0x51, 0x4a, 0x29, 0x45, 0x62, 0x88, \
  0x0c, 0xa1, 0x94, 0x32, 0x90, 0xc0, \
  0xa1, 0x34, 0x26, 0x84, 0x89, 0x18, \
  0x12, 0xa2, 0x54, 0x4a, 0x84, 0x70, \
  0x8a, 0x51, 0x4a, 0x29, 0x17, 0x00, \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x33, 0x26, 0x64, 0xcc, 0x90, 0x98, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0x48, \
  0xa4, 0x34, 0x86, 0x90, 0x81, 0x28, \
  0x50, 0xaa, 0x15, 0x42, 0x83, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x16, 0x00, \
  0x09, 0x61, 0x2c, 0x25, 0xc4, 0x30, \
  0x64, 0x4c, 0x89, 0x91, 0x28, 0xa0, \
  0x51, 0x4a, 0x29, 0x45, 0x62, 0x88, \
  0x0c, 0xa1, 0x94, 0x32, 0x90, 0xc0, \
  0xa1, 0x34, 0x26, 0x84, 0x89, 0x18, \
  0x12, 0xa2, 0x54, 0x4a, 0x84, 0x70, \
  0x8a, 0x51, 0x4a, 0x29, 0x17, 0x00, \
  0x86, 0x90, 0xd2, 0x1a, 0x29, 0xb0, \
  0x23, 0x24, 0x64, 0x8c, 0xb2, 0x10, \
  0x16, 0x22, 0xc4, 0x58, 0x86, 0x60, \
  0x4c, 0x29, 0x85, 0x30, 0xc1, 0x50, \
  0x41, 0xc8, 0x39, 0x07, 0x04, 0x98, \
  0xf4, 0x18, 0x9c, 0x65, 0x5b, 0x90, \
  0xb0, 0xfd, 0xb2, 0xf3, 0x8a, 0xc0

#define kMaskRandom45_27 \
  0x64, 0x4c, 0x89, 0x91, 0x28, 0xa0, \
  0x51, 0x4a, 0x29, 0x45, 0x62, 0x88, \
  0x0c, 0xa1, 0x94, 0x32, 0x90, 0xc0, \
  0xa1, 0x34, 0x26, 0x84, 0x89, 0x18, \
  0x12, 0xa2, 0x54, 0x4a, 0x84, 0x70, \
  0x8a, 0x51, 0x4a, 0x29, 0x17, 0x00, \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x33, 0x26, 0x64, 0xcc, 0x90, 0x98, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0x48, \
  0xa4, 0x34, 0x86, 0x90, 0x81, 0x28, \
  0x50, 0xaa, 0x15, 0x42, 0x83, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x16, 0x00, \
  0x09, 0x61, 0x2c, 0x25, 0xc4, 0x30, \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x33, 0x26, 0x64, 0xcc, 0x90, 0x98, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0x48, \
  0xa4, 0x34, 0x86, 0x90, 0x81, 0x28, \
  0x50, 0xaa, 0x15, 0x42, 0x83, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x16, 0x00, \
  0x09, 0x61, 0x2c, 0x25, 0xc4, 0x30, \
  0x64, 0x4c, 0x89, 0x91, 0x28, 0xa0, \
  0x51, 0x4a, 0x29, 0x45, 0x62, 0x88, \
  0x0c, 0xa1, 0x94, 0x32, 0x90, 0xc0, \
  0xa1, 0x34, 0x26, 0x84, 0x89, 0x18, \
  0x12, 0xa2, 0x54, 0x4a, 0x84, 0x70, \
  0x8a, 0x51, 0x4a, 0x29, 0x17, 0x00, \
  0xc6, 0xca, 0xea, 0x70, 0xfe, 0xc8

#define kMaskRandom45_28 \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x33, 0x26, 0x64, 0xcc, 0x90, 0x98, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0x48, \
  0xa4, 0x34, 0x86, 0x90, 0x81, 0x28, \
  0x50, 0xaa, 0x15, 0x42, 0x83, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x16, 0x00, \
  0x09, 0x61, 0x2c, 0x25, 0xc4, 0x30, \
  0x64, 0x4c, 0x89, 0x91, 0x28, 0xa0, \
  0x51, 0x4a, 0x29, 0x45, 0x62, 0x88, \
  0x0c, 0xa1, 0x94, 0x32, 0x90, 0xc0, \
  0xa1, 0x34, 0x26, 0x84, 0x89, 0x18, \
  0x12, 0xa2, 0x54, 0x4a, 0x84, 0x70, \
  0x8a, 0x51, 0x4a, 0x29, 0x17, 0x00, \
  0xc6, 0xca, 0xea, 0x70, 0xfe, 0xc8, \
  0x64, 0x4c, 0x89, 0x91, 0x28, 0xa0, \
  0x51, 0x4a, 0x29, 0x45, 0x62, 0x88, \
  0x0c, 0xa1, 0x94, 0x32, 0x90, 0xc0, \
  0xa1, 0x34, 0x26, 0x84, 0x89, 0x18, \
  0x12, 0xa2, 0x54, 0x4a, 0x84, 0x70, \
  0x8a, 0x51, 0x4a, 0x29, 0x17, 0x00, \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x33, 0x26, 0x64, 0xcc, 0x90, 0x98, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0x48, \
  0xa4, 0x34, 0x86, 0x90, 0x81, 0x28, \
  0x50, 0xaa, 0x15, 0x42, 0x83, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x16, 0x00, \
  0x09, 0x61, 0x2c, 0x25, 0xc4, 0x30, \
  0x44, 0x46, 0x28, 0xfb, 0x66, 0x80

#define kMaskRandom45_29 \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x33, 0x26, 0x64, 0xcc, 0x90, 0x98, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0x48, \
  0xa4, 0x34, 0x86, 0x90, 0x81, 0x28, \
  0x50, 0xaa, 0x15, 0x42, 0x83, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x16, 0x00, \
  0x09, 0x61, 0x2c, 0x25, 0xc4, 0x30, \
  0x64, 0x4c, 0x89, 0x91, 0x28, 0xa0, \
  0x51, 0x4a, 0x29, 0x45, 0x62, 0x88, \
  0x0c, 0xa1, 0x94, 0x32, 0x90, 0xc0, \
  0xa1, 0x34, 0x26, 0x84, 0x89, 0x18, \
  0x12, 0xa2, 0x54, 0x4a, 0x84, 0x70, \
  0x8a, 0x51, 0x4a, 0x29, 0x17, 0x00, \
  0xc6, 0xca, 0xea, 0x70, 0xfe, 0xc8, \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x33, 0x26, 0x64, 0xcc, 0x90, 0x98, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0x48, \
  0xa4, 0x34, 0x86, 0x90, 0x81, 0x28, \
  0x50, 0xaa, 0x15, 0x42, 0x83, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x16, 0x00, \
  0x09, 0x61, 0x2c, 0x25, 0xc4, 0x30, \
  0x0c, 0x81, 0x90, 0x32, 0x10, 0x30, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x21, 0x20, \
  0x05, 0x40, 0xa8, 0x15, 0x00, 0xc8, \
  0x43, 0x08, 0x61, 0x0c, 0x0a, 0x08, \
  0x1a, 0x03, 0x40, 0x68, 0x05, 0x40, \
  0x60, 0x2c, 0x05, 0x80, 0x9c, 0x00, \
  0x14, 0x22, 0x84, 0x50, 0xe2, 0x80

#define kMaskRandom45_3 \
  0xca, 0xd9, 0x5b, 0x2b, 0x4d, 0x90, \
  0xf1, 0x5e, 0x2b, 0xc5, 0x24, 0xe8, \
  0xb6, 0x35, 0xc5, 0xd8, 0x9f, 0x40

#define kMaskRandom45_30 \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x33, 0x26, 0x64, 0xcc, 0x90, 0x98, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0x48, \
  0xa4, 0x34, 0x86, 0x90, 0x81, 0x28, \
  0x50, 0xaa, 0x15, 0x42, 0x83, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x16, 0x00, \
  0x09, 0x61, 0x2c, 0x25, 0xc4, 0x30, \
  0x0c, 0x81, 0x90, 0x32, 0x10, 0x30, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x21, 0x20, \
  0x05, 0x40, 0xa8, 0x15, 0x00, 0xc8, \
  0x43, 0x08, 0x61, 0x0c, 0x0a, 0x08, \
  0x1a, 0x03, 0x40, 0x68, 0x05, 0x40, \
  0x60, 0x2c, 0x05, 0x80, 0x9c, 0x00, \
  0x14, 0x22, 0x84, 0x50, 0xe2, 0x80, \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x33, 0x26, 0x64, 0xcc, 0x90, 0x98, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0x48, \
  0xa4, 0x34, 0x86, 0x90, 0x81, 0x28, \
  0x50, 0xaa, 0x15, 0x42, 0x83, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x16, 0x00, \
  0x09, 0x61, 0x2c, 0x25, 0xc4, 0x30, \
  0x64, 0x4c, 0x89, 0x91, 0x28, 0xa0, \
  0x51, 0x4a, 0x29, 0x45, 0x62, 0x88, \
  0x0c, 0xa1, 0x94, 0x32, 0x90, 0xc0, \
  0xa1, 0x34, 0x26, 0x84, 0x89, 0x18, \
  0x12, 0xa2, 0x54, 0x4a, 0x84, 0x70, \
  0x8a, 0x51, 0x4a, 0x29, 0x17, 0x00, \
  0xc6, 0xca, 0xea, 0x70, 0xfe, 0xc8, \
  0x1c, 0xc9, 0x43, 0x25, 0xa7, 0x00

#define kMaskRandom45_31 \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x33, 0x26, 0x64, 0xcc, 0x90, 0x98, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0x48, \
  0xa4, 0x34, 0x86, 0x90, 0x81, 0x28, \
  0x50, 0xaa, 0x15, 0x42, 0x83, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x16, 0x00, \
  0x09, 0x61, 0x2c, 0x25, 0xc4, 0x30, \
  0x0c, 0x81, 0x90, 0x32, 0x10, 0x30, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x21, 0x20, \
  0x05, 0x40, 0xa8, 0x15, 0x00, 0xc8, \
  0x43, 0x08, 0x61, 0x0c, 0x0a, 0x08, \
  0x1a, 0x03, 0x40, 0x68, 0x05, 0x40, \
  0x60, 0x2c, 0x05, 0x80, 0x9c, 0x00, \
  0x14, 0x22, 0x84, 0x50, 0xe2, 0x80, \
  0x0c, 0x81, 0x90, 0x32, 0x10, 0x30, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x21, 0x20, \
  0x05, 0x40, 0xa8, 0x15, 0x00, 0xc8, \
  0x43, 0x08, 0x61, 0x0c, 0x0a, 0x08, \
  0x1a, 0x03, 0x40, 0x68, 0x05, 0x40, \
  0x60, 0x2c, 0x05, 0x80, 0x9c, 0x00, \
  0x14, 0x22, 0x84, 0x50, 0xe2, 0x80, \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x33, 0x26, 0x64, 0xcc, 0x90, 0x98, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0x48, \
  0xa4, 0x34, 0x86, 0x90, 0x81, 0x28, \
  0x50, 0xaa, 0x15, 0x42, 0x83, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x16, 0x00, \
  0x09, 0x61, 0x2c, 0x25, 0xc4, 0x30, \
  0x86, 0xc1, 0x47, 0xeb, 0x67, 0xd0

#define kMaskRandom45_32 \
  0x0c, 0x81, 0x90, 0x32, 0x10, 0x30, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x21, 0x20, \
  0x05, 0x40, 0xa8, 0x15, 0x00, 0xc8, \
  0x43, 0x08, 0x61, 0x0c, 0x0a, 0x08, \
  0x1a, 0x03, 0x40, 0x68, 0x05, 0x40, \
  0x60, 0x2c, 0x05, 0x80, 0x9c, 0x00, \
  0x14, 0x22, 0x84, 0x50, 0xe2, 0x80, \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x33, 0x26, 0x64, 0xcc, 0x90, 0x98, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0x48, \
  0xa4, 0x34, 0x86, 0x90, 0x81, 0x28, \
  0x50, 0xaa, 0x15, 0x42, 0x83, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x16, 0x00, \
  0x09, 0x61, 0x2c, 0x25, 0xc4, 0x30, \
  0x86, 0xc1, 0x47, 0xeb, 0x67, 0xd0, \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x33, 0x26, 0x64, 0xcc, 0x90, 0x98, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0x48, \
  0xa4, 0x34, 0x86, 0x90, 0x81, 0x28, \
  0x50, 0xaa, 0x15, 0x42, 0x83, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x16, 0x00, \
  0x09, 0x61, 0x2c, 0x25, 0xc4, 0x30, \
  0x0c, 0x81, 0x90, 0x32, 0x10, 0x30, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x21, 0x20, \
  0x05, 0x40, 0xa8, 0x15, 0x00, 0xc8, \
  0x43, 0x08, 0x61, 0x0c, 0x0a, 0x08, \
  0x1a, 0x03, 0x40, 0x68, 0x05, 0x40, \
  0x60, 0x2c, 0x05, 0x80, 0x9c, 0x00, \
  0x14, 0x22, 0x84, 0x50, 0xe2, 0x80, \
  0x40, 0x7e, 0xc1, 0x30, 0x29, 0x50

#define kMaskRandom45_33 \
  0x0c, 0x81, 0x90, 0x32, 0x10, 0x30, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x21, 0x20, \
  0x05, 0x40, 0xa8, 0x15, 0x00, 0xc8, \
  0x43, 0x08, 0x61, 0x0c, 0x0a, 0x08, \
  0x1a, 0x03, 0x40, 0x68, 0x05, 0x40, \
  0x60, 0x2c, 0x05, 0x80, 0x9c, 0x00, \
  0x14, 0x22, 0x84, 0x50, 0xe2, 0x80, \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x33, 0x26, 0x64, 0xcc, 0x90, 0x98, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0x48, \
  0xa4, 0x34, 0x86, 0x90, 0x81, 0x28, \
  0x50, 0xaa, 0x15, 0x42, 0x83, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x16, 0x00, \
  0x09, 0x61, 0x2c, 0x25, 0xc4, 0x30, \
  0x86, 0xc1, 0x47, 0xeb, 0x67, 0xd0, \
  0x0c, 0x81, 0x90, 0x32, 0x10, 0x30, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x21, 0x20, \
  0x05, 0x40, 0xa8, 0x15, 0x00, 0xc8, \
  0x43, 0x08, 0x61, 0x0c, 0x0a, 0x08, \
  0x1a, 0x03, 0x40, 0x68, 0x05, 0x40, \
  0x60, 0x2c, 0x05, 0x80, 0x9c, 0x00, \
  0x14, 0x22, 0x84, 0x50, 0xe2, 0x80, \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x62, 0x6c, 0x4d, 0x89, 0xf2, 0x10, \
  0x8c, 0x11, 0x82, 0x30, 0x12, 0x20, \
  0x01, 0x60, 0x2c, 0x05, 0xd0, 0x88, \
  0x07, 0x80, 0xf0, 0x1e, 0x0c, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x01, 0xc8, \
  0x18, 0xa3, 0x14, 0x62, 0xc5, 0x08, \
  0x91, 0x12, 0x22, 0x44, 0x02, 0x48, \
  0x78, 0x0f, 0x01, 0xe0, 0x00, 0x70

#define kMaskRandom45_34 \
  0x0c, 0x81, 0x90, 0x32, 0x10, 0x30, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x21, 0x20, \
  0x05, 0x40, 0xa8, 0x15, 0x00, 0xc8, \
  0x43, 0x08, 0x61, 0x0c, 0x0a, 0x08, \
  0x1a, 0x03, 0x40, 0x68, 0x05, 0x40, \
  0x60, 0x2c, 0x05, 0x80, 0x9c, 0x00, \
  0x14, 0x22, 0x84, 0x50, 0xe2, 0x80, \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x62, 0x6c, 0x4d, 0x89, 0xf2, 0x10, \
  0x8c, 0x11, 0x82, 0x30, 0x12, 0x20, \
  0x01, 0x60, 0x2c, 0x05, 0xd0, 0x88, \
  0x07, 0x80, 0xf0, 0x1e, 0x0c, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x01, 0xc8, \
  0x18, 0xa3, 0x14, 0x62, 0xc5, 0x08, \
  0x91, 0x12, 0x22, 0x44, 0x02, 0x48, \
  0x78, 0x0f, 0x01, 0xe0, 0x00, 0x70, \
  0x0c, 0x81, 0x90, 0x32, 0x10, 0x30, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x21, 0x20, \
  0x05, 0x40, 0xa8, 0x15, 0x00, 0xc8, \
  0x43, 0x08, 0x61, 0x0c, 0x0a, 0x08, \
  0x1a, 0x03, 0x40, 0x68, 0x05, 0x40, \
  0x60, 0x2c, 0x05, 0x80, 0x9c, 0x00, \
  0x14, 0x22, 0x84, 0x50, 0xe2, 0x80, \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x33, 0x26, 0x64, 0xcc, 0x90, 0x98, \
  0x91, 0x92, 0x32, 0x46, 0x48, 0x48, \
  0xa4, 0x34, 0x86, 0x90, 0x81, 0x28, \
  0x50, 0xaa, 0x15, 0x42, 0x83, 0x50, \
  0x84, 0xd0, 0x9a, 0x13, 0x16, 0x00, \
  0x09, 0x61, 0x2c, 0x25, 0xc4, 0x30, \
  0x86, 0xc1, 0x47, 0xeb, 0x67, 0xd0, \
  0x1f, 0x78, 0x45, 0x5e, 0x46, 0x50

#define kMaskRandom45_35 \
  0x0c, 0x81, 0x90, 0x32, 0x10, 0x30, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x21, 0x20, \
  0x05, 0x40, 0xa8, 0x15, 0x00, 0xc8, \
  0x43, 0x08, 0x61, 0x0c, 0x0a, 0x08, \
  0x1a, 0x03, 0x40, 0x68, 0x05, 0x40, \
  0x60, 0x2c, 0x05, 0x80, 0x9c, 0x00, \
  0x14, 0x22, 0x84, 0x50, 0xe2, 0x80, \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x62, 0x6c, 0x4d, 0x89, 0xf2, 0x10, \
  0x8c, 0x11, 0x82, 0x30, 0x12, 0x20, \
  0x01, 0x60, 0x2c, 0x05, 0xd0, 0x88, \
  0x07, 0x80, 0xf0, 0x1e, 0x0c, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x01, 0xc8, \
  0x18, 0xa3, 0x14, 0x62, 0xc5, 0x08, \
  0x91, 0x12, 0x22, 0x44, 0x02, 0x48, \
  0x78, 0x0f, 0x01, 0xe0, 0x00, 0x70, \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x62, 0x6c, 0x4d, 0x89, 0xf2, 0x10, \
  0x8c, 0x11, 0x82, 0x30, 0x12, 0x20, \
  0x01, 0x60, 0x2c, 0x05, 0xd0, 0x88, \
  0x07, 0x80, 0xf0, 0x1e, 0x0c, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x01, 0xc8, \
  0x18, 0xa3, 0x14, 0x62, 0xc5, 0x08, \
  0x91, 0x12, 0x22, 0x44, 0x02, 0x48, \
  0x78, 0x0f, 0x01, 0xe0, 0x00, 0x70, \
  0x0c, 0x81, 0x90, 0x32, 0x10, 0x30, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x21, 0x20, \
  0x05, 0x40, 0xa8, 0x15, 0x00, 0xc8, \
  0x43, 0x08, 0x61, 0x0c, 0x0a, 0x08, \
  0x1a, 0x03, 0x40, 0x68, 0x05, 0x40, \
  0x60, 0x2c, 0x05, 0x80, 0x9c, 0x00, \
  0x14, 0x22, 0x84, 0x50, 0xe2, 0x80, \
  0xe4, 0xd4, 0x6e, 0x08, 0xc9, 0x58

#define kMaskRandom45_36 \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x62, 0x6c, 0x4d, 0x89, 0xf2, 0x10, \
  0x8c, 0x11, 0x82, 0x30, 0x12, 0x20, \
  0x01, 0x60, 0x2c, 0x05, 0xd0, 0x88, \
  0x07, 0x80, 0xf0, 0x1e, 0x0c, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x01, 0xc8, \
  0x18, 0xa3, 0x14, 0x62, 0xc5, 0x08, \
  0x91, 0x12, 0x22, 0x44, 0x02, 0x48, \
  0x78, 0x0f, 0x01, 0xe0, 0x00, 0x70, \
  0x0c, 0x81, 0x90, 0x32, 0x10, 0x30, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x21, 0x20, \
  0x05, 0x40, 0xa8, 0x15, 0x00, 0xc8, \
  0x43, 0x08, 0x61, 0x0c, 0x0a, 0x08, \
  0x1a, 0x03, 0x40, 0x68, 0x05, 0x40, \
  0x60, 0x2c, 0x05, 0x80, 0x9c, 0x00, \
  0x14, 0x22, 0x84, 0x50, 0xe2, 0x80, \
  0xe4, 0xd4, 0x6e, 0x08, 0xc9, 0x58, \
  0x0c, 0x81, 0x90, 0x32, 0x10, 0x30, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x21, 0x20, \
  0x05, 0x40, 0xa8, 0x15, 0x00, 0xc8, \
  0x43, 0x08, 0x61, 0x0c, 0x0a, 0x08, \
  0x1a, 0x03, 0x40, 0x68, 0x05, 0x40, \
  0x60, 0x2c, 0x05, 0x80, 0x9c, 0x00, \
  0x14, 0x22, 0x84, 0x50, 0xe2, 0x80, \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x62, 0x6c, 0x4d, 0x89, 0xf2, 0x10, \
  0x8c, 0x11, 0x82, 0x30, 0x12, 0x20, \
  0x01, 0x60, 0x2c, 0x05, 0xd0, 0x88, \
  0x07, 0x80, 0xf0, 0x1e, 0x0c, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x01, 0xc8, \
  0x18, 0xa3, 0x14, 0x62, 0xc5, 0x08, \
  0x91, 0x12, 0x22, 0x44, 0x02, 0x48, \
  0x78, 0x0f, 0x01, 0xe0, 0x00, 0x70, \
  0xd0, 0x1a, 0xf0, 0x14, 0xf0, 0xe8

#define kMaskRandom45_37 \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x62, 0x6c, 0x4d, 0x89, 0xf2, 0x10, \
  0x8c, 0x11, 0x82, 0x30, 0x12, 0x20, \
  0x01, 0x60, 0x2c, 0x05, 0xd0, 0x88, \
  0x07, 0x80, 0xf0, 0x1e, 0x0c, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x01, 0xc8, \
  0x18, 0xa3, 0x14, 0x62, 0xc5, 0x08, \
  0x91, 0x12, 0x22, 0x44, 0x02, 0x48, \
  0x78, 0x0f, 0x01, 0xe0, 0x00, 0x70, \
  0x0c, 0x81, 0x90, 0x32, 0x10, 0x30, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x21, 0x20, \
  0x05, 0x40, 0xa8, 0x15, 0x00, 0xc8, \
  0x43, 0x08, 0x61, 0x0c, 0x0a, 0x08, \
  0x1a, 0x03, 0x40, 0x68, 0x05, 0x40, \
  0x60, 0x2c, 0x05, 0x80, 0x9c, 0x00, \
  0x14, 0x22, 0x84, 0x50, 0xe2, 0x80, \
  0xe4, 0xd4, 0x6e, 0x08, 0xc9, 0x58, \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x62, 0x6c, 0x4d, 0x89, 0xf2, 0x10, \
  0x8c, 0x11, 0x82, 0x30, 0x12, 0x20, \
  0x01, 0x60, 0x2c, 0x05, 0xd0, 0x88, \
  0x07, 0x80, 0xf0, 0x1e, 0x0c, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x01, 0xc8, \
  0x18, 0xa3, 0x14, 0x62, 0xc5, 0x08, \
  0x91, 0x12, 0x22, 0x44, 0x02, 0x48, \
  0x78, 0x0f, 0x01, 0xe0, 0x00, 0x70, \
  0x64, 0x4c, 0x89, 0x91, 0x28, 0xa0, \
  0x51, 0x4a, 0x29, 0x45, 0x22, 0x88, \
  0xa9, 0x15, 0x22, 0xa4, 0x40, 0x68, \
  0x04, 0xc0, 0x98, 0x13, 0x12, 0x10, \
  0xd0, 0x1a, 0x03, 0x40, 0x05, 0x10, \
  0x82, 0x50, 0x4a, 0x09, 0x00, 0x70, \
  0x21, 0x24, 0x24, 0x84, 0xdc, 0x00, \
  0x0c, 0x21, 0x84, 0x30, 0x84, 0x88, \
  0x4a, 0x09, 0x41, 0x28, 0x2b, 0x00, \
  0x12, 0xa2, 0x54, 0x4a, 0xd1, 0x40

#define kMaskRandom45_38 \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x62, 0x6c, 0x4d, 0x89, 0xf2, 0x10, \
  0x8c, 0x11, 0x82, 0x30, 0x12, 0x20, \
  0x01, 0x60, 0x2c, 0x05, 0xd0, 0x88, \
  0x07, 0x80, 0xf0, 0x1e, 0x0c, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x01, 0xc8, \
  0x18, 0xa3, 0x14, 0x62, 0xc5, 0x08, \
  0x91, 0x12, 0x22, 0x44, 0x02, 0x48, \
  0x78, 0x0f, 0x01, 0xe0, 0x00, 0x70, \
  0x64, 0x4c, 0x89, 0x91, 0x28, 0xa0, \
  0x51, 0x4a, 0x29, 0x45, 0x22, 0x88, \
  0xa9, 0x15, 0x22, 0xa4, 0x40, 0x68, \
  0x04, 0xc0, 0x98, 0x13, 0x12, 0x10, \
  0xd0, 0x1a, 0x03, 0x40, 0x05, 0x10, \
  0x82, 0x50, 0x4a, 0x09, 0x00, 0x70, \
  0x21, 0x24, 0x24, 0x84, 0xdc, 0x00, \
  0x0c, 0x21, 0x84, 0x30, 0x84, 0x88, \
  0x4a, 0x09, 0x41, 0x28, 0x2b, 0x00, \
  0x12, 0xa2, 0x54, 0x4a, 0xd1, 0x40, \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x62, 0x6c, 0x4d, 0x89, 0xf2, 0x10, \
  0x8c, 0x11, 0x82, 0x30, 0x12, 0x20, \
  0x01, 0x60, 0x2c, 0x05, 0xd0, 0x88, \
  0x07, 0x80, 0xf0, 0x1e, 0x0c, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x01, 0xc8, \
  0x18, 0xa3, 0x14, 0x62, 0xc5, 0x08, \
  0x91, 0x12, 0x22, 0x44, 0x02, 0x48, \
  0x78, 0x0f, 0x01, 0xe0, 0x00, 0x70, \
  0x0c, 0x81, 0x90, 0x32, 0x10, 0x30, \
  0x80, 0x70, 0x0e, 0x01, 0xc0, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x21, 0x20, \
  0x05, 0x40, 0xa8, 0x15, 0x00, 0xc8, \
  0x43, 0x08, 0x61, 0x0c, 0x0a, 0x08, \
  0x1a, 0x03, 0x40, 0x68, 0x05, 0x40, \
  0x60, 0x2c, 0x05, 0x80, 0x9c, 0x00, \
  0x14, 0x22, 0x84, 0x50, 0xe2, 0x80, \
  0xe4, 0xd4, 0x6e, 0x08, 0xc9, 0x58, \
  0x04, 0x67, 0x1b, 0xba, 0x1d, 0xa0

#define kMaskRandom45_39 \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x62, 0x6c, 0x4d, 0x89, 0xf2, 0x10, \
  0x8c, 0x11, 0x82, 0x30, 0x12, 0x20, \
  0x01, 0x60, 0x2c, 0x05, 0xd0, 0x88, \
  0x07, 0x80, 0xf0, 0x1e, 0x0c, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x01, 0xc8, \
  0x18, 0xa3, 0x14, 0x62, 0xc5, 0x08, \
  0x91, 0x12, 0x22, 0x44, 0x02, 0x48, \
  0x78, 0x0f, 0x01, 0xe0, 0x00, 0x70, \
  0x64, 0x4c, 0x89, 0x91, 0x28, 0xa0, \
  0x51, 0x4a, 0x29, 0x45, 0x22, 0x88, \
  0xa9, 0x15, 0x22, 0xa4, 0x40, 0x68, \
  0x04, 0xc0, 0x98, 0x13, 0x12, 0x10, \
  0xd0, 0x1a, 0x03, 0x40, 0x05, 0x10, \
  0x82, 0x50, 0x4a, 0x09, 0x00, 0x70, \
  0x21, 0x24, 0x24, 0x84, 0xdc, 0x00, \
  0x0c, 0x21, 0x84, 0x30, 0x84, 0x88, \
  0x4a, 0x09, 0x41, 0x28, 0x2b, 0x00, \
  0x12, 0xa2, 0x54, 0x4a, 0xd1, 0x40, \
  0x64, 0x4c, 0x89, 0x91, 0x28, 0xa0, \
  0x51, 0x4a, 0x29, 0x45, 0x22, 0x88, \
  0xa9, 0x15, 0x22, 0xa4, 0x40, 0x68, \
  0x04, 0xc0, 0x98, 0x13, 0x12, 0x10, \
  0xd0, 0x1a, 0x03, 0x40, 0x05, 0x10, \
  0x82, 0x50, 0x4a, 0x09, 0x00, 0x70, \
  0x21, 0x24, 0x24, 0x84, 0xdc, 0x00, \
  0x0c, 0x21, 0x84, 0x30, 0x84, 0x88, \
  0x4a, 0x09, 0x41, 0x28, 0x2b, 0x00, \
  0x12, 0xa2, 0x54, 0x4a, 0xd1, 0x40, \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x62, 0x6c, 0x4d, 0x89, 0xf2, 0x10, \
  0x8c, 0x11, 0x82, 0x30, 0x12, 0x20, \
  0x01, 0x60, 0x2c, 0x05, 0xd0, 0x88, \
  0x07, 0x80, 0xf0, 0x1e, 0x0c, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x01, 0xc8, \
  0x18, 0xa3, 0x14, 0x62, 0xc5, 0x08, \
  0x91, 0x12, 0x22, 0x44, 0x02, 0x48, \
  0x78, 0x0f, 0x01, 0xe0, 0x00, 0x70, \
  0x3b, 0x48, 0xc7, 0x6d, 0x29, 0xe8

#define kMaskRandom45_4 \
  0xc4, 0xd8, 0x9b, 0x13, 0x45, 0x90, \
  0x31, 0x66, 0x2c, 0xc5, 0x8a, 0x58, \
  0x4b, 0x29, 0x65, 0x2c, 0x91, 0x68, \
  0x2c, 0xa5, 0x94, 0xb2, 0xa2, 0xa8

#define kMaskRandom45_40 \
  0x64, 0x4c, 0x89, 0x91, 0x28, 0xa0, \
  0x51, 0x4a, 0x29, 0x45, 0x22, 0x88, \
  0xa9, 0x15, 0x22, 0xa4, 0x40, 0x68, \
  0x04, 0xc0, 0x98, 0x13, 0x12, 0x10, \
  0xd0, 0x1a, 0x03, 0x40, 0x05, 0x10, \
  0x82, 0x50, 0x4a, 0x09, 0x00, 0x70, \
  0x21, 0x24, 0x24, 0x84, 0xdc, 0x00, \
  0x0c, 0x21, 0x84, 0x30, 0x84, 0x88, \
  0x4a, 0x09, 0x41, 0x28, 0x2b, 0x00, \
  0x12, 0xa2, 0x54, 0x4a, 0xd1, 0x40, \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x62, 0x6c, 0x4d, 0x89, 0xf2, 0x10, \
  0x8c, 0x11, 0x82, 0x30, 0x12, 0x20, \
  0x01, 0x60, 0x2c, 0x05, 0xd0, 0x88, \
  0x07, 0x80, 0xf0, 0x1e, 0x0c, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x01, 0xc8, \
  0x18, 0xa3, 0x14, 0x62, 0xc5, 0x08, \
  0x91, 0x12, 0x22, 0x44, 0x02, 0x48, \
  0x78, 0x0f, 0x01, 0xe0, 0x00, 0x70, \
  0x3b, 0x48, 0xc7, 0x6d, 0x29, 0xe8, \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x62, 0x6c, 0x4d, 0x89, 0xf2, 0x10, \
  0x8c, 0x11, 0x82, 0x30, 0x12, 0x20, \
  0x01, 0x60, 0x2c, 0x05, 0xd0, 0x88, \
  0x07, 0x80, 0xf0, 0x1e, 0x0c, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x01, 0xc8, \
  0x18, 0xa3, 0x14, 0x62, 0xc5, 0x08, \
  0x91, 0x12, 0x22, 0x44, 0x02, 0x48, \
  0x78, 0x0f, 0x01, 0xe0, 0x00, 0x70, \
  0x64, 0x4c, 0x89, 0x91, 0x28, 0xa0, \
  0x51, 0x4a, 0x29, 0x45, 0x22, 0x88, \
  0xa9, 0x15, 0x22, 0xa4, 0x40, 0x68, \
  0x04, 0xc0, 0x98, 0x13, 0x12, 0x10, \
  0xd0, 0x1a, 0x03, 0x40, 0x05, 0x10, \
  0x82, 0x50, 0x4a, 0x09, 0x00, 0x70, \
  0x21, 0x24, 0x24, 0x84, 0xdc, 0x00, \
  0x0c, 0x21, 0x84, 0x30, 0x84, 0x88, \
  0x4a, 0x09, 0x41, 0x28, 0x2b, 0x00, \
  0x12, 0xa2, 0x54, 0x4a, 0xd1, 0x40, \
  0xd9, 0x40, 0x46, 0xe6, 0x4f, 0xd8

#define kMaskRandom45_41 \
  0x64, 0x4c, 0x89, 0x91, 0x28, 0xa0, \
  0x51, 0x4a, 0x29, 0x45, 0x22, 0x88, \
  0xa9, 0x15, 0x22, 0xa4, 0x40, 0x68, \
  0x04, 0xc0, 0x98, 0x13, 0x12, 0x10, \
  0xd0, 0x1a, 0x03, 0x40, 0x05, 0x10, \
  0x82, 0x50, 0x4a, 0x09, 0x00, 0x70, \
  0x21, 0x24, 0x24, 0x84, 0xdc, 0x00, \
  0x0c, 0x21, 0x84, 0x30, 0x84, 0x88, \
  0x4a, 0x09, 0x41, 0x28, 0x2b, 0x00, \
  0x12, 0xa2, 0x54, 0x4a, 0xd1, 0x40, \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x62, 0x6c, 0x4d, 0x89, 0xf2, 0x10, \
  0x8c, 0x11, 0x82, 0x30, 0x12, 0x20, \
  0x01, 0x60, 0x2c, 0x05, 0xd0, 0x88, \
  0x07, 0x80, 0xf0, 0x1e, 0x0c, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x01, 0xc8, \
  0x18, 0xa3, 0x14, 0x62, 0xc5, 0x08, \
  0x91, 0x12, 0x22, 0x44, 0x02, 0x48, \
  0x78, 0x0f, 0x01, 0xe0, 0x00, 0x70, \
  0x3b, 0x48, 0xc7, 0x6d, 0x29, 0xe8, \
  0x64, 0x4c, 0x89, 0x91, 0x28, 0xa0, \
  0x51, 0x4a, 0x29, 0x45, 0x22, 0x88, \
  0xa9, 0x15, 0x22, 0xa4, 0x40, 0x68, \
  0x04, 0xc0, 0x98, 0x13, 0x12, 0x10, \
  0xd0, 0x1a, 0x03, 0x40, 0x05, 0x10, \
  0x82, 0x50, 0x4a, 0x09, 0x00, 0x70, \
  0x21, 0x24, 0x24, 0x84, 0xdc, 0x00, \
  0x0c, 0x21, 0x84, 0x30, 0x84, 0x88, \
  0x4a, 0x09, 0x41, 0x28, 0x2b, 0x00, \
  0x12, 0xa2, 0x54, 0x4a, 0xd1, 0x40, \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x33, 0x26, 0x64, 0xcc, 0x90, 0x98, \
  0x99, 0x13, 0x22, 0x64, 0x08, 0x48, \
  0x05, 0x80, 0xb0, 0x16, 0x00, 0x38, \
  0x80, 0xb0, 0x16, 0x02, 0x86, 0x08, \
  0x84, 0x50, 0x8a, 0x11, 0x20, 0x60, \
  0x40, 0x68, 0x0d, 0x01, 0xb5, 0x00, \
  0x0a, 0x81, 0x50, 0x2a, 0x43, 0x00, \
  0x68, 0x0d, 0x01, 0xa0, 0x12, 0x40, \
  0x10, 0x22, 0x04, 0x40, 0xc4, 0x80, \
  0x30, 0x46, 0x08, 0xc1, 0x60, 0x10

#define kMaskRandom45_42 \
  0x64, 0x4c, 0x89, 0x91, 0x28, 0xa0, \
  0x51, 0x4a, 0x29, 0x45, 0x22, 0x88, \
  0xa9, 0x15, 0x22, 0xa4, 0x40, 0x68, \
  0x04, 0xc0, 0x98, 0x13, 0x12, 0x10, \
  0xd0, 0x1a, 0x03, 0x40, 0x05, 0x10, \
  0x82, 0x50, 0x4a, 0x09, 0x00, 0x70, \
  0x21, 0x24, 0x24, 0x84, 0xdc, 0x00, \
  0x0c, 0x21, 0x84, 0x30, 0x84, 0x88, \
  0x4a, 0x09, 0x41, 0x28, 0x2b, 0x00, \
  0x12, 0xa2, 0x54, 0x4a, 0xd1, 0x40, \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x33, 0x26, 0x64, 0xcc, 0x90, 0x98, \
  0x99, 0x13, 0x22, 0x64, 0x08, 0x48, \
  0x05, 0x80, 0xb0, 0x16, 0x00, 0x38, \
  0x80, 0xb0, 0x16, 0x02, 0x86, 0x08, \
  0x84, 0x50, 0x8a, 0x11, 0x20, 0x60, \
  0x40, 0x68, 0x0d, 0x01, 0xb5, 0x00, \
  0x0a, 0x81, 0x50, 0x2a, 0x43, 0x00, \
  0x68, 0x0d, 0x01, 0xa0, 0x12, 0x40, \
  0x10, 0x22, 0x04, 0x40, 0xc4, 0x80, \
  0x30, 0x46, 0x08, 0xc1, 0x60, 0x10, \
  0x64, 0x4c, 0x89, 0x91, 0x28, 0xa0, \
  0x51, 0x4a, 0x29, 0x45, 0x22, 0x88, \
  0xa9, 0x15, 0x22, 0xa4, 0x40, 0x68, \
  0x04, 0xc0, 0x98, 0x13, 0x12, 0x10, \
  0xd0, 0x1a, 0x03, 0x40, 0x05, 0x10, \
  0x82, 0x50, 0x4a, 0x09, 0x00, 0x70, \
  0x21, 0x24, 0x24, 0x84, 0xdc, 0x00, \
  0x0c, 0x21, 0x84, 0x30, 0x84, 0x88, \
  0x4a, 0x09, 0x41, 0x28, 0x2b, 0x00, \
  0x12, 0xa2, 0x54, 0x4a, 0xd1, 0x40, \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x62, 0x6c, 0x4d, 0x89, 0xf2, 0x10, \
  0x8c, 0x11, 0x82, 0x30, 0x12, 0x20, \
  0x01, 0x60, 0x2c, 0x05, 0xd0, 0x88, \
  0x07, 0x80, 0xf0, 0x1e, 0x0c, 0x18, \
  0xa0, 0x94, 0x12, 0x82, 0x01, 0xc8, \
  0x18, 0xa3, 0x14, 0x62, 0xc5, 0x08, \
  0x91, 0x12, 0x22, 0x44, 0x02, 0x48, \
  0x78, 0x0f, 0x01, 0xe0, 0x00, 0x70, \
  0x3b, 0x48, 0xc7, 0x6d, 0x29, 0xe8, \
  0xac, 0xcc, 0x04, 0x41, 0x97, 0x30

#define kMaskRandom45_43 \
  0x64, 0x4c, 0x89, 0x91, 0x28, 0xa0, \
  0x51, 0x4a, 0x29, 0x45, 0x22, 0x88, \
  0xa9, 0x15, 0x22, 0xa4, 0x40, 0x68, \
  0x04, 0xc0, 0x98, 0x13, 0x12, 0x10, \
  0xd0, 0x1a, 0x03, 0x40, 0x05, 0x10, \
  0x82, 0x50, 0x4a, 0x09, 0x00, 0x70, \
  0x21, 0x24, 0x24, 0x84, 0xdc, 0x00, \
  0x0c, 0x21, 0x84, 0x30, 0x84, 0x88, \
  0x4a, 0x09, 0x41, 0x28, 0x2b, 0x00, \
  0x12, 0xa2, 0x54, 0x4a, 0xd1, 0x40, \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x33, 0x26, 0x64, 0xcc, 0x90, 0x98, \
  0x99, 0x13, 0x22, 0x64, 0x08, 0x48, \
  0x05, 0x80, 0xb0, 0x16, 0x00, 0x38, \
  0x80, 0xb0, 0x16, 0x02, 0x86, 0x08, \
  0x84, 0x50, 0x8a, 0x11, 0x20, 0x60, \
  0x40, 0x68, 0x0d, 0x01, 0xb5, 0x00, \
  0x0a, 0x81, 0x50, 0x2a, 0x43, 0x00, \
  0x68, 0x0d, 0x01, 0xa0, 0x12, 0x40, \
  0x10, 0x22, 0x04, 0x40, 0xc4, 0x80, \
  0x30, 0x46, 0x08, 0xc1, 0x60, 0x10, \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x33, 0x26, 0x64, 0xcc, 0x90, 0x98, \
  0x99, 0x13, 0x22, 0x64, 0x08, 0x48, \
  0x05, 0x80, 0xb0, 0x16, 0x00, 0x38, \
  0x80, 0xb0, 0x16, 0x02, 0x86, 0x08, \
  0x84, 0x50, 0x8a, 0x11, 0x20, 0x60, \
  0x40, 0x68, 0x0d, 0x01, 0xb5, 0x00, \
  0x0a, 0x81, 0x50, 0x2a, 0x43, 0x00, \
  0x68, 0x0d, 0x01, 0xa0, 0x12, 0x40, \
  0x10, 0x22, 0x04, 0x40, 0xc4, 0x80, \
  0x30, 0x46, 0x08, 0xc1, 0x60, 0x10, \
  0x64, 0x4c, 0x89, 0x91, 0x28, 0xa0, \
  0x51, 0x4a, 0x29, 0x45, 0x22, 0x88, \
  0xa9, 0x15, 0x22, 0xa4, 0x40, 0x68, \
  0x04, 0xc0, 0x98, 0x13, 0x12, 0x10, \
  0xd0, 0x1a, 0x03, 0x40, 0x05, 0x10, \
  0x82, 0x50, 0x4a, 0x09, 0x00, 0x70, \
  0x21, 0x24, 0x24, 0x84, 0xdc, 0x00, \
  0x0c, 0x21, 0x84, 0x30, 0x84, 0x88, \
  0x4a, 0x09, 0x41, 0x28, 0x2b, 0x00, \
  0x12, 0xa2, 0x54, 0x4a, 0xd1, 0x40, \
  0x9e, 0xce, 0x8b, 0xaa, 0x34, 0x68

#define kMaskRandom45_44 \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x33, 0x26, 0x64, 0xcc, 0x90, 0x98, \
  0x99, 0x13, 0x22, 0x64, 0x08, 0x48, \
  0x05, 0x80, 0xb0, 0x16, 0x00, 0x38, \
  0x80, 0xb0, 0x16, 0x02, 0x86, 0x08, \
  0x84, 0x50, 0x8a, 0x11, 0x20, 0x60, \
  0x40, 0x68, 0x0d, 0x01, 0xb5, 0x00, \
  0x0a, 0x81, 0x50, 0x2a, 0x43, 0x00, \
  0x68, 0x0d, 0x01, 0xa0, 0x12, 0x40, \
  0x10, 0x22, 0x04, 0x40, 0xc4, 0x80, \
  0x30, 0x46, 0x08, 0xc1, 0x60, 0x10, \
  0x64, 0x4c, 0x89, 0x91, 0x28, 0xa0, \
  0x51, 0x4a, 0x29, 0x45, 0x22, 0x88, \
  0xa9, 0x15, 0x22, 0xa4, 0x40, 0x68, \
  0x04, 0xc0, 0x98, 0x13, 0x12, 0x10, \
  0xd0, 0x1a, 0x03, 0x40, 0x05, 0x10, \
  0x82, 0x50, 0x4a, 0x09, 0x00, 0x70, \
  0x21, 0x24, 0x24, 0x84, 0xdc, 0x00, \
  0x0c, 0x21, 0x84, 0x30, 0x84, 0x88, \
  0x4a, 0x09, 0x41, 0x28, 0x2b, 0x00, \
  0x12, 0xa2, 0x54, 0x4a, 0xd1, 0x40, \
  0x9e, 0xce, 0x8b, 0xaa, 0x34, 0x68, \
  0x64, 0x4c, 0x89, 0x91, 0x28, 0xa0, \
  0x51, 0x4a, 0x29, 0x45, 0x22, 0x88, \
  0xa9, 0x15, 0x22, 0xa4, 0x40, 0x68, \
  0x04, 0xc0, 0x98, 0x13, 0x12, 0x10, \
  0xd0, 0x1a, 0x03, 0x40, 0x05, 0x10, \
  0x82, 0x50, 0x4a, 0x09, 0x00, 0x70, \
  0x21, 0x24, 0x24, 0x84, 0xdc, 0x00, \
  0x0c, 0x21, 0x84, 0x30, 0x84, 0x88, \
  0x4a, 0x09, 0x41, 0x28, 0x2b, 0x00, \
  0x12, 0xa2, 0x54, 0x4a, 0xd1, 0x40, \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x33, 0x26, 0x64, 0xcc, 0x90, 0x98, \
  0x99, 0x13, 0x22, 0x64, 0x08, 0x48, \
  0x05, 0x80, 0xb0, 0x16, 0x00, 0x38, \
  0x80, 0xb0, 0x16, 0x02, 0x86, 0x08, \
  0x84, 0x50, 0x8a, 0x11, 0x20, 0x60, \
  0x40, 0x68, 0x0d, 0x01, 0xb5, 0x00, \
  0x0a, 0x81, 0x50, 0x2a, 0x43, 0x00, \
  0x68, 0x0d, 0x01, 0xa0, 0x12, 0x40, \
  0x10, 0x22, 0x04, 0x40, 0xc4, 0x80, \
  0x30, 0x46, 0x08, 0xc1, 0x60, 0x10, \
  0xf8, 0x40, 0xe3, 0x2e, 0x16, 0x00

#define kMaskRandom45_45 \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x33, 0x26, 0x64, 0xcc, 0x90, 0x98, \
  0x99, 0x13, 0x22, 0x64, 0x08, 0x48, \
  0x05, 0x80, 0xb0, 0x16, 0x00, 0x38, \
  0x80, 0xb0, 0x16, 0x02, 0x86, 0x08, \
  0x84, 0x50, 0x8a, 0x11, 0x20, 0x60, \
  0x40, 0x68, 0x0d, 0x01, 0xb5, 0x00, \
  0x0a, 0x81, 0x50, 0x2a, 0x43, 0x00, \
  0x68, 0x0d, 0x01, 0xa0, 0x12, 0x40, \
  0x10, 0x22, 0x04, 0x40, 0xc4, 0x80, \
  0x30, 0x46, 0x08, 0xc1, 0x60, 0x10, \
  0x64, 0x4c, 0x89, 0x91, 0x28, 0xa0, \
  0x51, 0x4a, 0x29, 0x45, 0x22, 0x88, \
  0xa9, 0x15, 0x22, 0xa4, 0x40, 0x68, \
  0x04, 0xc0, 0x98, 0x13, 0x12, 0x10, \
  0xd0, 0x1a, 0x03, 0x40, 0x05, 0x10, \
  0x82, 0x50, 0x4a, 0x09, 0x00, 0x70, \
  0x21, 0x24, 0x24, 0x84, 0xdc, 0x00, \
  0x0c, 0x21, 0x84, 0x30, 0x84, 0x88, \
  0x4a, 0x09, 0x41, 0x28, 0x2b, 0x00, \
  0x12, 0xa2, 0x54, 0x4a, 0xd1, 0x40, \
  0x9e, 0xce, 0x8b, 0xaa, 0x34, 0x68, \
  0x46, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x33, 0x26, 0x64, 0xcc, 0x90, 0x98, \
  0x99, 0x13, 0x22, 0x64, 0x08, 0x48, \
  0x05, 0x80, 0xb0, 0x16, 0x00, 0x38, \
  0x80, 0xb0, 0x16, 0x02, 0x86, 0x08, \
  0x84, 0x50, 0x8a, 0x11, 0x20, 0x60, \
  0x40, 0x68, 0x0d, 0x01, 0xb5, 0x00, \
  0x0a, 0x81, 0x50, 0x2a, 0x43, 0x00, \
  0x68, 0x0d, 0x01, 0xa0, 0x12, 0x40, \
  0x10, 0x22, 0x04, 0x40, 0xc4, 0x80, \
  0x30, 0x46, 0x08, 0xc1, 0x60, 0x10, \
  0x64, 0x4c, 0x89, 0x19, 0x08, 0x30, \
  0x51, 0x4a, 0x28, 0xcc, 0x81, 0x18, \
  0xa9, 0x15, 0x22, 0x64, 0x20, 0x28, \
  0x04, 0xc0, 0x98, 0x16, 0x10, 0xc0, \
  0xd0, 0x1a, 0x02, 0x02, 0xc0, 0x88, \
  0x82, 0x50, 0x4a, 0x11, 0x0a, 0x40, \
  0x21, 0x24, 0x25, 0x01, 0xcc, 0x00, \
  0x0c, 0x21, 0x84, 0x2a, 0x04, 0x48, \
  0x4a, 0x09, 0x41, 0xa0, 0x31, 0x00, \
  0x12, 0xa2, 0x54, 0x40, 0x92, 0x10, \
  0x9e, 0xce, 0x88, 0xc1, 0x45, 0x00, \
  0xfb, 0x97, 0x5d, 0x7d, 0x42, 0x20

#define kMaskRandom45_5 \
  0xc6, 0xd8, 0xdb, 0x1b, 0x29, 0xb0, \
  0x63, 0x6c, 0x6d, 0x8d, 0xb2, 0x58, \
  0x1d, 0xa3, 0xb4, 0x76, 0x87, 0x70, \
  0xad, 0x55, 0xaa, 0xb5, 0x54, 0xe0, \
  0xb2, 0xb6, 0x56, 0xca, 0xdc, 0x18

#define kMaskRandom45_6 \
  0x64, 0x4c, 0x89, 0x91, 0x28, 0xa0, \
  0x51, 0x4a, 0x29, 0x45, 0x62, 0x88, \
  0xa8, 0x35, 0x04, 0x32, 0x90, 0xc0, \
  0xc4, 0xa0, 0x96, 0x84, 0x89, 0x18, \
  0x03, 0x60, 0x6c, 0x4a, 0x84, 0x70, \
  0x90, 0xd2, 0x1a, 0x29, 0x17, 0x00

#define kMaskRandom45_7 \
  0xc6, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x13, 0x26, 0x64, 0xcc, 0x90, 0x98, \
  0x8d, 0x13, 0xa2, 0x46, 0x48, 0x48, \
  0x8b, 0x41, 0x6a, 0x90, 0x81, 0x28, \
  0x52, 0xaa, 0x15, 0x42, 0x83, 0x50, \
  0xa2, 0xd4, 0x1a, 0x13, 0x16, 0x00, \
  0x61, 0xa8, 0x2c, 0x25, 0xc4, 0x30

#define kMaskRandom45_8 \
  0x28, 0x85, 0x38, 0x32, 0x10, 0x30, \
  0x21, 0xf4, 0x06, 0x01, 0xc0, 0x18, \
  0xe9, 0x1d, 0x02, 0x82, 0x21, 0x20, \
  0x17, 0x02, 0xe0, 0x15, 0x00, 0xc8, \
  0x83, 0xa0, 0x55, 0x0c, 0x0a, 0x08, \
  0x46, 0x18, 0xe8, 0x68, 0x05, 0x40, \
  0x50, 0x6a, 0x0d, 0x80, 0x9c, 0x00, \
  0x1c, 0x23, 0x84, 0x50, 0xe2, 0x80

#define kMaskRandom45_9 \
  0x44, 0x48, 0xc9, 0x19, 0x29, 0xb0, \
  0x28, 0x2d, 0x0d, 0x89, 0xf2, 0x10, \
  0x25, 0x14, 0xa2, 0x30, 0x12, 0x20, \
  0x59, 0x0a, 0x20, 0x05, 0xd0, 0x88, \
  0x03, 0xa0, 0x34, 0x1e, 0x0c, 0x18, \
  0xc0, 0xd0, 0x1a, 0x82, 0x01, 0xc8, \
  0xa2, 0x30, 0x44, 0x62, 0xc5, 0x08, \
  0x14, 0x82, 0xd2, 0x44, 0x02, 0x48, \
  0x9a, 0x03, 0x81, 0xe0, 0x00, 0x70

#define kMaskRandom46_1 \
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfc

#define kMaskRandom46_10 \
  0x64, 0x4a, 0x28, 0xc8, 0x94, 0x50, \
  0x51, 0x48, 0xa2, 0xa2, 0x91, 0x44, \
  0xa9, 0x10, 0x1b, 0x52, 0x20, 0x34, \
  0x04, 0xc4, 0x84, 0x09, 0x89, 0x08, \
  0xd0, 0x01, 0x45, 0xa0, 0x02, 0x88, \
  0x82, 0x40, 0x1d, 0x04, 0x80, 0x38, \
  0x21, 0x37, 0x00, 0x42, 0x6e, 0x00, \
  0x0c, 0x21, 0x22, 0x18, 0x42, 0x44, \
  0x4a, 0x0a, 0xc0, 0x94, 0x15, 0x80, \
  0x12, 0xb4, 0x50, 0x25, 0x68, 0xa0

#define kMaskRandom46_11 \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x33, 0x24, 0x26, 0x66, 0x48, 0x4c, \
  0x99, 0x02, 0x13, 0x32, 0x04, 0x24, \
  0x05, 0x80, 0x0e, 0x0b, 0x00, 0x1c, \
  0x80, 0xa1, 0x83, 0x01, 0x43, 0x04, \
  0x84, 0x48, 0x19, 0x08, 0x90, 0x30, \
  0x40, 0x6d, 0x40, 0x80, 0xda, 0x80, \
  0x0a, 0x90, 0xc0, 0x15, 0x21, 0x80, \
  0x68, 0x04, 0x90, 0xd0, 0x09, 0x20, \
  0x10, 0x31, 0x20, 0x20, 0x62, 0x40, \
  0x30, 0x58, 0x04, 0x60, 0xb0, 0x08

#define kMaskRandom46_12 \
  0x64, 0x4a, 0x28, 0xc8, 0x94, 0x50, \
  0x51, 0x58, 0xa2, 0xa2, 0xb1, 0x44, \
  0x0c, 0xa4, 0x30, 0x19, 0x48, 0x60, \
  0xa1, 0x22, 0x47, 0x42, 0x44, 0x8c, \
  0x12, 0xa1, 0x1c, 0x25, 0x42, 0x38, \
  0x8a, 0x45, 0xc1, 0x14, 0x8b, 0x80, \
  0x86, 0x8a, 0x6d, 0x0d, 0x14, 0xd8, \
  0x23, 0x2c, 0x84, 0x46, 0x59, 0x08, \
  0x16, 0x21, 0x98, 0x2c, 0x43, 0x30, \
  0x4c, 0x30, 0x54, 0x98, 0x60, 0xa8, \
  0x41, 0xc1, 0x26, 0x83, 0x82, 0x4c, \
  0x19, 0x56, 0xe4, 0x32, 0xad, 0xc8

#define kMaskRandom46_13 \
  0x64, 0x4a, 0x28, 0xc8, 0x94, 0x50, \
  0x51, 0x58, 0xa2, 0xa2, 0xb1, 0x44, \
  0x0c, 0xa4, 0x30, 0x19, 0x48, 0x60, \
  0xa1, 0x22, 0x47, 0x42, 0x44, 0x8c, \
  0x12, 0xa1, 0x1c, 0x25, 0x42, 0x38, \
  0x8a, 0x45, 0xc1, 0x14, 0x8b, 0x80, \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x33, 0x24, 0x26, 0x66, 0x48, 0x4c, \
  0x91, 0x92, 0x13, 0x23, 0x24, 0x24, \
  0xa4, 0x20, 0x4b, 0x48, 0x40, 0x94, \
  0x50, 0xa0, 0xd4, 0xa1, 0x41, 0xa8, \
  0x84, 0xc5, 0x81, 0x09, 0x8b, 0x00, \
  0x09, 0x71, 0x0c, 0x12, 0xe2, 0x18

#define kMaskRandom46_14 \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x33, 0x24, 0x26, 0x66, 0x48, 0x4c, \
  0x91, 0x92, 0x13, 0x23, 0x24, 0x24, \
  0xa4, 0x20, 0x4b, 0x48, 0x40, 0x94, \
  0x50, 0xa0, 0xd4, 0xa1, 0x41, 0xa8, \
  0x84, 0xc5, 0x81, 0x09, 0x8b, 0x00, \
  0x09, 0x71, 0x0c, 0x12, 0xe2, 0x18, \
  0x64, 0x4a, 0x28, 0xc8, 0x94, 0x50, \
  0x51, 0x58, 0xa2, 0xa2, 0xb1, 0x44, \
  0x0c, 0xa4, 0x30, 0x19, 0x48, 0x60, \
  0xa1, 0x22, 0x47, 0x42, 0x44, 0x8c, \
  0x12, 0xa1, 0x1c, 0x25, 0x42, 0x38, \
  0x8a, 0x45, 0xc1, 0x14, 0x8b, 0x80, \
  0x9c, 0x3f, 0xb3, 0x38, 0x7f, 0x64

#define kMaskRandom46_15 \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x33, 0x24, 0x26, 0x66, 0x48, 0x4c, \
  0x91, 0x92, 0x13, 0x23, 0x24, 0x24, \
  0xa4, 0x20, 0x4b, 0x48, 0x40, 0x94, \
  0x50, 0xa0, 0xd4, 0xa1, 0x41, 0xa8, \
  0x84, 0xc5, 0x81, 0x09, 0x8b, 0x00, \
  0x09, 0x71, 0x0c, 0x12, 0xe2, 0x18, \
  0x0c, 0x84, 0x0c, 0x19, 0x08, 0x18, \
  0x80, 0x70, 0x07, 0x00, 0xe0, 0x0c, \
  0xa0, 0x88, 0x49, 0x41, 0x10, 0x90, \
  0x05, 0x40, 0x32, 0x0a, 0x80, 0x64, \
  0x43, 0x02, 0x82, 0x86, 0x05, 0x04, \
  0x1a, 0x01, 0x50, 0x34, 0x02, 0xa0, \
  0x60, 0x27, 0x00, 0xc0, 0x4e, 0x00, \
  0x14, 0x38, 0xa0, 0x28, 0x71, 0x40

#define kMaskRandom46_16 \
  0x0c, 0x84, 0x0c, 0x19, 0x08, 0x18, \
  0x80, 0x70, 0x07, 0x00, 0xe0, 0x0c, \
  0xa0, 0x88, 0x49, 0x41, 0x10, 0x90, \
  0x05, 0x40, 0x32, 0x0a, 0x80, 0x64, \
  0x43, 0x02, 0x82, 0x86, 0x05, 0x04, \
  0x1a, 0x01, 0x50, 0x34, 0x02, 0xa0, \
  0x60, 0x27, 0x00, 0xc0, 0x4e, 0x00, \
  0x14, 0x38, 0xa0, 0x28, 0x71, 0x40, \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x33, 0x24, 0x26, 0x66, 0x48, 0x4c, \
  0x91, 0x92, 0x13, 0x23, 0x24, 0x24, \
  0xa4, 0x20, 0x4b, 0x48, 0x40, 0x94, \
  0x50, 0xa0, 0xd4, 0xa1, 0x41, 0xa8, \
  0x84, 0xc5, 0x81, 0x09, 0x8b, 0x00, \
  0x09, 0x71, 0x0c, 0x12, 0xe2, 0x18, \
  0xfa, 0xd9, 0xf5, 0xf5, 0xb3, 0xe8

#define kMaskRandom46_17 \
  0x0c, 0x84, 0x0c, 0x19, 0x08, 0x18, \
  0x80, 0x70, 0x07, 0x00, 0xe0, 0x0c, \
  0xa0, 0x88, 0x49, 0x41, 0x10, 0x90, \
  0x05, 0x40, 0x32, 0x0a, 0x80, 0x64, \
  0x43, 0x02, 0x82, 0x86, 0x05, 0x04, \
  0x1a, 0x01, 0x50, 0x34, 0x02, 0xa0, \
  0x60, 0x27, 0x00, 0xc0, 0x4e, 0x00, \
  0x14, 0x38, 0xa0, 0x28, 0x71, 0x40, \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x62, 0x7c, 0x84, 0xc4, 0xf9, 0x08, \
  0x8c, 0x04, 0x89, 0x18, 0x09, 0x10, \
  0x01, 0x74, 0x22, 0x02, 0xe8, 0x44, \
  0x07, 0x83, 0x06, 0x0f, 0x06, 0x0c, \
  0xa0, 0x80, 0x73, 0x41, 0x00, 0xe4, \
  0x18, 0xb1, 0x42, 0x31, 0x62, 0x84, \
  0x91, 0x00, 0x93, 0x22, 0x01, 0x24, \
  0x78, 0x00, 0x1c, 0xf0, 0x00, 0x38

#define kMaskRandom46_18 \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x62, 0x7c, 0x84, 0xc4, 0xf9, 0x08, \
  0x8c, 0x04, 0x89, 0x18, 0x09, 0x10, \
  0x01, 0x74, 0x22, 0x02, 0xe8, 0x44, \
  0x07, 0x83, 0x06, 0x0f, 0x06, 0x0c, \
  0xa0, 0x80, 0x73, 0x41, 0x00, 0xe4, \
  0x18, 0xb1, 0x42, 0x31, 0x62, 0x84, \
  0x91, 0x00, 0x93, 0x22, 0x01, 0x24, \
  0x78, 0x00, 0x1c, 0xf0, 0x00, 0x38, \
  0x0c, 0x84, 0x0c, 0x19, 0x08, 0x18, \
  0x80, 0x70, 0x07, 0x00, 0xe0, 0x0c, \
  0xa0, 0x88, 0x49, 0x41, 0x10, 0x90, \
  0x05, 0x40, 0x32, 0x0a, 0x80, 0x64, \
  0x43, 0x02, 0x82, 0x86, 0x05, 0x04, \
  0x1a, 0x01, 0x50, 0x34, 0x02, 0xa0, \
  0x60, 0x27, 0x00, 0xc0, 0x4e, 0x00, \
  0x14, 0x38, 0xa0, 0x28, 0x71, 0x40, \
  0x82, 0x32, 0x57, 0x04, 0x64, 0xac

#define kMaskRandom46_19 \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x62, 0x7c, 0x84, 0xc4, 0xf9, 0x08, \
  0x8c, 0x04, 0x89, 0x18, 0x09, 0x10, \
  0x01, 0x74, 0x22, 0x02, 0xe8, 0x44, \
  0x07, 0x83, 0x06, 0x0f, 0x06, 0x0c, \
  0xa0, 0x80, 0x73, 0x41, 0x00, 0xe4, \
  0x18, 0xb1, 0x42, 0x31, 0x62, 0x84, \
  0x91, 0x00, 0x93, 0x22, 0x01, 0x24, \
  0x78, 0x00, 0x1c, 0xf0, 0x00, 0x38, \
  0x64, 0x4a, 0x28, 0xc8, 0x94, 0x50, \
  0x51, 0x48, 0xa2, 0xa2, 0x91, 0x44, \
  0xa9, 0x10, 0x1b, 0x52, 0x20, 0x34, \
  0x04, 0xc4, 0x84, 0x09, 0x89, 0x08, \
  0xd0, 0x01, 0x45, 0xa0, 0x02, 0x88, \
  0x82, 0x40, 0x1d, 0x04, 0x80, 0x38, \
  0x21, 0x37, 0x00, 0x42, 0x6e, 0x00, \
  0x0c, 0x21, 0x22, 0x18, 0x42, 0x44, \
  0x4a, 0x0a, 0xc0, 0x94, 0x15, 0x80, \
  0x12, 0xb4, 0x50, 0x25, 0x68, 0xa0

#define kMaskRandom46_2 \
  0xec, 0xdd, 0x99, 0xd9, 0xbb, 0x30, \
  0x9b, 0xb2, 0x77, 0x37, 0x64, 0xec

#define kMaskRandom46_20 \
  0x64, 0x4a, 0x28, 0xc8, 0x94, 0x50, \
  0x51, 0x48, 0xa2, 0xa2, 0x91, 0x44, \
  0xa9, 0x10, 0x1b, 0x52, 0x20, 0x34, \
  0x04, 0xc4, 0x84, 0x09, 0x89, 0x08, \
  0xd0, 0x01, 0x45, 0xa0, 0x02, 0x88, \
  0x82, 0x40, 0x1d, 0x04, 0x80, 0x38, \
  0x21, 0x37, 0x00, 0x42, 0x6e, 0x00, \
  0x0c, 0x21, 0x22, 0x18, 0x42, 0x44, \
  0x4a, 0x0a, 0xc0, 0x94, 0x15, 0x80, \
  0x12, 0xb4, 0x50, 0x25, 0x68, 0xa0, \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x62, 0x7c, 0x84, 0xc4, 0xf9, 0x08, \
  0x8c, 0x04, 0x89, 0x18, 0x09, 0x10, \
  0x01, 0x74, 0x22, 0x02, 0xe8, 0x44, \
  0x07, 0x83, 0x06, 0x0f, 0x06, 0x0c, \
  0xa0, 0x80, 0x73, 0x41, 0x00, 0xe4, \
  0x18, 0xb1, 0x42, 0x31, 0x62, 0x84, \
  0x91, 0x00, 0x93, 0x22, 0x01, 0x24, \
  0x78, 0x00, 0x1c, 0xf0, 0x00, 0x38, \
  0xdb, 0x4a, 0x7b, 0xb6, 0x94, 0xf4

#define kMaskRandom46_21 \
  0x64, 0x4a, 0x28, 0xc8, 0x94, 0x50, \
  0x51, 0x48, 0xa2, 0xa2, 0x91, 0x44, \
  0xa9, 0x10, 0x1b, 0x52, 0x20, 0x34, \
  0x04, 0xc4, 0x84, 0x09, 0x89, 0x08, \
  0xd0, 0x01, 0x45, 0xa0, 0x02, 0x88, \
  0x82, 0x40, 0x1d, 0x04, 0x80, 0x38, \
  0x21, 0x37, 0x00, 0x42, 0x6e, 0x00, \
  0x0c, 0x21, 0x22, 0x18, 0x42, 0x44, \
  0x4a, 0x0a, 0xc0, 0x94, 0x15, 0x80, \
  0x12, 0xb4, 0x50, 0x25, 0x68, 0xa0, \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x33, 0x24, 0x26, 0x66, 0x48, 0x4c, \
  0x99, 0x02, 0x13, 0x32, 0x04, 0x24, \
  0x05, 0x80, 0x0e, 0x0b, 0x00, 0x1c, \
  0x80, 0xa1, 0x83, 0x01, 0x43, 0x04, \
  0x84, 0x48, 0x19, 0x08, 0x90, 0x30, \
  0x40, 0x6d, 0x40, 0x80, 0xda, 0x80, \
  0x0a, 0x90, 0xc0, 0x15, 0x21, 0x80, \
  0x68, 0x04, 0x90, 0xd0, 0x09, 0x20, \
  0x10, 0x31, 0x20, 0x20, 0x62, 0x40, \
  0x30, 0x58, 0x04, 0x60, 0xb0, 0x08

#define kMaskRandom46_22 \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x33, 0x24, 0x26, 0x66, 0x48, 0x4c, \
  0x99, 0x02, 0x13, 0x32, 0x04, 0x24, \
  0x05, 0x80, 0x0e, 0x0b, 0x00, 0x1c, \
  0x80, 0xa1, 0x83, 0x01, 0x43, 0x04, \
  0x84, 0x48, 0x19, 0x08, 0x90, 0x30, \
  0x40, 0x6d, 0x40, 0x80, 0xda, 0x80, \
  0x0a, 0x90, 0xc0, 0x15, 0x21, 0x80, \
  0x68, 0x04, 0x90, 0xd0, 0x09, 0x20, \
  0x10, 0x31, 0x20, 0x20, 0x62, 0x40, \
  0x30, 0x58, 0x04, 0x60, 0xb0, 0x08, \
  0x64, 0x4a, 0x28, 0xc8, 0x94, 0x50, \
  0x51, 0x48, 0xa2, 0xa2, 0x91, 0x44, \
  0xa9, 0x10, 0x1b, 0x52, 0x20, 0x34, \
  0x04, 0xc4, 0x84, 0x09, 0x89, 0x08, \
  0xd0, 0x01, 0x45, 0xa0, 0x02, 0x88, \
  0x82, 0x40, 0x1d, 0x04, 0x80, 0x38, \
  0x21, 0x37, 0x00, 0x42, 0x6e, 0x00, \
  0x0c, 0x21, 0x22, 0x18, 0x42, 0x44, \
  0x4a, 0x0a, 0xc0, 0x94, 0x15, 0x80, \
  0x12, 0xb4, 0x50, 0x25, 0x68, 0xa0, \
  0xea, 0x8d, 0x1b, 0xd5, 0x1a, 0x34

#define kMaskRandom46_23 \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x33, 0x24, 0x26, 0x66, 0x48, 0x4c, \
  0x99, 0x02, 0x13, 0x32, 0x04, 0x24, \
  0x05, 0x80, 0x0e, 0x0b, 0x00, 0x1c, \
  0x80, 0xa1, 0x83, 0x01, 0x43, 0x04, \
  0x84, 0x48, 0x19, 0x08, 0x90, 0x30, \
  0x40, 0x6d, 0x40, 0x80, 0xda, 0x80, \
  0x0a, 0x90, 0xc0, 0x15, 0x21, 0x80, \
  0x68, 0x04, 0x90, 0xd0, 0x09, 0x20, \
  0x10, 0x31, 0x20, 0x20, 0x62, 0x40, \
  0x30, 0x58, 0x04, 0x60, 0xb0, 0x08, \
  0x46, 0x42, 0x0c, 0x8c, 0x84, 0x18, \
  0x33, 0x20, 0x46, 0x66, 0x40, 0x8c, \
  0x99, 0x08, 0x0b, 0x32, 0x10, 0x14, \
  0x05, 0x84, 0x30, 0x0b, 0x08, 0x60, \
  0x80, 0xb0, 0x23, 0x01, 0x60, 0x44, \
  0x84, 0x42, 0x91, 0x08, 0x85, 0x20, \
  0x40, 0x73, 0x00, 0x80, 0xe6, 0x00, \
  0x0a, 0x81, 0x12, 0x15, 0x02, 0x24, \
  0x68, 0x0c, 0x40, 0xd0, 0x18, 0x80, \
  0x10, 0x24, 0x84, 0x20, 0x49, 0x08, \
  0x30, 0x51, 0x40, 0x60, 0xa2, 0x80, \
  0x5f, 0x50, 0x88, 0xbe, 0xa1, 0x10

#define kMaskRandom46_24 \
  0x64, 0x4a, 0x28, 0xc8, 0x94, 0x50, \
  0x51, 0x58, 0xa2, 0xa2, 0xb1, 0x44, \
  0x0c, 0xa4, 0x30, 0x19, 0x48, 0x60, \
  0xa1, 0x22, 0x47, 0x42, 0x44, 0x8c, \
  0x12, 0xa1, 0x1c, 0x25, 0x42, 0x38, \
  0x8a, 0x45, 0xc1, 0x14, 0x8b, 0x80, \
  0x86, 0x8a, 0x6d, 0x0d, 0x14, 0xd8, \
  0x23, 0x2c, 0x84, 0x46, 0x59, 0x08, \
  0x16, 0x21, 0x98, 0x2c, 0x43, 0x30, \
  0x4c, 0x30, 0x54, 0x98, 0x60, 0xa8, \
  0x41, 0xc1, 0x26, 0x83, 0x82, 0x4c, \
  0x19, 0x56, 0xe4, 0x32, 0xad, 0xc8, \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x33, 0x24, 0x26, 0x66, 0x48, 0x4c, \
  0x99, 0x02, 0x13, 0x32, 0x04, 0x24, \
  0x05, 0x80, 0x0e, 0x0b, 0x00, 0x1c, \
  0x80, 0xa1, 0x83, 0x01, 0x43, 0x04, \
  0x84, 0x48, 0x19, 0x08, 0x90, 0x30, \
  0x40, 0x6d, 0x40, 0x80, 0xda, 0x80, \
  0x0a, 0x90, 0xc0, 0x15, 0x21, 0x80, \
  0x68, 0x04, 0x90, 0xd0, 0x09, 0x20, \
  0x10, 0x31, 0x20, 0x20, 0x62, 0x40, \
  0x30, 0x58, 0x04, 0x60, 0xb0, 0x08, \
  0x18, 0x8b, 0x03, 0xb4, 0x3b, 0x10

#define kMaskRandom46_25 \
  0x64, 0x4a, 0x28, 0xc8, 0x94, 0x50, \
  0x51, 0x58, 0xa2, 0xa2, 0xb1, 0x44, \
  0x0c, 0xa4, 0x30, 0x19, 0x48, 0x60, \
  0xa1, 0x22, 0x47, 0x42, 0x44, 0x8c, \
  0x12, 0xa1, 0x1c, 0x25, 0x42, 0x38, \
  0x8a, 0x45, 0xc1, 0x14, 0x8b, 0x80, \
  0x86, 0x8a, 0x6d, 0x0d, 0x14, 0xd8, \
  0x23, 0x2c, 0x84, 0x46, 0x59, 0x08, \
  0x16, 0x21, 0x98, 0x2c, 0x43, 0x30, \
  0x4c, 0x30, 0x54, 0x98, 0x60, 0xa8, \
  0x41, 0xc1, 0x26, 0x83, 0x82, 0x4c, \
  0x19, 0x56, 0xe4, 0x32, 0xad, 0xc8, \
  0x64, 0x4a, 0x28, 0xc8, 0x94, 0x50, \
  0x51, 0x58, 0xa2, 0xa2, 0xb1, 0x44, \
  0x0c, 0xa4, 0x30, 0x19, 0x48, 0x60, \
  0xa1, 0x22, 0x47, 0x42, 0x44, 0x8c, \
  0x12, 0xa1, 0x1c, 0x25, 0x42, 0x38, \
  0x8a, 0x45, 0xc1, 0x14, 0x8b, 0x80, \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x33, 0x24, 0x26, 0x66, 0x48, 0x4c, \
  0x91, 0x92, 0x13, 0x23, 0x24, 0x24, \
  0xa4, 0x20, 0x4b, 0x48, 0x40, 0x94, \
  0x50, 0xa0, 0xd4, 0xa1, 0x41, 0xa8, \
  0x84, 0xc5, 0x81, 0x09, 0x8b, 0x00, \
  0x09, 0x71, 0x0c, 0x12, 0xe2, 0x18

#define kMaskRandom46_26 \
  0x64, 0x4a, 0x28, 0xc8, 0x94, 0x50, \
  0x51, 0x58, 0xa2, 0xa2, 0xb1, 0x44, \
  0x0c, 0xa4, 0x30, 0x19, 0x48, 0x60, \
  0xa1, 0x22, 0x47, 0x42, 0x44, 0x8c, \
  0x12, 0xa1, 0x1c, 0x25, 0x42, 0x38, \
  0x8a, 0x45, 0xc1, 0x14, 0x8b, 0x80, \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x33, 0x24, 0x26, 0x66, 0x48, 0x4c, \
  0x91, 0x92, 0x13, 0x23, 0x24, 0x24, \
  0xa4, 0x20, 0x4b, 0x48, 0x40, 0x94, \
  0x50, 0xa0, 0xd4, 0xa1, 0x41, 0xa8, \
  0x84, 0xc5, 0x81, 0x09, 0x8b, 0x00, \
  0x09, 0x71, 0x0c, 0x12, 0xe2, 0x18, \
  0x64, 0x4a, 0x28, 0xc8, 0x94, 0x50, \
  0x51, 0x58, 0xa2, 0xa2, 0xb1, 0x44, \
  0x0c, 0xa4, 0x30, 0x19, 0x48, 0x60, \
  0xa1, 0x22, 0x47, 0x42, 0x44, 0x8c, \
  0x12, 0xa1, 0x1c, 0x25, 0x42, 0x38, \
  0x8a, 0x45, 0xc1, 0x14, 0x8b, 0x80, \
  0x86, 0x8a, 0x6d, 0x0d, 0x14, 0xd8, \
  0x23, 0x2c, 0x84, 0x46, 0x59, 0x08, \
  0x16, 0x21, 0x98, 0x2c, 0x43, 0x30, \
  0x4c, 0x30, 0x54, 0x98, 0x60, 0xa8, \
  0x41, 0xc1, 0x26, 0x83, 0x82, 0x4c, \
  0x19, 0x56, 0xe4, 0x32, 0xad, 0xc8, \
  0x2d, 0x6d, 0xd2, 0x57, 0xd6, 0x2c

#define kMaskRandom46_27 \
  0x64, 0x4a, 0x28, 0xc8, 0x94, 0x50, \
  0x51, 0x58, 0xa2, 0xa2, 0xb1, 0x44, \
  0x0c, 0xa4, 0x30, 0x19, 0x48, 0x60, \
  0xa1, 0x22, 0x47, 0x42, 0x44, 0x8c, \
  0x12, 0xa1, 0x1c, 0x25, 0x42, 0x38, \
  0x8a, 0x45, 0xc1, 0x14, 0x8b, 0x80, \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x33, 0x24, 0x26, 0x66, 0x48, 0x4c, \
  0x91, 0x92, 0x13, 0x23, 0x24, 0x24, \
  0xa4, 0x20, 0x4b, 0x48, 0x40, 0x94, \
  0x50, 0xa0, 0xd4, 0xa1, 0x41, 0xa8, \
  0x84, 0xc5, 0x81, 0x09, 0x8b, 0x00, \
  0x09, 0x71, 0x0c, 0x12, 0xe2, 0x18, \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x33, 0x24, 0x26, 0x66, 0x48, 0x4c, \
  0x91, 0x92, 0x13, 0x23, 0x24, 0x24, \
  0xa4, 0x20, 0x4b, 0x48, 0x40, 0x94, \
  0x50, 0xa0, 0xd4, 0xa1, 0x41, 0xa8, \
  0x84, 0xc5, 0x81, 0x09, 0x8b, 0x00, \
  0x09, 0x71, 0x0c, 0x12, 0xe2, 0x18, \
  0x64, 0x4a, 0x28, 0xc8, 0x94, 0x50, \
  0x51, 0x58, 0xa2, 0xa2, 0xb1, 0x44, \
  0x0c, 0xa4, 0x30, 0x19, 0x48, 0x60, \
  0xa1, 0x22, 0x47, 0x42, 0x44, 0x8c, \
  0x12, 0xa1, 0x1c, 0x25, 0x42, 0x38, \
  0x8a, 0x45, 0xc1, 0x14, 0x8b, 0x80, \
  0x9c, 0x3f, 0xb3, 0x38, 0x7f, 0x64

#define kMaskRandom46_28 \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x33, 0x24, 0x26, 0x66, 0x48, 0x4c, \
  0x91, 0x92, 0x13, 0x23, 0x24, 0x24, \
  0xa4, 0x20, 0x4b, 0x48, 0x40, 0x94, \
  0x50, 0xa0, 0xd4, 0xa1, 0x41, 0xa8, \
  0x84, 0xc5, 0x81, 0x09, 0x8b, 0x00, \
  0x09, 0x71, 0x0c, 0x12, 0xe2, 0x18, \
  0x64, 0x4a, 0x28, 0xc8, 0x94, 0x50, \
  0x51, 0x58, 0xa2, 0xa2, 0xb1, 0x44, \
  0x0c, 0xa4, 0x30, 0x19, 0x48, 0x60, \
  0xa1, 0x22, 0x47, 0x42, 0x44, 0x8c, \
  0x12, 0xa1, 0x1c, 0x25, 0x42, 0x38, \
  0x8a, 0x45, 0xc1, 0x14, 0x8b, 0x80, \
  0x9c, 0x3f, 0xb3, 0x38, 0x7f, 0x64, \
  0x64, 0x4a, 0x28, 0xc8, 0x94, 0x50, \
  0x51, 0x58, 0xa2, 0xa2, 0xb1, 0x44, \
  0x0c, 0xa4, 0x30, 0x19, 0x48, 0x60, \
  0xa1, 0x22, 0x47, 0x42, 0x44, 0x8c, \
  0x12, 0xa1, 0x1c, 0x25, 0x42, 0x38, \
  0x8a, 0x45, 0xc1, 0x14, 0x8b, 0x80, \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x33, 0x24, 0x26, 0x66, 0x48, 0x4c, \
  0x91, 0x92, 0x13, 0x23, 0x24, 0x24, \
  0xa4, 0x20, 0x4b, 0x48, 0x40, 0x94, \
  0x50, 0xa0, 0xd4, 0xa1, 0x41, 0xa8, \
  0x84, 0xc5, 0x81, 0x09, 0x8b, 0x00, \
  0x09, 0x71, 0x0c, 0x12, 0xe2, 0x18, \
  0xfa, 0x52, 0xf9, 0x72, 0xd9, 0x68

#define kMaskRandom46_29 \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x33, 0x24, 0x26, 0x66, 0x48, 0x4c, \
  0x91, 0x92, 0x13, 0x23, 0x24, 0x24, \
  0xa4, 0x20, 0x4b, 0x48, 0x40, 0x94, \
  0x50, 0xa0, 0xd4, 0xa1, 0x41, 0xa8, \
  0x84, 0xc5, 0x81, 0x09, 0x8b, 0x00, \
  0x09, 0x71, 0x0c, 0x12, 0xe2, 0x18, \
  0x64, 0x4a, 0x28, 0xc8, 0x94, 0x50, \
  0x51, 0x58, 0xa2, 0xa2, 0xb1, 0x44, \
  0x0c, 0xa4, 0x30, 0x19, 0x48, 0x60, \
  0xa1, 0x22, 0x47, 0x42, 0x44, 0x8c, \
  0x12, 0xa1, 0x1c, 0x25, 0x42, 0x38, \
  0x8a, 0x45, 0xc1, 0x14, 0x8b, 0x80, \
  0x9c, 0x3f, 0xb3, 0x38, 0x7f, 0x64, \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x33, 0x24, 0x26, 0x66, 0x48, 0x4c, \
  0x91, 0x92, 0x13, 0x23, 0x24, 0x24, \
  0xa4, 0x20, 0x4b, 0x48, 0x40, 0x94, \
  0x50, 0xa0, 0xd4, 0xa1, 0x41, 0xa8, \
  0x84, 0xc5, 0x81, 0x09, 0x8b, 0x00, \
  0x09, 0x71, 0x0c, 0x12, 0xe2, 0x18, \
  0x0c, 0x84, 0x0c, 0x19, 0x08, 0x18, \
  0x80, 0x70, 0x07, 0x00, 0xe0, 0x0c, \
  0xa0, 0x88, 0x49, 0x41, 0x10, 0x90, \
  0x05, 0x40, 0x32, 0x0a, 0x80, 0x64, \
  0x43, 0x02, 0x82, 0x86, 0x05, 0x04, \
  0x1a, 0x01, 0x50, 0x34, 0x02, 0xa0, \
  0x60, 0x27, 0x00, 0xc0, 0x4e, 0x00, \
  0x14, 0x38, 0xa0, 0x28, 0x71, 0x40

#define kMaskRandom46_3 \
  0xca, 0xd3, 0x65, 0x95, 0xa6, 0xc8, \
  0xf1, 0x49, 0x3b, 0xe2, 0x92, 0x74, \
  0x76, 0x27, 0xd0, 0xec, 0x4f, 0xa0

#define kMaskRandom46_30 \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x33, 0x24, 0x26, 0x66, 0x48, 0x4c, \
  0x91, 0x92, 0x13, 0x23, 0x24, 0x24, \
  0xa4, 0x20, 0x4b, 0x48, 0x40, 0x94, \
  0x50, 0xa0, 0xd4, 0xa1, 0x41, 0xa8, \
  0x84, 0xc5, 0x81, 0x09, 0x8b, 0x00, \
  0x09, 0x71, 0x0c, 0x12, 0xe2, 0x18, \
  0x0c, 0x84, 0x0c, 0x19, 0x08, 0x18, \
  0x80, 0x70, 0x07, 0x00, 0xe0, 0x0c, \
  0xa0, 0x88, 0x49, 0x41, 0x10, 0x90, \
  0x05, 0x40, 0x32, 0x0a, 0x80, 0x64, \
  0x43, 0x02, 0x82, 0x86, 0x05, 0x04, \
  0x1a, 0x01, 0x50, 0x34, 0x02, 0xa0, \
  0x60, 0x27, 0x00, 0xc0, 0x4e, 0x00, \
  0x14, 0x38, 0xa0, 0x28, 0x71, 0x40, \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x33, 0x24, 0x26, 0x66, 0x48, 0x4c, \
  0x91, 0x92, 0x13, 0x23, 0x24, 0x24, \
  0xa4, 0x20, 0x4b, 0x48, 0x40, 0x94, \
  0x50, 0xa0, 0xd4, 0xa1, 0x41, 0xa8, \
  0x84, 0xc5, 0x81, 0x09, 0x8b, 0x00, \
  0x09, 0x71, 0x0c, 0x12, 0xe2, 0x18, \
  0x64, 0x4a, 0x28, 0xc8, 0x94, 0x50, \
  0x51, 0x58, 0xa2, 0xa2, 0xb1, 0x44, \
  0x0c, 0xa4, 0x30, 0x19, 0x48, 0x60, \
  0xa1, 0x22, 0x47, 0x42, 0x44, 0x8c, \
  0x12, 0xa1, 0x1c, 0x25, 0x42, 0x38, \
  0x8a, 0x45, 0xc1, 0x14, 0x8b, 0x80, \
  0x9c, 0x3f, 0xb3, 0x38, 0x7f, 0x64, \
  0x99, 0xf6, 0x0a, 0xdd, 0x16, 0xb0

#define kMaskRandom46_31 \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x33, 0x24, 0x26, 0x66, 0x48, 0x4c, \
  0x91, 0x92, 0x13, 0x23, 0x24, 0x24, \
  0xa4, 0x20, 0x4b, 0x48, 0x40, 0x94, \
  0x50, 0xa0, 0xd4, 0xa1, 0x41, 0xa8, \
  0x84, 0xc5, 0x81, 0x09, 0x8b, 0x00, \
  0x09, 0x71, 0x0c, 0x12, 0xe2, 0x18, \
  0x0c, 0x84, 0x0c, 0x19, 0x08, 0x18, \
  0x80, 0x70, 0x07, 0x00, 0xe0, 0x0c, \
  0xa0, 0x88, 0x49, 0x41, 0x10, 0x90, \
  0x05, 0x40, 0x32, 0x0a, 0x80, 0x64, \
  0x43, 0x02, 0x82, 0x86, 0x05, 0x04, \
  0x1a, 0x01, 0x50, 0x34, 0x02, 0xa0, \
  0x60, 0x27, 0x00, 0xc0, 0x4e, 0x00, \
  0x14, 0x38, 0xa0, 0x28, 0x71, 0x40, \
  0x0c, 0x84, 0x0c, 0x19, 0x08, 0x18, \
  0x80, 0x70, 0x07, 0x00, 0xe0, 0x0c, \
  0xa0, 0x88, 0x49, 0x41, 0x10, 0x90, \
  0x05, 0x40, 0x32, 0x0a, 0x80, 0x64, \
  0x43, 0x02, 0x82, 0x86, 0x05, 0x04, \
  0x1a, 0x01, 0x50, 0x34, 0x02, 0xa0, \
  0x60, 0x27, 0x00, 0xc0, 0x4e, 0x00, \
  0x14, 0x38, 0xa0, 0x28, 0x71, 0x40, \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x33, 0x24, 0x26, 0x66, 0x48, 0x4c, \
  0x91, 0x92, 0x13, 0x23, 0x24, 0x24, \
  0xa4, 0x20, 0x4b, 0x48, 0x40, 0x94, \
  0x50, 0xa0, 0xd4, 0xa1, 0x41, 0xa8, \
  0x84, 0xc5, 0x81, 0x09, 0x8b, 0x00, \
  0x09, 0x71, 0x0c, 0x12, 0xe2, 0x18, \
  0xfa, 0xd9, 0xf5, 0xf5, 0xb3, 0xe8

#define kMaskRandom46_32 \
  0x0c, 0x84, 0x0c, 0x19, 0x08, 0x18, \
  0x80, 0x70, 0x07, 0x00, 0xe0, 0x0c, \
  0xa0, 0x88, 0x49, 0x41, 0x10, 0x90, \
  0x05, 0x40, 0x32, 0x0a, 0x80, 0x64, \
  0x43, 0x02, 0x82, 0x86, 0x05, 0x04, \
  0x1a, 0x01, 0x50, 0x34, 0x02, 0xa0, \
  0x60, 0x27, 0x00, 0xc0, 0x4e, 0x00, \
  0x14, 0x38, 0xa0, 0x28, 0x71, 0x40, \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x33, 0x24, 0x26, 0x66, 0x48, 0x4c, \
  0x91, 0x92, 0x13, 0x23, 0x24, 0x24, \
  0xa4, 0x20, 0x4b, 0x48, 0x40, 0x94, \
  0x50, 0xa0, 0xd4, 0xa1, 0x41, 0xa8, \
  0x84, 0xc5, 0x81, 0x09, 0x8b, 0x00, \
  0x09, 0x71, 0x0c, 0x12, 0xe2, 0x18, \
  0xfa, 0xd9, 0xf5, 0xf5, 0xb3, 0xe8, \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x33, 0x24, 0x26, 0x66, 0x48, 0x4c, \
  0x91, 0x92, 0x13, 0x23, 0x24, 0x24, \
  0xa4, 0x20, 0x4b, 0x48, 0x40, 0x94, \
  0x50, 0xa0, 0xd4, 0xa1, 0x41, 0xa8, \
  0x84, 0xc5, 0x81, 0x09, 0x8b, 0x00, \
  0x09, 0x71, 0x0c, 0x12, 0xe2, 0x18, \
  0x0c, 0x84, 0x0c, 0x19, 0x08, 0x18, \
  0x80, 0x70, 0x07, 0x00, 0xe0, 0x0c, \
  0xa0, 0x88, 0x49, 0x41, 0x10, 0x90, \
  0x05, 0x40, 0x32, 0x0a, 0x80, 0x64, \
  0x43, 0x02, 0x82, 0x86, 0x05, 0x04, \
  0x1a, 0x01, 0x50, 0x34, 0x02, 0xa0, \
  0x60, 0x27, 0x00, 0xc0, 0x4e, 0x00, \
  0x14, 0x38, 0xa0, 0x28, 0x71, 0x40, \
  0x69, 0xcd, 0xeb, 0x51, 0xc9, 0xa8

#define kMaskRandom46_33 \
  0x0c, 0x84, 0x0c, 0x19, 0x08, 0x18, \
  0x80, 0x70, 0x07, 0x00, 0xe0, 0x0c, \
  0xa0, 0x88, 0x49, 0x41, 0x10, 0x90, \
  0x05, 0x40, 0x32, 0x0a, 0x80, 0x64, \
  0x43, 0x02, 0x82, 0x86, 0x05, 0x04, \
  0x1a, 0x01, 0x50, 0x34, 0x02, 0xa0, \
  0x60, 0x27, 0x00, 0xc0, 0x4e, 0x00, \
  0x14, 0x38, 0xa0, 0x28, 0x71, 0x40, \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x33, 0x24, 0x26, 0x66, 0x48, 0x4c, \
  0x91, 0x92, 0x13, 0x23, 0x24, 0x24, \
  0xa4, 0x20, 0x4b, 0x48, 0x40, 0x94, \
  0x50, 0xa0, 0xd4, 0xa1, 0x41, 0xa8, \
  0x84, 0xc5, 0x81, 0x09, 0x8b, 0x00, \
  0x09, 0x71, 0x0c, 0x12, 0xe2, 0x18, \
  0xfa, 0xd9, 0xf5, 0xf5, 0xb3, 0xe8, \
  0x0c, 0x84, 0x0c, 0x19, 0x08, 0x18, \
  0x80, 0x70, 0x07, 0x00, 0xe0, 0x0c, \
  0xa0, 0x88, 0x49, 0x41, 0x10, 0x90, \
  0x05, 0x40, 0x32, 0x0a, 0x80, 0x64, \
  0x43, 0x02, 0x82, 0x86, 0x05, 0x04, \
  0x1a, 0x01, 0x50, 0x34, 0x02, 0xa0, \
  0x60, 0x27, 0x00, 0xc0, 0x4e, 0x00, \
  0x14, 0x38, 0xa0, 0x28, 0x71, 0x40, \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x62, 0x7c, 0x84, 0xc4, 0xf9, 0x08, \
  0x8c, 0x04, 0x89, 0x18, 0x09, 0x10, \
  0x01, 0x74, 0x22, 0x02, 0xe8, 0x44, \
  0x07, 0x83, 0x06, 0x0f, 0x06, 0x0c, \
  0xa0, 0x80, 0x73, 0x41, 0x00, 0xe4, \
  0x18, 0xb1, 0x42, 0x31, 0x62, 0x84, \
  0x91, 0x00, 0x93, 0x22, 0x01, 0x24, \
  0x78, 0x00, 0x1c, 0xf0, 0x00, 0x38

#define kMaskRandom46_34 \
  0x0c, 0x84, 0x0c, 0x19, 0x08, 0x18, \
  0x80, 0x70, 0x07, 0x00, 0xe0, 0x0c, \
  0xa0, 0x88, 0x49, 0x41, 0x10, 0x90, \
  0x05, 0x40, 0x32, 0x0a, 0x80, 0x64, \
  0x43, 0x02, 0x82, 0x86, 0x05, 0x04, \
  0x1a, 0x01, 0x50, 0x34, 0x02, 0xa0, \
  0x60, 0x27, 0x00, 0xc0, 0x4e, 0x00, \
  0x14, 0x38, 0xa0, 0x28, 0x71, 0x40, \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x62, 0x7c, 0x84, 0xc4, 0xf9, 0x08, \
  0x8c, 0x04, 0x89, 0x18, 0x09, 0x10, \
  0x01, 0x74, 0x22, 0x02, 0xe8, 0x44, \
  0x07, 0x83, 0x06, 0x0f, 0x06, 0x0c, \
  0xa0, 0x80, 0x73, 0x41, 0x00, 0xe4, \
  0x18, 0xb1, 0x42, 0x31, 0x62, 0x84, \
  0x91, 0x00, 0x93, 0x22, 0x01, 0x24, \
  0x78, 0x00, 0x1c, 0xf0, 0x00, 0x38, \
  0x0c, 0x84, 0x0c, 0x19, 0x08, 0x18, \
  0x80, 0x70, 0x07, 0x00, 0xe0, 0x0c, \
  0xa0, 0x88, 0x49, 0x41, 0x10, 0x90, \
  0x05, 0x40, 0x32, 0x0a, 0x80, 0x64, \
  0x43, 0x02, 0x82, 0x86, 0x05, 0x04, \
  0x1a, 0x01, 0x50, 0x34, 0x02, 0xa0, \
  0x60, 0x27, 0x00, 0xc0, 0x4e, 0x00, \
  0x14, 0x38, 0xa0, 0x28, 0x71, 0x40, \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x33, 0x24, 0x26, 0x66, 0x48, 0x4c, \
  0x91, 0x92, 0x13, 0x23, 0x24, 0x24, \
  0xa4, 0x20, 0x4b, 0x48, 0x40, 0x94, \
  0x50, 0xa0, 0xd4, 0xa1, 0x41, 0xa8, \
  0x84, 0xc5, 0x81, 0x09, 0x8b, 0x00, \
  0x09, 0x71, 0x0c, 0x12, 0xe2, 0x18, \
  0xfa, 0xd9, 0xf5, 0xf5, 0xb3, 0xe8, \
  0x60, 0xf0, 0x13, 0xf0, 0x4d, 0xe0

#define kMaskRandom46_35 \
  0x0c, 0x84, 0x0c, 0x19, 0x08, 0x18, \
  0x80, 0x70, 0x07, 0x00, 0xe0, 0x0c, \
  0xa0, 0x88, 0x49, 0x41, 0x10, 0x90, \
  0x05, 0x40, 0x32, 0x0a, 0x80, 0x64, \
  0x43, 0x02, 0x82, 0x86, 0x05, 0x04, \
  0x1a, 0x01, 0x50, 0x34, 0x02, 0xa0, \
  0x60, 0x27, 0x00, 0xc0, 0x4e, 0x00, \
  0x14, 0x38, 0xa0, 0x28, 0x71, 0x40, \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x62, 0x7c, 0x84, 0xc4, 0xf9, 0x08, \
  0x8c, 0x04, 0x89, 0x18, 0x09, 0x10, \
  0x01, 0x74, 0x22, 0x02, 0xe8, 0x44, \
  0x07, 0x83, 0x06, 0x0f, 0x06, 0x0c, \
  0xa0, 0x80, 0x73, 0x41, 0x00, 0xe4, \
  0x18, 0xb1, 0x42, 0x31, 0x62, 0x84, \
  0x91, 0x00, 0x93, 0x22, 0x01, 0x24, \
  0x78, 0x00, 0x1c, 0xf0, 0x00, 0x38, \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x62, 0x7c, 0x84, 0xc4, 0xf9, 0x08, \
  0x8c, 0x04, 0x89, 0x18, 0x09, 0x10, \
  0x01, 0x74, 0x22, 0x02, 0xe8, 0x44, \
  0x07, 0x83, 0x06, 0x0f, 0x06, 0x0c, \
  0xa0, 0x80, 0x73, 0x41, 0x00, 0xe4, \
  0x18, 0xb1, 0x42, 0x31, 0x62, 0x84, \
  0x91, 0x00, 0x93, 0x22, 0x01, 0x24, \
  0x78, 0x00, 0x1c, 0xf0, 0x00, 0x38, \
  0x0c, 0x84, 0x0c, 0x19, 0x08, 0x18, \
  0x80, 0x70, 0x07, 0x00, 0xe0, 0x0c, \
  0xa0, 0x88, 0x49, 0x41, 0x10, 0x90, \
  0x05, 0x40, 0x32, 0x0a, 0x80, 0x64, \
  0x43, 0x02, 0x82, 0x86, 0x05, 0x04, \
  0x1a, 0x01, 0x50, 0x34, 0x02, 0xa0, \
  0x60, 0x27, 0x00, 0xc0, 0x4e, 0x00, \
  0x14, 0x38, 0xa0, 0x28, 0x71, 0x40, \
  0x82, 0x32, 0x57, 0x04, 0x64, 0xac

#define kMaskRandom46_36 \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x62, 0x7c, 0x84, 0xc4, 0xf9, 0x08, \
  0x8c, 0x04, 0x89, 0x18, 0x09, 0x10, \
  0x01, 0x74, 0x22, 0x02, 0xe8, 0x44, \
  0x07, 0x83, 0x06, 0x0f, 0x06, 0x0c, \
  0xa0, 0x80, 0x73, 0x41, 0x00, 0xe4, \
  0x18, 0xb1, 0x42, 0x31, 0x62, 0x84, \
  0x91, 0x00, 0x93, 0x22, 0x01, 0x24, \
  0x78, 0x00, 0x1c, 0xf0, 0x00, 0x38, \
  0x0c, 0x84, 0x0c, 0x19, 0x08, 0x18, \
  0x80, 0x70, 0x07, 0x00, 0xe0, 0x0c, \
  0xa0, 0x88, 0x49, 0x41, 0x10, 0x90, \
  0x05, 0x40, 0x32, 0x0a, 0x80, 0x64, \
  0x43, 0x02, 0x82, 0x86, 0x05, 0x04, \
  0x1a, 0x01, 0x50, 0x34, 0x02, 0xa0, \
  0x60, 0x27, 0x00, 0xc0, 0x4e, 0x00, \
  0x14, 0x38, 0xa0, 0x28, 0x71, 0x40, \
  0x82, 0x32, 0x57, 0x04, 0x64, 0xac, \
  0x0c, 0x84, 0x0c, 0x19, 0x08, 0x18, \
  0x80, 0x70, 0x07, 0x00, 0xe0, 0x0c, \
  0xa0, 0x88, 0x49, 0x41, 0x10, 0x90, \
  0x05, 0x40, 0x32, 0x0a, 0x80, 0x64, \
  0x43, 0x02, 0x82, 0x86, 0x05, 0x04, \
  0x1a, 0x01, 0x50, 0x34, 0x02, 0xa0, \
  0x60, 0x27, 0x00, 0xc0, 0x4e, 0x00, \
  0x14, 0x38, 0xa0, 0x28, 0x71, 0x40, \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x62, 0x7c, 0x84, 0xc4, 0xf9, 0x08, \
  0x8c, 0x04, 0x89, 0x18, 0x09, 0x10, \
  0x01, 0x74, 0x22, 0x02, 0xe8, 0x44, \
  0x07, 0x83, 0x06, 0x0f, 0x06, 0x0c, \
  0xa0, 0x80, 0x73, 0x41, 0x00, 0xe4, \
  0x18, 0xb1, 0x42, 0x31, 0x62, 0x84, \
  0x91, 0x00, 0x93, 0x22, 0x01, 0x24, \
  0x78, 0x00, 0x1c, 0xf0, 0x00, 0x38, \
  0x6c, 0x3a, 0x45, 0x70, 0xd7, 0x00

#define kMaskRandom46_37 \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x62, 0x7c, 0x84, 0xc4, 0xf9, 0x08, \
  0x8c, 0x04, 0x89, 0x18, 0x09, 0x10, \
  0x01, 0x74, 0x22, 0x02, 0xe8, 0x44, \
  0x07, 0x83, 0x06, 0x0f, 0x06, 0x0c, \
  0xa0, 0x80, 0x73, 0x41, 0x00, 0xe4, \
  0x18, 0xb1, 0x42, 0x31, 0x62, 0x84, \
  0x91, 0x00, 0x93, 0x22, 0x01, 0x24, \
  0x78, 0x00, 0x1c, 0xf0, 0x00, 0x38, \
  0x0c, 0x84, 0x0c, 0x19, 0x08, 0x18, \
  0x80, 0x70, 0x07, 0x00, 0xe0, 0x0c, \
  0xa0, 0x88, 0x49, 0x41, 0x10, 0x90, \
  0x05, 0x40, 0x32, 0x0a, 0x80, 0x64, \
  0x43, 0x02, 0x82, 0x86, 0x05, 0x04, \
  0x1a, 0x01, 0x50, 0x34, 0x02, 0xa0, \
  0x60, 0x27, 0x00, 0xc0, 0x4e, 0x00, \
  0x14, 0x38, 0xa0, 0x28, 0x71, 0x40, \
  0x82, 0x32, 0x57, 0x04, 0x64, 0xac, \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x62, 0x7c, 0x84, 0xc4, 0xf9, 0x08, \
  0x8c, 0x04, 0x89, 0x18, 0x09, 0x10, \
  0x01, 0x74, 0x22, 0x02, 0xe8, 0x44, \
  0x07, 0x83, 0x06, 0x0f, 0x06, 0x0c, \
  0xa0, 0x80, 0x73, 0x41, 0x00, 0xe4, \
  0x18, 0xb1, 0x42, 0x31, 0x62, 0x84, \
  0x91, 0x00, 0x93, 0x22, 0x01, 0x24, \
  0x78, 0x00, 0x1c, 0xf0, 0x00, 0x38, \
  0x64, 0x4a, 0x28, 0xc8, 0x94, 0x50, \
  0x51, 0x48, 0xa2, 0xa2, 0x91, 0x44, \
  0xa9, 0x10, 0x1b, 0x52, 0x20, 0x34, \
  0x04, 0xc4, 0x84, 0x09, 0x89, 0x08, \
  0xd0, 0x01, 0x45, 0xa0, 0x02, 0x88, \
  0x82, 0x40, 0x1d, 0x04, 0x80, 0x38, \
  0x21, 0x37, 0x00, 0x42, 0x6e, 0x00, \
  0x0c, 0x21, 0x22, 0x18, 0x42, 0x44, \
  0x4a, 0x0a, 0xc0, 0x94, 0x15, 0x80, \
  0x12, 0xb4, 0x50, 0x25, 0x68, 0xa0

#define kMaskRandom46_38 \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x62, 0x7c, 0x84, 0xc4, 0xf9, 0x08, \
  0x8c, 0x04, 0x89, 0x18, 0x09, 0x10, \
  0x01, 0x74, 0x22, 0x02, 0xe8, 0x44, \
  0x07, 0x83, 0x06, 0x0f, 0x06, 0x0c, \
  0xa0, 0x80, 0x73, 0x41, 0x00, 0xe4, \
  0x18, 0xb1, 0x42, 0x31, 0x62, 0x84, \
  0x91, 0x00, 0x93, 0x22, 0x01, 0x24, \
  0x78, 0x00, 0x1c, 0xf0, 0x00, 0x38, \
  0x64, 0x4a, 0x28, 0xc8, 0x94, 0x50, \
  0x51, 0x48, 0xa2, 0xa2, 0x91, 0x44, \
  0xa9, 0x10, 0x1b, 0x52, 0x20, 0x34, \
  0x04, 0xc4, 0x84, 0x09, 0x89, 0x08, \
  0xd0, 0x01, 0x45, 0xa0, 0x02, 0x88, \
  0x82, 0x40, 0x1d, 0x04, 0x80, 0x38, \
  0x21, 0x37, 0x00, 0x42, 0x6e, 0x00, \
  0x0c, 0x21, 0x22, 0x18, 0x42, 0x44, \
  0x4a, 0x0a, 0xc0, 0x94, 0x15, 0x80, \
  0x12, 0xb4, 0x50, 0x25, 0x68, 0xa0, \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x62, 0x7c, 0x84, 0xc4, 0xf9, 0x08, \
  0x8c, 0x04, 0x89, 0x18, 0x09, 0x10, \
  0x01, 0x74, 0x22, 0x02, 0xe8, 0x44, \
  0x07, 0x83, 0x06, 0x0f, 0x06, 0x0c, \
  0xa0, 0x80, 0x73, 0x41, 0x00, 0xe4, \
  0x18, 0xb1, 0x42, 0x31, 0x62, 0x84, \
  0x91, 0x00, 0x93, 0x22, 0x01, 0x24, \
  0x78, 0x00, 0x1c, 0xf0, 0x00, 0x38, \
  0x0c, 0x84, 0x0c, 0x19, 0x08, 0x18, \
  0x80, 0x70, 0x07, 0x00, 0xe0, 0x0c, \
  0xa0, 0x88, 0x49, 0x41, 0x10, 0x90, \
  0x05, 0x40, 0x32, 0x0a, 0x80, 0x64, \
  0x43, 0x02, 0x82, 0x86, 0x05, 0x04, \
  0x1a, 0x01, 0x50, 0x34, 0x02, 0xa0, \
  0x60, 0x27, 0x00, 0xc0, 0x4e, 0x00, \
  0x14, 0x38, 0xa0, 0x28, 0x71, 0x40, \
  0x82, 0x32, 0x57, 0x04, 0x64, 0xac, \
  0x72, 0x2b, 0xa5, 0xd4, 0xb9, 0x30

#define kMaskRandom46_39 \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x62, 0x7c, 0x84, 0xc4, 0xf9, 0x08, \
  0x8c, 0x04, 0x89, 0x18, 0x09, 0x10, \
  0x01, 0x74, 0x22, 0x02, 0xe8, 0x44, \
  0x07, 0x83, 0x06, 0x0f, 0x06, 0x0c, \
  0xa0, 0x80, 0x73, 0x41, 0x00, 0xe4, \
  0x18, 0xb1, 0x42, 0x31, 0x62, 0x84, \
  0x91, 0x00, 0x93, 0x22, 0x01, 0x24, \
  0x78, 0x00, 0x1c, 0xf0, 0x00, 0x38, \
  0x64, 0x4a, 0x28, 0xc8, 0x94, 0x50, \
  0x51, 0x48, 0xa2, 0xa2, 0x91, 0x44, \
  0xa9, 0x10, 0x1b, 0x52, 0x20, 0x34, \
  0x04, 0xc4, 0x84, 0x09, 0x89, 0x08, \
  0xd0, 0x01, 0x45, 0xa0, 0x02, 0x88, \
  0x82, 0x40, 0x1d, 0x04, 0x80, 0x38, \
  0x21, 0x37, 0x00, 0x42, 0x6e, 0x00, \
  0x0c, 0x21, 0x22, 0x18, 0x42, 0x44, \
  0x4a, 0x0a, 0xc0, 0x94, 0x15, 0x80, \
  0x12, 0xb4, 0x50, 0x25, 0x68, 0xa0, \
  0x64, 0x4a, 0x28, 0xc8, 0x94, 0x50, \
  0x51, 0x48, 0xa2, 0xa2, 0x91, 0x44, \
  0xa9, 0x10, 0x1b, 0x52, 0x20, 0x34, \
  0x04, 0xc4, 0x84, 0x09, 0x89, 0x08, \
  0xd0, 0x01, 0x45, 0xa0, 0x02, 0x88, \
  0x82, 0x40, 0x1d, 0x04, 0x80, 0x38, \
  0x21, 0x37, 0x00, 0x42, 0x6e, 0x00, \
  0x0c, 0x21, 0x22, 0x18, 0x42, 0x44, \
  0x4a, 0x0a, 0xc0, 0x94, 0x15, 0x80, \
  0x12, 0xb4, 0x50, 0x25, 0x68, 0xa0, \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x62, 0x7c, 0x84, 0xc4, 0xf9, 0x08, \
  0x8c, 0x04, 0x89, 0x18, 0x09, 0x10, \
  0x01, 0x74, 0x22, 0x02, 0xe8, 0x44, \
  0x07, 0x83, 0x06, 0x0f, 0x06, 0x0c, \
  0xa0, 0x80, 0x73, 0x41, 0x00, 0xe4, \
  0x18, 0xb1, 0x42, 0x31, 0x62, 0x84, \
  0x91, 0x00, 0x93, 0x22, 0x01, 0x24, \
  0x78, 0x00, 0x1c, 0xf0, 0x00, 0x38, \
  0xdb, 0x4a, 0x7b, 0xb6, 0x94, 0xf4

#define kMaskRandom46_4 \
  0xc4, 0xd1, 0x65, 0x89, 0xa2, 0xc8, \
  0x31, 0x62, 0x96, 0x62, 0xc5, 0x2c, \
  0x4b, 0x24, 0x5a, 0x96, 0x48, 0xb4, \
  0x2c, 0xa8, 0xaa, 0x59, 0x51, 0x54

#define kMaskRandom46_40 \
  0x64, 0x4a, 0x28, 0xc8, 0x94, 0x50, \
  0x51, 0x48, 0xa2, 0xa2, 0x91, 0x44, \
  0xa9, 0x10, 0x1b, 0x52, 0x20, 0x34, \
  0x04, 0xc4, 0x84, 0x09, 0x89, 0x08, \
  0xd0, 0x01, 0x45, 0xa0, 0x02, 0x88, \
  0x82, 0x40, 0x1d, 0x04, 0x80, 0x38, \
  0x21, 0x37, 0x00, 0x42, 0x6e, 0x00, \
  0x0c, 0x21, 0x22, 0x18, 0x42, 0x44, \
  0x4a, 0x0a, 0xc0, 0x94, 0x15, 0x80, \
  0x12, 0xb4, 0x50, 0x25, 0x68, 0xa0, \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x62, 0x7c, 0x84, 0xc4, 0xf9, 0x08, \
  0x8c, 0x04, 0x89, 0x18, 0x09, 0x10, \
  0x01, 0x74, 0x22, 0x02, 0xe8, 0x44, \
  0x07, 0x83, 0x06, 0x0f, 0x06, 0x0c, \
  0xa0, 0x80, 0x73, 0x41, 0x00, 0xe4, \
  0x18, 0xb1, 0x42, 0x31, 0x62, 0x84, \
  0x91, 0x00, 0x93, 0x22, 0x01, 0x24, \
  0x78, 0x00, 0x1c, 0xf0, 0x00, 0x38, \
  0xdb, 0x4a, 0x7b, 0xb6, 0x94, 0xf4, \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x62, 0x7c, 0x84, 0xc4, 0xf9, 0x08, \
  0x8c, 0x04, 0x89, 0x18, 0x09, 0x10, \
  0x01, 0x74, 0x22, 0x02, 0xe8, 0x44, \
  0x07, 0x83, 0x06, 0x0f, 0x06, 0x0c, \
  0xa0, 0x80, 0x73, 0x41, 0x00, 0xe4, \
  0x18, 0xb1, 0x42, 0x31, 0x62, 0x84, \
  0x91, 0x00, 0x93, 0x22, 0x01, 0x24, \
  0x78, 0x00, 0x1c, 0xf0, 0x00, 0x38, \
  0x64, 0x4a, 0x28, 0xc8, 0x94, 0x50, \
  0x51, 0x48, 0xa2, 0xa2, 0x91, 0x44, \
  0xa9, 0x10, 0x1b, 0x52, 0x20, 0x34, \
  0x04, 0xc4, 0x84, 0x09, 0x89, 0x08, \
  0xd0, 0x01, 0x45, 0xa0, 0x02, 0x88, \
  0x82, 0x40, 0x1d, 0x04, 0x80, 0x38, \
  0x21, 0x37, 0x00, 0x42, 0x6e, 0x00, \
  0x0c, 0x21, 0x22, 0x18, 0x42, 0x44, \
  0x4a, 0x0a, 0xc0, 0x94, 0x15, 0x80, \
  0x12, 0xb4, 0x50, 0x25, 0x68, 0xa0, \
  0x7c, 0xc8, 0x93, 0x63, 0x3c, 0x80

#define kMaskRandom46_41 \
  0x64, 0x4a, 0x28, 0xc8, 0x94, 0x50, \
  0x51, 0x48, 0xa2, 0xa2, 0x91, 0x44, \
  0xa9, 0x10, 0x1b, 0x52, 0x20, 0x34, \
  0x04, 0xc4, 0x84, 0x09, 0x89, 0x08, \
  0xd0, 0x01, 0x45, 0xa0, 0x02, 0x88, \
  0x82, 0x40, 0x1d, 0x04, 0x80, 0x38, \
  0x21, 0x37, 0x00, 0x42, 0x6e, 0x00, \
  0x0c, 0x21, 0x22, 0x18, 0x42, 0x44, \
  0x4a, 0x0a, 0xc0, 0x94, 0x15, 0x80, \
  0x12, 0xb4, 0x50, 0x25, 0x68, 0xa0, \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x62, 0x7c, 0x84, 0xc4, 0xf9, 0x08, \
  0x8c, 0x04, 0x89, 0x18, 0x09, 0x10, \
  0x01, 0x74, 0x22, 0x02, 0xe8, 0x44, \
  0x07, 0x83, 0x06, 0x0f, 0x06, 0x0c, \
  0xa0, 0x80, 0x73, 0x41, 0x00, 0xe4, \
  0x18, 0xb1, 0x42, 0x31, 0x62, 0x84, \
  0x91, 0x00, 0x93, 0x22, 0x01, 0x24, \
  0x78, 0x00, 0x1c, 0xf0, 0x00, 0x38, \
  0xdb, 0x4a, 0x7b, 0xb6, 0x94, 0xf4, \
  0x64, 0x4a, 0x28, 0xc8, 0x94, 0x50, \
  0x51, 0x48, 0xa2, 0xa2, 0x91, 0x44, \
  0xa9, 0x10, 0x1b, 0x52, 0x20, 0x34, \
  0x04, 0xc4, 0x84, 0x09, 0x89, 0x08, \
  0xd0, 0x01, 0x45, 0xa0, 0x02, 0x88, \
  0x82, 0x40, 0x1d, 0x04, 0x80, 0x38, \
  0x21, 0x37, 0x00, 0x42, 0x6e, 0x00, \
  0x0c, 0x21, 0x22, 0x18, 0x42, 0x44, \
  0x4a, 0x0a, 0xc0, 0x94, 0x15, 0x80, \
  0x12, 0xb4, 0x50, 0x25, 0x68, 0xa0, \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x33, 0x24, 0x26, 0x66, 0x48, 0x4c, \
  0x99, 0x02, 0x13, 0x32, 0x04, 0x24, \
  0x05, 0x80, 0x0e, 0x0b, 0x00, 0x1c, \
  0x80, 0xa1, 0x83, 0x01, 0x43, 0x04, \
  0x84, 0x48, 0x19, 0x08, 0x90, 0x30, \
  0x40, 0x6d, 0x40, 0x80, 0xda, 0x80, \
  0x0a, 0x90, 0xc0, 0x15, 0x21, 0x80, \
  0x68, 0x04, 0x90, 0xd0, 0x09, 0x20, \
  0x10, 0x31, 0x20, 0x20, 0x62, 0x40, \
  0x30, 0x58, 0x04, 0x60, 0xb0, 0x08

#define kMaskRandom46_42 \
  0x64, 0x4a, 0x28, 0xc8, 0x94, 0x50, \
  0x51, 0x48, 0xa2, 0xa2, 0x91, 0x44, \
  0xa9, 0x10, 0x1b, 0x52, 0x20, 0x34, \
  0x04, 0xc4, 0x84, 0x09, 0x89, 0x08, \
  0xd0, 0x01, 0x45, 0xa0, 0x02, 0x88, \
  0x82, 0x40, 0x1d, 0x04, 0x80, 0x38, \
  0x21, 0x37, 0x00, 0x42, 0x6e, 0x00, \
  0x0c, 0x21, 0x22, 0x18, 0x42, 0x44, \
  0x4a, 0x0a, 0xc0, 0x94, 0x15, 0x80, \
  0x12, 0xb4, 0x50, 0x25, 0x68, 0xa0, \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x33, 0x24, 0x26, 0x66, 0x48, 0x4c, \
  0x99, 0x02, 0x13, 0x32, 0x04, 0x24, \
  0x05, 0x80, 0x0e, 0x0b, 0x00, 0x1c, \
  0x80, 0xa1, 0x83, 0x01, 0x43, 0x04, \
  0x84, 0x48, 0x19, 0x08, 0x90, 0x30, \
  0x40, 0x6d, 0x40, 0x80, 0xda, 0x80, \
  0x0a, 0x90, 0xc0, 0x15, 0x21, 0x80, \
  0x68, 0x04, 0x90, 0xd0, 0x09, 0x20, \
  0x10, 0x31, 0x20, 0x20, 0x62, 0x40, \
  0x30, 0x58, 0x04, 0x60, 0xb0, 0x08, \
  0x64, 0x4a, 0x28, 0xc8, 0x94, 0x50, \
  0x51, 0x48, 0xa2, 0xa2, 0x91, 0x44, \
  0xa9, 0x10, 0x1b, 0x52, 0x20, 0x34, \
  0x04, 0xc4, 0x84, 0x09, 0x89, 0x08, \
  0xd0, 0x01, 0x45, 0xa0, 0x02, 0x88, \
  0x82, 0x40, 0x1d, 0x04, 0x80, 0x38, \
  0x21, 0x37, 0x00, 0x42, 0x6e, 0x00, \
  0x0c, 0x21, 0x22, 0x18, 0x42, 0x44, \
  0x4a, 0x0a, 0xc0, 0x94, 0x15, 0x80, \
  0x12, 0xb4, 0x50, 0x25, 0x68, 0xa0, \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x62, 0x7c, 0x84, 0xc4, 0xf9, 0x08, \
  0x8c, 0x04, 0x89, 0x18, 0x09, 0x10, \
  0x01, 0x74, 0x22, 0x02, 0xe8, 0x44, \
  0x07, 0x83, 0x06, 0x0f, 0x06, 0x0c, \
  0xa0, 0x80, 0x73, 0x41, 0x00, 0xe4, \
  0x18, 0xb1, 0x42, 0x31, 0x62, 0x84, \
  0x91, 0x00, 0x93, 0x22, 0x01, 0x24, \
  0x78, 0x00, 0x1c, 0xf0, 0x00, 0x38, \
  0xdb, 0x4a, 0x7b, 0xb6, 0x94, 0xf4, \
  0xfc, 0x6e, 0x89, 0x54, 0x4f, 0x00

#define kMaskRandom46_43 \
  0x64, 0x4a, 0x28, 0xc8, 0x94, 0x50, \
  0x51, 0x48, 0xa2, 0xa2, 0x91, 0x44, \
  0xa9, 0x10, 0x1b, 0x52, 0x20, 0x34, \
  0x04, 0xc4, 0x84, 0x09, 0x89, 0x08, \
  0xd0, 0x01, 0x45, 0xa0, 0x02, 0x88, \
  0x82, 0x40, 0x1d, 0x04, 0x80, 0x38, \
  0x21, 0x37, 0x00, 0x42, 0x6e, 0x00, \
  0x0c, 0x21, 0x22, 0x18, 0x42, 0x44, \
  0x4a, 0x0a, 0xc0, 0x94, 0x15, 0x80, \
  0x12, 0xb4, 0x50, 0x25, 0x68, 0xa0, \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x33, 0x24, 0x26, 0x66, 0x48, 0x4c, \
  0x99, 0x02, 0x13, 0x32, 0x04, 0x24, \
  0x05, 0x80, 0x0e, 0x0b, 0x00, 0x1c, \
  0x80, 0xa1, 0x83, 0x01, 0x43, 0x04, \
  0x84, 0x48, 0x19, 0x08, 0x90, 0x30, \
  0x40, 0x6d, 0x40, 0x80, 0xda, 0x80, \
  0x0a, 0x90, 0xc0, 0x15, 0x21, 0x80, \
  0x68, 0x04, 0x90, 0xd0, 0x09, 0x20, \
  0x10, 0x31, 0x20, 0x20, 0x62, 0x40, \
  0x30, 0x58, 0x04, 0x60, 0xb0, 0x08, \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x33, 0x24, 0x26, 0x66, 0x48, 0x4c, \
  0x99, 0x02, 0x13, 0x32, 0x04, 0x24, \
  0x05, 0x80, 0x0e, 0x0b, 0x00, 0x1c, \
  0x80, 0xa1, 0x83, 0x01, 0x43, 0x04, \
  0x84, 0x48, 0x19, 0x08, 0x90, 0x30, \
  0x40, 0x6d, 0x40, 0x80, 0xda, 0x80, \
  0x0a, 0x90, 0xc0, 0x15, 0x21, 0x80, \
  0x68, 0x04, 0x90, 0xd0, 0x09, 0x20, \
  0x10, 0x31, 0x20, 0x20, 0x62, 0x40, \
  0x30, 0x58, 0x04, 0x60, 0xb0, 0x08, \
  0x64, 0x4a, 0x28, 0xc8, 0x94, 0x50, \
  0x51, 0x48, 0xa2, 0xa2, 0x91, 0x44, \
  0xa9, 0x10, 0x1b, 0x52, 0x20, 0x34, \
  0x04, 0xc4, 0x84, 0x09, 0x89, 0x08, \
  0xd0, 0x01, 0x45, 0xa0, 0x02, 0x88, \
  0x82, 0x40, 0x1d, 0x04, 0x80, 0x38, \
  0x21, 0x37, 0x00, 0x42, 0x6e, 0x00, \
  0x0c, 0x21, 0x22, 0x18, 0x42, 0x44, \
  0x4a, 0x0a, 0xc0, 0x94, 0x15, 0x80, \
  0x12, 0xb4, 0x50, 0x25, 0x68, 0xa0, \
  0xea, 0x8d, 0x1b, 0xd5, 0x1a, 0x34

#define kMaskRandom46_44 \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x33, 0x24, 0x26, 0x66, 0x48, 0x4c, \
  0x99, 0x02, 0x13, 0x32, 0x04, 0x24, \
  0x05, 0x80, 0x0e, 0x0b, 0x00, 0x1c, \
  0x80, 0xa1, 0x83, 0x01, 0x43, 0x04, \
  0x84, 0x48, 0x19, 0x08, 0x90, 0x30, \
  0x40, 0x6d, 0x40, 0x80, 0xda, 0x80, \
  0x0a, 0x90, 0xc0, 0x15, 0x21, 0x80, \
  0x68, 0x04, 0x90, 0xd0, 0x09, 0x20, \
  0x10, 0x31, 0x20, 0x20, 0x62, 0x40, \
  0x30, 0x58, 0x04, 0x60, 0xb0, 0x08, \
  0x64, 0x4a, 0x28, 0xc8, 0x94, 0x50, \
  0x51, 0x48, 0xa2, 0xa2, 0x91, 0x44, \
  0xa9, 0x10, 0x1b, 0x52, 0x20, 0x34, \
  0x04, 0xc4, 0x84, 0x09, 0x89, 0x08, \
  0xd0, 0x01, 0x45, 0xa0, 0x02, 0x88, \
  0x82, 0x40, 0x1d, 0x04, 0x80, 0x38, \
  0x21, 0x37, 0x00, 0x42, 0x6e, 0x00, \
  0x0c, 0x21, 0x22, 0x18, 0x42, 0x44, \
  0x4a, 0x0a, 0xc0, 0x94, 0x15, 0x80, \
  0x12, 0xb4, 0x50, 0x25, 0x68, 0xa0, \
  0xea, 0x8d, 0x1b, 0xd5, 0x1a, 0x34, \
  0x64, 0x4a, 0x28, 0xc8, 0x94, 0x50, \
  0x51, 0x48, 0xa2, 0xa2, 0x91, 0x44, \
  0xa9, 0x10, 0x1b, 0x52, 0x20, 0x34, \
  0x04, 0xc4, 0x84, 0x09, 0x89, 0x08, \
  0xd0, 0x01, 0x45, 0xa0, 0x02, 0x88, \
  0x82, 0x40, 0x1d, 0x04, 0x80, 0x38, \
  0x21, 0x37, 0x00, 0x42, 0x6e, 0x00, \
  0x0c, 0x21, 0x22, 0x18, 0x42, 0x44, \
  0x4a, 0x0a, 0xc0, 0x94, 0x15, 0x80, \
  0x12, 0xb4, 0x50, 0x25, 0x68, 0xa0, \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x33, 0x24, 0x26, 0x66, 0x48, 0x4c, \
  0x99, 0x02, 0x13, 0x32, 0x04, 0x24, \
  0x05, 0x80, 0x0e, 0x0b, 0x00, 0x1c, \
  0x80, 0xa1, 0x83, 0x01, 0x43, 0x04, \
  0x84, 0x48, 0x19, 0x08, 0x90, 0x30, \
  0x40, 0x6d, 0x40, 0x80, 0xda, 0x80, \
  0x0a, 0x90, 0xc0, 0x15, 0x21, 0x80, \
  0x68, 0x04, 0x90, 0xd0, 0x09, 0x20, \
  0x10, 0x31, 0x20, 0x20, 0x62, 0x40, \
  0x30, 0x58, 0x04, 0x60, 0xb0, 0x08, \
  0x1a, 0x8a, 0x00, 0x1c, 0x89, 0x54

#define kMaskRandom46_45 \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x33, 0x24, 0x26, 0x66, 0x48, 0x4c, \
  0x99, 0x02, 0x13, 0x32, 0x04, 0x24, \
  0x05, 0x80, 0x0e, 0x0b, 0x00, 0x1c, \
  0x80, 0xa1, 0x83, 0x01, 0x43, 0x04, \
  0x84, 0x48, 0x19, 0x08, 0x90, 0x30, \
  0x40, 0x6d, 0x40, 0x80, 0xda, 0x80, \
  0x0a, 0x90, 0xc0, 0x15, 0x21, 0x80, \
  0x68, 0x04, 0x90, 0xd0, 0x09, 0x20, \
  0x10, 0x31, 0x20, 0x20, 0x62, 0x40, \
  0x30, 0x58, 0x04, 0x60, 0xb0, 0x08, \
  0x64, 0x4a, 0x28, 0xc8, 0x94, 0x50, \
  0x51, 0x48, 0xa2, 0xa2, 0x91, 0x44, \
  0xa9, 0x10, 0x1b, 0x52, 0x20, 0x34, \
  0x04, 0xc4, 0x84, 0x09, 0x89, 0x08, \
  0xd0, 0x01, 0x45, 0xa0, 0x02, 0x88, \
  0x82, 0x40, 0x1d, 0x04, 0x80, 0x38, \
  0x21, 0x37, 0x00, 0x42, 0x6e, 0x00, \
  0x0c, 0x21, 0x22, 0x18, 0x42, 0x44, \
  0x4a, 0x0a, 0xc0, 0x94, 0x15, 0x80, \
  0x12, 0xb4, 0x50, 0x25, 0x68, 0xa0, \
  0xea, 0x8d, 0x1b, 0xd5, 0x1a, 0x34, \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x33, 0x24, 0x26, 0x66, 0x48, 0x4c, \
  0x99, 0x02, 0x13, 0x32, 0x04, 0x24, \
  0x05, 0x80, 0x0e, 0x0b, 0x00, 0x1c, \
  0x80, 0xa1, 0x83, 0x01, 0x43, 0x04, \
  0x84, 0x48, 0x19, 0x08, 0x90, 0x30, \
  0x40, 0x6d, 0x40, 0x80, 0xda, 0x80, \
  0x0a, 0x90, 0xc0, 0x15, 0x21, 0x80, \
  0x68, 0x04, 0x90, 0xd0, 0x09, 0x20, \
  0x10, 0x31, 0x20, 0x20, 0x62, 0x40, \
  0x30, 0x58, 0x04, 0x60, 0xb0, 0x08, \
  0x46, 0x42, 0x0c, 0x8c, 0x84, 0x18, \
  0x33, 0x20, 0x46, 0x66, 0x40, 0x8c, \
  0x99, 0x08, 0x0b, 0x32, 0x10, 0x14, \
  0x05, 0x84, 0x30, 0x0b, 0x08, 0x60, \
  0x80, 0xb0, 0x23, 0x01, 0x60, 0x44, \
  0x84, 0x42, 0x91, 0x08, 0x85, 0x20, \
  0x40, 0x73, 0x00, 0x80, 0xe6, 0x00, \
  0x0a, 0x81, 0x12, 0x15, 0x02, 0x24, \
  0x68, 0x0c, 0x40, 0xd0, 0x18, 0x80, \
  0x10, 0x24, 0x84, 0x20, 0x49, 0x08, \
  0x30, 0x51, 0x40, 0x60, 0xa2, 0x80, \
  0x5f, 0x50, 0x88, 0xbe, 0xa1, 0x10

#define kMaskRandom46_46 \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x33, 0x24, 0x26, 0x66, 0x48, 0x4c, \
  0x99, 0x02, 0x13, 0x32, 0x04, 0x24, \
  0x05, 0x80, 0x0e, 0x0b, 0x00, 0x1c, \
  0x80, 0xa1, 0x83, 0x01, 0x43, 0x04, \
  0x84, 0x48, 0x19, 0x08, 0x90, 0x30, \
  0x40, 0x6d, 0x40, 0x80, 0xda, 0x80, \
  0x0a, 0x90, 0xc0, 0x15, 0x21, 0x80, \
  0x68, 0x04, 0x90, 0xd0, 0x09, 0x20, \
  0x10, 0x31, 0x20, 0x20, 0x62, 0x40, \
  0x30, 0x58, 0x04, 0x60, 0xb0, 0x08, \
  0x46, 0x42, 0x0c, 0x8c, 0x84, 0x18, \
  0x33, 0x20, 0x46, 0x66, 0x40, 0x8c, \
  0x99, 0x08, 0x0b, 0x32, 0x10, 0x14, \
  0x05, 0x84, 0x30, 0x0b, 0x08, 0x60, \
  0x80, 0xb0, 0x23, 0x01, 0x60, 0x44, \
  0x84, 0x42, 0x91, 0x08, 0x85, 0x20, \
  0x40, 0x73, 0x00, 0x80, 0xe6, 0x00, \
  0x0a, 0x81, 0x12, 0x15, 0x02, 0x24, \
  0x68, 0x0c, 0x40, 0xd0, 0x18, 0x80, \
  0x10, 0x24, 0x84, 0x20, 0x49, 0x08, \
  0x30, 0x51, 0x40, 0x60, 0xa2, 0x80, \
  0x5f, 0x50, 0x88, 0xbe, 0xa1, 0x10, \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x33, 0x24, 0x26, 0x66, 0x48, 0x4c, \
  0x99, 0x02, 0x13, 0x32, 0x04, 0x24, \
  0x05, 0x80, 0x0e, 0x0b, 0x00, 0x1c, \
  0x80, 0xa1, 0x83, 0x01, 0x43, 0x04, \
  0x84, 0x48, 0x19, 0x08, 0x90, 0x30, \
  0x40, 0x6d, 0x40, 0x80, 0xda, 0x80, \
  0x0a, 0x90, 0xc0, 0x15, 0x21, 0x80, \
  0x68, 0x04, 0x90, 0xd0, 0x09, 0x20, \
  0x10, 0x31, 0x20, 0x20, 0x62, 0x40, \
  0x30, 0x58, 0x04, 0x60, 0xb0, 0x08, \
  0x64, 0x4a, 0x28, 0xc8, 0x94, 0x50, \
  0x51, 0x48, 0xa2, 0xa2, 0x91, 0x44, \
  0xa9, 0x10, 0x1b, 0x52, 0x20, 0x34, \
  0x04, 0xc4, 0x84, 0x09, 0x89, 0x08, \
  0xd0, 0x01, 0x45, 0xa0, 0x02, 0x88, \
  0x82, 0x40, 0x1d, 0x04, 0x80, 0x38, \
  0x21, 0x37, 0x00, 0x42, 0x6e, 0x00, \
  0x0c, 0x21, 0x22, 0x18, 0x42, 0x44, \
  0x4a, 0x0a, 0xc0, 0x94, 0x15, 0x80, \
  0x12, 0xb4, 0x50, 0x25, 0x68, 0xa0, \
  0xea, 0x8d, 0x1b, 0xd5, 0x1a, 0x34, \
  0xd5, 0xdf, 0x59, 0xb9, 0xba, 0x10

#define kMaskRandom46_5 \
  0xc6, 0xca, 0x6d, 0x8d, 0x94, 0xd8, \
  0x63, 0x6c, 0x96, 0xc6, 0xd9, 0x2c, \
  0x1d, 0xa1, 0xdc, 0x3b, 0x43, 0xb8, \
  0xad, 0x55, 0x39, 0x5a, 0xaa, 0x70, \
  0xb2, 0xb7, 0x07, 0x65, 0x6e, 0x0c

#define kMaskRandom46_6 \
  0x64, 0x4a, 0x28, 0xc8, 0x94, 0x50, \
  0x51, 0x58, 0xa2, 0xa2, 0xb1, 0x44, \
  0x0c, 0xa4, 0x30, 0x19, 0x48, 0x60, \
  0xa1, 0x22, 0x47, 0x42, 0x44, 0x8c, \
  0x12, 0xa1, 0x1c, 0x25, 0x42, 0x38, \
  0x8a, 0x45, 0xc1, 0x14, 0x8b, 0x80

#define kMaskRandom46_7 \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x33, 0x24, 0x26, 0x66, 0x48, 0x4c, \
  0x91, 0x92, 0x13, 0x23, 0x24, 0x24, \
  0xa4, 0x20, 0x4b, 0x48, 0x40, 0x94, \
  0x50, 0xa0, 0xd4, 0xa1, 0x41, 0xa8, \
  0x84, 0xc5, 0x81, 0x09, 0x8b, 0x00, \
  0x09, 0x71, 0x0c, 0x12, 0xe2, 0x18

#define kMaskRandom46_8 \
  0x0c, 0x84, 0x0c, 0x19, 0x08, 0x18, \
  0x80, 0x70, 0x07, 0x00, 0xe0, 0x0c, \
  0xa0, 0x88, 0x49, 0x41, 0x10, 0x90, \
  0x05, 0x40, 0x32, 0x0a, 0x80, 0x64, \
  0x43, 0x02, 0x82, 0x86, 0x05, 0x04, \
  0x1a, 0x01, 0x50, 0x34, 0x02, 0xa0, \
  0x60, 0x27, 0x00, 0xc0, 0x4e, 0x00, \
  0x14, 0x38, 0xa0, 0x28, 0x71, 0x40

#define kMaskRandom46_9 \
  0x46, 0x4a, 0x6c, 0x8c, 0x94, 0xd8, \
  0x62, 0x7c, 0x84, 0xc4, 0xf9, 0x08, \
  0x8c, 0x04, 0x89, 0x18, 0x09, 0x10, \
  0x01, 0x74, 0x22, 0x02, 0xe8, 0x44, \
  0x07, 0x83, 0x06, 0x0f, 0x06, 0x0c, \
  0xa0, 0x80, 0x73, 0x41, 0x00, 0xe4, \
  0x18, 0xb1, 0x42, 0x31, 0x62, 0x84, \
  0x91, 0x00, 0x93, 0x22, 0x01, 0x24, \
  0x78, 0x00, 0x1c, 0xf0, 0x00, 0x38

#define kMaskRandom47_1 \
  0xff, 0xff, 0xff, 0xff, 0xff, 0xfe

#define kMaskRandom47_10 \
  0x64, 0x4a, 0x28, 0x22, 0x8a, 0x28, \
  0x51, 0x48, 0xa2, 0x8a, 0x68, 0xa6, \
  0xa9, 0x10, 0x1a, 0x00, 0x90, 0x0a, \
  0x04, 0xc4, 0x84, 0x21, 0x06, 0x12, \
  0xd0, 0x01, 0x44, 0x94, 0x29, 0x42, \
  0x82, 0x40, 0x1c, 0x81, 0x48, 0x14, \
  0x21, 0x37, 0x01, 0x40, 0xd4, 0x04, \
  0x0c, 0x21, 0x23, 0x11, 0x01, 0x18, \
  0x4a, 0x0a, 0xc1, 0x0c, 0x10, 0xc0, \
  0x12, 0xb4, 0x50, 0xa8, 0x1a, 0x80

#define kMaskRandom47_11 \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x68, \
  0x33, 0x24, 0x27, 0x40, 0x64, 0x22, \
  0x99, 0x02, 0x12, 0x2a, 0x22, 0x82, \
  0x05, 0x80, 0x0e, 0x06, 0xa0, 0x2a, \
  0x80, 0xa1, 0x83, 0x19, 0x11, 0x90, \
  0x84, 0x48, 0x18, 0x51, 0x05, 0x10, \
  0x40, 0x6d, 0x40, 0x10, 0x91, 0x08, \
  0x0a, 0x90, 0xc1, 0x32, 0x03, 0x20, \
  0x68, 0x04, 0x90, 0x45, 0x24, 0x52, \
  0x10, 0x31, 0x20, 0x8c, 0x08, 0xc0, \
  0x30, 0x58, 0x05, 0x18, 0x58, 0x04

#define kMaskRandom47_12 \
  0x64, 0x4a, 0x28, 0x20, 0xc2, 0x0c, \
  0x51, 0x58, 0xa2, 0x04, 0x60, 0x46, \
  0x0c, 0xa4, 0x30, 0x80, 0xa8, 0x0a, \
  0xa1, 0x22, 0x46, 0x43, 0x04, 0x30, \
  0x12, 0xa1, 0x1d, 0x02, 0x30, 0x22, \
  0x8a, 0x45, 0xc0, 0x29, 0x02, 0x90, \
  0x86, 0x8a, 0x6d, 0x30, 0x13, 0x00, \
  0x23, 0x2c, 0x84, 0x11, 0x21, 0x12, \
  0x16, 0x21, 0x98, 0xc4, 0x0c, 0x40, \
  0x4c, 0x30, 0x54, 0x48, 0x44, 0x84, \
  0x41, 0xc1, 0x27, 0x14, 0x11, 0x40, \
  0x19, 0x56, 0xe5, 0x08, 0x90, 0x88

#define kMaskRandom47_13 \
  0x64, 0x4a, 0x28, 0xa2, 0x8a, 0x28, \
  0x51, 0x58, 0xa3, 0x8a, 0x38, 0xa2, \
  0x0c, 0xa4, 0x30, 0x43, 0x04, 0x30, \
  0xa1, 0x22, 0x46, 0x24, 0x62, 0x46, \
  0x12, 0xa1, 0x1c, 0x11, 0xc1, 0x1c, \
  0x8a, 0x45, 0xc0, 0x5c, 0x05, 0xc0, \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x33, 0x24, 0x26, 0x42, 0x64, 0x26, \
  0x91, 0x92, 0x13, 0x21, 0x32, 0x12, \
  0xa4, 0x20, 0x4a, 0x04, 0xa0, 0x4a, \
  0x50, 0xa0, 0xd4, 0x0d, 0x40, 0xd4, \
  0x84, 0xc5, 0x80, 0x58, 0x05, 0x80, \
  0x09, 0x71, 0x0d, 0x10, 0xd1, 0x0c

#define kMaskRandom47_14 \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x33, 0x24, 0x26, 0x42, 0x64, 0x26, \
  0x91, 0x92, 0x13, 0x21, 0x32, 0x12, \
  0xa4, 0x20, 0x4a, 0x04, 0xa0, 0x4a, \
  0x50, 0xa0, 0xd4, 0x0d, 0x40, 0xd4, \
  0x84, 0xc5, 0x80, 0x58, 0x05, 0x80, \
  0x09, 0x71, 0x0d, 0x10, 0xd1, 0x0c, \
  0x64, 0x4a, 0x28, 0xa2, 0x8a, 0x28, \
  0x51, 0x58, 0xa3, 0x8a, 0x38, 0xa2, \
  0x0c, 0xa4, 0x30, 0x43, 0x04, 0x30, \
  0xa1, 0x22, 0x46, 0x24, 0x62, 0x46, \
  0x12, 0xa1, 0x1c, 0x11, 0xc1, 0x1c, \
  0x8a, 0x45, 0xc0, 0x5c, 0x05, 0xc0, \
  0x9c, 0x3f, 0xb3, 0xe5, 0xad, 0x1c

#define kMaskRandom47_15 \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x33, 0x24, 0x26, 0x42, 0x64, 0x26, \
  0x91, 0x92, 0x13, 0x21, 0x32, 0x12, \
  0xa4, 0x20, 0x4a, 0x04, 0xa0, 0x4a, \
  0x50, 0xa0, 0xd4, 0x0d, 0x40, 0xd4, \
  0x84, 0xc5, 0x80, 0x58, 0x05, 0x80, \
  0x09, 0x71, 0x0d, 0x10, 0xd1, 0x0c, \
  0x0c, 0x84, 0x0c, 0x40, 0xc4, 0x0c, \
  0x80, 0x70, 0x07, 0x00, 0x70, 0x06, \
  0xa0, 0x88, 0x48, 0x84, 0x88, 0x48, \
  0x05, 0x40, 0x32, 0x03, 0x20, 0x32, \
  0x43, 0x02, 0x82, 0x28, 0x22, 0x82, \
  0x1a, 0x01, 0x50, 0x15, 0x01, 0x50, \
  0x60, 0x27, 0x00, 0x70, 0x07, 0x00, \
  0x14, 0x38, 0xa1, 0x8a, 0x18, 0xa0

#define kMaskRandom47_16 \
  0x0c, 0x84, 0x0c, 0x40, 0xc4, 0x0c, \
  0x80, 0x70, 0x07, 0x00, 0x70, 0x06, \
  0xa0, 0x88, 0x48, 0x84, 0x88, 0x48, \
  0x05, 0x40, 0x32, 0x03, 0x20, 0x32, \
  0x43, 0x02, 0x82, 0x28, 0x22, 0x82, \
  0x1a, 0x01, 0x50, 0x15, 0x01, 0x50, \
  0x60, 0x27, 0x00, 0x70, 0x07, 0x00, \
  0x14, 0x38, 0xa1, 0x8a, 0x18, 0xa0, \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x33, 0x24, 0x26, 0x42, 0x64, 0x26, \
  0x91, 0x92, 0x13, 0x21, 0x32, 0x12, \
  0xa4, 0x20, 0x4a, 0x04, 0xa0, 0x4a, \
  0x50, 0xa0, 0xd4, 0x0d, 0x40, 0xd4, \
  0x84, 0xc5, 0x80, 0x58, 0x05, 0x80, \
  0x09, 0x71, 0x0d, 0x10, 0xd1, 0x0c, \
  0xfa, 0xd9, 0xf5, 0xfe, 0xdc, 0x14

#define kMaskRandom47_17 \
  0x0c, 0x84, 0x0c, 0x40, 0xc4, 0x0c, \
  0x80, 0x70, 0x07, 0x00, 0x70, 0x06, \
  0xa0, 0x88, 0x48, 0x84, 0x88, 0x48, \
  0x05, 0x40, 0x32, 0x03, 0x20, 0x32, \
  0x43, 0x02, 0x82, 0x28, 0x22, 0x82, \
  0x1a, 0x01, 0x50, 0x15, 0x01, 0x50, \
  0x60, 0x27, 0x00, 0x70, 0x07, 0x00, \
  0x14, 0x38, 0xa1, 0x8a, 0x18, 0xa0, \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x62, 0x7c, 0x85, 0xc8, 0x5c, 0x84, \
  0x8c, 0x04, 0x88, 0x48, 0x84, 0x88, \
  0x01, 0x74, 0x23, 0x42, 0x34, 0x22, \
  0x07, 0x83, 0x06, 0x30, 0x63, 0x06, \
  0xa0, 0x80, 0x72, 0x07, 0x20, 0x72, \
  0x18, 0xb1, 0x43, 0x14, 0x31, 0x42, \
  0x91, 0x00, 0x92, 0x09, 0x20, 0x92, \
  0x78, 0x00, 0x1c, 0x01, 0xc0, 0x1c

#define kMaskRandom47_18 \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x62, 0x7c, 0x85, 0xc8, 0x5c, 0x84, \
  0x8c, 0x04, 0x88, 0x48, 0x84, 0x88, \
  0x01, 0x74, 0x23, 0x42, 0x34, 0x22, \
  0x07, 0x83, 0x06, 0x30, 0x63, 0x06, \
  0xa0, 0x80, 0x72, 0x07, 0x20, 0x72, \
  0x18, 0xb1, 0x43, 0x14, 0x31, 0x42, \
  0x91, 0x00, 0x92, 0x09, 0x20, 0x92, \
  0x78, 0x00, 0x1c, 0x01, 0xc0, 0x1c, \
  0x0c, 0x84, 0x0c, 0x40, 0xc4, 0x0c, \
  0x80, 0x70, 0x07, 0x00, 0x70, 0x06, \
  0xa0, 0x88, 0x48, 0x84, 0x88, 0x48, \
  0x05, 0x40, 0x32, 0x03, 0x20, 0x32, \
  0x43, 0x02, 0x82, 0x28, 0x22, 0x82, \
  0x1a, 0x01, 0x50, 0x15, 0x01, 0x50, \
  0x60, 0x27, 0x00, 0x70, 0x07, 0x00, \
  0x14, 0x38, 0xa1, 0x8a, 0x18, 0xa0, \
  0x82, 0x32, 0x56, 0x68, 0xa1, 0x5c

#define kMaskRandom47_19 \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x62, 0x7c, 0x85, 0xc8, 0x5c, 0x84, \
  0x8c, 0x04, 0x88, 0x48, 0x84, 0x88, \
  0x01, 0x74, 0x23, 0x42, 0x34, 0x22, \
  0x07, 0x83, 0x06, 0x30, 0x63, 0x06, \
  0xa0, 0x80, 0x72, 0x07, 0x20, 0x72, \
  0x18, 0xb1, 0x43, 0x14, 0x31, 0x42, \
  0x91, 0x00, 0x92, 0x09, 0x20, 0x92, \
  0x78, 0x00, 0x1c, 0x01, 0xc0, 0x1c, \
  0x64, 0x4a, 0x28, 0xa2, 0x8a, 0x28, \
  0x51, 0x48, 0xa2, 0x8a, 0x28, 0xa2, \
  0xa9, 0x10, 0x1b, 0x01, 0xb0, 0x1a, \
  0x04, 0xc4, 0x84, 0x48, 0x44, 0x84, \
  0xd0, 0x01, 0x44, 0x14, 0x41, 0x44, \
  0x82, 0x40, 0x1c, 0x01, 0xc0, 0x1c, \
  0x21, 0x37, 0x01, 0x70, 0x17, 0x00, \
  0x0c, 0x21, 0x22, 0x12, 0x21, 0x22, \
  0x4a, 0x0a, 0xc0, 0xac, 0x0a, 0xc0, \
  0x12, 0xb4, 0x51, 0x45, 0x14, 0x50

#define kMaskRandom47_2 \
  0xec, 0xdd, 0x99, 0xd9, 0x9d, 0x98, \
  0x9b, 0xb2, 0x77, 0x27, 0x72, 0x76

#define kMaskRandom47_20 \
  0x64, 0x4a, 0x28, 0xa2, 0x8a, 0x28, \
  0x51, 0x48, 0xa2, 0x8a, 0x28, 0xa2, \
  0xa9, 0x10, 0x1b, 0x01, 0xb0, 0x1a, \
  0x04, 0xc4, 0x84, 0x48, 0x44, 0x84, \
  0xd0, 0x01, 0x44, 0x14, 0x41, 0x44, \
  0x82, 0x40, 0x1c, 0x01, 0xc0, 0x1c, \
  0x21, 0x37, 0x01, 0x70, 0x17, 0x00, \
  0x0c, 0x21, 0x22, 0x12, 0x21, 0x22, \
  0x4a, 0x0a, 0xc0, 0xac, 0x0a, 0xc0, \
  0x12, 0xb4, 0x51, 0x45, 0x14, 0x50, \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x62, 0x7c, 0x85, 0xc8, 0x5c, 0x84, \
  0x8c, 0x04, 0x88, 0x48, 0x84, 0x88, \
  0x01, 0x74, 0x23, 0x42, 0x34, 0x22, \
  0x07, 0x83, 0x06, 0x30, 0x63, 0x06, \
  0xa0, 0x80, 0x72, 0x07, 0x20, 0x72, \
  0x18, 0xb1, 0x43, 0x14, 0x31, 0x42, \
  0x91, 0x00, 0x92, 0x09, 0x20, 0x92, \
  0x78, 0x00, 0x1c, 0x01, 0xc0, 0x1c, \
  0xdb, 0x4a, 0x7b, 0x31, 0x45, 0x2a

#define kMaskRandom47_21 \
  0x64, 0x4a, 0x28, 0xa2, 0x8a, 0x28, \
  0x51, 0x48, 0xa2, 0x8a, 0x28, 0xa2, \
  0xa9, 0x10, 0x1b, 0x01, 0xb0, 0x1a, \
  0x04, 0xc4, 0x84, 0x48, 0x44, 0x84, \
  0xd0, 0x01, 0x44, 0x14, 0x41, 0x44, \
  0x82, 0x40, 0x1c, 0x01, 0xc0, 0x1c, \
  0x21, 0x37, 0x01, 0x70, 0x17, 0x00, \
  0x0c, 0x21, 0x22, 0x12, 0x21, 0x22, \
  0x4a, 0x0a, 0xc0, 0xac, 0x0a, 0xc0, \
  0x12, 0xb4, 0x51, 0x45, 0x14, 0x50, \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x33, 0x24, 0x26, 0x42, 0x64, 0x26, \
  0x99, 0x02, 0x12, 0x21, 0x22, 0x12, \
  0x05, 0x80, 0x0e, 0x00, 0xe0, 0x0e, \
  0x80, 0xa1, 0x82, 0x18, 0x21, 0x82, \
  0x84, 0x48, 0x18, 0x81, 0x88, 0x18, \
  0x40, 0x6d, 0x40, 0xd4, 0x0d, 0x40, \
  0x0a, 0x90, 0xc1, 0x0c, 0x10, 0xc0, \
  0x68, 0x04, 0x90, 0x49, 0x04, 0x90, \
  0x10, 0x31, 0x21, 0x12, 0x11, 0x20, \
  0x30, 0x58, 0x05, 0x80, 0x58, 0x04

#define kMaskRandom47_22 \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x33, 0x24, 0x26, 0x42, 0x64, 0x26, \
  0x99, 0x02, 0x12, 0x21, 0x22, 0x12, \
  0x05, 0x80, 0x0e, 0x00, 0xe0, 0x0e, \
  0x80, 0xa1, 0x82, 0x18, 0x21, 0x82, \
  0x84, 0x48, 0x18, 0x81, 0x88, 0x18, \
  0x40, 0x6d, 0x40, 0xd4, 0x0d, 0x40, \
  0x0a, 0x90, 0xc1, 0x0c, 0x10, 0xc0, \
  0x68, 0x04, 0x90, 0x49, 0x04, 0x90, \
  0x10, 0x31, 0x21, 0x12, 0x11, 0x20, \
  0x30, 0x58, 0x05, 0x80, 0x58, 0x04, \
  0x64, 0x4a, 0x28, 0xa2, 0x8a, 0x28, \
  0x51, 0x48, 0xa2, 0x8a, 0x28, 0xa2, \
  0xa9, 0x10, 0x1b, 0x01, 0xb0, 0x1a, \
  0x04, 0xc4, 0x84, 0x48, 0x44, 0x84, \
  0xd0, 0x01, 0x44, 0x14, 0x41, 0x44, \
  0x82, 0x40, 0x1c, 0x01, 0xc0, 0x1c, \
  0x21, 0x37, 0x01, 0x70, 0x17, 0x00, \
  0x0c, 0x21, 0x22, 0x12, 0x21, 0x22, \
  0x4a, 0x0a, 0xc0, 0xac, 0x0a, 0xc0, \
  0x12, 0xb4, 0x51, 0x45, 0x14, 0x50, \
  0xea, 0x8d, 0x1a, 0x35, 0x55, 0xdc

#define kMaskRandom47_23 \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x33, 0x24, 0x26, 0x42, 0x64, 0x26, \
  0x99, 0x02, 0x12, 0x21, 0x22, 0x12, \
  0x05, 0x80, 0x0e, 0x00, 0xe0, 0x0e, \
  0x80, 0xa1, 0x82, 0x18, 0x21, 0x82, \
  0x84, 0x48, 0x18, 0x81, 0x88, 0x18, \
  0x40, 0x6d, 0x40, 0xd4, 0x0d, 0x40, \
  0x0a, 0x90, 0xc1, 0x0c, 0x10, 0xc0, \
  0x68, 0x04, 0x90, 0x49, 0x04, 0x90, \
  0x10, 0x31, 0x21, 0x12, 0x11, 0x20, \
  0x30, 0x58, 0x05, 0x80, 0x58, 0x04, \
  0x46, 0x42, 0x0c, 0x20, 0xc2, 0x0c, \
  0x33, 0x20, 0x46, 0x04, 0x60, 0x46, \
  0x99, 0x08, 0x0a, 0x80, 0xa8, 0x0a, \
  0x05, 0x84, 0x30, 0x43, 0x04, 0x30, \
  0x80, 0xb0, 0x23, 0x02, 0x30, 0x22, \
  0x84, 0x42, 0x90, 0x29, 0x02, 0x90, \
  0x40, 0x73, 0x01, 0x30, 0x13, 0x00, \
  0x0a, 0x81, 0x12, 0x11, 0x21, 0x12, \
  0x68, 0x0c, 0x40, 0xc4, 0x0c, 0x40, \
  0x10, 0x24, 0x84, 0x48, 0x44, 0x84, \
  0x30, 0x51, 0x41, 0x14, 0x11, 0x40, \
  0x5f, 0x50, 0x89, 0x08, 0x90, 0x88

#define kMaskRandom47_24 \
  0x64, 0x4a, 0x28, 0x20, 0xc2, 0x0c, \
  0x51, 0x58, 0xa2, 0x04, 0x60, 0x46, \
  0x0c, 0xa4, 0x30, 0x80, 0xa8, 0x0a, \
  0xa1, 0x22, 0x46, 0x43, 0x04, 0x30, \
  0x12, 0xa1, 0x1d, 0x02, 0x30, 0x22, \
  0x8a, 0x45, 0xc0, 0x29, 0x02, 0x90, \
  0x86, 0x8a, 0x6d, 0x30, 0x13, 0x00, \
  0x23, 0x2c, 0x84, 0x11, 0x21, 0x12, \
  0x16, 0x21, 0x98, 0xc4, 0x0c, 0x40, \
  0x4c, 0x30, 0x54, 0x48, 0x44, 0x84, \
  0x41, 0xc1, 0x27, 0x14, 0x11, 0x40, \
  0x19, 0x56, 0xe5, 0x08, 0x90, 0x88, \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x68, \
  0x33, 0x24, 0x27, 0x40, 0x64, 0x22, \
  0x99, 0x02, 0x12, 0x2a, 0x22, 0x82, \
  0x05, 0x80, 0x0e, 0x06, 0xa0, 0x2a, \
  0x80, 0xa1, 0x83, 0x19, 0x11, 0x90, \
  0x84, 0x48, 0x18, 0x51, 0x05, 0x10, \
  0x40, 0x6d, 0x40, 0x10, 0x91, 0x08, \
  0x0a, 0x90, 0xc1, 0x32, 0x03, 0x20, \
  0x68, 0x04, 0x90, 0x45, 0x24, 0x52, \
  0x10, 0x31, 0x20, 0x8c, 0x08, 0xc0, \
  0x30, 0x58, 0x05, 0x18, 0x58, 0x04, \
  0x27, 0x41, 0x35, 0x57, 0x7e, 0x6a

#define kMaskRandom47_25 \
  0x64, 0x4a, 0x28, 0x20, 0xc2, 0x0c, \
  0x51, 0x58, 0xa2, 0x04, 0x60, 0x46, \
  0x0c, 0xa4, 0x30, 0x80, 0xa8, 0x0a, \
  0xa1, 0x22, 0x46, 0x43, 0x04, 0x30, \
  0x12, 0xa1, 0x1d, 0x02, 0x30, 0x22, \
  0x8a, 0x45, 0xc0, 0x29, 0x02, 0x90, \
  0x86, 0x8a, 0x6d, 0x30, 0x13, 0x00, \
  0x23, 0x2c, 0x84, 0x11, 0x21, 0x12, \
  0x16, 0x21, 0x98, 0xc4, 0x0c, 0x40, \
  0x4c, 0x30, 0x54, 0x48, 0x44, 0x84, \
  0x41, 0xc1, 0x27, 0x14, 0x11, 0x40, \
  0x19, 0x56, 0xe5, 0x08, 0x90, 0x88, \
  0x64, 0x4a, 0x28, 0xa2, 0x8a, 0x28, \
  0x51, 0x58, 0xa3, 0x8a, 0x38, 0xa2, \
  0x0c, 0xa4, 0x30, 0x43, 0x04, 0x30, \
  0xa1, 0x22, 0x46, 0x24, 0x62, 0x46, \
  0x12, 0xa1, 0x1c, 0x11, 0xc1, 0x1c, \
  0x8a, 0x45, 0xc0, 0x5c, 0x05, 0xc0, \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x33, 0x24, 0x26, 0x42, 0x64, 0x26, \
  0x91, 0x92, 0x13, 0x21, 0x32, 0x12, \
  0xa4, 0x20, 0x4a, 0x04, 0xa0, 0x4a, \
  0x50, 0xa0, 0xd4, 0x0d, 0x40, 0xd4, \
  0x84, 0xc5, 0x80, 0x58, 0x05, 0x80, \
  0x09, 0x71, 0x0d, 0x10, 0xd1, 0x0c

#define kMaskRandom47_26 \
  0x64, 0x4a, 0x28, 0xa2, 0x8a, 0x28, \
  0x51, 0x58, 0xa3, 0x8a, 0x38, 0xa2, \
  0x0c, 0xa4, 0x30, 0x43, 0x04, 0x30, \
  0xa1, 0x22, 0x46, 0x24, 0x62, 0x46, \
  0x12, 0xa1, 0x1c, 0x11, 0xc1, 0x1c, \
  0x8a, 0x45, 0xc0, 0x5c, 0x05, 0xc0, \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x33, 0x24, 0x26, 0x42, 0x64, 0x26, \
  0x91, 0x92, 0x13, 0x21, 0x32, 0x12, \
  0xa4, 0x20, 0x4a, 0x04, 0xa0, 0x4a, \
  0x50, 0xa0, 0xd4, 0x0d, 0x40, 0xd4, \
  0x84, 0xc5, 0x80, 0x58, 0x05, 0x80, \
  0x09, 0x71, 0x0d, 0x10, 0xd1, 0x0c, \
  0x64, 0x4a, 0x28, 0x20, 0xc2, 0x0c, \
  0x51, 0x58, 0xa2, 0x04, 0x60, 0x46, \
  0x0c, 0xa4, 0x30, 0x80, 0xa8, 0x0a, \
  0xa1, 0x22, 0x46, 0x43, 0x04, 0x30, \
  0x12, 0xa1, 0x1d, 0x02, 0x30, 0x22, \
  0x8a, 0x45, 0xc0, 0x29, 0x02, 0x90, \
  0x86, 0x8a, 0x6d, 0x30, 0x13, 0x00, \
  0x23, 0x2c, 0x84, 0x11, 0x21, 0x12, \
  0x16, 0x21, 0x98, 0xc4, 0x0c, 0x40, \
  0x4c, 0x30, 0x54, 0x48, 0x44, 0x84, \
  0x41, 0xc1, 0x27, 0x14, 0x11, 0x40, \
  0x19, 0x56, 0xe5, 0x08, 0x90, 0x88, \
  0x6c, 0xea, 0xc4, 0x42, 0x20, 0x9e

#define kMaskRandom47_27 \
  0x64, 0x4a, 0x28, 0xa2, 0x8a, 0x28, \
  0x51, 0x58, 0xa3, 0x8a, 0x38, 0xa2, \
  0x0c, 0xa4, 0x30, 0x43, 0x04, 0x30, \
  0xa1, 0x22, 0x46, 0x24, 0x62, 0x46, \
  0x12, 0xa1, 0x1c, 0x11, 0xc1, 0x1c, \
  0x8a, 0x45, 0xc0, 0x5c, 0x05, 0xc0, \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x33, 0x24, 0x26, 0x42, 0x64, 0x26, \
  0x91, 0x92, 0x13, 0x21, 0x32, 0x12, \
  0xa4, 0x20, 0x4a, 0x04, 0xa0, 0x4a, \
  0x50, 0xa0, 0xd4, 0x0d, 0x40, 0xd4, \
  0x84, 0xc5, 0x80, 0x58, 0x05, 0x80, \
  0x09, 0x71, 0x0d, 0x10, 0xd1, 0x0c, \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x33, 0x24, 0x26, 0x42, 0x64, 0x26, \
  0x91, 0x92, 0x13, 0x21, 0x32, 0x12, \
  0xa4, 0x20, 0x4a, 0x04, 0xa0, 0x4a, \
  0x50, 0xa0, 0xd4, 0x0d, 0x40, 0xd4, \
  0x84, 0xc5, 0x80, 0x58, 0x05, 0x80, \
  0x09, 0x71, 0x0d, 0x10, 0xd1, 0x0c, \
  0x64, 0x4a, 0x28, 0xa2, 0x8a, 0x28, \
  0x51, 0x58, 0xa3, 0x8a, 0x38, 0xa2, \
  0x0c, 0xa4, 0x30, 0x43, 0x04, 0x30, \
  0xa1, 0x22, 0x46, 0x24, 0x62, 0x46, \
  0x12, 0xa1, 0x1c, 0x11, 0xc1, 0x1c, \
  0x8a, 0x45, 0xc0, 0x5c, 0x05, 0xc0, \
  0x9c, 0x3f, 0xb3, 0xe5, 0xad, 0x1c

#define kMaskRandom47_28 \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x33, 0x24, 0x26, 0x42, 0x64, 0x26, \
  0x91, 0x92, 0x13, 0x21, 0x32, 0x12, \
  0xa4, 0x20, 0x4a, 0x04, 0xa0, 0x4a, \
  0x50, 0xa0, 0xd4, 0x0d, 0x40, 0xd4, \
  0x84, 0xc5, 0x80, 0x58, 0x05, 0x80, \
  0x09, 0x71, 0x0d, 0x10, 0xd1, 0x0c, \
  0x64, 0x4a, 0x28, 0xa2, 0x8a, 0x28, \
  0x51, 0x58, 0xa3, 0x8a, 0x38, 0xa2, \
  0x0c, 0xa4, 0x30, 0x43, 0x04, 0x30, \
  0xa1, 0x22, 0x46, 0x24, 0x62, 0x46, \
  0x12, 0xa1, 0x1c, 0x11, 0xc1, 0x1c, \
  0x8a, 0x45, 0xc0, 0x5c, 0x05, 0xc0, \
  0x9c, 0x3f, 0xb3, 0xe5, 0xad, 0x1c, \
  0x64, 0x4a, 0x28, 0xa2, 0x8a, 0x28, \
  0x51, 0x58, 0xa3, 0x8a, 0x38, 0xa2, \
  0x0c, 0xa4, 0x30, 0x43, 0x04, 0x30, \
  0xa1, 0x22, 0x46, 0x24, 0x62, 0x46, \
  0x12, 0xa1, 0x1c, 0x11, 0xc1, 0x1c, \
  0x8a, 0x45, 0xc0, 0x5c, 0x05, 0xc0, \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x33, 0x24, 0x26, 0x42, 0x64, 0x26, \
  0x91, 0x92, 0x13, 0x21, 0x32, 0x12, \
  0xa4, 0x20, 0x4a, 0x04, 0xa0, 0x4a, \
  0x50, 0xa0, 0xd4, 0x0d, 0x40, 0xd4, \
  0x84, 0xc5, 0x80, 0x58, 0x05, 0x80, \
  0x09, 0x71, 0x0d, 0x10, 0xd1, 0x0c, \
  0x86, 0x1e, 0xa6, 0xaf, 0x3d, 0x04

#define kMaskRandom47_29 \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x33, 0x24, 0x26, 0x42, 0x64, 0x26, \
  0x91, 0x92, 0x13, 0x21, 0x32, 0x12, \
  0xa4, 0x20, 0x4a, 0x04, 0xa0, 0x4a, \
  0x50, 0xa0, 0xd4, 0x0d, 0x40, 0xd4, \
  0x84, 0xc5, 0x80, 0x58, 0x05, 0x80, \
  0x09, 0x71, 0x0d, 0x10, 0xd1, 0x0c, \
  0x64, 0x4a, 0x28, 0xa2, 0x8a, 0x28, \
  0x51, 0x58, 0xa3, 0x8a, 0x38, 0xa2, \
  0x0c, 0xa4, 0x30, 0x43, 0x04, 0x30, \
  0xa1, 0x22, 0x46, 0x24, 0x62, 0x46, \
  0x12, 0xa1, 0x1c, 0x11, 0xc1, 0x1c, \
  0x8a, 0x45, 0xc0, 0x5c, 0x05, 0xc0, \
  0x9c, 0x3f, 0xb3, 0xe5, 0xad, 0x1c, \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x33, 0x24, 0x26, 0x42, 0x64, 0x26, \
  0x91, 0x92, 0x13, 0x21, 0x32, 0x12, \
  0xa4, 0x20, 0x4a, 0x04, 0xa0, 0x4a, \
  0x50, 0xa0, 0xd4, 0x0d, 0x40, 0xd4, \
  0x84, 0xc5, 0x80, 0x58, 0x05, 0x80, \
  0x09, 0x71, 0x0d, 0x10, 0xd1, 0x0c, \
  0x0c, 0x84, 0x0c, 0x40, 0xc4, 0x0c, \
  0x80, 0x70, 0x07, 0x00, 0x70, 0x06, \
  0xa0, 0x88, 0x48, 0x84, 0x88, 0x48, \
  0x05, 0x40, 0x32, 0x03, 0x20, 0x32, \
  0x43, 0x02, 0x82, 0x28, 0x22, 0x82, \
  0x1a, 0x01, 0x50, 0x15, 0x01, 0x50, \
  0x60, 0x27, 0x00, 0x70, 0x07, 0x00, \
  0x14, 0x38, 0xa1, 0x8a, 0x18, 0xa0

#define kMaskRandom47_3 \
  0xca, 0xd3, 0x65, 0x36, 0x53, 0x64, \
  0xf1, 0x49, 0x3a, 0x93, 0xa9, 0x3a, \
  0x76, 0x27, 0xd0, 0x7d, 0x07, 0xd0

#define kMaskRandom47_30 \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x33, 0x24, 0x26, 0x42, 0x64, 0x26, \
  0x91, 0x92, 0x13, 0x21, 0x32, 0x12, \
  0xa4, 0x20, 0x4a, 0x04, 0xa0, 0x4a, \
  0x50, 0xa0, 0xd4, 0x0d, 0x40, 0xd4, \
  0x84, 0xc5, 0x80, 0x58, 0x05, 0x80, \
  0x09, 0x71, 0x0d, 0x10, 0xd1, 0x0c, \
  0x0c, 0x84, 0x0c, 0x40, 0xc4, 0x0c, \
  0x80, 0x70, 0x07, 0x00, 0x70, 0x06, \
  0xa0, 0x88, 0x48, 0x84, 0x88, 0x48, \
  0x05, 0x40, 0x32, 0x03, 0x20, 0x32, \
  0x43, 0x02, 0x82, 0x28, 0x22, 0x82, \
  0x1a, 0x01, 0x50, 0x15, 0x01, 0x50, \
  0x60, 0x27, 0x00, 0x70, 0x07, 0x00, \
  0x14, 0x38, 0xa1, 0x8a, 0x18, 0xa0, \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x33, 0x24, 0x26, 0x42, 0x64, 0x26, \
  0x91, 0x92, 0x13, 0x21, 0x32, 0x12, \
  0xa4, 0x20, 0x4a, 0x04, 0xa0, 0x4a, \
  0x50, 0xa0, 0xd4, 0x0d, 0x40, 0xd4, \
  0x84, 0xc5, 0x80, 0x58, 0x05, 0x80, \
  0x09, 0x71, 0x0d, 0x10, 0xd1, 0x0c, \
  0x64, 0x4a, 0x28, 0xa2, 0x8a, 0x28, \
  0x51, 0x58, 0xa3, 0x8a, 0x38, 0xa2, \
  0x0c, 0xa4, 0x30, 0x43, 0x04, 0x30, \
  0xa1, 0x22, 0x46, 0x24, 0x62, 0x46, \
  0x12, 0xa1, 0x1c, 0x11, 0xc1, 0x1c, \
  0x8a, 0x45, 0xc0, 0x5c, 0x05, 0xc0, \
  0x9c, 0x3f, 0xb3, 0xe5, 0xad, 0x1c, \
  0x97, 0x43, 0x63, 0xc6, 0x09, 0x9c

#define kMaskRandom47_31 \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x33, 0x24, 0x26, 0x42, 0x64, 0x26, \
  0x91, 0x92, 0x13, 0x21, 0x32, 0x12, \
  0xa4, 0x20, 0x4a, 0x04, 0xa0, 0x4a, \
  0x50, 0xa0, 0xd4, 0x0d, 0x40, 0xd4, \
  0x84, 0xc5, 0x80, 0x58, 0x05, 0x80, \
  0x09, 0x71, 0x0d, 0x10, 0xd1, 0x0c, \
  0x0c, 0x84, 0x0c, 0x40, 0xc4, 0x0c, \
  0x80, 0x70, 0x07, 0x00, 0x70, 0x06, \
  0xa0, 0x88, 0x48, 0x84, 0x88, 0x48, \
  0x05, 0x40, 0x32, 0x03, 0x20, 0x32, \
  0x43, 0x02, 0x82, 0x28, 0x22, 0x82, \
  0x1a, 0x01, 0x50, 0x15, 0x01, 0x50, \
  0x60, 0x27, 0x00, 0x70, 0x07, 0x00, \
  0x14, 0x38, 0xa1, 0x8a, 0x18, 0xa0, \
  0x0c, 0x84, 0x0c, 0x40, 0xc4, 0x0c, \
  0x80, 0x70, 0x07, 0x00, 0x70, 0x06, \
  0xa0, 0x88, 0x48, 0x84, 0x88, 0x48, \
  0x05, 0x40, 0x32, 0x03, 0x20, 0x32, \
  0x43, 0x02, 0x82, 0x28, 0x22, 0x82, \
  0x1a, 0x01, 0x50, 0x15, 0x01, 0x50, \
  0x60, 0x27, 0x00, 0x70, 0x07, 0x00, \
  0x14, 0x38, 0xa1, 0x8a, 0x18, 0xa0, \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x33, 0x24, 0x26, 0x42, 0x64, 0x26, \
  0x91, 0x92, 0x13, 0x21, 0x32, 0x12, \
  0xa4, 0x20, 0x4a, 0x04, 0xa0, 0x4a, \
  0x50, 0xa0, 0xd4, 0x0d, 0x40, 0xd4, \
  0x84, 0xc5, 0x80, 0x58, 0x05, 0x80, \
  0x09, 0x71, 0x0d, 0x10, 0xd1, 0x0c, \
  0xfa, 0xd9, 0xf5, 0xfe, 0xdc, 0x14

#define kMaskRandom47_32 \
  0x0c, 0x84, 0x0c, 0x40, 0xc4, 0x0c, \
  0x80, 0x70, 0x07, 0x00, 0x70, 0x06, \
  0xa0, 0x88, 0x48, 0x84, 0x88, 0x48, \
  0x05, 0x40, 0x32, 0x03, 0x20, 0x32, \
  0x43, 0x02, 0x82, 0x28, 0x22, 0x82, \
  0x1a, 0x01, 0x50, 0x15, 0x01, 0x50, \
  0x60, 0x27, 0x00, 0x70, 0x07, 0x00, \
  0x14, 0x38, 0xa1, 0x8a, 0x18, 0xa0, \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x33, 0x24, 0x26, 0x42, 0x64, 0x26, \
  0x91, 0x92, 0x13, 0x21, 0x32, 0x12, \
  0xa4, 0x20, 0x4a, 0x04, 0xa0, 0x4a, \
  0x50, 0xa0, 0xd4, 0x0d, 0x40, 0xd4, \
  0x84, 0xc5, 0x80, 0x58, 0x05, 0x80, \
  0x09, 0x71, 0x0d, 0x10, 0xd1, 0x0c, \
  0xfa, 0xd9, 0xf5, 0xfe, 0xdc, 0x14, \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x33, 0x24, 0x26, 0x42, 0x64, 0x26, \
  0x91, 0x92, 0x13, 0x21, 0x32, 0x12, \
  0xa4, 0x20, 0x4a, 0x04, 0xa0, 0x4a, \
  0x50, 0xa0, 0xd4, 0x0d, 0x40, 0xd4, \
  0x84, 0xc5, 0x80, 0x58, 0x05, 0x80, \
  0x09, 0x71, 0x0d, 0x10, 0xd1, 0x0c, \
  0x0c, 0x84, 0x0c, 0x40, 0xc4, 0x0c, \
  0x80, 0x70, 0x07, 0x00, 0x70, 0x06, \
  0xa0, 0x88, 0x48, 0x84, 0x88, 0x48, \
  0x05, 0x40, 0x32, 0x03, 0x20, 0x32, \
  0x43, 0x02, 0x82, 0x28, 0x22, 0x82, \
  0x1a, 0x01, 0x50, 0x15, 0x01, 0x50, \
  0x60, 0x27, 0x00, 0x70, 0x07, 0x00, \
  0x14, 0x38, 0xa1, 0x8a, 0x18, 0xa0, \
  0xe5, 0x50, 0x45, 0x63, 0xc2, 0xf4

#define kMaskRandom47_33 \
  0x0c, 0x84, 0x0c, 0x40, 0xc4, 0x0c, \
  0x80, 0x70, 0x07, 0x00, 0x70, 0x06, \
  0xa0, 0x88, 0x48, 0x84, 0x88, 0x48, \
  0x05, 0x40, 0x32, 0x03, 0x20, 0x32, \
  0x43, 0x02, 0x82, 0x28, 0x22, 0x82, \
  0x1a, 0x01, 0x50, 0x15, 0x01, 0x50, \
  0x60, 0x27, 0x00, 0x70, 0x07, 0x00, \
  0x14, 0x38, 0xa1, 0x8a, 0x18, 0xa0, \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x33, 0x24, 0x26, 0x42, 0x64, 0x26, \
  0x91, 0x92, 0x13, 0x21, 0x32, 0x12, \
  0xa4, 0x20, 0x4a, 0x04, 0xa0, 0x4a, \
  0x50, 0xa0, 0xd4, 0x0d, 0x40, 0xd4, \
  0x84, 0xc5, 0x80, 0x58, 0x05, 0x80, \
  0x09, 0x71, 0x0d, 0x10, 0xd1, 0x0c, \
  0xfa, 0xd9, 0xf5, 0xfe, 0xdc, 0x14, \
  0x0c, 0x84, 0x0c, 0x40, 0xc4, 0x0c, \
  0x80, 0x70, 0x07, 0x00, 0x70, 0x06, \
  0xa0, 0x88, 0x48, 0x84, 0x88, 0x48, \
  0x05, 0x40, 0x32, 0x03, 0x20, 0x32, \
  0x43, 0x02, 0x82, 0x28, 0x22, 0x82, \
  0x1a, 0x01, 0x50, 0x15, 0x01, 0x50, \
  0x60, 0x27, 0x00, 0x70, 0x07, 0x00, \
  0x14, 0x38, 0xa1, 0x8a, 0x18, 0xa0, \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x62, 0x7c, 0x85, 0xc8, 0x5c, 0x84, \
  0x8c, 0x04, 0x88, 0x48, 0x84, 0x88, \
  0x01, 0x74, 0x23, 0x42, 0x34, 0x22, \
  0x07, 0x83, 0x06, 0x30, 0x63, 0x06, \
  0xa0, 0x80, 0x72, 0x07, 0x20, 0x72, \
  0x18, 0xb1, 0x43, 0x14, 0x31, 0x42, \
  0x91, 0x00, 0x92, 0x09, 0x20, 0x92, \
  0x78, 0x00, 0x1c, 0x01, 0xc0, 0x1c

#define kMaskRandom47_34 \
  0x0c, 0x84, 0x0c, 0x40, 0xc4, 0x0c, \
  0x80, 0x70, 0x07, 0x00, 0x70, 0x06, \
  0xa0, 0x88, 0x48, 0x84, 0x88, 0x48, \
  0x05, 0x40, 0x32, 0x03, 0x20, 0x32, \
  0x43, 0x02, 0x82, 0x28, 0x22, 0x82, \
  0x1a, 0x01, 0x50, 0x15, 0x01, 0x50, \
  0x60, 0x27, 0x00, 0x70, 0x07, 0x00, \
  0x14, 0x38, 0xa1, 0x8a, 0x18, 0xa0, \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x62, 0x7c, 0x85, 0xc8, 0x5c, 0x84, \
  0x8c, 0x04, 0x88, 0x48, 0x84, 0x88, \
  0x01, 0x74, 0x23, 0x42, 0x34, 0x22, \
  0x07, 0x83, 0x06, 0x30, 0x63, 0x06, \
  0xa0, 0x80, 0x72, 0x07, 0x20, 0x72, \
  0x18, 0xb1, 0x43, 0x14, 0x31, 0x42, \
  0x91, 0x00, 0x92, 0x09, 0x20, 0x92, \
  0x78, 0x00, 0x1c, 0x01, 0xc0, 0x1c, \
  0x0c, 0x84, 0x0c, 0x40, 0xc4, 0x0c, \
  0x80, 0x70, 0x07, 0x00, 0x70, 0x06, \
  0xa0, 0x88, 0x48, 0x84, 0x88, 0x48, \
  0x05, 0x40, 0x32, 0x03, 0x20, 0x32, \
  0x43, 0x02, 0x82, 0x28, 0x22, 0x82, \
  0x1a, 0x01, 0x50, 0x15, 0x01, 0x50, \
  0x60, 0x27, 0x00, 0x70, 0x07, 0x00, \
  0x14, 0x38, 0xa1, 0x8a, 0x18, 0xa0, \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x33, 0x24, 0x26, 0x42, 0x64, 0x26, \
  0x91, 0x92, 0x13, 0x21, 0x32, 0x12, \
  0xa4, 0x20, 0x4a, 0x04, 0xa0, 0x4a, \
  0x50, 0xa0, 0xd4, 0x0d, 0x40, 0xd4, \
  0x84, 0xc5, 0x80, 0x58, 0x05, 0x80, \
  0x09, 0x71, 0x0d, 0x10, 0xd1, 0x0c, \
  0xfa, 0xd9, 0xf5, 0xfe, 0xdc, 0x14, \
  0xef, 0xbb, 0xa6, 0x23, 0x5c, 0xbe

#define kMaskRandom47_35 \
  0x0c, 0x84, 0x0c, 0x40, 0xc4, 0x0c, \
  0x80, 0x70, 0x07, 0x00, 0x70, 0x06, \
  0xa0, 0x88, 0x48, 0x84, 0x88, 0x48, \
  0x05, 0x40, 0x32, 0x03, 0x20, 0x32, \
  0x43, 0x02, 0x82, 0x28, 0x22, 0x82, \
  0x1a, 0x01, 0x50, 0x15, 0x01, 0x50, \
  0x60, 0x27, 0x00, 0x70, 0x07, 0x00, \
  0x14, 0x38, 0xa1, 0x8a, 0x18, 0xa0, \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x62, 0x7c, 0x85, 0xc8, 0x5c, 0x84, \
  0x8c, 0x04, 0x88, 0x48, 0x84, 0x88, \
  0x01, 0x74, 0x23, 0x42, 0x34, 0x22, \
  0x07, 0x83, 0x06, 0x30, 0x63, 0x06, \
  0xa0, 0x80, 0x72, 0x07, 0x20, 0x72, \
  0x18, 0xb1, 0x43, 0x14, 0x31, 0x42, \
  0x91, 0x00, 0x92, 0x09, 0x20, 0x92, \
  0x78, 0x00, 0x1c, 0x01, 0xc0, 0x1c, \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x62, 0x7c, 0x85, 0xc8, 0x5c, 0x84, \
  0x8c, 0x04, 0x88, 0x48, 0x84, 0x88, \
  0x01, 0x74, 0x23, 0x42, 0x34, 0x22, \
  0x07, 0x83, 0x06, 0x30, 0x63, 0x06, \
  0xa0, 0x80, 0x72, 0x07, 0x20, 0x72, \
  0x18, 0xb1, 0x43, 0x14, 0x31, 0x42, \
  0x91, 0x00, 0x92, 0x09, 0x20, 0x92, \
  0x78, 0x00, 0x1c, 0x01, 0xc0, 0x1c, \
  0x0c, 0x84, 0x0c, 0x40, 0xc4, 0x0c, \
  0x80, 0x70, 0x07, 0x00, 0x70, 0x06, \
  0xa0, 0x88, 0x48, 0x84, 0x88, 0x48, \
  0x05, 0x40, 0x32, 0x03, 0x20, 0x32, \
  0x43, 0x02, 0x82, 0x28, 0x22, 0x82, \
  0x1a, 0x01, 0x50, 0x15, 0x01, 0x50, \
  0x60, 0x27, 0x00, 0x70, 0x07, 0x00, \
  0x14, 0x38, 0xa1, 0x8a, 0x18, 0xa0, \
  0x82, 0x32, 0x56, 0x68, 0xa1, 0x5c

#define kMaskRandom47_36 \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x62, 0x7c, 0x85, 0xc8, 0x5c, 0x84, \
  0x8c, 0x04, 0x88, 0x48, 0x84, 0x88, \
  0x01, 0x74, 0x23, 0x42, 0x34, 0x22, \
  0x07, 0x83, 0x06, 0x30, 0x63, 0x06, \
  0xa0, 0x80, 0x72, 0x07, 0x20, 0x72, \
  0x18, 0xb1, 0x43, 0x14, 0x31, 0x42, \
  0x91, 0x00, 0x92, 0x09, 0x20, 0x92, \
  0x78, 0x00, 0x1c, 0x01, 0xc0, 0x1c, \
  0x0c, 0x84, 0x0c, 0x40, 0xc4, 0x0c, \
  0x80, 0x70, 0x07, 0x00, 0x70, 0x06, \
  0xa0, 0x88, 0x48, 0x84, 0x88, 0x48, \
  0x05, 0x40, 0x32, 0x03, 0x20, 0x32, \
  0x43, 0x02, 0x82, 0x28, 0x22, 0x82, \
  0x1a, 0x01, 0x50, 0x15, 0x01, 0x50, \
  0x60, 0x27, 0x00, 0x70, 0x07, 0x00, \
  0x14, 0x38, 0xa1, 0x8a, 0x18, 0xa0, \
  0x82, 0x32, 0x56, 0x68, 0xa1, 0x5c, \
  0x0c, 0x84, 0x0c, 0x40, 0xc4, 0x0c, \
  0x80, 0x70, 0x07, 0x00, 0x70, 0x06, \
  0xa0, 0x88, 0x48, 0x84, 0x88, 0x48, \
  0x05, 0x40, 0x32, 0x03, 0x20, 0x32, \
  0x43, 0x02, 0x82, 0x28, 0x22, 0x82, \
  0x1a, 0x01, 0x50, 0x15, 0x01, 0x50, \
  0x60, 0x27, 0x00, 0x70, 0x07, 0x00, \
  0x14, 0x38, 0xa1, 0x8a, 0x18, 0xa0, \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x62, 0x7c, 0x85, 0xc8, 0x5c, 0x84, \
  0x8c, 0x04, 0x88, 0x48, 0x84, 0x88, \
  0x01, 0x74, 0x23, 0x42, 0x34, 0x22, \
  0x07, 0x83, 0x06, 0x30, 0x63, 0x06, \
  0xa0, 0x80, 0x72, 0x07, 0x20, 0x72, \
  0x18, 0xb1, 0x43, 0x14, 0x31, 0x42, \
  0x91, 0x00, 0x92, 0x09, 0x20, 0x92, \
  0x78, 0x00, 0x1c, 0x01, 0xc0, 0x1c, \
  0x0e, 0xd7, 0x38, 0x20, 0x87, 0x66

#define kMaskRandom47_37 \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x62, 0x7c, 0x85, 0xc8, 0x5c, 0x84, \
  0x8c, 0x04, 0x88, 0x48, 0x84, 0x88, \
  0x01, 0x74, 0x23, 0x42, 0x34, 0x22, \
  0x07, 0x83, 0x06, 0x30, 0x63, 0x06, \
  0xa0, 0x80, 0x72, 0x07, 0x20, 0x72, \
  0x18, 0xb1, 0x43, 0x14, 0x31, 0x42, \
  0x91, 0x00, 0x92, 0x09, 0x20, 0x92, \
  0x78, 0x00, 0x1c, 0x01, 0xc0, 0x1c, \
  0x0c, 0x84, 0x0c, 0x40, 0xc4, 0x0c, \
  0x80, 0x70, 0x07, 0x00, 0x70, 0x06, \
  0xa0, 0x88, 0x48, 0x84, 0x88, 0x48, \
  0x05, 0x40, 0x32, 0x03, 0x20, 0x32, \
  0x43, 0x02, 0x82, 0x28, 0x22, 0x82, \
  0x1a, 0x01, 0x50, 0x15, 0x01, 0x50, \
  0x60, 0x27, 0x00, 0x70, 0x07, 0x00, \
  0x14, 0x38, 0xa1, 0x8a, 0x18, 0xa0, \
  0x82, 0x32, 0x56, 0x68, 0xa1, 0x5c, \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x62, 0x7c, 0x85, 0xc8, 0x5c, 0x84, \
  0x8c, 0x04, 0x88, 0x48, 0x84, 0x88, \
  0x01, 0x74, 0x23, 0x42, 0x34, 0x22, \
  0x07, 0x83, 0x06, 0x30, 0x63, 0x06, \
  0xa0, 0x80, 0x72, 0x07, 0x20, 0x72, \
  0x18, 0xb1, 0x43, 0x14, 0x31, 0x42, \
  0x91, 0x00, 0x92, 0x09, 0x20, 0x92, \
  0x78, 0x00, 0x1c, 0x01, 0xc0, 0x1c, \
  0x64, 0x4a, 0x28, 0xa2, 0x8a, 0x28, \
  0x51, 0x48, 0xa2, 0x8a, 0x28, 0xa2, \
  0xa9, 0x10, 0x1b, 0x01, 0xb0, 0x1a, \
  0x04, 0xc4, 0x84, 0x48, 0x44, 0x84, \
  0xd0, 0x01, 0x44, 0x14, 0x41, 0x44, \
  0x82, 0x40, 0x1c, 0x01, 0xc0, 0x1c, \
  0x21, 0x37, 0x01, 0x70, 0x17, 0x00, \
  0x0c, 0x21, 0x22, 0x12, 0x21, 0x22, \
  0x4a, 0x0a, 0xc0, 0xac, 0x0a, 0xc0, \
  0x12, 0xb4, 0x51, 0x45, 0x14, 0x50

#define kMaskRandom47_38 \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x62, 0x7c, 0x85, 0xc8, 0x5c, 0x84, \
  0x8c, 0x04, 0x88, 0x48, 0x84, 0x88, \
  0x01, 0x74, 0x23, 0x42, 0x34, 0x22, \
  0x07, 0x83, 0x06, 0x30, 0x63, 0x06, \
  0xa0, 0x80, 0x72, 0x07, 0x20, 0x72, \
  0x18, 0xb1, 0x43, 0x14, 0x31, 0x42, \
  0x91, 0x00, 0x92, 0x09, 0x20, 0x92, \
  0x78, 0x00, 0x1c, 0x01, 0xc0, 0x1c, \
  0x64, 0x4a, 0x28, 0xa2, 0x8a, 0x28, \
  0x51, 0x48, 0xa2, 0x8a, 0x28, 0xa2, \
  0xa9, 0x10, 0x1b, 0x01, 0xb0, 0x1a, \
  0x04, 0xc4, 0x84, 0x48, 0x44, 0x84, \
  0xd0, 0x01, 0x44, 0x14, 0x41, 0x44, \
  0x82, 0x40, 0x1c, 0x01, 0xc0, 0x1c, \
  0x21, 0x37, 0x01, 0x70, 0x17, 0x00, \
  0x0c, 0x21, 0x22, 0x12, 0x21, 0x22, \
  0x4a, 0x0a, 0xc0, 0xac, 0x0a, 0xc0, \
  0x12, 0xb4, 0x51, 0x45, 0x14, 0x50, \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x62, 0x7c, 0x85, 0xc8, 0x5c, 0x84, \
  0x8c, 0x04, 0x88, 0x48, 0x84, 0x88, \
  0x01, 0x74, 0x23, 0x42, 0x34, 0x22, \
  0x07, 0x83, 0x06, 0x30, 0x63, 0x06, \
  0xa0, 0x80, 0x72, 0x07, 0x20, 0x72, \
  0x18, 0xb1, 0x43, 0x14, 0x31, 0x42, \
  0x91, 0x00, 0x92, 0x09, 0x20, 0x92, \
  0x78, 0x00, 0x1c, 0x01, 0xc0, 0x1c, \
  0x0c, 0x84, 0x0c, 0x40, 0xc4, 0x0c, \
  0x80, 0x70, 0x07, 0x00, 0x70, 0x06, \
  0xa0, 0x88, 0x48, 0x84, 0x88, 0x48, \
  0x05, 0x40, 0x32, 0x03, 0x20, 0x32, \
  0x43, 0x02, 0x82, 0x28, 0x22, 0x82, \
  0x1a, 0x01, 0x50, 0x15, 0x01, 0x50, \
  0x60, 0x27, 0x00, 0x70, 0x07, 0x00, \
  0x14, 0x38, 0xa1, 0x8a, 0x18, 0xa0, \
  0x82, 0x32, 0x56, 0x68, 0xa1, 0x5c, \
  0x7b, 0x47, 0xa5, 0xde, 0x9a, 0xd4

#define kMaskRandom47_39 \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x62, 0x7c, 0x85, 0xc8, 0x5c, 0x84, \
  0x8c, 0x04, 0x88, 0x48, 0x84, 0x88, \
  0x01, 0x74, 0x23, 0x42, 0x34, 0x22, \
  0x07, 0x83, 0x06, 0x30, 0x63, 0x06, \
  0xa0, 0x80, 0x72, 0x07, 0x20, 0x72, \
  0x18, 0xb1, 0x43, 0x14, 0x31, 0x42, \
  0x91, 0x00, 0x92, 0x09, 0x20, 0x92, \
  0x78, 0x00, 0x1c, 0x01, 0xc0, 0x1c, \
  0x64, 0x4a, 0x28, 0xa2, 0x8a, 0x28, \
  0x51, 0x48, 0xa2, 0x8a, 0x28, 0xa2, \
  0xa9, 0x10, 0x1b, 0x01, 0xb0, 0x1a, \
  0x04, 0xc4, 0x84, 0x48, 0x44, 0x84, \
  0xd0, 0x01, 0x44, 0x14, 0x41, 0x44, \
  0x82, 0x40, 0x1c, 0x01, 0xc0, 0x1c, \
  0x21, 0x37, 0x01, 0x70, 0x17, 0x00, \
  0x0c, 0x21, 0x22, 0x12, 0x21, 0x22, \
  0x4a, 0x0a, 0xc0, 0xac, 0x0a, 0xc0, \
  0x12, 0xb4, 0x51, 0x45, 0x14, 0x50, \
  0x64, 0x4a, 0x28, 0xa2, 0x8a, 0x28, \
  0x51, 0x48, 0xa2, 0x8a, 0x28, 0xa2, \
  0xa9, 0x10, 0x1b, 0x01, 0xb0, 0x1a, \
  0x04, 0xc4, 0x84, 0x48, 0x44, 0x84, \
  0xd0, 0x01, 0x44, 0x14, 0x41, 0x44, \
  0x82, 0x40, 0x1c, 0x01, 0xc0, 0x1c, \
  0x21, 0x37, 0x01, 0x70, 0x17, 0x00, \
  0x0c, 0x21, 0x22, 0x12, 0x21, 0x22, \
  0x4a, 0x0a, 0xc0, 0xac, 0x0a, 0xc0, \
  0x12, 0xb4, 0x51, 0x45, 0x14, 0x50, \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x62, 0x7c, 0x85, 0xc8, 0x5c, 0x84, \
  0x8c, 0x04, 0x88, 0x48, 0x84, 0x88, \
  0x01, 0x74, 0x23, 0x42, 0x34, 0x22, \
  0x07, 0x83, 0x06, 0x30, 0x63, 0x06, \
  0xa0, 0x80, 0x72, 0x07, 0x20, 0x72, \
  0x18, 0xb1, 0x43, 0x14, 0x31, 0x42, \
  0x91, 0x00, 0x92, 0x09, 0x20, 0x92, \
  0x78, 0x00, 0x1c, 0x01, 0xc0, 0x1c, \
  0xdb, 0x4a, 0x7b, 0x31, 0x45, 0x2a

#define kMaskRandom47_4 \
  0xc4, 0xd1, 0x65, 0x16, 0x51, 0x64, \
  0x31, 0x62, 0x96, 0x29, 0x62, 0x96, \
  0x4b, 0x24, 0x5a, 0x45, 0xa4, 0x5a, \
  0x2c, 0xa8, 0xaa, 0x8a, 0xa8, 0xaa

#define kMaskRandom47_40 \
  0x64, 0x4a, 0x28, 0xa2, 0x8a, 0x28, \
  0x51, 0x48, 0xa2, 0x8a, 0x28, 0xa2, \
  0xa9, 0x10, 0x1b, 0x01, 0xb0, 0x1a, \
  0x04, 0xc4, 0x84, 0x48, 0x44, 0x84, \
  0xd0, 0x01, 0x44, 0x14, 0x41, 0x44, \
  0x82, 0x40, 0x1c, 0x01, 0xc0, 0x1c, \
  0x21, 0x37, 0x01, 0x70, 0x17, 0x00, \
  0x0c, 0x21, 0x22, 0x12, 0x21, 0x22, \
  0x4a, 0x0a, 0xc0, 0xac, 0x0a, 0xc0, \
  0x12, 0xb4, 0x51, 0x45, 0x14, 0x50, \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x62, 0x7c, 0x85, 0xc8, 0x5c, 0x84, \
  0x8c, 0x04, 0x88, 0x48, 0x84, 0x88, \
  0x01, 0x74, 0x23, 0x42, 0x34, 0x22, \
  0x07, 0x83, 0x06, 0x30, 0x63, 0x06, \
  0xa0, 0x80, 0x72, 0x07, 0x20, 0x72, \
  0x18, 0xb1, 0x43, 0x14, 0x31, 0x42, \
  0x91, 0x00, 0x92, 0x09, 0x20, 0x92, \
  0x78, 0x00, 0x1c, 0x01, 0xc0, 0x1c, \
  0xdb, 0x4a, 0x7b, 0x31, 0x45, 0x2a, \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x62, 0x7c, 0x85, 0xc8, 0x5c, 0x84, \
  0x8c, 0x04, 0x88, 0x48, 0x84, 0x88, \
  0x01, 0x74, 0x23, 0x42, 0x34, 0x22, \
  0x07, 0x83, 0x06, 0x30, 0x63, 0x06, \
  0xa0, 0x80, 0x72, 0x07, 0x20, 0x72, \
  0x18, 0xb1, 0x43, 0x14, 0x31, 0x42, \
  0x91, 0x00, 0x92, 0x09, 0x20, 0x92, \
  0x78, 0x00, 0x1c, 0x01, 0xc0, 0x1c, \
  0x64, 0x4a, 0x28, 0xa2, 0x8a, 0x28, \
  0x51, 0x48, 0xa2, 0x8a, 0x28, 0xa2, \
  0xa9, 0x10, 0x1b, 0x01, 0xb0, 0x1a, \
  0x04, 0xc4, 0x84, 0x48, 0x44, 0x84, \
  0xd0, 0x01, 0x44, 0x14, 0x41, 0x44, \
  0x82, 0x40, 0x1c, 0x01, 0xc0, 0x1c, \
  0x21, 0x37, 0x01, 0x70, 0x17, 0x00, \
  0x0c, 0x21, 0x22, 0x12, 0x21, 0x22, \
  0x4a, 0x0a, 0xc0, 0xac, 0x0a, 0xc0, \
  0x12, 0xb4, 0x51, 0x45, 0x14, 0x50, \
  0xc4, 0xae, 0x5e, 0x33, 0xf5, 0x1a

#define kMaskRandom47_41 \
  0x64, 0x4a, 0x28, 0xa2, 0x8a, 0x28, \
  0x51, 0x48, 0xa2, 0x8a, 0x28, 0xa2, \
  0xa9, 0x10, 0x1b, 0x01, 0xb0, 0x1a, \
  0x04, 0xc4, 0x84, 0x48, 0x44, 0x84, \
  0xd0, 0x01, 0x44, 0x14, 0x41, 0x44, \
  0x82, 0x40, 0x1c, 0x01, 0xc0, 0x1c, \
  0x21, 0x37, 0x01, 0x70, 0x17, 0x00, \
  0x0c, 0x21, 0x22, 0x12, 0x21, 0x22, \
  0x4a, 0x0a, 0xc0, 0xac, 0x0a, 0xc0, \
  0x12, 0xb4, 0x51, 0x45, 0x14, 0x50, \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x62, 0x7c, 0x85, 0xc8, 0x5c, 0x84, \
  0x8c, 0x04, 0x88, 0x48, 0x84, 0x88, \
  0x01, 0x74, 0x23, 0x42, 0x34, 0x22, \
  0x07, 0x83, 0x06, 0x30, 0x63, 0x06, \
  0xa0, 0x80, 0x72, 0x07, 0x20, 0x72, \
  0x18, 0xb1, 0x43, 0x14, 0x31, 0x42, \
  0x91, 0x00, 0x92, 0x09, 0x20, 0x92, \
  0x78, 0x00, 0x1c, 0x01, 0xc0, 0x1c, \
  0xdb, 0x4a, 0x7b, 0x31, 0x45, 0x2a, \
  0x64, 0x4a, 0x28, 0xa2, 0x8a, 0x28, \
  0x51, 0x48, 0xa2, 0x8a, 0x28, 0xa2, \
  0xa9, 0x10, 0x1b, 0x01, 0xb0, 0x1a, \
  0x04, 0xc4, 0x84, 0x48, 0x44, 0x84, \
  0xd0, 0x01, 0x44, 0x14, 0x41, 0x44, \
  0x82, 0x40, 0x1c, 0x01, 0xc0, 0x1c, \
  0x21, 0x37, 0x01, 0x70, 0x17, 0x00, \
  0x0c, 0x21, 0x22, 0x12, 0x21, 0x22, \
  0x4a, 0x0a, 0xc0, 0xac, 0x0a, 0xc0, \
  0x12, 0xb4, 0x51, 0x45, 0x14, 0x50, \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x33, 0x24, 0x26, 0x42, 0x64, 0x26, \
  0x99, 0x02, 0x12, 0x21, 0x22, 0x12, \
  0x05, 0x80, 0x0e, 0x00, 0xe0, 0x0e, \
  0x80, 0xa1, 0x82, 0x18, 0x21, 0x82, \
  0x84, 0x48, 0x18, 0x81, 0x88, 0x18, \
  0x40, 0x6d, 0x40, 0xd4, 0x0d, 0x40, \
  0x0a, 0x90, 0xc1, 0x0c, 0x10, 0xc0, \
  0x68, 0x04, 0x90, 0x49, 0x04, 0x90, \
  0x10, 0x31, 0x21, 0x12, 0x11, 0x20, \
  0x30, 0x58, 0x05, 0x80, 0x58, 0x04

#define kMaskRandom47_42 \
  0x64, 0x4a, 0x28, 0xa2, 0x8a, 0x28, \
  0x51, 0x48, 0xa2, 0x8a, 0x28, 0xa2, \
  0xa9, 0x10, 0x1b, 0x01, 0xb0, 0x1a, \
  0x04, 0xc4, 0x84, 0x48, 0x44, 0x84, \
  0xd0, 0x01, 0x44, 0x14, 0x41, 0x44, \
  0x82, 0x40, 0x1c, 0x01, 0xc0, 0x1c, \
  0x21, 0x37, 0x01, 0x70, 0x17, 0x00, \
  0x0c, 0x21, 0x22, 0x12, 0x21, 0x22, \
  0x4a, 0x0a, 0xc0, 0xac, 0x0a, 0xc0, \
  0x12, 0xb4, 0x51, 0x45, 0x14, 0x50, \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x33, 0x24, 0x26, 0x42, 0x64, 0x26, \
  0x99, 0x02, 0x12, 0x21, 0x22, 0x12, \
  0x05, 0x80, 0x0e, 0x00, 0xe0, 0x0e, \
  0x80, 0xa1, 0x82, 0x18, 0x21, 0x82, \
  0x84, 0x48, 0x18, 0x81, 0x88, 0x18, \
  0x40, 0x6d, 0x40, 0xd4, 0x0d, 0x40, \
  0x0a, 0x90, 0xc1, 0x0c, 0x10, 0xc0, \
  0x68, 0x04, 0x90, 0x49, 0x04, 0x90, \
  0x10, 0x31, 0x21, 0x12, 0x11, 0x20, \
  0x30, 0x58, 0x05, 0x80, 0x58, 0x04, \
  0x64, 0x4a, 0x28, 0xa2, 0x8a, 0x28, \
  0x51, 0x48, 0xa2, 0x8a, 0x28, 0xa2, \
  0xa9, 0x10, 0x1b, 0x01, 0xb0, 0x1a, \
  0x04, 0xc4, 0x84, 0x48, 0x44, 0x84, \
  0xd0, 0x01, 0x44, 0x14, 0x41, 0x44, \
  0x82, 0x40, 0x1c, 0x01, 0xc0, 0x1c, \
  0x21, 0x37, 0x01, 0x70, 0x17, 0x00, \
  0x0c, 0x21, 0x22, 0x12, 0x21, 0x22, \
  0x4a, 0x0a, 0xc0, 0xac, 0x0a, 0xc0, \
  0x12, 0xb4, 0x51, 0x45, 0x14, 0x50, \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x62, 0x7c, 0x85, 0xc8, 0x5c, 0x84, \
  0x8c, 0x04, 0x88, 0x48, 0x84, 0x88, \
  0x01, 0x74, 0x23, 0x42, 0x34, 0x22, \
  0x07, 0x83, 0x06, 0x30, 0x63, 0x06, \
  0xa0, 0x80, 0x72, 0x07, 0x20, 0x72, \
  0x18, 0xb1, 0x43, 0x14, 0x31, 0x42, \
  0x91, 0x00, 0x92, 0x09, 0x20, 0x92, \
  0x78, 0x00, 0x1c, 0x01, 0xc0, 0x1c, \
  0xdb, 0x4a, 0x7b, 0x31, 0x45, 0x2a, \
  0x3c, 0xb0, 0x36, 0x3b, 0x14, 0xa2

#define kMaskRandom47_43 \
  0x64, 0x4a, 0x28, 0xa2, 0x8a, 0x28, \
  0x51, 0x48, 0xa2, 0x8a, 0x28, 0xa2, \
  0xa9, 0x10, 0x1b, 0x01, 0xb0, 0x1a, \
  0x04, 0xc4, 0x84, 0x48, 0x44, 0x84, \
  0xd0, 0x01, 0x44, 0x14, 0x41, 0x44, \
  0x82, 0x40, 0x1c, 0x01, 0xc0, 0x1c, \
  0x21, 0x37, 0x01, 0x70, 0x17, 0x00, \
  0x0c, 0x21, 0x22, 0x12, 0x21, 0x22, \
  0x4a, 0x0a, 0xc0, 0xac, 0x0a, 0xc0, \
  0x12, 0xb4, 0x51, 0x45, 0x14, 0x50, \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x33, 0x24, 0x26, 0x42, 0x64, 0x26, \
  0x99, 0x02, 0x12, 0x21, 0x22, 0x12, \
  0x05, 0x80, 0x0e, 0x00, 0xe0, 0x0e, \
  0x80, 0xa1, 0x82, 0x18, 0x21, 0x82, \
  0x84, 0x48, 0x18, 0x81, 0x88, 0x18, \
  0x40, 0x6d, 0x40, 0xd4, 0x0d, 0x40, \
  0x0a, 0x90, 0xc1, 0x0c, 0x10, 0xc0, \
  0x68, 0x04, 0x90, 0x49, 0x04, 0x90, \
  0x10, 0x31, 0x21, 0x12, 0x11, 0x20, \
  0x30, 0x58, 0x05, 0x80, 0x58, 0x04, \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x33, 0x24, 0x26, 0x42, 0x64, 0x26, \
  0x99, 0x02, 0x12, 0x21, 0x22, 0x12, \
  0x05, 0x80, 0x0e, 0x00, 0xe0, 0x0e, \
  0x80, 0xa1, 0x82, 0x18, 0x21, 0x82, \
  0x84, 0x48, 0x18, 0x81, 0x88, 0x18, \
  0x40, 0x6d, 0x40, 0xd4, 0x0d, 0x40, \
  0x0a, 0x90, 0xc1, 0x0c, 0x10, 0xc0, \
  0x68, 0x04, 0x90, 0x49, 0x04, 0x90, \
  0x10, 0x31, 0x21, 0x12, 0x11, 0x20, \
  0x30, 0x58, 0x05, 0x80, 0x58, 0x04, \
  0x64, 0x4a, 0x28, 0xa2, 0x8a, 0x28, \
  0x51, 0x48, 0xa2, 0x8a, 0x28, 0xa2, \
  0xa9, 0x10, 0x1b, 0x01, 0xb0, 0x1a, \
  0x04, 0xc4, 0x84, 0x48, 0x44, 0x84, \
  0xd0, 0x01, 0x44, 0x14, 0x41, 0x44, \
  0x82, 0x40, 0x1c, 0x01, 0xc0, 0x1c, \
  0x21, 0x37, 0x01, 0x70, 0x17, 0x00, \
  0x0c, 0x21, 0x22, 0x12, 0x21, 0x22, \
  0x4a, 0x0a, 0xc0, 0xac, 0x0a, 0xc0, \
  0x12, 0xb4, 0x51, 0x45, 0x14, 0x50, \
  0xea, 0x8d, 0x1a, 0x35, 0x55, 0xdc

#define kMaskRandom47_44 \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x33, 0x24, 0x26, 0x42, 0x64, 0x26, \
  0x99, 0x02, 0x12, 0x21, 0x22, 0x12, \
  0x05, 0x80, 0x0e, 0x00, 0xe0, 0x0e, \
  0x80, 0xa1, 0x82, 0x18, 0x21, 0x82, \
  0x84, 0x48, 0x18, 0x81, 0x88, 0x18, \
  0x40, 0x6d, 0x40, 0xd4, 0x0d, 0x40, \
  0x0a, 0x90, 0xc1, 0x0c, 0x10, 0xc0, \
  0x68, 0x04, 0x90, 0x49, 0x04, 0x90, \
  0x10, 0x31, 0x21, 0x12, 0x11, 0x20, \
  0x30, 0x58, 0x05, 0x80, 0x58, 0x04, \
  0x64, 0x4a, 0x28, 0xa2, 0x8a, 0x28, \
  0x51, 0x48, 0xa2, 0x8a, 0x28, 0xa2, \
  0xa9, 0x10, 0x1b, 0x01, 0xb0, 0x1a, \
  0x04, 0xc4, 0x84, 0x48, 0x44, 0x84, \
  0xd0, 0x01, 0x44, 0x14, 0x41, 0x44, \
  0x82, 0x40, 0x1c, 0x01, 0xc0, 0x1c, \
  0x21, 0x37, 0x01, 0x70, 0x17, 0x00, \
  0x0c, 0x21, 0x22, 0x12, 0x21, 0x22, \
  0x4a, 0x0a, 0xc0, 0xac, 0x0a, 0xc0, \
  0x12, 0xb4, 0x51, 0x45, 0x14, 0x50, \
  0xea, 0x8d, 0x1a, 0x35, 0x55, 0xdc, \
  0x64, 0x4a, 0x28, 0xa2, 0x8a, 0x28, \
  0x51, 0x48, 0xa2, 0x8a, 0x28, 0xa2, \
  0xa9, 0x10, 0x1b, 0x01, 0xb0, 0x1a, \
  0x04, 0xc4, 0x84, 0x48, 0x44, 0x84, \
  0xd0, 0x01, 0x44, 0x14, 0x41, 0x44, \
  0x82, 0x40, 0x1c, 0x01, 0xc0, 0x1c, \
  0x21, 0x37, 0x01, 0x70, 0x17, 0x00, \
  0x0c, 0x21, 0x22, 0x12, 0x21, 0x22, \
  0x4a, 0x0a, 0xc0, 0xac, 0x0a, 0xc0, \
  0x12, 0xb4, 0x51, 0x45, 0x14, 0x50, \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x33, 0x24, 0x26, 0x42, 0x64, 0x26, \
  0x99, 0x02, 0x12, 0x21, 0x22, 0x12, \
  0x05, 0x80, 0x0e, 0x00, 0xe0, 0x0e, \
  0x80, 0xa1, 0x82, 0x18, 0x21, 0x82, \
  0x84, 0x48, 0x18, 0x81, 0x88, 0x18, \
  0x40, 0x6d, 0x40, 0xd4, 0x0d, 0x40, \
  0x0a, 0x90, 0xc1, 0x0c, 0x10, 0xc0, \
  0x68, 0x04, 0x90, 0x49, 0x04, 0x90, \
  0x10, 0x31, 0x21, 0x12, 0x11, 0x20, \
  0x30, 0x58, 0x05, 0x80, 0x58, 0x04, \
  0xd4, 0x8a, 0xd4, 0xd3, 0x3f, 0xe6

#define kMaskRandom47_45 \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x33, 0x24, 0x26, 0x42, 0x64, 0x26, \
  0x99, 0x02, 0x12, 0x21, 0x22, 0x12, \
  0x05, 0x80, 0x0e, 0x00, 0xe0, 0x0e, \
  0x80, 0xa1, 0x82, 0x18, 0x21, 0x82, \
  0x84, 0x48, 0x18, 0x81, 0x88, 0x18, \
  0x40, 0x6d, 0x40, 0xd4, 0x0d, 0x40, \
  0x0a, 0x90, 0xc1, 0x0c, 0x10, 0xc0, \
  0x68, 0x04, 0x90, 0x49, 0x04, 0x90, \
  0x10, 0x31, 0x21, 0x12, 0x11, 0x20, \
  0x30, 0x58, 0x05, 0x80, 0x58, 0x04, \
  0x64, 0x4a, 0x28, 0xa2, 0x8a, 0x28, \
  0x51, 0x48, 0xa2, 0x8a, 0x28, 0xa2, \
  0xa9, 0x10, 0x1b, 0x01, 0xb0, 0x1a, \
  0x04, 0xc4, 0x84, 0x48, 0x44, 0x84, \
  0xd0, 0x01, 0x44, 0x14, 0x41, 0x44, \
  0x82, 0x40, 0x1c, 0x01, 0xc0, 0x1c, \
  0x21, 0x37, 0x01, 0x70, 0x17, 0x00, \
  0x0c, 0x21, 0x22, 0x12, 0x21, 0x22, \
  0x4a, 0x0a, 0xc0, 0xac, 0x0a, 0xc0, \
  0x12, 0xb4, 0x51, 0x45, 0x14, 0x50, \
  0xea, 0x8d, 0x1a, 0x35, 0x55, 0xdc, \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x33, 0x24, 0x26, 0x42, 0x64, 0x26, \
  0x99, 0x02, 0x12, 0x21, 0x22, 0x12, \
  0x05, 0x80, 0x0e, 0x00, 0xe0, 0x0e, \
  0x80, 0xa1, 0x82, 0x18, 0x21, 0x82, \
  0x84, 0x48, 0x18, 0x81, 0x88, 0x18, \
  0x40, 0x6d, 0x40, 0xd4, 0x0d, 0x40, \
  0x0a, 0x90, 0xc1, 0x0c, 0x10, 0xc0, \
  0x68, 0x04, 0x90, 0x49, 0x04, 0x90, \
  0x10, 0x31, 0x21, 0x12, 0x11, 0x20, \
  0x30, 0x58, 0x05, 0x80, 0x58, 0x04, \
  0x46, 0x42, 0x0c, 0x20, 0xc2, 0x0c, \
  0x33, 0x20, 0x46, 0x04, 0x60, 0x46, \
  0x99, 0x08, 0x0a, 0x80, 0xa8, 0x0a, \
  0x05, 0x84, 0x30, 0x43, 0x04, 0x30, \
  0x80, 0xb0, 0x23, 0x02, 0x30, 0x22, \
  0x84, 0x42, 0x90, 0x29, 0x02, 0x90, \
  0x40, 0x73, 0x01, 0x30, 0x13, 0x00, \
  0x0a, 0x81, 0x12, 0x11, 0x21, 0x12, \
  0x68, 0x0c, 0x40, 0xc4, 0x0c, 0x40, \
  0x10, 0x24, 0x84, 0x48, 0x44, 0x84, \
  0x30, 0x51, 0x41, 0x14, 0x11, 0x40, \
  0x5f, 0x50, 0x89, 0x08, 0x90, 0x88

#define kMaskRandom47_46 \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x33, 0x24, 0x26, 0x42, 0x64, 0x26, \
  0x99, 0x02, 0x12, 0x21, 0x22, 0x12, \
  0x05, 0x80, 0x0e, 0x00, 0xe0, 0x0e, \
  0x80, 0xa1, 0x82, 0x18, 0x21, 0x82, \
  0x84, 0x48, 0x18, 0x81, 0x88, 0x18, \
  0x40, 0x6d, 0x40, 0xd4, 0x0d, 0x40, \
  0x0a, 0x90, 0xc1, 0x0c, 0x10, 0xc0, \
  0x68, 0x04, 0x90, 0x49, 0x04, 0x90, \
  0x10, 0x31, 0x21, 0x12, 0x11, 0x20, \
  0x30, 0x58, 0x05, 0x80, 0x58, 0x04, \
  0x46, 0x42, 0x0c, 0x20, 0xc2, 0x0c, \
  0x33, 0x20, 0x46, 0x04, 0x60, 0x46, \
  0x99, 0x08, 0x0a, 0x80, 0xa8, 0x0a, \
  0x05, 0x84, 0x30, 0x43, 0x04, 0x30, \
  0x80, 0xb0, 0x23, 0x02, 0x30, 0x22, \
  0x84, 0x42, 0x90, 0x29, 0x02, 0x90, \
  0x40, 0x73, 0x01, 0x30, 0x13, 0x00, \
  0x0a, 0x81, 0x12, 0x11, 0x21, 0x12, \
  0x68, 0x0c, 0x40, 0xc4, 0x0c, 0x40, \
  0x10, 0x24, 0x84, 0x48, 0x44, 0x84, \
  0x30, 0x51, 0x41, 0x14, 0x11, 0x40, \
  0x5f, 0x50, 0x89, 0x08, 0x90, 0x88, \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x33, 0x24, 0x26, 0x42, 0x64, 0x26, \
  0x99, 0x02, 0x12, 0x21, 0x22, 0x12, \
  0x05, 0x80, 0x0e, 0x00, 0xe0, 0x0e, \
  0x80, 0xa1, 0x82, 0x18, 0x21, 0x82, \
  0x84, 0x48, 0x18, 0x81, 0x88, 0x18, \
  0x40, 0x6d, 0x40, 0xd4, 0x0d, 0x40, \
  0x0a, 0x90, 0xc1, 0x0c, 0x10, 0xc0, \
  0x68, 0x04, 0x90, 0x49, 0x04, 0x90, \
  0x10, 0x31, 0x21, 0x12, 0x11, 0x20, \
  0x30, 0x58, 0x05, 0x80, 0x58, 0x04, \
  0x64, 0x4a, 0x28, 0xa2, 0x8a, 0x28, \
  0x51, 0x48, 0xa2, 0x8a, 0x28, 0xa2, \
  0xa9, 0x10, 0x1b, 0x01, 0xb0, 0x1a, \
  0x04, 0xc4, 0x84, 0x48, 0x44, 0x84, \
  0xd0, 0x01, 0x44, 0x14, 0x41, 0x44, \
  0x82, 0x40, 0x1c, 0x01, 0xc0, 0x1c, \
  0x21, 0x37, 0x01, 0x70, 0x17, 0x00, \
  0x0c, 0x21, 0x22, 0x12, 0x21, 0x22, \
  0x4a, 0x0a, 0xc0, 0xac, 0x0a, 0xc0, \
  0x12, 0xb4, 0x51, 0x45, 0x14, 0x50, \
  0xea, 0x8d, 0x1a, 0x35, 0x55, 0xdc, \
  0x37, 0x9d, 0xcf, 0xe0, 0xe4, 0x20

#define kMaskRandom47_47 \
  0x46, 0x4a, 0x6c, 0xa6, 0xca, 0x6c, \
  0x33, 0x24, 0x26, 0x42, 0x64, 0x26, \
  0x99, 0x02, 0x12, 0x21, 0x22, 0x12, \
  0x05, 0x80, 0x0e, 0x00, 0xe0, 0x0e, \
  0x80, 0xa1, 0x82, 0x18, 0x21, 0x82, \
  0x84, 0x48, 0x18, 0x81, 0x88, 0x18, \
  0x40, 0x6d, 0x40, 0xd4, 0x0d, 0x40, \
  0x0a, 0x90, 0xc1, 0x0c, 0x10, 0xc0, \
  0x68, 0x04, 0x90, 0x49, 0x04, 0x90, \
  0x10, 0x31, 0x21, 0x12, 0x11, 0x20, \
  0x30, 0x58, 0x05, 0x80, 0x58, 0x04, \
  0x46, 0x42, 0x0c, 0x20, 0xc2, 0x0c, \
  0x33, 0x20, 0x46, 0x04, 0x60, 0x46, \
  0x99, 0x08, 0x0a, 0x80, 0xa8, 0x0a, \
  0x05, 0x84, 0x30, 0x43, 0x04, 0x30, \
  0x80, 0xb0, 0x23, 0x02, 0x30, 0x22, \
  0x84, 0x42, 0x90, 0x29, 0x02, 0x90, \
  0x40, 0x73, 0x01, 0x30, 0x13, 0x00, \
  0x0a, 0x81, 0x12, 0x11, 0x21, 0x12, \
  0x68, 0x0c, 0x40, 0xc4, 0x0c, 0x40, \
  0x10, 0x24, 0x84, 0x48, 0x44, 0x84, \
  0x30, 0x51, 0x41, 0x14, 0x11, 0x40, \
  0x5f, 0x50, 0x89, 0x08, 0x90, 0x88, \
  0x46, 0x4a, 0x6c, 0x20, 0xc2, 0x0c, \
  0x33, 0x24, 0x26, 0x04, 0x60, 0x46, \
  0x99, 0x02, 0x12, 0x80, 0xa8, 0x0a, \
  0x05, 0x80, 0x0e, 0x43, 0x04, 0x30, \
  0x80, 0xa1, 0x83, 0x02, 0x30, 0x22, \
  0x84, 0x48, 0x18, 0x29, 0x02, 0x90, \
  0x40, 0x6d, 0x41, 0x30, 0x13, 0x00, \
  0x0a, 0x90, 0xc0, 0x11, 0x21, 0x12, \
  0x68, 0x04, 0x90, 0xc4, 0x0c, 0x40, \
  0x10, 0x31, 0x20, 0x48, 0x44, 0x84, \
  0x30, 0x58, 0x05, 0x14, 0x11, 0x40, \
  0x46, 0x42, 0x0d, 0x08, 0x90, 0x88, \
  0x33, 0x20, 0x46, 0xa6, 0xca, 0x6c, \
  0x99, 0x08, 0x0a, 0x42, 0x64, 0x26, \
  0x05, 0x84, 0x30, 0x21, 0x22, 0x12, \
  0x80, 0xb0, 0x22, 0x00, 0xe0, 0x0e, \
  0x84, 0x42, 0x90, 0x18, 0x21, 0x82, \
  0x40, 0x73, 0x00, 0x81, 0x88, 0x18, \
  0x0a, 0x81, 0x12, 0xd4, 0x0d, 0x40, \
  0x68, 0x0c, 0x41, 0x0c, 0x10, 0xc0, \
  0x10, 0x24, 0x84, 0x49, 0x04, 0x90, \
  0x30, 0x51, 0x41, 0x12, 0x11, 0x20, \
  0x5f, 0x50, 0x89, 0x80, 0x58, 0x04, \
  0x1f, 0x2f, 0x63, 0x10, 0x64, 0xb2

#define kMaskRandom47_5 \
  0xc6, 0xca, 0x6c, 0xa6, 0xca, 0x6c, \
  0x63, 0x6c, 0x96, 0xc9, 0x6c, 0x96, \
  0x1d, 0xa1, 0xdc, 0x1d, 0xc1, 0xdc, \
  0xad, 0x55, 0x39, 0x53, 0x95, 0x38, \
  0xb2, 0xb7, 0x07, 0x70, 0x77, 0x06

#define kMaskRandom47_6 \
  0x64, 0x4a, 0x29, 0xa2, 0x9a, 0x28, \
  0x51, 0x58, 0xa2, 0x8a, 0x68, 0xa6, \
  0x0c, 0xa4, 0x30, 0x45, 0xa4, 0x5a, \
  0xa1, 0x22, 0x46, 0x2d, 0x82, 0xd8, \
  0x12, 0xa1, 0x1c, 0x17, 0x41, 0x74, \
  0x8a, 0x45, 0xc1, 0xd1, 0x1d, 0x10

#define kMaskRandom47_7 \
  0x46, 0x4a, 0x6d, 0xa6, 0xca, 0x6c, \
  0x33, 0x24, 0x26, 0x4a, 0x64, 0xa6, \
  0x91, 0x92, 0x12, 0x61, 0xa6, 0x0a, \
  0xa4, 0x20, 0x4a, 0x0c, 0x90, 0xd8, \
  0x50, 0xa0, 0xd5, 0x81, 0x70, 0x36, \
  0x84, 0xc5, 0x80, 0x55, 0x45, 0x54, \
  0x09, 0x71, 0x0d, 0x50, 0x9d, 0x08

#define kMaskRandom47_8 \
  0x0c, 0x84, 0x0d, 0x02, 0xc0, 0x2c, \
  0x80, 0x70, 0x06, 0x80, 0x78, 0x06, \
  0xa0, 0x88, 0x48, 0x21, 0x22, 0x12, \
  0x05, 0x40, 0x32, 0x0c, 0xa0, 0xca, \
  0x43, 0x02, 0x82, 0x40, 0x95, 0x08, \
  0x1a, 0x01, 0x51, 0x15, 0x41, 0x54, \
  0x60, 0x27, 0x00, 0x66, 0x06, 0x60, \
  0x14, 0x38, 0xa0, 0x99, 0x09, 0x90

#define kMaskRandom47_9 \
  0x46, 0x4a, 0x6d, 0xa6, 0xca, 0x6c, \
  0x62, 0x7c, 0x84, 0xc8, 0x4c, 0x84, \
  0x8c, 0x04, 0x88, 0x30, 0x83, 0x88, \
  0x01, 0x74, 0x23, 0x40, 0x94, 0x08, \
  0x07, 0x83, 0x07, 0x02, 0x70, 0x26, \
  0xa0, 0x80, 0x72, 0x45, 0x44, 0x54, \
  0x18, 0xb1, 0x42, 0x10, 0xe1, 0x0e, \
  0x91, 0x00, 0x92, 0x09, 0x20, 0x92, \
  0x78, 0x00, 0x1c, 0x03, 0x80, 0x38

#define kMaskRandom48_1 \
  0xff, 0xff, 0xff, 0xff, 0xff, 0xff

#define kMaskRandom48_10 \
  0x11, 0x45, 0x14, 0x11, 0x45, 0x14, \
  0x45, 0x34, 0x53, 0x45, 0x34, 0x53, \
  0x00, 0x48, 0x05, 0x00, 0x48, 0x05, \
  0x10, 0x83, 0x09, 0x10, 0x83, 0x09, \
  0x4a, 0x14, 0xa1, 0x4a, 0x14, 0xa1, \
  0x40, 0xa4, 0x0a, 0x40, 0xa4, 0x0a, \
  0xa0, 0x6a, 0x02, 0xa0, 0x6a, 0x02, \
  0x88, 0x80, 0x8c, 0x88, 0x80, 0x8c, \
  0x86, 0x08, 0x60, 0x86, 0x08, 0x60, \
  0x54, 0x0d, 0x40, 0x54, 0x0d, 0x40

#define kMaskRandom48_11 \
  0x53, 0x65, 0x34, 0x53, 0x65, 0x34, \
  0xa0, 0x32, 0x11, 0xa0, 0x32, 0x11, \
  0x15, 0x11, 0x41, 0x15, 0x11, 0x41, \
  0x03, 0x50, 0x15, 0x03, 0x50, 0x15, \
  0x8c, 0x88, 0xc8, 0x8c, 0x88, 0xc8, \
  0x28, 0x82, 0x88, 0x28, 0x82, 0x88, \
  0x08, 0x48, 0x84, 0x08, 0x48, 0x84, \
  0x99, 0x01, 0x90, 0x99, 0x01, 0x90, \
  0x22, 0x92, 0x29, 0x22, 0x92, 0x29, \
  0x46, 0x04, 0x60, 0x46, 0x04, 0x60, \
  0x8c, 0x2c, 0x02, 0x8c, 0x2c, 0x02

#define kMaskRandom48_12 \
  0x10, 0x61, 0x06, 0x10, 0x61, 0x06, \
  0x02, 0x30, 0x23, 0x02, 0x30, 0x23, \
  0x40, 0x54, 0x05, 0x40, 0x54, 0x05, \
  0x21, 0x82, 0x18, 0x21, 0x82, 0x18, \
  0x81, 0x18, 0x11, 0x81, 0x18, 0x11, \
  0x14, 0x81, 0x48, 0x14, 0x81, 0x48, \
  0x98, 0x09, 0x80, 0x98, 0x09, 0x80, \
  0x08, 0x90, 0x89, 0x08, 0x90, 0x89, \
  0x62, 0x06, 0x20, 0x62, 0x06, 0x20, \
  0x24, 0x22, 0x42, 0x24, 0x22, 0x42, \
  0x8a, 0x08, 0xa0, 0x8a, 0x08, 0xa0, \
  0x84, 0x48, 0x44, 0x84, 0x48, 0x44

#define kMaskRandom48_13 \
  0x51, 0x45, 0x14, 0x51, 0x45, 0x14, \
  0xc5, 0x1c, 0x51, 0xc5, 0x1c, 0x51, \
  0x21, 0x82, 0x18, 0x21, 0x82, 0x18, \
  0x12, 0x31, 0x23, 0x12, 0x31, 0x23, \
  0x08, 0xe0, 0x8e, 0x08, 0xe0, 0x8e, \
  0x2e, 0x02, 0xe0, 0x2e, 0x02, 0xe0, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x90, 0x99, 0x09, 0x90, 0x99, 0x09, \
  0x02, 0x50, 0x25, 0x02, 0x50, 0x25, \
  0x06, 0xa0, 0x6a, 0x06, 0xa0, 0x6a, \
  0x2c, 0x02, 0xc0, 0x2c, 0x02, 0xc0, \
  0x88, 0x68, 0x86, 0x88, 0x68, 0x86

#define kMaskRandom48_14 \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x90, 0x99, 0x09, 0x90, 0x99, 0x09, \
  0x02, 0x50, 0x25, 0x02, 0x50, 0x25, \
  0x06, 0xa0, 0x6a, 0x06, 0xa0, 0x6a, \
  0x2c, 0x02, 0xc0, 0x2c, 0x02, 0xc0, \
  0x88, 0x68, 0x86, 0x88, 0x68, 0x86, \
  0x51, 0x45, 0x14, 0x51, 0x45, 0x14, \
  0xc5, 0x1c, 0x51, 0xc5, 0x1c, 0x51, \
  0x21, 0x82, 0x18, 0x21, 0x82, 0x18, \
  0x12, 0x31, 0x23, 0x12, 0x31, 0x23, \
  0x08, 0xe0, 0x8e, 0x08, 0xe0, 0x8e, \
  0x2e, 0x02, 0xe0, 0x2e, 0x02, 0xe0, \
  0xf2, 0xd6, 0x8e, 0xf2, 0xd6, 0x8e

#define kMaskRandom48_15 \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x90, 0x99, 0x09, 0x90, 0x99, 0x09, \
  0x02, 0x50, 0x25, 0x02, 0x50, 0x25, \
  0x06, 0xa0, 0x6a, 0x06, 0xa0, 0x6a, \
  0x2c, 0x02, 0xc0, 0x2c, 0x02, 0xc0, \
  0x88, 0x68, 0x86, 0x88, 0x68, 0x86, \
  0x20, 0x62, 0x06, 0x20, 0x62, 0x06, \
  0x80, 0x38, 0x03, 0x80, 0x38, 0x03, \
  0x42, 0x44, 0x24, 0x42, 0x44, 0x24, \
  0x01, 0x90, 0x19, 0x01, 0x90, 0x19, \
  0x14, 0x11, 0x41, 0x14, 0x11, 0x41, \
  0x0a, 0x80, 0xa8, 0x0a, 0x80, 0xa8, \
  0x38, 0x03, 0x80, 0x38, 0x03, 0x80, \
  0xc5, 0x0c, 0x50, 0xc5, 0x0c, 0x50

#define kMaskRandom48_16 \
  0x20, 0x62, 0x06, 0x20, 0x62, 0x06, \
  0x80, 0x38, 0x03, 0x80, 0x38, 0x03, \
  0x42, 0x44, 0x24, 0x42, 0x44, 0x24, \
  0x01, 0x90, 0x19, 0x01, 0x90, 0x19, \
  0x14, 0x11, 0x41, 0x14, 0x11, 0x41, \
  0x0a, 0x80, 0xa8, 0x0a, 0x80, 0xa8, \
  0x38, 0x03, 0x80, 0x38, 0x03, 0x80, \
  0xc5, 0x0c, 0x50, 0xc5, 0x0c, 0x50, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x90, 0x99, 0x09, 0x90, 0x99, 0x09, \
  0x02, 0x50, 0x25, 0x02, 0x50, 0x25, \
  0x06, 0xa0, 0x6a, 0x06, 0xa0, 0x6a, \
  0x2c, 0x02, 0xc0, 0x2c, 0x02, 0xc0, \
  0x88, 0x68, 0x86, 0x88, 0x68, 0x86, \
  0xff, 0x6e, 0x0a, 0xff, 0x6e, 0x0a

#define kMaskRandom48_17 \
  0x20, 0x62, 0x06, 0x20, 0x62, 0x06, \
  0x80, 0x38, 0x03, 0x80, 0x38, 0x03, \
  0x42, 0x44, 0x24, 0x42, 0x44, 0x24, \
  0x01, 0x90, 0x19, 0x01, 0x90, 0x19, \
  0x14, 0x11, 0x41, 0x14, 0x11, 0x41, \
  0x0a, 0x80, 0xa8, 0x0a, 0x80, 0xa8, \
  0x38, 0x03, 0x80, 0x38, 0x03, 0x80, \
  0xc5, 0x0c, 0x50, 0xc5, 0x0c, 0x50, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0xe4, 0x2e, 0x42, 0xe4, 0x2e, 0x42, \
  0x24, 0x42, 0x44, 0x24, 0x42, 0x44, \
  0xa1, 0x1a, 0x11, 0xa1, 0x1a, 0x11, \
  0x18, 0x31, 0x83, 0x18, 0x31, 0x83, \
  0x03, 0x90, 0x39, 0x03, 0x90, 0x39, \
  0x8a, 0x18, 0xa1, 0x8a, 0x18, 0xa1, \
  0x04, 0x90, 0x49, 0x04, 0x90, 0x49, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e

#define kMaskRandom48_18 \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0xe4, 0x2e, 0x42, 0xe4, 0x2e, 0x42, \
  0x24, 0x42, 0x44, 0x24, 0x42, 0x44, \
  0xa1, 0x1a, 0x11, 0xa1, 0x1a, 0x11, \
  0x18, 0x31, 0x83, 0x18, 0x31, 0x83, \
  0x03, 0x90, 0x39, 0x03, 0x90, 0x39, \
  0x8a, 0x18, 0xa1, 0x8a, 0x18, 0xa1, \
  0x04, 0x90, 0x49, 0x04, 0x90, 0x49, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0x20, 0x62, 0x06, 0x20, 0x62, 0x06, \
  0x80, 0x38, 0x03, 0x80, 0x38, 0x03, \
  0x42, 0x44, 0x24, 0x42, 0x44, 0x24, \
  0x01, 0x90, 0x19, 0x01, 0x90, 0x19, \
  0x14, 0x11, 0x41, 0x14, 0x11, 0x41, \
  0x0a, 0x80, 0xa8, 0x0a, 0x80, 0xa8, \
  0x38, 0x03, 0x80, 0x38, 0x03, 0x80, \
  0xc5, 0x0c, 0x50, 0xc5, 0x0c, 0x50, \
  0x34, 0x50, 0xae, 0x34, 0x50, 0xae

#define kMaskRandom48_19 \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0xe4, 0x2e, 0x42, 0xe4, 0x2e, 0x42, \
  0x24, 0x42, 0x44, 0x24, 0x42, 0x44, \
  0xa1, 0x1a, 0x11, 0xa1, 0x1a, 0x11, \
  0x18, 0x31, 0x83, 0x18, 0x31, 0x83, \
  0x03, 0x90, 0x39, 0x03, 0x90, 0x39, \
  0x8a, 0x18, 0xa1, 0x8a, 0x18, 0xa1, \
  0x04, 0x90, 0x49, 0x04, 0x90, 0x49, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0x51, 0x45, 0x14, 0x51, 0x45, 0x14, \
  0x45, 0x14, 0x51, 0x45, 0x14, 0x51, \
  0x80, 0xd8, 0x0d, 0x80, 0xd8, 0x0d, \
  0x24, 0x22, 0x42, 0x24, 0x22, 0x42, \
  0x0a, 0x20, 0xa2, 0x0a, 0x20, 0xa2, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0xb8, 0x0b, 0x80, 0xb8, 0x0b, 0x80, \
  0x09, 0x10, 0x91, 0x09, 0x10, 0x91, \
  0x56, 0x05, 0x60, 0x56, 0x05, 0x60, \
  0xa2, 0x8a, 0x28, 0xa2, 0x8a, 0x28

#define kMaskRandom48_2 \
  0xec, 0xce, 0xcc, 0xec, 0xce, 0xcc, \
  0x93, 0xb9, 0x3b, 0x93, 0xb9, 0x3b

#define kMaskRandom48_20 \
  0x51, 0x45, 0x14, 0x51, 0x45, 0x14, \
  0x45, 0x14, 0x51, 0x45, 0x14, 0x51, \
  0x80, 0xd8, 0x0d, 0x80, 0xd8, 0x0d, \
  0x24, 0x22, 0x42, 0x24, 0x22, 0x42, \
  0x0a, 0x20, 0xa2, 0x0a, 0x20, 0xa2, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0xb8, 0x0b, 0x80, 0xb8, 0x0b, 0x80, \
  0x09, 0x10, 0x91, 0x09, 0x10, 0x91, \
  0x56, 0x05, 0x60, 0x56, 0x05, 0x60, \
  0xa2, 0x8a, 0x28, 0xa2, 0x8a, 0x28, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0xe4, 0x2e, 0x42, 0xe4, 0x2e, 0x42, \
  0x24, 0x42, 0x44, 0x24, 0x42, 0x44, \
  0xa1, 0x1a, 0x11, 0xa1, 0x1a, 0x11, \
  0x18, 0x31, 0x83, 0x18, 0x31, 0x83, \
  0x03, 0x90, 0x39, 0x03, 0x90, 0x39, \
  0x8a, 0x18, 0xa1, 0x8a, 0x18, 0xa1, \
  0x04, 0x90, 0x49, 0x04, 0x90, 0x49, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0x98, 0xa2, 0x95, 0x98, 0xa2, 0x95

#define kMaskRandom48_21 \
  0x51, 0x45, 0x14, 0x51, 0x45, 0x14, \
  0x45, 0x14, 0x51, 0x45, 0x14, 0x51, \
  0x80, 0xd8, 0x0d, 0x80, 0xd8, 0x0d, \
  0x24, 0x22, 0x42, 0x24, 0x22, 0x42, \
  0x0a, 0x20, 0xa2, 0x0a, 0x20, 0xa2, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0xb8, 0x0b, 0x80, 0xb8, 0x0b, 0x80, \
  0x09, 0x10, 0x91, 0x09, 0x10, 0x91, \
  0x56, 0x05, 0x60, 0x56, 0x05, 0x60, \
  0xa2, 0x8a, 0x28, 0xa2, 0x8a, 0x28, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x10, 0x91, 0x09, 0x10, 0x91, 0x09, \
  0x00, 0x70, 0x07, 0x00, 0x70, 0x07, \
  0x0c, 0x10, 0xc1, 0x0c, 0x10, 0xc1, \
  0x40, 0xc4, 0x0c, 0x40, 0xc4, 0x0c, \
  0x6a, 0x06, 0xa0, 0x6a, 0x06, 0xa0, \
  0x86, 0x08, 0x60, 0x86, 0x08, 0x60, \
  0x24, 0x82, 0x48, 0x24, 0x82, 0x48, \
  0x89, 0x08, 0x90, 0x89, 0x08, 0x90, \
  0xc0, 0x2c, 0x02, 0xc0, 0x2c, 0x02

#define kMaskRandom48_22 \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x10, 0x91, 0x09, 0x10, 0x91, 0x09, \
  0x00, 0x70, 0x07, 0x00, 0x70, 0x07, \
  0x0c, 0x10, 0xc1, 0x0c, 0x10, 0xc1, \
  0x40, 0xc4, 0x0c, 0x40, 0xc4, 0x0c, \
  0x6a, 0x06, 0xa0, 0x6a, 0x06, 0xa0, \
  0x86, 0x08, 0x60, 0x86, 0x08, 0x60, \
  0x24, 0x82, 0x48, 0x24, 0x82, 0x48, \
  0x89, 0x08, 0x90, 0x89, 0x08, 0x90, \
  0xc0, 0x2c, 0x02, 0xc0, 0x2c, 0x02, \
  0x51, 0x45, 0x14, 0x51, 0x45, 0x14, \
  0x45, 0x14, 0x51, 0x45, 0x14, 0x51, \
  0x80, 0xd8, 0x0d, 0x80, 0xd8, 0x0d, \
  0x24, 0x22, 0x42, 0x24, 0x22, 0x42, \
  0x0a, 0x20, 0xa2, 0x0a, 0x20, 0xa2, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0xb8, 0x0b, 0x80, 0xb8, 0x0b, 0x80, \
  0x09, 0x10, 0x91, 0x09, 0x10, 0x91, \
  0x56, 0x05, 0x60, 0x56, 0x05, 0x60, \
  0xa2, 0x8a, 0x28, 0xa2, 0x8a, 0x28, \
  0x1a, 0xaa, 0xee, 0x1a, 0xaa, 0xee

#define kMaskRandom48_23 \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x10, 0x91, 0x09, 0x10, 0x91, 0x09, \
  0x00, 0x70, 0x07, 0x00, 0x70, 0x07, \
  0x0c, 0x10, 0xc1, 0x0c, 0x10, 0xc1, \
  0x40, 0xc4, 0x0c, 0x40, 0xc4, 0x0c, \
  0x6a, 0x06, 0xa0, 0x6a, 0x06, 0xa0, \
  0x86, 0x08, 0x60, 0x86, 0x08, 0x60, \
  0x24, 0x82, 0x48, 0x24, 0x82, 0x48, \
  0x89, 0x08, 0x90, 0x89, 0x08, 0x90, \
  0xc0, 0x2c, 0x02, 0xc0, 0x2c, 0x02, \
  0x10, 0x61, 0x06, 0x10, 0x61, 0x06, \
  0x02, 0x30, 0x23, 0x02, 0x30, 0x23, \
  0x40, 0x54, 0x05, 0x40, 0x54, 0x05, \
  0x21, 0x82, 0x18, 0x21, 0x82, 0x18, \
  0x81, 0x18, 0x11, 0x81, 0x18, 0x11, \
  0x14, 0x81, 0x48, 0x14, 0x81, 0x48, \
  0x98, 0x09, 0x80, 0x98, 0x09, 0x80, \
  0x08, 0x90, 0x89, 0x08, 0x90, 0x89, \
  0x62, 0x06, 0x20, 0x62, 0x06, 0x20, \
  0x24, 0x22, 0x42, 0x24, 0x22, 0x42, \
  0x8a, 0x08, 0xa0, 0x8a, 0x08, 0xa0, \
  0x84, 0x48, 0x44, 0x84, 0x48, 0x44

#define kMaskRandom48_24 \
  0x10, 0x61, 0x06, 0x10, 0x61, 0x06, \
  0x02, 0x30, 0x23, 0x02, 0x30, 0x23, \
  0x40, 0x54, 0x05, 0x40, 0x54, 0x05, \
  0x21, 0x82, 0x18, 0x21, 0x82, 0x18, \
  0x81, 0x18, 0x11, 0x81, 0x18, 0x11, \
  0x14, 0x81, 0x48, 0x14, 0x81, 0x48, \
  0x98, 0x09, 0x80, 0x98, 0x09, 0x80, \
  0x08, 0x90, 0x89, 0x08, 0x90, 0x89, \
  0x62, 0x06, 0x20, 0x62, 0x06, 0x20, \
  0x24, 0x22, 0x42, 0x24, 0x22, 0x42, \
  0x8a, 0x08, 0xa0, 0x8a, 0x08, 0xa0, \
  0x84, 0x48, 0x44, 0x84, 0x48, 0x44, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x10, 0x91, 0x09, 0x10, 0x91, 0x09, \
  0x00, 0x70, 0x07, 0x00, 0x70, 0x07, \
  0x0c, 0x10, 0xc1, 0x0c, 0x10, 0xc1, \
  0x40, 0xc4, 0x0c, 0x40, 0xc4, 0x0c, \
  0x6a, 0x06, 0xa0, 0x6a, 0x06, 0xa0, \
  0x86, 0x08, 0x60, 0x86, 0x08, 0x60, \
  0x24, 0x82, 0x48, 0x24, 0x82, 0x48, \
  0x89, 0x08, 0x90, 0x89, 0x08, 0x90, \
  0xc0, 0x2c, 0x02, 0xc0, 0x2c, 0x02, \
  0x88, 0x32, 0x59, 0x88, 0x32, 0x59

#define kMaskRandom48_25 \
  0x10, 0x61, 0x06, 0x10, 0x61, 0x06, \
  0x02, 0x30, 0x23, 0x02, 0x30, 0x23, \
  0x40, 0x54, 0x05, 0x40, 0x54, 0x05, \
  0x21, 0x82, 0x18, 0x21, 0x82, 0x18, \
  0x81, 0x18, 0x11, 0x81, 0x18, 0x11, \
  0x14, 0x81, 0x48, 0x14, 0x81, 0x48, \
  0x98, 0x09, 0x80, 0x98, 0x09, 0x80, \
  0x08, 0x90, 0x89, 0x08, 0x90, 0x89, \
  0x62, 0x06, 0x20, 0x62, 0x06, 0x20, \
  0x24, 0x22, 0x42, 0x24, 0x22, 0x42, \
  0x8a, 0x08, 0xa0, 0x8a, 0x08, 0xa0, \
  0x84, 0x48, 0x44, 0x84, 0x48, 0x44, \
  0x51, 0x45, 0x14, 0x51, 0x45, 0x14, \
  0xc5, 0x1c, 0x51, 0xc5, 0x1c, 0x51, \
  0x21, 0x82, 0x18, 0x21, 0x82, 0x18, \
  0x12, 0x31, 0x23, 0x12, 0x31, 0x23, \
  0x08, 0xe0, 0x8e, 0x08, 0xe0, 0x8e, \
  0x2e, 0x02, 0xe0, 0x2e, 0x02, 0xe0, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x90, 0x99, 0x09, 0x90, 0x99, 0x09, \
  0x02, 0x50, 0x25, 0x02, 0x50, 0x25, \
  0x06, 0xa0, 0x6a, 0x06, 0xa0, 0x6a, \
  0x2c, 0x02, 0xc0, 0x2c, 0x02, 0xc0, \
  0x88, 0x68, 0x86, 0x88, 0x68, 0x86

#define kMaskRandom48_26 \
  0x51, 0x45, 0x14, 0x51, 0x45, 0x14, \
  0xc5, 0x1c, 0x51, 0xc5, 0x1c, 0x51, \
  0x21, 0x82, 0x18, 0x21, 0x82, 0x18, \
  0x12, 0x31, 0x23, 0x12, 0x31, 0x23, \
  0x08, 0xe0, 0x8e, 0x08, 0xe0, 0x8e, \
  0x2e, 0x02, 0xe0, 0x2e, 0x02, 0xe0, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x90, 0x99, 0x09, 0x90, 0x99, 0x09, \
  0x02, 0x50, 0x25, 0x02, 0x50, 0x25, \
  0x06, 0xa0, 0x6a, 0x06, 0xa0, 0x6a, \
  0x2c, 0x02, 0xc0, 0x2c, 0x02, 0xc0, \
  0x88, 0x68, 0x86, 0x88, 0x68, 0x86, \
  0x10, 0x61, 0x06, 0x10, 0x61, 0x06, \
  0x02, 0x30, 0x23, 0x02, 0x30, 0x23, \
  0x40, 0x54, 0x05, 0x40, 0x54, 0x05, \
  0x21, 0x82, 0x18, 0x21, 0x82, 0x18, \
  0x81, 0x18, 0x11, 0x81, 0x18, 0x11, \
  0x14, 0x81, 0x48, 0x14, 0x81, 0x48, \
  0x98, 0x09, 0x80, 0x98, 0x09, 0x80, \
  0x08, 0x90, 0x89, 0x08, 0x90, 0x89, \
  0x62, 0x06, 0x20, 0x62, 0x06, 0x20, \
  0x24, 0x22, 0x42, 0x24, 0x22, 0x42, \
  0x8a, 0x08, 0xa0, 0x8a, 0x08, 0xa0, \
  0x84, 0x48, 0x44, 0x84, 0x48, 0x44, \
  0x3e, 0x20, 0x79, 0xe5, 0x55, 0x70

#define kMaskRandom48_27 \
  0x51, 0x45, 0x14, 0x51, 0x45, 0x14, \
  0xc5, 0x1c, 0x51, 0xc5, 0x1c, 0x51, \
  0x21, 0x82, 0x18, 0x21, 0x82, 0x18, \
  0x12, 0x31, 0x23, 0x12, 0x31, 0x23, \
  0x08, 0xe0, 0x8e, 0x08, 0xe0, 0x8e, \
  0x2e, 0x02, 0xe0, 0x2e, 0x02, 0xe0, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x90, 0x99, 0x09, 0x90, 0x99, 0x09, \
  0x02, 0x50, 0x25, 0x02, 0x50, 0x25, \
  0x06, 0xa0, 0x6a, 0x06, 0xa0, 0x6a, \
  0x2c, 0x02, 0xc0, 0x2c, 0x02, 0xc0, \
  0x88, 0x68, 0x86, 0x88, 0x68, 0x86, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x90, 0x99, 0x09, 0x90, 0x99, 0x09, \
  0x02, 0x50, 0x25, 0x02, 0x50, 0x25, \
  0x06, 0xa0, 0x6a, 0x06, 0xa0, 0x6a, \
  0x2c, 0x02, 0xc0, 0x2c, 0x02, 0xc0, \
  0x88, 0x68, 0x86, 0x88, 0x68, 0x86, \
  0x51, 0x45, 0x14, 0x51, 0x45, 0x14, \
  0xc5, 0x1c, 0x51, 0xc5, 0x1c, 0x51, \
  0x21, 0x82, 0x18, 0x21, 0x82, 0x18, \
  0x12, 0x31, 0x23, 0x12, 0x31, 0x23, \
  0x08, 0xe0, 0x8e, 0x08, 0xe0, 0x8e, \
  0x2e, 0x02, 0xe0, 0x2e, 0x02, 0xe0, \
  0xf2, 0xd6, 0x8e, 0xf2, 0xd6, 0x8e

#define kMaskRandom48_28 \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x90, 0x99, 0x09, 0x90, 0x99, 0x09, \
  0x02, 0x50, 0x25, 0x02, 0x50, 0x25, \
  0x06, 0xa0, 0x6a, 0x06, 0xa0, 0x6a, \
  0x2c, 0x02, 0xc0, 0x2c, 0x02, 0xc0, \
  0x88, 0x68, 0x86, 0x88, 0x68, 0x86, \
  0x51, 0x45, 0x14, 0x51, 0x45, 0x14, \
  0xc5, 0x1c, 0x51, 0xc5, 0x1c, 0x51, \
  0x21, 0x82, 0x18, 0x21, 0x82, 0x18, \
  0x12, 0x31, 0x23, 0x12, 0x31, 0x23, \
  0x08, 0xe0, 0x8e, 0x08, 0xe0, 0x8e, \
  0x2e, 0x02, 0xe0, 0x2e, 0x02, 0xe0, \
  0xf2, 0xd6, 0x8e, 0xf2, 0xd6, 0x8e, \
  0x51, 0x45, 0x14, 0x51, 0x45, 0x14, \
  0xc5, 0x1c, 0x51, 0xc5, 0x1c, 0x51, \
  0x21, 0x82, 0x18, 0x21, 0x82, 0x18, \
  0x12, 0x31, 0x23, 0x12, 0x31, 0x23, \
  0x08, 0xe0, 0x8e, 0x08, 0xe0, 0x8e, \
  0x2e, 0x02, 0xe0, 0x2e, 0x02, 0xe0, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x90, 0x99, 0x09, 0x90, 0x99, 0x09, \
  0x02, 0x50, 0x25, 0x02, 0x50, 0x25, \
  0x06, 0xa0, 0x6a, 0x06, 0xa0, 0x6a, \
  0x2c, 0x02, 0xc0, 0x2c, 0x02, 0xc0, \
  0x88, 0x68, 0x86, 0x88, 0x68, 0x86, \
  0x32, 0xe3, 0xc0, 0x4a, 0xf2, 0x2a

#define kMaskRandom48_29 \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x90, 0x99, 0x09, 0x90, 0x99, 0x09, \
  0x02, 0x50, 0x25, 0x02, 0x50, 0x25, \
  0x06, 0xa0, 0x6a, 0x06, 0xa0, 0x6a, \
  0x2c, 0x02, 0xc0, 0x2c, 0x02, 0xc0, \
  0x88, 0x68, 0x86, 0x88, 0x68, 0x86, \
  0x51, 0x45, 0x14, 0x51, 0x45, 0x14, \
  0xc5, 0x1c, 0x51, 0xc5, 0x1c, 0x51, \
  0x21, 0x82, 0x18, 0x21, 0x82, 0x18, \
  0x12, 0x31, 0x23, 0x12, 0x31, 0x23, \
  0x08, 0xe0, 0x8e, 0x08, 0xe0, 0x8e, \
  0x2e, 0x02, 0xe0, 0x2e, 0x02, 0xe0, \
  0xf2, 0xd6, 0x8e, 0xf2, 0xd6, 0x8e, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x90, 0x99, 0x09, 0x90, 0x99, 0x09, \
  0x02, 0x50, 0x25, 0x02, 0x50, 0x25, \
  0x06, 0xa0, 0x6a, 0x06, 0xa0, 0x6a, \
  0x2c, 0x02, 0xc0, 0x2c, 0x02, 0xc0, \
  0x88, 0x68, 0x86, 0x88, 0x68, 0x86, \
  0x20, 0x62, 0x06, 0x20, 0x62, 0x06, \
  0x80, 0x38, 0x03, 0x80, 0x38, 0x03, \
  0x42, 0x44, 0x24, 0x42, 0x44, 0x24, \
  0x01, 0x90, 0x19, 0x01, 0x90, 0x19, \
  0x14, 0x11, 0x41, 0x14, 0x11, 0x41, \
  0x0a, 0x80, 0xa8, 0x0a, 0x80, 0xa8, \
  0x38, 0x03, 0x80, 0x38, 0x03, 0x80, \
  0xc5, 0x0c, 0x50, 0xc5, 0x0c, 0x50

#define kMaskRandom48_3 \
  0x9b, 0x29, 0xb2, 0x9b, 0x29, 0xb2, \
  0x49, 0xd4, 0x9d, 0x49, 0xd4, 0x9d, \
  0x3e, 0x83, 0xe8, 0x3e, 0x83, 0xe8

#define kMaskRandom48_30 \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x90, 0x99, 0x09, 0x90, 0x99, 0x09, \
  0x02, 0x50, 0x25, 0x02, 0x50, 0x25, \
  0x06, 0xa0, 0x6a, 0x06, 0xa0, 0x6a, \
  0x2c, 0x02, 0xc0, 0x2c, 0x02, 0xc0, \
  0x88, 0x68, 0x86, 0x88, 0x68, 0x86, \
  0x20, 0x62, 0x06, 0x20, 0x62, 0x06, \
  0x80, 0x38, 0x03, 0x80, 0x38, 0x03, \
  0x42, 0x44, 0x24, 0x42, 0x44, 0x24, \
  0x01, 0x90, 0x19, 0x01, 0x90, 0x19, \
  0x14, 0x11, 0x41, 0x14, 0x11, 0x41, \
  0x0a, 0x80, 0xa8, 0x0a, 0x80, 0xa8, \
  0x38, 0x03, 0x80, 0x38, 0x03, 0x80, \
  0xc5, 0x0c, 0x50, 0xc5, 0x0c, 0x50, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x90, 0x99, 0x09, 0x90, 0x99, 0x09, \
  0x02, 0x50, 0x25, 0x02, 0x50, 0x25, \
  0x06, 0xa0, 0x6a, 0x06, 0xa0, 0x6a, \
  0x2c, 0x02, 0xc0, 0x2c, 0x02, 0xc0, \
  0x88, 0x68, 0x86, 0x88, 0x68, 0x86, \
  0x51, 0x45, 0x14, 0x51, 0x45, 0x14, \
  0xc5, 0x1c, 0x51, 0xc5, 0x1c, 0x51, \
  0x21, 0x82, 0x18, 0x21, 0x82, 0x18, \
  0x12, 0x31, 0x23, 0x12, 0x31, 0x23, \
  0x08, 0xe0, 0x8e, 0x08, 0xe0, 0x8e, \
  0x2e, 0x02, 0xe0, 0x2e, 0x02, 0xe0, \
  0xf2, 0xd6, 0x8e, 0xf2, 0xd6, 0x8e, \
  0x66, 0xf3, 0x9a, 0xdd, 0x68, 0x93

#define kMaskRandom48_31 \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x90, 0x99, 0x09, 0x90, 0x99, 0x09, \
  0x02, 0x50, 0x25, 0x02, 0x50, 0x25, \
  0x06, 0xa0, 0x6a, 0x06, 0xa0, 0x6a, \
  0x2c, 0x02, 0xc0, 0x2c, 0x02, 0xc0, \
  0x88, 0x68, 0x86, 0x88, 0x68, 0x86, \
  0x20, 0x62, 0x06, 0x20, 0x62, 0x06, \
  0x80, 0x38, 0x03, 0x80, 0x38, 0x03, \
  0x42, 0x44, 0x24, 0x42, 0x44, 0x24, \
  0x01, 0x90, 0x19, 0x01, 0x90, 0x19, \
  0x14, 0x11, 0x41, 0x14, 0x11, 0x41, \
  0x0a, 0x80, 0xa8, 0x0a, 0x80, 0xa8, \
  0x38, 0x03, 0x80, 0x38, 0x03, 0x80, \
  0xc5, 0x0c, 0x50, 0xc5, 0x0c, 0x50, \
  0x20, 0x62, 0x06, 0x20, 0x62, 0x06, \
  0x80, 0x38, 0x03, 0x80, 0x38, 0x03, \
  0x42, 0x44, 0x24, 0x42, 0x44, 0x24, \
  0x01, 0x90, 0x19, 0x01, 0x90, 0x19, \
  0x14, 0x11, 0x41, 0x14, 0x11, 0x41, \
  0x0a, 0x80, 0xa8, 0x0a, 0x80, 0xa8, \
  0x38, 0x03, 0x80, 0x38, 0x03, 0x80, \
  0xc5, 0x0c, 0x50, 0xc5, 0x0c, 0x50, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x90, 0x99, 0x09, 0x90, 0x99, 0x09, \
  0x02, 0x50, 0x25, 0x02, 0x50, 0x25, \
  0x06, 0xa0, 0x6a, 0x06, 0xa0, 0x6a, \
  0x2c, 0x02, 0xc0, 0x2c, 0x02, 0xc0, \
  0x88, 0x68, 0x86, 0x88, 0x68, 0x86, \
  0xff, 0x6e, 0x0a, 0xff, 0x6e, 0x0a

#define kMaskRandom48_32 \
  0x20, 0x62, 0x06, 0x20, 0x62, 0x06, \
  0x80, 0x38, 0x03, 0x80, 0x38, 0x03, \
  0x42, 0x44, 0x24, 0x42, 0x44, 0x24, \
  0x01, 0x90, 0x19, 0x01, 0x90, 0x19, \
  0x14, 0x11, 0x41, 0x14, 0x11, 0x41, \
  0x0a, 0x80, 0xa8, 0x0a, 0x80, 0xa8, \
  0x38, 0x03, 0x80, 0x38, 0x03, 0x80, \
  0xc5, 0x0c, 0x50, 0xc5, 0x0c, 0x50, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x90, 0x99, 0x09, 0x90, 0x99, 0x09, \
  0x02, 0x50, 0x25, 0x02, 0x50, 0x25, \
  0x06, 0xa0, 0x6a, 0x06, 0xa0, 0x6a, \
  0x2c, 0x02, 0xc0, 0x2c, 0x02, 0xc0, \
  0x88, 0x68, 0x86, 0x88, 0x68, 0x86, \
  0xff, 0x6e, 0x0a, 0xff, 0x6e, 0x0a, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x90, 0x99, 0x09, 0x90, 0x99, 0x09, \
  0x02, 0x50, 0x25, 0x02, 0x50, 0x25, \
  0x06, 0xa0, 0x6a, 0x06, 0xa0, 0x6a, \
  0x2c, 0x02, 0xc0, 0x2c, 0x02, 0xc0, \
  0x88, 0x68, 0x86, 0x88, 0x68, 0x86, \
  0x20, 0x62, 0x06, 0x20, 0x62, 0x06, \
  0x80, 0x38, 0x03, 0x80, 0x38, 0x03, \
  0x42, 0x44, 0x24, 0x42, 0x44, 0x24, \
  0x01, 0x90, 0x19, 0x01, 0x90, 0x19, \
  0x14, 0x11, 0x41, 0x14, 0x11, 0x41, \
  0x0a, 0x80, 0xa8, 0x0a, 0x80, 0xa8, \
  0x38, 0x03, 0x80, 0x38, 0x03, 0x80, \
  0xc5, 0x0c, 0x50, 0xc5, 0x0c, 0x50, \
  0xd5, 0x4a, 0x4f, 0x48, 0xb5, 0x31

#define kMaskRandom48_33 \
  0x20, 0x62, 0x06, 0x20, 0x62, 0x06, \
  0x80, 0x38, 0x03, 0x80, 0x38, 0x03, \
  0x42, 0x44, 0x24, 0x42, 0x44, 0x24, \
  0x01, 0x90, 0x19, 0x01, 0x90, 0x19, \
  0x14, 0x11, 0x41, 0x14, 0x11, 0x41, \
  0x0a, 0x80, 0xa8, 0x0a, 0x80, 0xa8, \
  0x38, 0x03, 0x80, 0x38, 0x03, 0x80, \
  0xc5, 0x0c, 0x50, 0xc5, 0x0c, 0x50, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x90, 0x99, 0x09, 0x90, 0x99, 0x09, \
  0x02, 0x50, 0x25, 0x02, 0x50, 0x25, \
  0x06, 0xa0, 0x6a, 0x06, 0xa0, 0x6a, \
  0x2c, 0x02, 0xc0, 0x2c, 0x02, 0xc0, \
  0x88, 0x68, 0x86, 0x88, 0x68, 0x86, \
  0xff, 0x6e, 0x0a, 0xff, 0x6e, 0x0a, \
  0x20, 0x62, 0x06, 0x20, 0x62, 0x06, \
  0x80, 0x38, 0x03, 0x80, 0x38, 0x03, \
  0x42, 0x44, 0x24, 0x42, 0x44, 0x24, \
  0x01, 0x90, 0x19, 0x01, 0x90, 0x19, \
  0x14, 0x11, 0x41, 0x14, 0x11, 0x41, \
  0x0a, 0x80, 0xa8, 0x0a, 0x80, 0xa8, \
  0x38, 0x03, 0x80, 0x38, 0x03, 0x80, \
  0xc5, 0x0c, 0x50, 0xc5, 0x0c, 0x50, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0xe4, 0x2e, 0x42, 0xe4, 0x2e, 0x42, \
  0x24, 0x42, 0x44, 0x24, 0x42, 0x44, \
  0xa1, 0x1a, 0x11, 0xa1, 0x1a, 0x11, \
  0x18, 0x31, 0x83, 0x18, 0x31, 0x83, \
  0x03, 0x90, 0x39, 0x03, 0x90, 0x39, \
  0x8a, 0x18, 0xa1, 0x8a, 0x18, 0xa1, \
  0x04, 0x90, 0x49, 0x04, 0x90, 0x49, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e

#define kMaskRandom48_34 \
  0x20, 0x62, 0x06, 0x20, 0x62, 0x06, \
  0x80, 0x38, 0x03, 0x80, 0x38, 0x03, \
  0x42, 0x44, 0x24, 0x42, 0x44, 0x24, \
  0x01, 0x90, 0x19, 0x01, 0x90, 0x19, \
  0x14, 0x11, 0x41, 0x14, 0x11, 0x41, \
  0x0a, 0x80, 0xa8, 0x0a, 0x80, 0xa8, \
  0x38, 0x03, 0x80, 0x38, 0x03, 0x80, \
  0xc5, 0x0c, 0x50, 0xc5, 0x0c, 0x50, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0xe4, 0x2e, 0x42, 0xe4, 0x2e, 0x42, \
  0x24, 0x42, 0x44, 0x24, 0x42, 0x44, \
  0xa1, 0x1a, 0x11, 0xa1, 0x1a, 0x11, \
  0x18, 0x31, 0x83, 0x18, 0x31, 0x83, \
  0x03, 0x90, 0x39, 0x03, 0x90, 0x39, \
  0x8a, 0x18, 0xa1, 0x8a, 0x18, 0xa1, \
  0x04, 0x90, 0x49, 0x04, 0x90, 0x49, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0x20, 0x62, 0x06, 0x20, 0x62, 0x06, \
  0x80, 0x38, 0x03, 0x80, 0x38, 0x03, \
  0x42, 0x44, 0x24, 0x42, 0x44, 0x24, \
  0x01, 0x90, 0x19, 0x01, 0x90, 0x19, \
  0x14, 0x11, 0x41, 0x14, 0x11, 0x41, \
  0x0a, 0x80, 0xa8, 0x0a, 0x80, 0xa8, \
  0x38, 0x03, 0x80, 0x38, 0x03, 0x80, \
  0xc5, 0x0c, 0x50, 0xc5, 0x0c, 0x50, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x90, 0x99, 0x09, 0x90, 0x99, 0x09, \
  0x02, 0x50, 0x25, 0x02, 0x50, 0x25, \
  0x06, 0xa0, 0x6a, 0x06, 0xa0, 0x6a, \
  0x2c, 0x02, 0xc0, 0x2c, 0x02, 0xc0, \
  0x88, 0x68, 0x86, 0x88, 0x68, 0x86, \
  0xff, 0x6e, 0x0a, 0xff, 0x6e, 0x0a, \
  0x40, 0x72, 0x4c, 0xe8, 0xf2, 0x42

#define kMaskRandom48_35 \
  0x20, 0x62, 0x06, 0x20, 0x62, 0x06, \
  0x80, 0x38, 0x03, 0x80, 0x38, 0x03, \
  0x42, 0x44, 0x24, 0x42, 0x44, 0x24, \
  0x01, 0x90, 0x19, 0x01, 0x90, 0x19, \
  0x14, 0x11, 0x41, 0x14, 0x11, 0x41, \
  0x0a, 0x80, 0xa8, 0x0a, 0x80, 0xa8, \
  0x38, 0x03, 0x80, 0x38, 0x03, 0x80, \
  0xc5, 0x0c, 0x50, 0xc5, 0x0c, 0x50, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0xe4, 0x2e, 0x42, 0xe4, 0x2e, 0x42, \
  0x24, 0x42, 0x44, 0x24, 0x42, 0x44, \
  0xa1, 0x1a, 0x11, 0xa1, 0x1a, 0x11, \
  0x18, 0x31, 0x83, 0x18, 0x31, 0x83, \
  0x03, 0x90, 0x39, 0x03, 0x90, 0x39, \
  0x8a, 0x18, 0xa1, 0x8a, 0x18, 0xa1, \
  0x04, 0x90, 0x49, 0x04, 0x90, 0x49, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0xe4, 0x2e, 0x42, 0xe4, 0x2e, 0x42, \
  0x24, 0x42, 0x44, 0x24, 0x42, 0x44, \
  0xa1, 0x1a, 0x11, 0xa1, 0x1a, 0x11, \
  0x18, 0x31, 0x83, 0x18, 0x31, 0x83, \
  0x03, 0x90, 0x39, 0x03, 0x90, 0x39, \
  0x8a, 0x18, 0xa1, 0x8a, 0x18, 0xa1, \
  0x04, 0x90, 0x49, 0x04, 0x90, 0x49, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0x20, 0x62, 0x06, 0x20, 0x62, 0x06, \
  0x80, 0x38, 0x03, 0x80, 0x38, 0x03, \
  0x42, 0x44, 0x24, 0x42, 0x44, 0x24, \
  0x01, 0x90, 0x19, 0x01, 0x90, 0x19, \
  0x14, 0x11, 0x41, 0x14, 0x11, 0x41, \
  0x0a, 0x80, 0xa8, 0x0a, 0x80, 0xa8, \
  0x38, 0x03, 0x80, 0x38, 0x03, 0x80, \
  0xc5, 0x0c, 0x50, 0xc5, 0x0c, 0x50, \
  0x34, 0x50, 0xae, 0x34, 0x50, 0xae

#define kMaskRandom48_36 \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0xe4, 0x2e, 0x42, 0xe4, 0x2e, 0x42, \
  0x24, 0x42, 0x44, 0x24, 0x42, 0x44, \
  0xa1, 0x1a, 0x11, 0xa1, 0x1a, 0x11, \
  0x18, 0x31, 0x83, 0x18, 0x31, 0x83, \
  0x03, 0x90, 0x39, 0x03, 0x90, 0x39, \
  0x8a, 0x18, 0xa1, 0x8a, 0x18, 0xa1, \
  0x04, 0x90, 0x49, 0x04, 0x90, 0x49, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0x20, 0x62, 0x06, 0x20, 0x62, 0x06, \
  0x80, 0x38, 0x03, 0x80, 0x38, 0x03, \
  0x42, 0x44, 0x24, 0x42, 0x44, 0x24, \
  0x01, 0x90, 0x19, 0x01, 0x90, 0x19, \
  0x14, 0x11, 0x41, 0x14, 0x11, 0x41, \
  0x0a, 0x80, 0xa8, 0x0a, 0x80, 0xa8, \
  0x38, 0x03, 0x80, 0x38, 0x03, 0x80, \
  0xc5, 0x0c, 0x50, 0xc5, 0x0c, 0x50, \
  0x34, 0x50, 0xae, 0x34, 0x50, 0xae, \
  0x20, 0x62, 0x06, 0x20, 0x62, 0x06, \
  0x80, 0x38, 0x03, 0x80, 0x38, 0x03, \
  0x42, 0x44, 0x24, 0x42, 0x44, 0x24, \
  0x01, 0x90, 0x19, 0x01, 0x90, 0x19, \
  0x14, 0x11, 0x41, 0x14, 0x11, 0x41, \
  0x0a, 0x80, 0xa8, 0x0a, 0x80, 0xa8, \
  0x38, 0x03, 0x80, 0x38, 0x03, 0x80, \
  0xc5, 0x0c, 0x50, 0xc5, 0x0c, 0x50, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0xe4, 0x2e, 0x42, 0xe4, 0x2e, 0x42, \
  0x24, 0x42, 0x44, 0x24, 0x42, 0x44, \
  0xa1, 0x1a, 0x11, 0xa1, 0x1a, 0x11, \
  0x18, 0x31, 0x83, 0x18, 0x31, 0x83, \
  0x03, 0x90, 0x39, 0x03, 0x90, 0x39, \
  0x8a, 0x18, 0xa1, 0x8a, 0x18, 0xa1, \
  0x04, 0x90, 0x49, 0x04, 0x90, 0x49, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0x71, 0xba, 0x8b, 0xf3, 0xfa, 0x9d

#define kMaskRandom48_37 \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0xe4, 0x2e, 0x42, 0xe4, 0x2e, 0x42, \
  0x24, 0x42, 0x44, 0x24, 0x42, 0x44, \
  0xa1, 0x1a, 0x11, 0xa1, 0x1a, 0x11, \
  0x18, 0x31, 0x83, 0x18, 0x31, 0x83, \
  0x03, 0x90, 0x39, 0x03, 0x90, 0x39, \
  0x8a, 0x18, 0xa1, 0x8a, 0x18, 0xa1, \
  0x04, 0x90, 0x49, 0x04, 0x90, 0x49, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0x20, 0x62, 0x06, 0x20, 0x62, 0x06, \
  0x80, 0x38, 0x03, 0x80, 0x38, 0x03, \
  0x42, 0x44, 0x24, 0x42, 0x44, 0x24, \
  0x01, 0x90, 0x19, 0x01, 0x90, 0x19, \
  0x14, 0x11, 0x41, 0x14, 0x11, 0x41, \
  0x0a, 0x80, 0xa8, 0x0a, 0x80, 0xa8, \
  0x38, 0x03, 0x80, 0x38, 0x03, 0x80, \
  0xc5, 0x0c, 0x50, 0xc5, 0x0c, 0x50, \
  0x34, 0x50, 0xae, 0x34, 0x50, 0xae, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0xe4, 0x2e, 0x42, 0xe4, 0x2e, 0x42, \
  0x24, 0x42, 0x44, 0x24, 0x42, 0x44, \
  0xa1, 0x1a, 0x11, 0xa1, 0x1a, 0x11, \
  0x18, 0x31, 0x83, 0x18, 0x31, 0x83, \
  0x03, 0x90, 0x39, 0x03, 0x90, 0x39, \
  0x8a, 0x18, 0xa1, 0x8a, 0x18, 0xa1, \
  0x04, 0x90, 0x49, 0x04, 0x90, 0x49, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0x51, 0x45, 0x14, 0x51, 0x45, 0x14, \
  0x45, 0x14, 0x51, 0x45, 0x14, 0x51, \
  0x80, 0xd8, 0x0d, 0x80, 0xd8, 0x0d, \
  0x24, 0x22, 0x42, 0x24, 0x22, 0x42, \
  0x0a, 0x20, 0xa2, 0x0a, 0x20, 0xa2, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0xb8, 0x0b, 0x80, 0xb8, 0x0b, 0x80, \
  0x09, 0x10, 0x91, 0x09, 0x10, 0x91, \
  0x56, 0x05, 0x60, 0x56, 0x05, 0x60, \
  0xa2, 0x8a, 0x28, 0xa2, 0x8a, 0x28

#define kMaskRandom48_38 \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0xe4, 0x2e, 0x42, 0xe4, 0x2e, 0x42, \
  0x24, 0x42, 0x44, 0x24, 0x42, 0x44, \
  0xa1, 0x1a, 0x11, 0xa1, 0x1a, 0x11, \
  0x18, 0x31, 0x83, 0x18, 0x31, 0x83, \
  0x03, 0x90, 0x39, 0x03, 0x90, 0x39, \
  0x8a, 0x18, 0xa1, 0x8a, 0x18, 0xa1, \
  0x04, 0x90, 0x49, 0x04, 0x90, 0x49, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0x51, 0x45, 0x14, 0x51, 0x45, 0x14, \
  0x45, 0x14, 0x51, 0x45, 0x14, 0x51, \
  0x80, 0xd8, 0x0d, 0x80, 0xd8, 0x0d, \
  0x24, 0x22, 0x42, 0x24, 0x22, 0x42, \
  0x0a, 0x20, 0xa2, 0x0a, 0x20, 0xa2, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0xb8, 0x0b, 0x80, 0xb8, 0x0b, 0x80, \
  0x09, 0x10, 0x91, 0x09, 0x10, 0x91, \
  0x56, 0x05, 0x60, 0x56, 0x05, 0x60, \
  0xa2, 0x8a, 0x28, 0xa2, 0x8a, 0x28, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0xe4, 0x2e, 0x42, 0xe4, 0x2e, 0x42, \
  0x24, 0x42, 0x44, 0x24, 0x42, 0x44, \
  0xa1, 0x1a, 0x11, 0xa1, 0x1a, 0x11, \
  0x18, 0x31, 0x83, 0x18, 0x31, 0x83, \
  0x03, 0x90, 0x39, 0x03, 0x90, 0x39, \
  0x8a, 0x18, 0xa1, 0x8a, 0x18, 0xa1, \
  0x04, 0x90, 0x49, 0x04, 0x90, 0x49, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0x20, 0x62, 0x06, 0x20, 0x62, 0x06, \
  0x80, 0x38, 0x03, 0x80, 0x38, 0x03, \
  0x42, 0x44, 0x24, 0x42, 0x44, 0x24, \
  0x01, 0x90, 0x19, 0x01, 0x90, 0x19, \
  0x14, 0x11, 0x41, 0x14, 0x11, 0x41, \
  0x0a, 0x80, 0xa8, 0x0a, 0x80, 0xa8, \
  0x38, 0x03, 0x80, 0x38, 0x03, 0x80, \
  0xc5, 0x0c, 0x50, 0xc5, 0x0c, 0x50, \
  0x34, 0x50, 0xae, 0x34, 0x50, 0xae, \
  0x2a, 0x7a, 0xf6, 0x8c, 0xde, 0x51

#define kMaskRandom48_39 \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0xe4, 0x2e, 0x42, 0xe4, 0x2e, 0x42, \
  0x24, 0x42, 0x44, 0x24, 0x42, 0x44, \
  0xa1, 0x1a, 0x11, 0xa1, 0x1a, 0x11, \
  0x18, 0x31, 0x83, 0x18, 0x31, 0x83, \
  0x03, 0x90, 0x39, 0x03, 0x90, 0x39, \
  0x8a, 0x18, 0xa1, 0x8a, 0x18, 0xa1, \
  0x04, 0x90, 0x49, 0x04, 0x90, 0x49, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0x51, 0x45, 0x14, 0x51, 0x45, 0x14, \
  0x45, 0x14, 0x51, 0x45, 0x14, 0x51, \
  0x80, 0xd8, 0x0d, 0x80, 0xd8, 0x0d, \
  0x24, 0x22, 0x42, 0x24, 0x22, 0x42, \
  0x0a, 0x20, 0xa2, 0x0a, 0x20, 0xa2, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0xb8, 0x0b, 0x80, 0xb8, 0x0b, 0x80, \
  0x09, 0x10, 0x91, 0x09, 0x10, 0x91, \
  0x56, 0x05, 0x60, 0x56, 0x05, 0x60, \
  0xa2, 0x8a, 0x28, 0xa2, 0x8a, 0x28, \
  0x51, 0x45, 0x14, 0x51, 0x45, 0x14, \
  0x45, 0x14, 0x51, 0x45, 0x14, 0x51, \
  0x80, 0xd8, 0x0d, 0x80, 0xd8, 0x0d, \
  0x24, 0x22, 0x42, 0x24, 0x22, 0x42, \
  0x0a, 0x20, 0xa2, 0x0a, 0x20, 0xa2, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0xb8, 0x0b, 0x80, 0xb8, 0x0b, 0x80, \
  0x09, 0x10, 0x91, 0x09, 0x10, 0x91, \
  0x56, 0x05, 0x60, 0x56, 0x05, 0x60, \
  0xa2, 0x8a, 0x28, 0xa2, 0x8a, 0x28, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0xe4, 0x2e, 0x42, 0xe4, 0x2e, 0x42, \
  0x24, 0x42, 0x44, 0x24, 0x42, 0x44, \
  0xa1, 0x1a, 0x11, 0xa1, 0x1a, 0x11, \
  0x18, 0x31, 0x83, 0x18, 0x31, 0x83, \
  0x03, 0x90, 0x39, 0x03, 0x90, 0x39, \
  0x8a, 0x18, 0xa1, 0x8a, 0x18, 0xa1, \
  0x04, 0x90, 0x49, 0x04, 0x90, 0x49, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0x98, 0xa2, 0x95, 0x98, 0xa2, 0x95

#define kMaskRandom48_4 \
  0x8b, 0x28, 0xb2, 0x8b, 0x28, 0xb2, \
  0x14, 0xb1, 0x4b, 0x14, 0xb1, 0x4b, \
  0x22, 0xd2, 0x2d, 0x22, 0xd2, 0x2d, \
  0x45, 0x54, 0x55, 0x45, 0x54, 0x55

#define kMaskRandom48_40 \
  0x51, 0x45, 0x14, 0x51, 0x45, 0x14, \
  0x45, 0x14, 0x51, 0x45, 0x14, 0x51, \
  0x80, 0xd8, 0x0d, 0x80, 0xd8, 0x0d, \
  0x24, 0x22, 0x42, 0x24, 0x22, 0x42, \
  0x0a, 0x20, 0xa2, 0x0a, 0x20, 0xa2, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0xb8, 0x0b, 0x80, 0xb8, 0x0b, 0x80, \
  0x09, 0x10, 0x91, 0x09, 0x10, 0x91, \
  0x56, 0x05, 0x60, 0x56, 0x05, 0x60, \
  0xa2, 0x8a, 0x28, 0xa2, 0x8a, 0x28, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0xe4, 0x2e, 0x42, 0xe4, 0x2e, 0x42, \
  0x24, 0x42, 0x44, 0x24, 0x42, 0x44, \
  0xa1, 0x1a, 0x11, 0xa1, 0x1a, 0x11, \
  0x18, 0x31, 0x83, 0x18, 0x31, 0x83, \
  0x03, 0x90, 0x39, 0x03, 0x90, 0x39, \
  0x8a, 0x18, 0xa1, 0x8a, 0x18, 0xa1, \
  0x04, 0x90, 0x49, 0x04, 0x90, 0x49, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0x98, 0xa2, 0x95, 0x98, 0xa2, 0x95, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0xe4, 0x2e, 0x42, 0xe4, 0x2e, 0x42, \
  0x24, 0x42, 0x44, 0x24, 0x42, 0x44, \
  0xa1, 0x1a, 0x11, 0xa1, 0x1a, 0x11, \
  0x18, 0x31, 0x83, 0x18, 0x31, 0x83, \
  0x03, 0x90, 0x39, 0x03, 0x90, 0x39, \
  0x8a, 0x18, 0xa1, 0x8a, 0x18, 0xa1, \
  0x04, 0x90, 0x49, 0x04, 0x90, 0x49, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0x51, 0x45, 0x14, 0x51, 0x45, 0x14, \
  0x45, 0x14, 0x51, 0x45, 0x14, 0x51, \
  0x80, 0xd8, 0x0d, 0x80, 0xd8, 0x0d, \
  0x24, 0x22, 0x42, 0x24, 0x22, 0x42, \
  0x0a, 0x20, 0xa2, 0x0a, 0x20, 0xa2, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0xb8, 0x0b, 0x80, 0xb8, 0x0b, 0x80, \
  0x09, 0x10, 0x91, 0x09, 0x10, 0x91, \
  0x56, 0x05, 0x60, 0x56, 0x05, 0x60, \
  0xa2, 0x8a, 0x28, 0xa2, 0x8a, 0x28, \
  0x20, 0x5f, 0x68, 0xd5, 0xa2, 0x1b

#define kMaskRandom48_41 \
  0x51, 0x45, 0x14, 0x51, 0x45, 0x14, \
  0x45, 0x14, 0x51, 0x45, 0x14, 0x51, \
  0x80, 0xd8, 0x0d, 0x80, 0xd8, 0x0d, \
  0x24, 0x22, 0x42, 0x24, 0x22, 0x42, \
  0x0a, 0x20, 0xa2, 0x0a, 0x20, 0xa2, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0xb8, 0x0b, 0x80, 0xb8, 0x0b, 0x80, \
  0x09, 0x10, 0x91, 0x09, 0x10, 0x91, \
  0x56, 0x05, 0x60, 0x56, 0x05, 0x60, \
  0xa2, 0x8a, 0x28, 0xa2, 0x8a, 0x28, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0xe4, 0x2e, 0x42, 0xe4, 0x2e, 0x42, \
  0x24, 0x42, 0x44, 0x24, 0x42, 0x44, \
  0xa1, 0x1a, 0x11, 0xa1, 0x1a, 0x11, \
  0x18, 0x31, 0x83, 0x18, 0x31, 0x83, \
  0x03, 0x90, 0x39, 0x03, 0x90, 0x39, \
  0x8a, 0x18, 0xa1, 0x8a, 0x18, 0xa1, \
  0x04, 0x90, 0x49, 0x04, 0x90, 0x49, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0x98, 0xa2, 0x95, 0x98, 0xa2, 0x95, \
  0x51, 0x45, 0x14, 0x51, 0x45, 0x14, \
  0x45, 0x14, 0x51, 0x45, 0x14, 0x51, \
  0x80, 0xd8, 0x0d, 0x80, 0xd8, 0x0d, \
  0x24, 0x22, 0x42, 0x24, 0x22, 0x42, \
  0x0a, 0x20, 0xa2, 0x0a, 0x20, 0xa2, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0xb8, 0x0b, 0x80, 0xb8, 0x0b, 0x80, \
  0x09, 0x10, 0x91, 0x09, 0x10, 0x91, \
  0x56, 0x05, 0x60, 0x56, 0x05, 0x60, \
  0xa2, 0x8a, 0x28, 0xa2, 0x8a, 0x28, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x10, 0x91, 0x09, 0x10, 0x91, 0x09, \
  0x00, 0x70, 0x07, 0x00, 0x70, 0x07, \
  0x0c, 0x10, 0xc1, 0x0c, 0x10, 0xc1, \
  0x40, 0xc4, 0x0c, 0x40, 0xc4, 0x0c, \
  0x6a, 0x06, 0xa0, 0x6a, 0x06, 0xa0, \
  0x86, 0x08, 0x60, 0x86, 0x08, 0x60, \
  0x24, 0x82, 0x48, 0x24, 0x82, 0x48, \
  0x89, 0x08, 0x90, 0x89, 0x08, 0x90, \
  0xc0, 0x2c, 0x02, 0xc0, 0x2c, 0x02

#define kMaskRandom48_42 \
  0x51, 0x45, 0x14, 0x51, 0x45, 0x14, \
  0x45, 0x14, 0x51, 0x45, 0x14, 0x51, \
  0x80, 0xd8, 0x0d, 0x80, 0xd8, 0x0d, \
  0x24, 0x22, 0x42, 0x24, 0x22, 0x42, \
  0x0a, 0x20, 0xa2, 0x0a, 0x20, 0xa2, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0xb8, 0x0b, 0x80, 0xb8, 0x0b, 0x80, \
  0x09, 0x10, 0x91, 0x09, 0x10, 0x91, \
  0x56, 0x05, 0x60, 0x56, 0x05, 0x60, \
  0xa2, 0x8a, 0x28, 0xa2, 0x8a, 0x28, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x10, 0x91, 0x09, 0x10, 0x91, 0x09, \
  0x00, 0x70, 0x07, 0x00, 0x70, 0x07, \
  0x0c, 0x10, 0xc1, 0x0c, 0x10, 0xc1, \
  0x40, 0xc4, 0x0c, 0x40, 0xc4, 0x0c, \
  0x6a, 0x06, 0xa0, 0x6a, 0x06, 0xa0, \
  0x86, 0x08, 0x60, 0x86, 0x08, 0x60, \
  0x24, 0x82, 0x48, 0x24, 0x82, 0x48, \
  0x89, 0x08, 0x90, 0x89, 0x08, 0x90, \
  0xc0, 0x2c, 0x02, 0xc0, 0x2c, 0x02, \
  0x51, 0x45, 0x14, 0x51, 0x45, 0x14, \
  0x45, 0x14, 0x51, 0x45, 0x14, 0x51, \
  0x80, 0xd8, 0x0d, 0x80, 0xd8, 0x0d, \
  0x24, 0x22, 0x42, 0x24, 0x22, 0x42, \
  0x0a, 0x20, 0xa2, 0x0a, 0x20, 0xa2, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0xb8, 0x0b, 0x80, 0xb8, 0x0b, 0x80, \
  0x09, 0x10, 0x91, 0x09, 0x10, 0x91, \
  0x56, 0x05, 0x60, 0x56, 0x05, 0x60, \
  0xa2, 0x8a, 0x28, 0xa2, 0x8a, 0x28, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0xe4, 0x2e, 0x42, 0xe4, 0x2e, 0x42, \
  0x24, 0x42, 0x44, 0x24, 0x42, 0x44, \
  0xa1, 0x1a, 0x11, 0xa1, 0x1a, 0x11, \
  0x18, 0x31, 0x83, 0x18, 0x31, 0x83, \
  0x03, 0x90, 0x39, 0x03, 0x90, 0x39, \
  0x8a, 0x18, 0xa1, 0x8a, 0x18, 0xa1, \
  0x04, 0x90, 0x49, 0x04, 0x90, 0x49, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0x98, 0xa2, 0x95, 0x98, 0xa2, 0x95, \
  0x66, 0xcf, 0xa3, 0x47, 0x69, 0x00

#define kMaskRandom48_43 \
  0x51, 0x45, 0x14, 0x51, 0x45, 0x14, \
  0x45, 0x14, 0x51, 0x45, 0x14, 0x51, \
  0x80, 0xd8, 0x0d, 0x80, 0xd8, 0x0d, \
  0x24, 0x22, 0x42, 0x24, 0x22, 0x42, \
  0x0a, 0x20, 0xa2, 0x0a, 0x20, 0xa2, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0xb8, 0x0b, 0x80, 0xb8, 0x0b, 0x80, \
  0x09, 0x10, 0x91, 0x09, 0x10, 0x91, \
  0x56, 0x05, 0x60, 0x56, 0x05, 0x60, \
  0xa2, 0x8a, 0x28, 0xa2, 0x8a, 0x28, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x10, 0x91, 0x09, 0x10, 0x91, 0x09, \
  0x00, 0x70, 0x07, 0x00, 0x70, 0x07, \
  0x0c, 0x10, 0xc1, 0x0c, 0x10, 0xc1, \
  0x40, 0xc4, 0x0c, 0x40, 0xc4, 0x0c, \
  0x6a, 0x06, 0xa0, 0x6a, 0x06, 0xa0, \
  0x86, 0x08, 0x60, 0x86, 0x08, 0x60, \
  0x24, 0x82, 0x48, 0x24, 0x82, 0x48, \
  0x89, 0x08, 0x90, 0x89, 0x08, 0x90, \
  0xc0, 0x2c, 0x02, 0xc0, 0x2c, 0x02, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x10, 0x91, 0x09, 0x10, 0x91, 0x09, \
  0x00, 0x70, 0x07, 0x00, 0x70, 0x07, \
  0x0c, 0x10, 0xc1, 0x0c, 0x10, 0xc1, \
  0x40, 0xc4, 0x0c, 0x40, 0xc4, 0x0c, \
  0x6a, 0x06, 0xa0, 0x6a, 0x06, 0xa0, \
  0x86, 0x08, 0x60, 0x86, 0x08, 0x60, \
  0x24, 0x82, 0x48, 0x24, 0x82, 0x48, \
  0x89, 0x08, 0x90, 0x89, 0x08, 0x90, \
  0xc0, 0x2c, 0x02, 0xc0, 0x2c, 0x02, \
  0x51, 0x45, 0x14, 0x51, 0x45, 0x14, \
  0x45, 0x14, 0x51, 0x45, 0x14, 0x51, \
  0x80, 0xd8, 0x0d, 0x80, 0xd8, 0x0d, \
  0x24, 0x22, 0x42, 0x24, 0x22, 0x42, \
  0x0a, 0x20, 0xa2, 0x0a, 0x20, 0xa2, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0xb8, 0x0b, 0x80, 0xb8, 0x0b, 0x80, \
  0x09, 0x10, 0x91, 0x09, 0x10, 0x91, \
  0x56, 0x05, 0x60, 0x56, 0x05, 0x60, \
  0xa2, 0x8a, 0x28, 0xa2, 0x8a, 0x28, \
  0x1a, 0xaa, 0xee, 0x1a, 0xaa, 0xee

#define kMaskRandom48_44 \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x10, 0x91, 0x09, 0x10, 0x91, 0x09, \
  0x00, 0x70, 0x07, 0x00, 0x70, 0x07, \
  0x0c, 0x10, 0xc1, 0x0c, 0x10, 0xc1, \
  0x40, 0xc4, 0x0c, 0x40, 0xc4, 0x0c, \
  0x6a, 0x06, 0xa0, 0x6a, 0x06, 0xa0, \
  0x86, 0x08, 0x60, 0x86, 0x08, 0x60, \
  0x24, 0x82, 0x48, 0x24, 0x82, 0x48, \
  0x89, 0x08, 0x90, 0x89, 0x08, 0x90, \
  0xc0, 0x2c, 0x02, 0xc0, 0x2c, 0x02, \
  0x51, 0x45, 0x14, 0x51, 0x45, 0x14, \
  0x45, 0x14, 0x51, 0x45, 0x14, 0x51, \
  0x80, 0xd8, 0x0d, 0x80, 0xd8, 0x0d, \
  0x24, 0x22, 0x42, 0x24, 0x22, 0x42, \
  0x0a, 0x20, 0xa2, 0x0a, 0x20, 0xa2, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0xb8, 0x0b, 0x80, 0xb8, 0x0b, 0x80, \
  0x09, 0x10, 0x91, 0x09, 0x10, 0x91, \
  0x56, 0x05, 0x60, 0x56, 0x05, 0x60, \
  0xa2, 0x8a, 0x28, 0xa2, 0x8a, 0x28, \
  0x1a, 0xaa, 0xee, 0x1a, 0xaa, 0xee, \
  0x51, 0x45, 0x14, 0x51, 0x45, 0x14, \
  0x45, 0x14, 0x51, 0x45, 0x14, 0x51, \
  0x80, 0xd8, 0x0d, 0x80, 0xd8, 0x0d, \
  0x24, 0x22, 0x42, 0x24, 0x22, 0x42, \
  0x0a, 0x20, 0xa2, 0x0a, 0x20, 0xa2, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0xb8, 0x0b, 0x80, 0xb8, 0x0b, 0x80, \
  0x09, 0x10, 0x91, 0x09, 0x10, 0x91, \
  0x56, 0x05, 0x60, 0x56, 0x05, 0x60, \
  0xa2, 0x8a, 0x28, 0xa2, 0x8a, 0x28, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x10, 0x91, 0x09, 0x10, 0x91, 0x09, \
  0x00, 0x70, 0x07, 0x00, 0x70, 0x07, \
  0x0c, 0x10, 0xc1, 0x0c, 0x10, 0xc1, \
  0x40, 0xc4, 0x0c, 0x40, 0xc4, 0x0c, \
  0x6a, 0x06, 0xa0, 0x6a, 0x06, 0xa0, \
  0x86, 0x08, 0x60, 0x86, 0x08, 0x60, \
  0x24, 0x82, 0x48, 0x24, 0x82, 0x48, \
  0x89, 0x08, 0x90, 0x89, 0x08, 0x90, \
  0xc0, 0x2c, 0x02, 0xc0, 0x2c, 0x02, \
  0xc6, 0x40, 0x1f, 0x57, 0xc6, 0xe6

#define kMaskRandom48_45 \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x10, 0x91, 0x09, 0x10, 0x91, 0x09, \
  0x00, 0x70, 0x07, 0x00, 0x70, 0x07, \
  0x0c, 0x10, 0xc1, 0x0c, 0x10, 0xc1, \
  0x40, 0xc4, 0x0c, 0x40, 0xc4, 0x0c, \
  0x6a, 0x06, 0xa0, 0x6a, 0x06, 0xa0, \
  0x86, 0x08, 0x60, 0x86, 0x08, 0x60, \
  0x24, 0x82, 0x48, 0x24, 0x82, 0x48, \
  0x89, 0x08, 0x90, 0x89, 0x08, 0x90, \
  0xc0, 0x2c, 0x02, 0xc0, 0x2c, 0x02, \
  0x51, 0x45, 0x14, 0x51, 0x45, 0x14, \
  0x45, 0x14, 0x51, 0x45, 0x14, 0x51, \
  0x80, 0xd8, 0x0d, 0x80, 0xd8, 0x0d, \
  0x24, 0x22, 0x42, 0x24, 0x22, 0x42, \
  0x0a, 0x20, 0xa2, 0x0a, 0x20, 0xa2, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0xb8, 0x0b, 0x80, 0xb8, 0x0b, 0x80, \
  0x09, 0x10, 0x91, 0x09, 0x10, 0x91, \
  0x56, 0x05, 0x60, 0x56, 0x05, 0x60, \
  0xa2, 0x8a, 0x28, 0xa2, 0x8a, 0x28, \
  0x1a, 0xaa, 0xee, 0x1a, 0xaa, 0xee, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x10, 0x91, 0x09, 0x10, 0x91, 0x09, \
  0x00, 0x70, 0x07, 0x00, 0x70, 0x07, \
  0x0c, 0x10, 0xc1, 0x0c, 0x10, 0xc1, \
  0x40, 0xc4, 0x0c, 0x40, 0xc4, 0x0c, \
  0x6a, 0x06, 0xa0, 0x6a, 0x06, 0xa0, \
  0x86, 0x08, 0x60, 0x86, 0x08, 0x60, \
  0x24, 0x82, 0x48, 0x24, 0x82, 0x48, \
  0x89, 0x08, 0x90, 0x89, 0x08, 0x90, \
  0xc0, 0x2c, 0x02, 0xc0, 0x2c, 0x02, \
  0x10, 0x61, 0x06, 0x10, 0x61, 0x06, \
  0x02, 0x30, 0x23, 0x02, 0x30, 0x23, \
  0x40, 0x54, 0x05, 0x40, 0x54, 0x05, \
  0x21, 0x82, 0x18, 0x21, 0x82, 0x18, \
  0x81, 0x18, 0x11, 0x81, 0x18, 0x11, \
  0x14, 0x81, 0x48, 0x14, 0x81, 0x48, \
  0x98, 0x09, 0x80, 0x98, 0x09, 0x80, \
  0x08, 0x90, 0x89, 0x08, 0x90, 0x89, \
  0x62, 0x06, 0x20, 0x62, 0x06, 0x20, \
  0x24, 0x22, 0x42, 0x24, 0x22, 0x42, \
  0x8a, 0x08, 0xa0, 0x8a, 0x08, 0xa0, \
  0x84, 0x48, 0x44, 0x84, 0x48, 0x44

#define kMaskRandom48_46 \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x10, 0x91, 0x09, 0x10, 0x91, 0x09, \
  0x00, 0x70, 0x07, 0x00, 0x70, 0x07, \
  0x0c, 0x10, 0xc1, 0x0c, 0x10, 0xc1, \
  0x40, 0xc4, 0x0c, 0x40, 0xc4, 0x0c, \
  0x6a, 0x06, 0xa0, 0x6a, 0x06, 0xa0, \
  0x86, 0x08, 0x60, 0x86, 0x08, 0x60, \
  0x24, 0x82, 0x48, 0x24, 0x82, 0x48, \
  0x89, 0x08, 0x90, 0x89, 0x08, 0x90, \
  0xc0, 0x2c, 0x02, 0xc0, 0x2c, 0x02, \
  0x10, 0x61, 0x06, 0x10, 0x61, 0x06, \
  0x02, 0x30, 0x23, 0x02, 0x30, 0x23, \
  0x40, 0x54, 0x05, 0x40, 0x54, 0x05, \
  0x21, 0x82, 0x18, 0x21, 0x82, 0x18, \
  0x81, 0x18, 0x11, 0x81, 0x18, 0x11, \
  0x14, 0x81, 0x48, 0x14, 0x81, 0x48, \
  0x98, 0x09, 0x80, 0x98, 0x09, 0x80, \
  0x08, 0x90, 0x89, 0x08, 0x90, 0x89, \
  0x62, 0x06, 0x20, 0x62, 0x06, 0x20, \
  0x24, 0x22, 0x42, 0x24, 0x22, 0x42, \
  0x8a, 0x08, 0xa0, 0x8a, 0x08, 0xa0, \
  0x84, 0x48, 0x44, 0x84, 0x48, 0x44, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x10, 0x91, 0x09, 0x10, 0x91, 0x09, \
  0x00, 0x70, 0x07, 0x00, 0x70, 0x07, \
  0x0c, 0x10, 0xc1, 0x0c, 0x10, 0xc1, \
  0x40, 0xc4, 0x0c, 0x40, 0xc4, 0x0c, \
  0x6a, 0x06, 0xa0, 0x6a, 0x06, 0xa0, \
  0x86, 0x08, 0x60, 0x86, 0x08, 0x60, \
  0x24, 0x82, 0x48, 0x24, 0x82, 0x48, \
  0x89, 0x08, 0x90, 0x89, 0x08, 0x90, \
  0xc0, 0x2c, 0x02, 0xc0, 0x2c, 0x02, \
  0x51, 0x45, 0x14, 0x51, 0x45, 0x14, \
  0x45, 0x14, 0x51, 0x45, 0x14, 0x51, \
  0x80, 0xd8, 0x0d, 0x80, 0xd8, 0x0d, \
  0x24, 0x22, 0x42, 0x24, 0x22, 0x42, \
  0x0a, 0x20, 0xa2, 0x0a, 0x20, 0xa2, \
  0x00, 0xe0, 0x0e, 0x00, 0xe0, 0x0e, \
  0xb8, 0x0b, 0x80, 0xb8, 0x0b, 0x80, \
  0x09, 0x10, 0x91, 0x09, 0x10, 0x91, \
  0x56, 0x05, 0x60, 0x56, 0x05, 0x60, \
  0xa2, 0x8a, 0x28, 0xa2, 0x8a, 0x28, \
  0x1a, 0xaa, 0xee, 0x1a, 0xaa, 0xee, \
  0x10, 0xf9, 0xab, 0x12, 0x14, 0xef

#define kMaskRandom48_47 \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x10, 0x91, 0x09, 0x10, 0x91, 0x09, \
  0x00, 0x70, 0x07, 0x00, 0x70, 0x07, \
  0x0c, 0x10, 0xc1, 0x0c, 0x10, 0xc1, \
  0x40, 0xc4, 0x0c, 0x40, 0xc4, 0x0c, \
  0x6a, 0x06, 0xa0, 0x6a, 0x06, 0xa0, \
  0x86, 0x08, 0x60, 0x86, 0x08, 0x60, \
  0x24, 0x82, 0x48, 0x24, 0x82, 0x48, \
  0x89, 0x08, 0x90, 0x89, 0x08, 0x90, \
  0xc0, 0x2c, 0x02, 0xc0, 0x2c, 0x02, \
  0x10, 0x61, 0x06, 0x10, 0x61, 0x06, \
  0x02, 0x30, 0x23, 0x02, 0x30, 0x23, \
  0x40, 0x54, 0x05, 0x40, 0x54, 0x05, \
  0x21, 0x82, 0x18, 0x21, 0x82, 0x18, \
  0x81, 0x18, 0x11, 0x81, 0x18, 0x11, \
  0x14, 0x81, 0x48, 0x14, 0x81, 0x48, \
  0x98, 0x09, 0x80, 0x98, 0x09, 0x80, \
  0x08, 0x90, 0x89, 0x08, 0x90, 0x89, \
  0x62, 0x06, 0x20, 0x62, 0x06, 0x20, \
  0x24, 0x22, 0x42, 0x24, 0x22, 0x42, \
  0x8a, 0x08, 0xa0, 0x8a, 0x08, 0xa0, \
  0x84, 0x48, 0x44, 0x84, 0x48, 0x44, \
  0x10, 0x61, 0x06, 0x10, 0x61, 0x06, \
  0x02, 0x30, 0x23, 0x02, 0x30, 0x23, \
  0x40, 0x54, 0x05, 0x40, 0x54, 0x05, \
  0x21, 0x82, 0x18, 0x21, 0x82, 0x18, \
  0x81, 0x18, 0x11, 0x81, 0x18, 0x11, \
  0x14, 0x81, 0x48, 0x14, 0x81, 0x48, \
  0x98, 0x09, 0x80, 0x98, 0x09, 0x80, \
  0x08, 0x90, 0x89, 0x08, 0x90, 0x89, \
  0x62, 0x06, 0x20, 0x62, 0x06, 0x20, \
  0x24, 0x22, 0x42, 0x24, 0x22, 0x42, \
  0x8a, 0x08, 0xa0, 0x8a, 0x08, 0xa0, \
  0x84, 0x48, 0x44, 0x84, 0x48, 0x44, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x10, 0x91, 0x09, 0x10, 0x91, 0x09, \
  0x00, 0x70, 0x07, 0x00, 0x70, 0x07, \
  0x0c, 0x10, 0xc1, 0x0c, 0x10, 0xc1, \
  0x40, 0xc4, 0x0c, 0x40, 0xc4, 0x0c, \
  0x6a, 0x06, 0xa0, 0x6a, 0x06, 0xa0, \
  0x86, 0x08, 0x60, 0x86, 0x08, 0x60, \
  0x24, 0x82, 0x48, 0x24, 0x82, 0x48, \
  0x89, 0x08, 0x90, 0x89, 0x08, 0x90, \
  0xc0, 0x2c, 0x02, 0xc0, 0x2c, 0x02, \
  0x88, 0x32, 0x59, 0x88, 0x32, 0x59

#define kMaskRandom48_48 \
  0x10, 0x61, 0x06, 0x10, 0x61, 0x06, \
  0x02, 0x30, 0x23, 0x02, 0x30, 0x23, \
  0x40, 0x54, 0x05, 0x40, 0x54, 0x05, \
  0x21, 0x82, 0x18, 0x21, 0x82, 0x18, \
  0x81, 0x18, 0x11, 0x81, 0x18, 0x11, \
  0x14, 0x81, 0x48, 0x14, 0x81, 0x48, \
  0x98, 0x09, 0x80, 0x98, 0x09, 0x80, \
  0x08, 0x90, 0x89, 0x08, 0x90, 0x89, \
  0x62, 0x06, 0x20, 0x62, 0x06, 0x20, \
  0x24, 0x22, 0x42, 0x24, 0x22, 0x42, \
  0x8a, 0x08, 0xa0, 0x8a, 0x08, 0xa0, \
  0x84, 0x48, 0x44, 0x84, 0x48, 0x44, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x10, 0x91, 0x09, 0x10, 0x91, 0x09, \
  0x00, 0x70, 0x07, 0x00, 0x70, 0x07, \
  0x0c, 0x10, 0xc1, 0x0c, 0x10, 0xc1, \
  0x40, 0xc4, 0x0c, 0x40, 0xc4, 0x0c, \
  0x6a, 0x06, 0xa0, 0x6a, 0x06, 0xa0, \
  0x86, 0x08, 0x60, 0x86, 0x08, 0x60, \
  0x24, 0x82, 0x48, 0x24, 0x82, 0x48, \
  0x89, 0x08, 0x90, 0x89, 0x08, 0x90, \
  0xc0, 0x2c, 0x02, 0xc0, 0x2c, 0x02, \
  0x88, 0x32, 0x59, 0x88, 0x32, 0x59, \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x21, 0x32, 0x13, 0x21, 0x32, 0x13, \
  0x10, 0x91, 0x09, 0x10, 0x91, 0x09, \
  0x00, 0x70, 0x07, 0x00, 0x70, 0x07, \
  0x0c, 0x10, 0xc1, 0x0c, 0x10, 0xc1, \
  0x40, 0xc4, 0x0c, 0x40, 0xc4, 0x0c, \
  0x6a, 0x06, 0xa0, 0x6a, 0x06, 0xa0, \
  0x86, 0x08, 0x60, 0x86, 0x08, 0x60, \
  0x24, 0x82, 0x48, 0x24, 0x82, 0x48, \
  0x89, 0x08, 0x90, 0x89, 0x08, 0x90, \
  0xc0, 0x2c, 0x02, 0xc0, 0x2c, 0x02, \
  0x10, 0x61, 0x06, 0x10, 0x61, 0x06, \
  0x02, 0x30, 0x23, 0x02, 0x30, 0x23, \
  0x40, 0x54, 0x05, 0x40, 0x54, 0x05, \
  0x21, 0x82, 0x18, 0x21, 0x82, 0x18, \
  0x81, 0x18, 0x11, 0x81, 0x18, 0x11, \
  0x14, 0x81, 0x48, 0x14, 0x81, 0x48, \
  0x98, 0x09, 0x80, 0x98, 0x09, 0x80, \
  0x08, 0x90, 0x89, 0x08, 0x90, 0x89, \
  0x62, 0x06, 0x20, 0x62, 0x06, 0x20, \
  0x24, 0x22, 0x42, 0x24, 0x22, 0x42, \
  0x8a, 0x08, 0xa0, 0x8a, 0x08, 0xa0, \
  0x84, 0x48, 0x44, 0x84, 0x48, 0x44, \
  0xff, 0x9b, 0xdf, 0xec, 0xae, 0x0e

#define kMaskRandom48_5 \
  0x53, 0x65, 0x36, 0x53, 0x65, 0x36, \
  0x64, 0xb6, 0x4b, 0x64, 0xb6, 0x4b, \
  0x0e, 0xe0, 0xee, 0x0e, 0xe0, 0xee, \
  0xa9, 0xca, 0x9c, 0xa9, 0xca, 0x9c, \
  0xb8, 0x3b, 0x83, 0xb8, 0x3b, 0x83

#define kMaskRandom48_6 \
  0xd1, 0x4d, 0x14, 0xd1, 0x4d, 0x14, \
  0x45, 0x34, 0x53, 0x45, 0x34, 0x53, \
  0x22, 0xd2, 0x2d, 0x22, 0xd2, 0x2d, \
  0x16, 0xc1, 0x6c, 0x16, 0xc1, 0x6c, \
  0x0b, 0xa0, 0xba, 0x0b, 0xa0, 0xba, \
  0xe8, 0x8e, 0x88, 0xe8, 0x8e, 0x88

#define kMaskRandom48_7 \
  0xd3, 0x65, 0x36, 0xd3, 0x65, 0x36, \
  0x25, 0x32, 0x53, 0x25, 0x32, 0x53, \
  0x30, 0xd3, 0x05, 0x30, 0xd3, 0x05, \
  0x06, 0x48, 0x6c, 0x06, 0x48, 0x6c, \
  0xc0, 0xb8, 0x1b, 0xc0, 0xb8, 0x1b, \
  0x2a, 0xa2, 0xaa, 0x2a, 0xa2, 0xaa, \
  0xa8, 0x4e, 0x84, 0xa8, 0x4e, 0x84

#define kMaskRandom48_8 \
  0x81, 0x60, 0x16, 0x81, 0x60, 0x16, \
  0x40, 0x3c, 0x03, 0x40, 0x3c, 0x03, \
  0x10, 0x91, 0x09, 0x10, 0x91, 0x09, \
  0x06, 0x50, 0x65, 0x06, 0x50, 0x65, \
  0x20, 0x4a, 0x84, 0x20, 0x4a, 0x84, \
  0x8a, 0xa0, 0xaa, 0x8a, 0xa0, 0xaa, \
  0x33, 0x03, 0x30, 0x33, 0x03, 0x30, \
  0x4c, 0x84, 0xc8, 0x4c, 0x84, 0xc8

#define kMaskRandom48_9 \
  0xd3, 0x65, 0x36, 0xd3, 0x65, 0x36, \
  0x64, 0x26, 0x42, 0x64, 0x26, 0x42, \
  0x18, 0x41, 0xc4, 0x18, 0x41, 0xc4, \
  0xa0, 0x4a, 0x04, 0xa0, 0x4a, 0x04, \
  0x81, 0x38, 0x13, 0x81, 0x38, 0x13, \
  0x22, 0xa2, 0x2a, 0x22, 0xa2, 0x2a, \
  0x08, 0x70, 0x87, 0x08, 0x70, 0x87, \
  0x04, 0x90, 0x49, 0x04, 0x90, 0x49, \
  0x01, 0xc0, 0x1c, 0x01, 0xc0, 0x1c

#define kPacketMaskRandom1 1, \
  kMaskRandom1_1

#define kPacketMaskRandom2 2, \
  kMaskRandom2_1, \
  kMaskRandom2_2

#define kPacketMaskRandom3 3, \
  kMaskRandom3_1, \
  kMaskRandom3_2, \
  kMaskRandom3_3

#define kPacketMaskRandom4 4, \
  kMaskRandom4_1, \
  kMaskRandom4_2, \
  kMaskRandom4_3, \
  kMaskRandom4_4

#define kPacketMaskRandom5 5, \
  kMaskRandom5_1, \
  kMaskRandom5_2, \
  kMaskRandom5_3, \
  kMaskRandom5_4, \
  kMaskRandom5_5

#define kPacketMaskRandom6 6, \
  kMaskRandom6_1, \
  kMaskRandom6_2, \
  kMaskRandom6_3, \
  kMaskRandom6_4, \
  kMaskRandom6_5, \
  kMaskRandom6_6

#define kPacketMaskRandom7 7, \
  kMaskRandom7_1, \
  kMaskRandom7_2, \
  kMaskRandom7_3, \
  kMaskRandom7_4, \
  kMaskRandom7_5, \
  kMaskRandom7_6, \
  kMaskRandom7_7

#define kPacketMaskRandom8 8, \
  kMaskRandom8_1, \
  kMaskRandom8_2, \
  kMaskRandom8_3, \
  kMaskRandom8_4, \
  kMaskRandom8_5, \
  kMaskRandom8_6, \
  kMaskRandom8_7, \
  kMaskRandom8_8

#define kPacketMaskRandom9 9, \
  kMaskRandom9_1, \
  kMaskRandom9_2, \
  kMaskRandom9_3, \
  kMaskRandom9_4, \
  kMaskRandom9_5, \
  kMaskRandom9_6, \
  kMaskRandom9_7, \
  kMaskRandom9_8, \
  kMaskRandom9_9

#define kPacketMaskRandom10 10, \
  kMaskRandom10_1, \
  kMaskRandom10_2, \
  kMaskRandom10_3, \
  kMaskRandom10_4, \
  kMaskRandom10_5, \
  kMaskRandom10_6, \
  kMaskRandom10_7, \
  kMaskRandom10_8, \
  kMaskRandom10_9, \
  kMaskRandom10_10

#define kPacketMaskRandom11 11, \
  kMaskRandom11_1, \
  kMaskRandom11_2, \
  kMaskRandom11_3, \
  kMaskRandom11_4, \
  kMaskRandom11_5, \
  kMaskRandom11_6, \
  kMaskRandom11_7, \
  kMaskRandom11_8, \
  kMaskRandom11_9, \
  kMaskRandom11_10, \
  kMaskRandom11_11

#define kPacketMaskRandom12 12, \
  kMaskRandom12_1, \
  kMaskRandom12_2, \
  kMaskRandom12_3, \
  kMaskRandom12_4, \
  kMaskRandom12_5, \
  kMaskRandom12_6, \
  kMaskRandom12_7, \
  kMaskRandom12_8, \
  kMaskRandom12_9, \
  kMaskRandom12_10, \
  kMaskRandom12_11, \
  kMaskRandom12_12

#define kPacketMaskRandom13 13, \
  kMaskRandom13_1, \
  kMaskRandom13_2, \
  kMaskRandom13_3, \
  kMaskRandom13_4, \
  kMaskRandom13_5, \
  kMaskRandom13_6, \
  kMaskRandom13_7, \
  kMaskRandom13_8, \
  kMaskRandom13_9, \
  kMaskRandom13_10, \
  kMaskRandom13_11, \
  kMaskRandom13_12, \
  kMaskRandom13_13

#define kPacketMaskRandom14 14, \
  kMaskRandom14_1, \
  kMaskRandom14_2, \
  kMaskRandom14_3, \
  kMaskRandom14_4, \
  kMaskRandom14_5, \
  kMaskRandom14_6, \
  kMaskRandom14_7, \
  kMaskRandom14_8, \
  kMaskRandom14_9, \
  kMaskRandom14_10, \
  kMaskRandom14_11, \
  kMaskRandom14_12, \
  kMaskRandom14_13, \
  kMaskRandom14_14

#define kPacketMaskRandom15 15, \
  kMaskRandom15_1, \
  kMaskRandom15_2, \
  kMaskRandom15_3, \
  kMaskRandom15_4, \
  kMaskRandom15_5, \
  kMaskRandom15_6, \
  kMaskRandom15_7, \
  kMaskRandom15_8, \
  kMaskRandom15_9, \
  kMaskRandom15_10, \
  kMaskRandom15_11, \
  kMaskRandom15_12, \
  kMaskRandom15_13, \
  kMaskRandom15_14, \
  kMaskRandom15_15

#define kPacketMaskRandom16 16, \
  kMaskRandom16_1, \
  kMaskRandom16_2, \
  kMaskRandom16_3, \
  kMaskRandom16_4, \
  kMaskRandom16_5, \
  kMaskRandom16_6, \
  kMaskRandom16_7, \
  kMaskRandom16_8, \
  kMaskRandom16_9, \
  kMaskRandom16_10, \
  kMaskRandom16_11, \
  kMaskRandom16_12, \
  kMaskRandom16_13, \
  kMaskRandom16_14, \
  kMaskRandom16_15, \
  kMaskRandom16_16

#define kPacketMaskRandom17 17, \
  kMaskRandom17_1, \
  kMaskRandom17_2, \
  kMaskRandom17_3, \
  kMaskRandom17_4, \
  kMaskRandom17_5, \
  kMaskRandom17_6, \
  kMaskRandom17_7, \
  kMaskRandom17_8, \
  kMaskRandom17_9, \
  kMaskRandom17_10, \
  kMaskRandom17_11, \
  kMaskRandom17_12, \
  kMaskRandom17_13, \
  kMaskRandom17_14, \
  kMaskRandom17_15, \
  kMaskRandom17_16, \
  kMaskRandom17_17

#define kPacketMaskRandom18 18, \
  kMaskRandom18_1, \
  kMaskRandom18_2, \
  kMaskRandom18_3, \
  kMaskRandom18_4, \
  kMaskRandom18_5, \
  kMaskRandom18_6, \
  kMaskRandom18_7, \
  kMaskRandom18_8, \
  kMaskRandom18_9, \
  kMaskRandom18_10, \
  kMaskRandom18_11, \
  kMaskRandom18_12, \
  kMaskRandom18_13, \
  kMaskRandom18_14, \
  kMaskRandom18_15, \
  kMaskRandom18_16, \
  kMaskRandom18_17, \
  kMaskRandom18_18

#define kPacketMaskRandom19 19, \
  kMaskRandom19_1, \
  kMaskRandom19_2, \
  kMaskRandom19_3, \
  kMaskRandom19_4, \
  kMaskRandom19_5, \
  kMaskRandom19_6, \
  kMaskRandom19_7, \
  kMaskRandom19_8, \
  kMaskRandom19_9, \
  kMaskRandom19_10, \
  kMaskRandom19_11, \
  kMaskRandom19_12, \
  kMaskRandom19_13, \
  kMaskRandom19_14, \
  kMaskRandom19_15, \
  kMaskRandom19_16, \
  kMaskRandom19_17, \
  kMaskRandom19_18, \
  kMaskRandom19_19

#define kPacketMaskRandom20 20, \
  kMaskRandom20_1, \
  kMaskRandom20_2, \
  kMaskRandom20_3, \
  kMaskRandom20_4, \
  kMaskRandom20_5, \
  kMaskRandom20_6, \
  kMaskRandom20_7, \
  kMaskRandom20_8, \
  kMaskRandom20_9, \
  kMaskRandom20_10, \
  kMaskRandom20_11, \
  kMaskRandom20_12, \
  kMaskRandom20_13, \
  kMaskRandom20_14, \
  kMaskRandom20_15, \
  kMaskRandom20_16, \
  kMaskRandom20_17, \
  kMaskRandom20_18, \
  kMaskRandom20_19, \
  kMaskRandom20_20

#define kPacketMaskRandom21 21, \
  kMaskRandom21_1, \
  kMaskRandom21_2, \
  kMaskRandom21_3, \
  kMaskRandom21_4, \
  kMaskRandom21_5, \
  kMaskRandom21_6, \
  kMaskRandom21_7, \
  kMaskRandom21_8, \
  kMaskRandom21_9, \
  kMaskRandom21_10, \
  kMaskRandom21_11, \
  kMaskRandom21_12, \
  kMaskRandom21_13, \
  kMaskRandom21_14, \
  kMaskRandom21_15, \
  kMaskRandom21_16, \
  kMaskRandom21_17, \
  kMaskRandom21_18, \
  kMaskRandom21_19, \
  kMaskRandom21_20, \
  kMaskRandom21_21

#define kPacketMaskRandom22 22, \
  kMaskRandom22_1, \
  kMaskRandom22_2, \
  kMaskRandom22_3, \
  kMaskRandom22_4, \
  kMaskRandom22_5, \
  kMaskRandom22_6, \
  kMaskRandom22_7, \
  kMaskRandom22_8, \
  kMaskRandom22_9, \
  kMaskRandom22_10, \
  kMaskRandom22_11, \
  kMaskRandom22_12, \
  kMaskRandom22_13, \
  kMaskRandom22_14, \
  kMaskRandom22_15, \
  kMaskRandom22_16, \
  kMaskRandom22_17, \
  kMaskRandom22_18, \
  kMaskRandom22_19, \
  kMaskRandom22_20, \
  kMaskRandom22_21, \
  kMaskRandom22_22

#define kPacketMaskRandom23 23, \
  kMaskRandom23_1, \
  kMaskRandom23_2, \
  kMaskRandom23_3, \
  kMaskRandom23_4, \
  kMaskRandom23_5, \
  kMaskRandom23_6, \
  kMaskRandom23_7, \
  kMaskRandom23_8, \
  kMaskRandom23_9, \
  kMaskRandom23_10, \
  kMaskRandom23_11, \
  kMaskRandom23_12, \
  kMaskRandom23_13, \
  kMaskRandom23_14, \
  kMaskRandom23_15, \
  kMaskRandom23_16, \
  kMaskRandom23_17, \
  kMaskRandom23_18, \
  kMaskRandom23_19, \
  kMaskRandom23_20, \
  kMaskRandom23_21, \
  kMaskRandom23_22, \
  kMaskRandom23_23

#define kPacketMaskRandom24 24, \
  kMaskRandom24_1, \
  kMaskRandom24_2, \
  kMaskRandom24_3, \
  kMaskRandom24_4, \
  kMaskRandom24_5, \
  kMaskRandom24_6, \
  kMaskRandom24_7, \
  kMaskRandom24_8, \
  kMaskRandom24_9, \
  kMaskRandom24_10, \
  kMaskRandom24_11, \
  kMaskRandom24_12, \
  kMaskRandom24_13, \
  kMaskRandom24_14, \
  kMaskRandom24_15, \
  kMaskRandom24_16, \
  kMaskRandom24_17, \
  kMaskRandom24_18, \
  kMaskRandom24_19, \
  kMaskRandom24_20, \
  kMaskRandom24_21, \
  kMaskRandom24_22, \
  kMaskRandom24_23, \
  kMaskRandom24_24

#define kPacketMaskRandom25 25, \
  kMaskRandom25_1, \
  kMaskRandom25_2, \
  kMaskRandom25_3, \
  kMaskRandom25_4, \
  kMaskRandom25_5, \
  kMaskRandom25_6, \
  kMaskRandom25_7, \
  kMaskRandom25_8, \
  kMaskRandom25_9, \
  kMaskRandom25_10, \
  kMaskRandom25_11, \
  kMaskRandom25_12, \
  kMaskRandom25_13, \
  kMaskRandom25_14, \
  kMaskRandom25_15, \
  kMaskRandom25_16, \
  kMaskRandom25_17, \
  kMaskRandom25_18, \
  kMaskRandom25_19, \
  kMaskRandom25_20, \
  kMaskRandom25_21, \
  kMaskRandom25_22, \
  kMaskRandom25_23, \
  kMaskRandom25_24, \
  kMaskRandom25_25

#define kPacketMaskRandom26 26, \
  kMaskRandom26_1, \
  kMaskRandom26_2, \
  kMaskRandom26_3, \
  kMaskRandom26_4, \
  kMaskRandom26_5, \
  kMaskRandom26_6, \
  kMaskRandom26_7, \
  kMaskRandom26_8, \
  kMaskRandom26_9, \
  kMaskRandom26_10, \
  kMaskRandom26_11, \
  kMaskRandom26_12, \
  kMaskRandom26_13, \
  kMaskRandom26_14, \
  kMaskRandom26_15, \
  kMaskRandom26_16, \
  kMaskRandom26_17, \
  kMaskRandom26_18, \
  kMaskRandom26_19, \
  kMaskRandom26_20, \
  kMaskRandom26_21, \
  kMaskRandom26_22, \
  kMaskRandom26_23, \
  kMaskRandom26_24, \
  kMaskRandom26_25, \
  kMaskRandom26_26

#define kPacketMaskRandom27 27, \
  kMaskRandom27_1, \
  kMaskRandom27_2, \
  kMaskRandom27_3, \
  kMaskRandom27_4, \
  kMaskRandom27_5, \
  kMaskRandom27_6, \
  kMaskRandom27_7, \
  kMaskRandom27_8, \
  kMaskRandom27_9, \
  kMaskRandom27_10, \
  kMaskRandom27_11, \
  kMaskRandom27_12, \
  kMaskRandom27_13, \
  kMaskRandom27_14, \
  kMaskRandom27_15, \
  kMaskRandom27_16, \
  kMaskRandom27_17, \
  kMaskRandom27_18, \
  kMaskRandom27_19, \
  kMaskRandom27_20, \
  kMaskRandom27_21, \
  kMaskRandom27_22, \
  kMaskRandom27_23, \
  kMaskRandom27_24, \
  kMaskRandom27_25, \
  kMaskRandom27_26, \
  kMaskRandom27_27

#define kPacketMaskRandom28 28, \
  kMaskRandom28_1, \
  kMaskRandom28_2, \
  kMaskRandom28_3, \
  kMaskRandom28_4, \
  kMaskRandom28_5, \
  kMaskRandom28_6, \
  kMaskRandom28_7, \
  kMaskRandom28_8, \
  kMaskRandom28_9, \
  kMaskRandom28_10, \
  kMaskRandom28_11, \
  kMaskRandom28_12, \
  kMaskRandom28_13, \
  kMaskRandom28_14, \
  kMaskRandom28_15, \
  kMaskRandom28_16, \
  kMaskRandom28_17, \
  kMaskRandom28_18, \
  kMaskRandom28_19, \
  kMaskRandom28_20, \
  kMaskRandom28_21, \
  kMaskRandom28_22, \
  kMaskRandom28_23, \
  kMaskRandom28_24, \
  kMaskRandom28_25, \
  kMaskRandom28_26, \
  kMaskRandom28_27, \
  kMaskRandom28_28

#define kPacketMaskRandom29 29, \
  kMaskRandom29_1, \
  kMaskRandom29_2, \
  kMaskRandom29_3, \
  kMaskRandom29_4, \
  kMaskRandom29_5, \
  kMaskRandom29_6, \
  kMaskRandom29_7, \
  kMaskRandom29_8, \
  kMaskRandom29_9, \
  kMaskRandom29_10, \
  kMaskRandom29_11, \
  kMaskRandom29_12, \
  kMaskRandom29_13, \
  kMaskRandom29_14, \
  kMaskRandom29_15, \
  kMaskRandom29_16, \
  kMaskRandom29_17, \
  kMaskRandom29_18, \
  kMaskRandom29_19, \
  kMaskRandom29_20, \
  kMaskRandom29_21, \
  kMaskRandom29_22, \
  kMaskRandom29_23, \
  kMaskRandom29_24, \
  kMaskRandom29_25, \
  kMaskRandom29_26, \
  kMaskRandom29_27, \
  kMaskRandom29_28, \
  kMaskRandom29_29

#define kPacketMaskRandom30 30, \
  kMaskRandom30_1, \
  kMaskRandom30_2, \
  kMaskRandom30_3, \
  kMaskRandom30_4, \
  kMaskRandom30_5, \
  kMaskRandom30_6, \
  kMaskRandom30_7, \
  kMaskRandom30_8, \
  kMaskRandom30_9, \
  kMaskRandom30_10, \
  kMaskRandom30_11, \
  kMaskRandom30_12, \
  kMaskRandom30_13, \
  kMaskRandom30_14, \
  kMaskRandom30_15, \
  kMaskRandom30_16, \
  kMaskRandom30_17, \
  kMaskRandom30_18, \
  kMaskRandom30_19, \
  kMaskRandom30_20, \
  kMaskRandom30_21, \
  kMaskRandom30_22, \
  kMaskRandom30_23, \
  kMaskRandom30_24, \
  kMaskRandom30_25, \
  kMaskRandom30_26, \
  kMaskRandom30_27, \
  kMaskRandom30_28, \
  kMaskRandom30_29, \
  kMaskRandom30_30

#define kPacketMaskRandom31 31, \
  kMaskRandom31_1, \
  kMaskRandom31_2, \
  kMaskRandom31_3, \
  kMaskRandom31_4, \
  kMaskRandom31_5, \
  kMaskRandom31_6, \
  kMaskRandom31_7, \
  kMaskRandom31_8, \
  kMaskRandom31_9, \
  kMaskRandom31_10, \
  kMaskRandom31_11, \
  kMaskRandom31_12, \
  kMaskRandom31_13, \
  kMaskRandom31_14, \
  kMaskRandom31_15, \
  kMaskRandom31_16, \
  kMaskRandom31_17, \
  kMaskRandom31_18, \
  kMaskRandom31_19, \
  kMaskRandom31_20, \
  kMaskRandom31_21, \
  kMaskRandom31_22, \
  kMaskRandom31_23, \
  kMaskRandom31_24, \
  kMaskRandom31_25, \
  kMaskRandom31_26, \
  kMaskRandom31_27, \
  kMaskRandom31_28, \
  kMaskRandom31_29, \
  kMaskRandom31_30, \
  kMaskRandom31_31

#define kPacketMaskRandom32 32, \
  kMaskRandom32_1, \
  kMaskRandom32_2, \
  kMaskRandom32_3, \
  kMaskRandom32_4, \
  kMaskRandom32_5, \
  kMaskRandom32_6, \
  kMaskRandom32_7, \
  kMaskRandom32_8, \
  kMaskRandom32_9, \
  kMaskRandom32_10, \
  kMaskRandom32_11, \
  kMaskRandom32_12, \
  kMaskRandom32_13, \
  kMaskRandom32_14, \
  kMaskRandom32_15, \
  kMaskRandom32_16, \
  kMaskRandom32_17, \
  kMaskRandom32_18, \
  kMaskRandom32_19, \
  kMaskRandom32_20, \
  kMaskRandom32_21, \
  kMaskRandom32_22, \
  kMaskRandom32_23, \
  kMaskRandom32_24, \
  kMaskRandom32_25, \
  kMaskRandom32_26, \
  kMaskRandom32_27, \
  kMaskRandom32_28, \
  kMaskRandom32_29, \
  kMaskRandom32_30, \
  kMaskRandom32_31, \
  kMaskRandom32_32

#define kPacketMaskRandom33 33, \
  kMaskRandom33_1, \
  kMaskRandom33_2, \
  kMaskRandom33_3, \
  kMaskRandom33_4, \
  kMaskRandom33_5, \
  kMaskRandom33_6, \
  kMaskRandom33_7, \
  kMaskRandom33_8, \
  kMaskRandom33_9, \
  kMaskRandom33_10, \
  kMaskRandom33_11, \
  kMaskRandom33_12, \
  kMaskRandom33_13, \
  kMaskRandom33_14, \
  kMaskRandom33_15, \
  kMaskRandom33_16, \
  kMaskRandom33_17, \
  kMaskRandom33_18, \
  kMaskRandom33_19, \
  kMaskRandom33_20, \
  kMaskRandom33_21, \
  kMaskRandom33_22, \
  kMaskRandom33_23, \
  kMaskRandom33_24, \
  kMaskRandom33_25, \
  kMaskRandom33_26, \
  kMaskRandom33_27, \
  kMaskRandom33_28, \
  kMaskRandom33_29, \
  kMaskRandom33_30, \
  kMaskRandom33_31, \
  kMaskRandom33_32, \
  kMaskRandom33_33

#define kPacketMaskRandom34 34, \
  kMaskRandom34_1, \
  kMaskRandom34_2, \
  kMaskRandom34_3, \
  kMaskRandom34_4, \
  kMaskRandom34_5, \
  kMaskRandom34_6, \
  kMaskRandom34_7, \
  kMaskRandom34_8, \
  kMaskRandom34_9, \
  kMaskRandom34_10, \
  kMaskRandom34_11, \
  kMaskRandom34_12, \
  kMaskRandom34_13, \
  kMaskRandom34_14, \
  kMaskRandom34_15, \
  kMaskRandom34_16, \
  kMaskRandom34_17, \
  kMaskRandom34_18, \
  kMaskRandom34_19, \
  kMaskRandom34_20, \
  kMaskRandom34_21, \
  kMaskRandom34_22, \
  kMaskRandom34_23, \
  kMaskRandom34_24, \
  kMaskRandom34_25, \
  kMaskRandom34_26, \
  kMaskRandom34_27, \
  kMaskRandom34_28, \
  kMaskRandom34_29, \
  kMaskRandom34_30, \
  kMaskRandom34_31, \
  kMaskRandom34_32, \
  kMaskRandom34_33, \
  kMaskRandom34_34

#define kPacketMaskRandom35 35, \
  kMaskRandom35_1, \
  kMaskRandom35_2, \
  kMaskRandom35_3, \
  kMaskRandom35_4, \
  kMaskRandom35_5, \
  kMaskRandom35_6, \
  kMaskRandom35_7, \
  kMaskRandom35_8, \
  kMaskRandom35_9, \
  kMaskRandom35_10, \
  kMaskRandom35_11, \
  kMaskRandom35_12, \
  kMaskRandom35_13, \
  kMaskRandom35_14, \
  kMaskRandom35_15, \
  kMaskRandom35_16, \
  kMaskRandom35_17, \
  kMaskRandom35_18, \
  kMaskRandom35_19, \
  kMaskRandom35_20, \
  kMaskRandom35_21, \
  kMaskRandom35_22, \
  kMaskRandom35_23, \
  kMaskRandom35_24, \
  kMaskRandom35_25, \
  kMaskRandom35_26, \
  kMaskRandom35_27, \
  kMaskRandom35_28, \
  kMaskRandom35_29, \
  kMaskRandom35_30, \
  kMaskRandom35_31, \
  kMaskRandom35_32, \
  kMaskRandom35_33, \
  kMaskRandom35_34, \
  kMaskRandom35_35

#define kPacketMaskRandom36 36, \
  kMaskRandom36_1, \
  kMaskRandom36_2, \
  kMaskRandom36_3, \
  kMaskRandom36_4, \
  kMaskRandom36_5, \
  kMaskRandom36_6, \
  kMaskRandom36_7, \
  kMaskRandom36_8, \
  kMaskRandom36_9, \
  kMaskRandom36_10, \
  kMaskRandom36_11, \
  kMaskRandom36_12, \
  kMaskRandom36_13, \
  kMaskRandom36_14, \
  kMaskRandom36_15, \
  kMaskRandom36_16, \
  kMaskRandom36_17, \
  kMaskRandom36_18, \
  kMaskRandom36_19, \
  kMaskRandom36_20, \
  kMaskRandom36_21, \
  kMaskRandom36_22, \
  kMaskRandom36_23, \
  kMaskRandom36_24, \
  kMaskRandom36_25, \
  kMaskRandom36_26, \
  kMaskRandom36_27, \
  kMaskRandom36_28, \
  kMaskRandom36_29, \
  kMaskRandom36_30, \
  kMaskRandom36_31, \
  kMaskRandom36_32, \
  kMaskRandom36_33, \
  kMaskRandom36_34, \
  kMaskRandom36_35, \
  kMaskRandom36_36

#define kPacketMaskRandom37 37, \
  kMaskRandom37_1, \
  kMaskRandom37_2, \
  kMaskRandom37_3, \
  kMaskRandom37_4, \
  kMaskRandom37_5, \
  kMaskRandom37_6, \
  kMaskRandom37_7, \
  kMaskRandom37_8, \
  kMaskRandom37_9, \
  kMaskRandom37_10, \
  kMaskRandom37_11, \
  kMaskRandom37_12, \
  kMaskRandom37_13, \
  kMaskRandom37_14, \
  kMaskRandom37_15, \
  kMaskRandom37_16, \
  kMaskRandom37_17, \
  kMaskRandom37_18, \
  kMaskRandom37_19, \
  kMaskRandom37_20, \
  kMaskRandom37_21, \
  kMaskRandom37_22, \
  kMaskRandom37_23, \
  kMaskRandom37_24, \
  kMaskRandom37_25, \
  kMaskRandom37_26, \
  kMaskRandom37_27, \
  kMaskRandom37_28, \
  kMaskRandom37_29, \
  kMaskRandom37_30, \
  kMaskRandom37_31, \
  kMaskRandom37_32, \
  kMaskRandom37_33, \
  kMaskRandom37_34, \
  kMaskRandom37_35, \
  kMaskRandom37_36, \
  kMaskRandom37_37

#define kPacketMaskRandom38 38, \
  kMaskRandom38_1, \
  kMaskRandom38_2, \
  kMaskRandom38_3, \
  kMaskRandom38_4, \
  kMaskRandom38_5, \
  kMaskRandom38_6, \
  kMaskRandom38_7, \
  kMaskRandom38_8, \
  kMaskRandom38_9, \
  kMaskRandom38_10, \
  kMaskRandom38_11, \
  kMaskRandom38_12, \
  kMaskRandom38_13, \
  kMaskRandom38_14, \
  kMaskRandom38_15, \
  kMaskRandom38_16, \
  kMaskRandom38_17, \
  kMaskRandom38_18, \
  kMaskRandom38_19, \
  kMaskRandom38_20, \
  kMaskRandom38_21, \
  kMaskRandom38_22, \
  kMaskRandom38_23, \
  kMaskRandom38_24, \
  kMaskRandom38_25, \
  kMaskRandom38_26, \
  kMaskRandom38_27, \
  kMaskRandom38_28, \
  kMaskRandom38_29, \
  kMaskRandom38_30, \
  kMaskRandom38_31, \
  kMaskRandom38_32, \
  kMaskRandom38_33, \
  kMaskRandom38_34, \
  kMaskRandom38_35, \
  kMaskRandom38_36, \
  kMaskRandom38_37, \
  kMaskRandom38_38

#define kPacketMaskRandom39 39, \
  kMaskRandom39_1, \
  kMaskRandom39_2, \
  kMaskRandom39_3, \
  kMaskRandom39_4, \
  kMaskRandom39_5, \
  kMaskRandom39_6, \
  kMaskRandom39_7, \
  kMaskRandom39_8, \
  kMaskRandom39_9, \
  kMaskRandom39_10, \
  kMaskRandom39_11, \
  kMaskRandom39_12, \
  kMaskRandom39_13, \
  kMaskRandom39_14, \
  kMaskRandom39_15, \
  kMaskRandom39_16, \
  kMaskRandom39_17, \
  kMaskRandom39_18, \
  kMaskRandom39_19, \
  kMaskRandom39_20, \
  kMaskRandom39_21, \
  kMaskRandom39_22, \
  kMaskRandom39_23, \
  kMaskRandom39_24, \
  kMaskRandom39_25, \
  kMaskRandom39_26, \
  kMaskRandom39_27, \
  kMaskRandom39_28, \
  kMaskRandom39_29, \
  kMaskRandom39_30, \
  kMaskRandom39_31, \
  kMaskRandom39_32, \
  kMaskRandom39_33, \
  kMaskRandom39_34, \
  kMaskRandom39_35, \
  kMaskRandom39_36, \
  kMaskRandom39_37, \
  kMaskRandom39_38, \
  kMaskRandom39_39

#define kPacketMaskRandom40 40, \
  kMaskRandom40_1, \
  kMaskRandom40_2, \
  kMaskRandom40_3, \
  kMaskRandom40_4, \
  kMaskRandom40_5, \
  kMaskRandom40_6, \
  kMaskRandom40_7, \
  kMaskRandom40_8, \
  kMaskRandom40_9, \
  kMaskRandom40_10, \
  kMaskRandom40_11, \
  kMaskRandom40_12, \
  kMaskRandom40_13, \
  kMaskRandom40_14, \
  kMaskRandom40_15, \
  kMaskRandom40_16, \
  kMaskRandom40_17, \
  kMaskRandom40_18, \
  kMaskRandom40_19, \
  kMaskRandom40_20, \
  kMaskRandom40_21, \
  kMaskRandom40_22, \
  kMaskRandom40_23, \
  kMaskRandom40_24, \
  kMaskRandom40_25, \
  kMaskRandom40_26, \
  kMaskRandom40_27, \
  kMaskRandom40_28, \
  kMaskRandom40_29, \
  kMaskRandom40_30, \
  kMaskRandom40_31, \
  kMaskRandom40_32, \
  kMaskRandom40_33, \
  kMaskRandom40_34, \
  kMaskRandom40_35, \
  kMaskRandom40_36, \
  kMaskRandom40_37, \
  kMaskRandom40_38, \
  kMaskRandom40_39, \
  kMaskRandom40_40

#define kPacketMaskRandom41 41, \
  kMaskRandom41_1, \
  kMaskRandom41_2, \
  kMaskRandom41_3, \
  kMaskRandom41_4, \
  kMaskRandom41_5, \
  kMaskRandom41_6, \
  kMaskRandom41_7, \
  kMaskRandom41_8, \
  kMaskRandom41_9, \
  kMaskRandom41_10, \
  kMaskRandom41_11, \
  kMaskRandom41_12, \
  kMaskRandom41_13, \
  kMaskRandom41_14, \
  kMaskRandom41_15, \
  kMaskRandom41_16, \
  kMaskRandom41_17, \
  kMaskRandom41_18, \
  kMaskRandom41_19, \
  kMaskRandom41_20, \
  kMaskRandom41_21, \
  kMaskRandom41_22, \
  kMaskRandom41_23, \
  kMaskRandom41_24, \
  kMaskRandom41_25, \
  kMaskRandom41_26, \
  kMaskRandom41_27, \
  kMaskRandom41_28, \
  kMaskRandom41_29, \
  kMaskRandom41_30, \
  kMaskRandom41_31, \
  kMaskRandom41_32, \
  kMaskRandom41_33, \
  kMaskRandom41_34, \
  kMaskRandom41_35, \
  kMaskRandom41_36, \
  kMaskRandom41_37, \
  kMaskRandom41_38, \
  kMaskRandom41_39, \
  kMaskRandom41_40, \
  kMaskRandom41_41

#define kPacketMaskRandom42 42, \
  kMaskRandom42_1, \
  kMaskRandom42_2, \
  kMaskRandom42_3, \
  kMaskRandom42_4, \
  kMaskRandom42_5, \
  kMaskRandom42_6, \
  kMaskRandom42_7, \
  kMaskRandom42_8, \
  kMaskRandom42_9, \
  kMaskRandom42_10, \
  kMaskRandom42_11, \
  kMaskRandom42_12, \
  kMaskRandom42_13, \
  kMaskRandom42_14, \
  kMaskRandom42_15, \
  kMaskRandom42_16, \
  kMaskRandom42_17, \
  kMaskRandom42_18, \
  kMaskRandom42_19, \
  kMaskRandom42_20, \
  kMaskRandom42_21, \
  kMaskRandom42_22, \
  kMaskRandom42_23, \
  kMaskRandom42_24, \
  kMaskRandom42_25, \
  kMaskRandom42_26, \
  kMaskRandom42_27, \
  kMaskRandom42_28, \
  kMaskRandom42_29, \
  kMaskRandom42_30, \
  kMaskRandom42_31, \
  kMaskRandom42_32, \
  kMaskRandom42_33, \
  kMaskRandom42_34, \
  kMaskRandom42_35, \
  kMaskRandom42_36, \
  kMaskRandom42_37, \
  kMaskRandom42_38, \
  kMaskRandom42_39, \
  kMaskRandom42_40, \
  kMaskRandom42_41, \
  kMaskRandom42_42

#define kPacketMaskRandom43 43, \
  kMaskRandom43_1, \
  kMaskRandom43_2, \
  kMaskRandom43_3, \
  kMaskRandom43_4, \
  kMaskRandom43_5, \
  kMaskRandom43_6, \
  kMaskRandom43_7, \
  kMaskRandom43_8, \
  kMaskRandom43_9, \
  kMaskRandom43_10, \
  kMaskRandom43_11, \
  kMaskRandom43_12, \
  kMaskRandom43_13, \
  kMaskRandom43_14, \
  kMaskRandom43_15, \
  kMaskRandom43_16, \
  kMaskRandom43_17, \
  kMaskRandom43_18, \
  kMaskRandom43_19, \
  kMaskRandom43_20, \
  kMaskRandom43_21, \
  kMaskRandom43_22, \
  kMaskRandom43_23, \
  kMaskRandom43_24, \
  kMaskRandom43_25, \
  kMaskRandom43_26, \
  kMaskRandom43_27, \
  kMaskRandom43_28, \
  kMaskRandom43_29, \
  kMaskRandom43_30, \
  kMaskRandom43_31, \
  kMaskRandom43_32, \
  kMaskRandom43_33, \
  kMaskRandom43_34, \
  kMaskRandom43_35, \
  kMaskRandom43_36, \
  kMaskRandom43_37, \
  kMaskRandom43_38, \
  kMaskRandom43_39, \
  kMaskRandom43_40, \
  kMaskRandom43_41, \
  kMaskRandom43_42, \
  kMaskRandom43_43

#define kPacketMaskRandom44 44, \
  kMaskRandom44_1, \
  kMaskRandom44_2, \
  kMaskRandom44_3, \
  kMaskRandom44_4, \
  kMaskRandom44_5, \
  kMaskRandom44_6, \
  kMaskRandom44_7, \
  kMaskRandom44_8, \
  kMaskRandom44_9, \
  kMaskRandom44_10, \
  kMaskRandom44_11, \
  kMaskRandom44_12, \
  kMaskRandom44_13, \
  kMaskRandom44_14, \
  kMaskRandom44_15, \
  kMaskRandom44_16, \
  kMaskRandom44_17, \
  kMaskRandom44_18, \
  kMaskRandom44_19, \
  kMaskRandom44_20, \
  kMaskRandom44_21, \
  kMaskRandom44_22, \
  kMaskRandom44_23, \
  kMaskRandom44_24, \
  kMaskRandom44_25, \
  kMaskRandom44_26, \
  kMaskRandom44_27, \
  kMaskRandom44_28, \
  kMaskRandom44_29, \
  kMaskRandom44_30, \
  kMaskRandom44_31, \
  kMaskRandom44_32, \
  kMaskRandom44_33, \
  kMaskRandom44_34, \
  kMaskRandom44_35, \
  kMaskRandom44_36, \
  kMaskRandom44_37, \
  kMaskRandom44_38, \
  kMaskRandom44_39, \
  kMaskRandom44_40, \
  kMaskRandom44_41, \
  kMaskRandom44_42, \
  kMaskRandom44_43, \
  kMaskRandom44_44

#define kPacketMaskRandom45 45, \
  kMaskRandom45_1, \
  kMaskRandom45_2, \
  kMaskRandom45_3, \
  kMaskRandom45_4, \
  kMaskRandom45_5, \
  kMaskRandom45_6, \
  kMaskRandom45_7, \
  kMaskRandom45_8, \
  kMaskRandom45_9, \
  kMaskRandom45_10, \
  kMaskRandom45_11, \
  kMaskRandom45_12, \
  kMaskRandom45_13, \
  kMaskRandom45_14, \
  kMaskRandom45_15, \
  kMaskRandom45_16, \
  kMaskRandom45_17, \
  kMaskRandom45_18, \
  kMaskRandom45_19, \
  kMaskRandom45_20, \
  kMaskRandom45_21, \
  kMaskRandom45_22, \
  kMaskRandom45_23, \
  kMaskRandom45_24, \
  kMaskRandom45_25, \
  kMaskRandom45_26, \
  kMaskRandom45_27, \
  kMaskRandom45_28, \
  kMaskRandom45_29, \
  kMaskRandom45_30, \
  kMaskRandom45_31, \
  kMaskRandom45_32, \
  kMaskRandom45_33, \
  kMaskRandom45_34, \
  kMaskRandom45_35, \
  kMaskRandom45_36, \
  kMaskRandom45_37, \
  kMaskRandom45_38, \
  kMaskRandom45_39, \
  kMaskRandom45_40, \
  kMaskRandom45_41, \
  kMaskRandom45_42, \
  kMaskRandom45_43, \
  kMaskRandom45_44, \
  kMaskRandom45_45

#define kPacketMaskRandom46 46, \
  kMaskRandom46_1, \
  kMaskRandom46_2, \
  kMaskRandom46_3, \
  kMaskRandom46_4, \
  kMaskRandom46_5, \
  kMaskRandom46_6, \
  kMaskRandom46_7, \
  kMaskRandom46_8, \
  kMaskRandom46_9, \
  kMaskRandom46_10, \
  kMaskRandom46_11, \
  kMaskRandom46_12, \
  kMaskRandom46_13, \
  kMaskRandom46_14, \
  kMaskRandom46_15, \
  kMaskRandom46_16, \
  kMaskRandom46_17, \
  kMaskRandom46_18, \
  kMaskRandom46_19, \
  kMaskRandom46_20, \
  kMaskRandom46_21, \
  kMaskRandom46_22, \
  kMaskRandom46_23, \
  kMaskRandom46_24, \
  kMaskRandom46_25, \
  kMaskRandom46_26, \
  kMaskRandom46_27, \
  kMaskRandom46_28, \
  kMaskRandom46_29, \
  kMaskRandom46_30, \
  kMaskRandom46_31, \
  kMaskRandom46_32, \
  kMaskRandom46_33, \
  kMaskRandom46_34, \
  kMaskRandom46_35, \
  kMaskRandom46_36, \
  kMaskRandom46_37, \
  kMaskRandom46_38, \
  kMaskRandom46_39, \
  kMaskRandom46_40, \
  kMaskRandom46_41, \
  kMaskRandom46_42, \
  kMaskRandom46_43, \
  kMaskRandom46_44, \
  kMaskRandom46_45, \
  kMaskRandom46_46

#define kPacketMaskRandom47 47, \
  kMaskRandom47_1, \
  kMaskRandom47_2, \
  kMaskRandom47_3, \
  kMaskRandom47_4, \
  kMaskRandom47_5, \
  kMaskRandom47_6, \
  kMaskRandom47_7, \
  kMaskRandom47_8, \
  kMaskRandom47_9, \
  kMaskRandom47_10, \
  kMaskRandom47_11, \
  kMaskRandom47_12, \
  kMaskRandom47_13, \
  kMaskRandom47_14, \
  kMaskRandom47_15, \
  kMaskRandom47_16, \
  kMaskRandom47_17, \
  kMaskRandom47_18, \
  kMaskRandom47_19, \
  kMaskRandom47_20, \
  kMaskRandom47_21, \
  kMaskRandom47_22, \
  kMaskRandom47_23, \
  kMaskRandom47_24, \
  kMaskRandom47_25, \
  kMaskRandom47_26, \
  kMaskRandom47_27, \
  kMaskRandom47_28, \
  kMaskRandom47_29, \
  kMaskRandom47_30, \
  kMaskRandom47_31, \
  kMaskRandom47_32, \
  kMaskRandom47_33, \
  kMaskRandom47_34, \
  kMaskRandom47_35, \
  kMaskRandom47_36, \
  kMaskRandom47_37, \
  kMaskRandom47_38, \
  kMaskRandom47_39, \
  kMaskRandom47_40, \
  kMaskRandom47_41, \
  kMaskRandom47_42, \
  kMaskRandom47_43, \
  kMaskRandom47_44, \
  kMaskRandom47_45, \
  kMaskRandom47_46, \
  kMaskRandom47_47

#define kPacketMaskRandom48 48, \
  kMaskRandom48_1, \
  kMaskRandom48_2, \
  kMaskRandom48_3, \
  kMaskRandom48_4, \
  kMaskRandom48_5, \
  kMaskRandom48_6, \
  kMaskRandom48_7, \
  kMaskRandom48_8, \
  kMaskRandom48_9, \
  kMaskRandom48_10, \
  kMaskRandom48_11, \
  kMaskRandom48_12, \
  kMaskRandom48_13, \
  kMaskRandom48_14, \
  kMaskRandom48_15, \
  kMaskRandom48_16, \
  kMaskRandom48_17, \
  kMaskRandom48_18, \
  kMaskRandom48_19, \
  kMaskRandom48_20, \
  kMaskRandom48_21, \
  kMaskRandom48_22, \
  kMaskRandom48_23, \
  kMaskRandom48_24, \
  kMaskRandom48_25, \
  kMaskRandom48_26, \
  kMaskRandom48_27, \
  kMaskRandom48_28, \
  kMaskRandom48_29, \
  kMaskRandom48_30, \
  kMaskRandom48_31, \
  kMaskRandom48_32, \
  kMaskRandom48_33, \
  kMaskRandom48_34, \
  kMaskRandom48_35, \
  kMaskRandom48_36, \
  kMaskRandom48_37, \
  kMaskRandom48_38, \
  kMaskRandom48_39, \
  kMaskRandom48_40, \
  kMaskRandom48_41, \
  kMaskRandom48_42, \
  kMaskRandom48_43, \
  kMaskRandom48_44, \
  kMaskRandom48_45, \
  kMaskRandom48_46, \
  kMaskRandom48_47, \
  kMaskRandom48_48

// clang-format on
}  // namespace

namespace webrtc {
namespace fec_private_tables {

const uint8_t kPacketMaskRandomTbl[] = {
    48,
    kPacketMaskRandom1,  // 2 byte entries.
    kPacketMaskRandom2,
    kPacketMaskRandom3,
    kPacketMaskRandom4,
    kPacketMaskRandom5,
    kPacketMaskRandom6,
    kPacketMaskRandom7,
    kPacketMaskRandom8,
    kPacketMaskRandom9,
    kPacketMaskRandom10,
    kPacketMaskRandom11,
    kPacketMaskRandom12,
    kPacketMaskRandom13,
    kPacketMaskRandom14,
    kPacketMaskRandom15,
    kPacketMaskRandom16,
    kPacketMaskRandom17,  // Start of 6 byte entries.
    kPacketMaskRandom18,
    kPacketMaskRandom19,
    kPacketMaskRandom20,
    kPacketMaskRandom21,
    kPacketMaskRandom22,
    kPacketMaskRandom23,
    kPacketMaskRandom24,
    kPacketMaskRandom25,
    kPacketMaskRandom26,
    kPacketMaskRandom27,
    kPacketMaskRandom28,
    kPacketMaskRandom29,
    kPacketMaskRandom30,
    kPacketMaskRandom31,
    kPacketMaskRandom32,
    kPacketMaskRandom33,
    kPacketMaskRandom34,
    kPacketMaskRandom35,
    kPacketMaskRandom36,
    kPacketMaskRandom37,
    kPacketMaskRandom38,
    kPacketMaskRandom39,
    kPacketMaskRandom40,
    kPacketMaskRandom41,
    kPacketMaskRandom42,
    kPacketMaskRandom43,
    kPacketMaskRandom44,
    kPacketMaskRandom45,
    kPacketMaskRandom46,
    kPacketMaskRandom47,
    kPacketMaskRandom48,
};

}  // namespace fec_private_tables
}  // namespace webrtc
